# UNIT TEST FOR THE PACKAGER SCRIPT
import os, sys
import shutil
import imp
import tarfile

execfile('generate_pipeline_dirstructure.py')

# This script is intended to be used as a test of the packaging script. The
# output from the script is compared to the contents of a reference directory
# (packager.test.files). If any mismatch is found, it means that your
# particular installation of the packager is not working as expected (there is
# maybe a bug in the used version of the packager).

# Tests:
# 0. Cycle 1, with two group IDs
# 1. Cycle 0
# 2. Cycle 1, pipeline mode 1
# 3. Cycle 1, no pipeline, mode=copy
# 4. Cycle 2, no pipeline
# 5. Cycle 2, pipeline, with flux calibration
# 6. Cycle 2, no pipeline, one ephemeride directory
# 7. Cycle 2, no pipeline, two ephemeride directories
# 8. Cycle 2, no pipeline, one ephemeride file
# 8a. Cycle 2, no pipeline, two incompatible ephemerides: This is *intended to
#              fail* and checks to make sure that the packager DOES fail given
#              a pipeline directory with two incompatible ephemerides
# 9a. Cycle 2, no pipeline, two ephemerides with same name but different contents:
#              This is *intended to fail* and checks to make sure that the
#              packager DOES fail given a pipeline directory with two
#              incompatible ephemerides
# 10. Cycle 2, pipeline, no flux calibration

# Command to create the tarball of the unit tests:
# (From Adam Ginsburg's work: if you are on a mac, you may need
# COPYFILE_DISABLE=1; if you are working in a tracked directory, you may need
# --exclude=.svn instead of --exclude=.git)
# COPYFILE_DISABLE=1 tar --exclude=.git -czvf QA2_Pipeline_Packaging_module.unit-test.tar.gz QA2_Packaging_module.unit-test


#############################
# SOME VARIABLES TO SET:

# script for PI:
PIscr = 'scriptForPI.py'

# Reference directory:
testpath = 'packager.test.files'

# number of tests
n_tests = 13

# Names of the files in the reference directory where the tree structure of the
# packages is saved:
trees=[os.path.join(testpath,'test%itree.txt'%i) for i in range(n_tests)]

# Some tests are intended to fail: these should be skipped in verification
tests_to_skip_verification = [8, 9]

# Path to the data to be used in the test:
datapaths=['fake-analysis' for i in range(n_tests)]

# Names of the output test packages:
tests=['./testpack'+str(i) for i in range(n_tests)]

# Highest group and member ids for each testpackage ('1'->''):
readmes = [['','']]*n_tests
readmes[0] = ['2', '']

# Set this to an empty string if you want to test the packager version in your standard
# setup (i.e., that one obtained with from "QA2_Packaging_module import *". Otherwise,
# the packager version will be loaded by executing the specified file:
#packagerToTest = './science/qa2/QA2_Packaging_module.py'
packagerToTest='../QA2_Packaging_module.py'
#packagerToTest=''

##############################

first_eb = 'X4d5'
second_eb = 'X236'

# Load the packager:
if len(packagerToTest)>0:
    execfile(packagerToTest)
else:
    from QA2_Packaging_module import *


######################
# Determine the location of the test data:
#dataParent =  os.path.split(imp.find_module('QA2_Packaging_module')[1])[0]

# Update variables with this location:

#trees = [os.path.join(dataParent,tt) for tt in trees]
#datapaths = [os.path.join(dataParent,dd) for dd in datapaths]
#PIscr = os.path.join(dataParent,PIscr)
#testpath = os.path.join(dataParent,testpath)
#readf = os.path.join(dataParent,'README.header.txt')
readf = 'README.header.txt'
######################


# This is an extra step that can be done to examine the Packager itself, but is not necessary
# if os.path.exists('fake-pipeline-packaged'):
#     shutil.rmtree('fake-pipeline-packaged')
# QA_Pipeline_Packager('fake-pipeline-staged', 'fake-pipeline-packaged', mode='fake')


def test0():
    # Cleanup previous tests:
    shutil.rmtree('testpack0', ignore_errors=True)

    # Check Cycle 1 mode:
    print "\nUNIT TEST: Test 0 - Will execute in Cycle 1 mode (no pipeline). Tickets will be created.\n"

    QA_Packager(origpath=datapaths[0], packpath=tests[0], readme=readf, mode='ticket', append='', gzip_caltables=True, style='cycle1-nopipe',PIscript=PIscr)

    #Check appending data to already-existing package:
    print "\nUNIT TEST: Will add a new member to the first group. Tickets will be updated.\n"

    QA_Packager(origpath=datapaths[0], packpath=tests[0], readme=readf, mode='ticket', append='member', gzip_caltables=True, style='cycle1-nopipe',PIscript=PIscr)

    print "\nUNIT TEST: Will add a second group. Tickets will be updated.\n"

    QA_Packager(origpath=datapaths[0], packpath=tests[0], readme=readf, mode='ticket', append='group', gzip_caltables=True, style='cycle1-nopipe',PIscript=PIscr)

    return not any(verify_test_number(0))


def test1():
    # Cleanup previous tests:
    shutil.rmtree('testpack1', ignore_errors=True)

    # Check Cycle 0 mode:
    print "\nUNIT TEST: Test 1 - Will execute in Cycle 0 mode (no pipeline). Tickets will be created.\n"

    QA_Packager(origpath=datapaths[1], packpath=tests[1], readme=readf, mode='ticket', append='', gzip_caltables=True, style='cycle0-nopipe',PIscript=PIscr)

    return not any(verify_test_number(1))

def test2():
    # Cleanup previous tests:
    shutil.rmtree('testpack2', ignore_errors=True)

    # Check pipeline mode (in Cycle 1):
    print "\nUNIT TEST: Test 2 - Will execute in Cycle 1 mode (pipeline mode 1). Tickets will be created.\n"

    QA_Packager(origpath=datapaths[2], packpath=tests[2], readme=readf, mode='ticket', append='', gzip_caltables=True, style='cycle1-pipe1',PIscript=PIscr)

    return not any(verify_test_number(2))


def test3():
    # Cleanup previous tests:
    shutil.rmtree('testpack3', ignore_errors=True)

    # Check other mode than "ticket":
    print "\nUNIT TEST: Test 3 - Will execute in Cycle 1 mode (no pipeline) with mode=\'copy\'.\n"

    QA_Packager(origpath=datapaths[3], packpath=tests[3], readme=readf, mode='copy', append='', gzip_caltables=True, style='cycle1-nopipe',PIscript=PIscr)

    return not any(verify_test_number(3))

def test4():
    # Cleanup previous tests:
    shutil.rmtree('testpack4', ignore_errors=True)

    # Check Cycle 2 mode:
    print "\nUNIT TEST: Test 4 - Will execute in Cycle 2 mode (no pipeline). Tickets will be created.\n"

    QA_Packager(origpath=datapaths[4], packpath=tests[4], readme=readf, mode='ticket', append='', gzip_caltables=True, style='cycle2-nopipe',PIscript=PIscr)

    return not any(verify_test_number(4))

def test_cycle2_pipeline(flux_calibration=True, number=5):
    # Cleanup previous tests:
    shutil.rmtree('testpack{0:d}'.format(number), ignore_errors=True)

    print("\nUNIT TEST: Test {0} - Will execute in Cycle 2 mode"
          " (with pipeline, including the intermediate step of "
          "JAO->ARC packaging). Tickets will be created.\n".format(number))

    # Generate the mock pipeline directory structure
    if os.path.exists('fake-pipeline-analysis'):
        shutil.rmtree('fake-pipeline-analysis')
    make_fake_pipeline_analysis_dir()
    if os.path.exists('fake-pipeline-staged'):
        shutil.rmtree('fake-pipeline-staged')
    QA_Pipeline_Stager('fake-pipeline-analysis', 'fake-pipeline-staged',
                       mode='ticket', fake_imaging=True, PIscript=PIscr,
                       fake_flux_calibration=flux_calibration)
    datapaths[number] = 'fake-pipeline-staged'

    QA_Packager(origpath=datapaths[number], packpath=tests[number], readme=readf,
                mode='ticket', append='', gzip_caltables=True,
                style='cycle2-pipe1', PIscript=PIscr)

    return not any(verify_test_number(number))




def test_cycle2_TP_pipeline(flux_calibration=True, number=12):
    # Cleanup previous tests:
    shutil.rmtree('testpack{0:d}'.format(number), ignore_errors=True)

    print("\nUNIT TEST: Test {0} - Will execute in Cycle 2 TP mode"
          " (with pipeline, including the intermediate step of "
          "JAO->ARC packaging). Tickets will be created.\n".format(number))

    # Generate the mock pipeline directory structure
    if os.path.exists('fake-pipeline-analysis'):
        shutil.rmtree('fake-pipeline-analysis')
    make_fake_pipeline_analysis_dir(TP=True)
    if os.path.exists('fake-pipeline-staged'):
        shutil.rmtree('fake-pipeline-staged')
    QA_Pipeline_Stager('fake-pipeline-analysis', 'fake-pipeline-staged',
                       mode='ticket', fake_imaging=True, PIscript=PIscr,
                       fake_flux_calibration=flux_calibration)
    datapaths[number] = 'fake-pipeline-staged'

    QA_Packager(origpath=datapaths[number], packpath=tests[number], readme=readf,
                mode='ticket', append='', gzip_caltables=True,
                style='cycle2-pipe1')

    return not any(verify_test_number(number))





def test5():
    return test_cycle2_pipeline(flux_calibration=True, number=5)

# No-pipeline ephemeris checking
def test_ephem(ii, ephem_style):
    if ii not in (6,7,8):
        raise ValueError("Ephemeris tests must be #6, 7, or 8")
    # Cleanup previous tests:
    shutil.rmtree('testpack{0}'.format(ii), ignore_errors=True)

    print("\nUNIT TEST: Test {0} - Will execute in Cycle 2 mode"
          " (NO pipeline). Tickets will be created.\n".format(ii))
    print "UNIT TEST: Ephemeris file creation mode: {0}\n".format(ephem_style)

    make_ephems(ephem_style, target_directory=join(datapaths[ii], first_eb),
                second_target_directory=join(datapaths[ii], second_eb))
    QA_Packager(origpath=datapaths[ii], packpath=tests[ii], readme=readf,
                mode='ticket', append='', gzip_caltables=True,
                style='cycle2-nopipe', PIscript=PIscr)
    cleanup_ephems(ephem_style, target_directory=join(datapaths[ii], first_eb),
                   second_target_directory=join(datapaths[ii], second_eb))
    os.remove(join(datapaths[ii], first_eb, 'scriptForFluxCalibration.py'))
    if 'two' in ephem_style:
        os.remove(join(datapaths[ii], second_eb, 'scriptForFluxCalibration.py'))

    return not any(verify_test_number(ii))



def test8a_bad_ephems():
    # Check that using a bad set of ephemerides results in an exception
    print("\nUNIT TEST: Test 8a - Will execute in Cycle 2 mode with bad ephemerides.\n")
    dp = 'fake-analysis'
    make_ephems(ephem_style='two_bad', target_directory=join(dp, first_eb),
                second_target_directory=join(dp, second_eb))
    pp = 'testpath_bad'
    if os.path.exists(pp):
        shutil.rmtree(pp)
    QA_Packager(origpath=dp, packpath=pp, readme=readf,
                mode='ticket', append='', gzip_caltables=True,
                style='cycle2-nopipe', PIscript=PIscr)

    print "UNIT TEST: The previous line *SHOULD* say 'ERROR:  Found multiple ephemeris tables,...'"
    if os.path.exists(pp):
        print("ERROR in unit test: QA_Packager did not clean up the packaging directory"
              " when bad ephemerides found.")
        return False

    cleanup_ephems(ephem_style='two_bad', target_directory=join(dp, first_eb),
                   second_target_directory=join(dp, second_eb))
    os.remove(join(dp, first_eb, 'scriptForFluxCalibration.py'))
    os.remove(join(dp, second_eb, 'scriptForFluxCalibration.py'))

    return True

def test9a_bad_ephems():
    # Check that using a bad set of ephemerides results in an exception
    print("\nUNIT TEST: Test 9a - Will execute in Cycle 2 mode with bad ephemerides"
          " (two files with same name but different contents).\n")
    dp = 'fake-analysis'
    make_ephems(ephem_style='two_bad_file', target_directory=join(dp, first_eb),
                second_target_directory=join(dp, second_eb))
    pp = 'testpath_bad'
    if os.path.exists(pp):
        shutil.rmtree(pp)
    QA_Packager(origpath=dp, packpath=pp, readme=readf,
                mode='ticket', append='', gzip_caltables=True,
                style='cycle2-nopipe', PIscript=PIscr)

    print "UNIT TEST: The previous line *SHOULD* say 'ERROR:  Found multiple ephemeris tables,...'"
    if os.path.exists(pp):
        print("ERROR in unit test: QA_Packager did not clean up the packaging directory"
              " when bad ephemerides found.")
        return False

    cleanup_ephems(ephem_style='two_bad_file', target_directory=join(dp, first_eb),
                   second_target_directory=join(dp, second_eb))
    os.remove(join(dp, first_eb, 'scriptForFluxCalibration.py'))
    os.remove(join(dp, second_eb, 'scriptForFluxCalibration.py'))

    return True

def test10():
    # Test the pipeline with no scriptForFluxCalibration.py
    return test_cycle2_pipeline(flux_calibration=False, number=10)

def test11():
    # Regression test in version 27: fix a bug where
    # scriptForFluxCalibration.py is not included if it is in the calibrated/
    # directory instead of the script/ directory
    # Test the pipeline with scriptForFluxCalibration.py in calibrated
    return test_cycle2_pipeline(flux_calibration='calibrated', number=11)


def test12():
    # Regression test in version 27: fix a bug where
    # scriptForFluxCalibration.py is not included if it is in the calibrated/
    # directory instead of the script/ directory
    # Test the pipeline with scriptForFluxCalibration.py in calibrated
    return test_cycle2_TP_pipeline(flux_calibration='calibrated', number=12)



def verify_test(testdirectory, treefn, readme):
    Err = False

    testtree=open(treefn).readlines()
    print "UNIT TEST: Checking contents of "+testdirectory
    readfile=os.path.join(testdirectory,'sg_ouss_id','group_ouss_id'+readme[0],'member_ouss_id'+readme[1],'README')
    ff = open(readfile)
    lines = ff.readlines()
    for i,line in enumerate(lines):
        if '|-- ' in line:
            i0 = i
            break
    treelines = lines[i0:]
    for i,line in enumerate(testtree):
        if i >= len(treelines):
            print "UNIT TEST: ERROR IN LINE %i OF TREE %s: %s: there are more lines in testtree than tree"%(i,testdirectory,line)
            return True
        if line != treelines[i]:
            print "UNIT TEST: ERROR IN LINE %i OF TREE %s: \n[HAVE]%s\n[WANT]%s"%(i,testdirectory,line,treelines[i])
            Err=True

    return Err

def verify_ticket(testdirectory):
    Err = False

    print 'UNIT TEST: Checking '+testdirectory+'.ticket.tar'
    tar0 = sorted(os.popen('tar -tf '+testdirectory+'.ticket.tar').readlines())
    tar1 = sorted(os.popen('tar -tf '+os.path.join(testpath,base(testdirectory)+'.ticket.tar')).readlines())
    for ti,tline in enumerate(tar0):
        if ti >= len(tar1):
            print "UNIT TEST: ERROR IN LINE %i OF TREE %s: %s: there are more lines in testtree than tree"%(ti,testdirectory,tline)
            return True
        if tline != tar1[ti]:
            print "UNIT TEST: ERROR IN LINE %i OF %s. FILE %s SHOULD BE %s"%(ti,testdirectory,tline[:-1],tar1[ti][:-1])
            Err=True

    # Check that the ticket mode copy is not empty
    # (it should be a fully functional scriptForPI)
    tarfilename = os.path.join(base(testdirectory)+'.ticket.tar')
    tar = tarfile.open(tarfilename)
    for fobj in tar.getmembers():
        if fobj.name == 'script/scriptForPI.py':
            f = tar.extractfile(fobj)
            contents = f.read()
            if "# ALMA Data Reduction Script" not in contents:
                print("ERROR: Contents of the scriptForPI in the ticket for"
                      " test {0} are empty.".format(testdirectory))
                Err = True
    tar.close()

    return Err




def run_all_tests():
    OK = {
          'test0':test0(),
          'test1':test1(),
          'test2':test2(),
          'test3':test3(),
          'test4':test4(),
          'test5':test5(),
          'test6':test_ephem(6, 'ok'),
          'test7':test_ephem(7, 'two_ok'),
          'test8':test_ephem(8, 'file'),
          'test8a':test8a_bad_ephems(),
          'test9':test9a_bad_ephems(),
          'test10':test10(),
          'test11':test11(),
          'test12':test12(),
    }
    return OK

# Compare with reference outputs:

def verify_test_number(t):
    TreeErr = verify_test(tests[t], trees[t], readme=readmes[t])
    TicketErr = verify_ticket(tests[t])

    if TreeErr:
        print "There was a tree error in Test {0}".format(t)

    if TicketErr:
        print "There was a ticket error in Test {0}".format(t)

    return TreeErr, TicketErr

def verify_all():
    print "\nUNIT TEST: Will now check the package contents\n"

    TreeErrors = {}
    TicketErrors = {}

    for testnumber in range(n_tests):
        if testnumber in tests_to_skip_verification:
            print "UNIT TEST: Skipping {0} (this is intentional)".format(testnumber)
            continue
        TreeErr, TicketErr = verify_test_number(testnumber)
        TreeErrors[testnumber] = TreeErr
        TicketErrors[testnumber] = TicketErr

    OK = True
    if any(TreeErrors.values()):
        print "\nUNIT TEST: THERE HAS BEEN AT LEAST ONE ERROR IN THE TREE TESTS."
        print "UNIT TEST: Tree Errors: ",[t for t in TreeErrors if TreeErrors[t]]
        OK = False
    if any(TicketErrors.values()):
        print "\nUNIT TEST: THERE HAS BEEN AT LEAST ONE ERROR IN THE TICKET TESTS."
        print "UNIT TEST: Ticket Errors: ",[t for t in TicketErrors if TicketErrors[t]]
        OK = False

    if OK:
        print "\nUNIT TEST: ALL TESTS ARE OK."
      #  os.system('rm -rf testpack*')
      #  if os.path.exists('fake-pipeline-analysis'):
      #      shutil.rmtree('fake-pipeline-analysis')
      #  if os.path.exists('fake-pipeline-staged'):
      #      shutil.rmtree('fake-pipeline-staged')
    else:
        print "\nUNIT TEST: TESTS ENDED IN FAILURE.  PLEASE SEE ABOVE FOR DETAILS"

    print "\nUNIT TEST: END\n"

# Run the tests after defining all variables:
OK = run_all_tests()
if all(OK.values()):
    verify_all()
else:
    print "\nUNIT TEST: TESTS FAILED TO RUN TO COMPLETION.\n"
    print "The following tests failed: ",[k for k in OK if not OK[k]]
