#!/usr/bin/env python

"""
Class to execute the observing commands requested by AntennaIntegrator.py
from almaop@aos-gns.aiv.alma.cl

$Id $ v1.0 - smartin 10/2015
$Id $ v1.1 - smartin 11/2015 First commit
$Id $ v1.2 - smartin 07/2016 added a time.wait(1) after execution and before writting the communication file
$Id $ v2.0 - smartin 10/2016 automatically connecting to red-osf and stablishing connection properly
$Id $ v3.0 - smartin 11/2016 updated version for AOS-64. New communication path among other changes
$Id $ v3.1 - smartin 12/2016 changed communication directory to AIV for operators to be able to access and
                             CTRL+C handling.
"""

import os
import sys
import time
import calendar
import subprocess
from math import floor
from optparse import OptionParser

__author__ = 'smartin'


class AntennaIntegratorAOS:

    def __init__(self):  # , arrayname=''):
        # old path self.commandsfilepath = '/mnt/nas-red.osf.alma.cl/storage1/data2/data'
        self.commandsfilepath = '/mnt/nas-05.osf.alma.cl/groups/science/data/AAI'
        self.commandsfile = '%s/AntennaIntegrationCommunication.txt' % self.commandsfilepath
        self.executionoutputfile = '%s/AntennaIntegratorAOS.output' % self.commandsfilepath
        self.logfile = './AntennaIntegratorAOS.log'
        self.comandsfile_modifiedtime = 0

    def checkcommandsfile(self, verbose=False, rejectoldfile=False):
        # Check if the home communication directory exists, if not, remind to ssh into aos-gns.
        if not os.path.isdir(self.commandsfilepath.split('/AAI')[0]):
            print "ERROR: Directory %s not found" % self.commandsfilepath.split('/AAI')[0]
            print "You need to ssh almaop@aos64-gns-64"
            print "and then run this script."
            sys.exit()
        # If the AAI directory to be used by AAI does not exist, we do create it from almaop
        if not os.path.isdir(self.commandsfilepath):
            print "WARNING: Directory %s does not exist " % self.commandsfilepath
            print "Creating it"
            os.mkdir(self.commandsfilepath)
            subprocess.call(['chmod', '-R', 'ugo+rw', self.commandsfilepath])
        waitcounter = -1
        if os.path.isfile(self.commandsfile):
            # This option is just for checking at the beginning of the execution
            # to avoid reading commands from old files
            timesincecreation = int(floor(calendar.timegm(time.gmtime())-os.path.getmtime(self.commandsfile)))
            if rejectoldfile:
                # print timesincecreation
                oldthreshold = 300
                if timesincecreation > oldthreshold:
                    print "File %s does exist but it is older than %i seconds" % (self.commandsfile, oldthreshold)
                    print "File deleted"
                    subprocess.call(['rm', self.commandsfile])
            elif timesincecreation % 300 == 0 and timesincecreation != 0:
                # print timesincecreation
                print "Waiting %i minutes since last command" % (timesincecreation/60)

        if not os.path.isfile(self.commandsfile):
            print "File %s does not exist." % self.commandsfile
            print "Will open a terminal to connect to red-osf and run AntennaIntegrator.py"
            sshusername = raw_input('username: ')
#           NO gnome-terminal in ape1. We stick to xterm which is easier
#            commandtoexecute = 'gnome-terminal -t "AntennaIntegrator@red-osf" --geometry 150x30 -e ' \
#                               '"ssh -t %s@red-osf.osf.alma.cl ' \
#                               'echo Starting AntennaIntegrator.py; ' \
#                               'source .bashrc; ' \
#                               'AntennaIntegrator.py; ' \
#                               'bash"' % sshusername
            commandtoexecute = 'xterm -T "AntennaIntegrator@red-osf" -geometry 150x30 -e ' \
                               '"ssh -t %s@red-osf.osf.alma.cl ' \
                               '\'echo Starting AntennaIntegrator.py; ' \
                               'source .bashrc; ' \
                               'AntennaIntegrator.py\'; ' \
                               'bash"' % sshusername
            print commandtoexecute.split(' ')
            print commandtoexecute
            process = subprocess.Popen(commandtoexecute,
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE,
                                       shell=True)

        while not os.path.isfile(self.commandsfile):
            if waitcounter == -1:
                print "File %s does not exist. Waiting for its creation" % self.commandsfile
                waitcounter += 1
            time.sleep(1)
            # Though checking for file creation every second, it shows a message every 2 min.
            waitcounter += 1
            if waitcounter == 120:
                print "waiting..."
                # print os.path.isfile(self.commandsfile)
                waitcounter = 0
        if verbose:
            print "COMMUNICATION FILE %s FOUND" % self.commandsfile
        return True

    def is_commands_file_modified(self):
        last_modified = os.stat(self.commandsfile).st_mtime
        if last_modified > self.comandsfile_modifiedtime:
            self.comandsfile_modifiedtime = last_modified
            return True
        else:
            return False

    def get_command_to_execute(self):
        # Open file with commands and read all lines
        commandfile = open(self.commandsfile)
        commandalllines = commandfile.readlines()
        # print commandalllines
        # Get only the last line
        commandline = commandalllines[len(commandalllines)-1]
        # If the read command line contains the keyword -toexecute-
        if commandline.__contains__("-toexecute-"):
            # Get the command which should be right before the keyword
            commandtoexecute = commandline.split("-toexecute-")[0]
            if options.simulation:
                print "Simulation Mode: Executing in aos-gns.aiv.alma.cl: \033[33m %s \033[39m" % commandtoexecute
                uidcode = 'uid://simulation/uid/000'
            else:
                print "Executing in aos-gns.aiv.alma.cl: \033[33m %s \033[39m" % commandtoexecute
                process = subprocess.Popen(commandtoexecute,
                                           stdout=subprocess.PIPE,
                                           stderr=subprocess.PIPE,
                                           shell=True)
                out, err = process.communicate()
                print out
                print err
                outputfile = open(self.executionoutputfile, 'w')
                outputfile.write('*******************************\n')
                outputfile.write('EXECUTED COMMAND:\n')
                outputfile.write('%s\n' % commandtoexecute)
                outputfile.write('*******************************\n')
                outputfile.write(out)
                outputfile.write(err)
                outputfile.close()
                subprocess.call(['chmod', '666', self.executionoutputfile])
                # Check whether the output has some uid:// code
                print "Execution finished."
                time.sleep(1)
                if process.returncode == 0 and out.__contains__('uid'):
                    out.find('uid')
                    # We get the string after uid and cut it at the next ' ' or '\n'
                    uidcode = out[out.find('uid'):].split(' ')[0].split('\n')[0]
                    print "Execution UID: %s" % uidcode
                else:
                    # If not, just leave it as an empty string
                    uidcode = ''
            # We replace -toexecute- keyword by -executed- together with the uid of the execution if any
            commandalllines[len(commandalllines)-1] = commandline.replace('-toexecute-', '-executed-%s' % uidcode)
            auxoutfile = open('AntennaIntegratorAOS.auxfile', 'w')
            for outlines in commandalllines:
                auxoutfile.write('%s' % outlines)
            auxoutfile.close()
            subprocess.call(['mv', 'AntennaIntegratorAOS.auxfile', self.commandsfile])
            subprocess.call(['chmod', '666', self.commandsfile])
        elif commandline.__contains__("-exit-"):
            print "Antenna Integration exit command found."
            subprocess.call(['rm',  self.commandsfile])
            sys.exit(0)

if __name__ == '__main__':
    parser = OptionParser()
    # parser.add_option("-a", "--arrayName", dest="inputarrayname", default='', help="Array Name")
    parser.add_option("-S", "--simulation", action='store_true', dest="simulation", default=False,
                      help="Execute in simulation. No commands will be executed.")
    (options, args) = parser.parse_args()
    aiaos = AntennaIntegratorAOS()  # options.inputarrayname)
    aiaos.checkcommandsfile(verbose=True, rejectoldfile=True)
    while aiaos.checkcommandsfile():
        if aiaos.is_commands_file_modified():
            aiaos.get_command_to_execute()
        time.sleep(1)
