import os
import sys

from scipy.optimize import leastsq
from scipy import *
from numpy import *
from pylab import *
from taskinit import *

import analysisUtils as aU

def analyse(inputms):

    vis=aU.locate(inputms)[0]
    
    #get info about observations.
    tb.open(vis+'/ANTENNA')
    pos=tb.getcol('POSITION')
    ant=tb.getcol('NAME')
    pad=tb.getcol('STATION')
    tb.close()
    
    # get name of source.
    tb.open(vis+ '/SOURCE')
    src=tb.getcol('NAME')[0]
    tb.close()
    
    # get time interval
    tb.open(vis)
    t_int=unique(tb.getcol('INTERVAL'))[0]
    scan=tb.getcol('SCAN_NUMBER')
    nscans=size(unique(scan))
    sid=tb.getcol('STATE_ID')
    t0=tb.getcol('TIME')
    tb.close()
    
    #look for on/off scans
    if(os.path.isdir(vis+'/STATE')):
      tb.open(vis+'/STATE')
      obsmode=tb.getcol('OBS_MODE')
      for i in range(len(obsmode)):
          if 'ON_SOURCE' in obsmode[i]:on = i
          if 'OFF_SOURCE' in obsmode[i]:off = i
      scanon=unique(scan[find(sid==on)])    
      scanoff=unique(scan[find(sid==off)])
      if size(scanoff) > 0:
          onoff=1
      else:
          onoff=0

    imagefile=vis.split('/')[-1][:-3]
    [t,cc]=aU.getSpectralData(vis, 0)
    [t,ac0]=aU.getSpectralAutoData(vis, 0,0)
    [t,ac1]=aU.getSpectralAutoData(vis, 1,0)

    nint_per_scans=shape(cc)[2]/nscans
    print "%i integrations per scans with %i scans" %(nint_per_scans, nscans)

    dirname= inputms.split('.ms')[0]
    if not os.path.isdir(dirname): os.system('mkdir %s'%dirname)

    #do a first fit on the CC collapsed in time to find location and width of peak.
    x=arange(shape(cc)[1])
    zoom_tmp=arange(3000,5000)
    lmx=[0,0,0,1,0]
    mx=array([0,0,0,120,0])
    lmi=[0,1,0,1,0]
    mi=array([0,0,0,1,0])

    
    figure(12, figsize=(14, 8))
    clf()
    subplot(2,1,1)
    spectrogram=transpose(cc[0,zoom_tmp,:])
    max_spectrogram=abs(spectrogram).max()
    imshow(abs(spectrogram), aspect='auto',vmin=0, vmax=0.9*max_spectrogram)
    hot();
    ylabel('integration number')
    xlabel('channel')
    title('%s spectrogram' %imagefile)
    subplot(2,1,2)
    plot(x[zoom_tmp],abs(mean(spectrogram,0)))
    line_t=raw_input('Enter approximate channel of line you want to fit:   ')
    width_t=raw_input('Enter approximate width of line you want to fit:   ')
    amp_t=raw_input('Enter approximate amplitude of line you want to fit:   ')
    p,f,gerr, chi2=aU.onedgaussfit(x[zoom_tmp],abs(mean(spectrogram,0)),params=[0,float(amp_t), float(line_t), float(width_t),0],limitedmin=lmi, minpars=mi,limitedmax=lmx, maxpars=mx, fixed=[0,0,0,0,1],shh=True)
  
    plot(x[zoom_tmp], f, 'r')
    vlines(p[2]-3*p[3]-40, 0,1.5*p[1], linestyle='dashed')
    vlines(p[2]-3*p[3]-20, 0,1.5*p[1], linestyle='dashed')
    savefig('%s/%s_spectrogram.png' %(dirname,imagefile))

    # further fits will have fixed  line position and width
    amp=p[1]
    line=p[2]
    intline=int(p[2])
    wd=p[3]
    fx=array([0,0,1,1,0])
    print "doing further fits to auto/cross corr with fixd line postion %i width %f" %( line , wd)
        
       
    zoom=arange(intline-400,intline+400)
    zoom1=arange(intline-100,intline+100)

    if onoff == 1:            
        coherence_loss1=zeros(nscans/2)
        coherence_loss2=zeros(nscans/2)
        figure(1, figsize=(16, 7))
        for i in range(nscans/2):
            clf()
            j=2*i+1
            sfi=nint_per_scans*(2*i)
            efi=nint_per_scans*(2*i+1)-1
            sfj=nint_per_scans*(2*i+1)
            efj=nint_per_scans*(2*i+2)-1
            print sfi, efi, sfj, efj

            l0=floor(line-3*wd-40)
            l1=ceil(line-3*wd-20)
            print l0, l1
            ac0_on=abs(mean(ac0[0,:,sfi:efi],1))
            ac0_offline_on=abs(mean(mean(ac0[0,l0:l1,sfi:efi],1)))
            ac1_on=abs(mean(ac1[0,:,sfi:efi],1))
            ac1_offline_on=abs(mean(mean(ac1[0,l0:l1,sfi:efi],1)))
            cc_on =abs(mean(cc[0,:,sfi:efi],1))
            ac0_off=abs(mean(ac0[0,:,sfj:efj],1))
            ac0_offline_off=abs(mean(mean(ac0[0,l0:l1,sfj:efj],1)))
            ac1_off=abs(mean(ac1[0,:,sfj:efj],1))
            ac1_offline_off=abs(mean(mean(ac1[0,l0:l1,sfj:efj],1)))
            cc_off=abs(mean(cc[0,:,sfj:efj],1))
          

            scale0=ac0_offline_on/ac0_offline_off
            scale1=ac1_offline_on/ac1_offline_off
            print scale0, scale1

            ac0_off=ac0_off*scale0
            ac1_off=ac1_off*scale1
            diff0=(ac0_on-ac0_off)
            diff1=(ac1_on-ac1_off)

            
            ccp=sqrt(abs(diff0*diff1/(ac0_on*ac1_on)))
            ccm=cc_on
            coherence_loss1[i]=abs(ccm[intline]/ccp[intline])
                
            pac0,fac0,gerr,chi2=aU.onedgaussfit(zoom,diff0[zoom], params=[0, amp, line, wd, 1],fixed=fx, shh=True)
            pac1,fac1,gerr,chi2=aU.onedgaussfit(zoom,diff1[zoom], params=[0, amp, line, wd, 1],fixed=fx, shh=True)
            h0=mean(ac0_on[intline-wd/2:intline+wd/2])
            h1=mean(ac1_on[intline-wd/2:intline+wd/2])
            pcc, fcc,gerr,chi2=aU.onedgaussfit(zoom,cc_on[zoom], params=[1, amp, line, wd, 1], fixed=fx,shh=True)

            cc_predicted=sqrt( (pac0[1]*pac1[1])/((h0+pac0[1])*(h1+pac1[1])))
            cc_measured=pcc[1]
            coherence_loss2[i]=abs(cc_measured/cc_predicted)

        
            figure(1)
            clf()
            subplot(3,1,1)
            plot(zoom, ac0_on[zoom],'b')
            plot(zoom, ac0_off[zoom],'g')
            plot(zoom, ac1_on[zoom],'r')
            plot(zoom, ac1_off[zoom],'c')
            ylabel('Auto-Corr Amp')
            title('%s scan %i %s %s' %(imagefile,2*i,ant[0],ant[1]))
            #t3='s1= %1.3g ; s1+n1= %1.3g' %( pac0[1], pac0[0]+pac0[1])
            #figtext(0.73, 0.77, t3, color='k')
            legend(['Ant 1 ON','Ant 1 OFF','Ant 2 ON','Ant 2 OFF'])
            subplot(3,1,2)       
            plot(zoom,diff0[zoom])
            plot(zoom,diff1[zoom])
            hlines(0, intline-300,intline+300, linestyle='dashed')
            ylabel('Auto-Corr Amp')
            legend(['Ant 1 ON-OFF','Ant 2 ON-OFF'])
            subplot(3,1,3)
            plot(zoom,cc_on[zoom],'b')
            plot(zoom,cc_off[zoom],'g')
            plot(zoom,ccp[zoom],'r')       
            
            hlines(0, intline-300,intline+300, linestyle='dashed')
            legend(['Cross-Corr ON','Cross-Corr OFF','Cross-Corr predicted'])
            t0='C measured = %1.3g' % (coherence_loss1[i])
            t1='C from fits  = %1.3g' % (coherence_loss2[i])
            ylabel('Cross-Corr Amp')
            ylim([0,4*amp])
            figtext(0.73, 0.23, t0, color='k')
            figtext(0.73, 0.21, t1, color='k')                
            savefig('%s/%s_scan_%i_diff_zoom.png' %(dirname, imagefile,2*i), format='png')
            raw_input()
      
        return coherence_loss1, coherence_loss2
            
    elif onoff == 0:
        print "no off scans"
        coherence_loss2=zeros(nscans)
        figure(2, figsize=(16, 8))
        figure(3, figsize=(16, 5))
        for i in range(nscans):
            clf()
            sfi=nint_per_scans*(2*i)+1
            efi=nint_per_scans*(2*i+1)-1
            print sfi, efi
            
            ac0_tmp=mean(abs(ac0[0,:,sfi:efi]),1)
            ac1_tmp=mean(abs(ac1[0,:,sfi:efi]),1)
            cc_tmp=abs(mean(cc[0,:,sfi:efi],1))

            pac0,fac2,gerr, chi2=aU.onedgaussfit(zoom1,ac0_tmp[zoom1], params=[1, 0.02, line, wd, 0],fixed=fx, shh=False)
            pac1,fac3,gerr, chi2=aU.onedgaussfit(zoom1,ac1_tmp[zoom1], params=[1, 0.02, line, wd, 0],fixed=fx, shh=False)
            pcc, fcc,gerr,chi2=aU.onedgaussfit(zoom,cc_tmp[zoom], params=[0, 0.005, line, wd, 0],fixed=fx, shh=False)

            cc_predicted=sqrt( (pac0[1]*pac1[1])/((pac0[0]+pac0[1])*(pac1[0]+pac1[1])))
            cc_measured=pcc[1]
            coherence_loss2[i]=abs(cc_measured/cc_predicted)
            print coherence_loss2[i]
            
            figure(2)
            clf()
            subplot(3,1,1)
            plot(zoom, ac0_tmp[zoom])
            plot(zoom1, fac2,'r')
            t0='s1= %1.3g ; s1+n1= %1.3g' %( pac0[1], pac0[0]+pac0[1])
            figtext(0.73, 0.8, t0, color='k')
            title('%s scan %i' %(imagefile,2*i))
            ylabel('%s' % ant[0])
            legend(['mean(auto-corr)','fit'])
            subplot(3,1,2)
            plot(zoom, ac1_tmp[zoom])
            plot(zoom1, fac3,'r')
            t0='s2= %1.3g ; s2+n2= %1.3g' %( pac1[1], pac1[0]+pac1[1])
            figtext(0.73, 0.57, t0, color='k')
            ylabel('%s' % ant[1])
           
            subplot(3,1,3)
            plot(zoom,cc_tmp[zoom])
            plot(zoom, fcc,'r')
            legend(['mean(cross-corr)','fit'])
            t0='cc_predicted = %1.3g' % (cc_predicted)
            t1='cc_measured  = %1.3g' % (cc_measured)
            t2='C from fits = %1.3g' % (coherence_loss2[i])
            figtext(0.73, 0.25, t0, color='k')
            figtext(0.73, 0.235, t1, color='k')
            figtext(0.73, 0.22, t2, color='k')
            savefig('%s/%s_scan_%i_zoom.png' %(dirname, imagefile,2*i), format='png')

            figure(3)
            clf()
            subplot(2,1,1)
            plot(x, ac0_tmp)
            title('%s scan %i' %(imagefile,2*i))
            ylabel('%s' % ant[0])
            legend(['mean(auto-corr)'])
            subplot(2,1,2)
            plot(x, ac1_tmp)
            ylabel('%s' % ant[1])
            savefig('%s/%s_scan_%i_full.png' %(dirname, imagefile,2*i), format='png')

        return coherence_loss2


def residuals(p, y, x):
    err = y - peval(x,p)
    return err

def peval(x, p):
    #fit a 1D gaussian plus a linear term
    return p[0]+ p[1]*exp(-(x-p[2])**2/(2*p[3]**2)) +p[4]*(x-p[2])

def dofit(x,y, p0, quiet=True):    
    plsq = leastsq(residuals, p0, args=(y, x), maxfev=2000)
    
    if 0:
        clf()
        figure(1);
        subplot(2,1,1)
        plot(x,y, '.-')
        plot(x,peval(x,plsq[0]),'r')
        
        subplot(2,1,2)
        plot(x,y, '.-')
        plot(x,peval(x,plsq[0]),'r')
        xlim([plsq[0][2]-100, plsq[0][2]+100])

    if not quiet:
        print "Final parameters"
        for i in range(len(p0)):
            print " %.4f " % (plsq[0][i])

    return plsq[0]


def compute_corr(d1, d2):
    
    n=400
    cc=zeros(n)
    delay=zeros(n)
    for i in range(n):
        delay[i]=int(-n/2+i)
        d2_tmp=roll(d2,-n/2+i)
        cc[i]=corrcoef(d1, d2_tmp)[0,1]

    return [delay,cc]







