#!/usr/bin/env python
# R.Lucas 2009-07
# THis casa script will compute ITRF pad positions from the WGS84 positions
# in the AIV Operations manual
# - adjusted the height of the upper pads to match the measured TF1-TF5 baseline
# - also adjusted the TF5 pad errors.
me = metool.create()
class AntennaPositions:

    def __init__(self):

        # data from the AIV OPerations manual:
        # the pads positions measures in ITRF.
        self._padsWGS84 = {}
        self._padErrors={}
        self._padsITRF = {}
        self._padsWGS84['TF1'] = me.position('WGS84',
                                             qa.quantity('-67.58.49.781'),
                                             qa.quantity('-23.04.24.354'),
                                             '2967m')
        self._padErrors['TF1'] = pl.array([ 0.0, 0.0, 0.0]) 

        self._padsWGS84['TF2'] = me.position('WGS84',
                                             qa.quantity('-67.58.49.663'),
                                             qa.quantity('-23.04.25.124'),
                                             '2967m')
        self._padErrors['TF2'] = pl.array([ 0.0, 0.0, 0.0]) 

        self._padsWGS84['TF3'] = me.position('WGS84',
                                             qa.quantity('-67.58.49.548'),
                                             qa.quantity('-23.04.25.897'),
                                             '2967m')
        self._padErrors['TF3'] = pl.array([ 0.0, 0.0, 0.0]) 

        self._padsWGS84['TF4'] = me.position('WGS84',
                                             qa.quantity('-67.58.49.430'),
                                             qa.quantity('-23.04.26.670'),
                                             '2967m')
        self._padErrors['TF4'] = pl.array([ 0.0, 0.0, 0.0]) 

        self._padsWGS84['TF5'] = me.position('WGS84',
                                             qa.quantity('-67.58.47.439'),
                                             qa.quantity('-23.04.24.050'),
                                             '2974.8m')
        self._padErrors['TF5'] = pl.array([ -0.0711, 0.0051,  -0.0483])

        self._padsWGS84['TF6'] = me.position('WGS84',
                                             qa.quantity('-67.58.47.321'),
                                             qa.quantity('-23.04.24.823'),
                                             '2974.8m')
        self._padErrors['TF6'] = pl.array([ 0.0, 0.0, 0.0]) 
        
        self._padsWGS84['TF7'] = me.position('WGS84',
                                             qa.quantity('-67.58.47.206'),
                                             qa.quantity('-23.04.25.596'),
                                             '2974.8m')
        self._padErrors['TF7'] = pl.array([ 0.0, 0.0, 0.0]) 

        for pad in self._padsWGS84:
            self._padsITRF[pad] =  me.measure (self._padsWGS84[pad], 'ITRF')


            
    def getAntennaHeight(self, antenna = 'DV01'):
        if antenna.rstrip(string.digits) == 'DV':
            return qa.quantity('7.000m')
        elif antenna.rstrip(string.digits) == 'PM':
            return qa.quantity('7.500m')
        elif antenna.rstrip(string.digits) == 'DA':
            return qa.quantity('7.500m')

    def getPadPosition(self, pad):
        me.doframe(self._padsITRF[pad])
        return me.addxvalue(self._padsITRF[pad])['value'] + self._padErrors[pad]

    def getAntennaPosition(self, pad, antenna):
        me.doframe(self._padsITRF[pad])
        antennaVector = me.measure(me.baseline('AZEL', qa.quantity('0deg'), 
                                               qa.quantity('90deg'),
                                               self.getAntennaHeight(antenna)), 
                                   'ITRF')
        return self.getPadPosition(pad) + me.addxvalue(antennaVector)['value']

if __name__ == "__main__":

    ap = AntennaPositions()
    
    # what we have in the system from COmputing
    defPM03 = pl.array([2202243.530, -5445225.169, -2485282.440])
    defDV01 = pl.array([2202176.056, -5445240.616, -2485287.800])
    # what we measured (after some iterations)
    truePM03 = defPM03 + pl.array([(-1.5056) +(-0.0205) + ( 0.0458) +(-0.000139), 
                                       (-0.8888) +( 0.0838) + (-0.1123) +( 0.001109), 
                                       +(-0.0303) +( 0.0244) + (-0.0467) +( 0.000292)])
    # what we get from pad measurements
    myDV01 = ap.getAntennaPosition('TF1', 'DV01')
    myPM03 = ap.getAntennaPosition('TF5', 'PM03')
    myBase = myPM03 - myDV01
    defBase = defPM03 - defDV01
    trueBase = truePM03 - defDV01

    print 'default baseline: %s m' % defBase
    print 'error:            %s m' % (defBase-trueBase) 
    print 'error length:     %10.3f m' % sqrt(sum((defBase-trueBase)**2))
    print 'this baseline:    %s m' % myBase
    print 'error:            %s m' % (myBase-trueBase) 
    print 'error length:     %10.3f m' % sqrt(sum((myBase-trueBase)**2))

    print 'new TF1: %s m (ITRF) '% ap.getPadPosition('TF1')
    print 'new TF2: %s m (ITRF) '% ap.getPadPosition('TF2')
    print 'new TF3: %s m (ITRF) '% ap.getPadPosition('TF3')
    print 'new TF4: %s m (ITRF) '% ap.getPadPosition('TF4')
    print 'new TF5: %s m (ITRF) '% ap.getPadPosition('TF5')
    print 'new TF6: %s m (ITRF) '% ap.getPadPosition('TF6')
    print 'new TF7: %s m (ITRF) '% ap.getPadPosition('TF7')
    
    # get the error in Site coordiantes

    error = myBase-trueBase
    errorITRF = me.baseline('ITRF', qa.quantity(error[0],'m'), qa.quantity(error[1],'m'), qa.quantity(error[2],'m'))
    errorAZEL = me.measure(errorITRF, 'AZEL')

