#!/usr/bin/env python

 
"""
Script Edit Pad Delay for a given Pad on the TMCDB, based on the latest Tiltmenter measurement.

Usage: 

EditAnAwFromTilts.py -a DV0

Script reads AN0 and AW0 from latest Tiltmeter measurement, checks that the measurement was done after
the latest antenna move. Computes new AN and AW from that measurement and creates new XML file.

Script then gives the option to do the ImportPointingModel.py command (user has to type 'y' or 'yes).

Otherwise it prints the ImportPointingModel.py command on screen (and the new AN and AW values).

smartin: 01/2017: Automatic import of AN0 and AW0. No need to have a cvs account anymore.(modifications to
                  tmcdbPlayer_tmp.py and plot_tmcdb_content.py were needed.

#"""

# Execution starts here

import os, sys
import socket
import getpass
import pickle
import glob
from optparse import OptionParser;


username = getpass.getuser()

sys.path.append('/users/%s/AIV/science/scripts/R10.4_WORKING/' % username)

sys.path.append('/users/%s/AIV/science/analysis_scripts/' % username)

from pylab import *
import tmUtils as tm
from datetime import datetime,timedelta
import warnings
from  getNominalPtBandOffsets import getNominal
import re
import tmcdbPlayer_tmp as tp
import time;

import plot_tmcdb_content as tk

#################################
# Defining some useful functions
#################################

def grep_file(ff, search_term):
    line_found = ''
    for ffline in open(ff, 'r'):
        if re.search(search_term, ffline):
            print ffline,
            line_found = ffline
            if ffline is None:
                print 'no matches found'
    return line_found


def system_except(cmd):
    ret = os.system(cmd)
    if ret != 0:
        raise Exception("Command '%s' returned non-zero exit status %d" % (cmd, ret))


def read_monitor_data(filename):
    """
    Read Tiltmeter monito data
    """
    if not os.path.exists(filename):
        print "%s not found" % filename
        sys.exit()

    an0 = ''
    aw0 = ''
    date_tilt = ''
    print filename
    f = open(filename, 'r')
    last_bit = f.readlines()[-1:]
    for line  in last_bit:
        print line
        line=line.rstrip('\n')
        print line
        if "DV" in filename:
            (date_time, antenna, an0, aw0, xOffset, yOffset, outdoorTemp, intemp) = line.split()
        if "DA" in filename:
            (date_time, antenna, an0, aw0, xOffset, yOffset, outdoorTemp) = line.split()
        # print date_time, antenna,an0,aw0
        date_tilt = date_time.split('T')[0]
    return date_tilt,an0,aw0


#################################
# Options
#################################

parser = OptionParser()

parser.add_option("-a", "--antenna", dest="antenna",help="Antenna to get parameters. Ex.: DV01", default=None)

parser.add_option("--ip", "--inputPointingXml",
                  dest="inputPointingXml",
                  default=None,
                  help="input Xml generated from the TMCDB by ExportPointingModel")

parser.add_option("--ia", "--inputArrayXml",
                  dest="inputArrayXml",
                  default=None,
                  help="input Xml generated from the TMCDB by ExportArrayConfiguration")

parser.add_option("--op", "--outputPointingXml",
                  dest="outputPointingXml",
                  default="PointingModel-current.xml",
                  help="output Xml file, for ImportPointingModel.py, default 'PointingModel-current.xml'")

parser.add_option("--oa", "--outputArrayXml",
                  dest="outputArrayXml",
                  default="ArrayConfiguration-current.xml",
                  help="output Xml file, for ImportTiltTerm.py, default 'ArrayConfiguration-current.xml'")


# parser.add_option("--an", "--an_term", dest="an_term", help="New AN term", default="")
# parser.add_option("--aw", "--aw_term", dest="aw_term", help="New AW term", default="")
# parser.add_option("--an0", "--an0_term", dest="an0_term", help="New AN0 term", default="")
# parser.add_option("--aw0", "--aw0_term", dest="aw0_term", help="New AW0 term", default="")
parser.add_option("-T", "--tilt_terms", dest="tiltterms", default=None,
                  help="User input of comma separated tilt terms as AN0,AW0 or AN0,AW0,AN,AW")

parser.add_option("--AI", "--AntennaIntegrator", action="store_true", dest="ai",
                  help="Running as part of the Antenna Integrator (avoids the need to ssh@aos-gns)", default=False)


(options, args) = parser.parse_args()


ant = options.antenna
if ant is None:
    print "ERROR: No antenna provided"
    sys.exit()

print "#########################################"
print "Processing  Antenna %s" % (ant)
print "#########################################"
#################################
# Find Pad & Last Move data
#################################
# TODO We could get this directly from the XML, right? Since we already exported the TMCDB data, there is no
# need to use the tk to query it again
if options.ai:
    date_move, pad_last_move = tk.checkMoveData(ant, False, True, '/groups/science/AIV/PadData/antennaMoves.txt')
else:
    date_move, pad_last_move = tk.checkMoveData(ant)
print "Antenna %s was moved on '%s' to pad %s " % (ant, date_move, pad_last_move)

# If tilt terms not provided we get them from the observations normally
if options.tiltterms is None:
    #################################
    # Time
    #################################
    # COMMENTED BECAUSE IT IS NOT USED
    #today = tk.getCurrentTime()
    #date = today.split('T')[0]

    #################################
    # Update Tilmeter Monitor File
    #################################

    if options.ai:
        print 'NOTE: Running as part of Antenna Integrator. ' \
              'It is assumed that tiltmeter files are updated in /tmp/tiltmeterfiles/'
    else:
        os.system('scp -r '
                  '%s@ape1-gns.osf.alma.cl:/groups/science/data/RADIO/POINT/Tiltmeter/*_tiltmeterValues_backup.txt '
                  '/tmp/tiltmeterfiles/' % username)

    # file = '/tmp/tiltmeterfiles/%s_tiltmeterValues_backup.txt' % ant

    #################################
    # Read Tilmeter Monitor File
    #################################

    file = '/tmp/tiltmeterfiles/%s_tiltmeterValues_backup.txt' % ant

    ###############################
    # Get latest tilt measurements
    ###############################

    date_tilt, an0, aw0 = read_monitor_data(file)
    print "Tilt Measurement done on %s " % date_tilt

    ###############################
    # Compare date of latest tilt measurements
    # with last antenna move
    ###############################

    tilt_date = time.strptime(date_tilt, "%Y-%m-%d")
    # print tilt_date
    date_antenna_move = time.strptime(date_move, "%Y-%m-%d")
    # print date_antenna_move

    if tilt_date >= date_antenna_move:
        print "\033[32m#################################################"
        print "The tilt measurements was taken AFTER the antenna move   "
        print "Proceeding with AN & AN updated based on AW0 and AW0 values "
        print "#################################################\033[0m"

    if tilt_date < date_antenna_move:
        print "\033[31m#################################################"
        print "ERROR: The tilt measurements is older than the antenna move "
        print "Please Run a Titlmeter Measurement on %s and try again"%ant
        print "#################################################\033[0m"

        sys.exit()

# else if tiltterms are provided we check user did input two values
else:
    print "Tilt terms AN0 and AW0 provided by the user"
    input_tilt_len = len(options.tiltterms.split(','))
    if input_tilt_len != 2 and input_tilt_len != 4:
        print "ERROR: Expected 2 or 4 values for the tilt terms but %s provided" % input_tilt_len
        sys.exit()
    else:
        an0 = options.tiltterms.split(',')[0]
        aw0 = options.tiltterms.split(',')[1]
        if input_tilt_len == 4:
            new_an = options.tiltterms.split(',')[2]
            new_aw = options.tiltterms.split(',')[3]

# If new_an and new_aw have not yet been created
if 'new_an' not in locals():
    #  For pads at AOS AN=(AN0-5.92"); AW=(AW0+13.14")
    new_an = float(an0)-5.92
    new_aw = float(aw0)+13.14
print "New AN0 and AW0: %s , %s " % (str(an0), str(aw0))
print "New AN and AW: %s , %s " % (str(new_an), str(new_aw))
print "#################################################\033[0m"

#################################
# Time
#################################

today = tk.getCurrentTime()

#################################
# Input/Output File
#################################

outfileANAW = options.outputPointingXml
outfileAN0AW0 = options.outputArrayXml

if options.inputPointingXml is None:
    print '### Exporting TMCDB PointingModel ###'
    tmcdb_pointing_model = os.popen('ExportPointingModel.py -a %s' % ant).read()
    infile_pointing = tmcdb_pointing_model.split()[-1]
else:
    print "Exported Pointing model provided by user"
    infile_pointing = options.inputPointingXml
print "Pointing models for antenna %s from configuration 'CURRENT.AOS' saved to %s " % (ant, infile_pointing)

# print "Input  XML file %s " % infile
# print "Output XML file %s " % outfile
if options.inputArrayXml is None:
    print '### Exporting TMCDB Array Configuration ###'
    tmcdb_array_configuration = os.popen('ExportArrayConfiguration.py -a %s' % ant).read()
    infile_array = tmcdb_array_configuration.split()[-1]
else:
    print "Exported Array Configuration provided by user"
    infile_array = options.inputArrayXml
print "Pointing models for antenna %s from configuration 'CURRENT.AOS' saved to %s " % (ant, infile_array)


################################
# Edit AN, AW, AN0, and AW0
################################
# SHOW INFO IN ORIGINAL TMCDB EXPORTED FILES
term1 = 'Coeff name="AN"'
term2 = 'Coeff name="AW"'
term3 = 'MetrologyCoefficients'

print "ORIGINAL xml files and coefficients are \n %s \n %s " % (infile_pointing, infile_array)

grep_file(infile_pointing, term1)
grep_file(infile_pointing, term2)
grep_file(infile_array, term3)

# CREATE MODIFIED OUTPUT LINES MODIFYING THE CORRESPONDING PARAMETERS
new_line1 = '        <Coeff name="AN" value="%s">\n' % new_an
new_line2 = '        <Coeff name="AW" value="%s">\n' % new_aw
new_line3 = '        <MetrologyCoefficients AN0="%s" AW0="%s" version="0"/>\n' % (an0, aw0)
# print "new line is %s " % new_line1
# print "new line is %s " % new_line2

print "NEW xml files and coefficients are \n %s  \n %s" % (outfileANAW, outfileAN0AW0)

# Modifying the AN and AW in the Pointing Model file
old_f = open(infile_pointing,'r')
f = open(outfileANAW,'w')
for line in old_f:
    if term1 in line:
        line = new_line1
    if term2 in line:
        line = new_line2
    f.write(line)
f.close()
old_f.close()

# Modifying the AN0 and AW0 in the Array Configuration file
old_f = open(infile_array,'r')
f = open(outfileAN0AW0,'w')
for line in old_f:
        if term3 in line:
            line = new_line3
        f.write(line)
f.close()
old_f.close()


grep_file(outfileANAW, term1)
grep_file(outfileANAW, term2)
grep_file(outfileAN0AW0, term3)


os.system('chmod ugo+rwx -R %s' % outfileANAW)
os.system('chmod ugo+rwx -R %s' % outfileAN0AW0)

answer = raw_input('Would you like to proceed with the Import steps. Be careful, if answer is "y", '
                   'you will proceed with importing these model to the TMCDB. (y|[n]) ?')

if answer == 'y' or answer == 'Y' or answer == 'yes' or answer == 'YES' or answer == 'Yes':
    cmd_import = 'ImportPointingModel.py -f %s -m "Automatic Updating %s AN and AW based on AN0 and AW0" ' \
                 % (outfileANAW, ant)
    print cmd_import
    system_except(cmd_import)
    print "New AN and AW loaded in the TMCDB "
    cmd_import = 'ImportTiltTerm.py -x %s -m "Automatic Updating %s AN0 and AW0" ' \
                 % (outfileAN0AW0, ant)
    print cmd_import
    system_except(cmd_import)
    print "New AN0 and AW0 loaded in the TMCDB "

else:
    print 'You have chosen to do the Edit manually.'
    print "Edit AN AW AN0 AW0 in the TMCDB Manually "
    print "or run"
    print '\033[95m ImportPointingModel.py -f %s -m "Updating %s AN and AW based on AN0 and AW0, ' \
          '[YOUR COMMENT] \033[0m" ' % (outfileANAW, ant)
    print '\033[95m ImportTiltTerm.py -x %s -m "Updating %s AN0 and AW0, [YOUR COMMENT]" \033[0m '\
          % (outfileAN0AW0, ant)

# print "##################################"
# print "Input AN0 and AW0 on the TMCDB for %s on pad %s " % (ant, pad_last_move)
# print "(AN0, AW0) = (%.1f,%.1f) " % (float(an0),float(aw0))
# print "##################################"


