#!/usr/bin/env python
#ExtractFlags.py
#written by dbarkats
#$Id: ExtractFlags.py,v 1.9 2014/02/27 19:49:15 dbarkats Exp $

from optparse import OptionParser
import analysisUtils as au
import os

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''


# Execution starts here
if __name__ == '__main__':
    usage = '''
    ExtractFlags.sh --help

    This script should be run from the .sh ( not the .py).
    It will extract the list of flags for a selected antenna from an ASDM.

    It also prints below the list of flags the subScan timing for all subscans in the ASDM

    '''
    # Initialize parser
    parser = OptionParser()

    parser.add_option("-u", "--uid", dest="uid",
                  help="uid of input ASDM to be analysed")

    parser.add_option("-s", "--subscans",
                      dest="subscans",
                      action="store_true",
                      default=False,
                      help=" -s will print out the subscan timing information. Default = False")

    parser.add_option("-f", "--flags",
                      dest="flags",
                      action="store_true",
                      default=False,
                      help=" -f will print out the flag timing. Default = False")
        
    parser.add_option("-a", "--antennaList",dest = 'antennaList',default = None,
                      help="comma-separated list of antennas for which to extract flags. Default = 1st antenna in ASDM")

    (options, args) = parser.parse_args()

    from asdm import ASDM
    
    uid = options.uid
    antList = au.readAntennasFromASDM(uid)

    if options.antennaList == None:
        antennas = antList
    else:
        antennas = options.antennaList.split(',')

    a = ASDM()
    a.setFromFile(uid,True)
    subscanTable = a.subscanTable().get()  

    # get the start time of the ASDM
    dateStart, dateStart_mjdSec = au.getObservationStartDateFromASDM(uid)
    dateEnd, dateEnd_mjdSec = au.getObservationEndDateFromASDM(uid)
    # start of 1st subscan
    scanStartTime_mjdSec = subscanTable[0].startTime().get() *1e-9
    # end of 1st subscan
    scanEndTime_mjdSec = subscanTable[-1].endTime().get() *1e-9
                          
    delta_start = scanStartTime_mjdSec - dateStart_mjdSec
    delta_obs = dateEnd_mjdSec - scanStartTime_mjdSec
    delta_end = dateEnd_mjdSec - scanEndTime_mjdSec 

    au.printSwVersionFromASDM(uid)

    print '#############################'
    print 'ASDM started at %s and lasted %d seconds. \nTime to start 1st scan = %d seconds \nTime to end observation= %d seconds'%(dateStart,delta_obs, delta_start, delta_end)

    #print out flagging info
    if (options.flags):
        flagTable = a.flagTable().get()
        for ant in antennas:
            antI = antList.index(ant)
            print '\n#### FlagList for %s ####' %ant
            for row in flagTable: 
                if (row.antennaId()[0].get() == antI):
                    if row.startTime().get() > row.endTime().get():
                        flagThis = '         <<<<<<<<<<<<<<< startTime later than endTime'
                        print  bcolors.FAIL, str(row.startTime())[11:24], str(row.endTime())[11:24], row.reason(), flagThis, bcolors.ENDC
                    elif (row.endTime().get() - row.startTime().get())/1e9 > 10:
                        flagThis = '         <<<<<<<<<<<<<<< Flag lasts more than 10s'
                        print  bcolors.FAIL, str(row.startTime())[11:24], str(row.endTime())[11:24], row.reason(), flagThis, bcolors.ENDC
                    else:
                        print str(row.startTime())[11:24], str(row.endTime())[11:24], row.reason()

    #print out subscan timing
    if options.subscans:                
        print '\n#### SubScan timing List #####'
        print '    START        END     scan subscan'
        for row in subscanTable:
            print str(row.startTime())[11:24], str(row.endTime())[11:24], str(row.scanNumber()), str(row.subscanNumber())
 
    

