#!/usr/bin/env python
#ExtractFocusRun.py
#written by RLucas (2009-04-17)
#hacked version for R7.1.0, works after radiosetup, only for one antenna 
#
#will extract the data from  the ADSM file (need to change the file name)
#and dump it to a file so we can save  it in a focus_table.
#Focus tables are being stored in /groups/science/data/ANTENNA/RPOINT/Focus/
#focus file name format is  focus_xyz_20090417.txt
#Example:
# ExtractFocusRun -f uid___X54_X35fa_X1 uid___X54_X3a30_X1 [...] -o 
# focus_xyz_20090417.txt
#
#$Id: ExtractFocusRun_7.1.1.py,v 1.1 2010/10/12 12:50:44 rkneissl Exp $

from string import *
import os,sys,time
import casac
from numpy import *
from taskinit import *
import math as m

#from math import *
#from alma.asdm.asdm import ASDM
from asdm import *
from optparse import OptionParser
#import IPython
from pylab import *
import scipy.optimize


def files_callback(option, opt_str, value, parser):
    """ callback for a  variable number of arguments for option --files
    """
    assert value is None
    value = []
    def floatable(str):
        try:
            float(str)
            return True
        except ValueError:
            return False

    for arg in parser.rargs:
        # stop on --foo like options
        if arg[:2] == "--" and len(arg) > 2:
            break
        # stop on -a, but not on -3 or -3.0
        if arg[:1] == "-" and len(arg) > 1 and not floatable(arg):
            break
        value.append(arg)

    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

def getBand(fr):
    if fr < 115e9:
        band='ALMA_RB_03'
    elif fr < 270e9:
        band='ALMA_RB_06'
    elif fr < 360e9:
        band='ALMA_RB_07'
    else:
        band='ALMA_RB_09'
    return band
        

def readFocusParam(ant, date):
    """
    Retrieve the focus parameters for antenna ant for date  2009-04-23
    """
    file='/groups/science/FocusCurves/%s_focusCurves.txt' %ant
    b=open(file)
    obs=b.readlines()
    t0=qa.convert(date,'d')['value']

    fparam = {'X0' : {3 : 0, 6 : 0, 7 : 0, 9:0},
              'Y0' : {3 : 0, 6 : 0, 7 : 0, 9:0},
              'Z0' : {3 : 0, 6 : 0, 7 : 0, 9:0},
              'Z_sin' : 0, 'Y_cos' : 0, 'Z_temp' : 0.0}
    i=0
    while i  < size(obs):
        line=obs[i]
        #print i, line
        if line[0] != '\n' and line[0] != '#':
            start=line.split(',')[0].lstrip()
            stop= line.split(',')[1].lstrip()
            if t0-qa.convert(start,'d')['value'] >= 0 and t0-qa.convert(stop,'d')['value'] <= 0:
                break
            else:
                i=i+1
        else:
            i=i+1

    if i == size(obs):
        print "No focus parameters found within specified date. Go check %s" %file
        return

    fparam['X0'][3]=float(line.split(',')[2].lstrip())
    fparam['Y0'][3]=float(line.split(',')[3].lstrip())
    fparam['Z0'][3]=float(line.split(',')[4].lstrip())
    fparam['X0'][6]=float(line.split(',')[5].lstrip())
    fparam['Y0'][6]=float(line.split(',')[6].lstrip())
    fparam['Z0'][6]=float(line.split(',')[7].lstrip())
    fparam['X0'][7]=float(line.split(',')[8].lstrip())
    fparam['Y0'][7]=float(line.split(',')[9].lstrip())
    fparam['Z0'][7]=float(line.split(',')[10].lstrip())
    fparam['X0'][9]=float(line.split(',')[8].lstrip())
    fparam['Y0'][9]=float(line.split(',')[9].lstrip())
    fparam['Z0'][9]=float(line.split(',')[10].lstrip())
    fparam['Z_sin']=float(line.split(',')[11].lstrip())
    fparam['Y_cos']=float(line.split(',')[12].lstrip())
    fparam['Z_temp']=float(line.split(',')[13].lstrip())

    return fparam

def getFocusVal(antenna,band,el,temperature,date) :
    focusParameters = readFocusParam(antenna,date)
    x_focus = focusParameters['X0'][int(band)]
    y_focus = focusParameters['Y0'][int(band)]+focusParameters['Y_cos']*m.cos(el)
    z_focus = focusParameters['Z0'][int(band)]+focusParameters['Z_sin']*m.sin(el)+focusParameters['Z_temp']*(temperature-283.15)

    return [x_focus,y_focus,z_focus]

def getFocusVal50(antenna,band,el,temperature,date) :
    if band == 3:
        out=[0.134E-3,0.041E-3,0.092E-3]
    elif band == 6:
        out=[-0.113E-3,+0.076E-3,-0.131E-3]
    elif band == 7:
        out=[0.,0.,0.]
    else:
        out=[0.,0.,0.]
    band=7
    focusParameters = readFocusParam(antenna,date)
    x_focus = out[0]+focusParameters['X0'][int(band)]
    y_focus = out[1]+focusParameters['Y0'][int(band)]+focusParameters['Y_cos']*(m.cos(el)-m.cos(m.radians(50.)))
    z_focus = out[2]+focusParameters['Z0'][int(band)]+focusParameters['Z_sin']*(m.sin(el)-m.sin(m.radians(50.)))+focusParameters['Z_temp']*(temperature-273.15)

    return [x_focus,y_focus,z_focus]

class ExtractFocus:

    def __init__(self):
        self.temperature={}
        self.elevation={}
        self.azimuth={}
        self.focus={}
        self.focus1={}
        self.error={}
        self.antenna=['PM01','PM02','PM03','PM04','DV01','DV02','DV03','DV04','DV05','DV06','DV07','DV08','DV09','DV10','DV11','DV12']
        self.axes=['X','Y','Z']
        self.bands=['ALMA_RB_03', 'ALMA_RB_06' , 'ALMA_RB_07', 'ALMA_RB_09']
        self.lines = []
        self.startTime = ''
        self.endTime = ''
        self.files = []
        for axis in self.axes:
            for band in self.bands:
                for antenna in self.antenna:
                    self.temperature[(axis,band,antenna)] = []
                    self.elevation[(axis,band,antenna)] = []
                    self.azimuth[(axis,band,antenna)] = []
                    self.focus[(axis,band,antenna)] = []
                    self.focus1[(axis,band,antenna)] = []
                    self.error[(axis,band,antenna)] = []
        self.A={}
        self.B={}
                
    def readFile(self,uid):
        """
        read one data set and extract focus data calibration
        """
        self.files.append(uid)
        a = ASDM()
        a.setFromFile(uid)
        calFocus = a.calFocusTable().get()
        focus1=zeros((3,66))
        mm=1e3
        antnum={'PM01':1,'PM02':2,'PM03':3,'PM04':4,'DV01':5,'DV02':6,'DV03':7,'DV04':8,'DV05':9,'DV06':10,'DV07':11,'DV08':12,'DV09':13,'DV10':14,'DV11':15,'DV12':16}
        bandnum={'ALMA_RB_03':3, 'ALMA_RB_06':6, 'ALMA_RB_07':7, 'ALMA_RB_09':9}
        axisnum={'X':1,'Y':2,'Z':3}
        self.lines.append(' ')
        self.lines.append('#Date/time                T   Antenna    Axis     Focus offset (mm)      Error(mm)     Az        El    absolute Focus (mm)')
        for row in calFocus:
            fr = row.frequencyRange()[0].get()
            antenna = row.antennaName()
            temperature = row.ambientTemperature()
            band = getBand(fr)
            #print fr, getBand(fr)
            #band = str(row.receiverBand())
            for i in range(3):
                if not row.wereFixed()[i]:
                    axis = self.axes[i] 
                    focus = row.offset()[0][i].get()*mm
                    error = row.offsetError()[0][i].get()*mm
            print antenna, antnum[antenna]
            focus1[axisnum[axis]-1][antnum[antenna]] += focus
            az, el = row.pointingDirection()
            azd = degrees(az.get())
            eld = degrees(el.get())
            kind = (axis, band, antenna)
            self.elevation[kind].append(degrees(el.get()))  
            self.azimuth[kind].append(degrees(az.get()))
            self.temperature[kind].append(temperature)
            self.focus[kind].append(focus)
            self.focus1[kind].append(focus1[axisnum[axis]-1][antnum[antenna]])
            self.error[kind].append(error)
            if self.startTime=='':
                self.startTime=row.startValidTime().toFITS()
            self.endTime = row.startValidTime().toFITS()
            focusvals=getFocusVal50(antenna,bandnum[band],m.radians(eld),temperature.get(),row.startValidTime().toFITS())
            #print focusvals
            self.lines.append('%s %s %s %s %s  %8.4f %8.4f %8.2f %8.2f %8.4f' % \
                              (row.startValidTime().toFITS(), temperature, antenna, axis, band, focus,
                               error, azd, eld, (focus1[axisnum[axis]-1][antnum[antenna]]+focusvals[axisnum[axis]-1])))
        return

    def writeData(self,outfile):
        out_file = open(outfile,'w')
        for line in self.lines:
            out_file.write(line+'\n')
        out_file.close()
                    
    def plot(self):

        self.bandsym={'ALMA_RB_03':'D','ALMA_RB_06':'x','ALMA_RB_07':'s','ALMA_RB_09':'^'}
        self.axiscol={'X':'r','Y':'g','Z':'b'}

        for antenna in self.antenna:
            for axis in self.axes:
                for band in self.bands:
                    code = self.axiscol[axis] + self.bandsym[band]
                    plot(self.elevation[(axis,band,antenna)],self.focus[(axis,band,antenna)],code)

            #title(antenna+' focus')
            figtext(0.02, 0.98, self.files.__str__(), fontsize=12, ha='left',va='top')
            figtext(0.02, 0.94, self.startTime+' - '+self.endTime, fontsize=12,
                    ha='left',va='top')
            xlabel('elevation[degrees]')
            ylabel('focus[mm]')
            subplots_adjust(top=0.75)

    def fit(self, axis, band=None):
        ytext={'X': 0.90, 'Y': 0.86, 'Z': 0.82} 
        ydep={'X': cos, 'Y': cos, 'Z': sin} 
        col={'X': 'red', 'Y': 'green', 'Z': 'blue'} 
        xx = []
        yy = []
        for b in self.bands:
            if (band == None) or (b == band):
                xx.extend(self.elevation[(axis,b)])
                yy.extend(self.focus[(axis,b)])
        ax = array(xx)*pi/180.
        ay = array(yy)
        fitfunc= lambda p, x: p[0] + p[1]*ydep[axis](x)
        errfunc = lambda p, x, y: fitfunc(p, x)-y
        p0=[0.,1.]
        p1, success = scipy.optimize.leastsq(errfunc, copy.copy(p0), args=(ax, ay))
        self.B[axis] = p1[1]
        self.A[axis] = p1[0]
        el = arange(0.,90.,1)
        fit = self.A[axis] + self.B[axis]*ydep[axis](el*pi/180.)
        #plot(ax, ay, 'r.')
        plot(el,fit,self.axiscol[axis]+'-')
        ytext={'X': 0.90, 'Y': 0.86, 'Z': 0.82} 
        figtext(0.02,
                ytext[axis],
                '%s in red:   %6.3f + %6.3f %s(el)' % (axis, self.A[axis], self.B[axis],
                                                       str(ydep[axis])), 
                fontsize=12, ha='left',va='top')
        

# Execution starts here
if __name__ == '__main__':

    # Initialize parser
    parser = OptionParser()

    parser.add_option("-f", "--files", dest="files",
                  help="Input files to be analysed (separated by white space)",
                  action="callback", callback=files_callback)
    parser.add_option("-o", "--outfile", dest="outfile",
                      help="OUtput file name",
                      action='store')

    (options, args) = parser.parse_args()

    files = options.files
    outfile = options.outfile

    clf()
    # uidList = ['uid___X54_X35fa_X1',
    #            'uid___X54_X3a30_X1',
    #            'uid___X54_X34ba_X1']

    ef = ExtractFocus()

    focus1 = zeros(3)
    for uid in files:
        ef.readFile(uid)

    ef.writeData(outfile)

    #ef.plot()

    #ef.fit('X')
    #ef.fit('Y')
    #ef.fit('Z')

    show()

    


