#!/usr/bin/env python
#ExtractPointing.py
#written by RKneissl (2009-05-03) based on ExtractFocusRun.py by RLucas 
#
#will extract the data from  the ADSM file (need to change the file name)
#and dump it to a file so we can save  it in a pointing_table.
#pointing file name format is  pointing_20090417.txt
#Example:
# ExtractPointing -f uid___X54_X35fa_X1 uid___X54_X3a30_X1 [...] -o 
# pointing_20090417.txt
#
#$Id: ExtractPointing_6.1.py,v 1.1 2010/03/22 13:54:44 dbarkats Exp $

from math import *
#from alma.asdm.asdm import ASDM
from asdm import * 
from optparse import OptionParser
#import IPython
from pylab import *
import scipy.optimize
#from scipy import optimize

def files_callback(option, opt_str, value, parser):
    """ callback for a  variable number of arguments for option --files
    """
    assert value is None
    value = []
    def floatable(str):
        try:
            float(str)
            return True
        except ValueError:
            return False

    for arg in parser.rargs:
        # stop on --foo like options
        if arg[:2] == "--" and len(arg) > 2:
            break
        # stop on -a, but not on -3 or -3.0
        if arg[:1] == "-" and len(arg) > 1 and not floatable(arg):
            break
        value.append(arg)

    del parser.rargs[:len(value)]
    setattr(parser.values, option.dest, value)

def getBand(fr):
    if fr < 115e9:
        band='ALMA_RB_03'
    elif fr < 270e9:
        band='ALMA_RB_06'
    elif fr < 350e9:
        band='ALMA_RB_07'
    else:
        band='ALMA_RB_09'
    return band
        

class ExtractPointing:

    def __init__(self):
        self.elevation={}
        self.azimuth={}
        self.pointing={}
        self.error={}
        self.antenna=['PM01','PM02','PM03','DV01','DV02','DV03','DV04','DV05']
        self.axes=['Az','El']
        self.bands=['ALMA_RB_03', 'ALMA_RB_06' , 'ALMA_RB_07', 'ALMA_RB_09']
        self.lines = []
        self.startTime = ''
        self.endTime = ''
        self.files = []
        for axis in self.axes:
            for band in self.bands:
                for antenna in self.antenna:
                    self.elevation[(axis,band,antenna)] = []
                    self.azimuth[(axis,band,antenna)] = []
                    self.pointing[(axis,band,antenna)] = []
                    self.error[(axis,band,antenna)] = []
        self.A={}
        self.B={}
                
        
    def readFile(self,uid):
        """
        read one data set and extract focus data calibration
        """
        self.files.append(uid)
        a = ASDM()
        a.setFromFile(uid)
        calPointing = a.calPointingTable().get()
        arcsec=206264.8062470
        self.lines.append(' ')
        self.lines.append('Date/time             Antenna     Axis   Band      Az/El as    error as     Az d        El d')
        for row in calPointing:
            fr = row.frequencyRange()[0].get()
            antenna = row.antennaName()            
            band = getBand(fr)
            #print fr, getBand(fr)
            #band = str(row.receiverBand())
            for i in range(2):
                    axis = self.axes[i] 
                    pointing = row.collOffsetRelative()[0][i].get()*arcsec
                    error = row.collError()[0][i].get()*arcsec
                    az, el = row.direction()
                    azd = degrees(az.get())
                    eld = degrees(el.get())
                    kind = (axis, band,antenna)
                    self.elevation[kind].append(degrees(el.get()))  
                    self.azimuth[kind].append(degrees(az.get()))
                    self.pointing[kind].append(pointing)
                    self.error[kind].append(error)
                    if self.startTime=='':
                        self.startTime=row.startValidTime().toFITS()
                    self.endTime = row.startValidTime().toFITS()
                    self.lines.append('%s %s %s %s %8.3f    %8.3f    %8.2f    %8.2f ' % \
                                      (row.startValidTime().toFITS(), antenna, axis,
                                       band, pointing,error, azd, eld))
        return

    def writeData(self,outfile):
        out_file = open(outfile,'w')
        for line in self.lines:
            out_file.write(line+'\n')
        out_file.close()
                    
# Execution starts here
if __name__ == '__main__':

    # Initialize parser
    parser = OptionParser()

    parser.add_option("-f", "--files", dest="files",
                  help="Input files to be analysed (separated by white space)",
                  action="callback", callback=files_callback)
    parser.add_option("-o", "--outfile", dest="outfile",
                      help="OUtput file name",
                      action='store')

    (options, args) = parser.parse_args()

    files = options.files
    outfile = options.outfile

    # uidList = ['uid___X54_X35fa_X1',
    #            'uid___X54_X3a30_X1',
    #            'uid___X54_X34ba_X1']

    ef = ExtractPointing()

    for uid in files:
        ef.readFile(uid)

    ef.writeData(outfile)



