#!/usr/bin/env python
# GetBaselineData.py
#written by dbarkats

# $Id: GetBaselineData.py,v 1.10 2015/01/22 02:41:51 dbarkats Exp $

from optparse import OptionParser
import analysisUtils as au
import os,sys
from manualAsdmExport import manualAsdmExport
from pylab import *
import time
from dateutil import rrule
from datetime import datetime,timedelta
from calendar import monthrange
import pickle
import glob
from asdm import ASDM


def system_except(cmd):
    ret = os.system(cmd)
    if ret != 0:
        raise Exception("Command '%s' returned non-zero exit status %d" %(cmd, ret))

def shiftlogQueryForBaseline(d1,d2):
    """
    function to send queries to SLT to retrieve ASDM and additional info about all sky baseline runs.
    For now, shiftlog-query only runs on aos-gns, so I have to ssh those commands in.
    Takes 2 dates in ISO format as input.
    """
        
    print "### Querying SLT for asdm list ..."
    print "Query 1",
    tmpfile1 = 'tmp_query1.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND -f " > %s'%(d1,d2,tmpfile1)
    os.system(cmd)
    
    print "Query 2",
    tmpfile2 = 'tmp_query2.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND 1924-292,1751+096" > %s'%(d1,d2,tmpfile2)
    os.system(cmd)
    
    print "Query 3",
    tmpfile3 = 'tmp_query3.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND 3c279,1146+399" > %s'%(d1,d2,tmpfile3)
    # print cmd
    os.system(cmd)

    print "Query 4",
    tmpfile4 = 'tmp_query4.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -t SBEX -s "DelayCal.py AND 1625-254,1316-336" > %s'%(d1,d2,tmpfile4)
    # print cmd
    os.system(cmd)

    print "Query 5",
    tmpfile5 = 'tmp_query5.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND 3c454.3,2236+284" > %s'%(d1,d2,tmpfile5)
    # print cmd
    os.system(cmd)

    print "Query 6",
    tmpfile6 = 'tmp_query6.txt'
    cmd = ' shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND 0006-063,0047-252" > %s'%(d1,d2,tmpfile6)
    # print cmd
    os.system(cmd)

    print "Query 7",
    tmpfile7 = 'tmp_query7.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "DelayCal.py AND 0334-401,3c84" > %s'%(d1,d2,tmpfile7)
    # print cmd
    os.system(cmd)

    print "Query 8",
    tmpfile8 = 'tmp_query8.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t MMEX -s "ObsCalBaselines.py" > %s'%(d1,d2,tmpfile8)
    # print cmd
    os.system(cmd)

    print "Query 9"
    tmpfile9 = 'tmp_query9.txt'
    cmd = 'shiftlog-query -it OTHER -ib "%s" -ie "%s" -l OSF-AOS -t SBEX -pc 0000.0.00222.CSV > %s'%(d1,d2,tmpfile9)
    #print cmd
    #raw_input()
    os.system(cmd)

    # read the asdms from the downloaded web forms
    tmpfiles = glob.glob('tmp_*.txt')
    tmpfiles = sort(tmpfiles)
    asdmList = []
    start = []
    end = []
    status = []
    command = []
    for tmpfile in tmpfiles:
        f = open(tmpfile,'r')
        lines = f.readlines()
        f.close()
        found = 0
        asdmTmp = []
        for line in lines:
            if 'uid://' in line:
                if '[antennas:' in line: # skip AIV+ entries
                    continue
                found += 1
                sline = line.split('"')
                mode = sline[1]
                start.append(sline[3].split(' - ')[0])
                end.append(sline[3].split(' - ')[1])
                status.append(sline[13])
                asdmList.append(sline[11].strip('\"'))
                asdmTmp.append(sline[11].strip('\"'))
                if mode == 'MMEX':
                    command.append(sline[17])
                elif mode == 'SBEX':
                    tmp = 'project:%s SB:%s '%(sline[7],sline[9])
                    command.append(tmp)
                #print start[-1], end[-1], asdmList[-1],status[-1],command[-1]
        print 'Found %d ASDMs in %s: %s'%(found, tmpfile, ','.join(map(str, asdmTmp)))
    
    os.system('rm -Rf tmp_query*.txt')
      
    return start,end,asdmList,status,command


# Execution starts here
if __name__ == '__main__':
    usage = '''

    Main function to retrieve ASMD uid\'s of all sky baseline runs
    and  update cvs controlled files AIV/science/PadData/baselineData_sofar.txt
    If no inputs are provided, will retrieve the data from the last day
    It will automatically  update the file, cvs commit it and rsync updated file to
    www.alma.cl/~dbarkats/trend_analysis/baselineData_sofar.txt

    '''
    # Initialize parser
    parser = OptionParser()

    parser.add_option("-f", "--from", dest="fr",
                      help="date you want to start your check. format YYYY-MM-DDTHH:MM:SS ")

    parser.add_option("-t", "--to", dest="to",
                      help="date you want to stop your check on.format YYYY-MM-DDTHH:MM:SS ")

    (options, args) = parser.parse_args()

    now = datetime.now()
    today = ('%s'%now)[0:10]
    yesterday = ('%s'%(now - timedelta(days=1)))[0:10]
    start='%sT09:00:00'%yesterday
    end='%sT09:00:00'%today
    
    if options.fr is None:
        day1 = start
    else:
        day1 = options.fr 
    if options.to is None:
        day2 = end
    else:
        day2 = options.to
    print '### Getting baseline data from: %s to: %s'%(day1,day2)
    
    [start_n,end_n,asdmList_n,status_n,command_n] = shiftlogQueryForBaseline(day1,day2)
    nAsdm = size(asdmList_n)
    print "### Found %d ASDMs in this time range ...### \n"%nAsdm

    if nAsdm == 0:
        print "### Stopping here: nothing to update."
        sys.exit()
        
    # re-order by start time
    sorted_start = sorted(start_n, key = lambda  x: datetime.strptime(x,'%Y-%m-%dT%H:%M:%S'))
    ind = sorted(range(len(start_n)), key=lambda k: start_n[k]) 
    
    asdmList = [asdmList_n[i] for i in  ind]
    start = [start_n[i] for i in  ind]
    end = [end_n[i] for i in  ind]
    status = [status_n[i] for i in  ind]
    command = [command_n[i] for i in  ind]

    # write local version of file in homedir
    g = open('baselineData_sofar_local.txt','w')
    g.write("### Local version..\n")
    g.write("### Date:%s \n"%now)
    for i in range(size(asdmList)):
        g.write('%s, %s, %s, %s, %s \n \n'%(start[i], end[i], asdmList[i],status[i],command[i]))
    g.close()

    if options.fr is not None or options.to is not None:
        print "GetBaselineData.py called with non default date ranges. Stopping here"
        print "Only localfile: baselineData_sofar_local.txt has been updated"
        print "Nothing has been changed on  global file in ~/AIV/science/PadData/baselineData_sofar.txt"
        exit()
    
    print '#### CVS updating baselineData_sofar.txt... #### '
    # cvs up
    os.chdir('PadData/')
    up_cmd = 'cvs up baselineData_sofar.txt'
    system_except(up_cmd)
    
    print('### Editing ...###')
    # make a copy
    system_except('cp baselineData_sofar.txt baselineData_sofar.txt.backup')
    
    # Edit the file
    with open('baselineData_sofar.txt','a+') as f:
        for i in range(size(asdmList)):          
            f .write('%s, %s, %s, %s, %s \n \n'%(start[i], end[i], asdmList[i],status[i],command[i]))
    f.close()
    
    # cvs ci
    print '### CVS committing baselineData_sofar.txt...###'
    ci_cmd = 'cvs ci -m \"Daily Update for baselineData_sofar.txt: %s \" baselineData_sofar.txt'\
             %now
    a = os.popen(ci_cmd).read()
    print a
    
    # rsync to website
    print '### Rsyncing baselineData_sofar.txt to http://www.alma.cl/~dbarkats ...###'
    cmd = 'rsync -auv --progress /users/dbarkats/PadData/baselineData_sofar.txt dbarkats@ssh.alma.cl:~/public_html/trend_analysis/'
    system_except(cmd)
    
