#!/usr/bin/env python
#HPSwitchingAnalysis.py
#
#Script for reduction of data taken using HPSwitching.py (just for three switches)
#Author DGunawan (this is from V6-3)
#
#
#
#Runs in casapy: 
#execfile('HPSwitchingAnalysis.py')
#then
#SwitchingAnalysis(uid='uid___A002_X1898c6_X122', pscfile='20101229dv09b3pos-1.dat', movedAnt='DV09', refAnt='DV03', band='3')
#movedAnd in capital letters
#


sys.path.append("~/AIV/science/analysis_scripts/")
import os
import sys
import analysisUtils as aU
import math
import time
import numpy as numpy
from pylab import *
import scipy.io.array_import
from array import array


####################

def LinFit(x, f):
  return (f[1]+f[0]*x)

####################

def SwitchingAnalysis(uid='', pscfile='', movedAnt='', refAnt='', band=''): 

  msfile = uid+'.ms'
  if 'PM' in movedAnt:
   jj = 1.8
   kk = 2.3
  else:
   jj = 1.5
   kk = 2.0
  
  ##Info
  tb.open(msfile+'/SOURCE')
  sourceName = (tb.query('SPECTRAL_WINDOW_ID == 2')).getcol('NAME')
  tb.close()
  tb.open(msfile+'/POINTING')
  aa = tb.getcol('TARGET')
  tb.close()
  Elevation = numpy.degrees(aa[1,0,0])
  weather = aU.Weather(msfile)
  wind = weather.data['WIND_SPEED'].mean()
  temp = weather.data['TEMPERATURE'].mean()-273
  
  ##Do gaincal and get antenna gain
  caltab = uid + '.gcal'
  os.system('rm -rf '+caltab)
  gaincal(vis=msfile, caltable=caltab, spw='2', minsnr=-1, solint=0.096,refant=refAnt)
  antNames = aU.getAntennaNames(msfile)
  aM = where(antNames == movedAnt)[0][0]
  tb.close()
  tb.open(caltab)
  antQuery = tb.query('ANTENNA1 == '+str(aM))
  calTime = antQuery.getcol('TIME')
  calGain = antQuery.getcol('GAIN')
  tb.close()
  calAmp = numpy.sqrt(calGain.real**2.0+calGain.imag**2.0)
  calAmp = calAmp[0,0,:]
  obsDate = qa.time((qa.quantity(calTime[0],'s')),form=["ymd","local"])
   
  
  ##Read PSC file
  #Create a binary casa table from the PSC file.
  parsefile='tempFile'
  os.system('rm -rf '+parsefile)
  F1 = open(parsefile,'w')
  for line in open(pscfile):
      if 'source' in line:
          if not 'invalid' in line:
              F1.write(line)
  
  F1.close()
    
  outfile = pscfile + '.table'
  os.system('rm -rf '+outfile)
  tb.fromascii(outfile,parsefile,autoheader=true,sep=' ')
  tb.close()
  tb.open(outfile,nomodify=false)
  tb.renamecol('Column1',  'Time')
  tb.renamecol('Column2',  'Comm Az')
  tb.renamecol('Column3',  'Comm El')     
  tb.renamecol('Column6',  'on-TE Az')
  tb.renamecol('Column7',  'on-TE El')
  tb.renamecol('Column10', 'Pos Flag')
  tb.close()
  os.system('rm -rf '+parsefile)
  
  #Open the new table and get the time, commanded position, and the on position
  tb.open(outfile,nomodify=false)
  timedat= tb.getcol('Time')
  commAz = tb.getcol('Comm Az')
  commEl = tb.getcol('Comm El')
  onAz   = tb.getcol('on-TE Az')
  onEl   = tb.getcol('on-TE El')
  posflag= tb.getcol('Pos Flag')   
  nline  = tb.nrows()
  tb.close()
  pscTimeAll=[]
  deltaAz   =[]
  deltaEl   =[]
  for i in range (0,nline):
    timex=me.epoch('utc',timedat[i])
    timey=(timex['m0']['value'])*(3600*24)
    pscTimeAll.append(timey)
    # corect for cos(eL)
    deltaAzx=3600*((onAz[i]-commAz[i])*cos(numpy.radians(onEl[i])))
    deltaAz.append(deltaAzx)
    deltaElx=3600*(onEl[i]-commEl[i])
    deltaEl.append(deltaElx)
  
  
  ##Beam size
  clight = 2.99792458e11 #mm/s
  if band == '3':
    trBeam = 61.0
  elif band == '6':
    trBeam = 23.0
  elif band == '7':
    trBeam = 16.0
  elif band == '9':
    trBeam = 8.0
  else:
    obsFreq =  float(band) * 1e9
    beamsize = (251*clight)/(obsFreq*12.)
    trBeam = round(sqrt(-2*log(0.25)) * (beamsize/2))
  
   
  ##Gain Statistics
  #Separating the 'scans'
  scanJumps = []
  for i in range(0,(len(calTime) - 1)):
    scanJumps.append(calTime[i+1]-calTime[i])
  
  Jumps = where(numpy.array(scanJumps) > 10)[0]
  
  #Fit On-Source drift
  gAmpOn   = numpy.array(list(calAmp[0:Jumps[0]])+list(calAmp[Jumps[6]+1:Jumps[7]])+list(calAmp[Jumps[8]+1:Jumps[9]])+list(calAmp[Jumps[10]+1:]))
  gTimOn   = numpy.array(list(calTime[0:Jumps[0]])+list(calTime[Jumps[6]+1:Jumps[7]])+list(calTime[Jumps[8]+1:Jumps[9]])+list(calTime[Jumps[10]+1:]))
  params   = numpy.polyfit(gTimOn,gAmpOn,1)
  gFitAll  = LinFit(calTime,params)
  calAmpCor= calAmp-(gFitAll-average(gAmpOn))  
  #Average gain on-source, after correction for drift 
  avgOn1   = average(numpy.array(list(calAmpCor[0:Jumps[0]])))
  stdOn1   = std(numpy.array(list(calAmpCor[0:Jumps[0]])))
  avgOn2   = average(numpy.array(list(calAmpCor[Jumps[6]+1:Jumps[7]]))) 
  stdOn2   = std(numpy.array(list(calAmpCor[Jumps[6]+1:Jumps[7]])))
  avgOn3   = average(numpy.array(list(calAmpCor[Jumps[8]+1:Jumps[9]])))
  stdOn3   = std(numpy.array(list(calAmpCor[Jumps[8]+1:Jumps[9]])))
  avgOn4   = average(numpy.array(list(calAmpCor[Jumps[10]+1:])))
  stdOn4   = std(numpy.array(list(calAmpCor[Jumps[10]+1:])))
  avgOn    = average(numpy.array(list(calAmpCor[0:Jumps[0]])+list(calAmpCor[Jumps[6]+1:Jumps[7]])+list(calAmpCor[Jumps[8]+1:Jumps[9]])+list(calAmpCor[Jumps[10]+1:])))
  stdOn    = std(numpy.array(list(calAmpCor[0:Jumps[0]])+list(calAmpCor[Jumps[6]+1:Jumps[7]])+list(calAmpCor[Jumps[8]+1:Jumps[9]])+list(calAmpCor[Jumps[10]+1:])))
  #Average gain at half-power
  avgHP1   = average(numpy.array(list(calAmpCor[Jumps[1]+1:Jumps[2]])))
  stdHP1   = std(numpy.array(list(calAmpCor[Jumps[1]+1:Jumps[2]])))
  avgHP2   = average(numpy.array(list(calAmpCor[Jumps[4]+1:Jumps[5]])))
  stdHP2   = std(numpy.array(list(calAmpCor[Jumps[4]+1:Jumps[5]])))
  avgHP    = average(numpy.array(list(calAmpCor[Jumps[1]+1:Jumps[2]]) + list(calAmpCor[Jumps[4]+1:Jumps[5]])))
  stdHP    = std(numpy.array(list(calAmpCor[Jumps[1]+1:Jumps[2]]) + list(calAmpCor[Jumps[4]+1:Jumps[5]])))
  #Convert Gain to Offset. Offset value only valid for the halfpower amplitude. 
  gainOff=[]
  for i in range(len(calAmpCor)):
    if calAmpCor[i] < avgOn: 
       ampOff=trBeam*sqrt(log(avgOn/calAmpCor[i]));
    else:
       ampOff=trBeam*sqrt(log(calAmpCor[i]/avgOn));
    gainOff.append(ampOff)
  
  #Average Offset on-source. Does not actually have any meaning
  avgOnOff1 = average(numpy.array(list(gainOff[0:Jumps[0]])))
  stdOnOff1 = std(numpy.array(list(gainOff[0:Jumps[0]])))
  avgOnOff2 = average(numpy.array(list(gainOff[Jumps[6]+1:Jumps[7]]))) 
  stdOnOff2 = std(numpy.array(list(gainOff[Jumps[6]+1:Jumps[7]])))
  avgOnOff3 = average(numpy.array(list(gainOff[Jumps[8]+1:Jumps[9]])))
  stdOnOff3 = std(numpy.array(list(gainOff[Jumps[8]+1:Jumps[9]])))
  avgOnOff4 = average(numpy.array(list(gainOff[Jumps[10]+1:])))
  stdOnOff4 = std(numpy.array(list(gainOff[Jumps[10]+1:])))
  avgOnOff  = average(numpy.array(list(gainOff[0:Jumps[0]])+list(gainOff[Jumps[6]+1:Jumps[7]])+list(gainOff[Jumps[8]+1:Jumps[9]])+list(gainOff[Jumps[10]+1:])))
  stdOnOff  = std(numpy.array(list(gainOff[0:Jumps[0]])+list(gainOff[Jumps[6]+1:Jumps[7]])+list(gainOff[Jumps[8]+1:Jumps[9]])+list(gainOff[Jumps[10]+1:])))
  #Average Offset at half-power
  avgHPOff1 = average(numpy.array(list(gainOff[Jumps[1]+1:Jumps[2]])))
  stdHPOff1 = std(numpy.array(list(gainOff[Jumps[1]+1:Jumps[2]])))
  avgHPOff2 = average(numpy.array(list(gainOff[Jumps[4]+1:Jumps[5]])))
  stdHPOff2 = std(numpy.array(list(gainOff[Jumps[4]+1:Jumps[5]])))
  avgHPOff  = average(numpy.array(list(gainOff[Jumps[1]+1:Jumps[2]]) + list(gainOff[Jumps[4]+1:Jumps[5]])))
  stdHPOff  = std(numpy.array(list(gainOff[Jumps[1]+1:Jumps[2]]) + list(gainOff[Jumps[4]+1:Jumps[5]])))
  
  
  ##Switching Statistics
  
  #Change the PSC time to the frame of the data time (data is taken after the PSC script was run)
  calTimeZero = calTime - calTime[0]
  pscTimeZero = pscTimeAll - calTime[0]
  
  #Find where the switches are in the psc data. Only for 1.5 degree switch
  #In the PSC data, find jumps
  AzJumps = []
  ElJumps = []
  for i in range(0,nline-1):
    AzJumps.append(absolute(deltaAz[i+1] - deltaAz[i]))
    ElJumps.append(absolute(deltaEl[i+1] - deltaEl[i]))
  
  pscAzJumps = where(numpy.array(AzJumps) >= 3000)[0]
  pscElJumps = where(numpy.array(ElJumps) >= 3000)[0]
  
  #Find in the gainOffset data where the psc showed jumps
  gSwitches = []
  for i in range(0,len(pscAzJumps)):
    gSwitches.append(where(calTimeZero >= pscTimeZero[pscAzJumps[i]])[0][0])
  
  for i in range(0,len(pscElJumps)):
    gSwitches.append(where(calTimeZero >= pscTimeZero[pscElJumps[i]])[0][0])
    
  #Statistics
  gainHPOff = gainOff - avgHPOff
  avgO1p5s  = [] 
  stdO1p5s  = []
  avgO2s    = []
  stdO2s    = []
  maxO1p5s  = []
  tmaxO1p5s = []
  for i in range(0,(len(gSwitches))/2):
    a = gSwitches[(2*i)+1]
    b = where(calTimeZero >= calTimeZero[a]+jj)[0][0]
    c = where(calTimeZero >= calTimeZero[a]+kk)[0][0]
    d = where(calTimeZero >= calTimeZero[a]+8.)[0][0]
    max1p5s     = max(abs(gainHPOff[b:d]))
    tm1p5       = where(abs(gainHPOff) == max(abs(gainHPOff[b:d])))
    tmax1p5s = calTimeZero[tm1p5[0][0]]-calTimeZero[a]
    avg1p5s = average(abs(gainHPOff[b:d]))
    std1p5s = std(abs(gainHPOff[b:d]))
    avg2s   = average(abs(gainHPOff[c:d]))
    std2s   = std(abs(gainHPOff[c:d]))
    avgO1p5s.append(avg1p5s)
    stdO1p5s.append(std1p5s)
    avgO2s.append(avg2s)
    stdO2s.append(std2s)
    maxO1p5s.append(max1p5s)
    tmaxO1p5s.append(tmax1p5s)
                             
  
  ##Printing out results
  #Printing out the file names, weather, source
  print '--------------------------------------------------------------------------------------'
  print 'uid:        %25s                 Source:        %14s'          %(uid, sourceName[0])
  print 'PSC data:   %25s                 Elevation:     %14.1f'        %(pscfile, Elevation)
  print 'Date:       %25s                 Band:          %14s'          %(obsDate, band)
  print 'Mean Temperature [C] = %14.2f                 Moved Antenna: %14s'          %(temp,movedAnt)
  print 'Mean Wind Speed  [m/s] = %12.2f' %(wind)
  print '--------------------------------------------------------------------------------------'
  #Printing out the statistics
  print 'S/N On-Source:                  %10.5f'%(avgOn/stdOn)
  print 'S/N at HP:                      %10.5f'%(avgHP/stdHP)
  print 'AmpHP/AmpOnSource:              %10.5f'%(avgHP/avgOn)
  #print 'rms offset On-Source  (arcsec):    %10.5f'%(stdOnOff)
  print 'rms offset at HP (arcsec):      %10.5f'%(stdHPOff)
  print'--------------------------------------------------------------------------------------'
  for l in [0,5,10]:
    print 'Switch: ',(l/5)+1
    print 'Maximum pointing error after ',jj,'s    (arcsec):'+5*'%7.2f'%(maxO1p5s[0+l],maxO1p5s[1+l],maxO1p5s[2+l],maxO1p5s[3+l],maxO1p5s[4+l])
    print 'Time of the Max after the switch            (s):'+5*'%7.2f'%(tmaxO1p5s[0+l],tmaxO1p5s[1+l],tmaxO1p5s[2+l],tmaxO1p5s[3+l],tmaxO1p5s[4+l])
    print 'Average pointing error after ',jj,'s    (arcsec):'+5*'%7.2f'%(avgO1p5s[0+l],avgO1p5s[1+l],avgO1p5s[2+l],avgO1p5s[3+l],avgO1p5s[4+l]) 
    print '                  Standard deviation   (arcsec):'+5*'%7.2f'%(stdO1p5s[0+l],stdO1p5s[1+l],stdO1p5s[2+l],stdO1p5s[3+l],stdO1p5s[4+l])
    print 'Average pointing error after ',kk,'s    (arcsec):'+5*'%7.2f'%(avgO2s[0+l],avgO2s[1+l],avgO2s[2+l],avgO2s[3+l],avgO2s[4+l])
    print '                  Standard deviation   (arcsec):'+5*'%7.2f'%(stdO2s[0+l],stdO2s[1+l],stdO2s[2+l],stdO2s[3+l],stdO2s[4+l])
    print
  
  print '--------------------------------------------------------------------------------------'
  
  ##Printing out to file
  statFileOut = uid+'Stat.txt'
  F = open(statFileOut,'w')
  print >> F, '--------------------------------------------------------------------------------------'
  print >> F, 'Half-power Switching Test Result'
  print >> F, 'uid:        %25s                 Source:        %14s'          %(uid, sourceName[0])
  print >> F, 'PSC data:   %25s                 Elevation:     %14.1f'        %(pscfile, Elevation)
  print >> F, 'Date:       %25s                 Band:          %14s'          %(obsDate, band)
  print >> F, 'Mean Temperature [C] = %14.2f                 Moved Antenna: %14s'          %(temp,movedAnt)
  print >> F, 'Mean Wind Speed  [m/s] = %12.2f' %(wind)
  print >> F, '--------------------------------------------------------------------------------------'
  #Printing out the statistics
  print >> F, 'S/N On-Source:                  %10.5f'%(avgOn/stdOn)
  print >> F, 'S/N at HP:                      %10.5f'%(avgHP/stdHP)
  print >> F, 'AmpHP/AmpOnSource:              %10.5f'%(avgHP/avgOn)
  #print >> F, 'rms offset On-Source  (arcsec):    %10.5f'%(stdOnOff)
  print >> F, 'rms offset at HP (arcsec):      %10.5f'%(stdHPOff)
  print >> F, '--------------------------------------------------------------------------------------'
  for l in [0,5,10]:
    print >> F, 'Switch: ',(l/5)+1
    print >> F, 'Maximum pointing error after ',jj,'s    (arcsec): '+5*'%7.2f'%(maxO1p5s[0+l],maxO1p5s[1+l],maxO1p5s[2+l],maxO1p5s[3+l],maxO1p5s[4+l])
    print >> F, 'Time of the Max after the switch            (s): '+5*'%7.2f'%(tmaxO1p5s[0+l],tmaxO1p5s[1+l],tmaxO1p5s[2+l],tmaxO1p5s[3+l],tmaxO1p5s[4+l])
    print >> F, 'Average pointing error after ',jj,'s    (arcsec): '+5*'%7.2f'%(avgO1p5s[0+l],avgO1p5s[1+l],avgO1p5s[2+l],avgO1p5s[3+l],avgO1p5s[4+l]) 
    print >> F, '                  Standard deviation   (arcsec): '+5*'%7.2f'%(stdO1p5s[0+l],stdO1p5s[1+l],stdO1p5s[2+l],stdO1p5s[3+l],stdO1p5s[4+l])
    print >> F, 'Average pointing error after ',kk,'s    (arcsec): '+5*'%7.2f'%(avgO2s[0+l],avgO2s[1+l],avgO2s[2+l],avgO2s[3+l],avgO2s[4+l])
    print >> F, '                  Standard deviation   (arcsec): '+5*'%7.2f'%(stdO2s[0+l],stdO2s[1+l],stdO2s[2+l],stdO2s[3+l],stdO2s[4+l])
    print >> F
  
  print >> F, '--------------------------------------------------------------------------------------'
  
  F.close()
  
  ##Plots
  
  Three    = (zeros(len(calTimeZero)))+3.
  PointSix = (zeros(len(calTimeZero)))+0.6
  
  
  #Figure 1. Uncorrected, Corrected and linear fit
  pl.figure(1)
  pl.gcf().clf()
  pl.plot(calTimeZero,calAmp,'b.')
  pl.plot(calTimeZero,gFitAll,'y-')
  pl.plot(calTimeZero,calAmpCor,'g.')
  pl.xlabel("Time [s]")
  pl.ylabel("Amplitude")
  pl.title(uid+' '+movedAnt+'-'+pscfile)
  figname = uid+movedAnt+'_'+pscfile+'.png'
  pl.savefig(figname)
  
  #Figure 2. Offset
  pl.figure(2)
  pl.gcf().clf()
  pl.plot(calTimeZero,gainOff,'b.')
  #pl.plot(calTimeZero,zeros(len(calTimeZero)), 'y')
  pl.xlabel("Time [s]")
  pl.ylabel("Offset (arcsec)")
  pl.title(uid+' '+movedAnt)
  figname = uid+movedAnt+'Offset.png'
  pl.savefig(figname)
  
  #Figure 3. PSC Azimuth and Elevation
  pl.figure(3)
  pl.gcf().clf()
  pl.plot(pscTimeZero,deltaAz,'g.')
  pl.plot(pscTimeZero,deltaEl,'b.')
  pl.xlabel('Time [s]')
  pl.ylabel('PSC EL(blue) and AZ(green) [arcseconds]')
  pl.title(pscfile+ ' - '+ uid)
  figname=uid+movedAnt+'PSCAZEl.png'
  pl.savefig(figname)
  
  #Figure 4. Switch AZ
  pl.figure(4)
  pl.gcf().clf()
  pl.plot(pscTimeZero,deltaAz,'k')
  pl.plot(calTimeZero,gainHPOff,'r')
  pl.plot(calTimeZero,negative(Three),'b')
  pl.plot(calTimeZero,Three,'b')
  pl.plot(calTimeZero,negative(PointSix),'g')
  pl.plot(calTimeZero,PointSix,'g')
  pl.plot(calTimeZero,zeros(len(calTimeZero)),'y')
  for i in range(0,(len(pscAzJumps)/2)):
    bbb = pscTimeZero[pscAzJumps[(2*i)+1]]
    ccc = bbb + jj
    ddd = bbb + kk
    pl.plot([bbb,bbb],[400,-400],'pink')
    pl.plot([ccc,ccc],[400,-400],'c')
    pl.plot([ddd,ddd],[400,-400],'c')
  for i in range(0,(len(pscElJumps)/2)):
    bbb = pscTimeZero[pscElJumps[(2*i)+1]]
    ccc = bbb + jj
    ddd = bbb + kk
    pl.plot([bbb,bbb],[400,-400],'pink')
    pl.plot([ccc,ccc],[400,-400],'c')
    pl.plot([ddd,ddd],[400,-400],'c')
  
  pl.xlabel('Time [s]')
  pl.ylabel('Gain Offset(red) and PSC Az(black) [arcseconds]')
  pl.title(pscfile+ ' - '+ uid)
  pl.ylim(-4,4)
  pl.xlim(calTimeZero[gSwitches[0]],calTimeZero[gSwitches[9]+119])
  figname=uid+movedAnt+'AzSwitch1.png'
  pl.savefig(figname)
  pl.xlim(calTimeZero[gSwitches[10]],calTimeZero[gSwitches[19]+119])
  figname=uid+movedAnt+'AzSwitch2.png'
  pl.savefig(figname)
  pl.xlim(calTimeZero[gSwitches[20]],calTimeZero[gSwitches[29]+119])
  figname=uid+movedAnt+'AzSwitch3.png'
  pl.savefig(figname)
  
  
  #Figure 5. Switch EL
  pl.figure(5)
  pl.gcf().clf()
  pl.plot(pscTimeZero,deltaEl,'k')
  pl.plot(calTimeZero,gainHPOff,'r')
  pl.plot(calTimeZero,negative(Three),'b')
  pl.plot(calTimeZero,Three,'b')
  pl.plot(calTimeZero,negative(PointSix),'g')
  pl.plot(calTimeZero,PointSix,'g')
  pl.plot(calTimeZero,zeros(len(calTimeZero)),'y')
  for i in range(0,(len(pscAzJumps)/2)):
    bbb = pscTimeZero[pscAzJumps[(2*i)+1]]
    ccc = bbb + jj
    ddd = bbb + kk
    pl.plot([bbb,bbb],[4,-4],'pink')
    pl.plot([ccc,ccc],[4,-4],'c')
    pl.plot([ddd,ddd],[4,-4],'c')
  for i in range(0,(len(pscElJumps)/2)):
    bbb = pscTimeZero[pscElJumps[(2*i)+1]]
    ccc = bbb + jj
    ddd = bbb + kk
    pl.plot([bbb,bbb],[4,-4],'pink')
    pl.plot([ccc,ccc],[4,-4],'c')
    pl.plot([ddd,ddd],[4,-4],'c')
  
  pl.xlabel('Time [s]')
  pl.ylabel('Gain Offset(red) and PSC El(black) [arcseconds]')
  pl.title(pscfile+ ' - '+ uid)
  pl.ylim(-4,4)
  pl.xlim(calTimeZero[gSwitches[0]],calTimeZero[gSwitches[9]+119])
  figname=uid+movedAnt+'ElSwitch1.png'
  pl.savefig(figname)
  pl.xlim(calTimeZero[gSwitches[10]],calTimeZero[gSwitches[19]+119])
  figname=uid+movedAnt+'ElSwitch2.png'
  pl.savefig(figname)
  pl.xlim(calTimeZero[gSwitches[20]],calTimeZero[gSwitches[29]+119])
  figname=uid+movedAnt+'ElSwitch3.png'
  pl.savefig(figname)
  
  return  
