#import scipy.optimize
import copy

class OnOffScan:
    def __init__(self, vis, onscan, offscan, antenna):
        print " this script is only for reducing data from script spectral tests, no Sched blocks"
        self.vis = vis
        self.onscan = onscan
        self.offscan = offscan
        tb.open(self.vis)
        print "test "
        self.tbon = tb.query("SCAN_NUMBER== %d && DATA_DESC_ID==0 && ANTENNA1==%d" % (onscan, antenna))
        self.on = self.tbon.getcol('FLOAT_DATA').sum(2).flatten()
        self.tboff = tb.query("SCAN_NUMBER== %d && DATA_DESC_ID==0  && ANTENNA1==%d" % (offscan, antenna))
        self.off = self.tboff.getcol('FLOAT_DATA').sum(2).flatten()
        self.spec = (self.on.real-self.off.real)/self.off.real
        # restrict to subband 17
        #spec = fullspec[240*16:240*17]
        self.time = self.tbon.getcol('TIME').mean()
        (self.nchan,) = self.spec.shape
        self.chan = pl.arange(self.nchan)
        self.setRestFrequency(86.243440e9)
        self.setLSRVelocity()

    def getField(self):
        # need the field center
        fieldId = self.tbon.getcol('FIELD_ID')[0]
        #
        fieldTb = tbtool.create()
        fieldTb.open(self.vis+'/FIELD')
        self.sourceId = fieldTb.getcol('SOURCE_ID')[fieldId]
        fieldTb.close()

    def getSource(self):
        #
        self.getField()
        sourceTb = tbtool.create()
        sourceTb.open(self.vis+'/SOURCE')
        sourceTb0 = sourceTb.query("SOURCE_ID==%d && SPECTRAL_WINDOW_ID==0" % self.sourceId)
        self.sourceRa = sourceTb0.getcol('DIRECTION')[0][0]
        self.sourceDec = sourceTb0.getcol('DIRECTION')[1][0]
        self.sourceName = sourceTb0.getcol('NAME')[0]
        sourceTb.close()

    def setRestFrequency(self, fr=86.243440e9):
        self.restFrequency = fr

    def setLSRVelocity(self, vlsr=9000.):
        self.LSRVelocity = vlsr

    def getSpectralWindow(self):
        
        dataDescId = self.tbon.getcol('DATA_DESC_ID')[0]
        #
        dataDescTb = tbtool.create()
        dataDescTb.open(self.vis+'/DATA_DESCRIPTION')
        spwId = dataDescTb.getcol('SPECTRAL_WINDOW_ID')[0]
        #
        spwTb = tbtool.create()
        spwTb.open(self.vis+'/SPECTRAL_WINDOW')
        self.refFrequency = spwTb.getcol('REF_FREQUENCY')
        self.chanFreq = spwTb.getcell('CHAN_FREQ',spwId)
        self.numChan = spwTb.getcell('NUM_CHAN',spwId)
        netSideband = spwTb.getcell('NET_SIDEBAND',spwId)
        #
        # reverse order if LSB
        if netSideband is 2:
            for i in range(self.numChan/2):
                temp = self.chanFreq[i]
                self.chanFreq[i] = self.chanFreq[self.numChan-i-1]
                self.chanFreq[self.numChan-i-1] = temp
        #
        # convert to LSR frame
        obs = 'OSF'
        me.doframe(me.observatory(obs))
        me.doframe( me.epoch('UTC',qa.quantity(self.time,'s')))
        direction = me.direction('J2000',
                                 qa.quantity(self.sourceRa,'rad'),
                                 qa.quantity(self.sourceDec,'rad'))
        me.doframe(direction)
        self.chanLsrVel = self.chanFreq.copy()
        for i in range(self.numChan):
            frequency = me.frequency('TOPO',qa.quantity(self.chanFreq[i],'Hz'))
            lsrFrequency = me.measure(frequency,'LSRK')
            self.chanLsrVel[i]=  me.todoppler('RADIO',
                                              lsrFrequency,
                                              self.restFrequency)['m0']['value']
            

    def setX(self, axis='VLSR', xmin = None, xmax = None):
        self.axis = axis
        self.xmin = xmin
        self.xmax = xmax
        
    def plot(self):
        pl.clf()
        if self.axis is 'VLSR':
            pl.plot(scan.chanLsrVel, scan.spec, '-b')
            pl.xlim(self.xmin, self.xmax)
            pl.xlabel('V$_{LSR}$ / %11.6fGHz'% (self.restFrequency/1e9))
        elif self.axis is 'CHANNEL':
            pl.plot(scan.chan, scan.spec, '-b')
            pl.xlim(self.xmin, self.xmax)
            pl.xlabel('Channel')
        elif self.axis is 'FREQUENCY':
            pl.plot(scan.chanFreq, scan.spec, '-b')
            pl.xlim(self.xmin, self.xmax)
            pl.xlabel('Frequency')
        pl.ylabel('Autocorrelation')
        title = 'Vis: %s Source: %s OnScan: %d OffScan: %d' % \
                ( self.vis, self.sourceName, self.onscan, self.offscan)
        pl.figtext(0.02, 0.98, title, fontsize=12, ha='left',va='top')

    def hardPlot(self, xmin=None, xmax=None):
        self.getField()
        self.getSource()
        self.getSpectralWindow()
        self.setX('VLSR', xmin, xmax)
        self.plot()
        pl.savefig('%s_%s_%s.png' % (self.vis, self.onscan, self.sourceName))

#vis = 'uid___X5f_Xbce4_X1.ms'
#scan = OnOffScan(vis, 5, 6)
#scan.hardPlot()
