# OnOffScan.py
#
# Plotter for single-dish spectral data taken using sched blocks.
# 
# (nb: hacked to also work on data taken using scripts)
#
# run by casapy-test   execfile("OnOffScan.py")  vis='uid__.....ms'  scan=OnOffScan(vis,1,0)   scan.hardplot(2)
# where:
#   1 is the scan number  (starting from 1)
#   0 the antenna number  (starting from 0)
# and in scan.hardplot(2):
#   2 is the polarisation to be plotted (set to 0 to plot both, 1 to plot XX and 2 to plot YY only)
#
# if scan number is negative (eg -5), it will combine the first 5 scans onto the same plot (eg for viewing complete spectral scans)
# combines all the compatible ons and all the offs, as well as scans containing multiple SPW's
# works for dual or single polarisation data.
#
# This version (May 19) will also work for data taken using manual spectral-check scripts.
# The data in that case is saved as scans rather than subscans, so it takes the next scan as the OFF. This should be ok for all
# except multiple repeats (ie using option -r greater than 1).
#
# wdent, May 19
#
#
import copy
import matplotlib

class OnOffScan:
    def __init__(self, vis, Nscan, antenna):
        self.vis = vis
        self.Nscan=Nscan
        tb.open(self.vis)
        if Nscan < 0:
            scan_start=1
            scan_end=1-Nscan
        else:
            scan_start=Nscan
            scan_end=Nscan+1

        self.spA=[]           # initialises arrays for on-off data
        self.spB=[]
        self.Freq=[]

        self.plot_xmin=999.0e9      # initial values for plot xmin,xmax
        self.plot_xmax=0.

        N_spw=0

        for iscan in range(scan_start,scan_end):
#           print "iscan", iscan                 # the scan number being analysed

# first find DATA_DESC_ID list for this scan number:
           tb1=tb.query("SCAN_NUMBER==%d && ANTENNA1==%d && PROCESSOR_ID=0" % (iscan,antenna))
	   dd_ID=tb1.getcol('DATA_DESC_ID')
#           print "dd_ID", dd_ID

# next find which SPW ID's correspond to this:
	   dataDescTb=tbtool.create()
	   dataDescTb.open(vis+'/DATA_DESCRIPTION')
	   spw_ID = dataDescTb.getcol('SPECTRAL_WINDOW_ID')

	   spwTb = tbtool.create()
	   spwTb.open(vis+'/SPECTRAL_WINDOW')

#  find all the valid correlator spectral SPW's and hence dd_ID's (with num_chan>10, ie not channel_average data)
	   DD_list=[-1,-1,-1,-1,-1,-1,-1,-1]
	   self.DDN=0
	   for i in range(0,len(dd_ID)):
		numChan = spwTb.getcell('NUM_CHAN',spw_ID[dd_ID[i]])
#                print "i,numChan",i,numChan
		if numChan > 10:
		 have_DD=0
		 for j in range(0,self.DDN):
		    if DD_list[j]==dd_ID[i]:
		      have_DD=1

		 if have_DD == 0:		#don't have this ID, so add to list
		    DD_list[self.DDN]=dd_ID[i]
		    self.DDN = self.DDN + 1		# no. of valid DD'd (and spW's) (ie is correlator spectral data)

#           self.DDN=1
#           DD_list[0]=1
#	   print "DD_list",DD_list,self.DDN    # DD_list contains list of DATA_DESC_ID for correct SPW


# next find STATE_ID list for this scan number:
           st_ID=tb1.getcol('STATE_ID')

           stateTb=tbtool.create()
           stateTb.open(vis+'/STATE')

#  find all the valid OBS_MODEs and hence st_ID's for ON- and OFF-source in this list:

           st_ON_list=[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1]
           st_OFF_list=[-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1]
           stN_ON=0
           stN_OFF=0

	   print len(st_ID)," subscans in data"
           for i in range(0,len(st_ID)):
                OnOff = stateTb.getcell('OBS_MODE',st_ID[i])[15:17]     #this is either "ON" or "OF"
#		print i,OnOff,st_ID[i],stateTb.getcell('OBS_MODE',st_ID[i])
		if OnOff=="OF":
                   have_st=0
                   for j in range(0,stN_OFF):
                      if st_OFF_list[j]==st_ID[i]:
                        have_st=1
                   if have_st == 0:               #don't have this ID, so add to OFF list
                      st_OFF_list[stN_OFF]=st_ID[i]
                      stN_OFF = stN_OFF + 1
                elif OnOff=="ON":
                 have_st=0
                 for j in range(0,stN_ON):
                    if st_ON_list[j]==st_ID[i]:
                      have_st=1
                 if have_st == 0:               #don't have this ID, so add to ON list
                    st_ON_list[stN_ON]=st_ID[i]
                    stN_ON = stN_ON + 1


#	   print "ONs:", st_ON_list,stN_ON      #contains list of state_ID for ON
#	   print "OFFs:", st_OFF_list,stN_OFF    # similar for OFFs


	   tb.open(vis)


# outer loop over all valid DDIDs:
	   for I_D in range(0,self.DDN):
#	for I_D in range(0,1):
             N_spw=N_spw+1         # spw number (increments each spw and for each scan)
	     DDID=DD_list[I_D]        #data description ID
	     spID=spw_ID[DDID]	   #corresponding spectral window ID

	     for I_S in range(0,stN_ON):
#	     for I_S in range(0,1):
                STID=st_ON_list[I_S]
#		print "getting ON data for I_D,I_S,DDID,STID",I_D,I_S,DDID,spID,STID
#                print "STID",STID
		self.tbon = tb.query("SCAN_NUMBER== %d && DATA_DESC_ID==%d && STATE_ID==%d && ANTENNA1==%d" % (iscan, DDID, STID, antenna))
        	self.on = self.tbon.getcol('DATA')
#        	print self.on.shape

        	npols=self.on.shape[0]           # number of polarisations in the data
                print npols,"-polarisation data"
        	if I_S==0:
			self.on1_0 = self.on.sum(2)[0,:]
                        if npols==2:	self.on1_1 = self.on.sum(2)[1,:]
		else:
			self.on1_0 = self.on1_0+self.on.sum(2)[0,:]
                        if npols==2: self.on1_1 = self.on1_1+self.on.sum(2)[1,:]
#		print self.on1_0

        	STID=st_OFF_list[I_S]
		if STID == -1:
		  print "no OFF data in this scan - using next scan for OFF. This must be data from a python script"
# 		  print "getting OFF data for I_D,I_S,DDID,STID",I_D,I_S,DDID,spID,STID
        	  self.tboff = tb.query("SCAN_NUMBER== %d && DATA_DESC_ID==%d && STATE_ID==%d && ANTENNA1==%d" % ((iscan+1), DDID, (STID+2), antenna))
		else:
        	  self.tboff = tb.query("SCAN_NUMBER== %d && DATA_DESC_ID==%d && STATE_ID==%d && ANTENNA1==%d" % (iscan, DDID, STID, antenna))

        	self.off = self.tboff.getcol('DATA')
#        	print self.off.shape
#                print "doing off", npols

#                print "STID", STID
                if I_S==0:
                        self.off1_0 = self.off.sum(2)[0,:]
                        if npols==2:  self.off1_1 = self.off.sum(2)[1,:]
                else:
                        self.off1_0 = self.off1_0+self.off.sum(2)[0,:]
                        if npols==2:  self.off1_1 = self.off1_1+self.off.sum(2)[1,:]
#        	print self.off1_0.shape
#		print self.off1_1.shape

             self.spec0 = (self.on1_0.real-self.off1_0.real)/self.off1_0.real
             if npols==2:  self.spec1 = (self.on1_1.real-self.off1_1.real)/self.off1_1.real

# comment the following lines out to avoid subtracting the mean off each spectrum (to make them close to zero)
             m0=self.spec0.mean()
             self.spec0=self.spec0-m0
             if npols==2:
                  m1=self.spec1.mean()
                  self.spec1=self.spec1-m1


#	     self.spec0=self.on1_0		# uncomment these 2 lines if we want to show the unswitched spectra
#	     self.spec1=self.on1_1

#	     print "SPW number ", I_D, " shape (pol1, pol2) ", self.spec0.shape, self.spec1.shape

	     self.spA.append(self.spec0)

#	     print "pol1", self.spA
             if npols==2:  self.spB.append(self.spec1)
#             print "pol2", self.spB

             self.time = self.tbon.getcol('TIME').mean()
#             print self.spec0.shape,"shape"
             (self.nchan,) = self.spec0.shape
             self.chan = pl.arange(self.nchan)

             spwTb=tbtool.create()
             spwTb.open(vis+'/SPECTRAL_WINDOW')
             a=spwTb.getcol('REF_FREQUENCY')

             self.setLSRVelocity()
# this section gets freq axis paras:
             dataDescId = self.tbon.getcol('DATA_DESC_ID')[0]
        #
             dataDescTb = tbtool.create()
             dataDescTb.open(self.vis+'/DATA_DESCRIPTION')
             spwId = dataDescTb.getcol('SPECTRAL_WINDOW_ID')[0]
        #
             spwTb = tbtool.create()
             spwTb.open(self.vis+'/SPECTRAL_WINDOW')
             self.refFrequency = spwTb.getcol('REF_FREQUENCY')
             self.numChan = spwTb.getcell('NUM_CHAN',spID)
             self.chanFreq = spwTb.getcell('CHAN_FREQ',spID)
	     self.Freq.append(self.chanFreq)
#	     print "Freq shape ", self.chanFreq.shape

             netSideband = spwTb.getcell('NET_SIDEBAND',spID)
             xmin = self.chanFreq.min()
	     if xmin < self.plot_xmin:
	        self.plot_xmin=xmin
	     xmax = self.chanFreq.max()
             if xmax > self.plot_xmax:
	        self.plot_xmax=xmax
#            print self.numChan
#             print self.plot_xmin, self.plot_xmax
             print "this spw/scan, freq range: ", xmin/1.e9, xmax/1.e9, " GHz"

             self.N_spw=N_spw              # this will be the number of spw/scan  being plotted
             print "analysed spw/scan ", N_spw
             self.npols=npols


# now plot the data:
#        self.getField()
#        self.getSource()
#        self.plot()
#        pl.savefig('%s_%s_%s.png' % (self.vis, self.iscan, self.sourceName))


#                   --- - - --

    def getField(self):
        # need the field center
        fieldId = self.tbon.getcol('FIELD_ID')[0]
        #
        fieldTb = tbtool.create()
        fieldTb.open(self.vis+'/FIELD')
        self.sourceId = fieldTb.getcol('SOURCE_ID')[fieldId]
        fieldTb.close()

    def getSource(self):
        #
        self.getField()
        sourceTb = tbtool.create()
        sourceTb.open(self.vis+'/SOURCE')
        sourceTb0 = sourceTb.query("SOURCE_ID==%d && SPECTRAL_WINDOW_ID==0" % self.sourceId)
        self.sourceRa = sourceTb0.getcol('DIRECTION')[0][0]
        self.sourceDec = sourceTb0.getcol('DIRECTION')[1][0]
        self.sourceName = sourceTb0.getcol('NAME')[0]
        print self.sourceName
        sourceTb.close()

    def setRestFrequency(self, fr=0.):
        self.restFrequency = fr

    def setLSRVelocity(self, vlsr=0.):
        self.LSRVelocity = vlsr

    def setX(self, axis='FREQUENCY', xmin = None, xmax = None):
        self.axis = axis
        self.xmin = chanFreq.min()
        self.xmax = chanFreq.max()
        print "frequency range ",self.xmin,self.xmax

    def plot(self, polplot,xmin,xmax):
        pl.clf()
	for ip in range(0,self.N_spw):
          pl.grid(True)
#	  print scan.Freq[ip]
          if polplot==1 or polplot==0:
              pl.plot(scan.Freq[ip]/1.0e9, scan.spA[ip], '-r')
              print "plotting baseband ", ip, "pol 1"
          if self.npols==2 and (polplot==2 or polplot==0):
              pl.plot(scan.Freq[ip]/1.e9, scan.spB[ip], '-b')
              print "plotting baseband ", ip, "pol 2"
       
#  this section adds two extra lines to the plot to show the difference between basebands
#  so comment out this section if the basebands are not at exactly the same frequency (as it works by channel number, and would not make sense)
#	 spC=[]		# difference between basebands
#        spD=[]          # difference between basebands
#        spC=scan.spA[0]-scan.spA[1]
#        spD=scan.spB[0]-scan.spB[1]
#        pl.plot(scan.Freq[ip]/1.0e9, spC, '-g')
#	 pl.plot(scan.Freq[ip]/1.0e9, spD, '-y')

        if xmin == None:
	  pl.xlim(scan.plot_xmin/1.0e9, scan.plot_xmax/1.0e9)
	else:
 	  pl.xlim(xmin,xmax)

        pl.xlabel('Frequency/GHz')
#        locs,labels=pl.xticks()
#	pl.xticks(locs,map(lambda x: "%.2f" % x,locs))
        pl.ylabel('Autocorrelation (r=XX,b=YY))')
        title = 'Vis: %s Source: %s Scan: %d ' % \
                ( self.vis, self.sourceName, self.Nscan)
        pl.figtext(0.02, 0.98, title, fontsize=12, ha='left',va='top')


    def hardPlot(self, polplot, xmin=None, xmax=None):
#  polplot is the polarisation to be plotted (0 for both, 1=XX, 2=YY)
        self.getField()
        self.getSource()
        self.plot(polplot,xmin,xmax)
        pl.savefig('%s_%s_%s.png' % (self.vis, self.Nscan, self.sourceName))

