import os
import scipy
from scipy import polyfit
import pylab as pl
from taskinit import *

class Visibility:
    def __init__(self,inputMs):
        self.inputMs = inputMs
        nms = self.inputMs.index('.')
        tb.open(self.inputMs+'/ANTENNA')
        self.antList = tb.getcol('NAME')
        self.nants = pl.size(self.antList)
        tb.open(inputMs)
        if nms>0:
            self.asdm = self.inputMs[0:nms]

    def getChannelAverageData(self):
        self.chanAverPhase={}
        self.chanAverAmpli={}
        ants = self.antList
        andids = self.nants
        ants  = ['DA41','DV19']
        andids = 2
        for a2 in range(andids):
            for a1 in range(andids):
                if a1 < a2:
                    print a1,a2
                    
                    chanAverTb = \
                               tb.query('ANTENNA1==%s && ANTENNA2==%s && DATA_DESC_ID==1' %\
                                        (a1, a2))
                    self.chanAverData = chanAverTb.getcol('DATA')
                    self.chanAverTime = chanAverTb.getcol('TIME')
                    date  = int(self.chanAverTime[0]/ 86400)
                    self.chanAverTime = self.chanAverTime - date*86400
                    # single pol and single channel:
                    visi = self.chanAverData[0][0]
                    self.chanAverPhase[(ants[a1],ants[a2])] = pl.angle(visi)
                    self.chanAverAmpli[(ants[a1],ants[a2])] = pl.absolute(visi)
                    
        return self.chanAverTime,self.chanAverPhase,self.chanAverAmpli

    def getSpectralData(self, dd=0):
        andids = self.nants
        v.specData = {}
        for a2 in range(andids):
            for a1 in range(andids):
                if a1 < a2:
                    specTb = tb.query('ANTENNA1==%s && ANTENNA2==%s && DATA_DESC_ID==%s' %\
                              (a1, a2, dd))
                    self.specData[(ants[a1],ants[a2])] = specTb.getcol('DATA')
                    self.specTime = specTb.getcol('TIME')
                    date  = int(self.specTime[0]/ 86400)
                    self.specTime = self.specTime - date*86400

    def getSpectralAutoData(self, iant, dd):
        specTb = tb.query('ANTENNA1==%d && ANTENNA2==%d && DATA_DESC_ID==%d' \
                              % (iant, iant ,dd))
        self.specData = specTb.getcol('DATA')
        self.specTime = specTb.getcol('TIME')
        date  = int(self.specTime[0]/ 86400)
        self.specTime = self.specTime - date*86400


    def getAzEl(self):
        tb.open(self.inputMs+'/POINTING')
        self.pointingTime = {}
        self.azimuth = {}
        self.elevation = {}
        antennaId = 0
        for ant in antList:
            self.pointingTime[ant] = []
            self.azimuth[ant] = []
            self.elevation[ant] = []
            pointingTb_1 = tb.query('ANTENNA_ID=%d' % antennaId)
            self.pointingTime[ant] = pointingTb_1.getcol('TIME')
            self.azimuth[ant] = pointingTb_1.getcol('TARGET')[0][0]
            self.elevation[ant] = pointingTb_1.getcol('TARGET')[1][0]
            antennaId += 1
        

    def getDelayEvents(self, delayFile):

        f=open(delayFile)
        lines = f.readlines()
        f.close()
        self.delEvents=[]
        event={}
        antennaEvent={}
        coarseTable={}
        fineTable={}
        delayType=''
        for l in lines:
            if l.find('New Delay Event')>=0:
                print l
                if len(event) > 0:
                    self.delEvents.append(event.copy())
                    event.clear()
                event['time'] = self.toSeconds(l.split('Event:')[1].split('T')[1])
                event['ants'] = []

            if l.find('Antenna:')>=0:
                
                if len(antennaEvent) > 0:
                    event['ants'].append(antennaEvent.copy())
                    antennaEvent.clear()
                antennaEvent['Antenna']=l.split('Antenna:')[1].strip()
            if l.find('Start Time:')>=0:
                t = self.toSeconds(l.split('Start Time:')[1].split('T')[1])
                #if t< 42000: t += 86400
                antennaEvent['Start']=t
            if l.find('Stop Time:')>=0:
                t = self.toSeconds(l.split('Stop Time:')[1].split('T')[1])
                #if t< 42000: t += 86400
                antennaEvent['Stop']=t
            if l.find('Coarse Delays:')>=0:
                antennaEvent['Coarse']=[]
                delayType='Coarse'
            if l.find('Fine Delays:')>=0:
                antennaEvent['Fine']=[]
                delayType='Fine'
            if l.find('RelativeStartTime:')>=0:
                t= l.split('RelativeStartTime:')[1].strip().split(' ')[0]
                d= l.split('Delay:')[1].strip().split(' ')[0]
                antennaEvent[delayType].append([int(t), int(d)])

        return self.delEvents
    

    def antCoarseDelay(self,table,t):
        dt = (t-table['Start'])*1000
        d = table['Coarse'][0][1]
        for row in table['Coarse']:
            if (dt < row[0]) and d is not None:
                return d/1.
            if (dt > row[0]):
                d = row[1]
        if d is not None:
            return d/1.
        else:
            return row[0]

    def antFineDelay(self,table,t):
        dt = (t-table['Start'])*1000
        #d = None
        d = table['Fine'][0][1]
        for row in table['Fine']:
            if (dt < row[0]) and d is not None:
                return d/8.
            if (dt > row[0]):
                d = row[1]
        if d is not None:
            return d/8.
        else:
            return 0
    

    def antDelay(self,table,t):
        return antCoarseDelay(table,t)+antFineDelay(table,t)

    def toSeconds(self,s):
        sp = s.split(':')
        return (int(sp[0])*60+int(sp[1]))*60+float(sp[2])

    def setDelayEventFile(self, delayFile):
        self._delayFile = delayFile
        
    def plot(self, ant1, ant2, tbegin=None, tend=None,
             phase=True, phMin=None, phMax=None,
             ampli=False, ampMin=None, ampMax=None,
             fineDelay=True, coarseDelay=True):

        pl.clf()
        print "getting channel average data"
        self.getChannelAverageData()
        print "done"
        #date  = int(self.chanAverTime[0]/ 86400)
        #self.chanAverTime = self.chanAverTime - date*86400
        numPlot = 0
        fineDelayPlot=[]
        coarseDelayPlot=[]
        if fineDelay:
            for i in range(2):
                numPlot += 1
                fineDelayPlot.append(numPlot)
        if coarseDelay:
            for i in range(2):
                numPlot += 1
                coarseDelayPlot.append(numPlot)
        if ampli:
            numPlot += 1
            ampliPlot = numPlot
        if phase:
            numPlot += 1
            phasePlot = numPlot
        #
        # phase
        if phase:
            pl.subplot(numPlot, 1, phasePlot)
            pl.plot(self.chanAverTime, self.chanAverPhase[(ant1,ant2)], ',r')
            pl.ylabel('Phase (rad.)')
            pl.ylim(phMin, phMax)
            if tbegin is not None:
                pl.xlim(tbegin, tend)
            else:
                tbegin, tend = pl.xlim()
        #
        # ampli
        if ampli:
            pl.subplot(numPlot, 1, ampliPlot)
            pl.plot(self.chanAverTime, self.chanAverAmpli[(ant1,ant2)], ',r')
            pl.ylabel('Amplitude ()')
            pl.ylim(ampMin, ampMax)
            if tbegin is not None:
                pl.xlim(tbegin, tend)
            else:
                tbegin, tend = pl.xlim()
     
        self.getDelayEvents(self._delayFile)

        iant = 0
        for ant in (ant1, ant2):
            for e in self.delEvents:
                for a in e['ants']:
                    if a['Antenna'].strip() == ant:
                        t1 = a['Start']
                        t2 = a['Stop']
                        #print t1,t2
                        if (t2 > tbegin) and (t1 < tend):
                            t = []
                            fdelay =[]
                            cdelay =[]
                            dt = (t2-t1)/1000.
                            for i in range(1000):
                                it = t1+i*dt
                                t.append(it)
                                cdelay.append(self.antCoarseDelay(a,it))
                                #print a, it
                                fdelay.append(self.antFineDelay(a,it))
                            if fineDelay:
                                pl.subplot(numPlot,1,fineDelayPlot[iant])
                                pl.plot(t,fdelay)
                                pl.ylabel('Fine delay (250ps)')
                                pl.title(ant)
                            if coarseDelay:
                                pl.subplot(numPlot,1,coarseDelayPlot[iant])
                                pl.plot(t,cdelay)
                                pl.ylabel('Coarse delay (250ps)')
                                pl.title(ant)
                        
            iant += 1

        for iplot in range(numPlot):
            pl.subplot(numPlot, 1, iplot+1)
            pl.xlim(tbegin, tend)
            pl.xlim(tbegin, tend)
            
        pl.figtext(0.02, 0.98, "%s %s-%s"%(self.asdm, ant1,ant2), fontsize=12, ha='left',va='top')
        pl.savefig("%s-%s-%s.png"%(self.asdm, ant1,ant2))
        
    def getAveragePhase(self, ant1, ant2, tbegin, tend):
        meanph = 0
        meancount = 0
        weight = self.chanAverTime.copy()
        for i in range(len(weight)):
            if weight[i] < tbegin or weight[i] >= tend:
                weight[i] = 0
            else:
                weight[i] = 1
        ph = self.chanAverPhase[ant1,ant2]*weight
        if weight.sum() > 0:
            return ph.sum() / weight.sum()
        else:
            return None

    def getAverageSpectrum(self, ant1, ant2, tbegin, tend):
        weight = self.specTime.copy()
        for i in range(len(weight)):
            if weight[i] < tbegin or weight[i] >= tend:
                weight[i] = 0
            else:
                weight[i] = 1
        vw = self.specData[(ant1,ant2)][0]*weight
        return vw.sum(axis=1)

    def getDelay(self,ant1, ant2, tbegin, tend):
        ph = pl.angle(self.getAverageSpectrum(ant1, ant2, tbegin, tend))
        n = len(ph[10:])
        fr = 2e9*pl.arange(n)/256.
        a,b = polyfit(fr,ph[10:],1)
        print a
        return a/2/pi
        
            
    def setFloogDataFile(self, file):
        self._floogFile = file
        
        
    def getFloogData(self, ants, tmin=0, tmax=86400):
        self.floogTimes = {}
        self.floog={}
        self.flo2 = {}
        self.antennas = ants

        f = open(self._floogFile)
        self._lines = f.readlines()
        f.close()
        for ant in self.antennas:
            self.floogTimes[ant] = [] 
            self.floog[ant] = [] 
            self.flo2[ant] = [] 
            for l in self._lines:
                if l.find(ant)>=0:
                    timeString = l.split(ant)[0].split('T')[1].strip()
                    t = self.toSeconds(timeString)
                    if t>=tmin and t<=tmax:
                        self.floogTimes[ant].append(t)
                        self.floog[ant].append(float(l.split('Off:')[1].split('LO1')[0].strip()))
                        self.flo2[ant].append(float(l.split('Off:')[2].split('Ratio')[0].strip()))
            self.floogTimes[ant] = pl.array(self.floogTimes[ant])        
            self.floog[ant] =  pl.array( self.floog[ant])  
            self.flo2[ant] =  pl.array( self.flo2[ant])  

    def getFloogMeanData(self, ant, tbegin, tend):
        meanph = 0
        meancount = 0
        weight = self.floogTimes[ant].copy()
        for i in range(len(weight)):
            if weight[i] < tbegin or weight[i] >= tend:
                weight[i] = 0
            else:
                weight[i] = 1
        ph = self.floog[ant]*weight
        if weight.sum() > 0:
            return ph.sum() / weight.sum()
        else:
            return None

    def plotSpectra(self, tbegin, tend):
        pl.clf()
        
        nb = len(self.specData)
        for ib in range(nb):
            pl.subplot(nb, 1, ib+1)
            ant1, ant2 = self.specData.keys()[ib]
            ph = pl.angle(self.getAverageSpectrum(ant1, ant2, tbegin, tend))
            pl.plot(ph)
            pl.title(ant1+'-'+ant2)
            pl.ylabel('phase')
            pl.ylim(-pi,pi)

        pl.figtext(0.02, 0.98, "%s %s-%s"%(self.asdm, tbegin, tend), fontsize=12, ha='left',va='top')
        pl.savefig("%s-spectra-%s-%s.png"%(self.asdm, int(tbegin), int(tend)))

    def plotFFT(self,  tbegin=0, tend=86400, fmin=0, fmax=1e10):

        i1 = 0
        n = self.chanAverTime.size
        nb = len(self.chanAverPhase)
        i2 = n
        for i in range(n):
            if self.chanAverTime[i] < tbegin: i1 = i
        for i in range(n-1,-1,-1):
            if self.chanAverTime[i] > tend: i2 = i
        pl.clf()
        pl.rcParams['figure.subplot.hspace']=0.5
        for ib in range(nb):
            pl.subplot(nb, 1, ib+1)
            ant1, ant2 = self.chanAverPhase.keys()[ib]
            ph = self.chanAverPhase[ant1, ant2][i1:i2]
            t = self.chanAverTime[i1:i2]
            phfft = pl.fft(ph)
            phfft[0] = 0
            n = phfft.size
            x = pl.arange(n)/0.096/n
            pl.plot(x, phfft)
            pl.title(ant1+'-'+ant2)
            pl.xlim(max(0,fmin), min(1./0.096/2,fmax))
            pl.xlabel('frequency[Hz]')
        pl.figtext(0.02, 0.98, "%s %s-%s"%(self.asdm, tbegin, tend), fontsize=12, ha='left',va='top')
        pl.savefig("%s-phaseFFT-%s-%s.png"%(self.asdm, int(tbegin), int(tend)))
        
            

            



