#! /usr/bin/env python

"""
A class to enable the batch analysis of a pointing data set
the script calling this class is ReducePointingData.py

$Id: PointingDataReducer.py,v 1.36 2014/10/30 00:37:25 dbarkats Exp $

"""

import string
import getpass
from pylab import *
import os
import sys
import time
import glob

def makeTimeStamp():
    return time.strftime('%Y-%m-%dT%H:%M:%S')

class LogWriter:
    def __init__(self, outfile=''):
        self._outfp = open(outfile, 'a')

    def write(self, string):
        self._outfp.write('%s\n' % string)
        self._outfp.flush()
        print string

    def __del__(self):
        self._outfp.close()

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    
    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''

class PointingDataReducer:
    def __init__(self):
        '''
        
        '''
        self.username = getpass.getuser()
        self.pointing_data_path = '/users/%s/AIV/science' %self.username
        self.tpoint_path = '/users/%s/AIV/science/scripts/tpoint' %self.username
        self.lw = LogWriter('Log.txt')
        
        if (os.path.exists('/data/RADIO/POINT/TPOINTfiles')):
            self.archive_dir = '/data/RADIO/POINT/TPOINTfiles/'  # You are running on osf-red
        else:
            print "Run this from osf-red ONLY.  Exiting!."
            exit()
        
        return

    def _getAsdmAntennas(self, uid):
        cmd = 'grep \'<name>\' ' + uid + '/Antenna.xml'
        a = os.popen(cmd).read()
        b = a.split('<name>')

        returnVal = []

        for i in range(1, size(b) ):
            returnVal.append(b[i][0:4])
        return returnVal
   
    def read_datasofar(self):
        infile='%s/pointingDataSoFar_all.txt'% self.pointing_data_path
        print 'Reading input pointing data from %s' %infile
        b=open(infile)
        obs=b.readlines() 
        data=[]
        for line in obs:
            if (line[0] != '#') and (line[0] != '\n'):
                line2=line.rstrip('\n')
                print line2
                data.append(line2)
              
        return data

    def getDateFromASDM(self,uid):
        date =  os.popen('grep TimeOfCreation %s/ASDM.xml' %uid).read()
        date = date.split('TimeOfCreation>')[1].split('T')[0].replace('-','')
        return date

    def system_except(self, cmd):
        ret = os.system(cmd)
        if ret != 0:
            raise Exception("Command '%s' returned non-zero exit status %d" %(cmd, ret))

    def runRadioPoint(self, uid):
        self.uid=uid.replace("\n","")  # includes the whole path
        
        self.asdm=uid.split("/")[-1]   # just the name of the asdm without path
        
        f_indat = size(glob.glob('*-tpoint.dat'))
        f_inmod = size(glob.glob('*-inmod.mod'))
        if (f_indat !=0) and (f_inmod !=0):
            print "indat and inmod files already exists"
            print "Skipping to run RadioPoint..."
        else:
            print "### Running RadioPoint for all antennas... ###"
            self.system_except("RadioPoint -f %s" %(uid))


    def reducePointing(self, ant=''):
        print "### Analyzing antenna %s... ###" %(ant)

        output=[]
        date =  self.getDateFromASDM(self.uid)
        print date
        # define output filenames
        fig_outMod_png=ant+'_'+self.asdm+'_outModFile.png'
        fig_outMod2_png=ant+'_'+self.asdm+'_outModFile_2.png'
        fig_outMod4_png=ant+'_'+self.asdm+'_outModFile_4.png'
        fig_outMod5_png=ant+'_'+self.asdm+'_outModFile_5.png'
        fig_outMod5b_png=ant+'_'+self.asdm+'_outModFile_5b.png'
        fig_outMod=ant+'_'+self.asdm+'_outModFile.ps'
        fig_outMod2=ant+'_'+self.asdm+'_outModFile_2.ps'
        fig_outMod4=ant+'_'+self.asdm+'_outModFile_4.ps'
        fig_outMod5=ant+'_'+self.asdm+'_outModFile_5.ps'
        fig_outMod5b=ant+'_'+self.asdm+'_outModFile_5b.ps'
        fileoutMod0=ant+'_'+self.asdm+'_outModFile_0.dat'
        fileoutMod2=ant+'_'+self.asdm+'_outModFile_2.dat'
        fileoutMod4=ant+'_'+self.asdm+'_outModFile_4.dat'
        fileoutMod5=ant+'_'+self.asdm+'_outModFile_5.dat'
        fileoutMod5b=ant+'_'+self.asdm+'_outModFile_5b.dat'
        indat_file=self.uid+'-'+ ant+'-tpoint.dat'
        inmod_file=self.uid+'-'+ ant+'-inmod.mod'

        #(a,b,stationName)=aU.getWeatherFromASDM(uid)
        #print uid,b    

        # check if RadioPoint has indeed been run
        if not(os.path.exists(indat_file)) or not(os.path.exists(inmod_file)):
            print "RadioPoint somehow has not been run for this antenna. Go back to the code and figure out why. Skipping..."
            #exit()
            return ['']
            
        # copying indat and inmod to local dir
        self.system_except('cp ' + indat_file + ' lastTpointData.dat')
        self.system_except('cp ' + inmod_file + ' lastPointingModel.dat')
        
        # creating directory to store archival results
        if not (os.path.exists(self.archive_dir+ant)):
            self.system_except("mkdir %s" %(self.archive_dir+ant))
        self.new_archive_dir = self.archive_dir+ant+"/"+date
        if not (os.path.exists(self.new_archive_dir)):
            self.system_except("mkdir %s" %(self.new_archive_dir))

        #copying results of inmod and indat to archive dir on osf-red
        try:
            self.system_except('cp ' + indat_file + ' ' + self.new_archive_dir)
        except:
            print "Skipping copy of indat to osf-red archive_dir..."
        try:
            self.system_except('cp ' + inmod_file + ' ' + self.new_archive_dir)
        except:
            print "Skipping copy of inmod to osf-red archive_dir..."

        ########################################
        # Run Tpoint embedded and with Macros and user-defined Procedures
        ########################################
        # 
        # tpointPath=/users/username/AIV/science/scripts/tpoint  - defines the tpoint path
        # $tpointPath/tpointCmdMacro.dat   - how to run the different Procedures within a Macro
        # $tpointPath/userTpoint.ini        - init file, graphics device, etc
        # $tpointPath/userTpointProcs.dat   - where I define my new procedures, e.g  fitLast_2 
        # $tpointPath/reduceTpointFileList - runs the thing
    
        self.system_except("sed -e 's#__TPOINT_PATH__#%s#' '%s/userTpoint.ini' > userTpoint.ini" % (self.tpoint_path, self.tpoint_path))
        tpoint_cmd='%s/reduceTpointFileList %s >& tmp.txt ' %(self.tpoint_path,ant)
        self.system_except('rm -Rf tptLog.dat OUTMOD*.MOD outmod*.mod TPOINT-2-UNFIT.DAT tmp.txt')
        print "### Running tpoint... ###"
        print tpoint_cmd
        self.system_except(tpoint_cmd)
        
        # Rename output file name and images
        self.system_except('mv OUTMOD.MOD '+ fileoutMod0)
        self.system_except('mv OUTMOD_2.MOD '+ fileoutMod2)
        self.system_except('mv OUTMOD_4.MOD '+ fileoutMod4)
        self.system_except('mv OUTMOD_5.MOD '+ fileoutMod5)
        self.system_except('mv OUTMOD_5B.MOD '+ fileoutMod5b)
        self.system_except('mv TPOINT.PS '+ fig_outMod)
        self.system_except('mv TPOINT_2.PS '+ fig_outMod2)
        self.system_except('mv TPOINT_4.PS '+ fig_outMod4)
        self.system_except('mv TPOINT_5.PS '+ fig_outMod5)
        self.system_except('mv TPOINT_5B.PS '+ fig_outMod5b)

        print "### Converting PS files to PNG... ###" 
        self.system_except('convert -quality 90 -rotate 90 -density 200 %s %s &'\
                  %(fig_outMod,fig_outMod_png))
        self.system_except('convert -quality 90 -rotate 90 -density 200 %s %s &'\
                  %(fig_outMod2,fig_outMod2_png))
        self.system_except('convert -quality 90 -rotate 90 -density 200 %s %s &'\
                  %(fig_outMod4,fig_outMod4_png))
        self.system_except('convert -quality 90 -rotate 90 -density 200 %s %s &'\
                  %(fig_outMod5,fig_outMod5_png))
        self.system_except('convert -quality 90 -rotate 90 -density 200 %s %s &'\
                  %(fig_outMod5b,fig_outMod5b_png))
        
        print "### Analysing tpoint results... ###\n"
        # analyze the results of the fit
        b0=open(fileoutMod0)
        results_0=b0.readlines()

        for line_0 in results_0:
            if ('=CA' in line_0):
                ca_0=float(line_0.split()[1])
            if ('=IE' in line_0): 
                ie_0=float(line_0.split()[1])
            if ('=IA' in line_0):
                ia_0=float(line_0.split()[1])
            if ('=AN' in line_0): 
                an_0=float(line_0.split()[1])
            if ('=AW' in line_0): 
                aw_0=float(line_0.split()[1])
        b0.close()
    
        b2=open(fileoutMod2)
        results_2=b2.readlines()
               
        for line_2 in results_2:
            if ('CA' in line_2) and not ('=' in line_2):
                ca_2=float(line_2.split()[1])
                dca_2 = ca_2 - ca_0
            if ('IE' in line_2) and not ('=' in line_2):
                ie_2=float(line_2.split()[1])
                die_2=ie_2 - ie_0
        b2.close()

        b4=open(fileoutMod4)
        results_4=b4.readlines()
    
        for line_4 in results_4:
            if ('CA' in line_4) and not ('=' in line_4):
                ca_4=float(line_4.split()[1])
                dca_4= ca_4 -ca_0
            if ('IE' in line_4) and not ('=' in line_4):
                ie_4=float(line_4.split()[1])
                die_4 = ie_4 - ie_0
            if ('AN' in line_4) and not ('=' in line_4):
                an_4=float(line_4.split()[1])
                dan_4= an_4- an_0
            if ('AW' in line_4) and not ('=' in line_4):
                aw_4=float(line_4.split()[1])
                daw_4= aw_4- aw_0
        b4.close()
            
        b5=open(fileoutMod5)
        results_5=b5.readlines()
    
        for line_5 in results_5:
            if ('CA' in line_5) and not ('=' in line_5):
                ca_5=float(line_5.split()[1])
                dca_5= ca_5 - ca_0
            if ('IE' in line_5) and not ('=' in line_5):
                ie_5=float(line_5.split()[1])
                die_5 = ie_5 - ie_0
            #if ('IA' in line_5) and not ('=' in line_5):
            if ('IA' in line_5):
                ia_5=float(line_5.split()[1])
                dia_5 = ia_5 - ia_0
            if ('AN' in line_5) and not ('=' in line_5):
                an_5=float(line_5.split()[1])
                dan_5= an_5- an_0
            if ('AW' in line_5) and not ('=' in line_5):
                aw_5=float(line_5.split()[1])
                daw_5= aw_5- aw_0
        b5.close()

        b5b=open(fileoutMod5b)
        results_5b=b5b.readlines()
    
        for line_5b in results_5b:
            if ('CA' in line_5b) and not ('=' in line_5b):
                ca_5b=float(line_5b.split()[1])
                dca_5b= ca_5b - ca_0
            if ('IE' in line_5b) and not ('=' in line_5b):
                ie_5b=float(line_5b.split()[1])
                die_5b = ie_5b - ie_0
            #if ('IA' in line_5b) and not ('=' in line_5b):
            if ('IA' in line_5b):
                ia_5b=float(line_5b.split()[1])
                dia_5b = ia_5b - ia_0
            if ('AN' in line_5b) and not ('=' in line_5b):
                an_5b=float(line_5b.split()[1])
                dan_5b= an_5b- an_0
            if ('AW' in line_5b) and not ('=' in line_5b):
                aw_5b=float(line_5b.split()[1])
                daw_5b= aw_5b- aw_0
        b5b.close()

        lineout = '%s/%s %2.2f %2.2f %2.2f %2.2f %2.2f  % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 8.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f) % 8.2f (%+.2f) % 7.2f (%+.2f) % 7.2f (%+.2f)' %(results_0[0].split('/')[0], results_0[0].split('/')[1],float(results_0[1].split()[2]), float(results_2[1].split()[2]), float(results_4[1].split()[2]), float(results_5[1].split()[2]), float(results_5b[1].split()[2]), ca_2, dca_2, ie_2, die_2, ca_4, dca_4, ie_4, die_4, an_4, dan_4, aw_4, daw_4, ca_5, dca_5, ie_5, die_5, ia_5, dia_5, an_5, dan_5, aw_5, daw_5, ca_5b, dca_5b, ie_5b, die_5b, ia_5b, dia_5b, an_5b, dan_5b, aw_5b, daw_5b)
    
        output.append(lineout)
        return output
     

    def chooseNewPtModel(self, results):
        """
        
        Given an output file from reducePointing over all antennas
        Use some intelligence to decide which changes to the Pt Model
        should be applied to each antenna.
        Choices are:
           - RMS0 < 2arcsec ---> no changes to PtModel
           - RMS0 > 2arcsec AND RMS2 < 2arcsec ---> use fileoutMod2 PtModel
           - RMS0 > 2arcsec AND RMS2 > 2arcsec AND RMS4 < 2arcsec ---> use fileoutMod4 PtModel
           - RMS0 > 2arcsec AND RMS2 > 2arcsec AND RMS4 > 2arcsec AND RMS5 < 2 ---> use fileoutMod5 PtModel
           - RMS0 > 2arcsec AND RMS2 > 2arcsec AND RMS4 > 2arcsec AND RMS5 > 2 ---> manual review of PtModel

        """
        
        self.lw.write("\n########### Processed results for each antenna in %s #########" %self.uid)
        
        antApply = []  # list of antennas to apply
        antMan= []     # list of antennas to work on manually
        path =  os.path.dirname(self.uid)
        date = self.getDateFromASDM(self.uid)

        if (path.lstrip() == '') or (path.lstrip() == self.uid):
            path = '.'  # asdm is in local directory
        elif (path.lstrip().split("/")[1] != 'data'):
            print "The data file must be in /data/RADIO/POINT/AOS/ or in the local working directory"
            exit()
        print path
                
        for line in results:
            if line == '': continue
            ant =line.split()[0].split('/')[0]
            fileoutMod0=ant+'_'+self.asdm+'_outModFile_0.dat'
            new_archive_dir = self.archive_dir+ant+"/"+date
            
            #get the number of sources
            b=open(fileoutMod0)
            res= b.readlines()
            b.close()
            nsrc =float(res[1].split()[1])
            
            rms0 = float(line.split()[1])
            rms2 = float(line.split()[2])
            rms4 = float(line.split()[3])
            rms5 = float(line.split()[4])
            rms5b = float(line.split()[5])
            dia5 = float(line.split()[21].split('(')[1].split(')')[0])
            dia5b = float(line.split()[31].split('(')[1].split(')')[0])
            popSD0 = rms0
            popSD2 = rms2*sqrt(nsrc/(nsrc-2))
            popSD4 = rms4*sqrt(nsrc/(nsrc-4))
            popSD5 = rms5*sqrt(nsrc/(nsrc-5))
            popSD5b = rms5b*sqrt(nsrc/(nsrc-5))

            if nsrc == 0:
                self.lw.write("%s: %d srcs,  popSD0= ?????  popSD2= ?????  popSD4= ?????  popSD5= ?????  popSD5b= ?????     No sources remained after fit. Analyze this antenna manually."   %(ant,nsrc))
                antMan.append(ant)
                
            elif popSD0 < 2:
                #do nothing, do not need to update the PtModel
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   No changes needed"\
                      %(ant,nsrc,popSD0,popSD2,popSD4,popSD5,popSD5b))

            elif popSD2 < 2:
                # apply the fileoutMod2 PtModel
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   Changing CA, IE" \
                      %(ant,nsrc,popSD0,popSD2,popSD4,popSD5,popSD5b))
                self.system_except('cp ./%s_%s_outModFile_2.dat %s/%s_outmod.mod' %(ant,self.asdm,path,ant))
                antApply.append(ant)

            elif popSD4 < 2:
                # apply the fileoutMod4 PtModel
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   Changing CA, IE, AN, AW" \
                      %(ant,nsrc, popSD0,popSD2,popSD4,popSD5,popSD5b))
                self.system_except('cp ./%s_%s_outModFile_4.dat %s/%s_outmod.mod' %(ant,self.asdm,path,ant))
                antApply.append(ant)

            elif popSD5 < 2 or  dia5 > 30:
                # apply the fileoutMod5 PtModel. If delta IA is larger than 30", this would be an antenna integration.
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   Changing CA, IE, IA, AN, AW, but IA was large. Needs more Pt iteration" \
                      %(ant,nsrc,popSD0,popSD2,popSD4,popSD5,popSD5b))
                self.system_except('cp ./%s_%s_outModFile_5.dat %s/%s_outmod.mod' %(ant,self.asdm,path,ant))
                antApply.append(ant)

            elif popSD5b < 2 and dia5b < 30:
                # apply the fileoutMod5b PtModel
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   Changing CA, IE, IA, AN, AW" \
                      %(ant,nsrc,popSD0,popSD2,popSD4,popSD5,popSD5b))
                self.system_except('cp ./%s_%s_outModFile_5b.dat %s/%s_outmod.mod' %(ant,self.asdm,path,ant))
                antApply.append(ant)

            else :
                # user needs to do some work
                self.lw.write("%s: %d srcs,  popSD0= %2.2f\"  popSD2= %2.2f\"  popSD4= %2.2f\"  popSD5= %2.2f\"  popSD5b= %2.2f\"   Could not reach required 2\"  fitting CA,IE,IA,AN,AW. Reanalyze manually"\
                      %(ant,nsrc,popSD0,popSD2,popSD4,popSD5,popSD5b))
                antMan.append(ant)
            
            #Archive outmod to proper archive directory
            if os.path.isfile('%s/%s_outmod.mod'%(path,ant)):
                self.system_except('cp %s/%s_outmod.mod %s/%s-%s-outmod.mod' %(path,ant,new_archive_dir,self.asdm,ant))
            
        if size(antMan) != 0:
            self.lw.write("\nYou must work through manually the PtModels for antennas:%s"%(', '.join(map(str, antMan))))
            self.lw.write("You can use the png plots to help examine the results: eog ANT*.png ")
            self.lw.write("Derive a good output model for these antennas and name it: uid-ANT_outmod.mod \n")

        return antMan, antApply
            
       
    def applyPtModel(self, asdm, antApply = []):
        """
        Run the Export, Edit, Import steps directly on the antennas for which
        we have already derived a model
        """

        self.lw.write('\n### Applying Results for %s ###' %asdm) 
        if size(antApply) == 0:
            print "You must specify a list of antennas to apply the results for. Exiting"
            exit()

        date = self.getDateFromASDM(asdm)
        
        #Archive outmod to proper archive directory a second time
        self.lw.write('### Moving all output models to osf-red:/data/RADIO/POINT/TPOINTFiles/ANT/date/ ###')
        for ant in antApply:
            new_archive_dir = self.archive_dir+ant+"/"+date
            self.system_except('cp ./%s_outmod.mod %s/%s-%s-outmod.mod' %(ant,new_archive_dir,asdm,ant))
            
        antApplytext = (','.join(map(str, antApply)))
        self.lw.write('You have found a new pointing model for the following antennas: %s \n' %antApplytext )
        answer = raw_input('Would you like to proceed with the Export/Edit/Import steps for these antennas. Be careful, if answer is "y", you will proceed with importing these model to the TMCDB. {y|n} ?\n')
        
        if answer == 'y' or answer == 'Y' or answer == 'yes' or answer == 'YES' or answer == 'Yes':
            
            self.lw.write('### Exporting TMCDB PointingModel ###')
            tmcdbModel = os.popen('ExportPointingModel.py').read()
            print tmcdbModel
            cmd_edit = 'EditPointingModel.py -a %s -i %s' %(antApplytext,tmcdbModel.split()[-1])
            self.lw.write(cmd_edit)
            self.system_except(cmd_edit)

            inputComment = raw_input('Ready to ImportPointingModel.py. Please add a comment (asdm and list of antennas being updated are already added automatically) or type Enter to abort:\n')
            
            if inputComment == '':
                self.lw.write('You have chosen to NOT Import the new models to the TMCDB. You can still do it manually with the command: \n')
                self.lw.write('ImportPointingModel.py -f PointingModel-current.xml -m \'%s, Antennas updated: %s, Your comment  \''%(asdm, (' '.join(map(str, antApply)))))
            else:
                cmd_import = 'ImportPointingModel.py -f PointingModel-current.xml -m \'%s, Antennas updated: %s, %s  \''%(asdm, (' '.join(map(str, antApply))),inputComment)
                self.system_except(cmd_import)
               
        else:
            self.lw.write('You have chosen to do the Export/Edit/Import steps manually. Here\'s a reminder')
            self.lw.write("1/ ExportPointingModel.py")
            self.lw.write("2/ EditPointingModel.py -a %s -i PointingModel-XXX.xml (from step 1/)"%antApplytext)
            self.lw.write("3/ ImportPointingModel.py -f PointingModel-current.xml -m \"comments\" ")
            
        return


    def _fixTPointFiles(self):
        import glob
        import os
        ants = glob.glob('*')
        ants = sort(ants)
        
        for ant in ants:
            if not os.path.isdir(ant):
                continue
            os.chdir(ant)
            dates = glob.glob('*')
            dates = sort(dates)
            
            for date in dates:
                if not os.path.isdir(date):
                    continue
                os.chdir(date)
                files = glob.glob('uid*')
                
                for file in files:
                    newant = file.split('-')[1]
                    if ant != newant:
                        print file,'current dir = ./'+ ant+'/'+date+'/'
                        print ' mv %s ../../%s/%s/'%(file,newant,date)
                        os.system('mv %s ../../%s/%s/'%(file,newant,date))
                        raw_input()
                os.chdir('..')
            os.chdir('..')
