#! /usr/bin/env python
"""
A  script that calls PointingDataEditor.py to batch analyse a pointing data set.

$Id: ReducePointingData.py,v 1.14 2013/12/06 20:39:21 dbarkats Exp $

"""
import os
import sys
from pylab import *
from optparse import OptionParser
from PointingDataReducer import *

if __name__ == '__main__':
    usage = '''
    ReducePointingData.py --help

    cd ~
    mkdir pointing_analysis
    cd pointing_analysis

    ReducePointingData.py -f uid___A002_X59c7d4_X50f
    ReducePointingData.py -f /data/RADIO/POINT/AOS/20130106/uid___A002_X59c7d4_X50f
    ReducePointingData.py -f /data/RADIO/POINT/AOS/20130106/uid___A002_X59c7d4_X50f -a DV01,DV02
    ReducePointingData.py  # will pick up the data to be analyzed from ~/AIV/science//pointingDataSoFar_all.txt

    This script works on osf-red     
    '''
    #creating class ...
    pdr=PointingDataReducer()

    #options ....
    parser = OptionParser(usage=usage)
    
    parser.add_option("-a", "--antennas",
                      dest="antennas",
                      type= 'string',
                      help="Comma  separated list of antennas to run the data analysis on, ie.DV01,DV02,DV03. Default is to use all the antennas in the ASDM"
                      )

    parser.add_option("-f", "--filename",
                      dest="pointingFilename",
                      type= 'string',
                      help="ASDM filename containing the pointing data we want to analyze.Can be command separated list of ASDM. If none is provided, this script will read the filenames from the uncommented lines of ~/AIV/science/pointingDataSoFar_all.txt. This is much easier for batch processing of many ASDM")

    parser.add_option("-x","--applyOnly",
                      dest = 'applyOnly',
                      action="store_true",
                      default=False,
                      help="to only apply the resulting pointing models that are available in the working directory")
    
    (options, args) = parser.parse_args()

    os.system('rm -f Log.txt')
    lw = LogWriter('Log.txt')

    antApply = []
    if options.applyOnly:
        # search for antennas which have a model in working directory
        if size(antApply) == 0:
            uid = options.pointingFilename.split(',')
            if size(uid) > 1:
                print " You can only specify one ASDM with option applyOnly."
                exit()
            f_outmod = glob.glob('*_outmod.mod')
            f_outmod = sort(f_outmod)
            for model in f_outmod:
                antApply.append(model[-15:-11])
                
        pdr.applyPtModel(uid[0],antApply)
        exit()

        
    if options.pointingFilename == None :
        pointingData_list=pdr.read_datasofar()
    else:
        pointingData_list=options.pointingFilename.split(',')

    results = []
    legend = 'ant/pad   RMS0 RMS2 RMS4 RMS5 RMS5b     CA2 (\xce\x94)         IE2 (\xce\x94)         CA4 (\xce\x94)         IE4 (\xce\x94)         AN4 (\xce\x94)         AW4 (\xce\x94)         CA5 (\xce\x94)         IE5 (\xce\x94)          IA5 (\xce\x94)         AN5 (\xce\x94)         AW5 (\xce\x94)        CA5b (\xce\x94         IE5b (\xce\x94)         IA5b (\xce\x94)        AN5b (\xce\x94)        AW5b (\xce\x94)'

    
    for pointingData in pointingData_list:
        results=[]
        pointingData = pointingData.strip('/')
        
        antennasInAsdm = pdr._getAsdmAntennas(pointingData)
        if options.antennas == None :
            antennas = antennasInAsdm
        else:
            antennas = []
            antennas_t = options.antennas.split(',')
            for a in antennas_t:
                if a in antennasInAsdm:antennas.append(a)                    
        if size(antennas) == 0: 
            print "Antennas not in ASDM"
            sys.exit()

        pdr.runRadioPoint(pointingData)            
        print "Analyzing %d antennas: %s \n" %(size(antennas),antennas)
        for ant in antennas:
            r = pdr.reducePointing(ant)
            results.append(r[0])

        asdm = pointingData.split("/")[-1]
        lw.write("############## Raw output of ReducePointingData.py for %s   #################" \
                 %asdm)
        lw.write(legend)
        for line in results: lw.write(line)
        antMan,antApply = pdr.chooseNewPtModel(results)
        pdr.applyPtModel(asdm, antApply = antApply)
        
