#!/bin/sh

PYTHONSCRIPT=~/AIV/science/analysis_scripts/addAntPos.py
CURRENTDIR=$PWD

# Add defaults for new options available to addAntPos.py
# and parse command line argumemts  (thunter, 2015/11/10)
MINSIGMA="5.0" 
THRESHOLD="50"
ANTENNASARG=""
EXCLUDEARG=""
RESULTFILE=""
while [[ $# > 1 ]]
  do
  key="$1"
  case $key in
      -m|--minsigma)
	  MINSIGMA="$2"
	  shift # past argument
	  ;;
      -t|--threshold)
	  THRESHOLD="$2"
	  shift # past argument
	  ;;
      -r|--result)
	  RESULTFILE="$2"
	  shift # past argument
	  ;;
      -a|--antennas)
	  ANTENNASARG="-a $2"
	  shift # past argument
	  ;;
      -x|--excludeAntennas)
	  EXCLUDEARG="-x $2"
	  shift # past argument
	  ;;
      *)
          # unknown option, assume it is the file to support previous usage
	  RESULTFILE="$1"
	  ;;
  esac
  shift # past argument or value
done
if [ "$RESULTFILE" == "" ]
    then
    if [ "$1" == "-h" ] || [ "$1" == "--help" ]
	then
	echo "Usage: UpdatePosition.sh -m <minsigma> -t <threshold(microns)> -a <DV01,DV02> -x <PM02> -r <tc_antpos_result_directory>"
	echo "       (the -r flag can be omitted in front of the tc_antpos result)"
	exit
    else
        RESULTFILE="$1"
    fi
fi

HOSTNAME=`hostname`
case "$HOSTNAME" in
    *gns* )echo "you are on $HOSTNAME, which is a GNS machine"
	ARCHDIR=/groups/science/data/RADIO/SYSTEM/AntposResults/
	break;;
    * )
	echo "you are on $HOSTNAME, which is not a GNS machine"
	ACS_ROOT=/data1/opt/telcalsa/ACS-10.1.2/ACSSW-ALMA64-9_1_1-B-2012-11-28-03-00-00
	TELCALSA_ROOT=/data1/opt/telcalsa/casapy-34.0.19988-002-64b/telcalsa/telcalsa-20130131-alma-9_1_r3-b
	export LD_LIBRARY_PATH="$TELCALSA_ROOT/lib:$LD_LIBRARY_PATH"
    # (ahirota, 2015/09/09) removed ACS_ROOT from PYTHONPATH because otherwise addAntPos.py crashes due to inconsistent version of minidom library.
	export PYTHONPATH="$TELCALSA_ROOT/tasks:$TELCALSA_ROOT/lib/python/site-packages:/lib/python/site-packages:$PYTHONPATH"
	ARCHDIR=/data/RADIO/SYSTEM/AntposResults/ ;;
esac

cd ~
CVSUSER=$USER
echo "Updating your almaAntPos.txt as cvs user $CVSUSER"
cvs -d :pserver:$CVSUSER@nas-04.aiv.alma.cl:2401/project21/CVS update -C AIV/science/PadData/almaAntPos.txt
if [ $? -ne 0 ]
    then
    CVSUSER=aod
    echo "Running cvs update as user $USER failed.  Trying again as $CVSUSER."
    cvs -d :pserver:$CVSUSER@nas-04.aiv.alma.cl:2401/project21/CVS update -C AIV/science/PadData/almaAntPos.txt
    if [ $? -ne 0 ]
	then
	echo "This also failed."
	fi
    fi
echo "Updating your addAntPos.py as cvs user $CVSUSER"
cvs -d :pserver:$CVSUSER@nas-04.aiv.alma.cl:2401/project21/CVS update -C AIV/science/analysis_scripts/addAntPos.py
if [ $? -eq 0 ]
    then
    cd $CURRENTDIR
    cp -r $RESULTFILE $ARCHDIR
    echo "Running " $PYTHONSCRIPT $ANTENNASARG $EXCLUDEARG -m $MINSIGMA -t $THRESHOLD $RESULTFILE
    $PYTHONSCRIPT $ANTENNASARG $EXCLUDEARG -m $MINSIGMA -t $THRESHOLD $RESULTFILE
    if [ $? -eq 0 ]
        then
        cd ~
        echo "Input the comment for commiting the file to the CVS repository:"
        read COMMENT
	echo "Running cvs commit as $CVSUSER."
        cvs -d :pserver:$CVSUSER@nas-04.aiv.alma.cl:2401/project21/CVS commit -m "$COMMENT" AIV/science/PadData/almaAntPos.txt
        echo "New almaAntPos.txt is committed to the CVS repository"
        fi
    fi
cd $CURRENTDIR
