#!/usr/bin/env python

import os
if os.environ.get('CASAPATH') != None:
    os.environ.pop('CASAPATH')
import socket
import sys
import re
import getpass
import numpy as np

threshold = 0.0 # default = 50 microns (same as EditPositions.py)
minSigma = 0.0 # default (same as EditPositions.py)
antennas = [] # default (include all antennas)
excludeAntennas = [] # default (exclude no antennas)

hostname = socket.gethostname()
username = getpass.getuser()
if ('aod' in username) or ('almaop' in username):
    print "ERROR: This script should not be used as aod or almaop. Please log in and run it as yourself. Exiting..."
    sys.exit()
sys.path.append('/users/%s/AIV/science/analysis_scripts/'%username)

if ('red' in hostname) or ('casa' in hostname) or ('sciops' in hostname):
    print 'WARNING: You are running on an analysis machine \"%s\" as user \"%s\". Using casapy-telcal asdm bindings (APO-450). You must have the proper path setup. Contact Denis Barkats if the asdm module fails to load ####' %(hostname, username)
    from XmlObjectifier import *
elif ('gns' in hostname) or ('crc' in hostname):
    # on aos-gns or crc
    print '### You are running on an STE machine \"%s\" as user \"%s\" ###'  %(hostname, username)
    from Acspy.Util.XmlObjectifier import *

from asdm import *
import pylab as pl
import copy
from math import *
import datetime

def new_prettify(outObj):
    uglyXml = outObj.toprettyxml(indent=' '*2)
    prettyXml = '\n'.join([line for line in uglyXml.split('\n') if line.strip()])
    return prettyXml 
    
# only available in python 2.4
#from xml.dom.ext import PrettyPrint

now = datetime.datetime.now()
now = now.strftime('%Y-%m-%dT%H:%M:%S')

# Parse command-line options
for i in range(1,len(sys.argv)):
    if (sys.argv[i].find('-t') >= 0):
        i += 1
        threshold = float(sys.argv[i])
        print "threshold = %f microns" % (threshold)
    elif (sys.argv[i].find('-m') >= 0):
        i += 1
        minSigma = float(sys.argv[i])
        print "minSigma = %.1f" % (minSigma)
    elif (sys.argv[i].find('-a') >= 0):
        i += 1
        antennas = sys.argv[i].split(',')
        print "antennas to possibly update = ", antennas
    elif (sys.argv[i].find('-x') >= 0):
        i += 1
        excludeAntennas = sys.argv[i].split(',')
        print "antennas to exclude = ", excludeAntennas
    elif (sys.argv[i].find('-h') >= 0):
        print "Usage: %s -t <threshold(microns)> -m <minSigma> -a <DV01,DV02> -x <PM02> uid___AXXX_XXXXXXXX_XXX_delays_U_DXXX_WET_antpos_results"
        sys.exit(0)
    else:
        asdmName = sys.argv[i]

print "using results file = ", asdmName
threshold *= 0.000001 # convert from microns to meters
asdmName = os.path.split(asdmName)
if asdmName[1] == '':
    asdmName = os.path.split(asdmName[0])

a=ASDM()
apo=ASDMParseOptions()   # fixed on 5/11/2012 K. Tachihara
apo.asALMA()
apo.asV3()
apo.loadTablesOnDemand(True)
try:
    a.setFromFile(os.path.join(asdmName[0], asdmName[1]), apo)
except ConversionException, e:
    print e.getMessage()
    sys.exit(1)

# TODO:  CVS UPDATE those files first
f=open(os.path.expanduser('~/AIV/science/PadData/almaAntPos.txt'), 'r')
outObj=XmlObject(f.read())
f.close()

g=open(os.path.expanduser('~/AIV/science/PadData/antennaMoves.txt'), 'r')
gc=g.read().splitlines()
g.close()

gc=sorted(gc)

ij=-1
antMoves={}
for line in gc:
    line=line.strip()
    if line[0] == '#': continue
    ele=line.split()
    ij=ij+1
    antMoves[ij]={}
    antMoves[ij]['Date']=datetime.datetime.strptime(ele[0], '%Y-%m-%dT%H:%M')
    antMoves[ij]['Antenna']=ele[1]
    antMoves[ij]['From']=ele[2]
    antMoves[ij]['To']=ele[3]

list=a.calPositionTable().get()

refAnt=[]
posMethod=[]
startTime=[]
for row in list:
    if row.axesOffsetFixed() == False:
        sys.exit("ERROR: These baseline solutions used axes offsets fitting, which is not supported yet.")
    refAnt1=row.refAntennaNames()
    refAnt1=','.join(refAnt1)
    refAnt.append(refAnt1)
    posMethod.append(str(row.positionMethod()))
    startTime.append(str(row.startValidTime()))

refAnt=sorted(dict.fromkeys(refAnt).keys())
#if len(refAnt) != 1:
#    sys.exit("ERROR: Reference antenna is not the same for all solutions.")

posMethod=sorted(dict.fromkeys(posMethod).keys())
if len(posMethod) != 1:
    sys.exit("ERROR: Position method is not the same for all solutions.")

startTime=sorted(dict.fromkeys(startTime).keys(), reverse=True) # Take the final one, like Robert's CompareAntPosResults does.
#if len(startTime) != 1:
#    sys.exit("ERROR: Start time is not the same for all solutions.")
startTime=(re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', startTime[0]))[0]
startTime=datetime.datetime.strptime(startTime, '%Y-%m-%dT%H:%M:%S')

baseRunObj = XmlObject("""<BaselineRun />""")

baseRunObj.BaselineRun.setAttribute('asdm', asdmName[1])
baseRunObj.BaselineRun.setAttribute('measTime', datetime.datetime.strftime(startTime, '%Y-%m-%dT%H:%M:%S'))
baseRunObj.BaselineRun.setAttribute('refAnt', refAnt[0])
baseRunObj.BaselineRun.setAttribute('posMethod', posMethod[0])
baseRunObj.BaselineRun.setAttribute('reducTime', now)
updates = 0
for row in list:

    antObj = XmlObject("""<Antenna />""")

    stationName=row.stationName()
    antName=row.antennaName()

    antObj.Antenna.setAttribute('name', antName)
    antObj.Antenna.setAttribute('pad', stationName)

    ij=-1
    antMoves1={}
    for i in range(len(antMoves)):
        if antMoves[i]['Antenna'] == antName:
            ij=ij+1
            antMoves1[ij]=antMoves[i]

    if len(antMoves1) == 0:
        sys.exit("ERROR: No antenna move information available for "+str(antName))
    if startTime < antMoves1[0]['Date']:
        sys.exit("ERROR: No antenna move information available for "+str(antName))

    for i in range(len(antMoves1)-1, -1, -1):
        if antMoves1[i]['Date'] <= startTime:
            break

    if antMoves1[i]['To'] != stationName:
        sys.exit("ERROR: Current data file has antenna "+str(antName)+" on pad "+str(stationName)+" while latest move recorded is on pad "+str(antMoves1[i]['To']))

    putInTime=datetime.datetime.strftime(antMoves1[i]['Date'], '%Y-%m-%dT%H:%M:%S')
    antObj.Antenna.setAttribute('putInTime', putInTime)

    stationPosition=row.stationPosition()

    padPosObj = XmlObject("""<PadPosition />""")
    padPosObj.PadPosition.setAttribute('X', '%10.6f' %(stationPosition[0].get()))
    padPosObj.PadPosition.setAttribute('Y', '%10.6f' %(stationPosition[1].get()))
    padPosObj.PadPosition.setAttribute('Z', '%10.6f' %(stationPosition[2].get()))

    copy = padPosObj.PadPosition.cloneNode(1)
    antObj.Antenna.appendChild(copy)

    antPosition=row.antennaPosition()
    positionOffset=row.positionOffset()

    antennaVector=[]
    for i in range(3):
    	antennaVector.append(antPosition[i].get()+positionOffset[i].get())

    antVecObj = XmlObject("""<AntennaVector />""")
    antVecObj.AntennaVector.setAttribute('X', '%10.6f' %(antennaVector[0]))
    antVecObj.AntennaVector.setAttribute('Y', '%10.6f' %(antennaVector[1]))
    antVecObj.AntennaVector.setAttribute('Z', '%10.6f' %(antennaVector[2]))

    copy = antVecObj.AntennaVector.cloneNode(1)
    antObj.Antenna.appendChild(copy)

    antPositionErr=row.positionErr()
    posErr=[]
    for i in range(3):
    	posErr.append(antPositionErr[i].get())

    posErrObj = XmlObject("""<PositionError />""")
    posErrObj.PositionError.setAttribute('X', '%10.6f' %(posErr[0]))
    posErrObj.PositionError.setAttribute('Y', '%10.6f' %(posErr[1]))
    posErrObj.PositionError.setAttribute('Z', '%10.6f' %(posErr[2]))
    totalCorrection = np.linalg.norm([positionOffset[0].get(),
                                      positionOffset[1].get(),
                                      positionOffset[2].get()]) 
    if (antName not in refAnt):
        if totalCorrection > 0:
            # the following expression is identical to what Robert uses in PositionEditor.py
            totalError = (abs(posErr[0]*positionOffset[0].get()) + abs(posErr[1]*positionOffset[1].get()) + abs(posErr[2]*positionOffset[2].get()) )/totalCorrection
            snr = totalCorrection/totalError
        else:
            totalError = 0
            snr = 0
        if (snr > minSigma and totalCorrection>threshold  and (len(antennas)==0 or antName in antennas)
            and (antName not in excludeAntennas)):
            updating = "  updating"
            updates += 1
            copy = posErrObj.PositionError.cloneNode(1)
            antObj.Antenna.appendChild(copy)

            copy = antObj.Antenna.cloneNode(1)
            baseRunObj.BaselineRun.appendChild(copy)
        else:
            updating = ''
        print "%s: change=%f  snr=%4.1f   %s" % (antName, totalCorrection, snr, updating)
    else:
        print "%s: refAnt" % (antName)

if (updates > 0):
    copy = baseRunObj.BaselineRun.cloneNode(1)
    outObj.AntennaPositions.appendChild(copy)
    cvsfile = os.path.expanduser('~/AIV/science/PadData/almaAntPos.txt')
    print "Updating ", cvsfile
    f = open(cvsfile, 'w')
    prettyXml = new_prettify(outObj)
    f.write(prettyXml)
    f.close()
else:
    print "No need to update almaAntPos.txt as no changes were made."
    sys.exit(-1)
