#! /usr/bin/env python
"""
Script Name : ampCalReduce.py

Written on 08 Mar 2011  by Paolo G. Calisse
version 1.0
original copy: tfint-gns.aiv.alma.cl://users/pcalisse/AIV/Amplitude_calibration/software/v1.0/aivAmpCalReduce.py

Goal : To process aivAmpCalReduce.py results

from CASAPY
       1) folder='<name of the directory containing all the MS>' (may be a relative or absolute path
       2) import aivAmpCalReduce
       3) aivAmpCalReduce.reduceAllAmpCal(folder,<antenna name>)

from CLI
       1) aivAmpCalReduce -a '<antenna name>' -f '<folder name>'

the software will save in current directories plots of
all the skidip and A CSV file will log all the results obtained.

STILL TO DO:
- add IF bias evaluation 
- understand why using RJ jVal values gives efficiency > 1
- complete beam script (actually is this no more required?)
- convert to a command line script
- integrate to ampCal.py for real time analysis
- add verbose option to remove unrequired printout
- compute goodness of fit (chi-square) on leastsq
- add error budget to all quantities

"""

#def allImport ():

import sys, os, glob
sys.path.append("/users/pcalisse/cvs/AIV/science/analysis_scripts/")

#this is a temporary import: remove after CVS commit!!!
sys.path.append("/groups/science_CSV/analysis_scripts/")

import copy
import numpy as np
import pylab as pl
from matplotlib.font_manager import fontManager, FontProperties
import math as m
import matplotlib.pyplot as plt
import matplotlib as mpl
import analysisUtils as aU
import fileIOPython as fiop
from importasdm import importasdm
from taskinit import *
import time
import datetime as dt
from scipy import *
import csv
from scipy.optimize import leastsq

# a few useful constants
C2K          = 273.14 # conversion factor from Celsius to Kelvin     
h            = 6.626068e-27 # plank constant in erg*sec
k            = 1.3806504e-16 # Boltzman constant in erg/K 
c            = 2.99792458e10 # speed of light in cm/s
tCMB         = 2.725         # CMB temperature in K

# header for the CSV file
headerCSV=['UT', 'vis', 'Scan', 'Antenna', 'antenna ID', 'band', 'freq','source', \
           'Gain XX', 'Gain YY', 'Trx XX', 'Trx YY', 'Cold Load T', 'Hot Load T', \
           'Eff F XX', 'Eff F YY', 'tau0 XX', 'tau0 YY', 'Eff A XX', 'Eff B XX', \
           'Jy/K XX','Jy/K YY', 'Amb T', 'Pressure', 'RH', 'Mean Atm T', 'PWV (model)', 'Error Flag']


    #return
    

def reduceAllAmpCal(dir, antennaName):
    global fileCSV

    # create data file in CSV format
    msList = getMSList(dir)
    print "Processing %s directory: %d available MS:" % (os.path.abspath(dir), len(msList))
    print "\n".join(msList)
    fileCSV        = '%s/%s_AmpCal.csv' % (os.path.abspath(dir), antennaName)
    print "Results saved to %s" % (fileCSV)

    if os.path.exists(fileCSV):
        if confirm("Overwrite existing CSV %s file for antenna %s?" % (fileCSV, antennaName)):
            print " removing file %s..." % (fileCSV),
            os.remove(fileCSV)
            print ' done'
            
    # delete the existing csv directory"
    
    for infile in msList:
        #print "MS name : %s" % infile
        results=reduceSkyDips(infile,antennaName)

    print "\nData reduction complete.\a"

    return

    
def reduceSkyDips(self,antennaName):

    #initialize all return variables
    
    [antID,sourceName,freq,band] =   getScanInfo(self, antennaName)
    errorFlag = 0 # this is the error flag

    # check if antenna is available in given dataset
    if antID==-1:
        print "Antenna %s not available in %s dataset!" % (antennaName, os.path.basename(self))
        return
    if band==-1:
        print "Band not available in %s dataset!" % (band, self)

    obsModeStr     = 'SKYDIP'    # identify the scan numbers for the SKYDIP procedure
    scanNumber     = findScan(self,obsModeStr)

    # Show some information about the current MS
    print "\nMeasurement Set     : %s" % self
    print "Antenna             : %s (index = %d)" % (antennaName, antID)
    print "Source              : %s" % sourceName
    print "Band                : %s" % band
    print "Reference Frequency : %4.1f GHz" % freq
    print "Number of skydips   : %d\n" % len(scanNumber)

    startScanTime=[]
    endScanTime=[]

    label = ["COLD load", "HOT  load", "SKY signal"]

    for scan in range(len(scanNumber)):

        # reduceSkyDip(self,antennaName,scan,band,freq)
        
        # a reminder:
        # in most of the arrays below:
        # j = 0 : cold load
        # j = 1 : hot load
        # j = 2 : sky 

        # find the antenna ID
        antID          = getAntennaIndex(self,antennaName)
        sourceName     = getTarget(self)
        freq           = getRefFrequency(self)
        band           = getBand(self)

        obsModeStr     = 'SKYDIP'    # identify the scan numbers for the SKYDIP procedure
        scanNumber     = findScan(self,obsModeStr)
        print " Scan Number = ", str(scanNumber[scan])

        # read the relevant columns from the data table
        tb.open(self)
        tb_query = ("SCAN_NUMBER==%s && ANTENNA1=%s && ANTENNA2=%s" % (str(scanNumber[scan]), str(antID), str(antID)))
        subt=tb.query(tb_query, columns="TIME, SCAN_NUMBER, STATE_ID, FLOAT_DATA, FIELD_ID")
        tb.close()

        # create lists of relevant data
        timesData         = subt.getcol('TIME')
        totalPower        = subt.getcol('FLOAT_DATA')
        fieldID           = subt.getcol('FIELD_ID')
        stateID           = subt.getcol('STATE_ID')

        # build a list with the first and last index for each new FIELD_ID
        # this fields correspond to each airmass value 
        nAm             = max(fieldID) # number of different fields (corresponding to airmass)
        fieldStart      = np.zeros(nAm)
        fieldEnd        = np.zeros(nAm)
        startTime       = np.zeros(nAm)
        endTime         = np.zeros(nAm)
        
        # create arrays for the 2 different polarization
        # we may want to convert this to a single array later...
        totalPowerXX      = totalPower[0][0]
        totalPowerYY      = totalPower[1][0]

        if len(timesData)<>1176:
            print " Invalid SCAN length: %d rather than %d" % (len(timesData), nAm*3*48)
            errorFlag = errorFlag | 2048
            break
        
        # beginning and end of each airmass measurement
        for i in range(nAm):
            fieldStart[i]=np.nonzero(fieldID==i+1)[0][0]
            
        for i in range(nAm-1):
            fieldEnd[i]=fieldStart[i+1]-1
        fieldEnd[nAm-1]=len(fieldID)-1 # last one must be computed in a different way
        
        # read the relevant columns from the pointing table                                                  
        tb_query = ("ANTENNA_ID=%s" % str(antID))
        
        tb.open(self+'/POINTING')
        subt=tb.query(tb_query, columns="TIME, DIRECTION")
        timesAzEl          =(subt.getcol('TIME'))           
        Az                 =(subt.getcol('DIRECTION')[0])[0]
        El                 =(subt.getcol('DIRECTION')[1])[0]
        tb.close()

        # create interpolated time pointing to correspond to amplitude data
        # and compute airmasses
        nTimesData      = len(timesData)
        AzInt           = np.zeros(nTimesData)
        ElInt           = np.zeros(nTimesData)

        maxDT           = 0.1; # max acceptable delta time in sec.
        for i in range(nTimesData):
            idx = find_nearest(timesAzEl,timesData[i]) # find element of the pointing time array closest to data time array
            if (timesData[i]-timesAzEl[idx]) > maxDT:
                print "WARNING: time difference at index %d = %4.3f sec - data time = %s - pointing time = %s" \
                      % (i, timesData[i]-timesAzEl[idx], fmtHMS(timesData[i]), fmtHMS(timesAzEl[idx]))
                ErrorFlag = ErrorFlag | 1
            AzInt[i]    = Az[idx]
            ElInt[i]    = El[idx]
            
        maxDam = 0.1 # max acceptable delta airmass

        # read the relevant columns from the pointing table                                                  
        tb_query = ("ANTENNA_ID=%s" % str(antID))
        
        # read the relevant columns from the WEATHER table
        tb.open(self+'/WEATHER')
        subWeather         = tb.query("", columns="TIME, TEMPERATURE, REL_HUMIDITY, PRESSURE")
        timesWeather       = subWeather.getcol('TIME')           
        ambTemp            = subWeather.getcol('TEMPERATURE')
        relHum             = subWeather.getcol('REL_HUMIDITY')
        pres               = subWeather.getcol('PRESSURE')
        
        tb.close()

        # print relevant values from the pointing values
        print "\n DETECTOR from %s to %s" % (fmtHMS(timesData[0]), fmtHMS(timesData[len(timesData)-1]))

        print " POINTING from %s to %s" % (fmtHMS(timesAzEl[0]), fmtHMS(timesAzEl[len(timesAzEl)-1])),
        if (timesData[0]<timesAzEl[0] or timesData[len(timesData)-1]>timesAzEl[len(timesAzEl)-1]):
            print "- WARNING: Not enough data in the pointing table!"
            errorFlag = errorFlag | 2
        else:
            print "- enough pointing data"
        
        print " WEATHER  from %s to %s" % (fmtHMS(timesWeather[0]), fmtHMS(timesWeather[len(timesWeather)-1])),
        if (timesData[0]<timesWeather[0] or timesData[len(timesData)-1]>timesWeather[len(timesWeather)-1]):
            print "- WARNING: Not enough data in the weather table!"
            errorFlag = errorFlag | 4
        else:
            print "- enough weather data"
                                                   
        # create interpolated time weather to correspond to amplitude data
        # and compute airmasses
        nTimesWeather      = len(timesWeather)
        ambTempInt         = np.zeros(nTimesData)

        maxDT = 5.; # max acceptable delta time in sec.
        for i in range(nTimesData-1):
            idx = find_nearest(timesWeather,timesData[i]) # find element of the pointing time array closest to data time array 
            #print timesWeather[idx], timesData[i], timesWeather[idx]-timesData[i]
            if (timesData[i]-timesWeather[idx]) > maxDT:
                print "WARNING: time difference at index %d = %4.3f sec - data time = %s - pointing time = %s" \
                      % (i, timesData[i]-timesWeather[idx], fmtHMS(timesData[i]), fmtHMS(timesWeather[idx]))
                errorFlag = errorFlag | 8                
            ambTempInt[i]    = ambTemp[idx]

        # browse STATE table 
        tb.open(self+'/STATE')
        subt              = tb.query('', columns="OBS_MODE, SUB_SCAN, LOAD")
        tb.close()
        obsMode           = subt.getcol('OBS_MODE')           
        subScan           = subt.getcol('SUB_SCAN')
        load              = subt.getcol('LOAD')

        # create array for the hot load, cold load and sky 
        meanTimesData     = np.zeros(shape=(nAm,3))
        meanAmpXX         = np.zeros(shape=(nAm,3))
        meanAmpYY         = np.zeros(shape=(nAm,3))
        meanAmbTemp       = np.zeros(shape=(nAm,3))
        stdAmpXX          = np.zeros(shape=(nAm,3))
        stdAmpYY          = np.zeros(shape=(nAm,3))
        stdAmbTemp        = np.zeros(shape=(nAm,3))
        startIdx          = np.zeros(shape=(nAm,4))
        endIdx            = np.zeros(shape=(nAm,4))
        meanBBTemp        = np.zeros(shape=(3))
        am                = np.zeros(nAm)

        for i in range(nAm):
            for j in range(3):
                startIdx[i,j]=49*(3*i+j)
                endIdx[i,j] =49*(3*i+j+1)-1

        # check for the number of datapoint
        if len(timesData) <> 49*3*nAm :
            print " WARNING: incorrect number of data points: %d rather than %d" % (len(timesData), 49*3*nAm)
            errorFlag = errorFlag | 16
        else:
            print " %d data point available" % len(timesData)
                                                   
        # show relevant data on file on the screen
        print "\n am   target 1st   last      start      end    duration  data"
        for i in range(nAm):
            startTime[i]   = timesData[fieldStart[i]]
            endTime[i]     = timesData[fieldEnd[i]]
            print "%2d airmasses   : %4d %4d  %s %s   %3.1f      %3d" \
                  % (i+1, startIdx[i,0], endIdx[i,2], fmtHMS(startTime[i]), \
                     fmtHMS(endTime[i]), endTime[i]-startTime[i], fieldEnd[i]-fieldStart[i])
            for j in range(3):
                #print "i = %d, j = %d, startIdx = %d, endIdx = %d, len(timesData) = %d" % (i,j, startIdx[i,j], endIdx[i,j], len(timesData))
                print "    %-10s : %4d %4d" % (label[j], startIdx[i,j], endIdx[i,j]),
                print " %s %s " % (fmtHMS(timesData[startIdx[i,j]]), fmtHMS(timesData[endIdx[i,j]])),
                print " %4.1f      %3d " % (timesData[endIdx[i,j]]-timesData[startIdx[i,j]], endIdx[i,j]-startIdx[i,j])
                if endIdx[i,2]- startIdx[i,0] <> 146:
                    print "    WARNING: data point number incorrect!", endIdx[i,2] - startIdx[i,0]
                    errorFlag = errorFlag | 32
                if round(airMass(ElInt[startIdx[i,j]]),2) <> round(airMass(ElInt[endIdx[i,j]]),2):
                    print "    Airmass number not constant through all observation %3.1f to %3.1f" %\
                          (airMass(ElInt[startIdx[i,j]]),2), round(airMass(ElInt[endIdx[i,j]]),2)
                if stateID[startIdx[i,j]] <> stateID[endIdx[i,j]]:
                    print "    WARNING: STATE_ID changed from %d to %d" % (stateID[startIdx[i,j]], stateID[endIdx[i,j]])
                    errorFlag = errorFlag | 64

        Time0 = startTime[0]

        # plot setup
        plt.clf()
        fig1 = plt.figure(1)
        plt.xlabel("time")
        plt.ylabel("Total Power [Counts]")
        plot_title=("%s - SKYDIP %s - %s " % (antennaName, str(scan+1), self))
        plt.title(plot_title)
        plt.grid(True)
        plt.markerside=1

        #plot signal versus time
        plt.plot(timesData - Time0,totalPowerXX, '.')
        plt.plot(timesData - Time0,totalPowerYY, '.')

        plt.draw()
        
        # build times vector starting from the beginnning of the measurement
 
        # define arrays for BB temperature

        BBTemp         = np.zeros(shape=(nAm,3))

        # compute Hot, Cold and Sky average and std at each airmass
        for i in range(nAm):
            BBTemp[i,0]           = load[stateID[startIdx[i,0]]]
            BBTemp[i,1]           = load[stateID[startIdx[i,1]]]
            am[i]                 = airMass(ElInt[fieldStart[i]])

            for j in range(0,3): # cycle over number of measurements
                meanTimesData[i,j]= np.average(timesData[startIdx[i,j]:endIdx[i,j]])
                meanAmpXX[i,j]    = np.average(totalPowerXX[startIdx[i,j]:endIdx[i,j]])
                meanAmpYY[i,j]    = np.average(totalPowerYY[startIdx[i,j]:endIdx[i,j]])
                meanAmbTemp[i,j]  = np.average(ambTempInt[startIdx[i,j]:endIdx[i,j]])
                stdAmpXX[i,j]     = np.std(totalPowerXX[startIdx[i,j]:endIdx[i,j]])
                stdAmpYY[i,j]     = np.std(totalPowerYY[startIdx[i,j]:endIdx[i,j]])
                stdAmbTemp[i,j]   = np.std(ambTempInt[startIdx[i,j]:endIdx[i,j]])

                # debugging
                #print i, j, startIdx[i,j], endIdx[i,j], stdAmpXX[i,j], meanAmbTemp[i,j]-C2K

        BBTemp = BBTemp + C2K

        meanLoadAmpXX           = np.zeros(2)
        meanLoadAmpYY           = np.zeros(2)
        stdLoadAmpXX            = np.zeros(2)
        stdLoadAmpYY            = np.zeros(2)
 
        meanLoadAmpXX[0]        = np.average(meanAmpXX[:,0])
        meanLoadAmpYY[0]        = np.average(meanAmpYY[:,0])
        stdLoadAmpXX[0]         = np.std(meanAmpXX[:,0])
        stdLoadAmpYY[0]         = np.std(meanAmpYY[:,0])
        meanLoadAmpXX[1]        = np.average(meanAmpXX[:,1])
        meanLoadAmpYY[1]        = np.average(meanAmpYY[:,1])
        stdLoadAmpXX[1]         = np.std(meanAmpXX[:,1])
        stdLoadAmpYY[1]         = np.std(meanAmpYY[:,1])

        # compute average values of weather parameter
        meanRelHum              = np.average(relHum)
        meanPres                = np.average(pres)
        meanMeanAmbTemp         = np.average(meanAmbTemp)
        stdMeanAmbTemp          = np.average(stdAmbTemp)

        #
        # check all the values are reasonable
        # ***to be completed***
        #

        # check for zero values in Load temperatures
        goodTemp             = np.zeros(2)
        for j in range(2):
            for i in range(nAm):
                if BBTemp[i,j] == C2K:
                    print " WARNING: %s temp = 0.0 at airmass %d!" % (label[j], i)
                    errorFlag = errorFlag | 128
                else:
                    meanBBTemp[j]           += BBTemp[i,j]
                    goodTemp[j]             += 1.
            meanBBTemp[j]              = meanBBTemp[j]/goodTemp[j]

        #############################
        # data analysis starts here #
        #############################

        #
        # convert counts to power
        # 
        
        m           = 2.5/(2**16 - 1)
        mXX         = 1.64 
        bXX         = -0.096
        mYY         = 1.59
        bYY         = -0.0964

        print "\n XX conversion: V = %5.2f * counts %+6.3f" % (m*mXX, bXX)
        print " YY conversion: V = %5.2f * counts %+6.3f" % (m*mYY, bYY)
        
        meanPowerXX         = np.zeros(shape=(nAm,3))
        stdPowerXX          = np.zeros(shape=(nAm,3))
        meanPowerYY         = np.zeros(shape=(nAm,3))
        stdPowerYY          = np.zeros(shape=(nAm,3))
        
        meanPowerXX         = m * mXX * meanAmpXX  + bXX
        stdPowerXX          = abs(m * mXX * stdAmpXX   + bXX)
        meanPowerYY         = m * mYY * meanAmpYY  + bYY
        stdPowerYY          = abs(m * mYY * stdAmpYY   + bYY)

        # average values
        averagePowerXX      = np.zeros(2)
        averagePowerYY      = np.zeros(2)
        stdMeanPowerXX      = np.zeros(2)
        stdMeanPowerYY      = np.zeros(2)

        # compute average values of Hot and Cold load and ambient temperature
        averagePowerXX[0]   = np.average(meanPowerXX[:,0])
        averagePowerYY[0]   = np.average(meanPowerYY[:,0])
        stdMeanPowerXX[0]   = abs(np.std(meanPowerXX[:,0]))
        stdMeanPowerYY[0]   = abs(np.std(meanPowerYY[:,0]))
        averagePowerXX[1]   = np.average(meanPowerXX[:,1])
        averagePowerYY[1]   = np.average(meanPowerYY[:,1])
        stdMeanPowerXX[1]   = abs(np.std(meanPowerXX[:,1]))
        stdMeanPowerYY[1]   = abs(np.std(meanPowerYY[:,1]))

        # calc gain in W/K
        GainXX              = np.zeros(2)
        GainYY              = np.zeros(2)

        GainXX              = (averagePowerXX[1] - averagePowerXX[0])/(meanBBTemp[1] - meanBBTemp[0])
        GainYY              = (averagePowerYY[1] - averagePowerYY[0])/(meanBBTemp[1] - meanBBTemp[0])

        for j in range(0,3):
            # show values in Watts and counts
            print '\n %s in mW (count)' % label[j]
            
            print " am          <XX>                         <YY>                  Amb T  ",
            if j<2:
                print "BB Temp"
            else:
                print
            for i in range (nAm):
                print " %2d %6.1f +/- %4.1f mW (%5.0f)  %6.1f +/- %4.1f mW (%5.0f)  %3.1f +/- %3.1f" \
                      % (i+1, 1000*meanPowerXX[i,j], 1000*stdPowerXX[i,j],  meanAmpXX[i,j], \
                         1000*meanPowerYY[i,j], 1000*stdPowerYY[i,j],  meanAmpYY[i,j],\
                         meanAmbTemp[i,j] - C2K, stdAmbTemp[i,j]),
                if j<2:
                    print "  %4.1f" %  (BBTemp[i,j] - C2K)
                else:
                    print

            if j<2:
                print " ------------------------------- Average ----------------------------------------"
                # mean Load temperatures
                print  "    %6.1f +/- %4.1f mV (%5.0f)  %6.1f +/- %4.1f mV (%5.0f)  %3.1f +/- %3.1f"\
                      % (1000*averagePowerXX[j], 1000*stdMeanPowerXX[j], meanLoadAmpXX[j],\
                         1000*averagePowerYY[j], 1000*stdMeanPowerYY[j], meanLoadAmpYY[j],\
                         meanMeanAmbTemp - C2K, stdMeanAmbTemp),
                print "  %4.1f (%d good)" %  (meanBBTemp[j] - C2K, goodTemp[j])

        location = 'OSF'
        [tAtm, pwv0] = getTatm(meanMeanAmbTemp,meanPres,meanRelHum,location)

        # compute receiver temperature
        TrxXX = (averagePowerXX[1] * meanBBTemp[0] - averagePowerXX[0] * meanBBTemp[1])/(averagePowerXX[0] - averagePowerXX[1])
        TrxYY = (averagePowerYY[1] * meanBBTemp[0] - averagePowerYY[0] * meanBBTemp[1])/(averagePowerYY[0] - averagePowerYY[1])

        # Sky temperature
        TskyXX           = np.zeros(nAm)
        TskyYY           = np.zeros(nAm)
        print "\n SKY TEMPERATURE"
        print " #    am  XX [K]  YY [k]"
        for i in range(nAm):
            TskyXX[i] = meanBBTemp[0] - (meanPowerXX[i,0] - meanPowerXX[i,2]) / GainXX
            TskyYY[i] = meanBBTemp[0] - (meanPowerYY[i,0] - meanPowerYY[i,2]) / GainYY

            print " %2d  %5.3f %6.1f  %6.1f" % (i+1, am[i], TskyXX[i], TskyYY[i])

        #
        # fit the skydip model
        #

        # guess fit


        (etaFXX,tau0XX) = CalcEtaTau (freq, jVal(freq, meanMeanAmbTemp), jVal(freq, tCMB), jVal(freq, tAtm), am, TskyXX)
        (etaFYY,tau0YY) = CalcEtaTau (freq, jVal(freq, meanMeanAmbTemp), jVal(freq, tCMB), jVal(freq, tAtm), am, TskyYY)

        # show the results
        print "\n Location                     : %s" % location
        print " Mean Ambient Temperature     : %5.1f +/- %3.1f C" % (meanMeanAmbTemp - C2K, stdMeanAmbTemp)
        print " Mean Ambient Pressure        : %5.1f mBar" % meanPres
        print " Mean Relative Humidity       : %5.1f %%" % meanRelHum
        print " Mean Atmospheric temperature : %5.1f K" % tAtm

        print "\n Polarization                    XX           YY"
        print " Gains                        : %6.1e     %6.1e [W/K]" % (GainXX, GainYY)
        print " Trx                          : %6.1f       %6.1f [K]" % (TrxXX, TrxYY)
        print " Forward Efficiency           : %6.2f       %6.2f [%%]" % (etaFXX*100., etaFYY*100.)
        print " Sky Opacity                  : %6.1f       %6.1f [%%]" % (tau0XX*100., tau0YY*100.)
        print " Model Zenith PWV             :        %6.1f       [%s]" % (pwv0.value, pwv0.units)

        # output results to a csv file

        rowCSV     = [fmtDT(Time0), self, scan, antennaName, antID, band, freq, sourceName,\
                      GainXX, GainYY, TrxXX, TrxYY, BBTemp[0][0], BBTemp[1][1],\
                      etaFXX, etaFYY, tau0XX, tau0YY, 999., 999., 999., 999., meanMeanAmbTemp, meanPres, meanRelHum, tAtm, pwv0.value, errorFlag]
    
        Log2CSV(self, scan, antennaName, fileCSV, headerCSV, rowCSV)

        # skydip plot setup
        fig2 = plt.figure(2)
        plt.xlabel("airmass")
        plt.ylabel("Sky Temperature [K]")
        plot_title=("%s - SKYDIP %02d - %s " % (antennaName, scan+1, os.path.basename(self)))
        plt.title(plot_title)
        plt.grid(True)
        plt.markerside=1

        # build times vector starting from the beginnning of the measurement
        # timesData_0 = [x - Time0 for x in timesData]
        # meanTimesData_0 = [x - Time0 for x in meanTimesData]

        xmin    = am[0]
        xmax    = am[nAm-1]
        nplot   = (xmax-xmin)*10
        ymin    = 10
        ymax    = 140
        
        #x       = linspace(x,min,xmax,nplot)
        plt.plot(am, TskyXX, 'ro', \
                 am, fitfunc([etaFXX,tau0XX],am,jVal(freq, meanMeanAmbTemp),jVal(freq, tAtm),jVal(freq, tCMB)), 'r-',\
                 am, TskyYY, 'gs',
                 am, fitfunc([etaFYY,tau0YY],am,jVal(freq, meanMeanAmbTemp),jVal(freq, tAtm),jVal(freq, tCMB)), 'g-')

        plt.legend(('XX data', 'XX fit', 'YY data', 'YY fit'), 'upper left', shadow=True, fancybox=True) 

        leg         = plt.gca().get_legend()
        ltext       = leg.get_texts()
        llines      = leg.get_lines()
        frame       = leg.get_frame()

        plt.ylim((ymin,ymax))

        plt.draw()
        
        # draw the legend
        frame.set_facecolor('0.80')
        plt.setp(ltext, fontsize='small')
        plt.setp(llines, linewidth=1.5)
        
        #save plot
        imageName = '%s_%02d_%s_skydip.png' % (antennaName, scan+1, os.path.basename(self[2:]))
        fig2.savefig(imageName)
        print ' %s image saved' % imageName 

    return

##############
# beam mapping: calibration and aperture efficiencies
#############

def beamPeak (self,antennaName):
    
    print "\n **** BEAM MAPPING ****"
    obsModeStr     = 'TARGET'    # identify the scan numbers for the SKYDIP procedure
    scanNumber     = findScan(self,obsModeStr)

    # find the antenna ID
    antID          = getAntennaIndex(self,antennaName)
    sourceName     = getTarget(self)
    freq           = getRefFrequency(self)
    band           = getBand(self)

    # Show some information about the current MS
    print "\nMeasurement Set     : %s" % self
    print "Antenna             : %s (index = %d)" % (antennaName, antID)
    print "Source              : %s" % sourceName
    print "Band                : %s" % band
    print "Reference Frequency : %4.1f GHz" % freq
    print "Number of beam mapping : %d\n" % len(scanNumber)

    for scan in range(len(scanNumber)):

        # find the antenna ID
        antID          = getAntennaIndex(self,antennaName)
        sourceName     = getTarget(self)
        freq           = getRefFrequency(self)
        band           = getBand(self)

        print "\nScan Number = ", str(scanNumber[scan])
                                       
        # read the relevant columns from the data table
        tb.open(self)
        tb_query = ("SCAN_NUMBER==%s && ANTENNA1=%s && ANTENNA2=%s" % (str(scanNumber[scan]), str(antID), str(antID)))
        subt=tb.query(tb_query, columns="TIME, SCAN_NUMBER, STATE_ID, FLOAT_DATA, FIELD_ID")
        tb.close()

        # create lists of relevant data
        timesData         = subt.getcol('TIME')
        print "len(timesData) = %d ", len(timesData)
        totalPower        = subt.getcol('FLOAT_DATA')
        fieldID           = subt.getcol('FIELD_ID')
        stateID           = subt.getcol('STATE_ID')

        # create arrays for the 2 different polarization
        # we may want to convert this to a single array later...
        totalPowerXX      = totalPower[0][0]
        totalPowerYY      = totalPower[1][0]

        # build a list with the first and last index for each new FIELD_ID
        # this fields correspond to each airmass value 
        nAm             = max(fieldID) # number of different fields (corresponding to airmass)
        fieldStart      = np.zeros(nAm)
        fieldEnd        = np.zeros(nAm)
        startTime       = np.zeros(nAm)
        endTime         = np.zeros(nAm)
        
        # read the relevant columns from the pointing table                                                  
        tb_query = ("ANTENNA_ID=%s" % str(antID))
        
        tb.open(self+'/POINTING')
        subt=tb.query(tb_query, columns="TIME, DIRECTION")
        timesAzEl          =(subt.getcol('TIME'))           
        Az                 =(subt.getcol('DIRECTION')[0])[0]
        El                 =(subt.getcol('DIRECTION')[1])[0]
        tb.close()

        # create interpolated time pointing to correspond to amplitude data
        # and compute airmasses
        nTimesData      = len(timesData)
        AzInt           = np.zeros(nTimesData)
        ElInt           = np.zeros(nTimesData)

        maxDT           = 0.1; # max acceptable delta time in sec.
        for i in range(nTimesData):
            idx = find_nearest(timesAzEl,timesData[i]) # find element of the pointing time array closest to data time array
            if (timesData[i]-timesAzEl[idx]) > maxDT:
                print "WARNING: time difference at index %d = %4.3f sec - data time = %s - pointing time = %s" \
                      % (i, timesData[i]-timesAzEl[idx], fmtHMS(timesData[i]), fmtHMS(timesAzEl[idx]))
                errorFlag = errorFlag | 256
            AzInt[i]    = Az[idx]
            ElInt[i]    = El[idx]
            
        maxDam = 0.1 # max acceptable delta airmass

        # read the relevant columns from the pointing table                                                  
        tb_query = ("ANTENNA_ID=%s" % str(antID))
        
        # read the relevant columns from the WEATHER table
        tb.open(self+'/WEATHER')
        subWeather         = tb.query("", columns="TIME, TEMPERATURE, REL_HUMIDITY, PRESSURE")
        timesWeather       = subWeather.getcol('TIME')           
        ambTemp            = subWeather.getcol('TEMPERATURE')
        relHum             = subWeather.getcol('REL_HUMIDITY')
        pres               = subWeather.getcol('PRESSURE')
        
        tb.close()

        # print relevant values from the pointing values
        print "\n DETECTOR from %s to %s" % (fmtHMS(timesData[0]), fmtHMS(timesData[len(timesData)-1]))
        print " POINTING from %s to %s" % (fmtHMS(timesAzEl[0]), fmtHMS(timesAzEl[len(timesAzEl)-1])),
        if (timesData[0]<timesAzEl[0] or timesData[len(timesData)-1]>timesAzEl[len(timesAzEl)-1]):
            print "- WARNING: Not enough data in the pointing table!"
            errorFlag = errorFlag | 512
        else:
            print "- enough pointing data"
        
        print " WEATHER  from %s to %s" % (fmtHMS(timesWeather[0]), fmtHMS(timesWeather[len(timesWeather)-1])),
        if (timesData[0]<timesWeather[0] or timesData[len(timesData)-1]>timesWeather[len(timesWeather)-1]):
            print "- WARNING: Not enough data in the weather table!"
            errorFlag = errorFlag | 1024
        else:
            print "- enough weather data"

        totalPowerXXmax = []
        totalPowerYYmax = []
        
        # search for the maximum
        for i in range(len(totalPowerXX)):
            totalPowerXXmax.append(real(totalPowerXX[i]).max())
            totalPowerYYmax.append(real(totalPowerYY[i]).max())

    return

def Log2CSV(ms, scan, ant, fname, header, newRow):
    # output results to a csv file
    if os.path.isfile(fname):
        logWriter = csv.writer(open(fname, 'a'), delimiter=',', quotechar="'")
        #print newRow
    else:
        logWriter = csv.writer(open(fname, 'wa'), delimiter=',', quotechar="'")
        logWriter.writerow(header)
        print " file %s created" % fname

    logWriter.writerow(newRow)
    print "\n Results logged in file %s" % fname
    return

def getAntennaName(self):
    tb.open(self+'/ANTENNA')
    antennaTb=tb.toolcreate()
    antennaTb.open(self.vis+'/ANTENNA')
    self.antennaName=antennaTb.getcol('NAME')[self.antennaId]
    antennaTb,close()

def importAsdmList(asdmList=None,makeClean=False):
    if asdmList <> None:
        asdmList=aU.makeList(asdmList)
    else:
        asdmList=getAsdmList()
    for i in asdmList :
        msFile = i.replace('/','')+'.ms'
        if makeClean : os.system("rm -rf %s*" % msFile)
        importasdm(asdm=1, vis=msFile)

def getAsdmList():
    os.system("rm -rf junk")
    os.system("ls -d uid___* > junk")
    table = fiop.fileToTable("junk")
    asdmList = []
    for i in table :
        if "ms" in i[0] : continue
    else:
        asdmList.append(i[0])
    return asdmList

def getMSList():
    os.system("rm -rf junk")
    os.system("ls -d uid___* > junk")
    table = fiop.fileToTable("junk")
    msList = []
    for i in table:
        if ".ms" == i[0][-3:] : msList.append(i[0])
        else: continue
    for i in msList:
        print i
    print "-> Measurement Sets found: %d" % len(msList)
#    return msList

def getAntennaIndex(self,antennaName):
    tb.open(self+'/ANTENNA')
    names=tb.getcol('NAME')
    tb.close()
    j=0
    k=-1
    for i in names:
#        print "antenna %d: %s" % (j, i)
        if antennaName in i:
            k=j
            break
        j=j+1
    return k

def getTarget(self):
    sourceName = list(getSingleCol(self, 'SOURCE', 'NAME'))
    while 'Unknown' in sourceName:
        sourceName.remove('Unknown')
    return sourceName[0]

def getBand(self):
    frequency = list(getSingleCol(self, 'SPECTRAL_WINDOW', 'CHAN_FREQ'))
    band = -1
    while 'Unknown' in frequency:
        frequency.remove('Unknown')
    if (frequency[0] > 84e9) and (frequency[0] < 116e9) :
        band = '3'
    elif (frequency[0] > 125e9) and (frequency[0] < 163) :
        band = '4'
    elif (frequency[0] > 211e9) and (frequency[0] < 275e9) :
        band = '6'
    elif (frequency[0] > 275e9) and (frequency[0] < 373e9) :
        band = '7'
    elif (frequency[0] > 602e9) and (frequency[0] < 720e9) :
        band = '9'
    else:
        print "unknown frequency: %6.2f Hz"% (frequency[0]/1000)
    return band

# Return a single column read from a table
# self    = name of the MS directory
# tab     = name of the table
# colName = name of the column to be red
def getSingleCol(self, tab, colName):
    s=self
    if tab <> '':
        s = self+'/'+tab
    else:
        s = self
#    print "tabname = ", s
    tb.open(s)
    col = tb.getcol(colName)
    tb.close()
    return col

# Return multiple columns from a table

def getMultipleCol(self, tab, colNames):
    s=self
    if tab <> '':
        s = self+'/'+tab
    else:
        s = self
#    print "tabname = ", s
    tb.query(s)

    return colNames

# search in the MS if a certain observing mode is available
# vis     = a string with the MS
# obsMode = a string to search for, ex. POINTING or CALIBRATION
def findScan(self,obsMode):
    #load all the observing modes available in the MS
    stateID=getSingleCol(self,'STATE','OBS_MODE')
    out=[]
    #print the sequence of Obs modes available
    for i in range(0,stateID.__len__()):
#        print i, "stateID: ", stateid[i]
        if(stateID[i].find(obsMode)>0):
            out.append(i)
    # And now find the corresponding Scan Numbers in the MS
    tb.open(self)
    pointingScans=[]
    for i in range(0,out.__len__()):
        myq=tb.query('STATE_ID==%d' % out[i])
        scans=myq.getcol('SCAN_NUMBER')
        for item in scans:
            if item not in pointingScans: pointingScans.append(item)
    tb.close()
    return pointingScans

# compute the airmass number for a certain elevation
# x   =   elevation in rad
def airMass(x):

    # here we consider a not refracting atmosphere, 
    Re = 6371000 # Earth radius in meters
    T0 = 288.15 # atmospheric tempearture in K
    mAtm = 28.9644*1.665E-24 # mas of 
    kB = 1.3806503E-23 # Boltzmann constant in m2 kg s-2 K-1
    g = 9.80665 # gravity acceleration in m/s^2
    yatm = kB*T0/(mAtm*g) # atmospheric height
    # return 

    # airmass[i] = 1.0/(m.sin(elevFit[i])+0.50572*(96.07995-(90-elevFit[i]*180.0/3.14159))**-1.6364)
    return 1.0/(m.sin(x))

# format an ALMA time in a standard date-time
def fmtDT(t):
    timeOffset = -3506706000.0
    return dt.datetime.fromtimestamp(t+timeOffset)

#stop the program and starts the interactive console
def stopHere():
    import code; code.interact(local=locals())

# find the index of the nearest value in an array
def find_nearest(array,value):
    idx=(np.abs(array-value)).argmin()
    return idx

# format an ALMA time to a HMS format
def fmtHMS(t):
    timeOffset = -3506706000.0
    return time.strftime("%H:%M:%S", \
                         dt.datetime.fromtimestamp(t+timeOffset).timetuple())

def oneMoment(t):
    time.sleep(t)

def getTatm(toutAll,patmAll,hum,location='OSF') : 
    if location == 'OSF' : alt = casac.Quantity(2930.0,'m')
    elif location == 'AOS' : alt = casac.Quantity(5050.0,'m')
    tmp     = casac.Quantity(toutAll,'K')
    pre     = casac.Quantity(patmAll,'mbar')
    maxA    = casac.Quantity(48.0,'km')
    myatm   = at.initAtmProfile(alt,tmp,pre,maxA,hum)
    tempPro = at.getProfile()['temperature']
    tAtm    = sum(tempPro.value)/len(tempPro.value)
    pwv0    = at.getGroundWH2O()
    #tEbb    = at.getAverageTebbSky(0,pwv0).value, at.getAverageTebbSky(0,pwv0).units
    #print tEbb
    return tAtm, pwv0

# convert temperatures
def jVal(f,t) :
    x = h*f*1e9/k
    return x*(m.exp(x/t)-1)**(-1)

def djVal(f,t,dt) :
    x = h*f*1e9/k
    return abs(x*(m.exp(x/t)-1)**(-2)*(x*dt/t**2.0)*m.exp(x/t))

def CalcEtaTau (f, tOut, tCMB, tAtm, airMass, Tsky):

    # using RJ values
    residuals = lambda p,x, y : fitfunc(p,x,tOut,tAtm,tCMB) - y 

    p0 = [0.9,0.15] # initial guess on etaF and tau0

    p1, success     = leastsq(residuals, p0, args = (airMass, Tsky), xtol=0.01, ftol=0.01, maxfev=10000)
    #chi2           = sum(power(residuals(),2)
    
    if not success:
        print "No least square solution found!"
        
    # results of the fit: forward efficiency and zenith sky opacity 
    etaF    = p1[0]
    tau0    = p1[1]

    return etaF, tau0

def getRefFrequency(self) :
    tb.open(self + '/SPECTRAL_WINDOW')
    freqs = tb.getcol('REF_FREQUENCY')
    tb.close()
    return (freqs[0]/1e9)

def calcEfficiencies(msFile,elev,Psky,Psource,Trx,Tsky,tau,Tsource,polarization=0,sourceSize=0,sourceLarge=False,etaF=0.99,beamSize=None,diameter=12.0e2) :
    freq       = getRefFrequency(msFile)
    wavelength = c/(freq*1e9)
    if beamSize == None : beamSize = 1.15*(wavelength/diameter)*206265
    print beamSize
    Tcmb_      = jVal(freq,Tcmb)
    Tsource_   = jVal(freq,Tsource)
    Tuse_      = correctPlanet(sourceSize,Tsource_,beamSize)
    airmass    = 1.0/(m.sin(elev*3.14159/180.0)+0.50572*(96.07995-(90-elev))**-1.6364)
    if not sourceLarge :
        print polarization
        Psource_ = normalization*Psource*biasCounts[polarization]['slope']+biasCounts[polarization]['intercept']
        etaA = ((((Psource_)/Psky)-1)*(Trx+Tsky)*m.exp(airmass*tau))/Tuse_
        jyPerK = 8*k/(etaA*math.pi*diameter**2.0)/jy2cgs
        return [etaA,jyPerK]
    else :
        Psource_ = normalization*Psource*biasCounts[polarization]['slope']+biasCounts[polarization]['intercept']
        etaB = (((Psource_/Psky)-1)*(Trx+Tsky)*m.exp(airmass*tau)+etaF*Tcmb_)/Tuse_
        return [etaB,None]

def getScanInfo (self, antennaName):
    # find the antenna ID
    antID          = getAntennaIndex(self,antennaName)
    sourceName     = getTarget(self)
    freq           = getRefFrequency(self)
    band           = getBand(self)

    return antID, sourceName, freq, band

def getMSList (dir):
    filelist = glob.glob( os.path.join(dir, 'uid___*.ms'))
    return filelist

# check answer to questions yes/no
def confirm(prompt=None, resp=False):

    #    initialize
    
    if prompt is None:
        prompt = 'remove file = %s?' %fn

    if resp:
        prompt = '%s [%s]|%s: ' % (prompt, 'y', 'n')
    else:
        prompt = '%s [%s]|%s: ' % (prompt, 'y', 'n')

    while True:
        ans = raw_input(prompt)
        if not ans:
            return resp
        if ans not in ['y', 'Y', 'n', 'N']:
            print 'please enter y or n'
            continue
        if ans == 'y' or ans == 'Y':
            return True
        if ans == 'n' or ans == 'N':
            return False

# check a webpage (for example a journal) for available skydip on a given antenna
def checkJournal(antName, webpage):

    return

# atmospheric opacity model
# used for etaF and tau fitting
fitfunc = lambda p, x, tOut, tAtm, tCMB : (1-p[0])*tOut + p[0]*(1.0-exp(-p[1]*x))*tAtm + p[0] * tCMB * exp(-p[1]*x)
if __name__ == "__main__":

    #
    # Parser setup
    #
    
    from optparse import OptionParser

    parser = OptionParser(usage = "usage: %prog [options]", version = "%prog 1.0")

    # Parse & Transform data types
    parser.add_option("-a", "--antenna",
                      dest="antennaName",
                      default=None,
                      help="the Antenna to be calibrated (e.g. 'DV01')")

    parser.add_option("-d", "--dir",
                      dest="folder",
                      default=None,
                      help="Select the dir (e.g. '.')")

    parser.add_option("-q", "--quiet",
                      action="store_false",
                      dest="verbose",
                      default=True,
                      help="don't print too many  messages to stdout")
    
    (options, args) = parser.parse_args()

    print options
    print args

    sys.stdout=sys.stderr      # Prevent crashes upon print

    reduceAllAmpCal (options.folder, options.antennaName)
