#!/usr/bin/env python

"""
Focus Curves fitting  script
   usage: aivFocusFitting_7.1.1.py -f FILENAME -a ANTENNA NAME -b BAND  (see help for more options)

   f is for the focus data file
   a is for Antenna name
   b is for the frequency band (0, 3, 6, 7, and 9, defaulted to 0). 
   b = 0 is for the joint fit to gravitational, thermal changes and band offsets with R7.1 definition of focus parameters(Currently 3 bands: 3,6,7)).

   2010-11-17 sasayama


$Id: aivFocusFitting_7.1.1.py,v 1.1 2010/11/17 12:54:25 sasayama Exp $
"""





import scipy as sp
from scipy import optimize
from numpy import *
from optparse import OptionParser
import pylab as pl

from numpy import *
from scipy import *
from scipy.fftpack import *
from math import *
from pylab import *
params = {'legend.fontsize': 10}
rcParams.update(params)



# Initialize parser
parser = OptionParser()

parser.add_option("-f", "--files", dest="files",
              help="Input files to be analysed (separated by white space)")
parser.add_option("-a", "--ant", dest="ant",
              help="Input Antenna name to be analysed (separated by white space)")
parser.add_option("-b", "--band", dest="band",
                  choices=['0','3','6','7','9'],
                  default = '0',
                  help="Select band, ie 0 ,3, 6, 7, 9 (defaulted to 0). " +\
		  "b = 0 is for the joint fit to gravitational, thermal changes and " +\
		  "band offsets with R7.1 definition of focus parameters(Currently 3 bands: 3,6,7)")


(options, args) = parser.parse_args()

files = options.files
ant = options.ant
band = options.band


keys = ['03X', '03Y', '03Z', '06X', '06Y', '06Z', '07X', '07Y', '07Z', '09X', '09Y', '09Z']
Axes = ['X','Y','Z']
RBs  = ['03','06','07','09']
data = {}

for key in keys:
    data[key] = {'Date': [], 'T': [], 'Focus_offset': [], 'Error': [], 'Az': [], 'El': [], 'absolute_Focus': [],'flag3': [],'flag6': [],'flag7': []}


for axis in Axes:
    data[axis] =  {'El': [], 'flag3': [],'flag6': [],'flag7': [],'T': [], 'absolute_Focus': [],'Error': []}

if  band == '0':
    for line in open(files):
        fields = line.split()
        if len(fields) == 0 or line[0] == '#':
            continue
        for axis in Axes:
	    for RB in RBs:
	        Ax = "%s%s" % (RB,axis)
	        alma_rb = "ALMA_RB_%s" % RB
                if fields[2] == ant and fields[3] == axis and fields[4] == alma_rb and float(fields[6]) < 0.3:
                    data[Ax]['Date'].append(fields[0])
                    data[Ax]['T'].append(fields[1])
                    data[Ax]['Focus_offset'].append(fields[5])
                    data[Ax]['Error'].append(fields[6])
                    data[Ax]['Az'].append(fields[7])
                    data[Ax]['El'].append(fields[8])
                    data[Ax]['absolute_Focus'].append(fields[9])
		    if RB == '03':
                        data[Ax]['flag3'].append('1')
                        data[Ax]['flag6'].append('0')
                        data[Ax]['flag7'].append('0')
		    elif RB == '06':
                        data[Ax]['flag3'].append('0')
                        data[Ax]['flag6'].append('1')
                        data[Ax]['flag7'].append('0')
		    elif RB == '07':
                        data[Ax]['flag3'].append('0')
                        data[Ax]['flag6'].append('0')
                        data[Ax]['flag7'].append('1')
		
    for axis in Axes:
        b03_axis = "03%s" % axis
        b06_axis = "06%s" % axis
        b07_axis = "07%s" % axis
        data[axis]['El']=array([float(val) for val in (data[b03_axis]['El']+data[b06_axis]['El']+data[b07_axis]['El'])])
        data[axis]['flag3']=array([float(val) for val in (data[b03_axis]['flag3']+data[b06_axis]['flag3']+data[b07_axis]['flag3'])])
        data[axis]['flag6']=array([float(val) for val in (data[b03_axis]['flag6']+data[b06_axis]['flag6']+data[b07_axis]['flag6'])])
        data[axis]['flag7']=array([float(val) for val in (data[b03_axis]['flag7']+data[b06_axis]['flag7']+data[b07_axis]['flag7'])])
        data[axis]['T']=array([(float(val[:-1])-273) for val in (data[b03_axis]['T']+data[b06_axis]['T']+data[b07_axis]['T'])])
        data[axis]['absolute_Focus']=array([float(val) for val in (data[b03_axis]['absolute_Focus']+data[b06_axis]['absolute_Focus']+data[b07_axis]['absolute_Focus'])])
        data[axis]['Error']=array([float(val) for val in (data[b03_axis]['Error']+data[b06_axis]['Error']+data[b07_axis]['Error'])])


    ######## fitting Y for the R7.1 definition of focus parameters
    def Y(Yp, x, s, t, u):
        Y0b7, Y0b7_3, Y0b7_6, YC = Yp
        return ( Y0b7_3 + YC*cos(x/180*pi))*s + (Y0b7_6 + YC*cos(x/180*pi))*t + Y0b7 + YC*cos(x/180*pi)*u

    def residueY(Yp, y, x, s, t, u, sigma):
        err = (y - Y(Yp,x,s,t,u)) / sigma
        return err

    p_guess = (3.5, 0.0, 0.0, -4.0)
    N = len (data['Y']['El'])
    x = data['Y']['El']
    s = data['Y']['flag3']
    t = data['Y']['flag6']
    u = data['Y']['flag7']
    y = data['Y']['absolute_Focus']
    sigma = sqrt(data['Y']['Error']**2)

    result = optimize.leastsq(residueY, p_guess, args=(y, x, s, t, u, sigma), full_output=1)

    yp = result[0]
    c = result[1]
    err_y = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueY(yp, y, x, s, t, u, sigma)**2 ) / (N - 4)
    Yp = ["Y0b7", "Y0b7_3", "Y0b7_6", "YC"]

    print "\n*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(4):
        print "%s = %+3.5f +/- %3.5f " % (Yp[i], yp[i], err_y[i])

    ######## fitting Z for the R7.1 definition of focus parameters @ OSF
    def Z(Zp, x, s, t, u, v):
        Z0b7, Z0b7_3, Z0b7_6, ZS, ZT = Zp
        return (Z0b7_3 + ZS*sin(x/180*pi) + ZT*(v-10))*s + (Z0b7_6 + ZS*sin(x/180*pi) + ZT*(v-10))*t + Z0b7 + (ZS*sin(x/180*pi) + ZT*(v-10))*u

    def residueZ(Zp, z, x, s, t, u, v, sigma):
        err = (z - Z(Zp,x,s,t,u,v)) / sigma
        return err

    p_guess = (-3, 0.0, 0.0, 2.0, 0.1)
    N = len (data['Z']['El'])
    x = data['Z']['El']
    s = data['Z']['flag3']
    t = data['Z']['flag6']
    u = data['Z']['flag7']
    v = data['Z']['T']
    z = data['Z']['absolute_Focus']
    sigma = sqrt(data['Z']['Error']**2)

    result = optimize.leastsq(residueZ, p_guess, args=(z, x, s, t, u, v, sigma), full_output=1)

    zp = result[0]
    c = result[1]
    err_z = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueZ(zp, z, x, s, t, u, v, sigma)**2 ) / (N - 5)
    Zp = ["Z0b7", "Z0b7_3", "Z0b7_6", "ZS", "ZT"]

    print "\n*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(5):
        print "%s = %+3.5f +/- %3.5f " % (Zp[i], zp[i], err_z[i])


    ######## fitting X for the R7.1 definition of focus parameters @ OSF    
    def X(Xp, x, s, t):
        X0b7, X0b7_3, X0b7_6 = Xp
        return X0b7_3*s + X0b7_6*t + X0b7

    def residueX(Xp, a, x, s, t, sigma):
        err = (a - X(Xp,x,s,t)) / sigma
        return err


    p_guess = (0.0, 0.0, 0.0)
    N = len (data['X']['El'])
    x = data['X']['El']
    s = data['X']['flag3']
    t = data['X']['flag6']
    a = data['X']['absolute_Focus']
    sigma = sqrt(data['X']['Error']**2)

    result = optimize.leastsq(residueX, p_guess, args=(a, x, s, t, sigma), full_output=1)

    xp = result[0]
    c = result[1]
    err_x = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueX(xp, a, x, s, t, sigma)**2 ) / (N - 3)
    Xp = ["X0b7", "X0b7_3", "X0b7_6"]

    print "\n*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(3):
        print "%s = %+3.5f +/- %3.5f " % (Xp[i], xp[i], err_x[i])


    ######## fitting Z for the R7.1 definition of focus parameters @ AOS
    def Z(Zp, x, s, t, u, v):
        Z0b7, Z0b7_3, Z0b7_6, ZS, ZT = Zp
        return (Z0b7_3 + ZS*sin(x/180*pi) + ZT*v)*s + (Z0b7_6 + ZS*sin(x/180*pi) + ZT*v)*t + Z0b7 + (ZS*sin(x/180*pi) + ZT*v)*u

    def residueZ(Zp, z, x, s, t, u, v, sigma):
        err = (z - Z(Zp,x,s,t,u,v)) / sigma
        return err

    p_guess = (-3, 0.0, 0.0, 2.0, 0.1)
    N = len (data['Z']['El'])
    x = data['Z']['El']
    s = data['Z']['flag3']
    t = data['Z']['flag6']
    u = data['Z']['flag7']
    v = data['Z']['T']
    z = data['Z']['absolute_Focus']
    sigma = sqrt(data['Z']['Error']**2)

    result = optimize.leastsq(residueZ, p_guess, args=(z, x, s, t, u, v, sigma), full_output=1)

    zp2 = result[0]
    c2 = result[1]
    err_z2 = sp.sqrt( sp.diag(c2) )
    chisq = sp.sum( residueZ(zp2, z, x, s, t, u, v, sigma)**2 ) / (N - 5)
    Zp2 = ["Z0b7 (AOS)", "Z0b7_3 (AOS)", "Z0b7_6 (AOS)", "ZS (AOS)", "ZT(AOS)"]

    print "\n*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(5):
        print "%s = %+3.5f +/- %3.5f " % (Zp2[i], zp2[i], err_z2[i])

    ### For JIRA 
    print "\n ##### For JIRA comments: #####\n" 
    print "The fit result of YC, ZS and ZT: \n"
    print "YC = %+3.5f +/- %3.5f " % (yp[3], err_y[3])
    print "ZS = %+3.5f +/- %3.5f " % (zp[3], err_z[3])
    print "ZT = %+3.5f +/- %3.5f " % (zp[4], err_z[4])


    print "\nThe fit result for offsets:\n" 
    print "X0b7   = %+3.5f +/- %3.5f " % (xp[0], err_x[0])
    print "X0b7_6 = %+3.5f +/- %3.5f " % (xp[2], err_x[2])
    print "X0b7_3 = %+3.5f +/- %3.5f \n" % (xp[1], err_x[1])
    
    print "Y0b7   = %+3.5f +/- %3.5f " % (yp[0], err_y[0])
    print "Y0b7_6 = %+3.5f +/- %3.5f " % (yp[2], err_y[2])
    print "Y0b7_3 = %+3.5f +/- %3.5f \n" % (yp[1], err_y[1])

    print "Z0b7   = %+3.5f +/- %3.5f " % (zp[0], err_z[0])
    print "Z0b7_6 = %+3.5f +/- %3.5f " % (zp[2], err_z[2])
    print "Z0b7_3 = %+3.5f +/- %3.5f \n" % (zp[1], err_z[1])








    ### Print the best estimate of the focus positions at 50 degrees elevation 
    print "\n\nThe best estimate of the focus positions at 50 degrees elevation at T= 10  deg C are:\n" +\
    "Band 7:" 
    B7Y50 = yp[0]+yp[3]*cos(50.0/180.0*pi)
    B7Z50 = zp[0]+zp[3]*sin(50.0/180.0*pi)
    print "X7_50 = %+3.3f" % xp[0] 
    print "Y7_50 = %+3.3f %+3.3f*cos(50.0/180.0*pi) = %+3.3f" % (yp[0], yp[3],B7Y50)
    print "Z7_50 = %+3.3f %+3.3f*sin(50.0/180.0*pi) = %+3.3f" % (zp[0], zp[3],B7Z50)

    print "\nBand 6:"
    print "X6_50 = %+3.3f + X7_50 = %+3.3f" % (xp[2], xp[2]+xp[0])
    print "Y6_50 = %+3.3f + Y7_50 = %+3.3f" % (yp[2], yp[2]+B7Y50)
    print "Z6_50 = %+3.3f + Z7_50 = %+3.3f" % (zp[2], zp[2]+B7Z50)

    print "\nBand 3:"
    print "X3_50 = %+3.3f + X7_50 = %+3.3f" % (xp[1], xp[1]+xp[0])
    print "Y3_50 = %+3.3f + Y7_50 = %+3.3f" % (yp[1], yp[1]+B7Y50)
    print "Z3_50 = %+3.3f + Z7_50 = %+3.3f" % (zp[1], zp[1]+B7Z50)
    
    
    ### Print Focus models for OSF
    print "\nFocus model to be installed in \"%s_focusCurves.txt\" at OSF (T_ref=10 deg C). :"  % ant
    print "#start, stop, X0_b3, Y0_b3, Z0_b3, X0_b6, Y0_b6, Z0_b6, X0_b7, Y0_b7, Z0_b7, Z_sin, Y_cos, Z_temp"
    print "\"start\", \"stop\", %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, 0.000"  % (xp[1]+xp[0]-0.134,yp[1]+B7Y50-0.041,zp[1]+B7Z50-0.092,xp[2]+xp[0]+0.113,yp[2]+B7Y50-0.076,zp[2]+B7Z50+0.131,xp[0],B7Y50,B7Z50,zp[3],yp[3])

    ### Print Focus models for AOS
    print "\n\nFocus model to be installed in \"%s_focusCurves.txt\" at AOS (T_ref=0 deg C). :" % ant
    B7Z50aos = zp2[0]+zp2[3]*sin(50.0/180.0*pi)
    print "\"start\", \"stop\", %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, %+3.3f, 0.000"  % (xp[1]+xp[0]-0.134,yp[1]+B7Y50-0.041,zp2[1]+B7Z50aos-0.092,xp[2]+xp[0]+0.113,yp[2]+B7Y50-0.076,zp2[2]+B7Z50aos+0.131,xp[0],B7Y50,B7Z50aos,zp2[3],yp[3])




    ######## Plots focus curves 
    x=arange(10,90,0.1)
    pl.clf()
    pl.plot(data['03X']['El'],data['03X']['absolute_Focus'], 'gs')
    pl.plot(data['03Y']['El'],data['03Y']['absolute_Focus'], 'g^')
    pl.plot(data['03Z']['El'],data['03Z']['absolute_Focus'], 'go')
    pl.plot(data['06X']['El'],data['06X']['absolute_Focus'], 'bs')
    pl.plot(data['06Y']['El'],data['06Y']['absolute_Focus'], 'b^')
    pl.plot(data['06Z']['El'],data['06Z']['absolute_Focus'], 'bo')
    pl.plot(data['07X']['El'],data['07X']['absolute_Focus'], 'rs')
    pl.plot(data['07Y']['El'],data['07Y']['absolute_Focus'], 'r^')
    pl.plot(data['07Z']['El'],data['07Z']['absolute_Focus'], 'ro')
    pl.plot([10,90],[(xp[0]+xp[1]),(xp[0]+xp[1])],color='g')
    pl.plot([10,90],[(xp[0]+xp[2]),(xp[0]+xp[2])],color='b')
    pl.plot([10,90],[xp[0],xp[0]],color='r')
    pl.plot(x,yp[0]+yp[1]+yp[3]*cos(x/180*pi),color='g')
    pl.plot(x,yp[0]+yp[2]+yp[3]*cos(x/180*pi),color='b')
    pl.plot(x,yp[0]+yp[3]*cos(x/180*pi),color='r')
    pl.plot(x,zp[0]+zp[1]+zp[3]*sin(x/180*pi),color='g')
    pl.plot(x,zp[0]+zp[2]+zp[3]*sin(x/180*pi),color='b')
    pl.plot(x,zp[0]+zp[3]*sin(x/180*pi),color='r')
    pl.xlabel("Elevation [deg]")
    pl.ylabel("Focus setting [mm]")
    pl.legend(('B3_x','B3_y','B3_z','B6_x','B6_y','B6_z','B7_x','B7_y','B7_z'), loc=4, numpoints=1)
    title = "%s Focus Curves @ OSF (T_ref = 10 deg C)" % ant
    pl.title( title )
    pl.show()







######## fitting pnly one Band


else :
    for line in open(files):
        fields = line.split()
        if len(fields) == 0 or line[0] == '#':
            continue
        for axis in Axes:
	    RB = "0%s" % band
	    Ax = "%s%s" % (RB,axis)
	    alma_rb = "ALMA_RB_%s" % RB
            if fields[2] == ant and fields[3] == axis and fields[4] == alma_rb and float(fields[6]) < 0.3:
                data[Ax]['Date'].append(fields[0])
                data[Ax]['T'].append(float(fields[1][:-1])-273)
                data[Ax]['Focus_offset'].append(float(fields[5]))
                data[Ax]['Error'].append(float(fields[6]))
                data[Ax]['Az'].append(float(fields[7]))
                data[Ax]['El'].append(float(fields[8]))
                data[Ax]['absolute_Focus'].append(float(fields[9]))

    ######## fitting Y for the R7.1 definition of focus parameters
    def Y(Yp, x):
        Y0, YC = Yp
        return Y0 + YC*cos(x/180*pi)

    def residueY(Yp, y, x, sigma):
        err = (y - Y(Yp,x)) / sigma
        return err

    BY = "%sY" % RB
    p_guess = (3.5, -4.0)
    N = len (data[BY]['El'])
    x = array(data[BY]['El'])
    y = array(data[BY]['absolute_Focus'])
    sigma = sqrt(array(data[BY]['Error'])**2)

    result = optimize.leastsq(residueY, p_guess, args=(y, x, sigma), full_output=1)

    yp = result[0]
    c = result[1]
    err_y = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueY(yp, y, x, sigma)**2 ) / (N - 2)
    Yp = ["Y0", "YC"]

    print "*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(2):
        print "%s = %+12.8f +- %12.8f " % (Yp[i], yp[i], err_y[i])

    ######## fitting Z for the R7.1 definition of focus parameters @ OSF
    def Z(Zp, x):
        Z0, ZS = Zp
        return Z0 + ZS*sin(x/180*pi)

    def residueZ(Zp, z, x, sigma):
        err = (z - Z(Zp,x)) / sigma
        return err

    BZ = "%sZ" % RB
    p_guess = (-3, 2.0)
    N = len (data[BZ]['El'])
    x = array(data[BZ]['El'])
    z = array(data[BZ]['absolute_Focus'])
    sigma = sqrt(array(data[BZ]['Error'])**2)

    result = optimize.leastsq(residueZ, p_guess, args=(z, x, sigma), full_output=1)

    zp = result[0]
    c = result[1]
    err_z = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueZ(zp, z, x, sigma)**2 ) / (N - 2)
    Zp = ["Z0", "ZS"]

    print "*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    for i in range(2):
        print "%s = %+12.8f +- %12.8f " % (Zp[i], zp[i], err_z[i])


    ######## fitting X for the R7.1 definition of focus parameters @ OSF    

    def X(Xp, x):
        X0 = Xp
        return X0

    def residueX(Xp, a, x, sigma):
        err = (a - X(Xp,x)) / sigma
        return err

    BX = "%sX" % RB
    p_guess = (0.0)
    N = len (data[BX]['El'])
    x = array(data[BX]['El'])
    a = array(data[BX]['absolute_Focus'])
    sigma = sqrt(array(data[BX]['Error'])**2)

    result = optimize.leastsq(residueX, p_guess, args=(a, x, sigma), full_output=1)

    xp = result[0]
    c = result[1]
    err_x = sp.sqrt( sp.diag(c) )
    chisq = sp.sum( residueX(xp, a, x, sigma)**2 ) / (N - 1)

    print "*** fitting result (reduced chisq = %7.3f) ****" % (chisq)
    print "X0 = %+12.8f +- %12.8f " % (xp, err_x[0])


    ### For JIRA 
    print "\n ##### For JIRA comments: #####\n" 
    print "The fit result of YC and ZS: \n"
    print "YC = %+3.5f +/- %3.5f " % (yp[1], err_y[1])
    print "ZS = %+3.5f +/- %3.5f " % (zp[1], err_z[1])


    print "\nThe fit result for offsets:\n" 
    print "X0b7   = %+3.5f +/- %3.5f " % (xp, err_x[0])
    print "Y0b7   = %+3.5f +/- %3.5f " % (yp[0], err_y[0])
    print "Z0b7   = %+3.5f +/- %3.5f " % (zp[0], err_z[0])













    ### Print the best estimate of the focus positions at 50 degrees elevation 
    print "\n\nThe best estimate of the focus positions at 50 degrees elevation at T= 10  deg C are:\n" 
    print "X_50 = %+3.3f" % xp 
    print "Y_50 = %+3.3f %+3.3f*cos(50.0/180.0*pi) = %+3.3f" % (yp[0], yp[1],yp[0]+yp[1]*cos(50.0/180.0*pi))
    print "Z_50 = %+3.3f %+3.3f*sin(50.0/180.0*pi) = %+3.3f" % (zp[0], zp[1],zp[0]+zp[1]*sin(50.0/180.0*pi))



    x=arange(10,90,0.1)

    pl.clf()
    pl.plot(data[BX]['El'],data[BX]['absolute_Focus'], 'ms')
    pl.plot(data[BY]['El'],data[BY]['absolute_Focus'], 'm^')
    pl.plot(data[BZ]['El'],data[BZ]['absolute_Focus'], 'mo')
    pl.plot([10,90],[xp,xp],color='m')
    pl.plot(x,yp[0]+yp[1]*cos(x/180*pi),color='m')
    pl.plot(x,zp[0]+zp[1]*sin(x/180*pi),color='m')
    pl.xlabel("Elevation [deg]")
    pl.ylabel("Focus setting [mm]")
    pl.legend(("B%s_x" % band,"B%s_y" % band,"B%s_z" % band), loc=4, numpoints=1)
    title = "%s Band%s Focus Curve" % (ant, band)
    pl.title( title )
    pl.show()












