import os
import time
import re
import pickle
import glob
import matplotlib.ticker 
import time
import shutil
import pylab as pl
import datetime
import casadef
import numpy as np
import findContinuum
import subprocess
from analysisUtils import primaryBeamArcsec, numberOfChannelsInCube
from analysisUtils import nonzeroPixels, getFitsBeam, grep, taskExecutionTimes
from analysisUtils import imviewField, roundFiguresToString, gaussianBeamOffset
from analysisUtils import angularScaleBaseline, imagePrimaryBeam, gaussianBeamResponse
from analysisUtils import surmiseCycleFromImage, invertChannelRanges, flattenMask
from analysisUtils import imheadlist, transition, restFrequency, imageHistogram
from taskinit import *
from imhead_cli import imhead_cli as imhead
from imstat_cli import imstat_cli as imstat
from imregrid_cli import imregrid_cli as imregrid
from immoments_cli import immoments_cli as immoments
from importfits_cli import importfits_cli as importfits
if (casadef.casa_version > '4.2.2'):
    from specsmooth_cli import specsmooth_cli as specsmooth
from makemask_cli import makemask_cli as makemask

# global variable
startTime = 0

#from recipes.makepb import makePB
#makePB('/lustre/naasc/sciops/PipelineTestData/cycle3/2013.1.00305.S_uid___A001_X13d_X72.manual/uid___A002_Xa7a216_X28c.ms.split.cal',
#   outimage='/lustre/naasc/sciops/PipelineTestData/cycle3/2013.1.00305.S_uid___A001_X13d_X72.manual/HD142527_C18O21_image.pb',
#   field='4',
#   imtemplate='/lustre/naasc/sciops/PipelineTestData/cycle3/2013.1.00305.S_uid___A001_X13d_X72.manual/HD142527_C18O21_image.image')

ANALYZEMSIMAGE_REVISION_STRING = "$Id: analyzemsimage.py,v 1.75 2017/02/17 22:22:21 thunter Exp $" 
rsyncPermissions = '--no-p'

def importIfFITS(cube, overwrite=False):
    """
    Checks whether a cube is in FITS format, and if so, import it into
    a CASA cube, if it does not already exist in the same directory.
    """
    if (cube.lower().find('fits') > 0):
        cubeFits = cube
#        cube = cubeFits.replace('.fits','.image').replace('.FITS','.image')
        cube = rreplace(cubeFits,'.fits','.image')
        cube = rreplace(cube,'.FITS','.image')
        if (not os.path.exists(cube) or overwrite):
            print "importfits('%s', '%s', overwrite=True)" % (cubeFits, cube)
            importfits(cubeFits , cube, overwrite=True)
        else:
            print "Using pre-existing CASA image = ", cube
    return(cube)

def findContDat(img):
    fc = img.replace('s31','*').replace('.cube.I.iter1.image','.mfs.I.findcont.residual_findContinuum.dat')
    imgs = glob.glob(fc)
    if (len(imgs) < 1):
        print "Failed to find _findContinuum.dat:", fc
    return imgs[0]

def findSpectralAxis(img):
    """
    CASA images typically have the fourth axis as spectral, but exportfits 
    changes this to the third axis.  This function finds where it is.
    Returns: a number from 0 to 3 (most typically 2 or 3)
    """
    myia = createCasaTool(iatool)
    myia.open(img)
    mycs = myia.coordsys()
    try:
        iax = mycs.findaxisbyname("spectral")
    except:
        print "ERROR: can't find spectral axis.  Assuming it is 3."
        iax = 3
    mycs.done()
    myia.close()
    return iax

def createCasaTool(mytool):
    """
    A wrapper to handle the changing ways in which casa tools are invoked.
    Todd Hunter
    """
    if (type(casac.Quantity) != type):  # casa 4.x
        myt = mytool()
    else:  # casa 3.x
        myt = mytool.create()
    return(myt)

def version():
    """
    Returns the complete CVS version string for the current file.
    """
    return(ANALYZEMSIMAGE_REVISION_STRING)
 
def getCvsVersion(versionString):
    """
    Extracts the meaningful fields from a CVS version string.
    Example: 'analyzemscal.py,v 1.24 2013/01/18 19:26:56'
    """
    tokens = versionString.split()        
    myVersion = tokens[1] + ' ' + tokens[2] + ' ' + tokens[3] + ' ' + tokens[4]
    return(myVersion)

def myCopyfile(a, b):
    """
    Checks whether a specified file exists in a specified location and
    if not, it copies it there, preserving the timestamps.
    a: source file
    b: destination
    """
    if (os.path.dirname(a) == b):
        return
    copyIndent = ' '*10         
#    print "%srsync from %s to %s" % (copyIndent, a, b)
    #shutil.copyfile(a,b)
    os.system('rsync -au %s %s %s' % (rsyncPermissions,a,b))

def addplot(webpagedir, myplot, width=800, br=True, debug=False):
    """
    Copies a single plot file into the specified web area, and returns 
    the text that places plot on the current html page.
    """
    myplot0 = os.path.basename(myplot)
    if (br):
        brk = '<br>'
    else:
        brk = ''
    mystring = """<a href="%s"><img width="%d" src="%s"></img></a>%s\n\n""" % (myplot0, width, myplot0, brk)
    destination = webpagedir+myplot0
    myCopyfile(myplot, destination)
    return(mystring)

def addplotrow(webpagedir, title, plotForColumn1='', plotForColumn2='', plotForColumn3='', 
               width=400, pdf1='', pdf2='', pdf3='', textForColumn1='', textForColumn2='', 
               textForColumn3='', justifyTitle='left',colspanTitle=1,bold=False, debug=False,
               regriddedColumn=3):
    """
    Copies up to 2 plot files into the web area, and adds a row to the current html table
    If pdf1 is not blank, then the link is made to the PDF file rather than the full-sized
    version of the png.
    """
    if (regriddedColumn == 2):
        swap = textForColumn2
        textForColumn2 = textForColumn3
        textForColumn3 = swap
        swap = plotForColumn2
        plotForColumn2 = plotForColumn3
        plotForColumn3 = swap
        swap = pdf2
        pdf2 = pdf3
        pdf3 = pdf
    elif (regriddedColumn < 1):
        # Then don't show it at all
        textForColumn3 = ''
        plotForColumn3 = ''
        pdf3 = ''
        if (colspanTitle == 4):
            colspanTitle = 3
#    print "plotForColumn2 = ", plotForColumn2
    copyIndent = ' '*10         
    plot1="c1_" + plotForColumn1.split('/')[-1]
    if (plotForColumn2 != None):
        plot2="c2_" + plotForColumn2.split('/')[-1]
    else:
        plot2 = ''
    if (plotForColumn3 != None):
        plot3="c3_" + plotForColumn3.split('/')[-1]
    else:
        plot3=''
    if (1==0):    # debugging use
        if (textForColumn1 == ''):
            textForColumn1 = plotForColumn1
        if (textForColumn2 == ''):
            textForColumn2 = plotForColumn2
    if bold:
        textForColumn1 = '<b>' + textForColumn1
        textForColumn2 = '<b>' + textForColumn2
        textForColumn3 = '<b>' + textForColumn3
    bgcolor = '#E8E8E8'
    if (title.find('<b>') < 0):
        title = '<b>' + title
    mystring = """<tr><td align="%s" bgcolor="%s" colspan="%d">%s</td>""" % (justifyTitle,bgcolor,colspanTitle,title)
    if (colspanTitle == 1):
        bgcolor = 'white'
    if (colspanTitle < 3 or (colspanTitle==3 and regriddedColumn>0)):
        if (pdf1 == ''):
            if (plot1 == 'c1_'):
                mystring += """<td bgcolor=%s>%s</td>\n\n""" % (bgcolor,textForColumn1)
            else:
                mystring += """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td>\n""" % (textForColumn1, plot1, width, plot1)
        else:
            mystring = """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td>\n""" % (textForColumn1, os.path.basename(pdf1), width, plot1)
        if (pdf2 == ''):
            if (plot2 == 'c2_'):
                mystring += """<td bgcolor=%s>%s</td> \n\n""" % (bgcolor,textForColumn2)
            else:
                mystring += """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td>\n\n""" % (textForColumn2, plot2, width, plot2)
        else:
            mystring += """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td>\n\n""" % (textForColumn2, os.path.basename(pdf2), width, plot2)
        if (regriddedColumn > 0):
            if (pdf3 == ''):
                if (plot3 == 'c3_'):
                    mystring += """<td bgcolor=%s>%s</td> </tr>\n\n""" % (bgcolor,textForColumn3)
                else:
                    mystring += """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td> </tr>\n\n""" % (textForColumn3, plot3, width, plot3)
            else:
                mystring += """<td>%s<br><a href="%s"><img width="%d" src="%s"></img></a></td> </tr>\n\n""" % (textForColumn3, os.path.basename(pdf3), width, plot3)
        else:
            mystring += """</tr>\n\n"""
    else:
        mystring += """</tr>\n\n"""
    # Make sure that the plots and PDFs get copied to the webpage, automatically.
    if os.path.exists(plotForColumn1):
        destination = webpagedir+plot1
        myCopyfile(plotForColumn1, destination)
    elif (plotForColumn1 != ''):
        print "ERROR: plotForColumn1 = %s does not exist!" % (plotForColumn1)

    if plotForColumn2 != None:
        if os.path.exists(plotForColumn2):
            destination = webpagedir+plot2
            myCopyfile(plotForColumn2, destination)
        elif (plotForColumn2 != ''):
            print "ERROR: plotForColumn2 = %s does not exist!" % (plotForColumn2)
        
    if plotForColumn3 != None:
        if os.path.exists(plotForColumn3):
            destination = webpagedir+plot3
            myCopyfile(plotForColumn3, destination)
        elif (plotForColumn3 != ''):
            print "ERROR: plotForColumn3 = %s does not exist!" % (plotForColumn3)
        
    if os.path.exists(pdf1):
        destination = webpagedir+os.path.basename(pdf1)
        myCopyfile(pdf1, destination)
    elif (pdf1 != ''):
        print "ERROR: pdf1 = %s does not exist!" % (pdf1)
    if os.path.exists(pdf2):
        destination = webpagedir+os.path.basename(pdf2)
        myCopyfile(pdf2, destination)
    elif (pdf2 != ''):
        print "ERROR: pdf2 = %s does not exist!" % (pdf2)
    if os.path.exists(pdf3):
        destination = webpagedir+os.path.basename(pdf3)
        myCopyfile(pdf3, destination)
    elif (pdf3 != ''):
        print "ERROR: pdf3 = %s does not exist!" % (pdf3)
    return(mystring)

def removeIfPresent(img):
    """
    Removes the specified directory, if it is present.
    """
    if os.path.exists(img):
        try:
            shutil.rmtree(img) 
        except:
            print "Could not remove file, perhaps because it is in use."
            cmd = "/usr/sbin/lsof +D %s" % (img)
            print "Running " + cmd
            os.system(cmd)
            print "You should quit or kill the offending process (if any)."
            sys.exit(0)

def showElapsedTime(mystring=''):
    # the trailing comma prevents the newline so that ...10...20 will be on same line
    line = "%s %s %s" % (computeElapsedTime(), time.strftime('%H:%M:%S',time.gmtime()), mystring) 
    if (mystring.lower().find('done') == 0):
        print line
    else:
        print line,
    return line

def buildMomentImage(img, moment, resultsDir, includepix=-1, suffix='', chans='', loc=-1):
    """
    Takes a CASA cube, and creates the specified moment image, writing it to
    the desired directory.
    moment: integer moment (e.g. 0..8=peak)
    """
#    print "Running buildMomentImage(loc=%d)" % (loc)
    img_mom = os.path.basename(img) + suffix + '.mom%d' % (moment)
    if (resultsDir != ''):
        img_mom = resultsDir + '/' + img_mom
    removeIfPresent(img_mom) # prevent table cache issue
    if (includepix > -1):
        includepix = [includepix,1e9]
    print "%d Running immoments('%s',moments=[%d],outfile='%s',includepix=%s,chans='%s')" % (loc,img,moment,img_mom,str(includepix),chans)
    showElapsedTime("immoments")
    immoments(img, moments=[moment], outfile=img_mom, includepix=includepix, chans=chans)
    print "Running imheadlist('%s')" % (img_mom)
    showElapsedTime("imhead")
    img_mom_head = imheadlist(img_mom) # imhead(img_mom, mode='list')
#    print "Done buildMomentImage(loc=%d)" % loc
    return(img_mom, img_mom_head)

def buildImagePng(img, radius, resultsDir, minIntensity=None, maxIntensity=None,
                  maxIntensityMinFraction=None, contourImage=None, levels=[], 
                  contourImage2=None, levels2=[], verbose=False, 
                  colorwedge=True, contourColor='yellow', contourColor2='yellow',
                  colormap='Rainbow 2'):
    """
    Takes a CASA image and creates a color plot with the specified image size
    and intensity range.  See help(au.imviewField) for the meaning of the other
    parameters.
    """
    plotfile = os.path.basename(img) + '.png'
    if (resultsDir != ''):
        plotfile = resultsDir + '/' + plotfile
    if verbose:
        print "buildImagePng: minIntensity = ", minIntensity
        print "buildImagePng: maxIntensity = ", maxIntensity
        showElapsedTime("starting imviewField")
    if (os.getenv('HOSTTYPE') == 'intel-mac'):
        pathToConvert = '/opt/local/bin/'
    else:
        pathToConvert = ''
    img_png = imviewField(img, radius, minIntensity, maxIntensity=maxIntensity, 
                          maxIntensityMinFraction=maxIntensityMinFraction,
                          colorwedge=colorwedge, plotfile=plotfile, 
                          contourImage=contourImage, levels=levels,
                          contourImage2=contourImage2, levels2=levels2, 
                          verbose=verbose, contourColor=contourColor,
                          contourColor2=contourColor2, colormap=colormap,
                          pathToConvert=pathToConvert)
    if verbose:
        showElapsedTime("done imviewField")
    if (img_png != None):
        if (len(img_png) > 0):
            os.system('chmod g+w ' + img_png)
    if verbose:
        showElapsedTime("done chmod")
    return img_png

def initializeWebpage(directoriesAndVersions, user, host, date, casaVersion, 
                      py0, py1, py2, py3, py4, cwd, commandLine, webpagedir):
    """
    Returns a string containing the header portion of a new .html file.
    """
    webpage = """
    <html>
      <head>
         <title>analyzemsimage</title>
         <style type="text/css">
            html, td {font-size: 90%%;}
            .headline {font-size:200%%; font-weight: bold;}              
            table {border-collapse:collapse;}
            table, th, td {border: 1px solid black;}           
            td {padding:0.25em}
            td {vertical-align:top;}
            td[colspan] {background-color: #F3F3F3;}
            .innertable td, .innertable tr, .innertable {border: none; padding: 0; padding-right: 1em;}            
            b.passquality {color: #227722;}
            b.nopassquality {color: #772222;}
         </style>
      </head>
      <body>
      <div class="headline"><a href="%s">analyzemsimage</a></div><br>
      <H3>Location of data and version information</H3>
      %s
      This report was generated by %s on %s (%.1fGB) at <b>%s</b> from CASA %s (python %d.%d.%d %s %d).<br>
      PWD = %s<br>
      Command line = %s<br><br>
    """ % (fileToWebAddress(os.path.dirname(webpagedir.rstrip('/'))),directoriesAndVersions, user, host, getMemorySize()/1e9, date, casaVersion, py0,py1,py2,py3,py4,
           cwd, commandLine)
    return webpage

def beamRadius(head):
    """
    Returns geometric mean of restoring beam, in arcsec
    """
    token = beamString(head).split()
    major = float(token[0].strip('"'))
    minor = float(token[2].strip('"'))
    return (major*minor)**0.5

def beamString(head, roundMajorAxis=3, roundPA=4):
    """
    Reads a CASA image header dictionary and produces a string of the format 
    "X.XX x Y.YY at ZZ.ZZ deg".  If no beam is found in the header, then return 'no beam'.
    roundMajorAxis: number of significant figures to show for bmaj and bmin
    roundPA: number of significant figures to show for position angle
    """
    if ('beammajor') in head:
        bmaj = head['beammajor']['value']
        bmin = head['beamminor']['value']
        bpa = head['beampa']['value']
    elif ('perplanebeams' in head):
        nchan = head['perplanebeams']['nChannels']
        bmaj = head['perplanebeams']["*"+str(int(np.floor(nchan/2.0)))]['major']['value']
        bmin = head['perplanebeams']["*"+str(int(np.floor(nchan/2.0)))]['minor']['value']
        bpa = head['perplanebeams']["*"+str(int(np.floor(nchan/2.0)))]['positionangle']['value']
    else:
        print "Did not find 'beammajor' or 'perplanebeams' in header.  Is this a dirty image?"
        return 'no beam'
    string = '%s" x %s" at %s deg' % (roundFiguresToString(bmaj,roundMajorAxis), 
                                      roundFiguresToString(bmin,roundMajorAxis), 
                                      roundFiguresToString(bpa,roundPA))
    return string

def plotSpectrum(img, head=None, stats=None, profileSigma=None, radius=None, mask='', 
                 ls='-', ylimits=None, pngnameSuffix='', outputpath='', 
                 pixel='', title='', contChannels='', region='', 
                 function='flux', unit='km/s', mJy=False, outfile=''):
    """
    Takes a CASA cube, header and imstat dictionary, and generates a spectrum
    plot over a specied radius or mask area.
    Inputs:
    head: header dictionary (if not specified, then it will run getHeader)
    stats: stats dictionary (only needed if profileSigma is specified)
    profileSigma: number of sigma below which to ignore the data (overrides mask)
    radius: in arcsec (overrides region)
    mask: mask to use (unless profileSigma is specified)
    region: region to use (both mask and region are passed to ia.getprofile)
    pngnameSuffix: insert this string before .png
    pixel: 'x,y', where to draw the radius from, default='' --> center of image
    function: 'flux','mean','max','median','min','rms','sum','stdev','variance'
    unit: units to use for spectral axis
    outfile: name of ASCII file to produce, or True for img+'.dat'
    Returns: the png filename
    """
    if head == None:
        head = getHeader(img)
    naxis1 = head['shape'][0]
    naxis2 = head['shape'][1]
    specaxis = findSpectralAxis(img)
    myia = createCasaTool(iatool)
    myia.open(img)
    if (profileSigma != None):
        threshold = profileSigma*stats['sigma']
        mask = "'%s'>%f"%(img,threshold)
    if (radius != None):
        if (pixel == ''):
            xpixel = int(0.5*naxis1)
            ypixel = int(0.5*naxis2)
        else:
            if (type(pixel) == str):
                xpixel,ypixel = pixel.split(',')
            xpixel,ypixel = pixel
        region = "circle[[%ipix,%ipix],%farcsec]"%(xpixel,ypixel,radius)

    profile = myia.getprofile(axis=specaxis, region=region, mask=mask, 
                              stretch=True, function=function, unit=unit) 
    pl.clf()
    desc = pl.subplot(111)
    idx = np.where(profile['values'] != 0.0)
    if mJy:
        profile['values'] *= 1000
    if unit == '' or unit.lower() == 'hz':
        # convert Hz to GHz
        pl.plot(profile['coords'][idx]*1e-9, profile['values'][idx], 'k-')
    else:
        pl.plot(profile['coords'][idx], profile['values'][idx], 'k-')
    if outfile == True:
        outfile = img+'.dat'
    if outfile != '':
        f = open(outfile,'w')
        f.write('# img: %s\n'%img)
        if (mask != None and mask != ''):
            f.write('# mask: %s\n'%mask)
        if (region != '' and type(region) == str):
            f.write('# region: %s\n'%region)
        for i in range(len(profile['coords'])):
            f.write('%.9f %.9f\n' % (profile['coords'][i],profile['values'][i]))
        f.close()
        print "Wrote ", outfile
    if (contChannels != ''):
        for contRange in contChannels.split(';'):
            x0,x1 = [int(i) for i in contRange.split('~')]
            if unit == '' or unit.lower() == 'hz':
                # convert Hz to GHz
                xvals = [profile['coords'][x0]*1e-9, profile['coords'][x1]*1e-9]
            else:
                xvals = [profile['coords'][x0], profile['coords'][x1]]
            yvals = [pl.mean(pl.ylim())] * 2
            print "xvals=%s, yvals=%s" % (str(xvals), str(yvals))
            pl.plot(xvals, yvals, 'g-')
    if unit == '' or unit.lower() == 'hz':
        pl.xlabel('Frequency (GHz)')
    elif unit == 'km/s':
        pl.xlabel('Velocity (km/s)')
    else:
        pl.xlabel(unit)
    if function == 'flux':
        if mJy:
            pl.ylabel('Flux density (mJy)')
        else:
            pl.ylabel('Flux density (Jy)')
    else:
        if mJy:
            pl.ylabel('%s intensity (mJy/beam)' % function)
        else:
            pl.ylabel('%s intensity (Jy/beam)' % function)
    if (ylimits == None):
        # drop edge pixels from autoscaling
        ylimits = np.array([np.min(profile['values'][idx][1:-1]),np.max(profile['values'][idx][1:-1])])
    pl.ylim(ylimits)
    yFormatter = matplotlib.ticker.ScalarFormatter(useOffset=False)
    desc.yaxis.set_major_formatter(yFormatter)
    desc.xaxis.set_major_formatter(yFormatter)
    pl.title(title)
    pl.draw()
    if (outputpath != ''):
        img = outputpath + '/' + os.path.basename(img)
    if (profileSigma == None):
        spectrumPng = img + pngnameSuffix + '_spectrum.png'
    else:
        spectrumPng = img + pngnameSuffix + '_sigma_spectrum.png'
    pl.savefig(spectrumPng)
    os.system('chmod g+w ' + spectrumPng)
    myia.close()
    return spectrumPng, pl.ylim()

def expandLimits(lim, newlim):
    lim = list(lim)
    newlim = list(newlim)
    if (newlim[0] < lim[0]):
        lim[0] = newlim[0]
    if (newlim[1] > lim[1]):
        lim[1] = newlim[1]
    return lim

def jy_mjy_string_stats(stats, keyword, digits):
    """
    Reads a keyword out of a dictionary and pass it to jy_mjy_string to create a string
    """
    if (stats == ''):
        return('')
    return(jy_mjy_string(stats[keyword], digits))

def jy_mjy_string(jy, digits):
    return(roundFiguresToString(jy,digits) + ' = %s mJy/b' % roundFiguresToString(jy*1000,digits))

def aggregateBandwidthString(bw, chanWidth):
    return('aggregate bandwidth = %g MHz (%d chan)'%(bw*1000, abs(bw*1e9/chanWidth)))

def getHeader(img, psf=''):
    """
    Reads the header of an image.  If the beam is not present, then look for it in
    the specified psf (or try to find a psf) and copy it into the original header.
    """
    showElapsedTime("imhead")
    head = imheadlist(img) # imhead(img, mode='list')
    # Add some information to the header dictionary
    head['imagefile'] = os.path.basename(img)
    head['npix'] = head['shape'][0] * head['shape'][1]
    if beamString(head) == 'no beam':
        if (psf == ''):
            imageStrings = [m.start() for m in re.finditer(r".image", img)]
            if (len(imageStrings) < 1):
                imageStrings = [m.start() for m in re.finditer(r".residual", img)]
                if (len(imageStrings) < 1):
                    imageStrings = [m.start() for m in re.finditer(r".pbcor", img)]
                    if (len(imageStrings) < 1):
                        imageStrings = [m.start() for m in re.finditer(r".pbc", img)]
                        if (len(imageStrings) < 1):
                            imageStrings = [m.start() for m in re.finditer(r".mask", img)]
            if (len(imageStrings) > 0):
                finalImage = imageStrings[-1]
                psf = img[:finalImage] + '.psf'
                print "Looking for psf = ", psf
        if os.path.exists(psf):
            print "psf imhead", # the comma prevents the newline so that ...10...20 will be on same line
            headpsf = imheadlist(psf) # imhead(psf, mode='list')
            if ('beammajor' in headpsf):
                head['beammajor'] = headpsf['beammajor']
                head['beamminor'] = headpsf['beamminor']
                head['bpa'] = headpsf['beampa']
            elif ('perplanebeams' in headpsf):
                head['perplanebeams'] = headpsf['perplanebeams']
            else:
                print "But still no beam found in psf."
        else:
            print "Did not find psf image"
    return head

def imageSizeString(header):
    cdelt1arcsec = np.degrees(abs(header['cdelt1']))*3600 
    cdelt2arcsec = np.degrees(abs(header['cdelt2']))*3600
    string = '%dx%d=%d pixels of %g" (%g"x%g")' % (header['shape'][0], header['shape'][1],
                                                   header['shape'][0]*header['shape'][1],
                                                   cdelt1arcsec,
                                                   header['shape'][0]*cdelt1arcsec,
                                                   header['shape'][1]*cdelt2arcsec)
    return string

def spectralSizeString(header, chanWidth, specaxis):
    nchan = header['shape'][specaxis]
    string = '%d channels * %s kHz = %s MHz' % (nchan,roundFiguresToString(abs(chanWidth*1e-3),4),
                                                roundFiguresToString(nchan*abs(chanWidth*1e-6),4))
    return string

def buildRegionsFile(img, contChannels, specAxis):
    """
    Not currently used.
    """
    regions = open(img + '.regions', 'w')
    ranges = contChannels.split(';')
    blc = [0,0,0,0]
    trc = [0,0,0,0]
    for i,channelRange in enumerate(ranges):
        chans = channelRange.split('~')
        if (len(chans) == 2):
            blc[specAxis] = int(chans[0])
            trc[specAxis] = int(chans[1])
            regions.write('box[%s, %s]\n' % (blc, trc))
        else:
            print "skipping blank channelRange %d of %d = " % (i+1,len(ranges)), channelRange
    regions.close()
    return regions

def computePBCircle(header, radius, contChannels):
    """
    Inputs: 
    header: dictionary output of imhead(mode='list')
    radius: value in arcsec
    Returns: things that can be used by region parameter of imstat
    1) A string containing the circle (no channels specified)
    2) A text filename containing multiple regions, i.e. a circle at the center 
       of the image in each group of channels
    """
    regionFile = header['imagefile'] + '.regions'
    f = open(regionFile, 'w')
    f.write('#CRTFv0')
    xpix = header['shape'][0]
    ypix = header['shape'][1]
    x = xpix/2
    y = ypix/2
    r = radius/(abs(np.degrees(header['cdelt1']))*3600)
    region = 'circle[[%dpix,%dpix], %fpix]' % (x,y,r)
    if (contChannels == ''):
        f.write(region + '\n')
    else:
        for c in contChannels.split(';'):
            chans = c.split('~')
            if (len(chans)==2):
                c0,c1 = chans
                region = 'circle[[%dpix,%dpix], %fpix] range=[%dchan,%dchan]\n' % (x,y,r,int(c0),int(c1))
                f.write(region)
    return region, regionFile

def computeBLCTRC(header, radius):
    """
    Inputs: 
    header: dictionary output of imhead(mode='list')
    radius: value in arcsec
    Returns:
    4 integers: blc_x, blc_y, trc_x, trc_y (all in pixels)
    """
    xpix = header['shape'][0]
    ypix = header['shape'][1]
    xcenter = xpix/2
    ycenter = ypix/2
    #         arcsec / (arcsec/pixel)
    xradius = radius/(abs(np.degrees(header['cdelt1']))*3600)
    yradius = radius/(abs(np.degrees(header['cdelt2']))*3600)
    x0 = int(round(xcenter-xradius))
    x1 = int(round(xcenter+xradius))
    y0 = int(round(ycenter-yradius))
    y1 = int(round(ycenter+yradius))
    if (x0 < 0): x0=0
    if (y0 < 0): y0=0
    if (x1 > xpix): x1 = xpix-1
    if (y1 > ypix): y1 = ypix-1
    return '%d,%d,%d,%d' % (x0,y0,x1,y1)

def buildMask(img, contChannels, radius, img_header, overwrite=False):
    """
    Builds a mask for an image from a list of continuum channels and
    the radius of a central box.
    Inputs:
    contChannels: string list from findContinuum, e.g. '5~45;60~100'
    radius: of central box, in arcsec
    Returns:
    New mask image.
    """
    ranges = contChannels.split(';')
    newmask = img + '.continuumChannels.mask'
    if (not os.path.exists(newmask) or overwrite):
        masks = []
        x0,y0,x1,y1 = [int(i) for i in computeBLCTRC(img_header, radius).split(',')]
        print "blc,trc = ", x0,y0, x1,y1
        for i,channelRange in enumerate(ranges):
            chans = [int(i) for i in channelRange.split('~')]
            if (len(chans) == 2):
                mask = 'box[[%dpix,%dpix],[%dpix,%dpix]] range=[%dpix,%dpix]'%(x0,y0,x1,y1,chans[0],chans[1])
                print "Appending mask = ", mask
                masks.append(mask)
            else:
                print "skipping blank channelRange %d of %d = " % (i+1,len(ranges)), channelRange
        print "makemask running"
        makemask(mode='copy', inpimage=img,
                 inpmask=masks, output=newmask, overwrite=True)
        print "makemask complete"
    return newmask

def getMemorySize():
    """
    Returns memory size in bytes
    """
    try:
        return(os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES'))
    except ValueError:
        # SC_PHYS_PAGES can be missing on OS X
        return int(subprocess.check_output(['sysctl', '-n', 'hw.memsize']).strip())

def fileToWebAddress(webfile):
    """
    Converts a file path in the NRAO-CV web area to a URL.
    """
    if (webfile == ''):
        return webfile
    return('https://safe.nrao.edu/'+webfile.replace('/home/safe.nrao.edu/content',''))

def getSourceNameFromPipelineImageName(img, verbose=False):
    """
    Extracts the field name from the pipeline image file name.
    """
    sourceName = os.path.basename(img)
    if verbose: print "A = ", sourceName
    if (sourceName.find('_0.') < 0):
        return 'sourcename'
    sourceName = sourceName.split('_0.')[1]
    if verbose: print "B = ", sourceName
    sourceName = sourceName.split('_sci')[0]
    if verbose: print "C = ", sourceName
    sourceName = sourceName.strip('_')
    if verbose: print "sourceName = ", sourceName
    return sourceName

def analyzemsimages(pipelineImages=[], manualImages=[], 
                    webfile='analyzemsimage/index.html',
                    pipelineDir='', manualDir='', automask=False, pblimit=0.5, 
                    mom0sigma=2.0, profileSigma=3.0, 
                    findcontVerbose=False, overwriteImport=False,
                    overwriteSpecsmooth=False, overwriteRegrid=False,
                    overwriteMask=False, overwriteFindContinuum=False,
                    webarea=True, imagedir='', verbose=False,
                    minmaxScale='manual', regriddedColumn=-1,
                    imageRadius=0, bins=40, log=''):
    """
    Accepts a list of pipeline-produced images and a list of corresponding manually-produced
    images and generates a webpage showing statistics of the images, moment 0 and 8 images and
    spectra.  If the .meanSpectrum file does not exist in the image directories, it will 
    run findContinuum and write the result to that directory (if you have write permission).
    Inputs:
    pipelineImages: a list of strings, e.g. ['pipe_img1.fits'] or 'img1.fits,img2.fits'
    manualIamges: a list of strings, e.g. ['manual_img1.image']
    webfile: full or relative path of html file to produce, e.g. '2013.1.00099.S/index.html'
    webarea: True: treat webfile as a relative directory, and precede it with
                   /home/safe.nrao.edu/content/alma/PipelineTestResults/
    imagedir: directory to write new images, default: current working directory
    automask: if False, then compute statistics over central primary beam (set by pblimit)
              if True, then use recipes.automask of moment8 (not yet implemented!)
    pblimit: compute statistics over the central primary beam with radius equal to this 
             Gaussian beam sensitivity level (0.0..1.0)
    minmaxScale: 'manual' or 'pipeline'; sets the min/max colorbar of residual image
    mom0sigma: threshold
    overwriteImport: if True, and image is FITS, then overwrite CASA image if it also exists
    overwriteSpecsmooth: if True, the regenerate specsmoothed pipeline image if present
    overwriteRegrid: if True, the regenerate the spatially regridded pipeline image if present
    overwriteFindContinuum: if True, then rerun findContinuum even if *_findContinuum.dat exists
    regriddedColumn: which column to place the pipelineRegridded result (2 or 3), 
                     <=0 means don't show it, and don't bother running specsmooth
    imageRadius: radius of image (in arcsec) to display as png
    bins: number of bins to use in the residual image histogram
    Example execution:
    analyzemsimage.analyzemsimages(pipelineImages=['uid___A001_X122_X42.s24_0.Arp220_sci.spw15.mfs.I.iter1.image',
                                                   'uid___A001_X122_X42.s24_0.Arp220_sci.spw17.mfs.I.iter1.image',
                                                   'uid___A001_X122_X42.s24_0.Arp220_sci.spw19.mfs.I.iter1.image'],
                                   manualImages=['calibrated_Arp220.cont_spw0.image',
                                                 'calibrated_Arp220.cont_spw1.image',
                                                 'calibrated_Arp220.cont_spw2.image'],
                                   webfile='arp220.html',
                                   pipelineDir='/lustre/naasc/sciops/PipelineTestData/cycle3/2013.1.00099.S_uid___A001_X122_X42.C3R4/',
                                   manualDir='/lustre/naasc/users/cbrogan/Pipeline_Imaging_Tests/2013.1.00099.S/'
    """
    global startTime
    startTime = time.time()
    if (type(pipelineImages) == str):
        pipelineImages = pipelineImages.split(',')
    if (type(manualImages) == str):
        manualImages = manualImages.split(',')
    if (webfile == ''):
        webfile = 'index.html'
    contsubResults = ''
    lineResults = ''
    contResults = ''
    if webarea:
        if (webfile[0] != '/'):
            webfile = '/home/safe.nrao.edu/content/alma/PipelineTestResults/' + webfile
        else:
            print "webfile begins with a /, so I will assume this is a full path."
        contsubResults = lookForFile(os.path.dirname(webfile.replace('analyzemsimage','analyzecontsub'))+'/*.html')
        contsubResults = fileToWebAddress(contsubResults)
        if (os.path.basename(webfile) != 'line.html'):
            lineResults = fileToWebAddress(os.path.dirname(webfile)+'/line.html')
        else:
            contResults = fileToWebAddress(os.path.dirname(webfile)+'/continuum.html')
  
    webpagedir = os.path.dirname(webfile) + '/'
    if (webpagedir == '/'): 
        webpagedir = ''
    elif (not os.path.exists(webpagedir)):
        print "----- Making web subdirectories:  %s -----" % (webpagedir)
        os.makedirs(webpagedir)
        os.system('chmod g+w ' + webpagedir)

    directoriesAndVersions = 'ami version = %s<br>\n' % (version())
    directoriesAndVersions += 'findContinuum version = %s<br>\n' % (findContinuum.version())
    py0,py1,py2,py3,py4 = sys.version_info
    spc = 31
    commandLine = "%s.analyzemsimages(pipelineImages=%s,<br>" % (analyzemsimage.__name__, pipelineImages) + \
                   + spc*'&nbsp;' + "manualImages=%s,<br>" % (manualImages)  + \
                   + spc*'&nbsp;' + "pipelineDir='%s',<br>" % (pipelineDir)  + \
                   + spc*'&nbsp;' + "manualDir='%s',<br>" % (manualDir)  + \
                   + spc*'&nbsp;' + "webfile='%s',<br>" % (webfile) + \
                   + spc*'&nbsp;' + "webarea=%s,<br>" % (webarea) + \
                   + spc*'&nbsp;' + "imagedir='%s',<br>" % (imagedir) + \
                   + spc*'&nbsp;' + "pblimit=%g, automask=%s, mom0sigma=%g, profileSigma=%g,<br>" % (pblimit,automask,mom0sigma,profileSigma) + \
                   + spc*'&nbsp;' + "overwriteImport=%s, overwriteSpecsmooth=%s, overwriteRegrid=%s, overwriteMask=%s,<br>" % (overwriteImport, overwriteSpecsmooth, overwriteRegrid, overwriteMask) + \
                   + spc*'&nbsp;' + "overwriteFindContinuum=%s, minmaxScale='%s',<br>" % (overwriteFindContinuum, minmaxScale) + \
                   + spc*'&nbsp;' + "regriddedColumn=%d, imageRadius=%g)<br>" % (regriddedColumn, imageRadius)

    # Prepend image list with pipelineDir
    if (pipelineDir != ''):
        pipelineImages = [pipelineDir.rstrip('/') + '/' + p for p in pipelineImages]
    if (manualDir != ''):
        manualImages = [manualDir.rstrip('/') + '/' + p for p in manualImages]
    if (len(pipelineImages) != len(manualImages)):
        print "Mismatch in number of pipeline (%d) vs. manual (%d) images." % (len(pipelineImages),len(manualImages))
        return
    if (len(pipelineImages) < 1):
        print "No images specified."
        return
    webpage = initializeWebpage(directoriesAndVersions, os.getenv('USER'), os.getenv('HOST'), 
                                str(datetime.datetime.now()), 
                                casadef.casa_version+' r'+casadef.subversion_revision, 
                                py0,py1,py2,py3,py4,os.getcwd(), commandLine, webpagedir)

    for i in range(len(pipelineImages)):
        if (not os.path.exists(pipelineImages[i])):
            print "Could not find pipelineImage = ", pipelineImages[i]
            return
        if (not os.path.exists(manualImages[i])):
            print "Could not find manualImage = ", manualImages[i]
            return
    manualChannels, manualChanWidth = numberOfChannelsInCube(manualImages[0], returnChannelWidth=True)
    sourceName = getSourceNameFromPipelineImageName(pipelineImages[0])
    if (manualChannels == 1):
        acsTxt = webpagedir + 'analyzemsimage_%s.continuum.txt' % (sourceName)
    else:
        acsTxt = webpagedir + 'analyzemsimage_%s.line.txt' % (sourceName)

    if (len(contsubResults) > 0):
        webpage += '<a href="%s">Link to contsub results page</a><BR><BR>' % (contsubResults)
    if (len(lineResults) > 0):
        webpage += '<a href="%s">Link to line results page</a><BR><BR>' % (lineResults)
    if (len(contResults) > 0):
        webpage += '<a href="%s">Link to continuum results page</a><BR><BR>' % (contResults)
    webpage += '<a href="%s">Link to extra results file</a><BR><BR>' % (os.path.basename(acsTxt))
    f = open(webfile,'w',0)
    f.write(webpage)
    f.close()
    os.system('chmod g+w ' + webfile)

    totalTime = 0
    txt = open(acsTxt,'w')
    txt.close()
    os.system('chmod g+w ' + webpagedir+os.path.basename(acsTxt))

    for i in range(len(pipelineImages)):
        # keep webarea=False to avoid double insertion of that directory
        result = analyzemsimage(pipelineImages[i], manualImages[i], acsTxt, pblimit,
                                automask, mom0sigma, profileSigma, findcontVerbose, 
                                overwriteImport, overwriteSpecsmooth, overwriteRegrid,
                                overwriteMask, overwriteFindContinuum,
                                appendhtml=True, webfile=webfile, webarea=False,
                                imagedir=imagedir, verbose=verbose, 
                                minmaxScale=minmaxScale, regriddedColumn=regriddedColumn,
                                imageRadius=imageRadius, bins=bins) 
        if result==None: return
        totalTime += result
    finishWebpage(webfile)
    print "Total elapsed time = %.1f sec = %.1f min" % (totalTime, totalTime/60.)

def escapeSlash(s):
    """
    Protect all appearances of forward slash, minus, and plus symbol with a backslash.
    """
#    return(s.replace('/','\/').replace('-','\-').replace('+','\+'))
    return('"'+s+'"')

def imstatAnnulus(image, pbimage='', innerLevel=0.5, outerLevel=0.3, verbose=False, perChannel=False):
    """
    Computes the imstat statistics on an annular region of an image as specified
    by the primary beam response level.  If the primary beam image is not specified,
    then it will try to find it with the following suffixes:  
           .pb,  .pb.tt0,  .flux,  .flux.pbcoverage
    """
    if verbose:
        global startTime
        startTime = time.time()
    if innerLevel < outerLevel:
        swap = innerLevel
        innerLevel = outerLevel
        outerLevel = swap
        print "Swapped the order of the levels"
    if (pbimage == ''):
        if (image.find('tt0') > 0):
            pbimage = image.replace('.image.tt0','.pb.tt0')
        else:
            pbimage = image.replace('.image','.pb')
        if not os.path.exists(pbimage):
            pbimage = image.replace('.image','.flux.pbcoverage')
            if not os.path.exists(pbimage):
                pbimage = image.replace('.image','.flux')
                if not os.path.exists(pbimage):
                    print "Could not infer the name of the pbimage"
                    return
        print "Found pbimage = ", pbimage
    elif (not os.path.exists(pbimage)):
        print "Could not find pbimage"
        return
    if perChannel:
        axis = findSpectralAxis(image)
        myshape = imhead(image, mode='get', hdkey='shape')
        axes = range(len(myshape))
        axes.remove(axis)
    else:
        axes = -1
    return runImstat(image, pbimage, level=outerLevel, innerAnnulusLevel=innerLevel, verbose=verbose, axes=axes)

def runImstat(Image, PB, Mask=False, channels='', region='', level=0.5, 
              innerAnnulusLevel=None, location=-1, verbose=True, pbMin=None,
              MaskFlattened=False, imageHead=None, maskHead=None, insideMask=False, axes=-1):
    """
    Runs imstat on an image, applying a mask based on the primary beam response
    and/or another mask image, over all channels or specific channel selection.
    PB: False, or the name of a primary beam image
    Mask: False, or the name of a mask image;  if specified, then use only pixels 
          outside of this mask (i.e. with value=0 meaning use the uncleaned area)
    insideMask: if True, then use values inside the mask (1); False => outside (0)
    Note: PB and Mask images should be of the same dimensionality (incl. number of channels)
    channels: passed to the chans parameter of imstat
    region: passed to the region parameter of imstat
    level: if PB is not False, then use pixels above this response level
    innerAnnulusLevel: if PB is not False, then use this level to define the
                       inner radius of an annulus from which to select pixels
    location: integer used for debugging where in the method this call originated
    pbMin: minimum value in the PB image
    axes: passed to imstat
    """
    if verbose:
        print "Running imstat %d on " % (location), os.path.basename(Image)
    # level only matters if PB exists
    if PB:
        if pbMin == None:
            # Sometimes the manual image is small and does not extend very far into the PB
            if verbose: showElapsedTime("imstat on PB")
            pbMin = imstat(PB)['min']
    if (PB and Mask):
        if (numberOfChannelsInCube(PB) == 1):
            Mask = MaskFlattened
        if (Mask):
            if (insideMask):
                mask = '%s>%g && %s>0.5' % (escapeSlash(PB), level, escapeSlash(Mask))
            else:
                mask = '%s>%g && %s<0.5' % (escapeSlash(PB), level, escapeSlash(Mask))
        else:
            mask = '%s>%g' % (escapeSlash(PB), level)
        if verbose: 
            print "Using PB mask and clean mask: imstat('%s', mask='%s', stretch=True)" % (Image,mask)
            showElapsedTime("imstat")
        Stats = imstat(Image, mask=mask, stretch=True, axes=axes) 
    elif PB:
        if (innerAnnulusLevel == None):
            mask = '%s>%g' % (escapeSlash(PB),level)
#            if (imageHead != None):
#                print "shape(img)=%s" % (imageHead['shape'])
#            if (maskHead != None):
#                print "shape(mask)=%d" % (maskHead['shape'])
            if verbose: 
                print "Using PB mask: imstat('%s', mask='%s', chans='%s')" % (Image, mask, channels)
                showElapsedTime("imstat")
            Stats = imstat(Image, mask=mask, chans=channels, stretch=True, axes=axes) # added stretch on 9-17-2016
        else:
            if (pbMin > innerAnnulusLevel):
                print "Image is not large enough to reach the specified PB level: ", Image
                print "Are you sure this is the science target, and not a calibrator image?"
                Stats = ''
            else:
                mask = '%s>%g && %s<%g'%(escapeSlash(PB),level,escapeSlash(PB),innerAnnulusLevel)
                if verbose: 
                    print "Using PB annular mask: imstat('%s', mask='%s', chans='%s', stretch=True)" % (Image, mask, channels)
                    showElapsedTime("imstat")
                Stats = imstat(Image, mask=mask, chans=channels, stretch=True, axes=axes)
    elif Mask:
        if insideMask:
            mask = '%s>0.5'%(escapeSlash(Mask))
        else:
            mask = '%s<0.5'%(escapeSlash(Mask))
        if verbose: 
            print "Using clean mask: imstat('%s', mask='%s', chans='%s', region='%s')" % (Image, mask, channels, region)
            showElapsedTime("imstat")
        Stats = imstat(Image, mask=mask, region=region, chans=channels, axes=axes)
    else:
        if verbose: 
            print "Using region,chans: imstat('%s', region='%s', chans='%s')" % (Image, region, channels)
            showElapsedTime("imstat")
        Stats = imstat(Image, region=region, chans=channels, axes=axes) #  regionName and chans and/or stokes cannot be specified simultaneously              
    if verbose: 
        showElapsedTime("done imstat")
    return Stats
#        pipelineRawRegions = buildRegionsFile(pipelineRawImage, pipelineRawContChannels, pipelineSpecAxis)
#        pipelineRawRegions = buildMask(pipelineRawImage, pipelineRawContChannels, radius, pipelineRawHead, overwriteMask)
#
# The following fails with error message: Overriding spectral specification must be limited to a sngle channel range.
#        pipelineRawStats = imstat(pipelineRawImage, chans=pipelineRawContChannels, region=region)  

def imageCircularMaskRadius(maskImage, header=None, channels=1):
    """
    Compute the radius of a mask (in arcsec) under the assumption that all the 
    pixels (with specified value) are in one circular patch.   This 
    is mainly to empirically confirm what the C3R3 pipeline 
    used as its clean mask.
    Returns: value in arcsec
    -Todd Hunter
    """
    if (header != None):
#        header = getHeader(maskImage)
        arcsecPerPixel = np.degrees(header['cdelt2'])*3600
    else:
        arcsecPerPixel = getFitsBeam(maskImage, omitBeam=True)[4]
    npixels = imagePixelsInMask(maskImage)/channels
    return((npixels*arcsecPerPixel**2/np.pi)**0.5)

def imageCircularMaskLevel(maskImage, channels=1, roundFigures=2):
    """
    Compute the primary beam level cutoff for a mask under the assumption that 
    all the pixels (with specified value) are in one circular patch. This is mainly 
    to empirically confirm what the C3R3 pipeline used as its clean mask.
    header: an imhead dictionary for an associated image (default: run imhead on the mask)
    roundFigures: number of significant digits to return
    -Todd Hunter
    """
    arcsec = imageCircularMaskRadius(maskImage, channels=channels)
    fwhm = imagePrimaryBeam(maskImage, returnRadiusInPixels=False)
    pblevel = gaussianBeamResponse(arcsec, fwhm=fwhm)
    return float(roundFiguresToString(pblevel,roundFigures))

def imagePixelsInImageMask(img, validValue=True):
    """
    Counts the number of pixels in the mask0 table of a CASA image.
    """
    pixels = 0
    if (os.path.exists(img)):
        if (img.find('/mask0') < 0):
            img += '/mask0'
        mytb = createCasaTool(tbtool)
        mytb.open(img)
        data = mytb.getcol('PagedArray')
        mytb.close()
        pixels = len(np.where(data!=validValue)[0])
        total = len(data.flatten())
    else:
        print "Could not find image"
        return
    print "%d/%d = %.2f%% (unmasked=%d)" % (pixels, total, 100*pixels/total, total-pixels)
    return pixels
    
def imagePixelsInMask(maskImage, validValue=1):
    """
    Counts the number of pixels with a non-zero value in a clean mask image.
    """
    pixels = 0
    if (maskImage != False):
        if (os.path.exists(maskImage)):
            shape = imhead(maskImage, mode='get', hdkey='shape')
            axis = findSpectralAxis(maskImage)
            myia = createCasaTool(iatool)
            myia.open(maskImage)
            channels = shape[axis]
            pixels = 0
            myrg = createCasaTool(rgtool)
            for channel in range(channels):
                myregion = myrg.box(blc=[0,0,0,channel], trc=[shape[0],shape[1],0,channel])
                data = myia.getregion(region=myregion)
                pixels += len(np.where(data == validValue)[0])
            myrg.done()
            myia.close()
        else:
            print "Could not find maskImage"
        print "%d/%d = %.2f%%" % (pixels, np.prod(shape), 100*pixels/np.prod(shape))
    return pixels

def readContDat(contDat):
    """
    Reads a summary text file created by findContinuum.py
    """
    f = open(contDat,'r')
    selection, png, bw = f.readlines()[0].strip('\n').split()
    print "Read %s: " % (contDat), selection, png, float(bw)
    return(selection, png, float(bw))

def rreplace(s, old, new='', occurrence=1):
    """
    Replaces the final n appearances of <old> in <s> with <new>.
    Giving simply the first 2 arguments is equivalent to rstrip 
    but only removing the final appearance of <old>.
    e.g. rreplace('a.image.image','.image','.model') -> a.image.model
    """
    li = s.rsplit(old, occurrence)
    return new.join(li)

def checkBenchmarkProductsPresent(manualImage):
    benchmarks =     [os.path.exists(manualImage+'.MOM0')]
    benchmarks.append(os.path.exists(manualImage+'.MOM0.LINEFREE'))
    benchmarks.append(os.path.exists(manualImage+'.PEAKMOM'))
    benchmarks.append(os.path.exists(manualImage+'.PEAKMOM.LINEFREE'))
    benchmarks.append(os.path.exists(manualImage+'.meanSpectrum'))
    benchmarks.append(os.path.exists(manualImage+'_findContinuum.dat'))
    howmany = np.sum(benchmarks)
    if (howmany == len(benchmarks)):
        print "*** Found the manual benchmark products. Will not recreate moment images or meanSpectrum. ****"
        return True
    elif (howmany > 0):
        print "*** Found some (%d/%d), but not all of the manual benchmark products. ***" % (howmany,len(benchmarks))
        return -1
    else:
        return False

def lookForFile(png):
    """
    Looks for a file matching a wildcard and returns the newest one
    """
    a = glob.glob(png)
    if (len(a) == 0):
        return ''
    else:
        a = max(glob.iglob(png), key=os.path.getctime)
        return a

def lookForNewestFileContainingStrings(name, contents, verbose=False, skip=0):
    """
    Takes a file path containing wildcards, and a list of content strings,
    and returns the newest matching file that contains all the strings.
    name: file path, e.g. '/users/casapy*log'
    contents: list of strings, e.g ['Begin Task: tclean','myimage']
    """
    names = sorted(glob.glob(name), reverse=True)
    if verbose: print "possible files:", names
    if (type(contents) != list):
        contents = [contents]
    for n in names[skip:]:
        i = 0
        if verbose: print "Searching ", n
        for content in contents:
            if (len(grep(n, content)[0]) > 0):
                if verbose: print " Found ", content
                i += 1
            elif verbose: print ' did not find "%s"' % (content)
        if (i == len(contents)):
            return n
    return

def computeElapsedTime():
    seconds = time.time()-startTime
    return('%02dm%02ds ' % (int(seconds/60),int(seconds-60*int(seconds/60))))

def getPipelinePBAndMask(pipelineImage):
    if (pipelineImage.find('.image.tt0') > 0):
#        pipelineRawPB = rreplace(pipelineImage,'.image.tt0')+'.pb'
        print "Using C4R2 name for primary beam: .pb.tt0"
        pipelineRawPB = rreplace(pipelineImage,'.image.tt0')+'.pb.tt0'  # C4R2
        pipelineRawMask = rreplace(pipelineImage,'.image.tt0')+'.mask'
    else:
        pipelineRawPB = rreplace(pipelineImage,'.image')+'.pb'
        pipelineRawMask = rreplace(pipelineImage,'.image')+'.mask'
    return(pipelineRawPB, pipelineRawMask)

def getNiterFromNewestLog(logdir, parameter='', verbose=True):
    """
    Gets the number of tclean iterations completed on an image from the
    casa log.
    logdir: directory containing casapy*.log files
    parameter: the tclean imagename argument
    Returns:
    the integer number of iterations, and the name of the log file it came from
    """
    if (not os.path.exists(logdir)):
        print "log dir does not exist: ", logdir
        return -1
    logwildcard = logdir + '/casapy*log'
    maxFiles = len(glob.glob(logwildcard))
    parameterEscapeChars = parameter.replace('[','\[').replace(']','\]')
    if verbose:
        print "Calling analyzemsimage.lookForNewestFileContainingStrings('%s', ['%s','Begin Task: tclean'])" % (logwildcard,parameterEscapeChars)
    log = lookForNewestFileContainingStrings(logwildcard, [parameterEscapeChars,'Begin Task: tclean'])
        
    iterations = -1
    if (log != None):
        skip = 0
        while (iterations == -1 and skip < maxFiles):
            print "Using log = ", log
            print "Running au.taskExecutionTimes('tclean','%s',showTimes=False,extraPhrase='mergeCycleExecutionRecord',parameter='%s', getIterations=True)" % (log, parameter)
            iterations = taskExecutionTimes('tclean', log, showTimes=False, 
                                            extraPhrase='mergeCycleExecutionRecord',
                                            parameter=parameter, getIterations=True, worldWrite=True)
            print "found iterations = ", iterations
            if iterations == -1:
                skip += 1
                log = lookForNewestFileContainingStrings(logwildcard, [parameter,'Begin Task: tclean'], skip=skip)
    return iterations, log
        
def analyzemsimage(pipelineImage, manualImage, acsTxt, pblimit=0.5, cleanMask='',
                   automask=False, mom0sigma=2.0, profileSigma=3.0, 
                   findcontVerbose=False, overwriteImport=False,
                   overwriteSpecsmooth=False, overwriteRegrid=False,
                   overwriteMask=False, overwriteFindContinuum=False,
                   appendhtml=False, webfile='', webarea=False, 
                   imagedir='',verbose=False, minmaxScale='manual',
                   regriddedColumn=-1, imageRadius=0, bins=40,
                   modelColormap = 'Misc. 2: Topography'): 
    """
    Compares two images (or cubes) and generates an html page with
    moment images and statistics.  If it is of finer channel spacing,
    the first image is regridded spectrally to match the second image.
    Inputs:
    pipelineImage: the image of an spw generated by the pipeline
    manualImage: the corresponding image of the same spw generated by a human
    acsTxt: name of text file that contains extra results to be appended to
    webfile: full path to write the webfile (and accompanying images)
    webarea: True: treat webfile as a relative directory, and precede it with
                   /home/safe.nrao.edu/content/alma/PipelineTestResults/
             Note: this should be set False when called from analyzemsimages
    imagedir: directory to write new images, default: current working directory
    appendhtml: if True, then append rather than overwrite to the webfile
    pblimit: size of central region to consider in units of primary beam response
    imageMask: if specified, avoid using pixels from the clean mask in statistics
    automask: if False, then compute statistics over central primary beam (set by pblimit)
              if True, then use recipes.automask of moment8 (not yet implemented)
    profileSigma: cutoff to use in generating spectra (in units of sigma)
    overwriteImport: if True, and image is FITS, then overwrite CASA image if it also exists
    overwriteSpecsmooth: if True, the regenerate specsmoothed pipeline image if present
    overwriteRegrid: if True, the regenerate the spatially regridded pipeline image if present
    overwriteFindContinuum: if True, then rerun findContinuum even if *_findContinuum.dat exists
    minmaxScale: 'manual' or 'pipeline'; sets the min/max colorbar of residual image
    regriddedColumn: which column to place the pipelineRegridded result (2 or 3)
                     <=0 means don't show it, and don't bother running specsmooth
    imageRadius: radius of image (in arcsec) to display as png
    Returns:
    * execution time in seconds
    """
    global startTime
    if (startTime == 0):
        startTime = time.time()
    singleStartTime = time.time()
    webpagedir = os.path.dirname(webfile) + '/'
    if webfile[-1] == '/':
        print "You must specify a webfile that does not end in /"
        return
    if webarea:
        webpagedir = '/home/safe.nrao.edu/content/alma/PipelineTestResults/' + webpagedir
    if (webpagedir == '/'): 
        webpagedir = ''
    elif (not os.path.exists(webpagedir)):
        print "----- Making web subdirectories:  %s -----" % (webpagedir)
        os.makedirs(webpagedir)

    if (not os.path.exists(imagedir) and imagedir != ''):
        print "----- Making image subdirectories:  %s -----" % (imagedir)
        os.makedirs(imagedir)
    if (imagedir != ''):
        if (imagedir[-1] != '/'): imagedir += '/'

    # Check if cubes are FITS, and if so, import them into CASA images and use them going forward
    pipelineImage = importIfFITS(pipelineImage, overwriteImport)
    manualImage = importIfFITS(manualImage, overwriteImport)

    benchmarkProductsPresent = checkBenchmarkProductsPresent(manualImage)
    if benchmarkProductsPresent == -1 and False: 
        print "*** Exiting the script so you can figure out what is wrong. ****"
        return time.time()-startTime

    # Check that both images are of the same dimensionality
    if verbose: print "Reading nchan from ", pipelineImage
    # Getting the header information from the original images
    pipelineRawImage = pipelineImage
    pipelineRawHead = getHeader(pipelineRawImage)
    pipelineRawChannels, pipelineRawChanWidth = numberOfChannelsInCube(pipelineRawImage, returnChannelWidth=True, header=pipelineRawHead)
    manualHead = getHeader(manualImage)
    manualChannels, manualChanWidth = numberOfChannelsInCube(manualImage, returnChannelWidth=True, header=manualHead)
    if (manualChannels == 1 and pipelineRawChannels == 1):
        cube = False
        nonEdgeChannels = ''
        pipelineRawNonEdgeChannels = ''
        manualNonEdgeChannels = ''
    elif (manualChannels > 1 and pipelineRawChannels > 1):
        cube = True
        manualNonEdgeChannels = '1~%d' % (manualChannels-2)
        pipelineRawNonEdgeChannels = '1~%d' % (pipelineRawChannels-2)
    else:
        print "Mismatch between pipeline and manual image: only one is a cube."
        return

    if (not appendhtml):
        if webfile == '':
            htmlfile = "%s_compared_to_%s.html" % (os.path.basename(pipelineImage), os.path.basename(manualImage))
            webfile = webpagedir + htmlfile
        directoriesAndVersions = ''
        directoriesAndVersions += 'ami version = %s<br>\n' % (version())
        directoriesAndVersions += 'findContinuum version = %s<br>\n' % (findContinuum.version())
        directoriesAndVersions += 'Pipeline image = %s<br>\n' % (pipelineImage)
        directoriesAndVersions += 'Manual image = %s<br>\n' % (manualImage)
        amiVersion = getCvsVersion(version())
        py0,py1,py2,py3,py4 = sys.version_info
        commandLine = "analyzemsimage.analyzemsimage(pipelineImage='%s', manualImage='%s', webfile='%s', automask=%s, mom0sigma=%.1f)" % (pipelineImage, manualImage, webfile, automask, mom0sigma)

        webpage = initializeWebpage(directoriesAndVersions, os.getenv('USER'), os.getenv('HOST'), 
                                    str(datetime.datetime.now()), 
                                    casadef.casa_version+' r'+casadef.subversion_revision, 
                                    py0,py1,py2,py3,py4,os.getcwd(), commandLine, webpagedir)
    else:
        webpage = ''

    pipelineSpecAxis = findSpectralAxis(pipelineImage)
    manualSpecAxis = findSpectralAxis(manualImage)
    pl_cdelt3 = 'cdelt%d' % (1+pipelineSpecAxis)
    man_cdelt3 = 'cdelt%d' % (1+manualSpecAxis)

    if (os.access(manualImage,os.W_OK)):
        # fixing up the manual image to be listed as ICRS
        if (manualHead['equinox'] != 'ICRS'):
            print "imhead put ICRS",
            imhead(imagename=manualImage, mode='put', hdkey='equinox', hdvalue='ICRS')

    bmaj,bmin,bpa,cdelt1,cdelt2,naxis1,naxis2,freqGHz = getFitsBeam(pipelineImage)
    primaryBeam = primaryBeamArcsec(frequency=freqGHz, showEquation=False)
    if verbose: print "primary beam = %.1f arcsec for %.3f GHz" % (primaryBeam, freqGHz)
    radius = primaryBeam*gaussianBeamOffset(pblimit)
    Ppblimit  = 0.2
    Pradius = primaryBeam*gaussianBeamOffset(Ppblimit)
    PprimaryBeam = 2*Pradius
    Ppblimit2 = 0.3
    Pradius2 = primaryBeam*gaussianBeamOffset(Ppblimit2)
    PprimaryBeam2 = 2*Pradius2
    print "pipelineImage = ", pipelineImage
    pipelineRawPB, pipelineRawMask = getPipelinePBAndMask(pipelineImage)
    pbLevels = [pblimit] # [Ppblimit, Ppblimit2, pblimit]
    maskLevels = [0.5]
    # Check for .pb and .mask images
    manualPB = rreplace(manualImage,'.image','.pb')
    manualMask = rreplace(manualImage,'.image','.mask')
    if (benchmarkProductsPresent):
        manualFlattenedMask = rreplace(manualImage, '.image','.flattenmask')
    else:
        manualFlattenedMask = manualMask + '.flattened'
    if (os.path.exists(manualMask)):
        if (not os.path.exists(manualFlattenedMask) and cube):
            manualFlattenedMask = flattenMask(manualMask)
    else:
        manualMask = False
    if cube:
        # Run findContinuum on the pipeline-generated full-resolution cube to 
        # identify line-free channels to get statistics
        contDat = findContDat(pipelineImage)
        if (not os.path.exists(contDat) or overwriteFindContinuum):
            pipelineRawContChannels, pipelineRawFindContPng, plRawAggregateBandwidth = \
                findContinuum.findContinuum(pipelineImage, centralArcsec=radius,
                                            verbose=findcontVerbose, alternateDirectory='')
        else:
            print "Using existing _findContinuum.dat for pipelineRawImage"
            pipelineRawContChannels, pipelineRawFindContPng, plRawAggregateBandwidth = readContDat(contDat)
        pl_raw_meanSpectrumFile = findContinuum.buildMeanSpectrumFilename(pipelineImage, meanSpectrumMethod='auto', 
                                                                          peakFilterFWHM=10)
        pipelineRawLineChannels = invertChannelRanges(pipelineRawContChannels, pipelineRawChannels)
    
        # Regrid spectrally to the manual image
        chan_bin = int(np.ceil(manualHead[man_cdelt3]/pipelineRawHead[pl_cdelt3]))
        # if the channel binning is greater than one, then smooth the spectral axis.
        specsmoothTime = 0
        if (chan_bin > 1):
          if (regriddedColumn > 0):
            pipelineImage_chanbin = imagedir + os.path.basename(pipelineImage)+'.specsmooth'
            if (overwriteSpecsmooth or not os.path.exists(pipelineImage_chanbin)):
                if (not os.path.exists(pipelineImage_chanbin)):
                    print "Did not find ", pipelineImage_chanbin
                print "Running specsmooth(width=%d)" % (chan_bin)
                sstime = time.time()
                specsmooth(imagename=pipelineImage, outfile=pipelineImage_chanbin, 
                           function='boxcar', width=chan_bin, dmethod='copy', 
                           overwrite=True)
                specsmoothTime += time.time()-sstime
                pipelineImage = pipelineImage_chanbin

            pipelineImage_chanbinPB = imagedir + os.path.basename(pipelineImage)+'.specsmooth.pb'
            if (os.path.exists(pipelineRawPB) and (overwriteSpecsmooth or not os.path.exists(pipelineImage_chanbinPB))):
                print "Running specsmooth(width=%d) on pb" % (chan_bin)
                sstime = time.time()
                specsmooth(imagename=pipelineRawPB, outfile=pipelineImage_chanbinPB, 
                           function='boxcar', width=chan_bin, dmethod='copy', 
                           overwrite=True)
                specsmoothTime += time.time()-sstime
                pipelineRawPB = pipelineImage_chanbinPB

#            pipelineImage_chanbinMask = imagedir + os.path.basename(pipelineImage)+'.specsmooth.mask'
#            if (os.path.exists(pipelineRawMask) and (overwriteSpecsmooth or not os.path.exists(pipelineImage_chanbinMask))):
#                print "Running specsmooth(width=%d) on mask" % (chan_bin)
#                specsmooth(imagename=pipelineRawMask, outfile=pipelineImage_chanbinMask, 
#                           function='boxcar', width=chan_bin, dmethod='copy', 
#                           overwrite=True)
#                pipelineRawMask = pipelineImage_chanbinMask
          pipelineRawMask = pipelineRawPB + '>0'

        contDat = manualImage + '_findContinuum.dat'
        # look for meanSpectrum and if there is a mismatch of the mask parameter, then set overwriteFindContinuum=True
        if (benchmarkProductsPresent):
            man_meanSpectrumFile = lookForFile(manualImage+'.meanSpectrum*png')
            overwriteFindContinuumManual = False
        else:
            man_meanSpectrumFile =  findContinuum.buildMeanSpectrumFilename(manualImage, meanSpectrumMethod='auto', 
                                                                          peakFilterFWHM=10)
            overwriteFindContinuumManual = overwriteFindContinuum
        if (os.path.exists(man_meanSpectrumFile) and not benchmarkProductsPresent):
            if (findContinuum.maskArgumentMismatch(manualMask, man_meanSpectrumFile)):
                overwriteFindContinuumManual = True
        if (not os.path.exists(contDat) or overwriteFindContinuumManual):
            manualContChannels, manualFindContPng, manAggregateBandwidth = \
                findContinuum.findContinuum(manualImage, centralArcsec=radius, mask=manualMask,
                                            verbose=findcontVerbose, alternateDirectory='')
        else:
            print "Using existing _findContinuum.dat for manualImage"
            manualContChannels, manualFindContPng, manAggregateBandwidth = readContDat(contDat)
            if len(os.path.dirname(contDat)) > 0:
                manualFindContPng = os.path.dirname(contDat) + '/' + os.path.basename(manualFindContPng)
            if verbose: print "manualFindContPng = ", manualFindContPng
        manualLineChannels = invertChannelRanges(manualContChannels, manualChannels)
    else:
        manualLineChannels = ''
        manualContChannels = ''
        pipelineRawLineChannels = ''
        pipelineRawContChannels = ''

    # Spatial regridding (and channel truncation) to the manual image
    if (regriddedColumn > 0):
        pipelineTemporaryImage = pipelineImage
        pipelineImage = imagedir + os.path.basename(pipelineImage)+'.regridded'
        if (overwriteRegrid or not os.path.exists(pipelineImage)):
            print "Regridding pipelineImage"
            regrid_okay = imregrid(imagename=pipelineTemporaryImage, template=manualImage, 
                                   output=pipelineImage, axes=[-1], asvelocity=False,
                                   overwrite=True,interpolation='cubic')
            if not regrid_okay:
                print "Spatial regrid failed. Check to make sure your initial image selection overlaps."
                return
        else:
            print "Using existing regridded pipeline image"

    pipelinePB, pipelineMask = getPipelinePBAndMask(pipelineImage)
    if (regriddedColumn > 0):
        if (os.path.exists(pipelineRawPB) and (overwriteRegrid or not os.path.exists(pipelinePB))):
            print "Regridding pipeline primary beam image"
            regrid_okay = imregrid(imagename=pipelineRawPB, template=manualImage, 
                                   output=pipelinePB, axes=[-1], asvelocity=False,
                                   overwrite=True,interpolation='cubic')
            if not regrid_okay:
                print "Spatial regrid of PB image failed. Check to make sure your initial image selection overlaps."
                return

    if False:
        pipelineMask = rreplace(pipelineImage,'.image')+'.mask'
        if (os.path.exists(pipelineRawMask) and (overwriteRegrid or not os.path.exists(pipelineMask))):
            print "imregrid('%s')" % (pipelineRawMask)
            regrid_okay = imregrid(imagename=pipelineRawMask, template=manualImage, 
                                   output=pipelineMask, axes=[-1], asvelocity=False,
                                   overwrite=True,interpolation='cubic')
            if not regrid_okay:
                print "Spatial regrid of Mask image failed. Check to make sure your initial image selection overlaps."
                return
    if (regriddedColumn <= 0):
        pipelineHead = pipelineRawHead
    else:
        pipelineHead = getHeader(pipelineImage)
    pipelineChannels, pipelineChanWidth = numberOfChannelsInCube(pipelineImage, returnChannelWidth=True, header=pipelineHead)
    if cube:
        # Run findContinuum on the (regridded) pipeline image
        contDat = pipelineImage + '_findContinuum.dat'
        if (not os.path.exists(contDat) or overwriteFindContinuum):
            pipelineContChannels, pipelineFindContPng, plAggregateBandwidth = \
                   findContinuum.findContinuum(pipelineImage, centralArcsec=radius, 
                                               verbose=findcontVerbose, alternateDirectory='')
        else:
            print "Using existing _findContinuum.dat for pipelineImage"
            pipelineContChannels, pipelineFindContPng, plAggregateBandwidth = readContDat(contDat)
        pl_meanSpectrumFile = findContinuum.buildMeanSpectrumFilename(pipelineImage, meanSpectrumMethod='auto', 
                                                                          peakFilterFWHM=10)
        pipelineLineChannels = invertChannelRanges(pipelineContChannels, pipelineChannels)
    else:
        pipelineLineChannels = ''
        pipelineContChannels = ''

    # Check for .pb and .mask images
    print "Running imstat('%s', chans='%s')" % (pipelineRawImage, pipelineRawNonEdgeChannels)
    showElapsedTime("imstat")
    pipelineRawWholeStats = imstat(pipelineRawImage, chans=pipelineRawNonEdgeChannels)

    # every following imstat will use a clean mask (if it exists), otherwise 50% PB area
    if (not os.path.exists(pipelineRawMask)):
        print "Did not find ", pipelineRawMask
        pipelineRawMask = False

    pipelineRawModelImage = rreplace(pipelineRawImage,'.image','.model')
    pipelineRawResidualImage = rreplace(pipelineRawImage,'.image','.residual')
    if (os.path.exists(pipelineRawPB)):  
#        pipelineRawAllChanStats = runImstat(pipelineRawImage, pipelineRawPB, level=Ppblimit, innerAnnulusLevel=Ppblimit2)
        print "Run imhead('%s',mode='list')" % (pipelineRawPB)
        showElapsedTime("imhead on raw PB")
        pipelineRawPBhead = imhead(pipelineRawPB,mode='list')
        print "runImstat('%s', '%s', level=%f, innerAnnulusLevel=%f" % (pipelineRawImage, pipelineRawPB, Ppblimit, Ppblimit2)
        pipelineRawAllChanPStats = runImstat(pipelineRawImage, pipelineRawPB, level=Ppblimit, 
                                             innerAnnulusLevel=Ppblimit2, location=0, 
                                             pbMin=pipelineRawPBhead['datamin'])
        pipelineRawAllChanPBStats = runImstat(pipelineRawImage, pipelineRawPB, level=pblimit, 
                                             location=1, pbMin=pipelineRawPBhead['datamin']) 
        pipelineRawResidualAllChanPStats = runImstat(pipelineRawResidualImage, pipelineRawPB, level=Ppblimit, 
                                                     innerAnnulusLevel=Ppblimit2, location=10, 
                                                     pbMin=pipelineRawPBhead['datamin'])
        pipelineRawResidualAllChanPBStats = runImstat(pipelineRawResidualImage, pipelineRawPB, level=pblimit, 
                                                     location=11, pbMin=pipelineRawPBhead['datamin']) 
    else:
        print "A) Need to implement computePBAnnulus since pipeline primary beam not found = ", pipelineRawPB
        pipelineRawPB = False
        return
#        regionAllChans, regionContChans = computePBCircle(pipelineRawHead, radius, pipelineRawContChannels)
#        pipelineRawAllChanStats = runImstat(pipelineRawImage, pipelineRawPB, region=regionAllChans,
#                                            pbMin=pipelineRawPBhead['datamin'],location=0)
#        pipelineRawAllChanPBStats = pipelineRawAllChanStats
#        regionAllPChans, regionContPChans = computePBCircle(pipelineRawHead, Pradius, pipelineRawContChannels)
#        pipelineRawAllChanPStats = runImstat(pipelineRawImage, pipelineRawPB, region=regionAllPChans,
#                                             pbMin=pipelineRawPBhead['datamin'],location=1)
#        pipelineRawAllChanPBStats = runImstat(pipelineRawImage, pipelineRawPB, level=pblevel, location=10)
#        pipelineRawResidualAllChanPStats = runImstat(pipelineRawResidualImage, pipelineRawPB, region=regionAllPChans,
#                                                     pbMin=pipelineRawPBhead['datamin'],location=11)
#        pipelineRawResidualAllChanPBStats = runImstat(pipelineRawResidualImage, pipelineRawPB, region=regionAllPChans, 
#                                                      level=pblimit, location=12)
    if (pipelineRawAllChanPStats == ''):
        print "Stopping so you can fix the launching script."
        return

    if cube:
#        pipelineRawStats = runImstat(pipelineRawImage, pipelineRawPB, pipelineRawMask, 
#                                     pipelineRawContChannels, region=regionContChans, 
#                                     level=pblimit)
        pipelineRawPStats = runImstat(pipelineRawImage, pipelineRawPB, 
                                      channels=pipelineRawContChannels, 
                                      level=Ppblimit, innerAnnulusLevel=Ppblimit2, location=2,
                                      pbMin=pipelineRawPBhead['datamin'])
        pipelineRawImageMom0,pipelineRawImageMom0Head = buildMomentImage(pipelineRawImage, 0, webpagedir, 
                                                                         chans=pipelineRawLineChannels,loc=0)
        pipelineRawImageMom0sigma,pipelineRawImageMom0sigmaHead = buildMomentImage(pipelineRawImage, 0, webpagedir,
                                                                                   mom0sigma*pipelineRawPStats['sigma'], 
                                                                                   suffix='threshold',
                                                                                   chans=pipelineRawLineChannels,loc=1)
        pipelineRawImageMom8,pipelineRawImageMom8Head = buildMomentImage(pipelineRawImage, 8, webpagedir,
                                                                         chans=pipelineRawLineChannels,loc=2) 
    else:
        showElapsedTime("imstat")
        pipelineRawContModelStats = imstat(pipelineRawImage)
        showElapsedTime("imstat")
        pipelineRawImageStats = imstat(pipelineRawImage)
        showElapsedTime("imstat")
        pipelineRawContResidualStats = imstat(pipelineRawResidualImage)

    showElapsedTime("imstat") # no mask, all channels
    pipelineWholeStats = imstat(pipelineImage, chans=manualNonEdgeChannels)
    pipelineModelImage = rreplace(pipelineImage,'.image','.model')
    pipelineResidualImage = rreplace(pipelineImage,'.image','.residual')
#    pipelineMask = rreplace(pipelineImage,'.image','.mask')
#    if (not os.path.exists(pipelineMask)):
#        pipelineMask = False
    if (os.path.exists(manualPB)):
        showElapsedTime("imhead on manual PB")
        manualPBhead = imhead(manualPB,mode='list')
    if (os.path.exists(pipelinePB)):
#        pipelineAllChanStats = runImstat(pipelineImage, pipelinePB, level=pblimit) # should not include mask because it is wider than 50%
        if (os.path.exists(manualPB)):
            # Will typically fail because pipeline image pixel size is not identical as manual image.
#            pipelineAllChanStats = runImstat(pipelineImage, manualPB, manualMask, level=pblimit, location=30,
#                                             pbMin=manualPBhead['datamin'],MaskFlattened=manualFlattenedMask) 
            pipelineAllChanStats = ''
        else:
            pipelineAllChanStats = ''
        showElapsedTime("imhead on PB")
        pipelinePBhead = imhead(pipelinePB, mode='list')
        if (not cube):
            pipelineStatsInMask = runImstat(pipelineImage, pipelinePB, level=Ppblimit2, location=20, insideMask=True)
        pipelineAllChanPStats = runImstat(pipelineImage, pipelinePB, level=Ppblimit, 
                                          innerAnnulusLevel=Ppblimit2, location=3,
                                          pbMin=pipelinePBhead['datamin'])  # area between 20%-30%
        pipelineAllChanPBStats = runImstat(pipelineImage, pipelinePB, level=pblimit, 
                                          location=30, pbMin=pipelinePBhead['datamin'])
        pipelineResidualAllChanPStats = runImstat(pipelineResidualImage, pipelinePB, level=Ppblimit, 
                                          innerAnnulusLevel=Ppblimit2, location=3,
                                          pbMin=pipelinePBhead['datamin'])  # area between 20%-30%
    else:
        pipelinePB = False
        pipelineAllChanPStats = ''
        print "B) Need to implement computePBAnnulus"
        return
        regionAllChans, regionContChans = computePBCircle(pipelineHead, radius, pipelineContChannels)
        pipelineAllChanStats = runImstat(pipelineImage, pipelinePB, region=regionAllChans,
                                         pbMin=pipelinePBhead['datamin'])
        regionAllPChans, regionContPChans = computePBCircle(pipelineHead, Pradius, pipelineContChannels)
        pipelineAllChanPStats = runImstat(pipelineImage, pipelinePB, region=regionAllPChans,
                                          pbMin=pipelinePBhead['datamin'])
        pipelineResidualAllChanPStats = runImstat(pipelineResidualImage, pipelinePB, region=regionAllPChans,
                                          pbMin=pipelinePBhead['datamin'])
    regionAllChans, regionContChans = computePBCircle(pipelineHead, radius, pipelineContChannels)


    regridModelAndResidual = False  # change to True if we want the middle column filled in for these two images

    if cube: 
#        pipelineStats = runImstat(pipelineImage, pipelinePB, pipelineContChannels, 
#                                  region=regionContChans, level=pblimit)
        pipelinePStats = runImstat(pipelineImage, pipelinePB, channels=pipelineContChannels, 
                                   level=Ppblimit, innerAnnulusLevel=Ppblimit2, location=4,
                                   pbMin=pipelinePBhead['datamin'])
        pipelineImageMom0,pipelineImageMom0Head = buildMomentImage(pipelineImage, 0, webpagedir,
                                                                   chans=pipelineLineChannels,loc=3)
        if pipelinePStats == '':
            pipelinePStats = pipelineRawPStats
        pipelineImageMom0sigma,pipelineImageMom0sigmaHead= buildMomentImage(pipelineImage, 0, webpagedir, 
                                                  mom0sigma*pipelinePStats['sigma'], suffix='threshold',
                                                                            chans=pipelineLineChannels,loc=4)
        pipelineImageMom8,pipelineImageMom8Head = buildMomentImage(pipelineImage, 8, webpagedir,
                                                                   chans=pipelineLineChannels,loc=5) 

        webpage += 'specsmooth time = %.0f seconds<br><br>' % (specsmoothTime)

    # Separate images into individual tables
    webpage += '<table border=1>'
    webpage += '<tr> <th>Common Information</th></tr>\n'
    beamMajor = float(beamString(pipelineRawHead).split('"')[0])
    beamMinor = float(beamString(pipelineRawHead).split('"')[1].split()[-1])
    geometricMeanBeam = (beamMajor * beamMinor)**0.5
    if ('date-obs' in manualHead):
        cycle = surmiseCycleFromImage(manualImage, manualHead)
        observationDate = 'observed on %s (ALMA Cycle %d)' % (manualHead['date-obs'], cycle)
        #    configuration = surmiseConfigurationFromImage(manualImage)
    webpage += addplotrow(webpagedir, 'Primary beam = %s" for frequency = %s GHz;</b> approximate max baseline = %sm; %s' % (roundFiguresToString(primaryBeam,3), roundFiguresToString(freqGHz,5), roundFiguresToString(angularScaleBaseline(freqGHz,beamMinor),2), observationDate), 
                          colspanTitle=4, regriddedColumn=regriddedColumn)

    if (manualImage.find('_allspw')>0):
        manualSpwString = 'aggregate'
    else:
        if (manualImage.find('_spw') > 0):
            A = manualImage.split('_spw')
            B = manualImage.split('_spw')[1]
            C = manualImage.split('_spw')[1].split('.')
            manualSpwString = manualImage.split('_spw')[1].split('.')[0] # e.g. '0'
        else:
            manualSpwString = '0'

    # deal with simulations named *cont_allspw.image
    if pipelineImage == manualImage: # .count('cont_allspw.image') > 0:
        if pipelineImage.count('cont_allspw.image') > 0:
            fullSpwString = 'allspw'  # only used in the .txt file
            spwString = '0'  # only used in the .txt file
        else:
            fullSpwString = 'spw' + manualSpwString  # only used in the .txt file
            spwString = manualSpwString  # only used in the .txt file
        transitionString = ''
        freqString = ''
    else:
        fullSpwString = pipelineImage.split('.spw')[1].split('.')[0]
        spwString = pipelineImage.split('.spw')[1].split('.')[0].split('_')[0]
        spwID = int(spwString.lstrip('spw'))
        sourceName = getSourceNameFromPipelineImageName(pipelineImage)
        vis = sorted(glob.glob(os.path.dirname(pipelineImage)+'/uid___A002*.ms'))[0]
        if (os.path.exists(vis)):
            print "Found vis = ", vis
            print "Running au.transition('%s',%d,'%s')" % (vis,spwID,sourceName)
            mySourceName = sourceName
            transitionString = transition(vis, spwID, mySourceName)
            if (len(transitionString) < 1):
                mySourceName = sourceName + '_'
                transitionString = transition(vis, spwID, mySourceName)
                if (len(transitionString) < 1):
                    transitionString = ['']
            transitionString = transitionString[0]            
            myRestFreq = restFrequency(vis, spwID, mySourceName, verbose=True)
            if (len(myRestFreq) < 1):
                freqString = ''
            else:
                freqString = roundFiguresToString(1e-9*myRestFreq[0], 8)+' GHz'
        else:
            transitionString = ''
            freqString = ''

    webpage += addplotrow(webpagedir, title='Transition & Rest Frequency',
                          textForColumn1=transitionString,
                          textForColumn2=freqString)


    if (regriddedColumn < 1):
        webpage += '<tr> <th>Item</th> <th>Pipeline raw image</th> <th>Manual image</th> </tr>\n'
    else:
        webpage += '<tr> <th>Item</th> <th>Pipeline raw image</th> <th>Manual image</th> <th>Pipeline regridded image</th> </tr>\n'
#    flushWebpage(webfile,webpage)
#    webpage = ''
#    webpage += addplotrow(webpagedir, title='<th>Item</th>', 
#                          textForColumn1='<th>Pipeline raw image</th>',
#                          textForColumn2='<th>Manual image</th>',
#                          textForColumn3='<th>Pipeline regridded image</th>',
#                          regriddedColumn=regriddedColumn
#                          )

    webpage += addplotrow(webpagedir, title='Filename',
                          textForColumn1=os.path.basename(pipelineRawImage),
                          textForColumn2=os.path.basename(manualImage),
                          textForColumn3=os.path.basename(pipelineImage), 
                          regriddedColumn=regriddedColumn
                          )

    webpage += addplotrow(webpagedir, title='Image size',
                          textForColumn1=imageSizeString(pipelineRawHead),
                          textForColumn2=imageSizeString(manualHead),
                          textForColumn3=imageSizeString(pipelineHead),
                          regriddedColumn=regriddedColumn
                          )
    if cube:
        webpage += addplotrow(webpagedir, title='Spectral axis',
                              textForColumn1=spectralSizeString(pipelineRawHead, pipelineRawChanWidth, pipelineSpecAxis),
                              textForColumn2=spectralSizeString(manualHead, manualChanWidth, manualSpecAxis),
                              textForColumn3=spectralSizeString(pipelineHead, pipelineChanWidth, pipelineSpecAxis),
                              regriddedColumn=regriddedColumn
                              )
        
    showElapsedTime("imstat")  # no mask, all channels
    manualWholeStats = imstat(manualImage, chans=manualNonEdgeChannels)
    manualModelImage = rreplace(rreplace(manualImage,'.image'),'.residual') + '.model'
    print "Running plain imstat on manualModelImage = ", manualModelImage
    showElapsedTime("imstat")
    manualModelStats = imstat(manualModelImage) 
    manualResidualImage = rreplace(manualImage,'.image','.residual')
    if (os.path.exists(manualPB)):
        if (cube):
            MaskFlattened = manualFlattenedMask
        else:
            MaskFlattened = False
        manualAllChanStats = runImstat(manualImage, manualPB, manualMask, level=pblimit, location=5,
                                       pbMin=manualPBhead['datamin'], MaskFlattened=MaskFlattened) 
        if (not cube):
            manualStatsInMask = runImstat(manualImage, False, manualMask, location=50, insideMask=True)
        manualResidualAllChanStats = runImstat(manualResidualImage, manualPB, manualMask, level=pblimit, 
                                               location=6,pbMin=manualPBhead['datamin'], MaskFlattened=MaskFlattened) 
        manualResidualAllChanPBStats = runImstat(manualResidualImage, manualPB, level=pblimit, 
                                                 location=6,pbMin=manualPBhead['datamin']) 
        manualResidualStats = runImstat(manualResidualImage, manualPB, manualMask, manualContChannels,
                                        location=60,pbMin=manualPBhead['datamin'], MaskFlattened=MaskFlattened)
        manualAllChanPStats = runImstat(manualImage, manualPB, level=Ppblimit, innerAnnulusLevel=Ppblimit2, 
                                        location=7,pbMin=manualPBhead['datamin'], MaskFlattened=MaskFlattened)
        manualResidualAllChanPStats = runImstat(manualResidualImage, manualPB, level=Ppblimit, innerAnnulusLevel=Ppblimit2, 
                                                location=70,pbMin=manualPBhead['datamin'], MaskFlattened=MaskFlattened)
    else:
        manualAllChanPStats = ''
        manualResidualAllChanPStats = ''
        print "manualPB does not exist = ", manualPB
        print "C) No primary beam image found for manual image.  Need to implement computePBAnnulus."
        manualPB = False
#        regionAllChans, regionContChans = computePBCircle(manualHead, radius, manualContChannels)
#        manualAllChanStats = runImstat(manualImage, manualPB, region=regionAllChans)
#        manualResidualAllChanStats = runImstat(manualResidualImage, manualPB, region=regionAllChans)
#        regionAllPChans, regionContPChans = computePBCircle(manualHead, Pradius, manualContChannels)
#        manualAllChanPStats = runImstat(manualImage, manualPB, region=regionAllPChans)
    regionAllChans, regionContChans = computePBCircle(manualHead, radius, manualContChannels)
#    flushWebpage(webfile,webpage)
#    webpage = ''
    if cube:
        manualStats = runImstat(manualImage, manualPB, manualMask, 
                                manualContChannels, location=8, pbMin=manualPBhead['datamin'], 
                                MaskFlattened=manualFlattenedMask)
        manualPStats = runImstat(manualImage, manualPB, level=Ppblimit, innerAnnulusLevel=Ppblimit2,
                                 channels=manualContChannels, location=9, pbMin=manualPBhead['datamin'])
        if (benchmarkProductsPresent):
            manualImageMom0 = manualImage+'.MOM0'
            manualImageMom8 = manualImage+'.PEAKMOM'
            manualImageMom0LineFree = manualImage+'.MOM0.LINEFREE'
            manualImageMom8LineFree = manualImage+'.PEAKMOM.LINEFREE'
            manualImageMom0sigma = False
            if (not os.path.exists(manualImageMom0)):
                immoments(manualImage, outfile=manualImageMom0)
            print "Running getHeader('%s')" % (manualImageMom0)
            manualImageMom0Head = getHeader(manualImageMom0)
            if (not os.path.exists(manualImageMom8)):
                immoments(manualImage, moments=8, outfile=manualImageMom8)
            print "Running getHeader('%s')" % (manualImageMom8)
            manualImageMom8Head = getHeader(manualImageMom8)
            if (os.path.exists(manualImageMom0LineFree)):
                manualImageMom0LineFreeHead = getHeader(manualImageMom0LineFree)
            if (os.path.exists(manualImageMom8LineFree)):
                manualImageMom8LineFreeHead = getHeader(manualImageMom8LineFree)
        else:
            manualImageMom0,manualImageMom0Head = buildMomentImage(manualImage, 0, webpagedir,
                                                                   chans=manualLineChannels,loc=6)
            manualImageMom0sigma,manualImageMom0sigmaHead = buildMomentImage(manualImage, 0, webpagedir, 
                                                mom0sigma*manualStats['sigma'], suffix='threshold',
                                                                         chans=manualLineChannels,loc=7)
            manualImageMom8,manualImageMom8Head = buildMomentImage(manualImage, 8, webpagedir,
                                                               chans=manualLineChannels,loc=8)
            manualImageMom0LineFree = False
            manualImageMom8LineFree = False
    else:
        if (minmaxScale == 'manual' and os.path.exists(manualPB)):
            minIntensityResidual = manualResidualStats['min'][0]
            maxIntensityResidual = manualResidualStats['max'][0]
            minIntensityModel = -2*manualModelStats['sigma'][0]
            maxIntensityModel = +4*manualModelStats['sigma'][0]
            minIntensityClean = manualWholeStats['min'][0]
            maxIntensityClean = manualWholeStats['max'][0]
        else:
            minIntensityResidual = pipelineRawContResidualStats['min'][0]
            maxIntensityResidual = pipelineRawContResidualStats['max'][0]
            minIntensityModel = -2*pipelineRawContModelStats['sigma'][0]
            maxIntensityModel = 4*pipelineRawContModelStats['sigma'][0]
            minIntensityClean = pipelineRawImageStats['min'][0]
            maxIntensityClean = pipelineRawImageStats['max'][0]
        pipelineRawContPng = buildImagePng(pipelineRawImage, imageRadius, webpagedir, 
                                           minIntensity=minIntensityClean,
                                           maxIntensity=maxIntensityClean,
                                           contourImage=pipelineRawPB, levels=pbLevels,
                                           verbose=verbose)
        pipelineContPng = buildImagePng(pipelineImage, imageRadius, webpagedir, 
                                        minIntensity=minIntensityClean, maxIntensity=maxIntensityClean,
                                        contourImage=pipelinePB, levels=pbLevels,verbose=verbose)
        manualContPng = buildImagePng(manualImage, imageRadius, webpagedir, 
                                      minIntensity=minIntensityClean, maxIntensity=maxIntensityClean,
                                      contourImage=manualPB, levels=pbLevels,verbose=verbose)
        print "Running analyzemsimage.buildImagePng('%s', %f, '%s', minIntensity=%f, maxIntensity=%f, contourImage='%s', levels=%s, contourImage2='%s', levels2=%s, verbose='%s')" % (pipelineRawModelImage,imageRadius,webpagedir,minIntensityModel,maxIntensityModel,pipelineRawPB,pbLevels,pipelineRawMask,maskLevels,verbose)
        pipelineRawContModelPng = buildImagePng(pipelineRawModelImage, imageRadius, webpagedir, 
                                                minIntensity=minIntensityModel,
                                                maxIntensity=maxIntensityModel,
                                                contourImage=pipelineRawPB, levels=pbLevels,
                                                contourImage2=pipelineRawMask, levels2=maskLevels,
                                                verbose=verbose, colormap=modelColormap)
        if verbose:
            print "Running analyzemsimage.buildImagePng('%s', %f, '%s', %f, %f, contourImage='%s', levels=%s, verbose=True)" % (manualModelImage, imageRadius, webpagedir, minIntensityModel, maxIntensityModel, manualMask, pbLevels)
        manualContModelPng = buildImagePng(manualModelImage, imageRadius, webpagedir, 
                                           minIntensity=minIntensityModel,
                                           maxIntensity=maxIntensityModel,
                                           contourImage=manualPB, levels=pbLevels,
                                           contourImage2=manualMask, levels2=maskLevels,
                                           verbose=verbose, colormap=modelColormap)
        manualContResidualPng = buildImagePng(manualResidualImage, imageRadius, webpagedir, 
                                              minIntensity=minIntensityResidual, 
                                              maxIntensity=maxIntensityResidual,
                                              contourImage=manualPB, levels=pbLevels,verbose=verbose)
        print "Running au.imageHistogram('%s', mask='auto', sigmaimage='auto', pbimage='auto', markSigma=[1,2], debug=True)" % (manualResidualImage)
        manualContResidualHistogram,xlim = imageHistogram(manualResidualImage, mask=manualMask, 
                                                     sigmaimage=manualImage, pbimage=manualPB, 
                                                     markSigma=[1,2], pblevel=pblimit, bins=bins,
                                                     plotfile=webpagedir+'%s.histogram.png'%os.path.basename(manualResidualImage))
        pipelineRawContResidualPng = buildImagePng(pipelineRawResidualImage, imageRadius, webpagedir, 
                                                   minIntensity=minIntensityResidual, 
                                                   maxIntensity=maxIntensityResidual,
                                                   contourImage=pipelineRawPB, levels=pbLevels,
                                                   verbose=verbose)
        print "Running au.imageHistogram('%s', mask='auto', sigmaimage='auto', pbimage='auto', markSigma=[1,2], debug=True)" % (pipelineRawResidualImage)
        # Note pblevel must be less than 0.3 or else not pixels can be used to compute stdev of img
        pipelineRawContResidualHistogram,xlim = imageHistogram(pipelineRawResidualImage, 
                                                               mask=pipelineRawMask, xlim=xlim,
                                                               sigmaimage=pipelineRawImage, 
                                                               pbimage=pipelineRawPB, 
                                                               markSigma=[1,2], pblevel=Ppblimit,
                                                               bins=bins,
                                                               plotfile=webpagedir+'%s.histogram.png'%os.path.basename(pipelineRawResidualImage))
        if (regridModelAndResidual):
            pipelineContModelPng = buildImagePng(pipelineModelImage, imageRadius, webpagedir, 
                                                 minIntensity=minIntensityModel,
                                                 maxIntensity=maxIntensityModel,
                                                 contourImage=pipelinePB, levels=pbLevels,
                                                 verbose=verbose, colormap=modelColormap)
            pipelineContResidualPng = buildImagePng(pipelineResidualImage, imageRadius, webpagedir, 
                                                    minIntensity=minIntensityResidual,
                                                    maxIntensity=maxIntensityResidual,
                                                    contourImage=pipelinePB, levels=pbLevels,verbose=verbose)


    if cube:
        # Do things only relevant if you have a cube
        manualImageMom0Stats = runImstat(manualImageMom0, False, location=10)
        pipelineRawImageMom0Stats = runImstat(pipelineRawImageMom0, False, location=11)
        pipelineImageMom0Stats = runImstat(pipelineImageMom0, False, location=12)
        manualImageMom8Stats = runImstat(manualImageMom8, False, location=13)
        pipelineRawImageMom8Stats = runImstat(pipelineRawImageMom8, False, location=14)
        pipelineImageMom8Stats = runImstat(pipelineImageMom8, False, location=15)
        if (minmaxScale == 'manual'):
            minMom0 = manualImageMom0Stats['min'][0]
            maxMom0 = manualImageMom0Stats['max'][0]
            minMom8 = manualImageMom8Stats['min'][0] 
            maxMom8 = manualImageMom8Stats['max'][0] 
        else:
            minMom0 = np.min([pipelineRawImageMom0Stats['min'][0], pipelineImageMom0Stats['min'][0]])
            maxMom0 = np.max([pipelineRawImageMom0Stats['max'][0], pipelineImageMom0Stats['max'][0]])
            minMom8 = np.min([pipelineRawImageMom8Stats['min'][0], pipelineImageMom8Stats['min'][0]])
            maxMom8 = np.max([pipelineRawImageMom8Stats['max'][0], pipelineImageMom8Stats['max'][0]])
        if not benchmarkProductsPresent:
            manualImageMom0sigmaStats = runImstat(manualImageMom0sigma, False, location=16)
            pipelineRawImageMom0sigmaStats = runImstat(pipelineRawImageMom0sigma, False, location=17)
            pipelineImageMom0sigmaStats = runImstat(pipelineImageMom0sigma, False, location=18)
            if (minmaxScale == 'manual'):
                minMom0sigma = manualImageMom0sigmaStats['min'][0]
                maxMom0sigma = manualImageMom0sigmaStats['max'][0]
            else:
                minMom0sigma = np.min([pipelineRawImageMom0sigmaStats['min'][0], pipelineImageMom0sigmaStats['min'][0]])
                maxMom0sigma = np.max([pipelineRawImageMom0sigmaStats['max'][0], pipelineImageMom0sigmaStats['max'][0]])
        pipelineRawImageMom0_png = buildImagePng(pipelineRawImageMom0, imageRadius, webpagedir, minMom0, maxMom0,verbose=verbose)
        pipelineRawImageMom8_png = buildImagePng(pipelineRawImageMom8, imageRadius, webpagedir, minMom8, maxMom8,verbose=verbose)

        pipelineImageMom0_png = buildImagePng(pipelineImageMom0, imageRadius, webpagedir, minMom0, maxMom0,verbose=verbose)
        pipelineImageMom8_png = buildImagePng(pipelineImageMom8, imageRadius, webpagedir, minMom8, maxMom8,verbose=verbose)

        if (benchmarkProductsPresent):
            manualImageMom0LineFree_png = buildImagePng(manualImageMom0LineFree, imageRadius, webpagedir, verbose=verbose)
            manualImageMom8LineFree_png = buildImagePng(manualImageMom8LineFree, imageRadius, webpagedir, verbose=verbose)
        else:
            pipelineRawImageMom0sigma_png = buildImagePng(pipelineRawImageMom0sigma, imageRadius, webpagedir,verbose=verbose)
            pipelineImageMom0sigma_png = buildImagePng(pipelineImageMom0sigma, imageRadius, webpagedir, minMom0sigma, maxMom0sigma,verbose=verbose)
            manualImageMom0sigma_png = buildImagePng(manualImageMom0sigma, imageRadius, webpagedir, minMom0sigma, maxMom0sigma,verbose=verbose)
        manualImageMom0_png = buildImagePng(manualImageMom0, imageRadius, webpagedir, minMom0, maxMom0,verbose=verbose)
        manualImageMom8_png = buildImagePng(manualImageMom8, imageRadius, webpagedir, minMom8, maxMom8,verbose=verbose)
    else:
        # Do things only relevant if you have a 2D image
        manualDirtyAutomask = rreplace(manualImage,'.image','.dirty.automask')
        if (os.path.exists(manualDirtyAutomask)):
            manualDirtyAutomaskPng = buildImagePng(manualMask, imageRadius, webpagedir,
                                                   minIntensity=0.0, maxIntensity=1.0,
                                                   contourImage=manualPB, levels=pbLevels,
                                                   contourImage2=manualDirtyAutomask, 
                                                   levels2=maskLevels, colorwedge=False, 
                                                   verbose=verbose, contourColor='blue',
                                                   contourColor2='red', colormap='Greyscale 2')

    webpage += addplotrow(webpagedir, 'Restoring beam', 
                          textForColumn1=beamString(pipelineRawHead),
                          textForColumn2=beamString(manualHead),
                          textForColumn3=beamString(pipelineHead),
                          regriddedColumn=regriddedColumn
                          )
    #  These calls run out of memory when run on cubes
    if not cube:
        # here is where the memory runs up to 97%
        pipelineRawMaskPixels = imagePixelsInMask(pipelineRawMask)/pipelineChannels
#        pipelineMaskPixels = imagePixelsInMask(pipelineMask)
        manualChannelsInMask = numberOfChannelsInCube(manualMask)
        manualMaskPixels = imagePixelsInMask(manualMask)/manualChannelsInMask
        value1 = 100.*pipelineRawMaskPixels/(pipelineRawChannels*pipelineRawHead['npix'])
        value2 = 100.*manualMaskPixels/(manualChannels*manualHead['npix'])
        if verbose: 
            showElapsedTime("calling addplotrow")
            print value1, value2
        webpage += addplotrow(webpagedir, 'Pixels in clean mask', justifyTitle='right',
                              textForColumn1='%d (%s%%)' % (pipelineRawMaskPixels,
                                                            roundFiguresToString(value1,3)),
    #                          textForColumn3='%d (%s%%)' % (pipelineMaskPixels,
    #                                                        roundFiguresToString(100.*pipelineMaskPixels/(pipelineChannels*pipelineHead['npix']),3)),
                              textForColumn2='%d (%s%%)' % (manualMaskPixels,
                                                            roundFiguresToString(value2,3)),
                              regriddedColumn=regriddedColumn
                              )
        print "Running imageCircularMaskRadius('%s',channels=%s)" % (pipelineRawMask,str(pipelineRawChannels))
        maskRadius = imageCircularMaskRadius(pipelineRawMask, channels=pipelineRawChannels)
        print "Done"
        maskRadius = roundFiguresToString(maskRadius,4)
        webpage += addplotrow(webpagedir, 'Inferred radius of clean mask', justifyTitle='right',
                              textForColumn1='%d pixels (%s") pblevel=%s' % ((pipelineRawMaskPixels/pipelineRawChannels/np.pi)**0.5,
                                                                             maskRadius,
                                                                             imageCircularMaskLevel(pipelineRawMask, pipelineRawChannels)),
                              regriddedColumn=regriddedColumn
                              )
    #                          textForColumn3='%d pixels (%s") pblevel=%s' %  ((pipelineMaskPixels/np.pi)**0.5,
    #                                                                         roundFiguresToString(imageCircularMaskRadius(pipelineMask),4),
    #                                                                         imageCircularMaskLevel(pipelineMask, pipelineChannels)))

    if cube:
        if verbose: print "Calling addplotrow(title='findContinuum')"
        webpage += addplotrow(webpagedir, title='findContinuum', 
                              plotForColumn1=pipelineRawFindContPng,
                              plotForColumn2=manualFindContPng,
                              plotForColumn3=pipelineFindContPng,
                              textForColumn1=aggregateBandwidthString(plRawAggregateBandwidth,pipelineRawChanWidth),
                              textForColumn2=aggregateBandwidthString(manAggregateBandwidth,manualChanWidth),
                              textForColumn3=aggregateBandwidthString(plAggregateBandwidth,pipelineChanWidth),
                              regriddedColumn=regriddedColumn
                              )
        if manualMask:
            manualFlattenedMaskPng = buildImagePng(manualFlattenedMask, imageRadius, webpagedir,
                                                   verbose=verbose)
            if verbose: print "Calling addplotrow(title='flattened mask', plotforColumn2='%s')" % (manualFlattenedMaskPng)
            webpage += addplotrow(webpagedir, title='flattened mask', 
                                  textForColumn1='fill this once pipeline uses a reasonable mask',
                                  plotForColumn2=manualFlattenedMaskPng,
                                  textForColumn3='fill this once pipeline uses a reasonable mask',
                                  regriddedColumn=regriddedColumn
                                  )
                              

    txt = open(acsTxt,'a')
    txtDict = {}
    txtDict['spw'] = manualSpwString # int(spwString.lstrip('spw'))
    if not cube:
        if (pipelineImage.find('.cont.I.') > 0):
            txt.write('Continuum properties for aggregate spw=%s\n' % fullSpwString)
        else:
            txt.write('Continuum properties for spw=%s\n' % spwString)

        myvalue = pipelineWholeStats['max'][0]
        txt.write('pipeline_cont_pk = %.9f\n' % myvalue)
        txtDict['pipeline_cont_pk'] = myvalue

        myvalue = manualWholeStats['max'][0]
        txt.write('manual_cont_pk = %.9f\n' % myvalue)
        txtDict['manual_cont_pk'] = myvalue

        myvalue = pipelineStatsInMask['max'][0]
        txt.write('pipeline_cont_peak_in_mask = %.9f\n' % myvalue)
        txtDict['pipeline_cont_peak_in_mask'] = myvalue

        myvalue = manualStatsInMask['max'][0]
        txt.write('manual_cont_peak_in_mask = %.9f\n' % myvalue)
        txtDict['manual_cont_peak_in_mask'] = myvalue

        myvalue = pipelineStatsInMask['flux'][0]
        txt.write('pipeline_cont_flux_in_mask = %.9f\n' % myvalue)
        txtDict['pipeline_cont_flux_in_mask'] = myvalue

        myvalue = manualStatsInMask['flux'][0]
        txt.write('manual_cont_flux_in_mask = %.9f\n' % myvalue)
        txtDict['manual_cont_flux_in_mask'] = myvalue

    title = 'Statistics of whole image'
    webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
    webpage += addplotrow(webpagedir, 'Peak', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawWholeStats,'max',3),
                          textForColumn3=jy_mjy_string_stats(pipelineWholeStats,'max',3),
                          textForColumn2=jy_mjy_string_stats(manualWholeStats,'max',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Minimum', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawWholeStats,'min',3),
                          textForColumn3=jy_mjy_string_stats(pipelineWholeStats,'min',3),
                          textForColumn2=jy_mjy_string_stats(manualWholeStats,'min',3),
                          regriddedColumn=regriddedColumn
                          )

    title = 'Statistics of clean image in annular region between the %g%%-%g%% response level of the primary beam (between diameters of %.1f"-%.1f")' % (Ppblimit*100, Ppblimit2*100,
                                                                                                   PprimaryBeam, PprimaryBeam2)
    if cube:
        title += ' -- all channels'
    webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
    if (manualAllChanPStats == ''):
        textForColumn2 = "image not big enough to measure"
    else:
        textForColumn2 = jy_mjy_string_stats(manualAllChanPStats,'mean',3)
    if (pipelineAllChanPStats == ''):
        textForColumn3 = "image not big enough to measure"
    else:
        textForColumn3 = jy_mjy_string_stats(pipelineAllChanPStats,'mean',3)
    webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPStats,'mean',3),
                          textForColumn2=textForColumn2,
                          textForColumn3=textForColumn3,
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPStats,'max',3),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPStats,'max',3),
                          textForColumn2=jy_mjy_string_stats(manualAllChanPStats,'max',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPStats,'min',3),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPStats,'min',3),
                          textForColumn2=jy_mjy_string_stats(manualAllChanPStats,'min',3),
                          regriddedColumn=regriddedColumn
                          )
    if not cube:
        myvalue = pipelineRawAllChanPStats['rms']
        txt.write('pipeline_cont_rms_imII = %.9f\n' % myvalue)
        txtDict['pipeline_cont_rms_imII'] = myvalue
        myvalue = manualAllChanPStats['rms']
        txt.write('manual_cont_rms_imII = %.9f\n' % myvalue)
        txtDict['manual_cont_rms_imII'] = myvalue
    rmsDigits = 5
    webpage += addplotrow(webpagedir, 'RMS', justifyTitle='right', bold=True,
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPStats,'rms',rmsDigits),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPStats,'rms',rmsDigits),
                          textForColumn2=jy_mjy_string_stats(manualAllChanPStats,'rms',rmsDigits),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=False,
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPStats,'sigma',3),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPStats,'sigma',3),
                          textForColumn2=jy_mjy_string_stats(manualAllChanPStats,'sigma',3),
                          regriddedColumn=regriddedColumn
                          )
    if (manualAllChanPStats == ''):
        textForColumn2 = ''
    else:
        textForColumn2 = roundFiguresToString(manualWholeStats['max']/manualAllChanPStats['sigma'],3)
    if (pipelineAllChanPStats == ''):
        textForColumn3 = ''
    else:
        textForColumn3 = roundFiguresToString(pipelineWholeStats['max']/pipelineAllChanPStats['sigma'],3)

    webpage += addplotrow(webpagedir, '"Apparent" SNR = Peak/Sigma', justifyTitle='right',
                          textForColumn1=roundFiguresToString(pipelineRawWholeStats['max']/pipelineRawAllChanPStats['sigma'],3),
                          textForColumn3=textForColumn3,
                          textForColumn2=textForColumn2,
                          regriddedColumn=regriddedColumn
                          )


    title = 'Statistics of residual image in annular region between the %g%%-%g%% response level of the primary beam (between diameters of %.1f"-%.1f")' % (Ppblimit*100, Ppblimit2*100,
                                                                                                   PprimaryBeam, PprimaryBeam2)
    if cube:
        title += ' -- all channels'
    webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
    if (manualResidualAllChanPStats == ''):
        textForColumn2 = "image not big enough to measure"
    else:
        textForColumn2 = jy_mjy_string_stats(manualResidualAllChanPStats,'mean',3)
    if (pipelineAllChanPStats == ''):
        textForColumn3 = "image not big enough to measure"
    else:
        textForColumn3 = jy_mjy_string_stats(pipelineResidualAllChanPStats,'mean',3)
    webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPStats,'mean',3),
                          textForColumn2=textForColumn2,
                          textForColumn3=textForColumn3,
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPStats,'max',3),
                          textForColumn3=jy_mjy_string_stats(pipelineResidualAllChanPStats,'max',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPStats,'max',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPStats,'min',3),
                          textForColumn3=jy_mjy_string_stats(pipelineResidualAllChanPStats,'min',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPStats,'min',3),
                          regriddedColumn=regriddedColumn
                          )
    if not cube:
        myvalue = pipelineRawResidualAllChanPStats['rms']
        txt.write('pipeline_cont_rms_resII = %.9f\n' % myvalue)
        txtDict['pipeline_cont_rms_resII'] = myvalue
        myvalue = manualResidualAllChanPStats['rms']
        txt.write('manual_cont_rms_resII = %.9f\n' % myvalue)
        txtDict['manual_cont_rms_resII'] = myvalue
    webpage += addplotrow(webpagedir, 'RMS', justifyTitle='right', bold=True,
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPStats,'rms',rmsDigits),
                          textForColumn3=jy_mjy_string_stats(pipelineResidualAllChanPStats,'rms',rmsDigits),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPStats,'rms',rmsDigits),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=False,
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPStats,'sigma',3),
                          textForColumn3=jy_mjy_string_stats(pipelineResidualAllChanPStats,'sigma',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPStats,'sigma',3),
                          regriddedColumn=regriddedColumn
                          )
    if (manualAllChanPStats == ''):
        textForColumn2 = ''
    else:
        textForColumn2 = roundFiguresToString(manualWholeStats['max']/manualResidualAllChanPStats['sigma'],3)
    if (pipelineAllChanPStats == ''):
        textForColumn3 = ''
    else:
        textForColumn3 = roundFiguresToString(pipelineWholeStats['max']/pipelineResidualAllChanPStats['sigma'],3)

    webpage += addplotrow(webpagedir, '"Apparent" SNR = Peak/Sigma', justifyTitle='right',
                          textForColumn1=roundFiguresToString(pipelineRawWholeStats['max']/pipelineRawResidualAllChanPStats['sigma'],3),
                          textForColumn3=textForColumn3,
                          textForColumn2=textForColumn2,
                          regriddedColumn=regriddedColumn
                          )


    title = 'Statistics of residual image in region above the %g%% response level of the primary beam (FWHM=%.1f")' % (pblimit*100, primaryBeam)
    if cube:
        title += ' -- all channels'
    webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
    webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawResidualAllChanPBStats,'mean',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPBStats,'mean',3),
#                          textForColumn3='fill this once pipeline uses a reasonable mask',
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPBStats,'mean',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPBStats,'max',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPBStats,'max',3),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPBStats,'max',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPBStats,'min',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPBStats,'min',3),
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPBStats,'min',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'RMS', justifyTitle='right', bold=True,
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPBStats,'rms',rmsDigits),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPBStats,'rms',rmsDigits),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPBStats,'rms',rmsDigits),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=False,
                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanPBStats,'sigma',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanPBStats,'sigma',3),
                          textForColumn3=jy_mjy_string_stats(pipelineAllChanPBStats,'sigma',3),
                          regriddedColumn=regriddedColumn
                          )
    if pipelineAllChanStats == '':
        textForColumn3 = ''
    else:
        textForColumn3 = roundFiguresToString(pipelineWholeStats['max']/pipelineAllChanPBStats['sigma'],3)
    webpage += addplotrow(webpagedir, 'True SNR = Peak/Sigma', justifyTitle='right',
                          textForColumn1=roundFiguresToString(pipelineRawWholeStats['max']/pipelineRawAllChanPBStats['sigma'],3),
                          textForColumn2=roundFiguresToString(manualWholeStats['max']/manualResidualAllChanPBStats['sigma'],3),
                          textForColumn3=textForColumn3,
                          regriddedColumn=regriddedColumn
                          )

    if not cube:
        myvalue = pipelineRawResidualAllChanPBStats['rms']
        txt.write('pipeline_cont_rms_res = %.9f\n' % myvalue)
        txtDict['pipeline_cont_rms_res'] = myvalue
        myvalue = manualResidualAllChanPBStats['rms']
        txt.write('manual_cont_rms_res = %.9f\n\n' % myvalue)
        txtDict['manual_cont_rms_res'] = myvalue
        txt.close()

        pickleFile = rreplace(acsTxt, '.txt', '.spw%s.p'%txtDict['spw'])
        print "Writing pickle file = ", pickleFile
        pickle.dump({manualSpwString:txtDict}, open(pickleFile,'wb'))
        os.system('chmod g+w ' + pickleFile)

    title = 'Statistics of residual image in region above the %g%% response level of the primary beam (FWHM=%.1f") excluding manual clean mask' % (pblimit*100, primaryBeam)
    if cube:
        title += ' -- all channels'
    webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
    webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
                          textForColumn1='fill this once pipeline uses a reasonable mask',
#                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanStats,'mean',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanStats,'mean',3),
                          textForColumn3='fill this once pipeline uses a reasonable mask',
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanStats,'mean',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
#                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanStats,'max',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanStats,'max',3),
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanStats,'max',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
#                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanStats,'min',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanStats,'min',3),
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanStats,'min',3),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'RMS', justifyTitle='right', bold=True,
#                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanStats,'rms',rmsDigits),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanStats,'rms',rmsDigits),
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanStats,'rms',rmsDigits),
                          regriddedColumn=regriddedColumn
                          )
    webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=False,
#                          textForColumn1=jy_mjy_string_stats(pipelineRawAllChanStats,'sigma',3),
                          textForColumn2=jy_mjy_string_stats(manualResidualAllChanStats,'sigma',3),
#                          textForColumn3=jy_mjy_string_stats(pipelineAllChanStats,'sigma',3),
                          regriddedColumn=regriddedColumn
                          )
    if pipelineAllChanStats == '':
        textForColumn3 = ''
    else:
        textForColumn3 = roundFiguresToString(pipelineWholeStats['max']/pipelineAllChanStats['sigma'],3)
    webpage += addplotrow(webpagedir, 'True SNR = Peak/Sigma', justifyTitle='right',
#                          textForColumn1=roundFiguresToString(pipelineRawAllChanStats['max']/pipelineRawAllChanStats['sigma'],3),
                          textForColumn2=roundFiguresToString(manualWholeStats['max']/manualResidualAllChanStats['sigma'],3),
#                          textForColumn3=textForColumn3,
                          regriddedColumn=regriddedColumn
                          )

    if not cube:
        myparameter = 'imagename="'+os.path.basename(manualImage).replace('.tt0','').rstrip('.image')+'"'
        print "Calling analyzemsimage.getNiterFromNewestLog('%s','%s')" % (os.path.dirname(manualImage), myparameter)
        result = getNiterFromNewestLog(os.path.dirname(manualImage),myparameter)
        if result != None:
            manualIterations, log = result
            if (manualIterations >= 0):
                textForColumn2 = 'clean iterations = %s<br>%s' % (manualIterations,os.path.basename(log))
            elif (log != None):
                textForColumn2 = '%s' % (os.path.basename(log))
            else:
                textForColumn2 = 'log = None'
        else:
            textForColumn2 = 'log dir not found'

        myparameter = 'image coordinates for [' + os.path.basename(pipelineRawImage).replace('.tt0','').rstrip('.image') + ']'
        print "Calling analyzemsimage.getNiterFromNewestLog('%s','%s')" % (os.path.dirname(pipelineRawImage), myparameter)
        result = getNiterFromNewestLog(os.path.dirname(pipelineRawImage),myparameter)
        if result != None:
            pipelineIterations, log = result
            if (pipelineIterations >= 0):
                textForColumn1 = 'clean iterations = %s<br>%s' % (pipelineIterations,os.path.basename(log))
            elif (log != None):
                textForColumn1 = '%s' % (os.path.basename(log))
            else:
                textForColumn1 = 'log = None'
        else:
            textForColumn1 = 'log dir not found'
        webpage += addplotrow(webpagedir, title='<b>Continuum clean image</b><br>with contours showing primary<br>beam response: %s'%(str(pbLevels)),
                              plotForColumn1=pipelineRawContPng,
                              plotForColumn3=pipelineContPng,
                              plotForColumn2=manualContPng,
                              textForColumn1=textForColumn1,
                              textForColumn2=textForColumn2,
                              regriddedColumn=regriddedColumn
                              )
        plotForColumn3 = ''
        if regridModelAndResidual:
            plotForColumn3 = pipelineContModelPng
        pipelineRawContModelPixels,pipelineRawContModelNPixels = nonzeroPixels(pipelineRawModelImage, returnPositiveNegativeCounts=True)
        pipelineRawContModelTotalPixels = pipelineRawContModelPixels + pipelineRawContModelNPixels
        textForColumn1 = '%d pixels w/components: %d positive, %d negative' % (pipelineRawContModelPixels+pipelineRawContModelNPixels,
                                                                      pipelineRawContModelPixels,pipelineRawContModelNPixels)
        manualContModelPixels,manualContModelNPixels = nonzeroPixels(manualModelImage, returnPositiveNegativeCounts=True)
        manualContModelTotalPixels = manualContModelPixels + manualContModelNPixels
        textForColumn2 = '%d pixels w/components: %d positive, %d negative' % (manualContModelPixels+manualContModelNPixels,
                                                                      manualContModelPixels,manualContModelNPixels)
        webpage += addplotrow(webpagedir, title='<b>Continuum model image</b><br>with contours showing clean mask and<br>primary beam response: %s' % (str(pbLevels)),
                              plotForColumn1=pipelineRawContModelPng,
                              plotForColumn3=plotForColumn3,
                              plotForColumn2=manualContModelPng,
                              textForColumn1=textForColumn1,
                              textForColumn2=textForColumn2,
                              regriddedColumn=regriddedColumn
                              )
        niter = open(webpagedir+'%s_niter.txt' % (os.path.basename(manualImage)),'w')
        niter.write('%s & %4d & %5d & %4d & %4d & %4d & %5d & %4d & %4d\n' % (os.path.basename(manualImage),
                    pipelineIterations, pipelineRawContModelTotalPixels, pipelineRawContModelPixels, pipelineRawContModelNPixels,
                    manualIterations, manualContModelTotalPixels, manualContModelPixels, manualContModelNPixels))
        niter.close()

        if (os.path.exists(manualDirtyAutomask)):
            print "Running imagePixelsInMask('%s')" % (manualDirtyAutomask)
            automaskPixels = imagePixelsInMask(manualDirtyAutomask)
            if cube:
                textForColumn1 = ''
                textForColumn2 = ''
            else:
                textForColumn1 = 'pixels in dirty automask = %d = %s times the manual mask<b>' % (automaskPixels, roundFiguresToString(automaskPixels*1.0/manualMaskPixels,3))
                textForColumn2 = 'pixels in manual mask = %d<b>' % (manualMaskPixels)
            webpage += addplotrow(webpagedir, title='Continuum manual mask image</b><br>with contours showing automask result<br>from the dirty image (red) and<br>primary beam response: %s (blue)' % (str(pbLevels)),
                                  plotForColumn2=manualDirtyAutomaskPng,
                                  textForColumn1=textForColumn1,
                                  textForColumn2=textForColumn2,
                                  regriddedColumn=regriddedColumn
                                  )

        plotForColumn3 = ''
        if regridModelAndResidual:
            plotForColumn3 = pipelineContResidualPng
        webpage += addplotrow(webpagedir, title='Continuum residual image</b><br>with contours showing primary<br>beam response: %s'%(str(pbLevels)),
                              plotForColumn1=pipelineRawContResidualPng,
                              plotForColumn3=plotForColumn3,
                              plotForColumn2=manualContResidualPng,
                              regriddedColumn=regriddedColumn
                              )
        
        plotForColumn3 = ''
        webpage += addplotrow(webpagedir, title='Continuum residual histogram</b><br>of pixels inside clean mask<br>with vertical lines marking sigma<br>in the clean image',
                              textForColumn1='sigma measured between pb=%.1f-%.1f' % (Ppblimit, Ppblimit2),
                              textForColumn2='sigma measured for pb>%.1f and outside mask' % pblimit,
                              plotForColumn1=pipelineRawContResidualHistogram,
                              plotForColumn3=plotForColumn3,
                              plotForColumn2=manualContResidualHistogram,
                              regriddedColumn=regriddedColumn
                              )
    else:  # this is the cube case
        title = 'Statistics of clean image in annular region between the %g%%-%g%% response level of the primary beam (between diameters of %.1f"-%.1f")' % (Ppblimit*100, Ppblimit2*100, PprimaryBeam, PprimaryBeam2)
        title += ' -- continuum channels only'
        webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
        if (manualPStats == ''):
            textForColumn2 = "image not big enough to measure"
        else:
            textForColumn2 = jy_mjy_string_stats(manualPStats,'mean',3)
        if (pipelinePStats == ''):
            textForColumn3 = "image not big enough to measure"
        else:
            textForColumn3 = jy_mjy_string_stats(pipelinePStats,'mean',3)
        webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
                              textForColumn1=jy_mjy_string_stats(pipelineRawPStats,'mean',3),
                              textForColumn2=textForColumn2,
                              textForColumn3=textForColumn3,
                              regriddedColumn=regriddedColumn
                              )
        webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
                              textForColumn1=jy_mjy_string_stats(pipelineRawPStats,'max',3))
        webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
                              textForColumn1=jy_mjy_string_stats(pipelineRawPStats,'min',3))
        webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=True,
                              textForColumn1=jy_mjy_string_stats(pipelineRawPStats,'sigma',3))
        textForColumn1 = roundFiguresToString(pipelineWholeStats['max']/pipelineRawPStats['sigma'],3)
        webpage += addplotrow(webpagedir, '"Apparent" SNR = Peak/Sigma', justifyTitle='right',
                              textForColumn1=textForColumn1)

        title = 'Statistics of residual image in region above the %g%% response level of the primary beam (FWHM=%.1f") excluding manual clean mask' % (pblimit*100, primaryBeam)
        title += ' -- continuum channels only'
        webpage += addplotrow(webpagedir, title, colspanTitle=4, regriddedColumn=regriddedColumn)
        
        webpage += addplotrow(webpagedir, 'Mean', justifyTitle='right',
#                              textForColumn1=jy_mjy_string(pipelineRawStats['mean'],3),
                              textForColumn2=jy_mjy_string(manualResidualStats['mean'],3),
#                              textForColumn3=jy_mjy_string(pipelineStats['mean'],3))
                              regriddedColumn=regriddedColumn
                              )
        webpage += addplotrow(webpagedir, 'Max', justifyTitle='right',
#                              textForColumn1=jy_mjy_string(pipelineRawStats['max'],3),
#                              textForColumn3=jy_mjy_string_stats(pipelinePStats,'max',3),
                              textForColumn2=jy_mjy_string(manualResidualStats['max'],3),
                              regriddedColumn=regriddedColumn
                              )
        webpage += addplotrow(webpagedir, 'Min', justifyTitle='right',
#                              textForColumn1=jy_mjy_string(pipelineRawStats['min'],3),
#                              textForColumn3=jy_mjy_string_stats(pipelinePStats,'min',3),
                              textForColumn2=jy_mjy_string(manualResidualStats['min'],3),
                              regriddedColumn=regriddedColumn
                              )
        webpage += addplotrow(webpagedir, 'Sigma', justifyTitle='right', bold=True,
#                              textForColumn1=jy_mjy_string(pipelineRawStats['sigma'],3),
#                              textForColumn3=jy_mjy_string_stats(pipelinePStats,'sigma',3),
                              textForColumn2=jy_mjy_string(manualResidualStats['sigma'],3),
                              regriddedColumn=regriddedColumn
                              )
        webpage += addplotrow(webpagedir, 'True SNR = Peak/Sigma', justifyTitle='right',
#                              textForColumn1=roundFiguresToString(pipelineRawStats['max']/pipelineRawStats['sigma'],3),
#                              textForColumn3=roundFiguresToString(pipelinePStats['max']/pipelinePStats['sigma'],3),
                              textForColumn2=roundFiguresToString(manualResidualStats['max']/manualResidualStats['sigma'],3),
                              regriddedColumn=regriddedColumn
                              )

        if (benchmarkProductsPresent):
            title = '<b>Moment 0 (includepix=-1)</b><br>Note: channel ranges may differ' 
        else:
            title = 'Moment 0 (includepix=-1)'
        webpage += addplotrow(webpagedir, title=title,
                              plotForColumn1=pipelineRawImageMom0_png,
                              plotForColumn3=pipelineImageMom0_png,
                              plotForColumn2=manualImageMom0_png,
                              regriddedColumn=regriddedColumn
                              )
        if (not benchmarkProductsPresent):
            webpage += addplotrow(webpagedir, 
                                  title='Moment 0<br>(includepix = %g sigma)'%(mom0sigma),
                                  textForColumn1 = 'threshold = %g' % (mom0sigma*pipelineRawPStats['sigma']),
                                  textForColumn3 = 'threshold = %g' % (mom0sigma*pipelinePStats['sigma']),
                                  textForColumn2 = 'threshold = %g' % (mom0sigma*manualStats['sigma']),
                                  plotForColumn1=pipelineRawImageMom0sigma_png,
                                  plotForColumn3=pipelineImageMom0sigma_png,
                                  plotForColumn2=manualImageMom0sigma_png,
                                  regriddedColumn=regriddedColumn
                                  )

        if (benchmarkProductsPresent):
            title = '<b>Moment 8</b><br>Note: channel ranges may differ' 
        else:
            title = 'Moment 8'
        webpage += addplotrow(webpagedir, title=title, 
                              plotForColumn1=pipelineRawImageMom8_png,
                              plotForColumn3=pipelineImageMom8_png,
                              plotForColumn2=manualImageMom8_png,
                              regriddedColumn=regriddedColumn
                              )

        # spectrum at peak of moment 8
        manualImageHead = getHeader(manualImage)
        spectralRadius = beamRadius(manualImageHead)*3.0
        titleString = 'pixel: %s, radius=%g arcsec' % (str(manualImageMom8Stats['maxpos'][:2]), spectralRadius)
        manualMoment8PeakSpectrumPng, ignore = plotSpectrum(manualImage, manualImageHead, manualImageMom8Stats['maxpos'], 
                                                            radius=spectralRadius, title=titleString, contChannels=manualContChannels,
                                                            outputpath=imagedir)

        pipelineImageHead = getHeader(pipelineImage)
        titleString = 'pixel: %s, radius=%g arcsec' % (str(pipelineImageMom8Stats['maxpos'][:2]), spectralRadius)
        pipelineMoment8PeakSpectrumPng, ignore = plotSpectrum(pipelineImage, pipelineImageHead, pipelineImageMom8Stats['maxpos'], 
                                                              radius=spectralRadius, title=titleString, contChannels=pipelineContChannels,
                                                              outputpath=imagedir)
        webpage += addplotrow(webpagedir, title='Spectrum at peak of Moment 8', plotForColumn1=pipelineMoment8PeakSpectrumPng,
                              plotForColumn2=manualMoment8PeakSpectrumPng)

        pipelineMom0FC = pipelineImage + '.mom0_fc'
        pipelineMom0FCStats = runImstat(pipelineMom0FC, False)
        manualImageMom0LineFreeStats = runImstat(manualImageMom0LineFree, False)

        # in project 2013.1.00722.S, the min/max range of pipelineMom0FC is much less than manualImageMom0LineFreeStats
        pipelineMom0FCPng = buildImagePng(pipelineMom0FC, imageRadius, webpagedir, 
#                                          minIntensity=manualImageMom0LineFreeStats['min'][0],
#                                          maxIntensity=manualImageMom0LineFreeStats['max'][0], 
                                          verbose=verbose)

        if (benchmarkProductsPresent):
            plotForColumn2 = manualImageMom0LineFree_png
            textForColumn2 = 'min=%g, max=%g'% (manualImageMom0LineFreeStats['min'][0],manualImageMom0LineFreeStats['max'][0]),
        else:
            textForColumn2 = ''
        webpage += addplotrow(webpagedir, title='Moment 0 (linefree)', 
                              plotForColumn1=pipelineMom0FCPng,
                              textForColumn1='min=%g, max=%g'% (pipelineMom0FCStats['min'][0],pipelineMom0FCStats['max'][0]),
                              plotForColumn2=plotForColumn2,
                              textForColumn2 = textForColumn2,
                              regriddedColumn=regriddedColumn
                              )

        if (benchmarkProductsPresent):
            webpage += addplotrow(webpagedir, title='Moment 8 (linefree)', 
                                  textForColumn1='the pipeline does not make a line-free cube',
                                  plotForColumn2=manualImageMom8LineFree_png,
                                  regriddedColumn=regriddedColumn
                                  )

        # generate spectra over the central half-power beam, no threshold
        pipelineRawSpectrumPng,ylim = plotSpectrum(pipelineRawImage, pipelineRawHead, 
                                                   pipelineRawPStats, 
                                                   profileSigma=None, radius=radius,
                                                   outputpath=imagedir)
        pipelineSpectrumPng,ylim2 = plotSpectrum(pipelineImage, pipelineHead, pipelinePStats, 
                                                 profileSigma=None, radius=radius,
                                                 outputpath=imagedir)
        ylim = expandLimits(ylim,ylim2)
        manualSpectrumPng,ylim2 = plotSpectrum(manualImage, manualHead, manualStats, 
                                               profileSigma=None, radius=radius, outputpath=imagedir)
        ylim = expandLimits(ylim,ylim2)
        halfPowerYlimits = ylim[:]

        # generate spectra over the emission mask
        pipelineRawSpectrumSigmaPng,ylim2 = plotSpectrum(pipelineRawImage, pipelineRawHead, 
                                                         pipelineRawPStats, profileSigma,
                                                         outputpath=imagedir)
        ylim = expandLimits(ylim,ylim2)
        pipelineSpectrumSigmaPng,ylim2 = plotSpectrum(pipelineImage, pipelineHead, 
                                                      pipelinePStats, profileSigma,
                                                      outputpath=imagedir)
        ylim = expandLimits(ylim,ylim2)
        manualSpectrumSigmaPng,ylim2 = plotSpectrum(manualImage, manualHead, manualStats, 
                                                    profileSigma, outputpath=imagedir)
        ylim = expandLimits(ylim,ylim2)
        
        # Re-generate spectra over the emission mask with new limits
        pipelineRawSpectrumSigmaPng,ignore = plotSpectrum(pipelineRawImage, pipelineRawHead, 
                                                          pipelineRawPStats, 
                                                          profileSigma, ls='.',ylimits=ylim,
                                                          outputpath=imagedir)
        pipelineSpectrumSigmaPng,ignore = plotSpectrum(pipelineImage, pipelineHead, 
                                                       pipelinePStats, 
                                                       profileSigma, ls='.', ylimits=ylim,
                                                       outputpath=imagedir)
        manualSpectrumSigmaPng,ignore = plotSpectrum(manualImage, manualHead, manualStats, 
                                                     profileSigma, ls='.', ylimits=ylim, outputpath=imagedir)
        if False:
            # Re-generate spectra over the central half-power beam with same limits
            pipelineRawSpectrumPng,ignore = plotSpectrum(pipelineRawImage, pipelineRawHead, 
                                                         pipelineRawPStats, profileSigma=None, 
                                                         radius=radius, ylimits=ylim,
                                                         outputpath=imagedir)
            pipelineSpectrumPng,ignore = plotSpectrum(pipelineImage, pipelineHead, pipelinePStats, 
                                                      profileSigma=None, radius=radius, ylimits=ylim,
                                                      outputpath=imagedir)
            manualSpectrumPng,ignore = plotSpectrum(manualImage, manualHead, manualStats, 
                                                    profileSigma=None, radius=radius, ylimits=ylim,
                                                    outputpath=imagedir)

        # Re-generate spectra over the central half-power beam (autoscaled)
        pipelineRawSpectrumAutoscaledPng,ignore = plotSpectrum(pipelineRawImage, pipelineRawHead, 
                                                               pipelineRawPStats, profileSigma=None, 
                                                               radius=radius, ylimits=halfPowerYlimits, 
                                                               pngnameSuffix='_autoscaled',
                                                               outputpath=imagedir)
        pipelineSpectrumAutoscaledPng,ignore = plotSpectrum(pipelineImage, pipelineHead, pipelinePStats, 
                                                            profileSigma=None, radius=radius, ylimits=halfPowerYlimits,
                                                            pngnameSuffix='_autoscaled',
                                                            outputpath=imagedir)
        manualSpectrumAutoscaledPng,ignore = plotSpectrum(manualImage, manualHead, manualStats, 
                                                          profileSigma=None, radius=radius, ylimits=halfPowerYlimits,
                                                          pngnameSuffix='_autoscaled', outputpath=imagedir)

        # Fill webpage with emission mask spectra
        thresholdString = "threshold = %g sigma" % (profileSigma)
        textForColumn1 = 'threshold=%g'%(profileSigma*pipelineRawPStats['sigma'])
        textForColumn3 = 'threshold=%g'%(profileSigma*pipelinePStats['sigma'])
        textForColumn2 = 'threshold=%g'%(profileSigma*manualStats['sigma'])
        webpage += addplotrow(webpagedir, title='Spectrum of pixels above<br>%s'%(thresholdString), 
                              plotForColumn1=pipelineRawSpectrumSigmaPng, 
                              textForColumn1=textForColumn1,
                              plotForColumn3=pipelineSpectrumSigmaPng,
                              textForColumn2=textForColumn2,
                              plotForColumn2=manualSpectrumSigmaPng,
                              textForColumn3=textForColumn3,
                              regriddedColumn=regriddedColumn
                              )
        if False:
            # Fill webpage with central half-power spectra
            thresholdString= "(no threshold, matched scale)"
            webpage += addplotrow(webpagedir, title='Spectrum of central<br>primary beam of %.1f"<br>%s'%(primaryBeam,thresholdString), 
                                  plotForColumn1=pipelineRawSpectrumPng, 
                                  plotForColumn3=pipelineSpectrumPng,
                                  plotForColumn2=manualSpectrumPng,
                                  regriddedColumn=regriddedColumn
                                  )

        # Fill webpage with central half-power spectra
        thresholdString= "(no threshold, autoscaled)"
        webpage += addplotrow(webpagedir, title='Spectrum of central<br>primary beam of %.1f"<br>%s'%(primaryBeam,thresholdString), 
                              plotForColumn1=pipelineRawSpectrumAutoscaledPng, 
                              plotForColumn3=pipelineSpectrumAutoscaledPng,
                              plotForColumn2=manualSpectrumAutoscaledPng,
                              regriddedColumn=regriddedColumn
                              )

    # Finished with this pair of images
    webpage += '</table>\n'
    webpage += 'Elapsed time = %s<br><br>' % (showElapsedTime())

    elapsedTime = time.time()-singleStartTime
        
    if not appendhtml:
        if os.path.exists(webfile): 
            os.remove(webfile)
        finishWebpage(webfile, webpage)
    else:
        f = open(webfile,'a')
        f.write(webpage)
        f.close()
    print "%.1f sec = %.1f min elapsed time" % (elapsedTime, elapsedTime/60.)
    return(elapsedTime)

def flushWebpage(webfile,webpage):
    f = open(webfile,"a")
    f.write(webpage)
    f.close()

def finishWebpage(webfile, data=''):
    """
    Write an optional final batch of data to the specified text file, then
    close the html tags.
    """
    f = open(webfile,"a")
    print >>f, data + """
         </body>
         </html>
      """
    f.close()
    print "Finished webpage = ", webfile

def help(match='', debug=False):
    """
    Print an alphabetized list of all the defined functions at the top level in
    analysisUtils.py.
    match: limit the list to those functions containing this string (case insensitive)
           If match is a list of strings or a list as a comma-delimited string, then
           the function must contain all strings to be considered a match.
    -- Todd Hunter
    """
    myfile = __file__
    if (myfile[-1] == 'c'):
        if (debug): print "ami loaded from .pyc file, looking at .py file instead"
        myfile = myfile[:-1]
    aufile = open(myfile,'r')
    lines = aufile.readlines()
    if (debug): print "Read %d lines from %s" % (len(lines), __file__)
    aufile.close()
    commands = []
    if (type(match) == str and match.find(',')>0):
        match = match.split(',')
    for line in lines:
        if (line.find('def ') == 0):
            commandline = line.split('def ')[1]
            tokens = commandline.split('(')
            if (len(tokens) > 1):
                command = tokens[0]
            else:
                command = commandline.strip('\n\r').strip('\r\n')
            if (match == ''):
                commands.append(command)
            elif (type(match) == str):
                if (command.lower().find(match.lower()) >= 0):
                    commands.append(command)
            elif (type(match) == list or type(match) == np.ndarray):
                add = True
                for m in match:
                    if (command.lower().find(m.lower()) < 0):
                        add = False
                if (add): commands.append(command)
            else:
                print "The match argument must be a string or a list of strings."
                return
                
    commands.sort()
    for command in commands:
        print command

