#!/usr/bin/env python

"""
Sanity check of Antenna Parameters prior to antenna move

Checks:

- Antenna Move is tracked in antennaMoves.txt
- DRX suggestd Pad Delay and compares it to TMCDB Pad Delay
- Checks Pad position in TMCDB compared to PadData.xml
- Checks Antenna Postition (vector)
  The z component should be 7.0m for the Vertex antennas,
  and 7.5m for the Melco and AEM antennas as the defaults.  
- Gets Tilt terms in ACU and TMCDB
- Checks Metrology mode

The script outputs at the end a command that can be used to update the
PadDelay based on the DRX prediction (the command can be used to input
any other value on the TMCDB without using the GUI)

Usage:  ant_move_check.py -a DA46 

#"""

# Execution starts here

import os, sys
import socket
import getpass
from optparse import OptionParser;
from pylab import *
import re
import tmcdbPlayer_tmp as tp
import time;
import plot_tmcdb_content as tk

#import tmUtils as tm
#from datetime import datetime,timedelta
#import warnings
#from  getNominalPtBandOffsets import getNominal
#import pickle
#import glob

username = getpass.getuser()

#sys.path.append('/users/%s/AIV/science/scripts/R10.4_WORKING/'%username)

#sys.path.append('/users/%s/AIV/science/analysis_scripts/'%username)

#import analysisUtils as au

hostname = socket.gethostname()
username = getpass.getuser()

if ('aod' in username) or ('almaop1' in username):
    print "ERROR: This script should not be used as aod or almaop. Please log in and run it as yourself. Exiting..."
    sys.exit()
#sys.path.append('/users/%s/AIV/science/analysis_scripts/'%username)
if ('red' in hostname) or ('casa' in hostname) or ('scops' in hostname):
    #print 'WARNING: You are running on an analysis machine \"%s\" as user \"%s\". Using casapy-telcal asdm bindings (APO-450). You must have the proper path setup. Contact Denis Barkats if the asdm module fails to load ####' %(hostname, username)
    from XmlObjectifier import *
elif ('gns' in hostname) or ('crc' in hostname):
    # on aos-gns or crc
    print '### You are running on an STE machine \"%s\" as user \"%s\" ###'  %(hostname, username)
    from Acspy.Util.XmlObjectifier import *

    
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    
    def disable(self):
        self.HEADER = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''

def grep_file (f,search_term):
  line_found=''
  for line in open(f, 'r'):
      if re.search(search_term, line):
          print line,
          line_found=line
          if line == None:
              print 'no matches found'
  return line_found


parser=OptionParser();
parser.add_option("-a","--antenna",dest="antenna",help="Antenna to get parameters. Ex.: DV01",default="");
parser.add_option("--AI", "--AntennaIntegrator", action="store_true", dest="ai",
                  help="Running as part of the Antenna Integrator (avoids ssh@aos-gns", default=False)
#parser.add_option("-p","--pad",dest="pad",help="Pad to get parameters. Ex.: A001",default="");

(options,args)=parser.parse_args();


#################################
# Define Antenna & Pad
#################################

            
ant=options.antenna
#pad=options.pad

print "Processing %s " % (ant)

#################################
# Time
#################################

today=tk.getCurrentTime()
date=today.split('T')[0]
#################################
# getPad
#################################
tt = tp.tmcdbPlayer()

pad=''
if ('red' in hostname):
   pad =tt.tmcdb.getCurrentAntennaPadInfo(ant).padName
elif ('gns' in hostname) or ('crc' in hostname):
   pad =tt.tmcdb.getCurrentAntennaPadInfo(ant).PadName
print "%s is in Pad %s" % (ant,pad)


##################################
# Get STE DRX, Metro and tilts
##################################

drx_metro_results='/tmp/drx_metro_results_%s_%s.txt'%(ant,today.split('T')[0])
os.system('rm -rf '+drx_metro_results)
up_cmd =''

# If run as part of the AntennaIntegrator we assume the information is in stored in its expected file
if options.ai:
    print 'NOTE: Running as part of Antenna Integrator. ' \
          'It is assumed that required information from aos-gns is stored in /tmp/drx_metro_results_%s_last.txt' % ant
    print ' This will be copied to /tmp/drx_metro_results_%s_%s.txt' % (ant, today.split('T')[0])
    up_cmd = 'cp /tmp/drx_metro_results_%s_last.txt /tmp/drx_metro_results_%s_%s.txt' % (ant, ant, today.split('T')[0])
    tp.system_except(up_cmd)
# Otherwise, if we run in standalone mode we retrieve the information differently
# depending on whether we are in red-osf or aos-gns
else:
    # From RED-OSF
    if ('red' in hostname):
         up_cmd = 'ssh aod@aos-gns.aiv.alma.cl \'/groups/science/scripts/R10.4_WORKING/util/DRXcheck.py -a %s;/groups/science/analysis_scripts/compareTilt.py -a %s;/users/ahales/scripts//getMetroMode.py -a %s\' > %s' % (ant,ant,ant,drx_metro_results)
         print "##################################"
         print "# Please Enter AOD password "
         print "##################################"
         tp.system_except(up_cmd)
    # From AOS-GNS
    elif ('gns' in hostname) or ('crc' in hostname):
        # on aos-gns or crc
         up_cmd1 = '/groups/science/scripts/R10.4_WORKING/util/DRXcheck.py -a %s >> %s' % (ant,drx_metro_results)
         up_cmd2 = '/groups/science/analysis_scripts/compareTilt.py -a %s >> %s' % (ant,drx_metro_results)
         up_cmd3 = '/users/ahales/scripts//getMetroMode.py -a %s >> %s' % (ant,drx_metro_results)

         print up_cmd1
         print up_cmd2
         print up_cmd3

         tp.system_except(up_cmd1)
         tp.system_except(up_cmd2)
         tp.system_except(up_cmd3)

os.system('chmod ugo+rwx -R %s' %  drx_metro_results)

# Get DRX Pad Delay

check_drx_result=grep_file(drx_metro_results,'Pad delay suggested @ AOS')
suggested_pad_delay=check_drx_result.split()[5]
suggested_pad_delay_float=float(suggested_pad_delay)
print "Suggested DRX Pad Delay is %s (ns)" % suggested_pad_delay
print "Suggested DRX Pad Delay is %f (ns)" , suggested_pad_delay_float
suggested_pad_delay_float_s=suggested_pad_delay_float*1e-9
print "Suggested DRX Pad Delay is %f (s)" , suggested_pad_delay_float_s



# Get Metro Tilts


check_metro_result=grep_file(drx_metro_results,'Tilts')
check_metro_result=check_metro_result.replace(" ", "")
  
print check_metro_result
ant_metro=check_metro_result.split(',')[1]
an0_tmcdb=check_metro_result.split(',')[2]
aw0_tmcdb=check_metro_result.split(',')[3]
an0_acu=check_metro_result.split(',')[4]
aw0_acu=check_metro_result.split(',')[5]

print ant_metro, an0_tmcdb,aw0_tmcdb,an0_acu,aw0_acu

# Get Metro mode

check_metro_result=grep_file(drx_metro_results,'Metrology current')
ant_metro_mode=check_metro_result.split(':')[1]
print ant, ant_metro_mode


#################################
# Check Antenna Move.txt file
#################################

################################
# Update Antenna Moves
################################


username = getpass.getuser()



# If run as part of the AntennaIntegrator we assume the commong cvs directory is updated
if options.ai:
    antennaMovesDir = '/groups/science/AIV/PadData/'
    antennaMovesFile = '/groups/science/AIV/PadData/antennaMoves.txt'
else:
    antennaMovesDir = '/users/%s/AIV/science/PadData/'%username
    antennaMovesFile = '/users/%s/AIV/science/PadData/antennaMoves.txt'%username
    print '#### CVS updating antennaMoves.txt... #### '
    wd = os.getcwd()
    os.chdir('%s/'%antennaMovesDir)
    up_cmd = 'cvs up antennaMoves.txt'
    tp.system_except(up_cmd)
    os.chdir(wd)


search_term=ant
last_move=''
f=antennaMovesFile
for line in open(f, 'r'):
    if re.search(search_term, line):
        print line,
        last_move=line
        if line == None:
            print 'no matches found'

print "%s Last Move in antennaMoves.txt"
print last_move



################################
# TMCDB Checks
################################

tt = tp.tmcdbPlayer()

################################
# Check Pad Position
################################

pad_pos,ant_pos,ant_off=tk.getPositionModelDate(ant,pad,today)

print "TMCDB Pad positions are  ", (pad_pos)
print "TMCDB Ant positions are  ", (ant_pos)
print "TMCDB Ant Offset    are  ", (ant_off)

search_term = pad
if options.ai:
    f = '/groups/science/AIV/PadData/PadData.xml'
else:
    f = '/users/%s/AIV/science/PadData/PadData.xml'%username


xml_pad_coord=''

for line in open(f, 'r'):
    if re.search(search_term, line):
        print line,
        xml_pad_coord=line
        if line == None:
            print 'no matches found'


################################
# Check Pad Tilts
################################

tilts=tk.getTiltParameters(ant,pad)

print "TMCDB Pad tilts are An0=%s Aw=%s " % (tilts[0],tilts[1])


################################
# Check Pad Delay
################################


padDelay=tk.getPadDelayModelDate(ant,today)

print "TMCDB Pad delay is %s " % (padDelay)


print "\n \n"

##################################
## Pointing/Focus Check
##################################
#
## Comment: there are no new antennas, so NA
#
#
##################################
## Check Metrology
##################################
#
#def check_metrology(ant):
#   print '#### ssh aos-gns to check ACU Metrology #### '
#   wd = os.getcwd()
#   #os.chdir('%s/'%antennaMovesDir)
#   metro_results='/tmp/metro_results_%s_%s.txt'%(ant,today.split('T')[0])
#   print "Saving Megrology results to %s " % metro_results 
#   
#   up_cmd = 'ssh aod@aos-gns.aiv.alma.cl \'/groups/science/analysis_scripts/compareTilt.py -a %s\' > %s' % (ant,metro_results)
#   
#   print up_cmd
#   tp.system_except(up_cmd)
#   os.system('chmod ugo+rwx -R %s' %  metro_results)
#   
#   check_metro_result=grep_file(metro_results,ant)
#   check_metro_result=check_metro_result.replace(" ", "")
#  
#   print check_metro_result
#   ant_metro=check_metro_result.split(',')[1]
#   an0_tmcdb=check_metro_result.split(',')[2]
#   aw0_tmcdb=check_metro_result.split(',')[3]
#   an0_acu=check_metro_result.split(',')[4]
#   aw0_acu=check_metro_result.split(',')[5]
#   
#   return ant_metro, an0_tmcdb,aw0_tmcdb,an0_acu,aw0_acu
#
#
#
###################################
### Check DRX
###################################
#
#
#def check_drx(ant):
#   print '#### ssh aos-gns to check DRX.py #### '
#   wd = os.getcwd()
#   #os.chdir('%s/'%antennaMovesDir)
#   drx_results='/tmp/drx_results_%s_%s.txt'%(ant,today.split('T')[0])
#   print "Saving DRX results to %s " % drx_results 
#   
#   up_cmd = 'ssh aod@aos-gns.aiv.alma.cl \'/groups/science/scripts/R10.4_WORKING/util/DRXcheck.py -a %s\' > %s' % (ant,drx_results)
#   print up_cmd
#   tp.system_except(up_cmd)
#   os.system('chmod ugo+rwx -R %s' %  drx_results)
#   
#   check_drx_result=grep_file(drx_results,'Pad delay suggested @ AOS')
#   suggested_pad_delay=check_drx_result.split(' ')[13]
#   #print "Suggested DRX Pad Delay is %s (ns)" % suggested_pad_delay
#   return suggested_pad_delay
#
#################################
# Print Summary
################################


print "#################################################"
print "# Antenna Integration Preliminary Checks        # "
print "#################################################"

print "\033[90m#################################################"
print "# Antenna Move Check                            #"
print "#################################################\033[0m"

print "# Last Move in antennaMoves.txt"
print "# Date           Ant  Old  New \n"
print last_move



print "\033[90m#################################################"
print "# Pad Position Check                            #"
print "# (Check Pad Position is correct in TMCDB)      #"
print "#################################################\033[0m"


print "TMCDB %s positions are       : "% (pad),(pad_pos)
#print "\n For reference, Coordinates from PadData.xml are"
x_ref=xml_pad_coord.split("\"")[3]
y_ref=xml_pad_coord.split("\"")[5]
z_ref=xml_pad_coord.split("\"")[7]
print "PadData.xml %s positions are :  (%s,   %s,   %s)"% (pad,x_ref,y_ref,z_ref)
#print x_ref
#print xml_pad_coord

print "\033[90m###############################################"
print "# Antenna Position Check                      #"
print "# (Check Antenna Position is correct in TMCDB)#"
print "###############################################\033[0m"


print "TMCDB Ant positions are  ", (ant_pos)
print "TMCDB Ant Offset    are  ", (ant_off)

print "\033[90m#################################################"
print "# Pad Delay Check                               #"
print "# (Check Pad Delay exist in TMCDB)              #"
print "#################################################\033[0m"


print "TMCDB Pad delay is:        %s (s) " % (padDelay)


print "Suggested DRX Pad Delay is %s (s)" % str(suggested_pad_delay_float_s)



print "\033[90m################################################"
print "# Tilt Term Check                              #"
#print "# (For new pads Tilt may not exist in TMCDB#"
#print "# You will update it in  the next steps)   # "
print "################################################\033[0m "


##ant_metro, an0_tmcdb,aw0_tmcdb,an0_acu,aw0_acu=check_metrology(ant)
#
#
print "\033[90m################################################"
print "## Check Tilt Terms in TMCDB and ACU           #"
print "(Just for reference, not important             #"
print " before starting integration                   #"
print "################################################\033[0m"
print "             TMCDB                ACU    " 
print "         AN0     AW0             AN0   AW0"
print "%s     %s   %s           %s  %s"% (ant_metro, an0_tmcdb,aw0_tmcdb,an0_acu,aw0_acu)

#print "TMCDB Pad tilts are: " 
#print "AN0=%s " % tilts[0]
#print "AW0=%s " % tilts[1]


print "\033[90m#################################################"
print "# Check Metrology                               #" 
print "#################################################\033[0m"

metro_spec=''
if re.search('CM', ant):
        metro_spec="[38, 3, 0, 0]"
if re.search('PM', ant):
        metro_spec="[23, 3, 0, 0]"
if re.search('DV', ant):
        metro_spec="[66, 0, 0, 0]"
if re.search('DA', ant):
        metro_spec="[12, 0, 0, 0]"

print check_metro_result

print "For this antenna type, it should be %s " % metro_spec

print "\033[90mxs#################################################"
print "# To Update Pad Delay based on DRX value        #"
print "#################################################\033[0m"

print "Run :\n"
print "EditFirstPadDelay.py -a %s -p %s -d %s" %(ant,pad,str(suggested_pad_delay_float_s))

print "\n And then Run ImportDelayModel.py \n"

print "\033[0m #################################################\033[0m"
print "\033[0m #################################################\033[0m"
