#!/usr/bin/env python

"""
asdmRadioPoint.py

purpose:       To extract pointing results, done by TelCal, from the ASDM table 
               CalPointingTable

usage  :       ./asdmRadioPoint.py -f uid___X5e_X5c70_X1 -a AntennaName
               where uid___X5e_X5c70_X1 is the name of an ASDM with pointing 
               information and AntennaName is the antenna name. Until we
               an ASDM interface available for general use, it has to be run on a STE. 
               Not sure if it will run a reduction machine.

written:       2009/09/14 by Paulo Cortes

modifications: -a option added to select antenna name for interferometric pointing 
                  observations.

"""


#Python imports
import string as st
import math
import sys
from AsdmObject import *
from optparse import OptionParser


# Initialize parser
parser = OptionParser()
parser.add_option("-f", "--file", dest="file",
                  help="Input file to be analysed")

parser.add_option("-a", "--antenna", dest="antennaName",
                  help="name for the antenna to get data from")

(options, args) = parser.parse_args()

#Check if antenna name was specified
if (options.antennaName is None):
   print 'Antenna Name should be input'
   sys.exit()

asdm    = options.file
antennaName = options.antennaName

print asdm

fName1    = str(asdm) + '.tpoint'
fdo1      = open(fName1,'w')
table     = AsdmObject(asdm)
header1_1 = '!\n! Test Data for PointingModelResult tpoint \n! \n\nALMA Operation Support Facility (OSF) \n: ALTAZ \n: ALLSKY \n'
# Latirue is mandatory in the RUN PARAMETERS record in the tpoint file
# The other parameters are omitted while I lear how to get them out of the asmd
header1_2 = '-23 4 21.5855  ! 2080 06 11   +0  760  2900 0.2 \n\n'

header2_1 = 'AZ_Relative    EL_Relative    AZ_Absolute   EL_Absolute   AZ_Error   EL_Errror   SQR_Error\n '
header2_2 = '[arcsec]       [arcsec]       [arcsec]      [arcsec]      [arcsec]   [arcsec]    [arcsec] \n\n '
fdo1.write(header1_1)
fdo1.write(header1_2)


# The lowest admisible  error is 0.5" and the maximum error is 2"
# A 2" error => 1 entry, but 1/(2**2) is 0.25 => a factor of 4 must be used
# to set the number od iterations to 1. In the same way the lowest error
# (0.5") will imply 16 iterations or 16 entries in the file.
for i,row in enumerate(table.getCalPointing()):
    #The mean square error in arcsec by using the rad2arcsec = 206265
    if (antennaName in row.antennaName): 
        print i, row.antennaName, antennaName
        offError = math.sqrt(float(row.collError[0][0])**2+float(row.collError[0][1])**2)*206265
        if ( (offError > 0.5) and (offError < 2)):
              niter = 1
        elif (offError < 0.5):
              niter = 1
        elif (offError > 2):
              niter = 0

        El_1 = float(row.direction[1])
        Az_1 = float(row.direction[0])
        El_2 = float(row.direction[1]) + float(row.collOffsetAbsolute[0][1])
        Az_2 = float(row.direction[0]) + float(row.collOffsetAbsolute[0][0])/math.cos(El_1)

        El_1 = math.degrees(El_1)
        El_2 = math.degrees(El_2)
        Az_1 = math.degrees(Az_1)
        Az_2 = math.degrees(Az_2)

        sigma = offError/3600
        line1 = str(Az_1) + ' ' + str(El_1) + ' ' + str(Az_2) + ' ' + str(El_2) + ' ' +\
                str(float(row.collError[0][0])*206265)+' '+str(float(row.collError[0][1])*206265)+' '+str(offError)+'\n'
        line2 = str(Az_1) + ' ' + str(El_1) + ' ' + str(Az_2) + ' ' + str(El_2) + ' ' +'\n'
        if (niter != 0):
           for j in range(niter):
               fdo1.write(line2)

fdo1.close()
