import copy
import matplotlib 
import numpy
from numpy import *
import time as tim

def index(arr,num):   # returns the first index in array arr with the value of num
    for index in (i for i in xrange(len(arr)) if arr[i] == num):
	return index

def smoothTriangle(data,degree,dropVals=False):
        """performs moving triangle smoothing with a variable degree."""
        """note that if dropVals is False, output length will be identical
        to input length, but with copies of data at the flanking regions"""
	if degree == 1: return data
        triangle=numpy.array(range(degree)+[degree]+range(degree)[::-1])+1
        smoothed=[]
        for i in range(degree,len(data)-degree*2):
                point=data[i:i+len(triangle)]*triangle
                smoothed.append(sum(point)/sum(triangle))
        if dropVals: return smoothed
        smoothed=[smoothed[0]]*(degree+degree/2)+smoothed
        while len(smoothed)<len(data):smoothed.append(smoothed[-1])
        return smoothed

def bp_scan(invis='',load='',plt='',Nant='',smoot='',bins=''):

# makes plots and calculates variations of TP AC or XC ampl or phase for all antennas in a ms.
# 
#  to run, start casapy, execfile('bandpass.py')
#  bp_scan(invis='ms_name',load='source',plt="Xdamp",Nant=4,smoot=1,bins=1)      for example
#
#  data will be writen to BPresults_ms_name.txt and to plot files.
#
#  load can be: hot, amb, source, zero or ref. tested on amb, hot and source.

#  spw 1,3,5,7 are assumed to be the ones to look at
#
#  plt should be amp for autocorr (AC) total power (TP) plots
#          damp for differential TP plots, where damp = p(i,t)/p_t_mean(i) - p_it_mean
#          Xamp, Xphase for X-correlated amp or phase
#          Xdamp, Xdphase differential amp or phase
# Xdamp will plot rms of differential vs time, where differential is difference between first sample
#   and sample at time t.
#
# smoot is the spectral smoothing (in channels)
# bins is the sample smoothing (in samples) (ie time)
# 
# Nant is number of antennas to be looked at. for AC is will cycle through the first Nant antennas
#    for XC is will cycle through all baselines for the first Nant antennas
#
#	pl.show()   *** may need to type this to get the plotting window to display!

# specifications for total power autocorrelation is 0.001 per 3600 seconds (arbitrarily set - check with ALMA doc)
	specsec_AC_hr = 0.001		# max variation per hour assuming constant drift

# specifications for cross correlation is 0.001 per 3600 seconds (see ALMA doc)
	specsec_XC_hr = 0.001		# max variation per hour assuming constant drift

# specifications for cross correlation is 0.01 degrees per 3600 seconds (see ALMA doc)
	specsec_XC_p_hr = 0.01		# max variation per hour assuming constant drift

# NB: set norm =Y to normalize to the average power level, N to not do this (in function)


	fileout="BPresults_"+invis.split('.ms')[0]+".txt"
	outfile=open(fileout,"w")   # filename to write Bandpass results

	fileout2="TPresults_"+invis.split('.ms')[0]+".txt"
	outfile2=open(fileout2,"w")   # filename to write Total Power results



#	smoot=1     # spectral smoothing (channels)
#	bins=1     # binning in samples (time)

#	invis='uid___A002_X122cc8_X35.ms'
	print " calculating ", plt, " for ",invis," for ",Nant," antennas on load ",load
	print " spectral smoothing ", smoot," time binning ", bins

	if plt[0]=="X":
	  print " running ",(Nant*(Nant-1)/2)," baselines in cross-correlation"
	  if Nant < 2:
	     print " ERROR. not enough antennas"
	     return
	  for i in range (0,Nant):
	     for j in range ((i+1),Nant):
		for spw in range (1,9,2):
		   bandpass(invis=invis,spw=spw,ant1=i,ant2=j,load=load,smoot=smoot,bins=bins,plt=plt,outfile=outfile,outfile2=outfile2)
	else:
	  print " running ",Nant,"antennas in autocorrelation"
	  for i in range (0,Nant):
		for spw in range (1,9,2):
		  bandpass(invis=invis,spw=spw,ant1=i,ant2=i,load=load,smoot=smoot,bins=bins,plt=plt,outfile=outfile,outfile2=outfile2)

	outfile.close()
	outfile2.close()


def bandpass(invis='',spw='',ant1='',ant2='',load='',smoot='',bins='',plt='',outfile='',outfile2=''):

	norm="Y"      # set to Y to nromlise the spectra by the total power before caliculatign the differential
	cal="N"       # set to Y to include calibration  Tsys. multiplies the signal by the geometric mean of Tsys for the 2 antennas


	if plt[0]=="X" and ant1==ant2:
	   print "WARNING - trying to plot cross-correlation of same antenna!"
#	pl.ion()

# specifications for total power autocorrelation is 0.001 per 3600 seconds (arbitrarily set - check with ALMA doc)
	specsec_AC_hr = 0.001		# max variation per hour assuming constant drift

# specifications for cross correlation is 0.001 per 3600 seconds (see ALMA doc)
	specsec_XC_hr = 0.001		# max variation per hour assuming constant drift

# specifications for cross correlation is 0.01 degrees per 3600 seconds (see ALMA doc)
	specsec_XC_p_hr = 0.01		# max variation per hour assuming constant drift


	print "------------------\n ms name : ",invis
	tb.open(invis) 

	Nscan=1
	antenna1=ant1
	antenna2=ant2

# find antenna name:
	dataDescTb=tbtool.create() 
	dataDescTb.open(invis+'/ANTENNA')
	ant_ID=dataDescTb.getcol('NAME')
	antname1=ant_ID[ant1]
	antname2=ant_ID[ant2]
	print " antennas ", antname1,antname2,"  spw ",spw


#	tb1=tb.query("SCAN_NUMBER==%d && ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0" % (Nscan,antenna1,antenna2))
	tb1=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0" % (antenna1,antenna2))

# find time range:
	tb1=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0" % (antenna1,antenna2))
	time=tb1.getcol('TIME')
	timerange = time[-1]-time[0]   # seconds
	print " total time ",timerange, " sec"

# next find which SPW ID's correspond to this:
	dataDescTb=tbtool.create()
	dataDescTb.open(invis+'/DATA_DESCRIPTION')
	spw_ID = dataDescTb.getcol('SPECTRAL_WINDOW_ID')
        pol_ID = dataDescTb.getcol('POLARIZATION_ID')
	spwTb = tbtool.create()
	spwTb.open(invis+'/SPECTRAL_WINDOW')

   
	for i in range(0,len(spw_ID)):
	        numChan = spwTb.getcell('NUM_CHAN',spw_ID[i])
	        print "Spectral Window %d has %d channels" % (spw_ID[i],numChan)

	stateTb = tbtool.create()
	stateTb.open(invis+'/STATE')


	dd_ID=index(spw_ID,spw)  # picks out correct spw

# look for state ID's for different loads

# load can be ref, amb, hot, zero, source:
#	st_ID is different for zero, ambient, hot, skyref, source
# assumption is that zero, skyref and source only have one state_ID, and that
#       amb and hot can have many st_ID's


	if load=="amb":
	  subscan=2   # with several different st_ID. need to find them
	  ss2=[]
#	  print stateTb.nrows(), " amb rows"
	  for i in range (0,stateTb.nrows()):
#		print i, stateTb.getcell('LOAD',i),stateTb.getcell('OBS_MODE',i).split(',')[0]
		if stateTb.getcell('LOAD',i) > 10.0 and stateTb.getcell('LOAD',i) < 30.0:  # load temp = ambient
#		   print i," is a valid state ID for amb"
		   tb2=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID==0 && STATE_ID==%d" % (antenna1,antenna2,i))
#		   print tb2.nrows(), " rows"
		   if tb2.nrows() > 0:
			ss2.append(i)   # this is the list of valid state ID's for amb
#	                print "revised ss2 ",ss2, tb2.nrows(), " rows"

	elif load=="hot":
	  subscan=3   # with several different st_ID. need to find them
	  ss3=[]
#	  print stateTb.nrows(), " hot rows"
	  for i in range (0,stateTb.nrows()):
#		print i, stateTb.getcell('LOAD',i),stateTb.getcell('OBS_MODE',i)
		if stateTb.getcell('LOAD',i) > 60.0 and stateTb.getcell('LOAD',i) < 90.0:
		   tb2=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID==0 && STATE_ID==%d" % (antenna1,antenna2,i))
#		   print i," is a valid state ID for hot"
#		   print tb2.nrows(), " rows"
		   if tb2.nrows() > 0:
			ss3.append(i)  # this is the list of valid state ID's for hot
#	                print "revised ss3 ",ss3, tb2.nrows()," rows"

	elif load=="source":
	  print stateTb.nrows(), " source row(s)"
	  for i in range (0,stateTb.nrows()):
		print i, stateTb.getcell('LOAD',i),stateTb.getcell('OBS_MODE',i).split(',')[0]
		if stateTb.getcell('LOAD',i) <0.5 and stateTb.getcell('OBS_MODE',i).split(',')[0] == "CALIBRATE_DELAY#ON_SOURCE":
		   print i," is a valid state ID for source"
	  	   st_ID=i
	  subscan=1

	elif load=='ref':
	  for i in range (0,stateTb.nrows()):
#		print i, stateTb.getcell('LOAD',i),stateTb.getcell('OBS_MODE',i).split(',')[0]
		if stateTb.getcell('LOAD',i) <0.5 and stateTb.getcell('OBS_MODE',i).split(',')[0] == "CALIBRATE_ATMOSPHERE#ON_SOURCE":
#		   print i," is a valid state ID for ref"
	  	   st_ID=i
	  subscan=1

	elif load=="zero":
	  for i in range (0,stateTb.nrows()):
#		print i, stateTb.getcell('LOAD',i),stateTb.getcell('OBS_MODE',i).split(',')[0]
		if stateTb.getcell('LOAD',i) <0.5 and stateTb.getcell('OBS_MODE',i).split(',')[0] == "CALIBRATE_ATMOSPHERE#OFF_SOURCE":
#		   print i," is a valid state ID for zero"
		   st_ID=i
	  subscan=4
	else:
	  print " load is not amb, hot, source, ref or zero "
	  return

	print "st_ID:",st_ID

# now extract the data:

	if load=="ref" or load=="source" or load=="zero":
		print "st_ID : ", st_ID, "load:", load, "  ", antname1
		print antenna1,antenna2,dd_ID,st_ID
#		tb2=tb.query("SCAN_NUMBER=%d && ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0 && DATA_DESC_ID==%d && STATE_ID==%d" % (0,antenna1,antenna2,dd_ID,st_ID))
# use the above for only one scan, otherwise comment out and use the next line:
		tb2=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0 && DATA_DESC_ID==%d && STATE_ID==%d" % (antenna1,antenna2,dd_ID,st_ID)) 
		on1=tb2.getcol('DATA')
#		print on1
	elif load=="amb":
		for i in range (0,(len(ss2))):
			st_ID=ss2[i]
#			print i,st_ID
			tb2=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0 && DATA_DESC_ID==%d && STATE_ID==%d" % (antenna1,antenna2,dd_ID,st_ID)) 
			if i==0:
			  on1=tb2.getcol('DATA')
#			  print "1st one"
			else:
			  on1=numpy.append(on1,tb2.getcol('DATA'),2)
#			  print " appended 1 "
#			print " new shape ",on1.shape
			scann=tb2.getcol('SCAN_NUMBER')
#			print 'scan number    :', scann


	elif load=="hot":
#		print "len ss3 ", len(ss3)
		for i in range (0,(len(ss3))):
			st_ID=ss3[i]
#			print "i st_ID ", i,st_ID
#			tb2=tb.query("SCAN_NUMBER=%d && ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0 && DATA_DESC_ID==%d && STATE_ID==%d" % (1,antenna1,antenna2,dd_ID,st_ID))
#			print "reading only SCAN 1"
			tb2=tb.query("ANTENNA1==%d && ANTENNA2==%d && PROCESSOR_ID=0 && DATA_DESC_ID==%d && STATE_ID==%d" % (antenna1,antenna2,dd_ID,st_ID)) 
			if i==0:
			  on1=tb2.getcol('DATA')
#			  print " 1st one"
			else:
			  on1=numpy.append(on1,tb2.getcol('DATA'),2)
#			  print " appended 1"
#			print " new shape ",on1.shape
			scann=tb2.getcol('SCAN_NUMBER')
#			print 'scan number    :', scann


	print "on1 shape:", on1.shape
        npol=on1.shape[0]
        nchan=on1.shape[1]
	n_samples=on1.shape[2]

	if cal == "Y":
	   print " now getting Tsys data"
	   tsys1=aU.TsysExplorer(invis,antenna=antenna1,spwID=spw)
	   tsys2=aU.TsysExplorer(invis,antenna=antenna2,spwID=spw)
	   print tsys1.tsys.shape
	   tsys_m=(tsys1.tsys[:,:,:]*tsys2.tsys[:,:,:])**0.5    # takes geometric mean of 2 antennas

	   print " multiplying all on1 data by closest Tsys"
	   sysdiv=float(n_samples)/tsys_m.shape[2]-0.00001
	   for i in range(0,n_samples):
		itsys=trunc(float(i)/sysdiv)
#		print sysdiv
#		print i, itsys
		on1[:,:,i]=on1[:,:,i]*tsys_m[:,:,itsys]



        print " number of polarisations: ",npol
        print " number of raw data channels: ",nchan
	if smoot > nchan/10:
	    print " ERROR: too much spectral smoothing!!  smoot= ",smoot
	    return
#	n_samples=20
	print " number of raw data samples: ", n_samples
	if bins > n_samples:
	    print " ERROR: number of samples < sample binning!!"
	    return


# option to bin the data in time
#	bins=16
	timestep=timerange/n_samples   # value in raw data

	npchan=1
	if npol == 2:
	   npchan=2
	elif npol == 4:
	   npchan=2
#	print "npchan=", npchan
# for 2-pol, use 0 and 1. for 4-pol use 0 and 2:

	on= zeros([npchan,nchan,(n_samples/bins)],complex)    # create array for the binned data

#	print "shapes ", on1.shape, on.shape
	ns=0
	for i in range (0,(n_samples),bins):
	  if ns < (n_samples/bins):
#	    print i,ns,on.shape, on1.shape
	    on[0,:,ns]=on1[0,:,i]
	    on[1,:,ns]=on1[(npol-1),:,i]
#	    on[1,:,ns]=on1[3,:,i]
	    for j in range ((i+1),(i+bins)):
#	      print i,j,ns
	      if j < (n_samples-1):
		on[0,:,ns]=on1[0,:,j]+on[0,:,ns]          # 1st poln
		on[1,:,ns]=on1[(npol-1),:,j]+on[1,:,ns]   # 2nd poln
#		on[1,:,ns]=on1[3,:,j]+on[1,:,ns]   # 2nd poln

	    on[0,:,ns]=on[0,:,ns]/bins
	    on[1,:,ns]=on[1,:,ns]/bins
	    ns=ns+1
        if bins>1:
           n_samples=ns        # misses out last (incomplete) binned sample
        print " final number of samples (after possible binning): ",n_samples," = ~",(timerange/n_samples)," sec per sample"
	timestep=timestep*bins

# option to drop edge channels:
        nedge=8
	if nedge > 0:
	   print " dropping ",nedge," channels from spectra & Tsys"
           on=on[:,nedge:-nedge,:]
	   if cal == "Y": tsys_m=tsys_m[:,nedge:-nedge,:]
	nchan=nchan-2*nedge


#	   for i in range (0,nedge):
#		on[:,i,:]=on[:,(i+nedge),:]
#		on[:,(nchan-i-2*nedge),:]=on[:,(nedge-i-1),:]


	on_m=on.mean(2)  # mean value over all the samples
#	print on_m.shape, on.shape
#        print on1[0,0,0], on[0,0,0]


  # to initialise the arrays
	onshape=[on.shape[0],on.shape[1],on.shape[2]]
	on_diff= zeros(onshape,complex)
	on_Xp=zeros(onshape,float)
#	on_sm_Xp=empty(on.shape)
#	on_sm_Xa=empty(on.shape)
	on_sm=zeros(onshape,complex)

	on_diff_Xa=zeros(onshape,float)
	on_diff_Xp=zeros(onshape,float)

	on_sm_diff= zeros(onshape,complex)
	on_sm_diff_Xa=zeros(onshape,float)
	on_sm_diff_Xp=zeros(onshape,float)



 # first caliculate arrays of amp & phase:
	for i in range (0,n_samples):    
	  on_Xa=absolute(on)       # amplitude (for X corr)
	  for ii in range(0,nchan):
	  	on_Xp[0,ii,i]=math.atan2(on[0,ii,i].imag, on[0,ii,i].real)*57.29     # X phase, for cross corr
 	  	if npol>1:
		   on_Xp[1,ii,i]=math.atan2(on[1,ii,i].imag, on[1,ii,i].real)*57.29     # Y phase, for cross corr

	on_m_Xa=on_Xa.mean(2)    # mean values over all samples
	on_m_Xp=on_Xp.mean(2)
#	print "on ",on[0,:,1]
#	print "on Xa ",on_Xa[0,:,1]
#	print "on mean Xa ",on_m_Xa[0,:]


#	    on_sm_Xa[ii,:,i]=smoothTriangle(on_Xa[ii,:,i],smoot)   # smooth amp
#	    on_sm_Xp[ii,:,i]=smoothTriangle(on_Xp[ii,:,i],smoot)   # smooth phase

#	print on_sm.shape, on_smm.shape
#	print "on sm mean Xa ",on_smm_Xa[0,:]


	print " calculating differentials. Normalisation =",norm
# these are the difference between the observed TP, X-corr amp & phase and the mean value over all the samples.
# looking at the changes in the bandpass over the sampling time.
	for i in range (0,n_samples):
	 for ii in range(0,2):

# first A/C amp
	  if norm=="Y":
	    mn=(on[ii,:,i].mean())/(on_m[ii,:].mean())
#	    print "mean ",mn
	  else:
	    mn=0.0
	  on_diff[ii,:,i]=on[ii,:,i]/on_m[ii,:] - mn  #  \normalised by subtracting mean


# next X/C amp:
	  if norm=="Y":
	    mn=(on_Xa[ii,:,i].mean())/(on_m_Xa[ii,:].mean())
#	    print ii, "mean ",mn
	  else:
	    mn=0.0
	  on_diff_Xa[ii,:,i]=on_Xa[ii,:,i]/on_m_Xa[ii,:] -mn  #  \normalised by subtracting mean

# next X/C phase:
	  if norm=="Y":
	    mn=(on_Xp[ii,:,i].mean())/(on_m_Xp[ii,:].mean())
	  else:
	    mn=0.0
	  on_diff_Xp[ii,:,i]=on_Xp[ii,:,i]/on_m_Xp[ii,:] - mn #     this probably won't work if phase wraps
#	  print "mean ",mn


#	  print on_diff
#          print "pol 1 maxmin", max(on[1,:,i]), min(on[1,:,i])
#          print i


	if plt=="amp":
	   print " spectral smoothing of data by factor ",smoot
	   for i in range (0,n_samples):
		if i== n_samples/2: print "50% complete"
		for ii in range(0,2):
		     on_sm[ii,:,i]=smoothTriangle(on[ii,:,i],smoot)    # smmoths data

	   print "calculating total power max/min"
	   on_mm0=( on_sm[0,:,1].max().real / on_sm[0,:,1].min().real )
#           print   on_sm[0,:,1].max().real, on_sm[0,:,1].min().real
	   on_mm0=log10(on_mm0)*10.0
	   on_mm1=( on_sm[1,:,1].max().real / on_sm[1,:,1].min().real )
	   on_mm1=log10(on_mm1)*10.0
	   print "          TP range Pol0, Pol1: ",on_mm0,on_mm1," dB"
	   if (on_mm0 > 8.0) or (on_mm1 > 8.0):
		print "          *****TP out of range of 8dB"


	if plt=="damp":

		print " spectral smoothing of data by factor ",smoot
		for i in range (0,n_samples):
		  if i== n_samples/2: print "50% complete"
		  for ii in range(0,2):
		    on_sm[ii,:,i]=smoothTriangle(on[ii,:,i],smoot)    # smmoths data

        	on_diff_p0=on_diff[0,:,:]
		on_diff_p1=on_diff[1,:,:]

		onshape2=[on.shape[1],on.shape[2]]		# create arrays
		on_sm_diff_p0=zeros(onshape2,float)
		on_sm_diff_p1=zeros(onshape2,float)
		on_sm_diff_p0_dev=zeros(onshape2,float)
		on_sm_diff_p1_dev=zeros(onshape2,float)
		rms0=zeros(n_samples)
		rms1=zeros(n_samples)
		rmstime=zeros(n_samples)

		print " smoothing results by factor ",smoot
		for i in range (0,n_samples):
		  if i== n_samples/2: print "50% complete"
		  on_sm_diff_p0[:,i]=smoothTriangle(on_diff_p0[:,i],smoot)    # smmoths data
		  on_sm_diff_p1[:,i]=smoothTriangle(on_diff_p1[:,i],smoot)    # smmoths data

#		print "on_diff ",on_diff_p0.shape, "max ",numpy.max(on_diff_p0)
#		print "on_sm_diff ",on_sm_diff_p0.shape, "max ",numpy.max(on_sm_diff_p0)

		hourflag=0
		for i in range(0,n_samples):
		   on_sm_diff_p0_dev[:,i]=on_sm_diff_p0[:,0]-on_sm_diff_p0[:,i]
		   on_sm_diff_p1_dev[:,i]=on_sm_diff_p1[:,0]-on_sm_diff_p1[:,i]
		   rms0[i]=numpy.std(on_sm_diff_p0_dev[:,i],axis=0)
		   rms1[i]=numpy.std(on_sm_diff_p1_dev[:,i],axis=0)
		   rmstime[i]=i*timestep
		   if rmstime[i] > 3600.0 and hourflag == 0:
			hourflag=1
			rms0_hour=rms0[i]
			rms1_hour=rms1[i]
#		   print " pol0 ",i,(i*timestep),rms0[i]
#		   print " pol1 ",i,(i*timestep),rms1[i]


#		print on_diff_Xa_p0

		if hourflag == 1:     # reached 1 hour in data
		  print " "
		  print " Pol0 rms fractional change in AC per hour:",rms0_hour
		  print " Pol1 rms fractional change in AC per hour:",rms1_hour

		  if rms0_hour > specsec_AC_hr :
		   print "   *** Pol 0 out of AC amp spec"
		  if rms1_hour > specsec_AC_hr :
		   print "   *** Pol 1 out of AC amp spec"
		  print " "
		else:
		  print " WARNING - dataset time interval is ",rmstime[-1], "-> less than 1 hour"
		  rms0_hour=rms0[-1]
		  rms1_hour=rms1[-1]
		  print " Pol0 rms fractional change in AC:",rms0_hour
		  print " Pol1 rms fractional change in AC:",rms1_hour

		  if rms0_hour > specsec_AC_hr :
		   print "   *** Pol 0 out of AC amp spec, for t<1hr"
		  if rms1_hour > specsec_AC_hr :
		   print "   *** Pol 1 out of AC amp spec, for t<1hr"
		  print " "



	if plt=="Xamp" or plt=="Xphase":
		
		on_sm_Xa=zeros(onshape,float)	# create arrays
		on_sm_Xp=zeros(onshape,float)


		print " spectrally smoothing data by factor ",smoot
		for i in range (0,n_samples):
		  if i== n_samples/2: print "50% complete"
		  for ii in range(0,2):
		    on_sm_Xa[ii,:,i]=smoothTriangle(on_Xa[ii,:,i],smoot)    # smmoths data
		    on_sm_Xp[ii,:,i]=smoothTriangle(on_Xp[ii,:,i],smoot)    # smmoths data
        	on_sm_Xa_p0=on_sm_Xa[0,:,:]
		on_sm_Xa_p1=on_sm_Xa[1,:,:]
        	on_sm_Xp_p0=on_sm_Xp[0,:,:]
		on_sm_Xp_p1=on_sm_Xp[1,:,:]



	if plt=="Xdamp":
        	on_diff_Xa_p0=on_diff_Xa[0,:,:]
		on_diff_Xa_p1=on_diff_Xa[1,:,:]


		onshape2=[on.shape[1],on.shape[2]]		# create arrays
		on_sm_diff_Xa_p0=zeros(onshape2,float)
		on_sm_diff_Xa_p1=zeros(onshape2,float)
		on_sm_diff_Xa_p0_dev=zeros(onshape2,float)
		on_sm_diff_Xa_p1_dev=zeros(onshape2,float)
		rms0=zeros(n_samples)
		rms1=zeros(n_samples)
		rmstime=zeros(n_samples)
		
#		print on_diff_Xa_p0[:,0].shape, "max ",numpy.max(on_diff_Xa_p0)
#		print on_diff_Xa_p1[:,0].shape, "max ",numpy.max(on_diff_Xa_p1)

		print " spectrally smoothing results by factor ",smoot
		for i in range (0,n_samples):
		  if i== n_samples/2: print "50% complete"
		  on_sm_diff_Xa_p0[:,i]=smoothTriangle(on_diff_Xa_p0[:,i],smoot)    # smmoths
		  on_sm_diff_Xa_p1[:,i]=smoothTriangle(on_diff_Xa_p1[:,i],smoot)    # smmoths 

#		print on_sm_diff_Xa_p0[:,0].shape, "max0 ",numpy.max(on_sm_diff_Xa_p0)
#		print on_sm_diff_Xa_p1[:,0].shape, "max1 ",numpy.max(on_sm_diff_Xa_p1)

		hourflag=0
		for i in range(0,n_samples):
		   on_sm_diff_Xa_p0_dev[:,i]=on_sm_diff_Xa_p0[:,0]-on_sm_diff_Xa_p0[:,i]
		   on_sm_diff_Xa_p1_dev[:,i]=on_sm_diff_Xa_p1[:,0]-on_sm_diff_Xa_p1[:,i]
		   rms0[i]=numpy.std(on_sm_diff_Xa_p0_dev[:,i],axis=0)
		   rms1[i]=numpy.std(on_sm_diff_Xa_p1_dev[:,i],axis=0)
		   rmstime[i]=i*timestep
		   if rmstime[i] > 3600.0 and hourflag == 0:
			hourflag=1
			rms0_hour=rms0[i]
			rms1_hour=rms1[i]
#		   print " pol0 ",i,(i*timestep),rms0[i]
#		   print " pol1 ",i,(i*timestep),rms1[i]


#		print on_diff_Xa_p0

		if hourflag == 1:     # reached 1 hour in data
		  print " "
		  print " Pol0 rms fractional change in XC per hour:",rms0_hour
		  print " Pol1 rms fractional change in XC per hour:",rms1_hour

		  if rms0_hour > specsec_XC_hr :
		   print "   *** Pol 0 out of XC amp spec"
		  if rms1_hour > specsec_XC_hr :
		   print "   *** Pol 1 out of XC amp spec"
		  print " "
		else:
		  print " WARNING - dataset time interval is ",rmstime[-1], "-> less than 1 hour"
		  rms0_hour=rms0[-1]
		  rms1_hour=rms1[-1]
		  print " Pol0 rms fractional change :",rms0_hour
		  print " Pol1 rms fractional change :",rms1_hour

		  if rms0_hour > specsec_XC_hr :
		   print "   *** Pol 0 out of XC amp spec, for t<1hr"
		  if rms1_hour > specsec_XC_hr :
		   print "   *** Pol 1 out of XC amp spec, for t<1hr"
		  print " "


	if plt=="Xdphase":
        	on_diff_Xp_p0=on_diff_Xp[0,:,:]
		on_diff_Xp_p1=on_diff_Xp[1,:,:]

		on_sm_diff_Xp_p0=empty(on_diff_Xp_p0.shape)		# create arrays
		on_sm_diff_Xp_p1=empty(on_diff_Xp_p0.shape)

		print " smoothing results by factor ",smoot
		for i in range (0,n_samples):
		  if i== n_samples/2: print "50% complete"
		  on_sm_diff_Xp_p0[:,i]=smoothTriangle(on_diff_Xp_p0[:,i],smoot)    # smmoths
		  on_sm_diff_Xp_p1[:,i]=smoothTriangle(on_diff_Xp_p1[:,i],smoot)    # smmoths 

#		print on_diff_Xp_p0

		print " "
        	print "  pol 0 maxmin differential", numpy.max(on_sm_diff_Xp_p0), numpy.min(on_sm_diff_Xp_p0)
		dfdt0=(numpy.max(on_sm_diff_Xp_p0) - numpy.min(on_sm_diff_Xp_p0))/(2.*sqrt(2))* 3600.0/timerange
		print " rms fractional change per hour:",dfdt0
		if (numpy.max(on_sm_diff_Xp_p0) - numpy.min(on_sm_diff_Xp_p0))/timerange > specsec_XC_p_hr :
		   print "   *** Pol 0 out of XC phase spec"
        	print "  pol 1 maxmin differential", numpy.max(on_sm_diff_Xp_p1), numpy.min(on_sm_diff_Xp_p1)
		dfdt1=(numpy.max(on_sm_diff_Xp_p1) - numpy.min(on_sm_diff_Xp_p1))/(2.*sqrt(2))* 3600.0/timerange
		print " rms fractional change per hour:",dfdt1
		if (numpy.max(on_sm_diff_Xp_p1) - numpy.min(on_sm_diff_Xp_p1))/timerange > specsec_XC_p_hr :
		   print "   *** Pol 1 out of XC phase spec"
		print " "



        if plt=="amp":
#       write TP data to file:
		line=invis.split('.ms')[0]+" "+antname1+antname2+" "+str(spw)+" "+load+" X "+str(on_mm0)+"\n"
#		print line
		outfile2.write(line)
		line=invis.split('.ms')[0]+" "+antname1+antname2+" "+str(spw)+" "+load+" Y "+str(on_mm1)+"\n"
#		print line
  		outfile2.write(line)


        if plt=="damp" or plt=="Xdamp" or plt=="Xdphase":
#       write data to file:
		line=invis.split('.ms')[0]+" "+antname1+antname2+" "+str(spw)+" "+load+" X "+str(rms0_hour)+"\n"
#		print line
		outfile.write(line)
		line=invis.split('.ms')[0]+" "+antname1+antname2+" "+str(spw)+" "+load+" Y "+str(rms1_hour)+"\n"
#		print line
  		outfile.write(line)


	refF=spwTb.getcol('REF_FREQUENCY')
	chanF=refF[spw] - spwTb.getcell('CHAN_FREQ',spw)   # this is now IF baseband frequency (ie 0-2GHz)

	if nedge > 0:   # optionally drop channels from freq array as well
           chanF=chanF[nedge:-nedge]   


#	pl.clf()

	#pl.subplot(211)
	#pl.plot(chanF,on_diff[0,:,:])
	#pl.subplot(212)
	#pl.plot(chanF,on_diff[1,:,:])
##
	spw_str=str(spw)
	titl=invis.split('.ms')[0]+" "+antname1+antname2+" SPW:"+spw_str+" "+load

#	print titl
#        pl.axes(autoscale_on=T)

	if plt == "amp":
	  pl.clf()
	  pl.subplot(211,autoscale_on=T)
	  pl.plot(chanF,on_sm[0,:,:])
	  pl.title(titl+" X TP")
	  pl.subplot(212)
	  pl.title(titl+" Y TP")
	  pl.plot(chanF,on_sm[1,:,:])
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_AC_amp.png"
	  pl.draw()
	  pl.savefig(plotfile)

#	  pl.clf()
#	  pl.subplot(211,autoscale_on=T)
#	  pl.plot(tsys_m.tsys[0,:,1],tsys_m.freq[0,:,1])
#	  pl.title(titl+" X Tsys")
#	  pl.subplot(212)
#	  pl.title(titl+" Y Tsys")
#	  pl.plot(tsys_m.tsys[1,:,1],tsys_m.freq[1,:,1])
#	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_AC_amp.png"
	  pl.draw()
#	  pl.savefig(plotfile)


        if plt == "damp":
	  pl.clf()
	  pl.subplot(211,autoscale_on=T)
#	  print "maxmin ", on_sm_diff_p0[:,:].max(), on_sm_diff_p0[:,:].min()
#	  for i in range (0,on_sm_diff_p0.shape[1]):
#	          print on_sm_diff_p0[:,i]
#		  print "i, maxmin ",i, on_sm_diff_p0[:,i].max(), on_sm_diff_p0[:,i].min()
#		  print on_sm_diff_p0.shape
	  pl.plot(chanF,on_sm_diff_p0[:,:])
	  pl.title(titl+" X dTP")
#	  pl.text(0.5,0.95,'spec',color='red',horizontalalignment='center',verticalalignment='top')
#	  pl.ylim(-0.01,0.01)
	  pl.subplot(212)
	  pl.plot(chanF,on_sm_diff_p1[:,:])
	  pl.title(titl+" Y dTP")
#	  pl.ylim(-0.01,0.01)
#          pl.show()
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_AC_diff.png"
	  pl.draw()
	  pl.savefig(plotfile)


        if plt == "Xamp":
	  pl.clf()
	  pl.subplot(211)
	  pl.plot(chanF,on_sm_Xa_p0[:,:])
	  pl.title(titl+" X Xamp")
	  pl.subplot(212)
	  pl.plot(chanF,on_sm_Xa_p1[:,:])
	  pl.title(titl+" Y Xamp")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_XC_amp.png"
	  pl.draw()
	  pl.savefig(plotfile)

        if plt == "Xphase":
	  pl.clf()
	  pl.subplot(211)
	  pl.plot(chanF,on_sm_Xp_p0[:,:])
	  pl.title(titl+" X Xphase")
	  pl.subplot(212)
	  pl.plot(chanF,on_sm_Xp_p1[:,:])
	  pl.title(titl+" Y Xphase")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_XC_phase.png"
	  pl.draw()
	  pl.savefig(plotfile)

        if plt == "Xdamp":
	  pl.clf()
	  pl.subplot(211)
	  pl.plot(chanF,on_sm_diff_Xa_p0[:,:])
	  pl.title(titl+" X Xdamp")
	  pl.subplot(212)
	  pl.plot(chanF,on_sm_diff_Xa_p1[:,:])
	  pl.title(titl+" Y Xdamp")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_XC_damp.png"
	  pl.draw()
	  pl.savefig(plotfile)
	  print " finished plotting Xdamp"
	  tim.sleep(2.0)

	  pl.clf()
	  pl.subplot(211)
	  pl.plot(rmstime,rms0)
	  pl.title(titl+" X Xdamp rms")
	  pl.subplot(212)
	  pl.plot(rmstime,rms1)
	  pl.title(titl+" Y Xdamp rms")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_XC_damp_rms.png"
	  pl.savefig(plotfile)
	  pl.draw()
	  print " finished plotting rms"
	  tim.sleep(0.5)


        if plt == "Xdphase":
	  pl.subplot(211)
	  pl.plot(chanF,on_sm_diff_Xp_p0[:,:])
	  pl.title(titl+" X Xdphase")
	  pl.subplot(212)
	  pl.plot(chanF,on_sm_diff_Xp_p1[:,:])
	  pl.title(titl+" Y Xdphase")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_XC_dphase.png"
	  pl.savefig(plotfile)


	if cal == "Y":
	  pl.subplot(211)
	  pl.plot(chanF,tsys_m[0,:,:])
	  pl.title(titl+" X Tsys")
	  pl.subplot(212)
	  pl.plot(chanF,tsys_m[1,:,:])
	  pl.title(titl+" Y Tsys")
	  plotfile=invis.split('.ms')[0]+"_"+antname1+antname2+"_SPW"+spw_str+"_"+load+"_Tsys.png"
	  pl.savefig(plotfile)

