#!/usr/bin/env python
#
# basex.py   baseline explorer
# To run, extract the ms.
# then run casapy, and execfile('basex.py')
# bb = basex(invis = 'uid://XXX_XXXXX_XXXXX.ms', outp=,pol=)
# bb.main()


import sys
import os
from optparse import OptionParser
#import casac
import pickle
import analysisUtils as aU

from pylab import *
import matplotlib

class basex():

  def __init__(self,invis=None, outp='snr',pol=0):
    '''
    basex --help
    
    This script produces a baseline adjacency matrix
    outp = snr, rms, amp
    pol = 0, 1
    
    '''
    if invis == None:
      print "You must  specify at least an invis.Exiting"
      exit()
    else:
      self.invis = invis
    self.ants=aU.getAntennaNames(invis)
    self.outp= outp
    self.pol  = pol
    
    return
    
  def onclick(self,event):
  #  print 'button=%d, xdata=%f, ydata=%f'%(event.button,event.xdata,event.ydata)
    xant=int(event.xdata)
    yant=int(event.ydata)
    print 'baseline', self.ants[xant],'-',self.ants[yant]
    print 'Amp= %1.3e, RMS =%1.3e, Delay=%3.2f ps '%(self.ampa[xant,yant],self.rmsa[xant,yant],self.delaya[xant,yant])
    sr=figure(num=2)
    clf()
    if xant==yant:
      plot(self.x,self.speca[xant,yant,:],'.',figure=sr)
      ylim([-180,180])
      xlabel('Freq [GHz]');ylabel('Phase/Amp')
      title('Autocorrelation : '+self.ants[xant])
    else:
      plot(self.x,self.speca[xant,yant,:],'.',figure=sr)
      if xant < yant:
        ylim([-180,180])
      xlabel('Freq [GHz]');ylabel('Phase/Amp')
      title('Phase  '+self.ants[xant]+' * '+self.ants[yant])
    return xant,yant

  # start of main script
  def main(self):
    '''
   
    '''
    nants = size(self.ants)
    ndrop=8      # channels to drop for rms calc
    fitslope='y'  # fit a delay slope to get the rms (recommended)

    sr=figure()    # open the spectrum plot window
    clf()
    data = aU.Visibility(self.invis,antenna1=0,antenna2=1,spwID=1,scan=1)
    on=data.specData
    nch=shape(on)[1]
    #print nch
    rmsa=zeros((nants,nants))
    delaya=zeros((nants,nants))
    ampa=zeros((nants,nants))
    speca=zeros((nants,nants,nch))
    vrange='full'   # full range for plots
    vm = aU.ValueMapping(self.invis)
    scans = vm.uniqueScans
    
    # start reading data:
    for scan in scans:
      data.setScan(scan)
      for ant1 in range(nants):
        for ant2 in range(ant1,nants):
          print scan, ant1,ant2
          #k=1 # for k in spwIDs:
          #data.setSpwID(k)
          data.setAntennaPair(ant1,ant2)
          x = data.specFreq[:,0] # frequency axis
          self.x = x
          onphase_r = aU.unwrap(data.tavgPhase[self.pol,:]*180/pi)   # in deg
          onamp_r = data.tavgAmp[self.pol,:]
          speca[ant1,ant2,:]=onphase_r    # one off-diagonal has the amp, the other the phase
          speca[ant2,ant1,:]=onamp_r
          if ant1==ant2:
            speca[ant1,ant2,:]=onamp_r    # diagonal has the autocorr
          if fitslope=='y':
            fit=polyfit(x[ndrop:(nch-ndrop)],onphase_r[ndrop:(nch-ndrop)],1)
            f = fit[1]+ fit[0]*x
            #print '%1.3f ns'%(fit[0]*1e9/360.)
            #clf()
            #plot(x[ndrop:(nch-ndrop)],onphase_r[ndrop:(nch-ndrop)],'.')
            #plot(x,f,'r')
            #grid(T)
            #ylim([-180,180])
            #ylabel('Phase [deg]');xlabel('Freq [GHz]')
            #raw_input()
            onphase_r=onphase_r-f

          rmsa[ant1,ant2]=std(onamp_r[ndrop:(nch-ndrop)])   # rms of amplitude across spectra
          rmsa[ant2,ant1]=rmsa[ant1,ant2]
          delaya[ant1,ant2]=fit[0]*1e12/360.  # delay in ps
          delaya[ant2,ant1]=delaya[ant1,ant2]
          ampa[ant1,ant2]=mean(onamp_r[ndrop:(nch-ndrop)])   # mean amplitude across spectra
          ampa[ant2,ant1]=ampa[ant1,ant2]
        
      # finished - remove the autocorrelations for now:
      ampa0=ampa
      for i in range(nants):
        ampa0[i,i]=0.0
      snra=ampa0/rmsa

      # store in pickle
      f1 = open('%s_scan%d_amp.pickle'%(self.invis.strip('.ms'),scan),'w')
      pickle.dump(ampa,f1)
      f2 = open('%s_scan%d_rms.pickle'%(self.invis.strip('.ms'),scan),'w')
      pickle.dump(rmsa,f2)
      f3 = open('%s_scan%d_snr.pickle'%(self.invis.strip('.ms'),scan),'w')
      pickle.dump(snra,f3)
      f4 = open('%s_scan%d_delay.pickle'%(self.invis.strip('.ms'),scan),'w')
      pickle.dump(delaya,f4)
    
      self.ampa = ampa
      self.delaya = delaya
      self.speca = speca
      self.rmsa = rmsa
      
      fig=figure(1)   # open the matrix window
      clf()
      xticks(arange(nants),self.ants[0:nants],horizontalalignment='left',size='small',rotation=-45)
      yticks(arange(nants),self.ants[0:nants],verticalalignment='bottom',size='small')
      cid=fig.canvas.mpl_connect('button_press_event',self.onclick)
      if self.outp=='rms':
        if vrange=='full':
          vmax=rmsa.max()
          vmin=rmsa.min()
        else:
          vmax=100.0
          vmin=0.0
        fig.add_subplot(fig=pcolor(rmsa,cmap='gray',vmax=vmax,vmin=vmin,picker=True))
        title('Baselines '+self.outp+' '+self.invis+'   Click to show spectrum')
      elif self.outp=='snr':
        print 'displaying snr'
        if vrange=='full':
          vmax=snra.max()   # this is to avoid amplitude of autocorrelations
          vmin=snra.min()
        else:
          vmax=10.0
          vmin=0.0
        fig.add_subplot(fig=pcolor(snra,cmap='hot',vmax=vmax,vmin=vmin,picker=True))
        title('Baselines '+self.outp+' '+self.invis+'   Click to show spectrum')
      elif self.outp=='amp':
        if vrange=='full':
          vmax=ampa0.max()
          vmin=ampa0.min()
        else:
          vmax=180.0
          vmin=-180.
        fig.add_subplot(fig=pcolor(ampa0,cmap='gray',vmax=vmax,vmin=vmin,picker=True))
        title('Baselines '+self.outp+' '+self.invis+'   Click to show spectrum')
        print "click Matrix to display phase (lower half), amp (upper half), or autocorrelation (diagonal)"
      colorbar()
      savefig('%s_matrix_scan%d.png'%(self.invis.strip('.ms'),scan))
#


