#!/usr/bin/env python
#
# codex.py   correlator data explorer   v1.5 (for casapy 4.0)
# To run v1.5:
# extract the ms, start casapy, and input start parameters:
# invis = 'uid....'     name of ms  - required
# execfile('codex.py')
#
# Displays the amplitude SNR (average value of all spw, polns) for each baseline as a colour matrix/grid.  
# clicking on a baseline point on this grid shows the baseline phase or ampl. spectrum in a separate window.
# blue=pol X, red=pol Y
# the current cursor position in the matrix (ie baseline) is also shown in the terminal.
# Clicking on central spine where ant1=ant2 plots the autocorrelation amplitude instead of cross-correlation.
# hit the key 'a' or 'p' to change to amplitude or phase in the spectrum plot.
# hit a number (0, 1, 2 or 3) to plot a different spectral window in the spw list.
# hit z to zoom in amplitude plot by factors of 2, b to zoom back.
# hit 'q' to quit
#
# the following are optional before the execfile:
# spwa=[1,5,9,11]   an array of spectral windows (from 1 up to 4) to display. defaults to [1,3,5,7]
# scan = 1    scan number. NB make sure that scan number corresponds to correct spw list (using listobs in casapy)
# nant = 0    number of antennas. defaults to 0 to use all antennas
# spwd = 1    shows spw 1 in spectral display. spectral display can also be changed in realtime


import sys
import os
import optparse
from optparse import OptionParser

import numpy
from numpy import *
import pylab
from pylab import *
import matplotlib
#import analysisUtils as aU
#import taskinit
#from taskinit import *
#import casac

# ---------------------------------------------------------------
#
outp='snr'    # currently only works for snr.
# spwd=1     # spectral window for display (actually data description ID, which is assumed to be spectral window)
scan=2     # default for ms from gonogo script
# nant=0    # default is 0. if <3, it will take all antennas.
spwa=[17,19,21,23]    # default for ms from gonogo script
ndrop=20      # channels to drop for rms calc
mplot='p'     # default to start with phase plot
#fitslope='y'  # (not currenntly used)fit a delay slope to get the rms
# invis='uid___A002_X59fcb9_Xe.ms'  # input file
# ---------------------------------------------------------------

# check that some parameters are set
try:
  nant
except NameError:
  nant=0
  print 'reading all antennas.  set nant=xx to display a limited number of antennas'

try:
   invis
except NameError:
   print "need to set a valid ms name using  invis='uid....'"

try:
   scan
except NameError:
   scan=1
   print "defaulting to scan number 1.   set scan=xx to display a different scan. eg for gonogo scan=2"
if scan=='':
   scan=1
   print "defaulting to scan number 1. set scan=xx before starting to display a different scan"

# spwa is an array of spw's
# spwi is a counter for spwa
# spwd is the array index for the currently displayed spw (so current spw is spwa[spwd])
try:
  spwa
except NameError:
  spwa=[17,19,21,23]
  print " using default spw list of 17,19,21,23. To just display selected spws, set spwa=[1,5,11,19] (for example) before starting codex (up to 4 spws). make sure this agrees with the scan number for those spws"


bln=''
xant=yant=-1
spwd=0
yzoom=0

pylab.clf()

print 'input file:',invis,'  num./antennas:',nant, ' scan:',scan,' spectral windows:',spwa

tb.open(invis+'/ANTENNA')
ans=tb.getcol('NAME')
tb.close()
tb.open(invis)

def phase(num):
  return atan2(num.imag,num.real)
def ampli(num):
  return sqrt(num.imag**2+num.real**2)

def onclick(event):
  global spwd,snra,vmax,vmin,fig,mplot,yzoom
#  print 'button=%d, xdata=%f, ydata=%f'%(event.button,event.xdata,event.ydata)
  if event.xdata != None and event.ydata != None :
    xant=int(event.xdata)
    yant=int(event.ydata)
    print 'Baseline:', ans[xant],'*',ans[yant],'  spw:',spwa[spwd]
    print 'X amplitude=',ampa[xant,yant,0,spwd],' rms=',rmsa[xant,yant,0,spwd],' snr=',snra[xant,yant,0,spwd]
    print 'Y amplitude=',ampa[xant,yant,1,spwd],' rms=',rmsa[xant,yant,1,spwd],' snr=',snra[xant,yant,1,spwd]
    sr=pylab.figure(num=2)
    pylab.clf()
    if xant==yant:
      pylab.plot(x,spec_a[xant,yant,0,spwd,:],'.',figure=sr,color='blue')
      pylab.plot(x,spec_a[xant,yant,1,spwd,:],'.',figure=sr,color='red')
      pylab.title(invis+'   Autocorr: '+ans[xant]+' spw '+str(spwa[spwd]))
    else:
      if mplot=='p':
        pylab.plot(x,spec_p[xant,yant,0,spwd,:]*57.29,'.',figure=sr,color='blue')
        pylab.plot(x,spec_p[xant,yant,1,spwd,:]*57.29,'.',figure=sr,color='red')
        pylab.title(invis+'  Phase: '+ans[xant]+'*'+ans[yant]+' spw '+str(spwa[spwd]))
      elif mplot=='a':
        pylab.plot(x,spec_a[xant,yant,0,spwd,:],'.',figure=sr,color='blue')
        pylab.plot(x,spec_a[xant,yant,1,spwd,:],'.',figure=sr,color='red')
        if yzoom > 0:
          ymax_data=max(spec_a[xant,yant,0,spwd,:])
          pylab.ylim([-ymax_data/yzoom,ymax_data/yzoom])
        pylab.title(invis+'  Ampl.: '+ans[xant]+'*'+ans[yant]+' spw '+str(spwa[spwd]))
    return xant,yant

def onmove(event):
  if event.xdata != None and event.ydata != None :
  	xant=int(event.xdata)
  	yant=int(event.ydata)
        if yant < nant and xant < nant:
          bln='cursor '+ans[xant]+' '+ans[yant]   # for line text
        else:
          bln= ''
  else:
	xant=yant=-1
	bln=''
  sys.stdout.flush()
#  print 'baselines', bln
  sys.stdout.write(bln+'\r')
  sys.stdout.flush()
  return xant,yant


def onkey(event):
   global spwd,snra,vmax,vmin,pylab,fig,mplot,yzoom
#   print "event key pressed ",event
   if event.key == 'q':
      print ' ended '
      fig.canvas.mpl_disconnect(cid1)
      fig.canvas.mpl_disconnect(cid2)
      fig.canvas.mpl_disconnect(cid3)
      pylab.close(2)
      pylab.close(1)
   elif event.key == '0' and spwd != 0:
      spwd=0
      print 'Change spectral plot to spw', spwa[spwd]
   elif event.key == '1' and spwd != 1:
      spwd=1
      print 'Change spectral plot to spw', spwa[spwd]
   elif event.key == '2' and spwd != 2:
      if len(spwa)>2:
        spwd=2
        print 'Change spectral plot to spw', spwa[spwd]
      else:
        print 'not enough spws in list'
   elif event.key == '3' and spwd != 3:
      if len(spwa)>3:
        spwd=3
        print 'Change spectral plot to spw', spwa[spwd]
      else:
        print 'not enough spws in list'
   elif event.key == 'p' and mplot != 'p':
      mplot='p'
      print 'Change spectral plot to phase'
   elif event.key == 'a' and mplot != 'a':
      mplot='a'
      print 'Change spectral plot to amplitude'
   elif event.key == 'z':
     if yzoom > 0:
       yzoom=yzoom*2
     else:
       yzoom=2
     print 'zoom ampl plot by ',yzoom
   elif event.key == 'b':
     yzoom=0
     print 'zoom ampl plot back to full'
   elif event.key == 'h':
      print 'plot options: h(help), a(amp), p(phase), 0 -',(len(spwa)-1),'(spw index in',spwa,'), z(zoom*2), b(zoom back), q(quit)'
   return


# ------------------------------------------------------------------
# start of main script

sr=pylab.figure()    # open the spectrum plot window
if nant < 3:
  nant=ans.size   # all antennas
spw1=spwa[0]
tb2=tb.query("SCAN_NUMBER==%d && ANTENNA1==%d &&ANTENNA2==%d && DATA_DESC_ID==%d"%(scan,0,0,spw1))
on=tb2.getcol('DATA')
print 'shape ',on.shape
if on.shape[0] < 1:
  print 'no data read. check spw list and scan number.'
  pylab.close(2)
  pylab.close(1)
  sys.exit()

npol=shape(on)[0]
nch=on.shape[1]
print nch, 'channels'

nsp=len(spwa)   # number of spectral windows
rmsa=numpy.zeros((nant,nant,2,nsp))
delaya=numpy.zeros((nant,nant,2,nsp))
ampa=numpy.zeros((nant,nant,2,nsp))
spec_p=numpy.zeros((nant,nant,2,nsp,nch))   # phase
spec_a=numpy.zeros((nant,nant,2,nsp,nch))   # ampl
vrange='full'   # full range for plots
ona=numpy.zeros((2,nch))   # time averaged temporary
onphase=numpy.zeros((2,nch))   #initialise temporary output arrays
onamp=numpy.zeros((2,nch))
onamp_diff=numpy.zeros((2,nch))

print 'reading ms data'
for spwi in range(0,nsp):
  spw=spwa[spwi]
  print " spw",spw
  for ant1 in range(0,nant):
    print 'antenna1 ',ant1
    for ant2 in range(ant1,nant):
#     print ant1,ant2
      tb2=tb.query("SCAN_NUMBER==%d && ANTENNA1==%d &&ANTENNA2==%d && DATA_DESC_ID==%d"%(scan,ant1,ant2,spw))
      on=tb2.getcol('DATA')
#      print on.shape[0]
      ona=numpy.average(on,axis=2)   # all subscans
#      print ona.shape
      x=arange(nch)
      for i in range(0,nch):
        onphase[0,i]=phase(ona[0,i])
        onamp[0,i]=ampli(ona[0,i])
        if npol==2:
         onphase[0,i]=phase(ona[0,i])
         onamp[0,i]=ampli(ona[0,i])
        elif npol==4:
         onphase[1,i]=phase(ona[3,i])
         onamp[1,i]=ampli(ona[3,i])
        elif npol==1:
         onphase[1,i]=phase(ona[0,i])
         onamp[1,i]=ampli(ona[0,i])


#     print 'done', onphase.shape

      spec_p[ant1,ant2,:,spwi,:]=onphase    # (last 3 indices are pol, spw, channel) used, as normally the spectrum displayed is phase
      spec_p[ant2,ant1,:,spwi,:]=onphase
      spec_a[ant1,ant2,:,spwi,:]=onamp
      spec_a[ant2,ant1,:,spwi,:]=onamp
#     print spec_p.shape
      if ant1==ant2:
        spec_p[ant1,ant2,:,spwi,:]=onamp    # save the autocorrelation amplitude if ants are the same

#     print 'ok'
#     if fitslope=='y':
#      fit=numpy.polyfit(x[ndrop:(nch-ndrop)],onphase_r[ndrop:(nch-ndrop)],1)
#      f=zeros(nch)
#      for i in range(0,nch):
#        f[i]=fit[1]+fit[0]*i
#      onphase_r=onphase_r-f
#    print numpy.std(onamp_r[ndrop:(nch-ndrop)])

    
# drop channels from edges
# do these for each polarisation independently:
      for k in range(0,2):
# this gets the differential between channels for the rms calc
        for ichani in range(ndrop,nch-ndrop):
          onamp_diff[k,ichani]=onamp[k,ichani]-onamp[k,(ichani-1)]
	rmsa[ant1,ant2,k,spwi]=numpy.std(onamp_diff[k,ndrop:(nch-ndrop)])   # rms of differential amplitude
    	rmsa[ant2,ant1,k,spwi]=rmsa[ant1,ant2,k,spwi]
#    delaya[ant1,ant2]=fit[0]*360./nch*0.5   # delay in ns
#    delaya[ant2,ant1]=delaya[ant1,ant2]
    	ampa[ant1,ant2,k,spwi]=numpy.mean(onamp[k,ndrop:(nch-ndrop)])   # amplitude
    	ampa[ant2,ant1,k,spwi]=ampa[ant1,ant2,k,spwi]

snra=ampa/rmsa

#print ampa[0,1,:,0],rmsa[0,1,:,0],snra[0,1,:,0]

#  finished - remove the autocorrelations for calculating range in values:
snra=ampa/rmsa

snra_d=snra
for i in range(0,nant):
  snra_d[i,i,:,:]=0.0
snra_d_all=numpy.zeros((nant,nant))

for ant1 in range(0,nant):
   #print 'antenna1 ',ant1
   for ant2 in range(ant1,nant):
     snra_d_all[ant1,ant2]=snra_d[ant1,ant2,0,:].min()  # this is the min snr for all spw in pol 0
     snra_d_all[ant2,ant1]=snra_d[ant1,ant2,1,:].min()  # this is the min snr for all spw in pol 1


print 'matrix size:',snra_d_all.shape

#
# 

fig=plt.figure(num=1)   # open the matrix window
pylab.xticks(arange(nant),ans[0:nant],horizontalalignment='left',size='small',rotation=-45)
pylab.yticks(arange(nant),ans[0:nant],verticalalignment='bottom',size='small')
cid1=fig.canvas.mpl_connect('button_press_event',onclick)
cid2=fig.canvas.mpl_connect('motion_notify_event',onmove)
cid3=fig.canvas.mpl_connect('key_press_event',onkey)

if outp=='snr':
   print 'displaying min snr of amplitude'
   if vrange=='full':
     vmax=snra_d_all.max()   # this is to avoid including amplitude of autocorrelations
     vmin=snra_d_all.min()
   else:
     vmax=100.0
     vmin=0.0
   fig.add_subplot(fig=pcolor(snra_d_all[:,:],cmap='gist_heat',vmax=vmax,vmin=vmin,picker=True))
   print outp+'   '+invis+' average all spw and poln'
   pylab.title(invis+':  Min.ampl.SNR in all spws. pol0 & pol1')
   print 'plotting spw'

#   fig.add_subplot(fig=pcolor(snra[:,:,0,3],cmap='hot',vmax=vmax,vmin=vmin,picker=True))
#   pylab.text(0.1,0.1,bln)
#elif outp=='amp':
#   if vrange=='full':
#     vmax=ampa0.max()
#     vmin=ampa0.min()
#   else:
#     vmax=180.0
#     vmin=-180.
#   fig.add_subplot(fig=pcolor(ampa,cmap='gray',vmax=vmax,vmin=vmin,picker=True))
#   pylab.title('Baselines '+outp+' '+invis+'   Click to show spectrum')
print "click on a baseline to display phase plot, or autocorrelation along the axis"
pylab.colorbar()
#
#


