# compare focus results of two asdms
from asdm import *
import pylab
import math

axes = ['X','Y','Z']
class FocusResult(ASDM):

    def getFocusResult(self, calDataSet):
        
        po = ASDMParseOptions()
        # The calibration result is an ASDM without ExecBlock table. It must
        # be defined "asALMA" to be readable by the method setFromFile().
        po.asALMA()
        po.loadTablesOnDemand(True)
        self.setFromFile(calDataSet, po)
        self.focusOffset = {}
        self.focusOffsetError = {}
        self.dataSetName = calDataSet

        for row in self.calFocusTable().get():
            antenna = row.antennaName()
            if antenna not in self.focusOffset.keys():
                self.focusOffset[antenna] = {}
                self.focusOffsetError[antenna] = {}
            for axis in range(3):
                if (not row.wereFixed()[axis]):
                    self.focusOffset[antenna][axis] = row.offset()[0][axis].get()
                    self.focusOffsetError[antenna][axis] = row.offsetError()[0][axis].get()

    def getPointingResult(self, calDataSet):
        
        po = ASDMParseOptions()
        # The calibration result is an ASDM without ExecBlock table. It must
        # be defined "asALMA" to be readable by the method setFromFile().
        po.asALMA()
        po.loadTablesOnDemand(True)
        self.setFromFile(calDataSet, po)
        self.pointingOffset = {}
        self.pointingOffsetError = {}
        self.dataSetName = calDataSet

        for row in self.calPointingTable().get():
            antenna = row.antennaName()
            if antenna not in self.pointingOffset.keys():
                self.pointingOffset[antenna] = {}
                self.pointingOffsetError[antenna] = {}
            for axis in range(2):
                self.pointingOffset[antenna][axis] = row.collOffsetRelative()[0][axis].get()
                self.pointingOffsetError[antenna][axis] = row.collError()[0][axis].get()

    def getFocusOffsetData(self, refData):
        self.focusOffset = {}
        
        self.focusOffsetError = {}
        f = open(refData)
        for l in f.readlines():
            #print 'l: ',l
            if len(l) > 0:
                w = l.split()
                #print 'w: ', w
                ant = w[0]
                if len(w)>1:
                    try:
                        self.focusOffset[ant]
                    except:
                        self.focusOffset[ant] = {}
                        self.focusOffsetError[ant] = {}
                    axes = ['XR', 'YR','ZR']
                    if w[1] in axes:
                        axis = axes.index(w[1])
                        self.focusOffset[ant][axis]=-float(w[2])
                        self.focusOffsetError[ant][axis]=0.0
        f.close()

    def getPointingOffsetData(self, refData):
        self.pointingOffset = {}
        
        self.pointingOffsetError = {}
        f = open(refData)
        for l in f.readlines():
            #print 'l: ',l
            if len(l) > 0:
                w = l.split()
                #print 'w: ', w
                ant = w[0]
                if len(w)>1:
                    # print w
                    if w[1] not in ['XR', 'YR','ZR']:
                        if ant not in self.pointingOffset.keys():
                            self.pointingOffset[ant] = [-math.radians(float(w[1])/3600.),
                                                        math.radians(float(w[2])/3600.)]
                            self.pointingOffsetError[ant] = [0,0]
        f.close()
                    


def plotAsdmResult(asdmList, refAsdm=None, refPointingAsdm=None, refFocusAsdm=None,
                   refData = None, refFocusData = None, refPointingData = None,
                   focus=True, pointing=True, pointingFocus=False,
                   refData2 = None, refFocusData2 = None, refPointingData2 = None,
                   flimit=2.5, plimit=20.):
    pylab.clf()
    asdm0 = FocusResult()
    ny = 0
    if focus: ny+=1
    if pointing: ny += 1
    if pointingFocus: ny += 1

    
    if refAsdm != None:
        if focus:
            asdm0.getFocusResult(refAsdm)
        if pointing:
            asdm0.getPointingResult(refAsdm)
            
    if refPointingAsdm != None:
        asdm0.getPointingResult(refPointingAsdm)
    #
    if refFocusData is None:
        refFocusData = refData
    if refPointingData is None:
        refPointingData = refData
    if refFocusData2 is None:
        refFocusData2 = refData2
    if refPointingData2 is None:
        refPointingData2 = refData2

    if refData2 is not None:
        asdm2 =  FocusResult()
        
    if refFocusData != None:
        asdm0.getFocusOffsetData(refFocusData)
        if refFocusData2 != None:
            asdm2.getFocusOffsetData(refFocusData2)

    if refPointingData != None:
        asdm0.getPointingOffsetData(refPointingData)
        if refPointingData2 != None:
            asdm2.getPointingOffsetData(refFocusData2)
            
    for asdm1 in asdmList:
        print asdm1
        asdm = FocusResult()
        asdm.getFocusResult(asdm1)
        asdm.getPointingResult(asdm1)
        # print asdm.focusOffset

        ip = 1
        if focus:
            for axis in range(3):

                pylab.subplot(ny, 3 ,axis+ip)

                for ant in asdm.focusOffset.keys():
                    if ant in asdm0.focusOffset.keys():
                        if ant[0:2]=='CM':
                            c='b'
                        elif ant[0:2]=='PM':
                            c='c'
                        elif ant[0:2]=='DA':
                            c='g'
                        else:
                            c='r'
                        xx = asdm0.focusOffset[ant][axis]*1000.
                        ex = asdm0.focusOffsetError[ant][axis]*1000.
                        if refData2 != None:
                            xx += asdm2.focusOffset[ant][axis]*1000.
                            ex += asdm2.focusOffsetError[ant][axis]*1000.
                        pylab.plot(xx,
                                asdm.focusOffset[ant][axis]*1000.,
                                c+'o')
                        pylab.errorbar(xx,
                                       asdm.focusOffset[ant][axis]*1000.,
                                       yerr=asdm.focusOffsetError[ant][axis]*1000.,
                                       xerr=ex,
                                       ecolor=c)
                        # print asdm0.focusOffset[ant][axis]*1000., asdm.focusOffset[ant][axis]*1000.
                        #pl.text(asdm0.focusOffset[ant][axis]*1000.,
                        #        asdm.focusOffset[ant][axis]*1000.,
                        #        ant,
                        #        color=c)


                pylab.plot([-flimit,flimit],[-flimit,flimit],'-.')
                pylab.xlim(-flimit,flimit)
                pylab.ylim(-flimit,flimit)
                if asdm1 == asdmList[-1]:
                    pylab.grid()
                    pylab.title(axes[axis]+' axis')
                    pylab.xlabel('Reference Focus')
                    # pylab.ylabel('Holographic Focus')
            ip += 3
                
        if pointing:
            for i in range(2):
                pylab.subplot(ny,3, ip+i)
                
                for ant in asdm.pointingOffset.keys():
                    if ant in asdm0.pointingOffset.keys():
                        if ant[0:2]=='CM':
                            c='b'
                        elif ant[0:2]=='PM':
                            c='c'
                        elif ant[0:2]=='DA':
                            c='g'
                        else:
                            c='r'
                        xx = 3600.*math.degrees(asdm0.pointingOffset[ant][i])
                        ex = 3600.*math.degrees(asdm0.pointingOffsetError[ant][i])
                        if refData2 != None:
                            xx += 3600.*math.degrees(asdm2.pointingOffset[ant][i])
                            ex += 3600.*math.degrees(asdm2.pointingOffsetError[ant][i])
                                
                        pylab.plot(xx,
                                3600.*math.degrees(asdm.pointingOffset[ant][i]),
                                c+'o')
                        pylab.errorbar(xx,
                                       3600.*math.degrees(asdm.pointingOffset[ant][i]),
                                       yerr=3600.*math.degrees(asdm.pointingOffsetError[ant][i]),
                                       xerr=ex,
                                       ecolor=c)
 

                pylab.plot([-plimit,plimit],[-plimit,plimit],'-.')
                pylab.xlim(-plimit,plimit)
                pylab.ylim(-plimit,plimit)
                paxes=['Az','El']
                if asdm1 == asdmList[-1]:
                    pylab.grid()
                    pylab.title(paxes[i]+' axis')
                    pylab.xlabel('Reference Pointing')
            ip += 3

        if pointingFocus:
            for i in range(2):
                # additional 
                pylab.subplot(ny,3,i+ip)
                for ant in asdm.pointingOffset.keys():
                    if ant in asdm0.focusOffset.keys():
                        if ant[0:2]=='CM':
                            c='b'
                        elif ant[0:2]=='PM':
                            c='c'
                        elif ant[0:2]=='DA':
                            c='g'
                        else:
                            c='r'
                        xx = 1000.*asdm.focusOffset[ant][i]
                        ex = 1000.*asdm.focusOffsetError[ant][i]
                        if refData2 != None:
                            xx += 1000.*asdm2.focusOffset[ant][i]
                            ex += 1000.*asdm2.focusOffsetError[ant][i]
                        # error in pointing result:
                        yy = 3600.*math.degrees(asdm.pointingOffset[ant][i]-asdm0.pointingOffset[ant][i])
                        ey = 3600.*math.degrees(asdm.pointingOffsetError[ant][i])
                        pylab.plot(xx, yy, c+'o')
                        pylab.errorbar(xx, yy, yerr=ey, xerr=ex, ecolor=c)


                    pylab.plot([-flimit,flimit],[-plimit,plimit],'-.g')
                    pylab.plot([-flimit,flimit],[plimit,-plimit],'-.g')
                    pylab.xlim(-flimit,flimit)
                    pylab.ylim(-plimit,plimit)
                    pylab.grid()
                    #paxes=['Az','El']
                    #if asdm1 == asdmList[-1]:
                    #    pylab.grid()
                    #    pylab.title(paxes[i]+' axis')
                    #    pylab.xlabel('Reference Pointing')

            ip += 3


    pl.savefig('%s-holoFocus.png'%asdmList[0])

#plotAsdmResult(['uid___A002_X783f4a_X3b4'],
#                     refAsdm = 'uid___A002_X783f4a_X3dc')
#plotAsdmResult(['uid___A002_X783f4a_X23e'], refData = '0001randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X266'], refData = '0001randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X29e'], refData = '0129randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X2c6'], refData = '0129randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X2ee'], refData = '0129randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X326'], refData = '0219randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X34e'], refData = '0219randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X388'], refData = '0219randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X3b4'], refData = '0219randomOffsets.txt')
#plotAsdmResult(['uid___A002_X783f4a_X3dc'], refData = '0324randomOffsets.txt')


#plotAsdmResult(['uid___A002_X783f4a_X34e'], refAsdm = 'uid___A002_X783f4a_X326')
#
# plotAsdmResult(['uid___A002_X785196_X148_focus_result'], refData = '0016-randomOffsets.txt')


plotAsdmResult(['uid___A002_X785196_X111_focus_result'],
               refFocusData = '0016-randomOffsets.txt',
               refPointingAsdm='uid___A002_X785196_X17c')

plotAsdmResult(['uid___A002_X785196_X148_focus_result'],
               refFocusData = '0016-randomOffsets.txt',
               refPointingAsdm='uid___A002_X785196_X17c')

plotAsdmResult(['uid___A002_X785196_X321_focus_result'],
               refFocusData = 'randomOffsets_2.txt',
               refPointingAsdm='uid___A002_X785196_X355')

plotAsdmResult(['uid___A002_X785196_X39a_focus_result'],
               refData = 'randomOffsets_2.txt',
               refData2 = 'randomOffsets_3.txt',
               flimit=7., plimit=25.)
