#!/usr/bin/env python

from Acspy.Clients.SimpleClient import PySimpleClient;
import datetime;
from optparse import OptionParser;
import os

cl=PySimpleClient();
tmcdb=cl.getComponent('TMCDBAccess');
DVantennas=["DV0"+str(i) for i in range(1,10)]+["DV"+str(i) for i in range(10,26)];
DAantennas=["DA"+str(i) for i in range(41,66)];
Allantennas=DVantennas+DAantennas;

def dprint(value):
    print str(value);


def ff(value):
    if not value=="None":
        fs=str(round(value,2));
    else:
        fs="None";
    while len(fs)<10:
        fs=" "+fs;
    return fs;

# =====================================================
# Print basic info
# =====================================================
dprint("List of antennas:");
dprint(DAantennas);
dprint(DVantennas);


# =====================================================
# Handle command line options
# =====================================================
parser=OptionParser();
parser.add_option("-a","--antennas",dest="antennas",help="Antennas for which to compare the tilt parameters. Ex.: DV01 or DV01,DV02,DA41 or all.",default="all");

(options,args)=parser.parse_args();

if options.antennas=="all":
    antennas_to_process=Allantennas;
else:
    antennas_to_process=[ant.strip(" ") for ant in options.antennas.split(",")];

dprint("Antennas to process:");
dprint(antennas_to_process);

# =====================================================
# Read tilt values into an array
# =====================================================
param_values=dict();
lines=[];
for ant in antennas_to_process:
    dprint("Processing antenna: "+ant);
    # READ TMCDB
    try:
        an_tmcdb=float(tmcdb.getMetrologyCoefficients(ant)[0]);
        aw_tmcdb=float(tmcdb.getMetrologyCoefficients(ant)[1]);
        could_access_tmcdb=True;
    except:
        an_tmcdb="None";
        aw_tmcdb="None";
        could_access_tmcdb=False;

    #READ ACU
    try:
        mount=cl.getComponent("CONTROL/{0}/Mount".format(ant));
        if ant in DVantennas:
            an_acu=float(mount.GET_METR_COEFF_0()[0]);
            aw_acu=float(mount.GET_METR_COEFF_1()[0]);
        if ant in DAantennas:
            an_acu=float(mount.GET_METR_COEFF_1()[0]);
            aw_acu=float(mount.GET_METR_COEFF_2()[0]);            
        could_access_acu=True;
    except:
        an_acu="None";
        aw_acu="None";
        could_access_acu=False;

    # PRODUCE LINE FOR OUTPUT FILE
    time=datetime.datetime.now().isoformat();
    line="Tilts {0}, {1}, {2}, {3}, {4}, {5}".format(time,ant,ff(an_tmcdb),ff(aw_tmcdb),ff(an_acu),ff(aw_acu));
    dprint(line);
    lines.append(line);

    # PRINT WARNINGS
    if not could_access_tmcdb:
        print "Warning: Could not access TMCDB for "+ant;
    if not could_access_acu:
        print "Warning: Could not access ACU for "+ant;

    if could_access_tmcdb and could_access_acu:
        parameters_are_same=(abs(an_tmcdb-an_acu)<0.01 and abs(aw_tmcdb-aw_acu)<0.01);
        if not parameters_are_same:
            print "Parameters do not coincide for "+ant;

    # DETROY MOUNT
    try:
        del(mount);
    except:
        print "mount not created";

# =====================================================
# Write results to file
# =====================================================
date=time.split("T")[0];
filename="/tmp/tilt_parameters%s.txt"%(date);
with open(filename,"w") as f:
    for line in lines:
        f.write(line+"\n");


os.system('chmod ugo+rwx -R %s' %  filename)

try:
    del(tmcdb);
except:
    print "tmcdb not created";

try:
    del(cl);
except:
    print "cl not created";


