#runs in casapy:
#  first type:     execfile('fastswitch_analysis.py') 
#  and then, e.g.: fastreduce(position.dat',deltatime=1.5,tolerance=3,tmax=1600)
#  it prints mean and rms on screen
#
#Author: Antonio Hales, ahales@alma.cl
#July 2009, Pere added printing of the maximum value, and minor corrections

import math
import os
from array import array
#from path import path
import time

# HERE WE ARE ANALYSING BOTH DIRECTIONS (+ AND -) AND CLUMPING THE RESULTS TOGETHER
# Depending of what we want to analyse (1 dir or both) we may have to increase the
# sleep time between switches in the observing script

def fastreduce (rawfile='psc_position.dat',deltatime=1.8,tolerance=3,flag=1000000, nbins=100,tmin=0, tmax=90000, switchnumber=2):
   """

   Script for analysing Fast Switching Data. Takes input Position Stream Client Data, finds when 1.5deg offset command
   was issued and computes the difference between commanded and actual position (absolute Az and El error) at the given 
   elapsed time (deltatime, e.g after 1.8 seconds for ACA, after 1.5 seconds for Main Array antennas). The Absolute Error
   at the elapsed time must not exceed 'tolerance' (in arcseconds) given in the Antenna Specifications 
 
   e.g for Main Array Antenna:
   execfile("../analysis.py") 
   fastreduce('position_fast_7_pa0.dat',deltatime=1.5,tolerance=3)
   fastreduce('position_fast_7_pa0.dat',deltatime=2.0,tolerance=0.6)
   fastreduce('position_0_30_90_lewis.dat',deltatime=1.5,tolerance=3,tmax=350,switchnumber=2)

   deltatime -- time at which settling error is analysed 
                (1.8s ACA/1.5MA and 2.3 ACA/2,0 MA at 3" 
		and 0.6" respectively)

   tolerance -- value for settling error  to be out of spec 
		(warning message printed)

   flag -- to flag extremely bad data that will affect plots and statistics

   nbins -- number of bins in histogram plot 

   tmax  -- Only data until tmax  (in seconds) will be used for Total offset Average calculation

   switchnumber -- Whether to analyse the first positional switch (i.e switchnumber=1) or the second
		   (i.e switchnumber=2). Bearing in mind the Fast Switching scripts starts at position A,
                   then moves to position B (first switch) and then moves back to position A (second switch).
		   So Far only works for 2nd switch ! Faulty data (false jump) will cause errors. Check PM03 data.

   Author: A. Hales, ahales@alma.cl

   """
   parsefile=rawfile
   mytb = tbtool.create()
   outfile = parsefile + '.table'
   mytb.fromascii(outfile,parsefile,autoheader=true,firstline=3,sep=' ')
   
   mytb.close()
   mytb.open(outfile,nomodify=false)
   mytb.renamecol('Column1', 'Time')
   mytb.renamecol('Column2', 'Comm Az')
   mytb.renamecol('Column3', 'Comm El')

   mytb.renamecol('Column6', 'on-TE Az')
   mytb.renamecol('Column7', 'on-TE El')
   mytb.renamecol('Column10', 'Pos Flag')
   
   tb.open(outfile,nomodify=false)
   timestring=tb.getcol('Time')
   commAz = tb.getcol('Comm Az')
   commEl = tb.getcol('Comm El')

   onAz = tb.getcol('on-TE Az')
   onEl = tb.getcol('on-TE El')
   
   deltaAz=array('d')
   deltaEl=array('d')
   deltaR=array('d')
   deltaSettling=array('d')
   timeseconds = array('d')
   posflag=tb.getcol('Pos Flag')
   
   spectra = tb.nrows()

   pl.close(1)
   pl.close(2)
   pl.ion()
   pl.title('Fast Switching Results '+rawfile,fontsize=14)
   pl.subplot(211)      
   pl.xlabel('Time since start [seconds]')
   pl.ylabel('Total Offset [arcseconds]')

   time0=timestring[1]
   a0=time0.split(":")
   s0=float(a0[0])*3600+float(a0[1])*60+float(a0[2])
   tjump=s0
   tjump=100000000000
   jumpnumber=0
   
   for i in range(0,spectra):
    timehours=timestring[i]
    a=timehours.split(":")
    s=float(a[0])*3600+float(a[1])*60+float(a[2])
    
    deltinhaAz=(onAz[i]*cos(onEl[i]*math.pi/180.0)-commAz[i]*cos(onEl[i]*math.pi/180.0))
    deltaAz.append(deltinhaAz)
    deltinhaEl=(onEl[i]-commEl[i])
    deltaEl.append(deltinhaEl)
    deltinhaR=3600*sqrt(deltinhaAz**2+deltinhaEl**2)
    #deltinhaR=3600*sqrt( (  (onAz[i]-commAz[i])*cos(onEl[i]*math.pi/180.0))**2 + (onEl[i]-commEl[i])**2)
    if (s-s0)>=tmin:
      deltaR.append(deltinhaR)
      timeseconds.append(s-s0)
      #print timehours, onAz[i], commAz[i],onEl[i], commEl[i], s,deltinhaAz,deltinhaEl,deltinhaR
   
      #if (deltinhaR>5300 and s<tjump):
      if (deltinhaR>5300 and s<tjump):
         tjump=s
         jumpnumber=jumpnumber+1
         print "I found one jump", i,jumpnumber,timehours, s, tjump, posflag[i]
         
         
      if((s-tjump>deltatime) and (s-tjump)<(deltatime+0.045) and (s-tjump)>0) :# and (jumpnumber>=2):
         print " settling error after ",(s-tjump)," seconds is %.2f\"" % deltinhaR
       
       #time.sleep(2) 
       #print timehours, (s,deltinhaAz,deltinhaEl,deltinhaR)
         if (jumpnumber==switchnumber): 
      	     #matplotlib.
             pl.axvline(x=s-s0)
      	     pl.axvline(x=tjump-s0,linestyle='--')
       # append values, but flags large errors ! do we want this ? will hide terribly  bad data ! 

         if deltinhaR>tolerance:
      	    print "Broke tolerance", timehours, s-s0, deltinhaR
       
         if deltinhaR<flag and (s-s0)<tmax and (jumpnumber==switchnumber): 
      	    deltaSettling.append(deltinhaR)
      	    jumpnumber=0
         tjump=100000000000
         #

   # Plot data      
   linetype = 'b-'
   pl.semilogy(timeseconds,deltaR, linetype)
   stats=meanrms(deltaSettling)
   mean=stats[0]
   std=stats[1]
   
   print "Mean dev for " + parsefile + " at ",deltatime, "seconds is %.2f\" +/- %.2f " % (mean, std)
 
   # Plot histogram of Total Offset Error
   pl.subplot(212)
   pl.hist(deltaSettling,nbins)
   pl.ylabel('Counts')
   deltatime= "%.1f" % deltatime
   labelx='Total Offset after ' + deltatime+'s' + ' [arcseconds]'
   pl.xlabel(labelx)

   # If wanted to plot result values 
   maxx=max(deltaSettling)
   minx=min(deltaSettling)
   mean="Mean is = %.3f" % mean
   std="RMS is =  %.3f" % std
   pl.text((maxx-(maxx-minx)/4), 3.6 , mean , fontsize=12)
   pl.text((maxx-(maxx-minx)/4), 3.2 , std , fontsize=12)

   tb.close()


def meanrms (myarray):

    array=myarray
    counter=0
    m1=myarray[0]
    s1=0
    mk=0
    sk=0
    totalf2775=array
    print array
    massimo=0

    for i in range(0,len(array)):
	counter=counter+1
	mk=m1+((array[i]-m1)/counter)
	sk=s1+((array[i]-m1)*(array[i]-mk))
	m1=mk
	s1=sk
	if abs(array[i]) > massimo:
		massimo = abs(array[i])

    sk=sqrt(sk/(counter-1))
    
    print "Maximum value =",massimo
    print "Mean over", counter," scans =", mk 
    print "Sigma over", counter,", scans =", sk 
    print "Mean and Std deviation over", counter," scans  = ", mk , sk 
    return (mk,sk)
    
