#!/usr/bin/env python

# handover_check.py
# to run, start up casapy, then:  execfile('handover_check.py')
# in the following, parameters marked     #  MAX ALLOWED VALUE    ***** are the ones which trigger a fault flag, in the normal test mode

import os
import sys
from optparse import OptionParser
import numpy
from numpy import *
import analysisUtils as aU
import shutil
import cmath

print '\n Prototype basic array checkout from a handover SB. To run:'
print '    check_data(uid://A002/X..../..)'
print '  This exports from the archive, and checks pointing, Tsys, Trx, signal, delays and spectral baselines for platforming'
print '  Summary of correlator quadrant and antenna status is given at the end'
print '  options: verb=F (no details); exist=T (use existing ms in current directory); high=T (high-quality (stringent) checkout)' 


# =======================================
def check_data(uid='',verb=T,exist=F,high=F):
# these options are for verbose output, the ms has not been created already, and not high accuracy checkout
# vis='uid___A002_X7a37f9_X7b.ms'
# verb=True

# switches for checks:
 dowvr=True
 
 asdm=string.replace(uid,'://','___')
 asdm=string.replace(asdm,'/','_')
 vis=asdm+'.ms'

 if exist != True:
   if os.path.isdir(asdm) == True:   # still exists, delete it and re-import
     shutil.rmtree(asdm,ignore_errors=True)
   if verb: print ' exporting asdm..'
   os.system("asdmExport %s > temp.mstest"%(uid))
   if os.path.isdir(asdm) == False:
     print 'failed to export asdm',asdm
     sys.exit()
   else:
     if verb: print asdm,' successfully exported'

   if verb: print ' importing ms..'
   if os.path.isdir(vis) == True:  # ms still exists, delete it and re-import
     shutil.rmtree(vis,ignore_errors=True)
   importasdm(asdm=asdm,asis='CalDelay CalPointing CalAtmosphere SysCal CalWVR',overwrite=T)
 #  print 'ms=',vis

   os.remove('temp.mstest')
   shutil.rmtree(asdm)

 if os.path.isdir(vis) == False:
   print 'ms does not exist'
   sys.exit()

 ants=aU.getAntennaNames(vis)
 nant=len(ants)
# gets number of CM antennas
 nCMant=0
 for iant in range(0,nant):
     if ants[iant][0:2]=='CM': nCMant=nCMant+1
# print nant,nCMant
 
 if verb and high == True:
    print ' starting stringent analysis of',vis
 elif verb:
    print ' starting analysis of',vis

 tb.open(vis)
 tim=tb.getcol('TIME')[0]
 dat=aU.mjdSecondsToMJDandUT(tim)[1]

# ------------------------------------------
# if dotsys:    # do Tsys and Trx check
 if verb: print '\n checking Tsys & Trx..'
# following limits are for band 3:
 tsysmax=100.0    #  MAX ALLOWED VALUE    *****
 trxmax=60.0    #  MAX ALLOWED VALUE    *****
 if high: trxmax=48.0
#
 tb.open(vis+'/SYSCAL')
#
 antennas = tb.getcol('ANTENNA_ID')
 spws = tb.getcol('SPECTRAL_WINDOW_ID')
 times = tb.getcol('TIME')
 intervals = tb.getcol('INTERVAL')
 nATMcals=len(set(times))    # number of ATMcals in this ms (should be 1)
 nspws=len(set(spws))   # number of spws
 nedge=8
 tsysa=zeros((2,nant,nspws))
 tsysa_flag=zeros((2,nant,nspws))
 trxa=zeros((2,nant,nspws))
 trxa_flag=zeros((2,nant,nspws))

 for iant in range(0,nant):
	for ispw in range(0,nspws):
		ic=iant*nspws+ispw
#		print ic
 # extract Tsys:
		tsysav0=mean(tb.getcell('TSYS_SPECTRUM',ic)[0][nedge:(128-nedge)])
                tsysav1=mean(tb.getcell('TSYS_SPECTRUM',ic)[1][nedge:(128-nedge)])
#		print tsysav0,tsysav1
		tsysa[0,iant,ispw]=tsysav0
                tsysa[1,iant,ispw]=tsysav1
# 
# extract Trx:
                trxav0=mean(tb.getcell('TRX_SPECTRUM',ic)[0][nedge:(128-nedge)])
                trxav1=mean(tb.getcell('TRX_SPECTRUM',ic)[1][nedge:(128-nedge)])
                trxa[0,iant,ispw]=trxav0
                trxa[1,iant,ispw]=trxav1

 # checks Tsys within right range
		if tsysav0 > tsysmax or tsysav0 < 0.0 :
			tsysa_flag[0,iant,ispw]=1
			if verb: print ants[iant],'P0 spw',spws[ic],' Tsys',"%.1f"%tsysav0,' (Trx',"%.1f"%trxav0,')'
                if tsysav1 > tsysmax or tsysav1 < 0.0 :
			tsysa_flag[1,iant,ispw]=1
                        if verb: print ants[iant],'P1 spw',spws[ic],' Tsys',"%.1f"%tsysav1,' (Trx',"%.1f"%trxav1,')'
# 
# check Trx within right range:
                if trxav0 > trxmax or trxav0 < 0.0 :
                        trxa_flag[0,iant,ispw]=1
                        if verb: print ants[iant],'P0 spw',spws[ic],' Trx',"%.1f"%trxav0,' (Tsys',"%.1f"%tsysav0,')'
                if trxav1 > trxmax or trxav1 < 0.0 :
			trxa_flag[1,iant,ispw]=1
                        if verb: print ants[iant],'P1 spw',spws[ic],' Trx',"%.1f"%trxav1,' (Tsys',"%.1f"%tsysav1,')'

 if verb and sum(tsysa_flag) == 0: print ' all Tsys OK'
 if verb and sum(trxa_flag) == 0: print ' all Trx OK'
 
# ------------------------------------------
 if nant == nCMant: dowvr=False
 if dowvr:   # check that wvr data from antennas is consistent
  if verb: print '\n checking wvr results..'
  wvr_factor=1.1      #  MAX ALLOWED VALUE    *****
  if high: wvr_factor=1.01
  wvr_flag=zeros(nant)

  tb.open(vis+'/ASDM_CALWVR')
  wvr_data=tb.getcol('water')*1000.0
  wvr_mean=mean(wvr_data)
  for iant in range(0,nant):
    if ants[iant][0:2] != 'CM':
	if wvr_data[iant] > wvr_factor*wvr_mean:
	   wvr_flag[iant]=1
	   if verb: print ants[iant],' pwv',"%.2f"%wvr_data[iant],'. Mean of all antennas=',"%.2f"%wvr_mean
  if verb and sum(wvr_flag) == 0: print ' all OK'

# ------------------------------------------
# if dopsignal   # using pointing data, check for signal using peakIntensity
 if verb: print '\n checking peak pointing signal..'
 fraction=0.5  #  MAX ALLOWED VALUE    *****  allowed peak signal can be as low as 0.5 of mean
 if high: fraction=0.99  # for high quality test
 ppeak_a=zeros(nant)
 ppeak_flag=zeros(nant)

 tb.open(vis+'/ASDM_CALPOINTING')
 ppeak=tb.getcol('peakIntensity')[0]   # average of 2 polarisations
# following gets the mean signal for the 12m antennas only:
 psum=0.0
 n12m=0
 if nCMant < nant:
   for iant in range(0,nant):
     if ants[0:1] != 'CM':
       psum=psum+ppeak[iant]
       n12m=n12m+1
   pmean=psum/(n12m*1.0)
 else:  # otherwise all antennas are CMs:
   for iant in range(0,nant):
     psum=psum+ppeak[iant]
   pmean=psum/(nCMant)
   
 for iant in range(0,nant):
 #       print ants[iant][0:2]
	if ants[iant][0:2] != 'CM':  # for 12m antennas
	   if ppeak[iant] < pmean*fraction:   # flags if not close to mean 12m signal
              ppeak_flag[iant]=1
              pfrac=ppeak[iant]/pmean
              if verb: print ants[iant],'fraction of mean pointing signal',"%.4f"%pfrac
	elif nCMant != nant:  # for case where there are some antennas
           if ppeak[iant] < (pmean/3.0)*fraction:   # flags if not close to 7m_signal/3 (ratio of areas of 12m & CM ants)
              ppeak_flag[iant]=1
              pfrac=ppeak[iant]/(pmean/3.0)
              if verb: print ants[iant],'fraction of mean pointing 12m signal/3.0 (for 7m antennas)',"%.4f"%pfrac
        else:  # for case with all CM antennas
           if ppeak[iant] < pmean*fraction:   # flags if not close to mean signal
              ppeak_flag[iant]=1
              pfrac=ppeak[iant]/pmean
              if verb: print ants[iant],'fraction of mean pointing signal',"%.4f"%pfrac
  
 if verb and sum(ppeak_flag) == 0: print ' all OK'

# -------------------------------------
# if dosbg:   # do SBG check
 if verb: print '\n checking sideband gain..'
 sbg_max=0.90
 if high: sbg_max=0.975
 rejection=-10.0*log(1.0-sbg_max)
 tb.open(vis+'/ASDM_CALATMOSPHERE')
#
 sbs = tb.getcol('sbGain')
 bbs=tb.getcol('basebandName')
 sbg=zeros((2,nant,nspws))
 sbg_flag=zeros((2,nant,nspws))

 for ipol in range(0,2):
   for iant in range(0,nant):
       for ispw in range(0,nspws):
          ibb=ispw+iant*4    # assume 4 spws
 # go through sbs array
          sidebandGain=sbs[ipol][ibb]
          sbg[ipol,iant,ispw]=sidebandGain
          if sidebandGain < sbg_max:
             if verb:
                print ants[iant],bbs[ibb],'P',ipol,'  sidebandGain:',sidebandGain
                sbg_flag[ipol,iant,ispw]=1

 if verb and sum(sbg_flag) == 0: print ' all OK'

# -----------------------------------------
# if dopoint:    # do pointing checkout
 if verb: print '\n checking pointing..'
# following limit is for initial pointing:
 point1_offmax=15.0      #  MAX ALLOWED VALUE    *****
 if high: point1_offmax=2.0

# array = poff, flag array=poff_flag

 tb.open(vis+'/ASDM_CALPOINTING')
 startValidTime = tb.getcol('startValidTime')
 npointings=len(set(startValidTime))   # number of pointings in this ms (should be 1 or 2)
 coll_offset = 206264.8*tb.getcol('collOffsetRelative')[0]
 antennaNames = tb.getcol('antennaName')
 coll_error = 206264.8*tb.getcol('collError')[0]
#
 poff_flag=zeros(nant)
 poff=sqrt(coll_offset[0]**2+coll_offset[1]**2)   # pointing offset vector length
 perr=sqrt(coll_error[0]**2+coll_error[1]**2)   # pointing offset error vector length
#
 for iant in range(0,nant):
     if ants[iant][0:2] != 'CM':
	if poff[iant] > point1_offmax or perr[iant] > (2.0*mean(perr)) :
		poff_flag[iant]=1
		if ppeak_flag[iant]==1:
		   if verb: print ants[iant],'initial pointing offset=',"%.2f"%poff[iant],'+/-',"%.2f"%perr[iant],' arcsec  ?LOW POINTING SIGNAL?'
		else:
                   if verb: print ants[iant],'initial pointing offset=',"%.2f"%poff[iant],'+/-',"%.2f"%perr[iant],' arcsec'
     else:  # CM antennas have lower pointing requirements
	if poff[iant] > point1_offmax*2.0 or perr[iant] > (5.0*mean(perr)) :
                poff_flag[iant]=1
                if ppeak_flag[iant]==1:
                   if verb: print ants[iant],'initial pointing offset=',"%.2f"%poff[iant],'+/-',"%.2f"%perr[iant],' arcsec  ?LOW POINTING SIGNAL?'
                else:
                   if verb: print ants[iant],'initial pointing offset=',"%.2f"%poff[iant],'+/-',"%.2f"%perr[iant],' arcsec'

 if verb and sum(poff_flag) == 0: print ' all OK'


# ------------------------------------------
# if dodelay:    # do delay check
 if verb: print '\n checking delays..'
# following is limit on delay and delay error
 if high != True:
   maxdelay=0.1      # nsec. MAX ALLOWED VALUE    *****
   maxdelayerr=0.1   # nsec. the delay will ONLY be flagged if the measured error is less than this. assumes no signal if this not achieved
 else:
   maxdelay=0.05
   maxdelayerr=0.05 # NB: the delay is done in FDM with chan_av=2, so the normal error is a bit higher than TDM

 delay_flag=zeros((2,nant,4))
 delay_a=zeros((2,nant,4))
 delayerr_a=zeros((2,nant,4))

 tb.open(vis+'/ASDM_CALDELAY')

 delays=1.0e9*tb.getcol('delayOffset')
 delayerr=1.0e9*tb.getcol('delayError')
 basebands=tb.getcol('basebandName')
 rxband=tb.getcol('receiverBand')[0]


 for iant in range(0,nant):
	for ispw in range(0,4):
		delay_a[0,iant,ispw]=delays[0][(iant+nant*ispw)]
                delay_a[1,iant,ispw]=delays[1][(iant+nant*ispw)]
                delayerr_a[0,iant,ispw]=delayerr[0][(iant+nant*ispw)]
                delayerr_a[1,iant,ispw]=delayerr[1][(iant+nant*ispw)]

# now go through and check for delay not large, or delay error not large. flag if _either_ is true.

 for iant in range(0,nant):
   mean_delay_ant=mean(delay_a[:,iant,:])      # these are the mean delay & delayerr for all spws & pols for each antenna
   mean_delayerr_ant=mean(delayerr_a[:,iant,:])
   for ispw in range(0,4):
	for ipol in range(0,2):
	  if abs(delay_a[ipol,iant,ispw]) > maxdelay and delayerr_a[ipol,iant,ispw] < maxdelayerr:  # only flags delays if the noise on the delay is OK
	    delay_flag[ipol,iant,ispw]=1    # 1 is minor delay error
            ibb=iant+nant*ispw
	    if abs(delay_a[ipol,iant,ispw]) > maxdelay*2.0: # if the delay is >2x the max allowed delay, this is flagged as a major delay error
	      delay_flag[ipol,iant,ispw]=2   # 2 is major delay error
	    if int(round(abs(delay_a[ipol,iant,ispw]))) == 16:  # if the delay is ~16ns, this may identify a Drx/tx reset problem
	      delay_flag[ipol,iant,ispw]=16   # 16 is possible DTX/DRX error
	    if verb: print ants[iant],'pol',ipol,basebands[ibb],'delay=',"%.4f"%delay_a[ipol,iant,ispw],'+/-',"%.4f"%delayerr_a[ipol,iant,ispw],'ns'
   if mean_delay_ant > maxdelay and mean_delayerr_ant < maxdelayerr:
	delay_flag[0,iant,0]=4   # 4 is delay error on all spws/pols, which means a cable/pad delay error
	if verb: print ants[iant], 'mean cable delay=',"%.4f"%mean_delay_ant,'+/-',"%.4f"%mean_delayerr_ant,'ns   **'

 if verb and sum(delay_flag) > (nant*8)/4: print '?check reference antenna?'   # possibly a delay error on the reference antenna
 if verb and sum(delay_flag) == 0: print ' all OK'

 tb.close()
# ---------------------------
# if dobandpass:   # use the bandpass solution to do checks for platforming, signal level, maybe other stuff
 if verb: print '\n check for platforming..'

 # finds a good reference antenna (without flags)
 refantenna=-1
 for iant in range(0,nant):
   allflag=sum(delay_flag[:,iant,:])+ppeak_flag[iant]+poff_flag[iant]+sum(tsysa_flag[:,iant,:])+sum(trxa_flag[:,iant,:])
   if ants[iant][0:2] != 'CM' and allflag == 0:  # this is a good antenna, no flags and 12m
	refantenna=iant
 if refantenna==-1: refantenna=0  # uses first antenna, if there are none without flags, or if only CM antennas used
 if exist != True:   # ms does not exist
 #  shutil.rmtree(vis+'.bandp',ignore_errors=True)
   if os.path.isdir(vis+'.bandp') != True:   # bandp does not exist so needs to be created
     if verb: print 'creating bandpass solutions with ref antenna',ants[refantenna]
     bandpass(vis,field='0',spw='17,19,21,23',scan='4',caltable=vis+'.bandp',refant=ants[refantenna],minsnr=1.5)
   else:
     if verb: print 'bandpass solution already exists - using it'
 else:
   if os.path.isdir(vis+'.bandp') == False:   # should exist but does not - creates one
     print 'bandp does not exist - creating with ref antenna',ants[refantenna]
   #  rmtables(vis+'.bandp')
     bandpass(vis,field='0',spw='17,19,21,23',scan='4',caltable=vis+'.bandp',refant=ants[refantenna],minsnr=1.5)
   else:
     if verb: print 'using existing bandpass solutions'  # already exists - use it
 
 nch=960  # this should not be hard-wired in a general case, but it is tailored to the latest SB which is FDM with channel average=4, 960channels
 bp0=numpy.zeros((3840,4,nant))  # nb: use 3840 here to avoid problem with earlier tests
 bp1=numpy.zeros((3840,4,nant))
 bp0_d=numpy.zeros((3840,4,nant))   # these are differential bandpasses
 bp1_d=numpy.zeros((3840,4,nant))
 bp0_d_std=numpy.zeros((4,nant))   # std of differential
 bp1_d_std=numpy.zeros((4,nant))
 bp0_d_flag=numpy.zeros((4,nant))  # flag of std
 bp1_d_flag=numpy.zeros((4,nant))
 scan=4
 tb.open(vis+'.bandp')
 ispwc=0
 if high:
   bp_sigma=1.01
 else:
   bp_sigma=2.0    #  MAX ALLOWED VALUE    *****
 if nCMant < nant:
     spw_array=[17,19,21,23]
 else:
     spw_array=[16,18,20,22]  #different spw array numbers for only CM antennas
 for ispw in spw_array:
#   print ispw
   for iant in range(0,nant):
     tb2=tb.query("SCAN_NUMBER==%d && ANTENNA1==%d && SPECTRAL_WINDOW_ID==%d "%(scan,iant,ispw))
     bp=tb2.getcol('CPARAM')[:,:,0]
     for ichan in range(0,nch):
	bp0[ichan,ispwc,iant]=abs(bp[:,:][0][ichan])  # try with absolute amplitude for now
	bp1[ichan,ispwc,iant]=abs(bp[:,:][1][ichan])
     for ichan in range(0,nch-1):   # finds differential amplitude solutions
	bp0_d[ichan,ispwc,iant]=bp0[(ichan+1),ispwc,iant]-bp0[ichan,ispwc,iant]
        bp1_d[ichan,ispwc,iant]=bp1[(ichan+1),ispwc,iant]-bp1[ichan,ispwc,iant]
     bp0_d_std[ispwc,iant]=std(bp0_d[:,ispwc,iant])
     bp1_d_std[ispwc,iant]=std(bp1_d[:,ispwc,iant])
   ispwc=ispwc+1
 bp0_allmean=mean(bp0)  # this is the average power 
 bp1_allmean=mean(bp1)
 bp0_std_mean=mean(bp0_d_std)
 bp1_std_mean=mean(bp1_d_std)
 bp_std_mean=mean([bp0_std_mean,bp1_std_mean])
 for ispw in range(0,4):
    for iant in range(0,nant):
      ibb=iant+nant*ispw
      if bp0_d_std[ispw,iant] > bp_sigma*bp_std_mean:
	bp0_d_flag[ispw,iant]=1
	if verb: print ants[iant],basebands[ibb],'P0',"%.4f"%(bp0_d_std[ispw,iant]/bp_std_mean),'deviation in bandpass flatness. possible platforming problem'
      if bp1_d_std[ispw,iant] > bp_sigma*bp_std_mean:
        bp1_d_flag[ispw,iant]=1
        if verb: print ants[iant],basebands[ibb],'P1',"%.4f"%(bp1_d_std[ispw,iant]/bp_std_mean),'deviation in bandpass flatness. possible platforming problem'
#    print basebands[ibb]
 if verb and (sum(bp0_d_flag)+sum(bp1_d_flag)) == 0: print ' all OK'

# ------------------------------------------
# if dobandpass   # using bandpass data, check for signal level using mean values of each bandpass solution. treats each poln/spw separately
 if verb: print '\n checking peak signal from bandpass..'

 fraction=0.5      #  MAX ALLOWED VALUE    *****  allowed peak signal can be as low as 0.5 of mean
 if high: fraction=0.99  # for high quality test
 peak_a=zeros(nant)
 peak_flag=zeros(nant)    # resets pointing flag array back to zero (so that summary report uses this rather than pointing peak)

 bp0_power=numpy.zeros((4,nant))
 bp1_power=numpy.zeros((4,nant))
 bp0_power_flag=numpy.zeros((4,nant))
 bp1_power_flag=numpy.zeros((4,nant))
 for ispw in range(0,4):
    for iant in range(0,nant):
      bp0_power[ispw,iant]=mean(bp0[:,ispw,iant])
      bp1_power[ispw,iant]=mean(bp1[:,ispw,iant])

# for iant in range(0,nant):
#   print ants[iant],bp0_power[:,iant]
#   print bp1_power[:,iant]

# following gets the mean power of all spws & both polns for the 12m antennas only:
 psum=0.0
 n12m=0
 for iant in range(0,nant):
   if ants[0:1] != 'CM':
      psum=psum+mean([mean(bp0_power[:,iant]),mean(bp1_power[:,iant]) ])
      n12m=n12m+1
 pmean=psum/(n12m*1.0)  # average power from bandpass for all 12m antennas
 if verb: print 'mean power 12m',pmean

 for iant in range(0,nant):
   if ants[iant][0:2] != 'CM':  # for 12m antennas
      for ispw in range(0,4):
 #       print ants[iant],ispw,'P0',bp0_power[ispw,iant],pmean*fraction
        if bp0_power[ispw,iant] < pmean*fraction or bp0_power[ispw,iant] > pmean/fraction:   # flags if not close to mean 12m signal
	   peak_flag[iant]=1
           pfrac=bp0_power[ispw,iant]/pmean
           if verb: print ants[iant],'P0 spw:',ispw,'fraction of mean signal',"%.4f"%pfrac
        if bp1_power[ispw,iant] < pmean*fraction or bp1_power[ispw,iant] > pmean/fraction:   # flags if not close to mean 12m signal
           peak_flag[iant]=1
           pfrac=bp1_power[ispw,iant]/pmean
           if verb: print ants[iant],'P1 spw:',ispw,'fraction of mean signal',"%.4f"%pfrac
   else:  # for CM antennas
      for ispw in range(0,4):
 #      print ants[iant][0:2]
        if bp0_power[ispw,iant] < pmean/3.0*fraction or bp0_power[ispw,iant] > pmean/3.0/fraction:   # flags if not close to mean 12m signal/3 (for 7m antennas)
           peak_flag[iant]=1
           pfrac=bp0_power[ispw,iant]/pmean
           if verb: print ants[iant],'P0 spw:',ispw,'fraction of mean 12m signal/3 (for CM antennas)',"%.4f"%pfrac
        if bp1_power[ispw,iant] < pmean/3.0*fraction or bp1_power[ispw,iant] > pmean/3.0/fraction:   # flags if not close to mean 12m signal/3 (for 7m antennas)
           peak_flag[iant]=1
           pfrac=bp1_power[ispw,iant]/pmean
           if verb: print ants[iant],'P1 spw:',ispw,'fraction of mean 12m signal/3 (for CM antennas)',"%.4f"%pfrac


 if verb and sum(peak_flag) == 0: print ' all OK'

# ----------------------------------------------
# if dophase   # use bandpass to check for anomalous phase problems across bandpass
# do checks on bandpass phase solution here, eg spikes, phase steps, or anomalous slopes... 
#  TO ADD/checkout

# ----------------------------------------------

 tb.close()

 print '='*27,' Array status summary ','='*27
 print dat,'  ',uid,' ',nant,'antennas (',nCMant,'CMs )   pwv=',"%.2f"%wvr_mean,'mm'
 if high: print ' ** from stringent test **'

 max_bbs=0.75   #  MAX ALLOWED VALUE    *****  if the fraction of antennas with problems in this bb is greater than this, possible correlator quadrant problem
# now checks by correlator quadrant, to see if most of the bandpass solutions on that quadrant have platforming errors: 
 for ispw in range(0,4):
   if sum(bp0_d_flag[ispw,:]) > (max_bbs*nant) or sum(bp1_d_flag[ispw,:]) > (max_bbs*nant):
      print '  Correlator Quadrant',ispw,'(',basebands[ispw*nant],')', ' possible problem'
   else:
      print '  Correlator Quadrant',ispw,' OK'

 print ''
# ---------------------------

# now go through all flags, and list problems with each antenna:

 aflag=zeros(nant)
 ngood=0
 for iant in range(0,nant):
	aflag[iant]=sum(delay_flag[:,iant,:])+peak_flag[iant]+poff_flag[iant]+sum(tsysa_flag[:,iant,:])+sum(trxa_flag[:,iant,:])
	if aflag[iant]==0:
	  note='OK'
	  ngood=ngood+1
	else:
	  note='  Problem:'

	  if poff_flag[iant] > 0:
	     note=note+' pointing error.'

	  if peak_flag[iant] > 0:
	     note=note+' low signal strength.'

	  if sum(delay_flag[:,iant,:]) > 0 :
	     meta_err=0
	     large_err=0
	     cable_delay=0
	     for ipol in range(0,2):
		for ispw in range(0,4):
		   if delay_flag[ipol,iant,ispw]==16: meta_err=1
		   if delay_flag[ipol,iant,ispw]==2: large_err=1
		   if delay_flag[ipol,iant,ispw]==4: cable_delay=1
	     if meta_err==1: note=note+' metaframe delay errors.'
	     if large_err==1: note=note+' large delay errors.'
	     if cable_delay==1: note=note+' cable/pad delay error?'
	     if meta_err != 1 and large_err !=1 and cable_delay !=1: note=note+' minor delay errors.'

	  if sum(trxa_flag[:,iant,:]) > 0:
	     note=note+' Trx errors.'

          if sum(sbg_flag[:,iant,:]) > 0 and peak_flag[iant] ==0:
             note=note+' sidebandGain errors.'

	  if sum(tsysa_flag[:,iant,:]) > sum(tsysa_flag[:,iant,:]) and peak_flag[iant] == 0:
	     note=note+' Tsys errors.'

	  if wvr_flag[iant] >0:
	     note=note+' WVR errors.'

          if sum(bp0_d_flag[:,iant])+sum(bp1_d_flag[:,iant]) > 0 and peak_flag[iant] == 0 and sum(tsysa_flag[:,iant,:]) == 0  :
	     note=note+' Platforming errors.'   # only if there is also a signal, and tsys is ok

	print ' ',ants[iant],note
 if sum(delay_flag) > (nant*8)/4: print 'Check delays in reference antenna',ants[0]   # possibly a delay error on the (first) reference antenna
 if high:
   print '='*75,'\n',ngood,'/',nant,' antennas pass stringent tests in band',rxband
 else:
   print '='*75,'\n',ngood,'/',nant,' antennas are fully working in band',rxband
 if 1.0*ngood/nant < 0.75 and high != True:
	print 'this is not good'

# -----------------------------

