#!/usr/bin/env python
"""
Show azimuth and/or elevation of astronomical object vs time.

Usage (see help for details):
  elplot.py [-d 2011-01-19] [-x LST] [-y AZEL] [-s '3c273,1924-292,Saturn']
    [-S '5:35/-5:27/Ori,17:46/-29.0/GC'] [-Q]

2013-09-24 tsawada: added '-l' option
2012-12-13 tsawada: added the capability to specify the names of "-S" sources
                    and made some cosmetic changes
2012-09-05 tsawada: removed dummy raw_input at the end of the script
2012-04-09 tsawada: added pluto, ceres, pallas, juno, and vesta data
2012-02-27 tsawada: added CLT and CLST support
2011-10-06 tsawada: added '-Q' option
2011-01-25 tsawada: disabled masking EL<0 part of AZ plot because
                    it does not work well with the currently-installed-in-STE
                    version of matplotlib
2011-01-19 tsawada: added '-S' option
2010-12-31 tsawada: minor cosmetic changes
2010-12-30 tsawada: added '-y' option
2010-??-?? tsawada

$Id: lst.py,v 1.1 2013/11/16 21:06:27 wdent Exp $
"""
from Tkinter import *
import datetime
import time
import pylab as pl
import sourceCatalog as sC
from optparse import OptionParser

root = Tk() 
time1 = '' 
clock = Label(root, font=('Courier', 50, 'bold'), bg='green') 
clock.pack(fill=BOTH, expand=4) 

### define constants ###

# longitude (E) of the sites converted to radians
sitelon = pl.radians(-67.457)

### define methods ###

def convert_colonsep_to_float(value):
    """
    Convert a colon-delimited string (like dd:mm:ss.s) into a float number.
    """
    value = value.strip()
    if value[0] == '-':
        sign = -1.
        value = value[1:]
    else:
        sign = 1.

    values = [float(item) for item in value.split(':')]
    result = 0.
    for (i, num) in enumerate(values):
        result += num * 60.**(-i)

    return sign*result


def normalize_angle(value):
    """
    Normalize angle (radians) into 0 - 2pi.
    """
    return value % (2.*pl.pi)


def convert_date_to_t2000(thedate):
    """
    Return time passed since 2000-01-01 12:00 UT (in unit of 36525 days).
    """
    return ((thedate-datetime.date(2000, 1, 1)).days-0.5)/36525.


def get_utc_to_lst_offset(thedate, sitelon):
    """
    Return the offset between UTC and LST for give date and site.
    """
    return sitelon + pl.radians(15.* (
        6.646065556 + 2400.051262/36525.*( \
        (thedate-datetime.date(1900, 1, 1)).days+0.5)))

################################
# MAIN
################################


thedate = datetime.date.today()
print thedate
lstoff= get_utc_to_lst_offset(thedate, sitelon)

lstoff=(lstoff/2./3.1415926535*24.)%24
print 'starting LST clock'

def tick(): 
	global time1 
# get the current local time from the PC 
	time2 = time.strftime('%H:%M:%S')
        time2_fl=convert_colonsep_to_float(time2)
        time2_lst=(time2_fl+lstoff)%24

# if time string has changed, update it 
	if time2 != time1: 
		time1 = time2 
#                print time2_lst
                time2_lst_txt=datetime.timedelta(0,0,0,0,0,time2_lst)
#                print time2_lst_txt
                time_txt='LST: '+str(time2_lst_txt)[0:8]+' '
		clock.config(text=time_txt)
# calls itself every 200 milliseconds 
# to update the time display as needed 
# could use >200 ms, but display gets jerky 
	clock.after(200, tick)
	
tick()
root.mainloop()
