# Script used by the NAASC for testing of the quality of continuum subtraction
# in pipeline imaging (C4R1 onward).

import glob
import os
import shutil
import numpy as np
import analysisUtils as au
from taskinit import *
from imstat_cli import imstat_cli as imstat

commonPath  = '/lustre/naasc/sciops/comm/'
textfile = 'makeMom0FC.txt'  # If you do not run all of them, then please change the name away from makeMom0FC.txt (or rename your previous .txt)
projects = ['reharris/pipeline/root/2013.1.00428.S_2016_05_25T21_16_10.828/SOUS_uid___A001_X147_X181/GOUS_uid___A001_X147_X182/MOUS_uid___A001_X147_X183/working',
            'reharris/pipeline/root/2013.1.00105.S_2016_05_20T13_57_39.639/SOUS_uid___A001_X12e_X23d/GOUS_uid___A001_X12e_X23e/MOUS_uid___A001_X12e_X23f/working',
            'reharris/pipeline/root/2013.1.00428.S_2016_05_25T21_16_10.828/SOUS_uid___A001_X147_X181/GOUS_uid___A001_X147_X182/MOUS_uid___A001_X147_X183/working',
            'reharris/pipeline/root/2013.1.00470.S_2016_05_25T21_20_30.260/SOUS_uid___A001_X12a_X3e/GOUS_uid___A001_X12a_X3f/MOUS_uid___A001_X12a_X40/working',
            'reharris/pipeline/root/2013.1.00576.S_2016_05_25T21_34_55.470/SOUS_uid___A001_X121_X522/GOUS_uid___A001_X121_X523/MOUS_uid___A001_X121_X524/working',
            'reharris/pipeline/root/2013.1.00658.S_2016_05_23T15_47_18.371/SOUS_uid___A001_X13c_X36/GOUS_uid___A001_X13c_X37/MOUS_uid___A001_X13c_X38/working',
            'reharris/pipeline/root/2013.1.00814.S_2016_05_23T15_47_31.558/SOUS_uid___A001_X13e_X78/GOUS_uid___A001_X13e_X79/MOUS_uid___A001_X13e_X7a/working',
            'amcnicho/pipeline/root/2013.1.01061.S_2016_05_20T14_07_53.181/SOUS_uid___A001_X13a_X13/GOUS_uid___A001_X13a_X14/MOUS_uid___A001_X13a_X15/working',
            'reharris/pipeline/root/2013.1.00426.S_2016_05_23T15_49_51.201/SOUS_uid___A001_X13c_X4e/GOUS_uid___A001_X13c_X4f/MOUS_uid___A001_X13c_X50/working',
            'amcnicho/pipeline/root/2013.1.00234.S_2016_05_20T14_07_49.793/SOUS_uid___A001_X121_X1bd/GOUS_uid___A001_X121_X1be/MOUS_uid___A001_X121_X1bf/working',
            'amcnicho/pipeline/root/2013.1.01391.S_2016_05_20T14_07_51.497/SOUS_uid___A001_X147_X27a/GOUS_uid___A001_X147_X27b/MOUS_uid___A001_X147_X27e/working',
            'amcnicho/pipeline/root/2013.1.00786.S_2016_05_20T14_07_37.854/SOUS_uid___A001_X13e_X177/GOUS_uid___A001_X13e_X178/MOUS_uid___A001_X13e_X179/working',
            'amcnicho/pipeline/root/2013.1.00722.S_2016_05_20T14_07_51.633/SOUS_uid___A001_X145_X134/GOUS_uid___A001_X145_X135/MOUS_uid___A001_X145_X136/working',
            ]

def makeMom0FC(cube, channelSelection=''):
    mom0FC = cube+'.manualmom0FC'
    if os.path.exists(mom0FC):
        shutil.rmtree(mom0FC)
    print "Running immoments('%s', moments=0, chans='%s', outfile='%s')" % (cube,channelSelection,mom0FC)
    immoments(cube, moments=0, chans=channelSelection, outfile=mom0FC)
    return mom0FC

def readChannelSelectionFromFC(findcontdat):
    f = open(findcontdat,'r')
    lines = f.readlines()
    selection = lines[0].split()[0]
    f.close()
    return selection

def findCorrespondingFindcontdat(cleanCube):
    findcontStage = au.extractPipelineStage(glob.glob('%s/*cont.I.findcont.residual'%os.path.dirname(cleanCube))[0])
    dirtyCube = cleanCube.replace('cube.I.iter1.image','cont.I.findcont.residual')
    dirtyCube = au.replacePipelineStage(dirtyCube, findcontStage)
    return (dirtyCube + '_findContinuum.dat')

txt = open(textfile, 'w')
txt.write("Statistics shown are imstat 'sigma' and ('medabsdevmed'/0.6745).\n")
for project in projects:
    txt.write('\n%s\n' % project)
    txt.flush()
    cleanCubes = sorted(glob.glob(commonPath+project+'/*cube*iter1.image'))
    for cleanCube in cleanCubes:
        findcontdat = findCorrespondingFindcontdat(cleanCube)
        dirtyCube = findcontdat.replace('_findContinuum.dat','')
        selection = readChannelSelectionFromFC(findcontdat)
        if (cleanCube != ''):
            # compare number of channels in cubes
            ndirty = au.numberOfChannelsInCube(dirtyCube, faster=True)
            nclean = au.numberOfChannelsInCube(cleanCube, faster=True)
            if (ndirty != nclean):
                nchanToRemove = (ndirty-nclean)/2
                selection = au.shiftChannelSelection(selection, nchanToRemove)
                print "Shifted selection by %d channels: %s" % (nchanToRemove, selection)
        mom0FC = makeMom0FC(cleanCube, selection)
        os.system('chgrp naascssg ' + mom0FC)
        os.system('chmod g+w ' + mom0FC)
        print "Created ", os.path.basename(mom0FC)
        png = au.imviewField(mom0FC)
        os.system('chgrp naascssg ' + png)
        os.system('chmod g+w ' + png)
        print "Created plot ", png
        if True:
            pbflat = au.replacePipelineStage(cleanCube, -2).replace('.cube.I.iter1.image', '.mfs.I.iter1.pb')
        else:
            pb = cleanCube.replace('.image','.pb')
            print "Running flattenMask"
            pbflat = au.flattenMask(pb)
        print "Running imstat('%s', mask='\"%s\">0.5')" % (mom0FC, pbflat)
        stats = imstat(mom0FC, mask='"%s">0.5'%(pbflat))
        if (len(stats['sigma']) < 1):
            txt.write('%s noValue noValue\n' % (os.path.basename(mom0FC)))
        else:
            txt.write('%s %f %f\n' % (os.path.basename(mom0FC), stats['sigma'], stats['medabsdevmed']/.6745))
        txt.flush()
txt.close()
os.system('chgrp naascssg ' + textfile)
os.system('chmod g+w ' + textfile)
print "Created ", textfile
