#! /usr/bin/env python

# who       when      what
# --------  --------  ----------------------------------------------
# nsaez    2010-02-05 created
# tsawada  2010-06-08 modified for R7.1 (ASDM export to $HOME instead of /tmp;
#                     change TelCal.ArchivedAsdm to ArchivedAsdm)
# dbarkats 2011-11-02 rewrote entirely to be working only on reduction machines
#                     and too allow exprt of a batch file
# dbarkats 2012-01-12 made modifications to the version rewritten by Ignacio to
#                     make sure it remained compatible.
# rkneissl 2012-06-14 included exporting of CalSurvey data
# itoledo  2012-12-28 to colorize by field casaplotms is dropped in favor of
#                     a casa script using plotms directly -- FAIL
# itoledo  2012-12-29 added listobs output
# itoledo  2013-01-23 added cycle1 project list and option
# dbarkats 2012-05-18 moved the pnames to a separate function


"""
Manual Asdm Export

$Id: manualAsdmExport.py,v 1.55 2015/10/18 14:11:09 jgarcia Exp $
"""

import os
import sys
from pylab import size
from string import join
from string import replace
from optparse import OptionParser
import socket
import subprocess
import time
from get_uid_cycleX import get_uid_cycle
import analysisUtils as au


class manualAsdmExport(object):
    """
    Manual Asdm Export class
    Get the most recent asdm
    and export the data to osf-red.aiv.alma.cl, sco-red.aiv.alma.cl or both.
    """

    def __init__(self):
        """
        Set standard parameters
        task: SYSTEM, BASELINES, WVR, AMPCAL, BEAMMAP, FOCUS, POINT, ASTROHOLO,
        TOWERHOLO, ALLSKY, OFFSET, FASTSWITCHING, SCIENCE, CYCLE0, CALSURVEY
        Can be run from osf-red, sco-red, or casa0X.
        """
        self.taskNameList = ['SYSTEM', 'BASELINES', 'WVR', 'AMPCAL',
                             'BEAMMAP', 'FOCUS', 'POINT', 'ASTROHOLO',
                             'TOWERHOLO', 'ALLSKY', 'OFFSET', 'SCIENCE',
                             'CYCLE0', 'CALSURVEY']
        self.hostname = socket.gethostname()

        self.hostname = socket.gethostname()
        if 'casa' in self.hostname:
            self.basepath = '/mnt/jaosco/data'
        elif 'red' in self.hostname:
            self.basepath = '/data2/data'
        elif 'scops' in self.hostname:
            self.basepath = '/data2/data'
        else:
            print 'You must run this code on either casa0X, scops or osf-red.'
            sys.exit()


    def _makePath(self, task, date, pcode, caltype):
        """
        Makes a given path, and set the proper privileges

        return path or None
        """
        antennas = 'AOS'
        if task == 'CYCLE0':
            path = '%s/%s/%s' % (self.basepath, 'Cycle0', pcode)
        elif task == 'CYCLE1':
            path = '%s/%s/%s' % (self.basepath, 'Cycle1', pcode)
        elif task == 'CYCLE2':
            path = '%s/%s/%s' % (self.basepath, 'Cycle2', pcode)
        elif task == 'CYCLE3':
            path = '%s/%s/%s' % (self.basepath, 'Cycle3', pcode)
        elif task == 'CALSURVEY':
            path = '%s/%s/%s/%s%s' % (self.basepath, 'Calsurvey', date,
                                      exportedAsdm, caltype)
        else:
            path = '%s/RADIO/%s/%s/%s' % (self.basepath, task, antennas, date)
        cmd = 'mkdir -p %s' % path
        cmd3 = 'chmod 777 %s' % path
        print "### Creating path: %s ### \n" % path

        if os.system(cmd) == 0:
            os.system(cmd3)
            return path
        else:
            return None

    def _getAsdmAntennas(self, path, exportedAsdm):
        cmd = 'grep \'<name>\' %s/Antenna.xml' % exportedAsdm
        a = os.popen(cmd).read()
        b = a.split('<name>')

        returnVal = []

        for i in range(1, size(b) - 1):
            returnVal.append(b[i][0:4])
        return returnVal

    def _getAsdm(self, asdm, xmlonly, verbose):
        """
        Retrieve an asdm from archive
        """
        # check asdm name format
        if '://' not in asdm:
            asdm = replace(asdm, '___', '://')
            asdm = replace(asdm, '_', '/')

        if verbose == '':
            print '#### Retrieving ASDM: %s from Archive #### \n'%asdm
        if xmlonly:
            cmd = '/opt/DataPacker/bin/asdmExport -x %s %s ' % (asdm, verbose)
        else:
            cmd = '/opt/DataPacker/bin/asdmExport %s %s' % (asdm, verbose)
        os.system(cmd)

        exportedAsdm = replace(asdm, ':', '_')
        exportedAsdm = replace(exportedAsdm, '/', '_').strip()
        
        return exportedAsdm


    def _parse_file(self, inputFile):
        b = open(inputFile)
        obs = b.readlines()
        tasks = []
        asdms = []
        for i in range(size(obs)):
            line = obs[i]
            if line[0] != '\n' and line[0] != '#':
                sline = line.split()
                print sline
                if size(sline) == 2:
                    tasks.append(line.split()[0].lstrip().upper())
                    asdms.append(line.split()[1].lstrip())
                elif size(sline) == 1:
                    tasks.append(None)
                    asdms.append(line.split()[0].lstrip())
                else:
                    print("file format should be 2 columns of TASKS, UIDs, or "
                          "1 column of UIDs")
                    sys.exit()
        b.close()
        return tasks, asdms

if __name__ == '__main__':
    usage = '''
        manualAsdmExport --help
        manualAsdmExport  [-t <task>] -u uid

        examples:

        manualAsdmExport -t Cycle0 uid://X02/X2acee/X1
        manualAsdmExport -t CALSURVEY -c f -u uid://X02/X2acee/X1
        manualAsdmExport -t SYSTEM -u uid://X02/X2acee/X1
        manualAsdmExport -t SYSTEM uid://X02/X2acee/X1

        example(3): also keep the ASDM but do not generate the plot
        manualAsdmExport  -t SYSTEM -k -n uid://X02/X2acee/X1

        example(4):  to read the inputs from a batch file
        manualAsdmExport  -f fileName
        Where the format of fileName is
        # Comment  1
        # Comment 2
        # taskName  uid
        # uid
        SYSTEM      uid://A002/X12345/X65
        POINT	uid://A002/X12345/X66
        FOCUS	uid://A002/X12345/X68
        Cycle0	uid://A002/X9876/X32
        uid://A002/X12345/X7
        uid___A002_X12345_X3
        '''

    #creating class ...
    me = manualAsdmExport()

    #options ....
    parser = OptionParser(usage=usage)

    parser.add_option("-t", "--task",
        dest="task",
        help="Task Type : " + join(me.taskNameList, ' '),
        default='CYCLE#')

    parser.add_option("-c", "--caltype",
        dest="caltype",
        help="for Calsurvey data give type: g (grid), c (cone search), "
             "f (flux check)",
        default=None)

    parser.add_option("-u", "--uid",
        dest="uid",
        help="UID  of the ASDM to retrieve. Can be a comma separated list for multiple asdms",
        default=None)

    parser.add_option("--here",
        dest="here",
        help="Forces the script to leave the exported files here,"
             " where the script is run. Overrides all other directory"
             " changing effects.",
        action="store_true",
        default=False)

    parser.add_option("-s", "--scans",
        dest="scans",
        help="scans to convert in asdm2MS. See asdm2MS help for syntax. "
             "Default is to fill all scans.")

    parser.add_option("-k", "--keepasdm",
        dest="keepasdm",
        metavar="keepasdm",
        action="store_true",
        default=False,
        help=" -k will keep the asdm in current directory after exporting. "
             " default = delete the asdm.")

    parser.add_option("-n", "--noplot",
        dest="noplot",
        action="store_true",
        default=False,
        help=" -n will not generate the plot of xcorr amp vs time. "
             "Default = generate the plot")

    parser.add_option("-v", "--verbose",
        dest="verbose",
        action="store_false",
        default=True,
        help=" -v option will output the verbose logs to debug the command. "
             "Default is True")

    parser.add_option("-f", "--file",
        dest="file",
        help="file = batch to process multiple uids at once",
        default=None)

    parser.add_option("-r", "--release",
        dest="release",
        help="Set the casa release version to use",
        default='4.4.0')

    (options, args) = parser.parse_args()

    print options.file

    #checking and parsing options....
    if options.file is not None:
        print "taking the inputs from file %s" %options.file
    elif len(args) == 0 and options.uid is None:
        uids = None
        print "You must specify at least a uid"
        sys.exit()
    elif len(args) == 0 and options.uid is not None:
        uids = options.uid.split(',')
    else:
        uids = args[0].split(',')

    if options.verbose:
        verbose = ''
        verbose2 = ''
    else:
        verbose = '1> /dev/null'
        verbose2 = '2>/dev/null'

    tasks = []
    asdms = []

    if options.file is not None:
        tasks, asdms = me._parse_file(options.file)
        options.noplot = True
    else:
        for uid in uids:
            tasks.append(options.task.upper())
            asdms.append(uid.strip())

    for i in range(size(asdms)):
        taskName = tasks[i]
        asdm = asdms[i]
        print taskName, asdm

        # quit if exporting SciVer data on osf-red or Casa0x
        if taskName == 'SCIENCE':
            if 'casa' in me.hostname:
                print 'You should ONLY export SciVer data on scops machines'
                sys.exit()
            elif 'red' in me.hostname:
                print 'You should ONLY export SciVer data on scops machines'
                sys.exit()
            else:
                print 'Exporting SciVer data to /data/RADIO/SCIENCE/AOS/date/'


        # getting asdm xmlonly
        exportedAsdm = me._getAsdm(asdm, True, verbose)

        # only check if no task is defined or if task is cycle0 AND did not
        # request to leave here.
        if (taskName is None or 'CYCLE' in taskName) and (options.here is False):
            os.chdir(exportedAsdm)
            puidt = subprocess.Popen('grep ObsProject ExecBlock.xml | cut '
                               '-f 2 -d\\"', stdout=subprocess.PIPE, shell=True)
            puid = puidt.communicate()[0].split()[0]
            print "Project UID: %s" % puid
            os.chdir('..')
            
            uid_cycle0 = get_uid_cycle(0)
            uid_cycle1 = get_uid_cycle(1)
            uid_cycle2 = get_uid_cycle(2)
            uid_cycle3 = get_uid_cycle(3)
            pcode = 0
            for uids in uid_cycle0:
                if puid in uids:
                    pcode = uids.split()[1]
                    print "This asdm correspond to a Cycle 0 project %s" % pcode
                    taskName = 'CYCLE0'
            if pcode == 0:
                for uids in uid_cycle1:
                    if puid == uids.split()[0]:
                        pcode = uids.split()[1]
                        print "This asdm correspond to a Cycle 1 project %s" % pcode
                        taskName = 'CYCLE1'
            if pcode == 0:
                for uids in uid_cycle2:
                    if puid in uids.split()[0]:
                        pcode = uids.split()[1]
                        print "This asdm correspond to a Cycle 2 project %s" % pcode
                        taskName = 'CYCLE2'
            if pcode == 0:
                for uids in uid_cycle3:
                    if puid in uids.split()[0]:
                        pcode = uids.split()[1]
                        print "This asdm correspond to a Cycle 3 project %s" % pcode
                        taskName = 'CYCLE3'
            if pcode == 0:
                print("This asdm does not match any project in the UID_Project"
                      "Number.txt file. \n It thus does not correspond to any "
                      "CYCLE 0, CYCLE 1, CYCLE 2 or CYCLE 3 project. \n Make sure you have included the "
                      "correct task name -t Task or -t Cycle0")
                os.system('rm -Rf ' + exportedAsdm + '*')
                sys.exit()
        else:
            pcode = 0

        if options.caltype is not None:
            caltype = '_%s' % options.caltype
        else:
            caltype = ''

        if options.here is False:
            # creating path
            print '#### Creating new path #### \n'
            asdm_date, asdm_date_mjdSec = au.getObservationStartDateFromASDM(exportedAsdm)
            asdm_date = asdm_date[0:10].replace('-','')            
            path = me._makePath(taskName, asdm_date, pcode, caltype)
        else:
            path = './'

        # delete xml-only ASDM
        print '#### Deleting xml-only ASDM and .TAR file #### \n'
        os.system('rm -Rf ' + exportedAsdm + '*')

        # changing to path
        os.chdir(path)

        # getting asdm
        exportedAsdm = me._getAsdm(asdm, False, verbose)

        # convert to MS in new path
        print '#### Converting ASDM to MS  #### \n'
        ts = au.makeTimeStamp()
        tmpfile = 'tmp_%s.txt' % ts
        if options.scans is not None:
            scans = 'scans = \'%s\'' % options.scans
        else:
            a = os.popen('grep scanNumber %s/Scan.xml | tail -1 ' %
                         exportedAsdm).read().split('>')[1].split('<')[0]
            print "#### Converting scans 1 through %s #### \n" % a
            scans = 'scans = \'1~%s\'' % a

        f = open(tmpfile, 'w')
        f.write(
            'importasdm(asdm=\'%s/%s\', useversion=\'v3\', asis=\'Antenna Station CalWVR Receiver\','
            ' overwrite=T, process_flags=T, %s)\n' %
            (path, exportedAsdm, scans))
        f.write('listobs(vis=\'%s/%s.ms\', listfile=\'curr.obs\')' %
                (path, exportedAsdm))
        f.close()
        os.system('casapy -c %s -r %s --nologger --log2term %s %s' %
                  (tmpfile, options.release, verbose, verbose2))
        os.system('cat curr.obs &')

        if options.keepasdm:
            print("### Keeping the asdm in its directory. are you sure you want"
                  " this?### \n ")
        else:
            print "### Deleting the asdm from its directory. thank you ###\n"
            os.system("rm  -Rf %s/%s" % (path, exportedAsdm))

        if options.noplot is False:
            print '#### Plotting in casaplotms #### \n'
            os.system('casaplotms vis=%s/%s.ms xaxis=time yaxis=amp '
                      'coloraxis="Field" avgchannel="9999" antenna="0&1" '
                      'corr="XX,YY" %s %s' %
                      (path, exportedAsdm, verbose, verbose2))

        # cleanup
        os.system('rm -f %s' % tmpfile)
        os.system('rm *.log curr.obs -f')

# ___oOo___
