#!/usr/bin/env python

import os
import re
import sys
from pylab import *
import pylab
from datetime import datetime
from datetime import timedelta

if len(sys.argv) != 3: sys.exit('Missing input arguments.')
if re.search('^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3}$', sys.argv[1]) == None: sys.exit('First argument has incorrect format.')
if re.search('^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3}$', sys.argv[2]) == None: sys.exit('Second argument has incorrect format.')

os.system('rm -f jlog.txt')

print 'Querying the database. Please wait.'
os.system('archiveLoggingCli '+sys.argv[1]+' '+sys.argv[2]+' -w jlog.txt &> /dev/null')
print 'Done.'

print 'Reading log file.'
fp = open('jlog.txt', mode='r')
fc = fp.read()
fp.close()
print 'Done.'

os.system('rm -f jlog.txt')

print 'Processing log entries.'
m = re.findall('<Info(.*?)</Info>', fc, re.MULTILINE|re.DOTALL)

logData = {}
ij = -1

for i in range(len(m)):
	ij = ij+1
	logData[ij] = {}
	logData[ij]['date'] = datetime.strptime((re.findall('TimeStamp=\"(.*?)\"', m[i], re.DOTALL))[0], '%Y-%m-%dT%H:%M:%S.%f')
	logData[ij]['type'] = 'Info'
	logData[ij]['source'] = (re.findall('SourceObject=\"(.*?)\"', m[i], re.DOTALL))[0]
	logData[ij]['message'] = (re.findall('<!\[CDATA\[(.*?)\]\]>', m[i], re.DOTALL))[0]

obsStarted = 0

for i in logData.keys():

	if re.search('Executor: received ExecBlockStartedEvent for SchedBlock', logData[i]['message'], re.IGNORECASE) != None and obsStarted == 0:
		fe = logData[i]['message'].split()
		uid = fe[5]
		obsStartTime = logData[i]['date']
		obsEndTime = 0
		subScanData = {}
		ij = -1
		scanNum = 0
		scanIntent = ''
		obsStarted = 1

	if re.search('Sending ExecBlockStartedEvent', logData[i]['message'], re.IGNORECASE) != None and obsStarted == 0:
		uid = '?'
		obsStartTime = logData[i]['date']
		obsEndTime = 0
		subScanData = {}
		ij = -1
		scanNum = 0
		scanIntent = ''
		obsStarted = 1

	if re.search('Starting scan number [0-9]+ on array', logData[i]['message'], re.IGNORECASE) != None and obsStarted == 1:
		fe = logData[i]['message'].split()
		scanNum = int(fe[3])
		scanIntent = fe[11]

	if re.search('Subscan [0-9]+ has an intent of', logData[i]['message'], re.IGNORECASE) != None and obsStarted == 1:
		ij = ij+1
		subScanData[ij] = {}
		fe = logData[i]['message'].split()
		subScanData[ij]['num'] = int(fe[1])
		subScanData[ij]['scan'] = scanNum
		subScanData[ij]['intent'] = scanIntent + '_' + fe[6]
		subScanData[ij]['duration'] = float(fe[8])
		subScanStartTime = datetime.strptime(fe[13], "%H:%M:%S.%f")
		if (subScanStartTime-obsStartTime).seconds < 12*3600:
			subScanData[ij]['startTime'] = datetime.combine(obsStartTime.date(), subScanStartTime.time())
		else:
			subScanData[ij]['startTime'] = datetime.combine(obsStartTime.date()+timedelta(1), subScanStartTime.time())
		

	if re.search('Received ExecBlockEndedEvent for ExecBlock', logData[i]['message'], re.IGNORECASE) != None and obsStarted == 1:
		obsEndTime = logData[i]['date']

		print '-------------'
		print '> UID = ', uid
		print '> Start time = ', obsStartTime
		print '> End time = ', obsEndTime
		obsDuration = obsEndTime-obsStartTime
		obsDuration = obsDuration.seconds + obsDuration.microseconds / 1e6
		print '> Total duration (seconds) = ', "%.2f" % obsDuration
		print '> Number of subscans = ', len(subScanData)
		if len(subScanData) != 0:

			subScanDuration = 0.
			for j in subScanData.keys():
				subScanDuration = subScanDuration+subScanData[j]['duration']
			print '> Total subscan duration (seconds, fraction of total duration) = ', "%.2f" % subScanDuration, "%.2f" % (subScanDuration / obsDuration)

			scanList = []
			for j in subScanData.keys():
				scanList.append(subScanData[j]['scan'])
			scanList = sorted(dict.fromkeys(scanList).keys())

			intentList = []
			for j in subScanData.keys():
				intentList.append(subScanData[j]['intent'])
			intentList = sorted(dict.fromkeys(intentList).keys())

			preSubScanDuration = subScanData[subScanData.keys()[0]]['startTime'] - obsStartTime
			preSubScanDuration = preSubScanDuration.seconds + preSubScanDuration.microseconds / 1e6
			print '> Duration from start of observation to start of 1st subscan (seconds) = ', "%.2f" % preSubScanDuration

			postSubScanDuration = obsEndTime - subScanData[subScanData.keys()[len(subScanData)-1]]['startTime']
			postSubScanDuration = postSubScanDuration.seconds + postSubScanDuration.microseconds / 1e6 - subScanData[subScanData.keys()[len(subScanData)-1]]['duration']
			print '> Duration from last subscan to end of observation (seconds) = ', "%.2f" % postSubScanDuration

			print '> Total subscan duration per scan (seconds, fraction of total subscan duration, fraction of total duration)'
			for j in scanList:
				scanSubScanDuration = 0
				for k in subScanData.keys():
					if subScanData[k]['scan'] == j:
						scanSubScanDuration = scanSubScanDuration+subScanData[k]['duration']
				print j, ' = ', "%.2f" % scanSubScanDuration, "%.2f" % (scanSubScanDuration / subScanDuration), "%.2f" % (scanSubScanDuration / obsDuration)

			print '> Total subscan duration per intent (seconds, fraction of total subscan duration, fraction of total duration)'
			for j in intentList:
				intentSubScanDuration = 0.
				for k in subScanData.keys():
					if subScanData[k]['intent'] == j:
						intentSubScanDuration = intentSubScanDuration+subScanData[k]['duration']
				print j, ' = ', "%.2f" % intentSubScanDuration, "%.2f" % (intentSubScanDuration / subScanDuration), "%.2f" % (intentSubScanDuration / obsDuration)

			print '> Average duration between subscans per scan, time to next scan (seconds)'
			for j in scanList:
				avgInterSubScanDuration = 0
				subScanCount = 0
				for k in subScanData.keys():
					if subScanData[k]['scan'] == j:
						avgInterSubScanDuration = avgInterSubScanDuration+subScanData[k]['duration']
						subScanCount = subScanCount+1
						klast = k
				if klast < len(subScanData)-1:
					avgScanDuration = subScanData[klast+1]['startTime'] - subScanData[klast]['startTime']
					avgScanDuration = avgScanDuration.seconds + avgScanDuration.microseconds / 1e6 - subScanData[klast]['duration']
					print j, ' = ', "%.2f" % (scanSubScanDuration / subScanCount), "%.2f" % avgScanDuration
				else:
					print j, ' = ', "%.2f" % (scanSubScanDuration / subScanCount)

		print ''

		obsStarted = 0
