
def obsTimings2(asdmName):

	import os
	import re
	import sys
	from asdm import ASDM
	from datetime import datetime
	from datetime import timedelta

	asdmName = os.path.split(asdmName)
	if asdmName[1] == '': asdmName=os.path.split(asdmName[0])

	a = ASDM()
	a.setFromFile(os.path.join(asdmName[0], asdmName[1]), True)

	b = a.execBlockTable().get()
	if len(b) != 1: sys.exit('ERROR: More than one line in execBlock table.')
	b = b[0]

	obsStartTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(b.startTime())))[0]
	obsStartTime = datetime.strptime(obsStartTime, '%Y-%m-%dT%H:%M:%S')
	obsEndTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(b.endTime())))[0]
	obsEndTime = datetime.strptime(obsEndTime, '%Y-%m-%dT%H:%M:%S')
	timeDiff = obsEndTime - obsStartTime
	obsDuration = 86400. * timeDiff.days + timeDiff.seconds
	print ''
	print 'total duration of observation = %d' %obsDuration

	b = a.scanTable().get()
	nScans = len(b) 

	scanTable = {}
	for i in range(nScans):
		c = b[i]
		scanNum = int(c.scanNumber())
		scanTable[scanNum] = {}
		scanTable[scanNum]['numSubscan'] = int(c.numSubscan())
		scanTable[scanNum]['scanIntent'] = c.scanIntent()
		scanStartTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(c.startTime())))[0]
		scanTable[scanNum]['startTime'] = datetime.strptime(scanStartTime, '%Y-%m-%dT%H:%M:%S')
		scanEndTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(c.endTime())))[0]
		scanTable[scanNum]['endTime'] = datetime.strptime(scanEndTime, '%Y-%m-%dT%H:%M:%S')

	b = a.subscanTable().get()
	nSubscans = len(b) 

	subscanTable = {}
	for i in range(nSubscans):
		subscanTable[i] = {}
		c = b[i]
		subscanTable[i]['scanNum'] = int(c.scanNumber())
		subscanTable[i]['subscanNum'] = int(c.subscanNumber())
		subscanStartTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(c.startTime())))[0]
		subscanTable[i]['startTime'] = datetime.strptime(subscanStartTime, '%Y-%m-%dT%H:%M:%S')
		subscanEndTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(c.endTime())))[0]
		subscanTable[i]['endTime'] = datetime.strptime(subscanEndTime, '%Y-%m-%dT%H:%M:%S')

	b = a.configDescriptionTable().get()
	nConfigs = len(b) 

	configDescTable = {}
	for i in range(nConfigs):
		c = b[i]
		configNum = int(c.configDescriptionId().get())
		configDescTable[configNum] = {}
		configDescTable[configNum]['procType'] = str(c.processorType())
		configDescTable[configNum]['specType'] = str(c.spectralType())

	b = a.mainTable().get()
	nData = len(b)

	mainTable = {}
	for i in range(nData):
		c = b[i]
		configNum = int(c.configDescriptionId().get())
		if configDescTable[configNum]['procType'] == 'CORRELATOR' and configDescTable[configNum]['specType'] == 'FULL_RESOLUTION':
			mainTable[i] = {}
			intTime = (re.findall('[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}', str(c.time())))[0]
			intTime = datetime.strptime(intTime, '%Y-%m-%dT%H:%M:%S')
			interval2 = long(c.interval().get() / 1000 / 2)
			interval2 = timedelta(microseconds=interval2)
			mainTable[i]['startTime'] = intTime - interval2
			mainTable[i]['endTime'] = intTime + interval2
			mainTable[i]['subscanDuration'] = mainTable[i]['endTime'] - mainTable[i]['startTime']
			mainTable[i]['scanNum'] = int(c.scanNumber())
			mainTable[i]['subscanNum'] = int(c.subscanNumber())
			mainTable[i]['scanIntent'] = scanTable[int(c.scanNumber())]['scanIntent']

	intentList = []
	for i in scanTable.keys():
		intentList.extend(scanTable[i]['scanIntent'])
	intentList = ['%s' %i for i in intentList]
	intentList = sorted(dict.fromkeys(intentList).keys())

	print ''
	print 'intent - obs efficiency - total duration'
	for i in intentList:
		intentDuration = 0.
		for j in mainTable.keys():
			subscanIntents = mainTable[j]['scanIntent']
			subscanIntents = ['%s' %k for k in subscanIntents]
			if i in subscanIntents:
				timeDiff = mainTable[j]['subscanDuration']
				intentDuration = intentDuration + 86400. * timeDiff.days + timeDiff.seconds + timeDiff.microseconds / 1.e6
		print '%s - %.3f - %.1f' %(i, 1.0*intentDuration/obsDuration, intentDuration)

	scanTable2 = {}
	for i in scanTable.keys():
		scanTable2[i] = {}
		startTime = obsEndTime
		endTime = obsStartTime
		integTime = 0.
		nSubscans = 0
		for j in mainTable.keys():
			if mainTable[j]['scanNum'] == i:
				timeDiff = mainTable[j]['subscanDuration']
				integTime = integTime + 86400. * timeDiff.days + timeDiff.seconds + timeDiff.microseconds / 1.e6
				nSubscans = nSubscans+1
				if mainTable[j]['startTime'] < startTime: startTime = mainTable[j]['startTime']
				if mainTable[j]['endTime'] > endTime: endTime = mainTable[j]['endTime']
		scanTable2[i]['integTime'] = integTime
		scanTable2[i]['nSubscans'] = nSubscans
		scanTable2[i]['startTime'] = startTime
		scanTable2[i]['endTime'] = endTime

	scanList = scanTable.keys()
	minTime = 86400.
	maxTime = 0.
	sumTime = 0.
	print ''
	print 'time between scans:'
	for i in range(len(scanList)-1):
		timeDiff = scanTable2[scanList[i+1]]['startTime'] - scanTable2[scanList[i]]['endTime']
		timeDiff = 86400. * timeDiff.days + timeDiff.seconds + timeDiff.microseconds / 1.e6
		if timeDiff < minTime: minTime = timeDiff
		if timeDiff > maxTime: maxTime = timeDiff
		sumTime = sumTime + timeDiff
		print scanList[i], scanList[i+1], timeDiff
	sumTime = sumTime / (len(scanList)-1)
	print '\ntime between scans: avg, max, min'
	print '%.1f %.1f %.1f' %(sumTime, maxTime, minTime)

#	print ''
#	print 'scan number - number of subscans - intents - scan efficiency'
#	for i in scanTable.keys():
#		timeDiff = scanTable[i]['endTime'] - scanTable[i]['startTime']
#		scanDuration = 86400. * timeDiff.days + timeDiff.seconds
#		nSubscan = scanTable[i]['numSubscan']
#		scanIntents = scanTable[i]['scanIntent']
#		scanIntents = ['%s' %k for k in scanIntents]
#		subscanDuration = 0.
#		for j in subscanTable.keys():
#			if subscanTable[j]['scanNum'] == i:
#				timeDiff = subscanTable[j]['endTime'] - subscanTable[j]['startTime']
#				subscanDuration = subscanDuration + 86400. * timeDiff.days + timeDiff.seconds
#		scanTable[i]['scanEfficiency'] = 1.0*subscanDuration/scanDuration
#		print '%s %d %s %.3f' %(i, nSubscan, ','.join(scanIntents), scanTable[i]['scanEfficiency'])

#	print ''
#	print 'intent - scan efficiencie: avg, max, min'
#	for i in intentList:
#		minEff = 1.
#		maxEff = 0.
#		sumEff = 0.
#		nScans = 0
#		for j in scanTable.keys():
#			scanIntents = scanTable[j]['scanIntent']
#			scanIntents = ['%s' %k for k in scanIntents]
#			if i in scanIntents:
#				if scanTable[j]['scanEfficiency'] > maxEff: maxEff = scanTable[j]['scanEfficiency']
#				if scanTable[j]['scanEfficiency'] < minEff: minEff = scanTable[j]['scanEfficiency']
#				sumEff = sumEff + scanTable[j]['scanEfficiency']
#				nScans = nScans + 1
#		sumEff = 1.0*sumEff / nScans
#		print '%s %.3f %.3f %.3f' %(i, sumEff, maxEff, minEff)

#	print ''
#	print 'scan number - number of subscans - intents - time between subscans: avg, max, min'
#	for i in scanTable.keys():
#		subscanList = []
#		for j in subscanTable.keys():
#			if subscanTable[j]['scanNum'] == i: subscanList.append(j)
#		nSubscan = scanTable[i]['numSubscan']
#		if len(subscanList) != nSubscan: sys.exit('ERROR: Mismatch of subscan numbers.')
#		scanIntents = scanTable[i]['scanIntent']
#		scanIntents = ['%s' %k for k in scanIntents]
#		minTime = 86400.
#		maxTime = 0.
#		sumTime = 0.
#		if len(subscanList) == 1:
#			scanTable[i]['timeBetweenSubscans'] = 0.
#			print '%s %d %s' %(i, nSubscan, ','.join(scanIntents))
#		else:
#			for j in range(len(subscanList)-1):
#				timeDiff = subscanTable[subscanList[j+1]]['startTime'] - subscanTable[subscanList[j]]['endTime']
#				timeDiff = 86400. * timeDiff.days + timeDiff.seconds
#				if timeDiff < minTime: minTime = timeDiff
#				if timeDiff > maxTime: maxTime = timeDiff
#				sumTime = sumTime + timeDiff
#			sumTime = sumTime / (nSubscan-1)
#			scanTable[i]['timeBetweenSubscans'] = sumTime
#			print '%s %d %s %.1f %.1f %.1f' %(i, nSubscan, ','.join(scanIntents), scanTable[i]['timeBetweenSubscans'], maxTime, minTime)
