#!/usr/bin/env python

# type execfile('planet-etamb.py') to set up, then follow instructions below
# plots and fits individual scans, flags bad fits, gets mean and rms of all scans per polarisation/antenna
# gets planet temp/diam using analysisUtilsSD, estimates coupling to source.
# outputs eta_mb for each poln/antenna separately. option to append to standard file.
# must be run using casapy

# **************************************************************

print ' planet-etamb.py'
print ' run using  etamb(vis="uid_name.ms")   [default takes TP data in spw1, appends result to file]'
print ' Other options: to use autocorrelation data or different spws, and/or not write to file use:'
print ' etamb(vis="uid_name.ms",data_spw=5,fwrite=F,sigma=3.0,tsys_spw=1,zero_spw=13,usecorr=F,antreq="DV04",usemax=T,dlength=100,usebeam=T,cutav=2)'
print '    these options will not write to file, take 3-sigma fit data, show only DV04 and not use corr data (with Zero point offset data in spw 13)'
print '    use the maximum value rather than the fit peak for extended sources like Jupiter'
print '    set the length of the cut (in arcsec) and usebeam=T to use the estimate of the mean fwhm (default dlength=100) to get the efficiency'
print '    dlength=span (from python command) x 2 , as this is the 1/2 width of the cuts'
print '    (otherwise will assume beam fwhm=1.11wl/d)'
print '    cutav is the number of cuts scans to average together (will average every 4 scans, as each of these is different: L & R/Az & El'
print '    check the correct spws using listobs'
print '    use plot_resid=T to plot only the residuals'
print '    nb: if usecorr=F, it needs zero point data. this is now a separate spwID, which must be specified'
print '    nb: in DUAL mode, telcal currently only generates Tsys from CORR data (not TP). need to rerun offline tc_atmosphere to change the syscal tables'
print ' Uses planetary data from CASA standard model'
print ' beamsize assumed is symmetrical, fwhm=1.11wl/d'


# dlength is full length (in arcsec) of scan (ie waht was typed in the command to run)

import matplotlib
import pylab
import numpy
from numpy import *
import scipy
from scipy import *
from scipy import stats

global verb

# gaussian fitting in python from https://github.com/tiagopereira/python_tips/wiki/Scipy%3A-curve-fitting
def gaussian (B,x):
	return B[3]+B[2]*exp(-(0.693*(x-B[0])**2/((B[1]/2)**2)))
  # modified so that: b[3] is y offset.  b[2] is amplitude.  b[1] is FULL width at 1/2 maximum.  b[0] is x offset

# return B[3]+B[2]*exp(-((x-B[0])**2)

def errfunc(p,x,y):
	return (y-gaussian(p,x))
#tsysa=aU.TsysExplorer(vis,antenna='CM03',spwID=6)
#print tsysa.tsys

verb=T

def etamb(vis='',data_spw=5,fwrite=T,sigma=2.0,tsys_spw=6,antreq='all',usemax=F,dlength=100.0,usebeam=T,usecorr=T,zero_spw=-1,cutav=1,plot_resid=F,masksidelobes=False):

 global planetf_mean,planetf_resid,width0,width1,xtemp,ytemp
 global x,y,mask

 if fwrite:
	 print 'will append data to file   ****** '
 else:
	 print 'NOT writing data to file   ****'

 if usebeam:
	 print 'using measured beamsize to get efficiency ***'
 else:
	 print 'using theoretical beamsize to get efficiency (1.11wl/d) *****'

 if usecorr:
	insource='CORR'
 else:
	insource='TP'
	if zero_spw<0:
		print ' *** you needs to specify a spw which contains the zero point data!!!'
	else:
		print 'using zero spw',zero_spw
 print 'using data from SPW ID=',data_spw, ' insource = ',insource


# also selected Tsys spw ID:

 print 'taking Tsys data from spw', tsys_spw
# currently for dual mode this is the CORR channel-averaged data used in telcal for Tsys for BB0.
# the TP Tsys calculation is not working yet (ICT ticket created for this)
# for non-dual-mode, this can be set to 1 (for BB1).

 tb.open(vis+'/ANTENNA')
 ans=tb.getcol('NAME')
 tb.close()

 tb.open(vis+'/POINTING')
 tb1=tb.query('ANTENNA_ID==0')
 dd=tb1.getcol('DIRECTION')
 elev=dd[1][0][0]*57.2957
 print 'elevation ',elev,' deg'

 p0=zeros(4)
 pylab.clf()

# tb.open(vis+'SYSCAL')  if Tsys needed

 nant=ans.size

# read in Tsys for all antennas/polns, on specified spw
 tsysa=zeros((nant,2))
 ts=aU.TsysExplorer(vis,spwID=tsys_spw)
 for iant in range(0,nant):
	anten=ans[iant]
	ts.setAntenna(anten)
	for apol in range(0,2):
#		print iant,anten,apol,tsys_spw
		try:
			tsysa[iant,apol]=ts.tsys[apol][0][0]
		except:
			print 'failed to get Tsys array for antenna ',anten,' - Tsys will be set to 1.0'
			tsysa[iant,apol]=1.0

 print ' Tsys array:'
 print tsysa
#		tsysa[iant,apol]=100.0
#		print anten,apol,tsysa[iant,apol]


# get scan number for pointing observation:
 vm=aU.ValueMapping(vis)
 scanp=aU.getScansForIntentFast(vm,vm.uniqueScans,'CALIBRATE_POINTING#ON_SOURCE')[0]


# get the number of stateIDs, which is the number of cuts / 2, then can loop through these:
# now open the data file:
 tb.open(vis)
 dat2=tb.query('ANTENNA1==%d && ANTENNA2==%d && SCAN_NUMBER==%d && DATA_DESC_ID==%d'%(0,0,scanp,data_spw))
 ncuts=len(set(dat2.getcol('STATE_ID')))   # number of unique items in the state_ID table, for this scan
 print 'pointing scan',scanp,' ncuts ',ncuts

 planetf=numpy.zeros((nant,2,ncuts))  # planet fractional flux
 planetf_mean=numpy.zeros((nant,2))   # mean fractional flux
 planetf_std=numpy.zeros((nant,2))   # std fractional flux
 planetf_width=numpy.zeros((nant,2)) # width of fit
 planetf_width_std=numpy.zeros((nant,2)) # width of fit
 beamf_width=numpy.zeros((nant,2)) # deconvolved beamwidth
 planetf_sidelobe=numpy.zeros((nant,2))  # sidelobe, as a fraction of total
 planetf_ell=numpy.zeros((nant,2))
 
 nu=ts.freq[0][0]    # mean freq of observations
 if 84.e9<nu and 116.e9>nu: band=3
 if 125.e9<nu and 163.e9>nu: band=4
 if 211.e9<nu and  275.e9>nu: band=6
 if 275.e9<nu and 373.e9>nu: band=7
 if 380.e9<nu and 520.e9>nu: band=8
 if 602.e9<nu and 720.e9>nu: band=9
 if 787e9<nu and 950e9>nu: band=10

 dateobs=aU.mjdsecToUT(ts.time)    # date of observations

 print 'date:', dateobs,' band',band

 ydata=zeros((nant,ncuts,2,126))
 used_cuts=zeros((nant,2))
 inn='dum'  # initial dummy value

 tb.open(vis)
 datacolumn=aU.getDataColumnName(vis)
 tb.open(vis)


# print datacolumn
 for iant in range(0,nant):   # loop over antennas
   # get theoretical beamsize estimate for this antenna:
    wl=3e8/nu
    if ans[iant][0]=="C":
	    diam=7.0
    else:
	    diam=12.0
    beam_the=206265.*1.11*wl/diam

    if antreq == 'all' or antreq == ans[iant]:
	print ''
	print 'antenna', ans[iant], ' band ', band
        
	for ipol in range(0,2):    # loop over pols
		if usecorr !=True:
			# get zero level:
			dat1=tb.query('ANTENNA1==%d && ANTENNA2==%d && SCAN_NUMBER==1 && DATA_DESC_ID==%d'%(iant,iant,zero_spw))
			y= real(dat1.getcol(datacolumn)[ipol,0,:])
			#		print iant,ans[iant],ipol,len(y)
			doZero=y.mean()
			print 'using Zero point level', doZero
		else:
			print 'not using Zero point measurements'
			doZero=0.0

#		print 'ZERO',doZero
#		print iant,ipol

		sky=numpy.zeros(64)    # sky for each cut   NB there may be < ncuts useful scans actually used in the end for this antenna/poln.
		planet=numpy.zeros(64) # planet for each cut
		frac=numpy.zeros(64); frac[:]=nan   # fraction for each cut
		width=numpy.zeros(64); width[:]=nan   # planet width for each cut
		width0=numpy.zeros(64); width0[:]=nan   # planet width Az for each cut
		width1=numpy.zeros(64); width1[:]=nan  # planet width El for each cut
		res_frac=numpy.zeros(64); res_frac[:]=nan   # residual, as a fraction of total integrated
		sidelobe_frac=numpy.zeros(64)   # inner sideblobe as a fraction of peak
		used_c=0
		if inn != 'a' and inn != '1':
			inn=raw_input("return (individual plots), a (all antennas), e (end without saving result): ")
			if inn == 'e':
				print 'finishing early'
				break
			pylab.clf()
		print 'pol',ipol
		if inn == 'a': pylab.clf()

		for nc in range(0,ncuts):   # data scan number
# first get all stateIDs for this scan:
			dat2=tb.query('ANTENNA1==%d && ANTENNA2==%d && SCAN_NUMBER==%d && DATA_DESC_ID==%d'%(iant,iant,scanp,data_spw))
			stateid=dat2.getcol('STATE_ID')[0]+nc*2   # this is the state ID for each cut in the right scan
			dat2=tb.query('ANTENNA1==%d && ANTENNA2==%d && SCAN_NUMBER==%d && DATA_DESC_ID==%d && STATE_ID==%d'%(iant,iant,scanp,data_spw,stateid))
#			print stateid

# get cut data:
			y= real(dat2.getcol(datacolumn)[ipol,0,:])   # NOT zero subtracted
			if nc == 0:
				ydata=zeros((4,(ncuts/4),len(y)))   # 4 directions, ncuts/4 repeats, number of samples per cut
				ydata_av=zeros((4,(ncuts/4/cutav),len(y)))   # same as above, but cutav less scans
			ydata[nc%4,nc/4,:]=y   # puts data into initial array

# average cuts together, cutav adjacent cuts (in same direction):
		ncutsav=ncuts/cutav
		if ncutsav<4:
			print ncuts,cutav,ncutsav,' is too few cuts after averaging - resetting ncutsav to minimum number (4) cuts '
			cutav=ncuts/4
			ncutsav=ncuts/cutav
#		print ncuts,cutav,(ncuts/4/cutav)
		for k in range(0,4):   # separate scans in different directions
			for i in range(0,(ncuts/4/cutav)):    # from 0 to number of independent measurements after averaging
				n_used_cuts=0
				for j in range(0,cutav):   # from 0 to the number of cuts to be averaged together
					# first check and correct problems with the start of the scan (ICT-923)
					istart=0
					y=ydata[k,(i*cutav+j),:]    # extracts 1 cut

		       			sky0=y[(istart+1):(istart+10)].mean()
				#	print i,j,k,istart,y[istart],sky0
					while abs(y[istart]) <0.001 and istart < (len(y)-10):    # this section is to get around flagged data (0.0) at scan start, ICT-923
						istart=istart+10
					if istart>0: print ' warning: samples to ', istart,' are flagged'

					if istart < len(y)/5:  # means there is enough data, so use it
						sky0=y[(istart+1):(istart+10)].mean()
						for i2 in range(istart):
							if y[i2] == 0.0: y[i2]=sky0     # sets the zero points to the mean sky level
#						if verb and y[1] ==0.0:
#							print y

						sky1=y[-10:-1].mean()   # assumes no dropped/flagged data at end
						sky_temp=(sky0+sky1)/2.  # mean sky level
						slop=(sky1-sky0)/len(y)   # slope of sky (1st order, removed at this stage)
						for i2 in range(len(y)):
							y[i2]=y[i2]-slop*real(i2)    # remove 1st-order polynomical (slope) of sky
						ydata_av[k,i,:]=ydata_av[k,i,:]+y[:]    # adds into average
						n_used_cuts=n_used_cuts+1
					else:    # this scan is no good, so don't include it
						print " data from one scan is all zeros!"

				if n_used_cuts>0:
					ydata_av[k,i,:]=ydata_av[k,i,:]/n_used_cuts     # gets the normalised average
				else:
					ydata_av[k,i,:]=ydata_av[k,i,:]*0.0    # sets to 0.0 if no cuts used
	#	print shape(ydata), shape(ydata_av)


		for nc in range(0,ncutsav):   # 4 directions, ncuts/cutav different samples
			nc0=nc%4   # number of independent direction
			nc1=nc/4   # number of sample
			y=ydata_av[nc0,nc1,:]
			if nc0%2==1:  # reverse for opposite direction scan
				y=y[::-1]
			sky0=y[1:20].mean()  # take 20 samples from each end to get first guess at sky. but ignore 1st & last samples

			x=arange(0,len(y))
			xg=arange(0,len(y),0.25)   # for gaussian plot
# get dstep from scan length:
			dstep=(1.0*dlength)/len(y)
#			print dlength,len(y),dstep,beam_the
# p0: 0=x0(mean posn.), 1=sigma(width), 2=ampl, 3=offset      First guesses at these:
			p0[0]=len(y)/2.0    # x position
			p0[1]=len(y)*(beam_the/dlength)    # x width, = beam full width, for now
			p0[2]=max(y)-sky_temp  # y amplitude
			p0[3]=sky_temp       # y offset

			if p0[2]<0: p0[2]=-p0[2]    # to catch errors in amp guess

			if verb: print 'guess: pos,wid,amp,offs',p0[0],p0[1],p0[2],p0[3]

                        try:
			    fita=scipy.optimize.leastsq(errfunc,p0,args=(x,y))    # first try without masking
			    mask=ones(len(y))
			    if masksidelobes:
				off1=2.5; off2=1.2
				mask[(p0[0]-p0[1]*off1):(p0[0]-p0[1]*off2)]=nan
				mask[(p0[0]+p0[1]*off2):(p0[0]+p0[1]*off1)]=nan
#				print 'mask',mask
#     second try includes masking, after subtracting offset:
                                y1=y[~isnan(mask)]; x1=x[~isnan(mask)]
                                fit=scipy.optimize.leastsq(errfunc,p0,args=(x1,y1))
				ytemp=y1; xtemp=x1
			    else:
			        fit=fita   # no masking of sidelobes
			    print 'fita ',fita, '   fit ',fit
 
                            xpos=fit[0][0]
			    xwidth=fit[0][1]
			    yoffset=fit[0][3]
			    if usemax:
				yamp=max(y)-yoffset
			    else:
				yamp=fit[0][2]
			except:  #case where fit fails
                            print ' fitting problem - taking defaults **'

			    xpos=p0[0]
			    xdwidth=p0[1]
			    yoffset=p0[3]
			    yamp=0.0
			if verb: print 'fit:  pos,wid,amp,offs',xpos,xwidth,yamp,yoffset
			if yoffset > 0.0:
				fraction=yamp/yoffset
			else:
				fraction=0.0

#   now estimate significance of this fit:
# first get rms of differential data (ch(n)-ch(n+1)):
                        position_offset=10.
			yd=zeros(len(y))
			for i in range(0,len(y)-1):
				yd[i]=y[i]-y[1+1]
			if xwidth<(len(y)/5.0) and abs(xpos-len(y)/2.)<2.*xwidth:
				for j in range(int(xpos-xwidth*2),int(xpos+xwidth*2)):
				    if j>0 and j<len(y):
					    yd[j]=0.0      # set the region around the peak to 0.0 for this calc. to avoid confusing rms
			cut_rms=std(yd)
#			print xwidth, abs(xpos-len(y)/2.), len(y)/5.0,sigma*cut_rms, yoffset
			if abs(xpos-len(y)/2.)<position_offset*xwidth and xwidth>1. and xwidth<(len(y)*sigma/5.0) and yamp>(sigma*cut_rms) and yamp>0.0 and yoffset > 0.0:
				qa=1
				frac[used_c]=fraction  
				width[used_c]=xwidth
				if (nc0/2)==0:
					width0[used_c]=xwidth
				else:
					width1[used_c]=xwidth
				print 'nc',nc,' xpos',xpos,' xwidth',xwidth
				y=y-yoffset   # subtract sky for plotting
			else:
				qa=0
				print 'nc',nc,' xpos',xpos,' xwidth',xwidth,' yamp',yamp,' rms',cut_rms,' yoffset',yoffset,'  => not included'
#				print y
				y=y-sky_temp   # subtract sky for plotting


#      calculate residuals from fit:
                        if qa==1:
				y_res=zeros(len(y))
				for i in range(len(y)):
					y_res[i]=y[i]-0
			
					y_res=(errfunc(fit[0],x,y)+yoffset)   # offset because fit has the offset in it
					res_frac[used_c]=sum(y_res)/sum(y)    # this is the residual, as a fraction of the total
	#		print sum(y_res),sum(y)
	#  find near-in sidelobe level, from separation =+/- width to +/-(width+width/2)
	                        s0=max((xpos-xwidth*1.5),5); s1=max((xpos-xwidth),20)
				s2=min((xpos+xwidth),(len(y)-20)); s3=min((xpos+xwidth*1.5),(len(y)-5))
#	     print xpos,s0,s1,s2,s3
                     #           print 's0,1,2,3', s0,s1,s2,s3
				sidel=(mean(y[s0:s1])+mean(y[s2:s3]))/2.0
				sidelobe_frac[used_c]=sidel/yamp

			print ''
#  plot cross-scan and fit, OR residuals only:
                        if not plot_resid:
				# this plots the data and fits
				if qa==1:    # only plot good cuts
					if (nc0/2)==0:
						pylab.plot(([x1*dstep for x1 in x[:-1]]),y[:-1],'bo',ms=3,label='Az'+str(nc0%2))
						pylab.plot(([x1*dstep for x1 in xg]),(gaussian(fit[0],xg)-yoffset),color='b')
					else:
						pylab.plot(([x1*dstep for x1 in x[:-1]]),y[:-1],'ro',ms=3,label='El'+str(nc0%2))
						pylab.plot(([x1*dstep for x1 in xg]),(gaussian(fit[0],xg)-yoffset),color='r')
						xtemp=x   # temporary!!
						ytemp=y
						print len(y)
			#	if qa==0:
			#		pylab.plot(([x1*dstep for x1 in x[:-1]]),y[:-1],'y.',ms=2,label='flag')    # optionally can plot bad points as yellow & black dashed line
	#				pylab.plot(xg,(gaussian(fit[0],xg)-yoffset),'k--')
				labl=ans[iant]+" B"+str(band)+" P"+str(ipol)+" spw"+str(data_spw)+" "+vis

 
 			else:  # plot residuals only
 
				if (nc0%2)==0:
					pylab.plot(([x1*dstep for x1 in x[:-1]]),(y_res[:-1]),'bo',ms=3)   # Az
				else:
					pylab.plot(([x1*dstep for x1 in x[:-1]]),(y_res[:-1]),'ro',ms=3)   # El
				labl=ans[iant]+" B"+str(band)+" P"+str(ipol)+" spw"+str(data_spw)+"  "+vis+"  residuals"

			try:
				if nc>0 and nc<3: pylab.legend(loc='upper right')
 			except:
				print 'legend problem'

			pylab.title(labl)

			ymax=max(y)
			pylab.plot([ ((len(y)/2)*dstep),((len(y)/2)*dstep)],[0.0,(abs(ymax)/10.0)],linestyle='dashed',color='green')
			pylab.xlabel('arcsec')
			if verb: print used_c,ans[iant],ipol,' planet fraction',fraction,' rms',cut_rms,' QA:',qa
			if qa==1: used_c=used_c+1   # increments number of used cuts
			
# ----- end of cut loop here

                if used_c >0:
			planetf_mean[iant,ipol]=mean(frac[0:used_c])
			planetf_std[iant,ipol]=std(frac[0:used_c])
			planetf_width[iant,ipol]=mean(width[0:used_c])*dstep    # FWHM, in pixels converted to arcsec using dstep
			planetf_width_std[iant,ipol]=std(width[0:used_c])*dstep    # std of FWHM, in pixels converted to arcsec if dstep is input
			planetf_sidelobe[iant,ipol]=mean(sidelobe_frac[0:used_c])*100.0   # mean inner sidelobe level, in percent
	#		if stats.nanmean(width1[0:used_c])>0.0001:  # to stop div zero
                	try:
				ellip=(stats.nanmean(width0[0:used_c]))/(stats.nanmean(width1[0:used_c]))
				# this method is used to avoid problems with fewer scans in Az cf. El
			except:
				ellip=0.0

#			print (stats.nanmean(width0[0:used_c])),(stats.nanmean(width1[0:used_c])),ellip

			planetf_ell[iant,ipol]=ellip    # ellipticity

#			else:
#				planetf_ell[iant,ipol]=0.0    # width1 is not fitted

		else:    # no data useful!
			planetf_mean[iant,ipol]=1e-6
			planetf_std[iant,ipol]=1e-6
			planetf_width[iant,ipol]=200.0
			planetf_width_std[iant,ipol]=1.0
			planetf_sidelobe[iant,ipol]=0.0
			planetf_ell[iant,ipol]=0.0

		print '- -'
		if cutav != 1: print ' averaging every ',cutav,' scans together in the same direction'
		if used_c==0: print 'Warning: all data from this scan is flagged'
		print used_c,'/',ncutsav, 'used. Mean ',planetf_mean[iant,ipol],' +/- ',planetf_std[iant,ipol], '  =',"%.1f"%(100.*planetf_std[iant,ipol]/planetf_mean[iant,ipol]),'%'
		print '            Mean width ',planetf_width[iant,ipol],' +/- ',planetf_width_std[iant,ipol],'  =',"%.1f"%(100.*planetf_width_std[iant,ipol]/planetf_width[iant,ipol]),'%'
		print '            Mean sidelobe ',planetf_sidelobe[iant,ipol], '%   mean ellipticity ',planetf_ell[iant,ipol]
                pylab.draw()
		nu_fil=str(int(nu/1.0e9))
#		print ans[iant]
#		print str(band)
#		print nu_fil
#		print vis[:-3]
		figname=ans[iant]+'_'+'B'+str(band)+'_P'+str(ipol)+'_'+nu_fil+'_'+vis[:-3]
		pylab.savefig(figname)
#                pylab.show()
	if inn == 'e':
		nant=iant
		break    # finishing early
 err=zeros((nant,2))

 print ''
 if usemax:
	 print 'SUMMARY planet fractional fluxes, from MAXIMUM value   Input spw:',data_spw
 else:
	 print 'SUMMARY planet fractional fluxes, from GAUSSIAN fit   Input spw:',data_spw
 for iant in range(0,nant):
   if antreq == 'all' or antreq == ans[iant]:
     for ipol in range(0,2):
	err[iant,ipol]=(1.*planetf_std[iant,ipol]/planetf_mean[iant,ipol])

 tb.close()

 target=aU.getFields(vis)[0]

# get planetary diam and brightness temp using analysisUtilsSD (must have this in ~/.casa/init.py)
# tp=aUsd.sso_params(vis)
# tp.setSrcName(target)
# tp.setSpwId(tsys_spw)
# tp.doCalc(dotbplot=F)
# Tbright=tp.getResults()['Brightness Temperature'][0]
# Psize=tp.getResults()['Apparent Size'][0][0]   # planet diam in arcsec
# nu=tp.getResults()['Frequency'][0][0]

 planet_data=aU.planetTemperature(vis=vis,spw=data_spw)     # this gets the planet brightness temperature,size. NB: uses wdent version of analysisUtils on osf-red!
 Tbright=planet_data[1]
 Psize=planet_data[2]
 
# now get the planet measured antenna temp. and work out eta_mb:
 planetTa=planetf_mean*tsysa

 wl=3.0e8/nu
 hnuk=6.626e-34*nu/1.38e-23
 Tp_RJ=hnuk/((exp(hnuk/Tbright)-1))
 Tcmb=2.78
 Tcmb_RJ=hnuk/((exp(hnuk/Tcmb)-1))
 J_pl=Tp_RJ-Tcmb_RJ

 print ' = = = = = = = = ='
 print target,Psize,'asec, ',Tbright,'K,   J=',J_pl,'K'
 print  "date: ",dateobs,", elev ",elev," deg, ",vis,"  ",insource
 print 'theoretical beams: 12m ',beam_the,' asec.  7m ',(beam_the*12./7.),' asec' 
# now do nominal sideband ratio correction to the planet antenna temperature, for the case of SD data, as the continuum
# signal is getting in both sidebands, but Tsys is for only one sideband
# NB - this assumes atmospheric transmission is equal in both sidebands, which may not be correct if the image
# sideband is in an opaque part of the atmosphere
 if band == 9 or band ==10:
	print 'band 9/10 DSB rx: correcting for sideband ratio of 0.5'
	planetTa=planetTa/2.0

 eta_mb=zeros((nant,2))
 beam=zeros(nant)
 eta_ap=zeros((nant,2))
 jyk=zeros((nant,2))
 
 for ipol in range(0,2):
   for iant in range (0,nant):
    if antreq == 'all' or antreq == ans[iant]:

     if ans[iant][0]=="C":
	     diam=7.0
     else:
	     diam=12.0

     if usebeam:   # uses measured beam size
# get and use deconvolved beam size, assuming planet is disk of size < fwhm.
          beamf_width[iant,ipol]=sqrt(planetf_width[iant,ipol]**2-numpy.log(2.0)/2.0*(Psize**2))
	  eta_c1=1-exp(-(log(2.0)*(Psize/beamf_width[iant,ipol])**2))
	  planet_area=pi*(Psize/2.)**2
	  beam_area=pi*(beamf_width[iant,ipol]/2.)**2
          eta_c=aU.diskCoupling(planet_area,beam_area)
#	  print eta_c,eta_c1
	  Ta=J_pl*eta_c
#	  print Ta,planetTa[iant,ipol]
	  eta_mb[iant,ipol]=planetTa[iant,ipol]/Ta
	  omega_b=1.133*(beamf_width[iant,ipol]/206265.0)**2.
#	  print omega_b,wl,diam,eta_mb[iant,ipol]
	  eta_ap[iant,ipol]=eta_mb[iant,ipol]/(3.1415*(diam/2.)**2*omega_b/(wl**2))
	  Ageom=3.1415*(diam/2.)**2
	  jyk[iant,ipol]=(2*1.38e-23/Ageom)*1.0e26/(eta_ap[iant,ipol])   # convert to Jy
#	  print iant,omega_b,eta_ap[iant,ipol],jyk[iant,ipol]

     else:     # uses theorertical beam size, 1.11*wl/d
	  beamf_width[iant,ipol]=206265.*1.11*wl/diam
	  eta_c1=1-exp(-(log(2.0)*(Psize/beamf_width[iant,ipol])**2))
	  planet_area=pi*(Psize/2.)**2
	  beam_area=pi*(beamf_width[iant,ipol]/2.)**2
          eta_c=aU.diskCoupling(planet_area,beam_area)
	  Ta=J_pl*eta_c
	  eta_mb[iant,ipol]=planetTa[iant,ipol]/Ta

     err[iant,ipol]=eta_mb[iant,ipol]*err[iant,ipol]
     if eta_mb[iant,ipol] < 0.0:   # sets the error to be 100% in this case
	     eta_mb[iant,ipol]=abs(eta_mb[iant,ipol])
	     err[iant,ipol]=eta_mb[iant,ipol]
#	print ipol,ans[iant],beam[iant],eta_c,Ta,eta_mb[iant,ipol]

 eta_ap_av=zeros(nant)
 jyk_av=zeros(nant)
 for iant in range(0,nant):
	 eta_ap_av[iant]=average(eta_ap[iant,:])
	 jyk_av[iant]=average(jyk[iant,:])
#	 print eta_ap[iant,:]
#	 print jyk[iant,:]

##print err
 print ' - - - -'*16
 if usebeam:
	 print "Ant  | Measured beam fwhm(ellipticity,sidelobe)    &   eta_mb     for",target," Tbright=","%.2f"%Tbright,"   Size=","%.2f"%Psize," nu=",nu/1e9,"GHz"
 else:
	 print "Ant  Theor.beam eta_mb  ",target," Tbright=","%.2f"%Tbright,"   Size=","%.2f"%Psize," nu=",nu/1e9,"GHz", ' eta_ap,   Jy/K '
 print "                pol X  (err)                           |         pol Y (err)                            | eta_ap, Jy/K  (pol.averaged)"
 print ' - - - -'*16
 for iant in range(0,nant):
   if antreq == 'all' or antreq == ans[iant]:
     print ans[iant],"|","%.2f"%beamf_width[iant,0],"+/-","%.2f"%planetf_width_std[iant,0],"asec (","%.2f"%planetf_ell[iant,0],",","%.1f"%planetf_sidelobe[iant,0],"%) ","%.3f"%eta_mb[iant,0],"+-","%.3f"%err[iant,0],"|"\
	     ,"%.1f"%beamf_width[iant,1],"+/-","%.2f"%planetf_width_std[iant,1],"asec (","%.2f"%planetf_ell[iant,1],"%.1f"%planetf_sidelobe[iant,1],"%) ","%.3f"%eta_mb[iant,1],"+-","%.3f"%err[iant,1],"|","%.3f"%eta_ap_av[iant],"   ","%.3f"%jyk_av[iant]
 print ' - - - -'*16


 if fwrite and inn !='e':   # writes to file
  filen='/users/wdent/CSV2919/New_band'+str(band)+'_etamb.txt'
  print 'appending results to ',filen,' for band', band
  f=open(filen,'a')
  f.write(' - - - - - - - - - - - - - - - \n')
  for iant in range(0,nant):
    if antreq == 'all' or antreq == ans[iant]:

      line=dateobs[:-2]+","+vis+","+ans[iant]+","+str("%.4f"%(nu/1.0e9))+","+target+","+insource+","+str("%.2f"%beamf_width[iant,0])+","+str("%.2f"%planetf_width_std[iant,0])\
      +","+str("%.2f"%planetf_ell[iant,0])+","+str("%.1f"%planetf_sidelobe[iant,0])+","+str("%.3f"%eta_mb[iant,0])+","+str("%.3f"%err[iant,0])\
      +","+str("%.1f"%beamf_width[iant,1])+","+str("%.2f"%planetf_width_std[iant,1])+","+str("%.2f"%planetf_ell[iant,1])+","+str("%.1f"%planetf_sidelobe[iant,1])\
      +","+str("%.3f"%eta_mb[iant,1])+","+str("%.3f"%err[iant,1])+","+str("%.3f"%eta_ap_av[iant])+","+str("%.3f"%jyk_av[iant])+"\n"
      f.write(line)

#      line=dateobs[:-2]+','+vis+", "+ans[iant]+", "+str("%.4f"%(nu/1.0e9))+", "+target+", "+insource+", eta_mb,"+str("%.3f"%eta_mb[iant,0])+", "+str("%.3f"%err[iant,0])+", "+str("%.3f"%eta_mb[iant,1])+", "+str("%.3f"%err[iant,1]+", "+str("%.3f"%beamf_width[iant,0])+", "+str("%.3f"%beamf_width[iant,1])+"\n")
#    f.write(line)
  f.close()
  print 'data written to file ',filen
 else:
  print 'no data written to file'

 # -------------
