import casa
import string
import pdb
def planets_up(time='now', date='today', frame='utc', observatory='ALMA'):
    """
    Find what are the planets up in the sky at a give time
    Parameters:

    time: a string of time example '19h20m00' or 'now'
    if you use 'now' ...make sure your computer clock is accurate
    
    date: a string date in IAU order e.g 2010/07/19 (for 19th of July 2010)

    frame: Frame of the time you gave ..can be one of the following
    'LAST', 'LMST', 'GMST1', 'GAST', 'UT1', 'UT2', 'UTC', 'TAI', 'TDT',
       'TCG', 'TDB', 'TCB', 'IAT', 'GMST', 'TT', 'ET', 'UT'

    observatory: common name of observatory e.g 'ALMA', 'OSF' 'VLA'

    """
    me=casa.measures
    qa=casa.quanta
    srclist=['MERCURY', 'VENUS', 'MARS', 'JUPITER', 'SATURN', 'URANUS',
       'NEPTUNE', 'PLUTO', 'SUN', 'MOON']
    me.doframe(me.observatory(observatory))

    if(string.lower(time)=='now'):
        me.doframe(me.epoch('utc', 'now'))
    else:
        epochstr=qa.add(qa.floor(qa.add(me.measure(me.epoch('utc', date), frame)['m0'], qa.quantity(0.5,'d'))), qa.convert(time, 'd'))
        me.doframe(me.measure(me.epoch(frame, epochstr),'utc'))
    for src in srclist:
        azel=me.measure(me.direction(src), 'AZELGEO') 
        if(azel['m1']['value'] > 0):
            print src, 'az=', qa.angle(azel['m0'], prec=10), 'el=', qa.angle(azel['m1'], prec=10)
            hadec=me.measure(azel, 'HADEC')
            print src, 'ha=', qa.angle(hadec['m0'], prec=10), 'dec=', qa.angle(hadec['m1'], prec=10)

            
