#! /usr/bin/env python
"""

$Id: plot_llc.py,v 1.3 2014/12/31 19:24:28 dbarkats Exp $

"""
import os
import sys
import matplotlib
matplotlib.use('Agg')
from pylab import *
import tmcdbPlayer as tp
from datetime import datetime, timedelta
import tmUtils as tU
import matplotlib.dates
import glob
from optparse import OptionParser

class llcPlayer(object):
    
    """
    TODO: replace double plot with single larger plot with voltage followed by  counts.
    """

    def __init__(self):
        
        t = tp.tmcdbPlayer()
        self.antList = t.getAntennaNames()
        self.plotdir = '/users/dbarkats/llc_daily_examiner/plots/'
        self.htmldir = '/users/dbarkats/llc_daily_examiner/'


    def plot_llc_data(self,date=''):
        """
        Plot time series of LLC counts/voltage for given antenna and date.
        date is of format YYYY-MM-DD
        """
        figs = (20,60)
        figs = (18,200)
        
        if date != '':
            now=datetime.strptime(date,"%Y-%m-%d")
        else:
            now = datetime.now()
        self.yesterday = ('%s'%(now - timedelta(days=1)))[0:10]
        self.yesterday_start='%sT00:00:00'%self.yesterday
        self.yesterday_end='%sT23:59:59'%self.yesterday

        figure(1,figsize=figs);clf()
        nsubplots = size(self.antList)
        for i in range(nsubplots):
            ant = self.antList[i]
            print ant
            self.makeLLCVoltageSubPlot(2*i+1,2*nsubplots+1, ant)
            self.makeLLCCountSubPlot(2*i+2,2*nsubplots+1, ant)
        suptitle('%s - LLC Stretcher voltage [V] / Counts'%self.yesterday,y=0.91)
        plotname = '%s/%s_llc_VoltageCounts_vs_time_allAntennas.png'%(self.plotdir,self.yesterday)
        print plotname
        print "Saving figure: %s"%plotname
        savefig('%s'%plotname)

         # remove extra files.
        os.system('rm -Rf 2014*LLC*.txt')
        

        return
        
        figure(2,figsize=figs);clf()
        nsubplots = size(self.antList)
        for i in range(nsubplots):
            ant = self.antList[i]
            print ant
            self.makeLLCCountSubPlot(i+1,nsubplots, ant)

        suptitle('%s - LLC fringe counter [counts]'%self.yesterday,y=0.91)
        plotname = '%s/%s_llc_count_vs_time_allAntennas.png'%(self.plotdir,self.yesterday)
        print "Saving figure: %s"%plotname
        savefig('%s'%plotname) 

       
        return


    def makeLLCVoltageSubPlot(self,n,nsubplots,ant):
        """
        makes a single subplot of LLC voltage vs time
        hardwired to make it for 1 day, which is 00UT to 23:59:59UT
        """
        
        ax=subplot(nsubplots,1,n)
        
        try:
            tU.show_monitor_data_in_subplot(ax,ant,'LLC','VF_MON',self.yesterday_start,self.yesterday_end, ylabel=ant,yrange=[0, 5], showxticklabels=True, title='',tickmark=',')
            self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
        except:
            print ''
            if n != nsubplots:
                ax.set_xticklabels(())
            else:
                try:
                    ax.set_xticklabels(self.xticklabels, size=8)
                except:
                    print "Failed to produce a plot for %s" %self.yesterday_start
            ax.set_yticklabels(())

        if n != nsubplots:  ax.set_xticklabels(())
       
        
        for item in ([ax.title, ax.xaxis.label, ax.yaxis.label]+ax.get_xticklabels()+ax.get_yticklabels()):
            item.set_fontsize(8)

        subplots_adjust(hspace = 0.0001)
        ax.yaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        
        return


    def makeLLCCountSubPlot(self,n,nsubplots,ant):
        """
        makes a single subplot of LLC Counts vs time
        hardwired to make it for 1 day, which is 00UT to 23:59:59UT
        """
        
        ax=subplot(nsubplots,1,n)
        
        try:
            tU.show_monitor_data_in_subplot(ax,ant,'LLC','CNTR',self.yesterday_start,self.yesterday_end, ylabel=ant,yrange=[-200,200], showxticklabels=True, removediscontinuity=True, title='',tickmark=',')
            self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
        except:
            print ''
            if n != nsubplots:
                ax.set_xticklabels(())
            else:
                try:
                    ax.set_xticklabels(self.xticklabels, size=8)
                except:
                    print "Failed to produce a plot for %s" %self.yesterday_start
            ax.set_yticklabels(())

        if n != nsubplots:  ax.set_xticklabels(())
       
        
        for item in ([ax.title, ax.xaxis.label, ax.yaxis.label]+ax.get_xticklabels()+ax.get_yticklabels()):
            item.set_fontsize(8)

        subplots_adjust(hspace = 0.0001)
        ax.yaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        
        return

#    def makeWeatherSubPlot(self):
#        """       
#        makes a  plot of available weather data for that one day       
#        """      
#        # retrieve Weather station data from NANTEN2
#        tm.get_Nanten_weather_data

# get  ALMA weather data from weather 1 and 2.
# plot all on the same plot with many subplots.

#        ax=subplot(5,1,1)
#        
#        self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
#        ax.set_ylabel('TEMPERATURE [C]')
#        ax.set_xticklabels(())
#        ax=subplot(5,1,2)
#       
#        self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
#        ax.set_ylabel('HUMIDITY [%%]')
#        ax.set_xticklabels(())
#
#        ax=subplot(5,1,3)
#
#        self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
#        ax.set_ylabel('PRESSURE [hPa]')
#        ax.set_xticklabels(())
#
#        ax=subplot(5,1,4)
#
#        self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
#        ax.set_ylabel('WIND SPEED [m/s]')
#        ax.set_xticklabels(())
#
#        ax=subplot(5,1,5)
#
#        self.xticklabels = [item.get_text() for item in ax.xaxis.get_ticklabels()]
#        ax.set_ylabel('WIND DIRECTION ')
#        ax.set_xticklabels(())
#             
#        for item in ([ax.title, ax.xaxis.label, ax.yaxis.label]+ax.get_xticklabels()+ax.get_yticklabels()):
#            item.set_fontsize(8)
#    
#        subplots_adjust(hspace = 0.0001)
#        ax.yaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
#        return
    
    def create_date_loop(self):
        """
        return a list of dates  that automatically ends today
        """
        
        # create date objects
        begin_year = datetime(2014, 05, 01)
        now = ('%s'%(datetime.now()))[0:10]
        end_year=datetime.strptime(now,"%Y-%m-%d")
        one_day = timedelta(days=1)
        d=[]
        
        next_day = begin_year
        for day in range(0, 366):  # includes potential leap year
            if next_day > end_year:
                break
            a=next_day.isoformat()#.replace('-','')
            d.append(a[0:10])
            # increment date object by one day
            next_day += one_day
            
        return d

    def create_html(self):
        """
        given an existing set of png plots already there,
        create the proper html pages to display them
        
        """
        print '### Making HTML LLC daily examiner... ###'
        isodates=[]
        png_file = self.plotdir + '*_llc_VoltageCounts_vs_time_allAntennas.png'
    
        files = glob.glob(png_file)
        n = size(files)
        for i in range(n):
            isodates.append(files[i].split('_llc')[0][-10::])

        isodates=sort(isodates)
        
        # create index.html
        b=open(self.htmldir+'/index.html','w')
        
        b.write( '<html> \n')
        b.write( ' \n')
        b.write( '<frameset noresize="noresize" cols="200,*"> \n')
        b.write( '\n' )
        b.write( '<frame src="plot_index.html">\n')
        b.write( '<frame src="' + isodates[-1] + '.html" name="plotpage">\n')
        b.write( ' \n' )
        b.write( '</frameset> \n' )
        b.write( '\n' )
        b.write( '</html> \n')
        b.close()
        os.system( 'chmod 666 ' + self.htmldir + '/index.html')
        
        # create tab on left (plot_index.html)
        b= open(self.htmldir + '/plot_index.html','w')
        b.write('<html>\n')
        b.write('<body bgcolor="#d0d0d0"> \n')
        b.write('Go to: \n')
        b.write('<pre> \n')
        for i in range(n):
            b.write('<a href ="' + isodates[n-i-1] + '.html" target="plotpage">' + isodates[n-i-1] + '</a> \n')
        b.write('</pre> \n ')
        b.write('</body> \n')
        b.write('</html> \n')
        b.close()
        os.system( 'chmod 666 ' + self.htmldir + '/plot_index.html')

        # create individual plot .html pages
        for i in range(n):
            b= open(self.htmldir + isodates[i]+'.html','w')
            b.write('<html>\n ')
            b.write('<body>\n')
            b.write('<table>\n')
            b.write('<tr> \n')
            b.write('  <td colspan="2"> \n')
            b.write('    <center><img src="plots/title.png"></center> \n')
            b.write('    <center><h3><a href ="' + (self.shiftarray(isodates, 1))[i] + \
                    '.html">[prev]</a> [' + isodates[i] + '] <a href ="' + (self.shiftarray(isodates, -1))[i] + \
                    '.html">[next]</a></h3></center> \n')
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('<tr> \n')
            b.write('  <td> \n')
            localfile='%s_llc_VoltageCounts_vs_time_allAntennas.png'%isodates[i]
            b.write('  <a href="plots/' + localfile + '" target="_blank"><img src="plots/'+localfile + '" width="30%"> \n')
            #localfile2 = localfile.replace('voltage','count')
            #b.write('  <a href="plots/' + localfile2 + '" target="_blank"><img src="plots/'+localfile2 + '" width="30%"> \n')
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('</table> \n')
            b.write('</body> \n')
            b.write('</html> \n')
            b.close()
            os.system( 'chmod 666 ' + self.htmldir + isodates[i]+'.html')
            

        host='dbarkats@ssh.alma.cl'
        cmd='rsync -auv --progress /users/dbarkats/llc_daily_examiner %s:~/public_html/ ' %(host)
        print "### Rsyncing to dbarkats@ssh.alma.cl... ###"
        os.system(cmd)
        
        return
    
    
    def shiftarray(self,seq, n):
        ss=size(seq)
        n = n % ss
        new=[]
        for i in range(ss):
            new.append(seq[i-n])
            
        return new



if __name__ == '__main__':
    usage = '''
    
    '''

    #options ....
    parser = OptionParser(usage=usage)

    parser.add_option("-o", "--overwrite",
                      dest="overwrite",
                      action="store_true",
                      default=False,
                      help="Set this option to force overwriting already daily examiner plots. Default: False"
                      )

    
    (options, args) = parser.parse_args()
    lP = llcPlayer()

    # determine start/stop times 
    dateList = lP.create_date_loop()
    # dateList =['2014-05-02']
    
    if 0:
        lP.plot_llc_data()
    else:
        # loop through making daily plots
        for i in range(size(dateList)):
            png_file =  '%s/%s_*.png'%(lP.plotdir,dateList[i])
            foo = glob.glob(png_file)
            
            if (foo == [] or options.overwrite):
                print dateList[i]
                lP.plot_llc_data(dateList[i])
            else:
                print dateList[i], 'PNG already exists'
                
    
    # create the html files
    lP.create_html()

    

    
