#!/usr/local/bin/python
 
"""

Script to generate antenna integration tracker

#"""

# Execution starts here

import os, sys
import socket
import getpass
import pickle
import glob
username = getpass.getuser()

sys.path.append('/users/%s/AIV/science/scripts/R10.4_WORKING/'%username)
sys.path.append('/users/%s/AIV/science/analysis_scripts/'%username)


import analysisUtils as au


#sys.path.append('/users/%s/AIV/science/scripts/R10.4_WORKING/'%username)
#sys.path.append('/users/%s/AIV/science/analysis_scripts//'%username)
#sys.path.append('/opt/telcalsa/current/telcalsa/current/lib/python/site-packages/');
from FocusModelEditor import FocusModelEditor
from PointingModelEditor import PointingModelEditor
from PositionEditor import PositionEditor
from DelayEditor import DelayEditor 

from pylab import *
import tmUtils as tm
from datetime import datetime,timedelta
import warnings
from  getNominalPtBandOffsets import getNominal
import re
import tmcdbPlayer_tmp as tp
import time;

tt = tp.tmcdbPlayer()


###############################################
# local information handling
###############################################
def getFilename(which,t=""):
	date=t.split("T")[0];
	today=getCurrentTime().split("T")[0];
	filenames=dict();
	filenames["delay_history"]='/tmp/trend_analysis/position_models/All_delay_history_%s.xml'%(today);
	filenames["position_history"]='/tmp/trend_analysis/position_models/All_position_History_%s.xml'%(today);
	filenames["pointing_model_old"]='/tmp/trend_analysis/pointing_models/%s_pointing_allVersions.pickle.xml'%(date);
	filenames["pointing_model_new"]='/tmp/trend_analysis/pointing_models/%s_pointing_allVersions.pickle'%(date);
	filenames["position_model"]='/tmp/trend_analysis/position_models/All_position_%s.xml'%(date);
	filenames["delay_model"]='/tmp/trend_analysis/delay_models/All_delay_%s.xml'%(date);
	filenames["array_config"]='/tmp/trend_analysis/delay_models/All_array_config_%s.xml'%(today);
	filenames["pickle_file"]='/tmp/trend_analysis/pointing_models/%s_pointing_allVersions.pickle.xml'%(date);
	filenames["pointing_model"]='/tmp/trend_analysis/pointing_models/%s_All_pointing.xml'%(date);
	filenames["position_log"]="positionHistory.log";
	#print "|"+which+"|";
	filename=filenames[which];
	#print filename;
	return filename;

def getExportCommand(which,t=""):
	exportcommands=dict();
	date=t.split("T")[0];
	filename=getFilename(which,date);
	exportcommands["position_history"]='ExportPositionModel.py -y -g -f %s;mv exportPositionModel.log %s' %(filename,getFilename("position_log",date));
	exportcommands["delay_history"]='ExportDelayModel.py -y -f %s' %(filename);
	exportcommands["pointing_model_old"]='ExportPointingModel.py -t %s -f %s ' %(t,filename);
	exportcommands["pointing_model_new"]='ExportPointingModel.py -t %s -f %s ' %(t,filename);
	exportcommands["position_model"]= 'ExportPositionModel.py -t %s -f %s ' %(t,filename);
	exportcommands["delay_model"]= 'ExportDelayModel.py -t %s -f %s ' %(t,filename);
	exportcommands["array_config"]='ExportArrayConfiguration.py -y -f %s ' %(filename);
	exportcommands["pickle_file"]='ExportPointingModel.py -y -f %s' %(filename);
	exportcommands["pointing_model"]='ExportPointingModel.py -t %s -f %s'%(t,filename);
	print "Requested export command: "+exportcommands[which];
	return exportcommands[which];

def fileIsUpdated(which,t=""):
	date=t.split("T")[0];
	return os.path.isfile(getFilename(which,t));
	#filename=getFilename(which,date);
	#timeStampModified=os.path.getmtime(filename);
	#timeStampNow=time.time();
       	#return (timeStampNow-timeStampModified>24.0*60.0*60.0);


def updateFile(which,t=""):
	cmd=getExportCommand(which,t);
	tp.system_except(cmd);


def updateFileIfNeeded(which,t=""):
	date=t.split("T")[0];
	print "checking if exists:";
	print which;
	print t;
	print getFilename(which,t);
	if fileIsUpdated(which,t):
		print "Exists";
		return;
	print "Does not exist";
	updateFile(which,t);


###############################################
# Check move Data
###############################################
def checkMoveData(ant, cvsup=True, verbose=True, antennamovesfile=None):
	print "Check move Data";
	move = tt.readAntennaMovesTxt(cvsup, verbose, antennamovesfile)
	date_move=move[ant].start[-1]
	pad_last_move=move[ant].pad[-1]
	print "%s last move on %s to %s \n" % (ant,date_move,pad_last_move)
	return date_move,pad_last_move;


###############################################
# check the antenna Pointing Model history 
###############################################
def checkAntennaPointingModelHistory(ant):
	print "check the antenna Pointing Model history";
	PtVersions=tt.getAllPointingModel(versionsOnly=True)
	version = PtVersions[ant]
	last_update=sorted(version.keys(),reverse=True)
	k=last_update[0]
	print "Last Pointing Update \n"
	print '%03d  %s  %s  %s \n'%(k,version[k]['timestamp'][0:19],version[k]['author'],version[k]['comment'])
	return version;
#
###############################################
# check the antenna Position history 
###############################################
def checkPositionHistory(ant):
	print "check the antenna Position history";
	print "### Exporting all antenna/pad history from TMCDB..."
	updateFileIfNeeded("position_history");
	outfile=getFilename("position_history");
	p = PositionEditor()
	p.getTmcdbPositionModel(outfile) 
	p.getTmcdbPadPositionVersions() # READ POSITION VERSION HISTORY FOR EACH PAD
	p.getTmcdbAntennaPositionVersions() # READ POSITION VERSION HISTORY FOR EACH ANTENNA
	antenna_and_pad_versions = p._versions # ARRAY WITH POSITION VERSIONS FOR ANTENNAS AND PADS
	v_new = antenna_and_pad_versions[ant].keys()[-1] # INDEX LAST HISTORY ENTRY
	v=v_new
	timestamp_new_antpos = antenna_and_pad_versions[ant][v]['timestamp'][:16]
	author_new_antpos = antenna_and_pad_versions[ant][v]['author']
	comment_antpos = antenna_and_pad_versions[ant][v]['comment']
	print "Last Antenna Position Update \n"
	print "%s %s %s \n" % (timestamp_new_antpos,author_new_antpos,comment_antpos);
	os.system('chmod ugo+rwx -R %s' %  outfile)
	return antenna_and_pad_versions,timestamp_new_antpos,author_new_antpos,comment_antpos;

###############################################
# check the pad history 
###############################################
def extractParam(line):
        print line;
        param=line.split("returning")[1].split("as column")[0].strip(" ").strip("'");
        return param;

def getLastPadHistoryEntryFromLog(pad):
	print pad;
        pad_id="";read_pad_id=False;read_versions=False;last_version=-2;last_modtime=-1;last_author="";last_comment="";
	filename=getFilename("position_log");
	print filename;
	# Try to open exportPositionModel.log
	# If it fails, force downloading position history (even if file exists), to update the log file.
	try:
		f=open(filename);
	except:
		updateFile("position_history");
		f=open(filename);
        for line in [s.strip() for s in f]:
                if pad in line:
                        read_pad_id=True;
                if read_pad_id and "BASEELEM1_1_" in line:
                        print line;
                        pad_id=extractParam(line);
                        print pad_id;
                        read_pad_id=False;
                if not (pad_id=="") and "binding '%s'"%(pad_id) in line:
                        read_versions=True;
                        print line;
                if read_versions:
                        if not "binding" in line and not "returning" in line:
                                read_versions=False;
                        if "returning" in line and "VERSION" in line:
                                version=int(extractParam(line));
                        if "returning" in line and "MODTIME" in line:
                                modtime=int(extractParam(line));
                        if "returning" in line and "WHO" in line:
                                author=extractParam(line);
                        if "returning" in line and "CHANGEDESC" in line:
                                comment=extractParam(line);
                                if version>last_version or (version==last_version and modtime<last_modtime):
                                        last_version=version;
                                        last_modtime=modtime;
                                        last_author=author;
                                        last_comment=comment;
	return last_version,last_modtime,last_author,last_comment;



def checkPadHistory(ant,antenna_and_pad_versions):
	print "check the pad history";
	try:
		pad = tt.tmcdb.getCurrentAntennaPadInfo(ant).padName
	except:
		return "OSF","","","";
	p_new = antenna_and_pad_versions[pad].keys()[-1] # INDEX LAST HISTORY ENTRY
	p=p_new
	timestamp_new_pad = (antenna_and_pad_versions[pad][p]['timestamp'])[:16]
	author_new_pad = antenna_and_pad_versions[pad][p]['author']
	comment_pad = antenna_and_pad_versions[pad][p]['comment']
	print "Last Pad Position Update \n"
	print "%s %s %s %s \n" % (pad,timestamp_new_pad,author_new_pad,comment_pad)
	print ">>>";
	version_from_log,modtime_from_log,author_from_log,comment_from_log=getLastPadHistoryEntryFromLog(pad);
	modtime_from_log=str(datetime.fromtimestamp(int(modtime_from_log)));
	print modtime_from_log;
	modtime_from_log=modtime_from_log.replace(" ","T");
	print "<<<";
	print "from log: %s, %s, %s, %s"%(version_from_log,modtime_from_log,author_from_log,comment_from_log);
	#return pad,timestamp_new_pad,author_new_pad,comment_pad;
	return pad,modtime_from_log,author_new_pad,comment_pad;


###############################################
# check the delay history 
###############################################
#def checkDelayHistory(ant):
#	print "";
#	print "";
#	print "";
#	print "*****************************************";
#	print "check the antenna Delay history";
#	updateFileIfNeeded("delay_history");
#	outfile=getFilename("delay_history");
#	print outfile;
#	f=open(outfile,"r");
#	read=False;
#	timestamp="";author="";comment="";
#	for line in [s.strip() for s in f]:		
#		print line;
#		if ("AntennaDelayModelHistory" in line) and (ant in line):
#			read=True;
# 		if read and ("timestamp" in line):
#			timestamp=line.split('"')[1];
#			author=line.split('"')[3];
#			comment=line.split('"')[5];
#			read=False;
#	f.close();
#	return timestamp,author,comment;

def checkDelayHistory(ant):
#	print "";
#	print "";
#	print "";
#	print "*****************************************";
#	print "check the antenna Delay history";
	updateFileIfNeeded("delay_history");
	outfile=getFilename("delay_history");
#	print outfile;
	f=open(outfile,"r");
	read=False;
	timestamp="";author="";comment="";
	for line in [s.strip() for s in f]:		
#		print line;
		if ("<AntennaDelayModelHistory" in line) and (ant in line):
			read=True;
 		if read and ('timestamp="' in line):
			timestamp=line.split('timestamp="')[1].split('"')[0];
		if read and ('author="' in line):
			author=line.split('author="')[1].split('"')[0];
		if read and ('description="' in line):
			comment=line.split('description="')[1].split('"')[0];
		if "/>" in line:
			read=False;
	f.close();
	return timestamp,author,comment;



###############################################
# Get Pt Model
###############################################
def getPointingModelDate(ant,date):
	print "Get pointign model %s"%(date);
	updateFileIfNeeded("pointing_model",date);
	outfile=getFilename("pointing_model",date);
	pointing_editor=PointingModelEditor();
	pointing_editor.getTmcdbPointingModel(outfile);
	pointing_model=pointing_editor._models[ant];
	return pointing_model;


###############################################
# GET POSITION MODEL INFO
###############################################
def getPositionModelDate(ant,pad,date):
# Returns:
# Pad, Antenna, and antenna offset position (3-D Vector)   
	print "Get Position model in %s " % date;
	updateFileIfNeeded("position_model",date);
	outfile=getFilename("position_model",date);
        p = PositionEditor()
        p.getTmcdbPositionModel(outfile)
        pad_pos=p.getTmcdbPadPosition(pad)
        ant_pos=p.getTmcdbAntennaPosition(ant)
        ant_off=p.getTmcdbAntennaOffset(ant)
        return pad_pos,ant_pos,ant_off

###############################################
# GET PAD DELAY
###############################################
def getPadDelayModelDate(ant,date):
# Returns:
# Pad delay   
	#print "Get Delay model in %s " % date;
	updateFileIfNeeded("delay_model",date);
	outfile=getFilename("delay_model",date);
        d = DelayEditor()
        d.getTmcdbDelayModel(outfile)
        padDelay=d.getTmcdbPadToCorrDelay(ant)
	os.system('chmod ugo+rwx -R %s' %  outfile)
        return padDelay

###############################################
# GET IF DELAY
###############################################
def getIFDelayModelDate(ant,date,baseband, polarization, ifswitch):
# Returns:
# IF delay   
	print "Get Delay model in %s " % date;
	updateFileIfNeeded("delay_model",date);
	outfile=getFilename("delay_model",date);
        d = DelayEditor()
        d.getTmcdbDelayModel(outfile)
        ifDelay=d.getTmcdbIFDelay(ant,baseband, polarization, ifswitch)
	os.system('chmod ugo+rwx -R %s' %  outfile)
	print ifDelay;
        return ifDelay

###############################################
# GET FE DELAY
###############################################
def getFEDelayModelDate(ant,date,band,polarization,sideband):
# Returns:
# FE delay   
	print "Get Delay model in %s " % date;
	updateFileIfNeeded("delay_model",date);
	outfile=getFilename("delay_model",date);
        d = DelayEditor()
        d.getTmcdbDelayModel(outfile)
        feDelay=d.getTmcdbFEDelay(ant,band,polarization,sideband);
	os.system('chmod ugo+rwx -R %s' %  outfile)
        return feDelay

###############################################
# GET TILT PARAMETERS
###############################################
def getTiltParameters(ant,pad):
	updateFileIfNeeded("array_config");
	outfile=getFilename("array_config");
	#outfile='/users/%s/trend_analysis/delay_models/All_array_config.xml'%(username);
	f=open(outfile,"r");
	read_params=False;
	read_meta=False;
	right_antenna_and_pad=False;
	AN0="";AW0="";timestamp="0000-00-00";author="";comment="";
	for line in [s.strip() for s in f]:
		if ("AntennaOnPad" in line) and (ant in line) and (pad in line):
			right_antenna_and_pad=True
			#readParams=True;
			#readMeta=True;
		if right_antenna_and_pad and ("MetrologyCoefficientsHistory" in line):
			read_params=True;
			read_meta=True;
		if right_antenna_and_pad and read_meta and ('timestamp="' in line):
			timestamp=line.split('timestamp="')[1].split('"')[0];
		if right_antenna_and_pad and read_meta and ('author="' in line):
			author=line.split('author="')[1].split('"')[0];
		if right_antenna_and_pad and read_meta and ('description="' in line):
			comment=line.split('description="')[1].split('"')[0];
		if right_antenna_and_pad and read_meta and ("/>" in line):
			read_meta=False;
		if right_antenna_and_pad and read_params and ('AN0="' in line):
			AN0=line.split('AN0="')[1].split('"')[0];
		if right_antenna_and_pad and read_params and ('AW0="' in line):
			AW0=line.split('AW0="')[1].split('"')[0];
			read_params=False;
		if right_antenna_and_pad and ("</AntennaOnPad>" in line):
			right_antenna_and_pad=False;
	return AN0,AW0,timestamp,author,comment;

###############################################
# Make HTML table
###############################################
def correctTstamp(tstamp):
	if not ("20" in tstamp):
		return "0000-00-00T00:00:00";
	#print "__________________";
	#print tstamp;
	YMD=tstamp.split("T")[0].split("-");
	if "T" in tstamp:
               hms_string=tstamp.split("T")[1];
               hms=[segment.split(".")[0].split("-")[0] for segment in hms_string.split(":")];
	else:
               hms=[];
	while len(hms)<3:
               hms.append("00");
	correctedTstamp="{0}-{1}-{2}T{3}:{4}:{5}".format(YMD[0],YMD[1],YMD[2],hms[0],hms[1],hms[2]);
	#print correctedTstamp;
	#print "__________________";
	return correctedTstamp;

def makeHeaderLine1(header_groups):
	header_style="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;font-weight:bold;text-align:center;";
	header_line1="<tr>";
	for key in sorted(header_groups.keys()):
		tags=header_groups[key];
		header_line1+="<td colspan='{0}' style='{1}'>{2}</td>".format(str(len(tags)),header_style,key.split("_")[1]);
	header_line1+="</tr>";
	return header_line1;

def makeHeaderLine2(header_groups):
	header_style="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;font-weight:bold;text-align:center;";
	header_line2="<tr>";
	for key in sorted(header_groups.keys()):
		for tag in header_groups[key]:
			header_line2+="<td style='{0}'>{1}</th>".format(header_style,tag);
	header_line2+="</tr>";
	return header_line2;

def makeHtmlTableHeader():
	header_groups=dict();
	header_groups["01_Antenna/Pad"]=["Antenna","Pad","Date Moved"];
	header_groups["02_Tilt"]=["AN0","AW0"];
	header_groups["03_1st Pointing"]=["IA"];
	header_groups["04_Pad Delay"]=[""];
	header_groups["05_Position"]=["Pad","Antenna"];
	#header_groups["06_Anntenna Pos."]=["X","Y","Z"];
	#header_groups["07_FE Delays"]=[""];
	header_groups["08_IF Delays"]=[""];
	header_groups["09_Pointing"]=["AN","AW","IE","CA"];
	header_groups["10_Ready"]=[""];
	header_line1=makeHeaderLine1(header_groups);
	header_line2=makeHeaderLine2(header_groups);
	header_line=header_line1+header_line2;
	return header_line;

def makeHtmlTableRow(params):
	td_style="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;text-align:left";
	row_html="<tr>";
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["ANTENNA"]);
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["PAD"]);
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["DATE_MOVED"]);
	keys=["AN0","AW0"];
	keys=keys+["IA"];
	keys=keys+["PAD_DELAY"];	
	#keys=keys+["PAD_POS_X","PAD_POS_Y","PAD_POS_Z"];
	#keys=keys+["ANT_POS_X","ANT_POS_Y","ANT_POS_Z"];
	keys=keys+["PAD_POS","ANT_POS"];
	#keys=keys+["FE_DELAYS"];
	keys=keys+["IF_DELAYS"];	
	keys=keys+["AN","AW","IE","CA"];
	if params["PAD"]=="OSF":
		for key in keys:
			row_html=row_html+"<td></td>";
		row_html=row_html+"<td></td>";
		return row_html;
	for key in keys:
		value=params[key]["VALUE"];
		change_time=params[key]["CHANGE_TIME"];
		author=params[key]["AUTHOR"];
		comment=params[key]["COMMENT"];
		title=change_time+"\n"+author+"\n"+comment;
		valid=params[key]["VALID"];
		is_critical=params[key]["IS_CRITICAL"];
		if valid:
			color="green";
		else:
			color=["red" if is_critical else "yellow"][0];
		#color=["green" if valid else "red"][0];
		row_html+="<td title='{0}' style='{1}'><font color='{2}'>{3}</font></td>".format(title,td_style,color,value);
	row_html+="</tr>";
	return row_html;

def makeHtmlTableRowJustColors(params):
	td_style="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;text-align:left;";
	td_style_red="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;text-align:center;background-color:red;";
	td_style_green="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;text-align:center;background-color:green;";
	td_style_yellow="border:1px solid #888;padding:5px;font-family:arial;font-size:15px;text-align:center;background-color:yellow;";
	# ------------------------------------
	row_html="<tr>";
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["ANTENNA"]);
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["PAD"]);
	row_html+="<td style='{0}'>{1}</td>".format(td_style,params["DATE_MOVED"]);
	keys=["AN0","AW0"];
	keys=keys+["IA"];
	keys=keys+["PAD_DELAY"];
	keys=keys+["PAD_POS","ANT_POS"];
	#keys=keys+["PAD_POS_X","PAD_POS_Y","PAD_POS_Z"];
	#keys=keys+["ANT_POS_X","ANT_POS_Y","ANT_POS_Z"];
	#keys=keys+["FE_DELAYS"];
	keys=keys+["IF_DELAYS"];	
	keys=keys+["AN","AW","IE","CA"];
	if params["PAD"]=="OSF":
		for key in keys:
			row_html=row_html+"<td></td>";
		row_html=row_html+"<td></td>";
		return row_html;
	for key in keys:
		value=params[key]["VALUE"];
		change_time=params[key]["CHANGE_TIME"];
		author=params[key]["AUTHOR"];
		comment=params[key]["COMMENT"];
		title=change_time+"\n"+author+"\n"+comment;
		valid=params[key]["VALID"];
		is_critical=params[key]["IS_CRITICAL"];
		if valid:
			td_style_cell=td_style_green;
		else:
			td_style_cell=[td_style_red if is_critical else td_style_yellow][0];
		td_text=["O" if valid else "X"][0];
		row_html+="<td title='{0}' style='{1}'><font color='{2}'>{3}</font></td>".format(title,td_style_cell,"black",td_text);
	is_ready_antenna=params["IS_READY_ANT"]["VALUE"];
	if(is_ready_antenna):
		row_html=row_html+"<td style='{0}'>O</td>".format(td_style_green);
	else:
		row_html=row_html+"<td style='{0}'>X</td>".format(td_style_red);
	row_html+="</tr>";
	return row_html;

def isValidChangeTime(pad_change_time,param_change_time):
	print "-----------------------";
	print "Comparing times";
	print param_change_time;
	print pad_change_time;
	print "-----------------------";
	dateValid=(param_change_time>=pad_change_time);	
	return dateValid;

def paramChanged(old_param,new_param):
	if (old_param is None) and (new_param is None):
		return False;
	if (old_param is None) or (new_param is None):
		return True;
	paramValid=not (float(new_param)==float(old_param));
	return paramValid;

def getCurrentTime():
	today="%s"%datetime.now();
	todayT=today.split(" ")[0]+"T"+today.split()[1];
	return correctTstamp(todayT);



###############################################
# Get parameters to write each row
###############################################
def getParametersForEachRow(ant):
	# ------------------------------------------------------------------------------
	# READ INFO
	# ------------------------------------------------------------------------------
	# ---------- CURRENT DATE
	#today =('%s'%datetime.now())[0:10];
	today=getCurrentTime();
	# ---------- READ POINTING MODEL HISTORY
	pointing_history=checkAntennaPointingModelHistory(ant);# HISTORY
	pointing_last_index=sorted(pointing_history.keys(),reverse=True)[0];
	# ---------- READ DATE AND PAD OF LAST MOVE
	date_move,pad_last_move=checkMoveData(ant);
	
	#############
	# DEBUG CHANGE DAte Move to Date_MOVE-1
	# date = au.strDate2MJD(date)
	# then reconver tto string 
	##############

	# Time the antenna was moved to new pad:
	change_time=correctTstamp(date_move);
	# substract 1 day from change_time to avoid filename conflict with models updated less than 1 day after integration:
	change_time=("%s"%(au.datetime.datetime.strptime(change_time, "%Y-%m-%dT%H:%M:%S")-au.datetime.timedelta(days=1))).replace(" ","T");
	change_time_pad=correctTstamp(change_time);
	print "******************************";
	print "CHANGE TIME PAD WITH ONE DAY REMOVED";
	print change_time;
	print "******************************";	
	# ---------- READ POSITION MODEL INFO
	positions_history_all_pads_and_antennas,timestamp_new_antpos,author_new_antpos,comment_antpos=checkPositionHistory(ant);#HISTORY
	print "PAD LAST MOVE";
	print pad_last_move;
	if pad_last_move=="OSF":
		params=dict();
		params["ANTENNA"]=ant;
		params["PAD"]="OSF";
		params["DATE_MOVED"]="";
		return params;
	
	pad_positions_history=positions_history_all_pads_and_antennas[pad_last_move];
	pad_positions_last_index=sorted(pad_positions_history.keys(),reverse=True)[0];
	pad_pos_last_entry=pad_positions_history[pad_positions_last_index];
	ant_positions_history=positions_history_all_pads_and_antennas[ant];
	ant_positions_last_index=sorted(ant_positions_history.keys(),reverse=True)[0];
	ant_pos_last_entry=ant_positions_history[ant_positions_last_index];
	print "=========================================";
	print "PAD POSITION LAST INDEX: "+str(pad_positions_last_index);
	print "ANTENNA POSITION LAST INDEX: "+str(ant_positions_last_index);
	print ant_positions_history;
	print "=========================================";
	

	# ---------- READ PAD CHANGE INFO
	# Returns last entry in history of pad parameters (position and delay)
	pad,timestamp_new_pad,author_new_pad,comment_pad=checkPadHistory(ant,positions_history_all_pads_and_antennas);#HISTORY
	
	print "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";
	print timestamp_new_pad;
	print "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";	
	# ---------- READ OLD AND NEW POINTING PARAMETERS
	pt_model_old=getPointingModelDate(ant,change_time);
	pt_model_new=getPointingModelDate(ant,today);
	#pt_param=getPointingModelDateMove(ant,change_time);
	pt_param=dict();
	pt_param["old_ia"]=pt_model_old["IA"];
	pt_param["old_ie"]=pt_model_old["IE"];
	pt_param["old_ca"]=pt_model_old["CA"];
	pt_param["old_an"]=pt_model_old["AN"];
	pt_param["old_aw"]=pt_model_old["AW"];
	pt_param["new_ia"]=pt_model_new["IA"];
	pt_param["new_ie"]=pt_model_new["IE"];
	pt_param["new_ca"]=pt_model_new["CA"];
	pt_param["new_an"]=pt_model_new["AN"];
	pt_param["new_aw"]=pt_model_new["AW"];
	# ---------- READ ANTENNA AND PAD POSITIONS 
	pad_pos_old,ant_pos_old,ant_off_old=getPositionModelDate(ant,pad_last_move,change_time_pad);
	pad_pos_new,ant_pos_new,ant_off_new=getPositionModelDate(ant,pad_last_move,today);
	# ---------- READ PAD DELAY
	print "~~~~~~~~~~~~~~~~~~~~~~~~";
	print date_move;
	print today;
	#pad_delay_old=getPadDelayModelDate(ant,date_move);
	pad_delay_old=getPadDelayModelDate(ant,change_time);


	pad_delay_new=getPadDelayModelDate(ant,today);
	print pad_delay_old;
	print pad_delay_new;
	print "~~~~~~~~~~~~~~~~~~~~~~~~";
	# ---------- READ IF DELAY
	#baseband="BB_1";
	#polarization="X";
	#ifswitch="USB_HIGH";
	#ifDelay_old=getIFDelayModelDate(ant,change_time,baseband,polarization,ifswitch);
	#ifDelay_new=getIFDelayModelDate(ant,today,baseband,polarization,ifswitch);

	ifDelays_old=[];
	ifDelays_new=[];
	for baseband in ["BB_1","BB_2","BB_3","BB_4"]:
		for polarization in ["X","Y"]:
			for ifswitch in ["USB_HIGH","USB_LOW","LSB_HIGH","LSB_LOW"]:
				delay_old=getIFDelayModelDate(ant,change_time,baseband,polarization,ifswitch);
				ifDelays_old.append(delay_old);
				delay_new=getIFDelayModelDate(ant,today,baseband,polarization,ifswitch)
				ifDelays_new.append(delay_new);
	ifDelay_values_changed=False;
	for i in range(len(ifDelays_old)):
		ifDelay_values_changed=ifDelay_values_changed or (not (ifDelays_new[i]==ifDelays_old[i]));
	print "IF delay values changed:";
	print ifDelay_values_changed;
	
	# ---------- READ FE DELAY
	band="ALMA_RB_03";
	polarization="X";
	sideband="USB";
	feDelay_old=getFEDelayModelDate(ant,date_move,band,polarization,sideband);
	feDelay_new=getFEDelayModelDate(ant,today,band,polarization,sideband);
	# ---------- READ TILT PARAMETERS
	AN0,AW0,time_tilt,author_tilt,comment_tilt=getTiltParameters(ant,pad_last_move);
	# ---------- READ DELAY HISTORY
	time_delay,author_delay,comment_delay=checkDelayHistory(ant);#HISTORY
	
	# ------------------------------------------------------------------------------
	# DERIVE PARAMETERS FOR TABLE
	# ------------------------------------------------------------------------------
	#change_time_pad=correctTstamp(change_time);
	print "******************************";
	print "CHANGE TIME PAD";
	print change_time_pad;
	print "******************************";	
	change_time_pointing=correctTstamp(pointing_history[pointing_last_index]["timestamp"]);
	change_time_pad_pos=correctTstamp(pad_pos_last_entry["timestamp"]);
	change_time_ant_pos=correctTstamp(ant_pos_last_entry["timestamp"]);
	change_time_tilt=correctTstamp(time_tilt);
	change_time_delay=correctTstamp(time_delay);
	tilt_is_critical={"DA":False,"CM":False,"PM":False,"DV":True};
	# DETERMINE IF DIFFERENT STEPS ARE DONE
	# POSITIONS
	#is_valid_pad_pos_x=(isValidChangeTime(change_time_pad,change_time_pad_pos) and paramChanged(pad_pos_old[0],pad_pos_new[0]));
	#is_valid_pad_pos_y=(isValidChangeTime(change_time_pad,change_time_pad_pos) and paramChanged(pad_pos_old[1],pad_pos_new[1]));
	#is_valid_pad_pos_z=(isValidChangeTime(change_time_pad,change_time_pad_pos) and paramChanged(pad_pos_old[2],pad_pos_new[2]));
	#is_valid_ant_pos_x=(isValidChangeTime(change_time_pad,change_time_ant_pos) and paramChanged(ant_pos_old[0],ant_pos_new[0]));
	#is_valid_ant_pos_y=(isValidChangeTime(change_time_pad,change_time_ant_pos) and paramChanged(ant_pos_old[1],ant_pos_new[1]));	
	#is_valid_ant_pos_z=(isValidChangeTime(change_time_pad,change_time_ant_pos) and paramChanged(ant_pos_old[2],ant_pos_new[2]));

	is_valid_pad_pos_x=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(pad_pos_old[0],pad_pos_new[0]));
	is_valid_pad_pos_y=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(pad_pos_old[1],pad_pos_new[1]));
	is_valid_pad_pos_z=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(pad_pos_old[2],pad_pos_new[2]));
	is_valid_ant_pos_x=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(ant_pos_old[0],ant_pos_new[0]));
	is_valid_ant_pos_y=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(ant_pos_old[1],ant_pos_new[1]));	
	is_valid_ant_pos_z=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(ant_pos_old[2],ant_pos_new[2]));
	
	is_valid_pad_pos=(is_valid_pad_pos_x and is_valid_pad_pos_y and is_valid_pad_pos_z);
	is_valid_ant_pos=(is_valid_ant_pos_x and is_valid_ant_pos_y and is_valid_ant_pos_z);
	is_critical_pad_pos=not (is_valid_pad_pos or is_valid_ant_pos);
	is_critical_ant_pos=not (is_valid_pad_pos or is_valid_ant_pos);
	is_ready_position=(is_valid_pad_pos or not is_critical_pad_pos) and (is_valid_ant_pos or not is_critical_ant_pos);
	# TILT
	print "---------------------------";
	print "print checking tilt change time";
	print change_time_pad;
	print change_time_tilt;
	print "---------------------------";
	is_valid_tilt=isValidChangeTime(change_time_pad,change_time_tilt);
	is_ready_tilt=is_valid_tilt or (not tilt_is_critical[ant[0:2]]);
	# IA
	is_valid_ia=isValidChangeTime(change_time_pad,change_time_pointing) and paramChanged(pt_param["old_ia"],pt_param["new_ia"]);
	is_critical_ia=True;
	is_ready_ia=(is_valid_ia or not is_critical_ia);
	# POINTING
	is_valid_an=isValidChangeTime(change_time_pad,change_time_pointing) and paramChanged(pt_param["old_an"],pt_param["new_an"]);
	is_valid_aw=isValidChangeTime(change_time_pad,change_time_pointing) and paramChanged(pt_param["old_aw"],pt_param["new_aw"]);
	is_valid_ie=isValidChangeTime(change_time_pad,change_time_pointing) and paramChanged(pt_param["old_ie"],pt_param["new_ie"]);
	is_valid_ca=isValidChangeTime(change_time_pad,change_time_pointing) and paramChanged(pt_param["old_ca"],pt_param["new_ca"]);

	is_critical_pointing=not is_ready_ia or not (is_valid_an or is_valid_aw or is_valid_ie or is_valid_ca);

	is_ready_pointing=is_valid_ia and (is_valid_ie or is_valid_an or isvalid_aw or is_valid_ca);
	# PAD DELAY
	#is_valid_pad_delay=(isValidChangeTime(change_time_pad,change_time_delay) and paramChanged(pad_delay_old,pad_delay_new));
	is_valid_pad_delay=(isValidChangeTime(change_time_pad,timestamp_new_pad) and paramChanged(pad_delay_old,pad_delay_new));
	is_critical_pad_delay=True;
	is_ready_pad_delay=is_valid_pad_delay;
	# IF DELAY
	is_valid_if_delay=isValidChangeTime(change_time_pad,change_time_delay) and ifDelay_values_changed
	is_critical_if_delay=False;
	is_ready_if_delay=(is_valid_if_delay or not is_critical_if_delay);
	# DETERMINE IF ANTENNA IS READY FOR USE
	is_ready_antenna=is_ready_tilt and is_ready_pointing and is_ready_position and is_ready_pad_delay and is_ready_if_delay; 
	# ------------------------------------------------------------------------------
	print "CHANGE TIMES";
	print "PAD: "+change_time_pad;
	print "Pointing model: "+change_time_pointing;
	print "Pad positions: "+change_time_pad_pos;
	print "Ant positions: "+change_time_ant_pos;
	print "---- For IA ----";
	print "IA Old: "+str(pt_param["old_ia"]);
	print "IA New: "+str(pt_param["new_ia"]);
	print "---- For IF delay ----"
	print "Change time: "+change_time_delay;
	print "----- PAD DELAY ----";
	print "Change time: "+timestamp_new_pad;
	print "Old value: "+str(pad_delay_old);
	print "New vaue: "+str(pad_delay_new);
	
	#print "ifDelay old: "+str(ifDelay_old);
	#print "ifDelay new: "+str(ifDelay_new);
	#
	# ------------------------------------------------------------------------------
	# CREATE PARAMETER DICTIONARY FOR TABLE
	# ------------------------------------------------------------------------------
	params=dict();
	params["ANTENNA"]=ant;
	params["PAD"]=pad_last_move;
	params["DATE_MOVED"]=date_move;

	params["AN"]={"VALUE":pt_param["new_an"],"CHANGE_TIME":change_time_pointing,"VALID":is_valid_an,"COMMENT":pointing_history[pointing_last_index]["comment"],"AUTHOR":pointing_history[pointing_last_index]["author"],"IS_CRITICAL":is_critical_pointing};
	params["AW"]={"VALUE":pt_param["new_aw"],"CHANGE_TIME":change_time_pointing,"VALID":is_valid_aw,"COMMENT":pointing_history[pointing_last_index]["comment"],"AUTHOR":pointing_history[pointing_last_index]["author"],"IS_CRITICAL":is_critical_pointing};
	params["IA"]={"VALUE":pt_param["new_ia"],"CHANGE_TIME":change_time_pointing,"VALID":is_valid_ia,"COMMENT":pointing_history[pointing_last_index]["comment"],"AUTHOR":pointing_history[pointing_last_index]["author"],"IS_CRITICAL":True};
	params["IE"]={"VALUE":pt_param["new_ie"],"CHANGE_TIME":change_time_pointing,"VALID":is_valid_ie,"COMMENT":pointing_history[pointing_last_index]["comment"],"AUTHOR":pointing_history[pointing_last_index]["author"],"IS_CRITICAL":is_critical_pointing};
	params["CA"]={"VALUE":pt_param["new_ca"],"CHANGE_TIME":change_time_pointing,"VALID":is_valid_ca,"COMMENT":pointing_history[pointing_last_index]["comment"],"AUTHOR":pointing_history[pointing_last_index]["author"],"IS_CRITICAL":is_critical_pointing};

	params["AN0"]={"VALUE":AN0,"CHANGE_TIME":change_time_tilt,"VALID":is_valid_tilt,"COMMENT":comment_tilt,"AUTHOR":author_tilt,"IS_CRITICAL":tilt_is_critical[ant[0:2]]};
	params["AW0"]={"VALUE":AW0,"CHANGE_TIME":change_time_tilt,"VALID":is_valid_tilt,"COMMENT":comment_tilt,"AUTHOR":author_tilt,"IS_CRITICAL":tilt_is_critical[ant[0:2]]};

	params["FE_DELAYS"]={"VALUE":feDelay_new,"CHANGE_TIME":change_time_delay,"VALID":(isValidChangeTime(change_time_pad,change_time_delay) and paramChanged(feDelay_old,feDelay_new)),"COMMENT":comment_delay,"AUTHOR":author_delay,"IS_CRITICAL":False};
	params["IF_DELAYS"]={"VALUE":ifDelays_new[0],"CHANGE_TIME":change_time_delay,"VALID":is_valid_if_delay,"COMMENT":comment_delay,"AUTHOR":author_delay,"IS_CRITICAL":is_critical_if_delay};

#	params["PAD_DELAY"]={"VALUE":pad_delay_new,"CHANGE_TIME":change_time_delay,"VALID":is_valid_pad_delay,"COMMENT":comment_delay,"AUTHOR":author_delay,"IS_CRITICAL":is_critical_pad_delay};
	params["PAD_DELAY"]={"VALUE":pad_delay_new,"CHANGE_TIME":timestamp_new_pad,"VALID":is_valid_pad_delay,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_delay};

	params["PAD_POS_X"]={"VALUE":pad_pos_new[0],"CHANGE_TIME":change_time_pad_pos,"VALID":is_valid_pad_pos_x,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_pos};
	params["PAD_POS_Y"]={"VALUE":pad_pos_new[1],"CHANGE_TIME":change_time_pad_pos,"VALID":is_valid_pad_pos_y,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_pos};
	params["PAD_POS_Z"]={"VALUE":pad_pos_new[2],"CHANGE_TIME":change_time_pad_pos,"VALID":is_valid_pad_pos_z,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_pos};			     

	params["ANT_POS_X"]={"VALUE":ant_pos_new[0],"CHANGE_TIME":change_time_ant_pos,"VALID":is_valid_ant_pos_x,"COMMENT":ant_pos_last_entry["comment"],"AUTHOR":ant_pos_last_entry["author"],"IS_CRITICAL":is_critical_ant_pos};
	params["ANT_POS_Y"]={"VALUE":ant_pos_new[1],"CHANGE_TIME":change_time_ant_pos,"VALID":is_valid_ant_pos_y,"COMMENT":ant_pos_last_entry["comment"],"AUTHOR":ant_pos_last_entry["author"],"IS_CRITICAL":is_critical_ant_pos};
	params["ANT_POS_Z"]={"VALUE":ant_pos_new[2],"CHANGE_TIME":change_time_ant_pos,"VALID":is_valid_ant_pos_z,"COMMENT":ant_pos_last_entry["comment"],"AUTHOR":ant_pos_last_entry["author"],"IS_CRITICAL":is_critical_ant_pos};

#	params["PAD_POS"]={"VALUE":"X:%s, Y:%s, Z:%s"%(pad_pos_new[0],pad_pos_new[1],pad_pos_new[2]),"CHANGE_TIME":change_time_pad_pos,"VALID":is_valid_pad_pos,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_pos};
	params["PAD_POS"]={"VALUE":"X:%s, Y:%s, Z:%s"%(pad_pos_new[0],pad_pos_new[1],pad_pos_new[2]),"CHANGE_TIME":timestamp_new_pad,"VALID":is_valid_pad_pos,"COMMENT":pad_pos_last_entry["comment"],"AUTHOR":pad_pos_last_entry["author"],"IS_CRITICAL":is_critical_pad_pos};
	params["ANT_POS"]={"VALUE":"X:%s, Y:%s, Z:%s"%(ant_pos_new[0],ant_pos_new[1],ant_pos_new[2]),"CHANGE_TIME":change_time_ant_pos,"VALID":is_valid_ant_pos,"COMMENT":ant_pos_last_entry["comment"],"AUTHOR":ant_pos_last_entry["author"],"IS_CRITICAL":is_critical_ant_pos};
	
	params["IS_READY_ANT"]={"VALUE":is_ready_antenna};
	
	return params;


def makeHtmlTablesFullAndColor(antennas):
	table_full="<table style='border:1px solid #888;border-collapse:collapse;'>";	
	table_colors="<table style='border:1px solid #888;border-collapse:collapse;'>";
	table_full+=makeHtmlTableHeader();
	table_colors+=makeHtmlTableHeader();
	for ant in antennas:
		print "====================================================================";
		print "ANTENNA: "+ant;
		print "====================================================================";
		params=getParametersForEachRow(ant);
		table_full+=makeHtmlTableRow(params);
		table_colors+=makeHtmlTableRowJustColors(params);
	table_full+="</table>";
	table_colors+="</table>";
	return table_full,table_colors;

def assembleHtmlPage(table):
	today=getCurrentTime();
	html="<div style='font-family:Tahoma, Geneva, sans-serif;font-size:35px;font-weight:bold;color:#009;'>Antenna Integration Tracking</div>"
	html+="<br/>";
	html+=("Last updated: "+str(today));
	html+="<br/><br/>";
	html+=table;
	return html;


if __name__=="__main__":
	PM=["PM01","PM02","PM03","PM04"];
	CM=["CM01","CM02","CM03","CM04","CM05","CM06","CM07","CM08","CM09","CM10","CM11","CM12"];
	DV=["DV0"+str(i) for i in range(1,10)]+["DV"+str(i) for i in range(10,26)];
	DA=["DA"+str(i) for i in range(41,66)];
	all_antennas=CM+PM+DA+DV;
	print all_antennas;
	print [len(all_antennas),len(PM),len(CM),len(DA),len(DV)];

	#antennas=["DV11","DV12","DV22","DA62"];
	#antennas=["CM11","DA41","DA45","DA47","DA62","DV01","DV05","DV07","DV11","DV12","DV22"]
	table_full,table_colors=makeHtmlTablesFullAndColor(all_antennas);
	#table_full,table_colors=makeHtmlTablesFullAndColor(["CM01","PM01","DA41","DV01"]);
	#table_full,table_colors=makeHtmlTablesFullAndColor(["DA43","DA52"]);
	#table_full,table_colors=makeHtmlTablesFullAndColor(["DV21","DV24","CM02","CM03"]);
	#table_full,table_colors=makeHtmlTablesFullAndColor(["DA56","DA65","CM02"]);
	

	page_full=assembleHtmlPage(table_full);
	f=open("content_table_full_tmp.html","w");
       	f.write(page_full);
	f.close();

	page_colors=assembleHtmlPage(table_colors);
	f=open("content_table_color_tmp.html","w");
	f.write(page_colors);
	f.close();

	os.system('scp -r  content_table_color_tmp.html ahales@ssh.alma.cl:public_html/trend_analysis/')
	os.system('scp -r  content_table_full_tmp.html ahales@ssh.alma.cl:public_html/trend_analysis/')
	
	


