# This is a regression script that tests either au.plotbandpass or the casa task
# plotbandpass2 (from buildmytasks) on a variety of datasets and parameters.
# This script is meant to be run inside casa, on a luster node.
# Ideally, you should launch it from the figdir so that logs will go there.
# To execute:
# In the au directory, start a casa and run: os.system('buildmytasks')
# In figdir directory, start casa and execfile('/users/thunter/test/AIV/science/analysis_scripts/mytasks.py')
# then execfile('~/test/AIV/science/analysis_scripts/plotbandpass_regression.py')
# - Todd Hunter,  September 24, 2013
# As of November 16, 2015, it takes 1h45m to run. (Used to be 3h23m)
# As of May 10, 2017, it takes 3h17m to run.
import commands
import struct
import os

def pngWidthHeight(filename):
    """
    Reads the width and height of a png image (in pixels).
    -Todd Hunter
    """
    if (os.path.exists(filename) == False):
        print "Cannot find file = ", filename
        return
    f = open(filename, 'rb')
    data = f.read()
    if (data[:8] == '\211PNG\r\n\032\n'and (data[12:16] == 'IHDR')):
        w, h = struct.unpack('>LL', data[16:24])
        width = int(w)
        height = int(h)
    else:
        raise Exception('not a png image')
    f.close()
    return width, height

def autoIncrement(start=0,step=1):
    i = start
    while True:
        yield i
        i+= step

#####################################################
# find/replace change debug=False to regression=True 
#####################################################

# The following line crashes casa 5.0....  Is it needed?
#execfile('/users/thunter/test/AIV/science/analysis_scripts/mytasks.py')

startTime = time.time()
doTask = True # plotbandpass2, after execfile('mytasks.py')
doInternalTask = False  # plotbandpass (in current casa)
tstart = 0
# set to 0 to do all, -1 to do only the ones with overlays
overlaySpw = False
overlayBaseband = False
overlayTime = False
overlayAntennaTime = False
showfdm = False
test61 = False
test86 = False
test93 = False
mydir = '/lustre/naasc/sciops/comm/thunter/plotbandpass'
if (tstart >= 0):
    print "Starting with test %d" % (tstart)
t = autoIncrement(tstart)
if (doTask):
    myfunction = plotbandpass2
    figdir = mydir+'/regression/task/'
elif (doInternalTask):
    myfunction = plotbandpass
    figdir = mydir+'/regression/internalTask/'
else:
    myfunction = au.plotbandpass
    figdir = mydir+'/regression/au/'
if (not os.path.exists(figdir)):
    os.mkdir(figdir)
    
os.chdir(mydir+'/regression/TWHya')
if (tstart == 0):
  # 0
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='amp',overlay='antenna',spw='',field='0',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(0),debug=False)
  width,height = pngWidthHeight(figdir+'regression00.spw02.t02.png')
  if (width != 864):
      print "Width is %d pixels, not 864" % (width)
      print "You need to restart CASA and try again."
      sys.exit(0)
  
if (overlayBaseband or tstart==0 or tstart==1):
  # 1
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='amp',overlay='baseband',spw='',field='3c279',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(1),debug=False)
if (False or tstart==0 or tstart==2):
  # 2
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='ampdb',overlay='antenna',spw='',
             field='!Titan,!TW Hya,!J1147-382=QSO',
             interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(2),debug=False)
if (False or tstart==0 or tstart==3):
  # 3
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='both',phase=[-180,180],plotrange=[0,0,0,2],
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(3),debug=False)
if (False or tstart==0 or tstart==4):
  # 4
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='amp',overlay='antenna',spw='',field='',
             chanrange='1200~2000',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(4),
             debug=False, chanrangeSetXrange=True)

if (False or tstart==0 or tstart==5):
  # 5
  myfunction('bandpass.bcal',showatm=True,xaxis='chan',yaxis='amp',overlay='antenna',spw='',field='',
                  plotrange=[1200,2000,0,0],interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(5),debug=False)
  # 6
  myfunction('bandpass.bcal',showatm=True,xaxis='chan',yaxis='amp',spw='',field='',plotrange=[1200,3840,0,0],
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(6),debug=False)
if (False or tstart==0 or tstart==7):
  # 7
  myfunction('bandpass.bcal',showtsky=False,xaxis='chan',yaxis='amp',overlay='antenna',spw='',field='',showatm=True,
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(7),debug=False)
if (tstart==0 or tstart==8):
  # 8
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='phase',overlay='antenna',spw='',field='',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(8),debug=False)
  # 9
  myfunction('bandpass.bcal',showtsky=False,xaxis='freq',yaxis='phase',overlay='antenna',spw='',field='',
             chanrange='1200~1800',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(9),
             debug=False, chanrangeSetXrange=True)
  #
if (False or tstart==0 or tstart==10):
  # 10
  myfunction('bandpass.bcal',overlay='antenna',yaxis='amp',field='0~1,4',xaxis='freq',showtsky=True,
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(10),debug=False)
if (False or tstart==0 or tstart==11):
  # 11
  myfunction('bandpass_b_skipspw19high.bcal',yaxis='amp',field='0',xaxis='freq',
                  caltable2='bandpass_bpoly_skipspw19high.bcal',showpoints=True,spw='0,1',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(11),debug=False)

if (False or tstart==0 or tstart==12):
  # 12
  myfunction('bandpass_b_skipspw19high.bcal',yaxis='phase',field='0',xaxis='freq',
                  caltable2='bandpass_bpoly_skipspw19high.bcal',showpoints=True,spw=0,
                  caltable3='bandpass_bpoly_skipspw19high.bcal',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(12),debug=False)

if (False or tstart==0 or tstart==13):
  # 13
  myfunction('bandpass_b_skipspw19high.bcal',yaxis='both',field='0',xaxis='freq',
                  caltable2='bandpass_bpoly_skipspw19high.bcal',showpoints=True,spw=0,
                  caltable3='bandpass_bpoly_skipspw19high.bcal',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(13),debug=False)

if (False or tstart==0 or tstart==14):
  # 14
  os.chdir(mydir+'/regression/TWHya')
  myfunction('X3c1.tsys.fdm',overlay='antenna',yaxis='amp',field='1',xaxis='chan',
                  showtsky=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(14)) 
if (False or tstart==0 or tstart==15):
  # 15
  myfunction('X3c1.tsys.fdm',overlay='antenna',yaxis='amp',field='1',xaxis='chan',
                  poln='y',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(15),debug=False)
if (False or tstart==0 or tstart==16):
  # 16
  myfunction('X3c1.tsys',overlay='antenna',yaxis='amp',field='0~1,4',xaxis='chan',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(16),debug=False)
if (False or tstart==0 or tstart==17 or overlayTime):
  # 17
  myfunction('X3c1.tsys',overlay='time',yaxis='amp',field='2',xaxis='chan',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(17),debug=False)
  #
if (False or tstart==0 or tstart==18 or showfdm):
  # 18
  myfunction('X3c1.tsys',overlay='',yaxis='amp',field='',xaxis='freq',showfdm=True,interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(18),debug=False)
if (False or tstart==0 or tstart==19 or overlayTime or showfdm):
  # 19
  myfunction('X3c1.tsys',overlay='time',yaxis='amp',field='',xaxis='freq',showfdm=True,interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(19),debug=False)
  
if (False or tstart==0 or tstart==20):
  # 20
  myfunction('X3c1.tsys',overlay='',yaxis='amp',field='2',xaxis='freq',chanrange='45~65',
             interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(20),debug=False,
             chanrangeSetXrange=True)

  # 21 
  myfunction('bandpass_bpoly_skipspw19high.bcal',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(21),debug=False,xaxis='freq')

if (False or tstart==0 or tstart==22):
  # 22
  os.chdir(mydir+'/regression/TWHya')
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(22),debug=False) 
  
  # 23
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',yaxis='ampdb',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(23),debug=False) 
  
  # 24
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',yaxis='phase', xaxis='freq',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(24),debug=False)
  
  # 25
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',chanrange='1000~3000',
             interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(25),debug=False,
             chanrangeSetXrange=True)
  
  # 26
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',showtsky=True,
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(26),debug=False)
  
  # 27
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',poln='x',
                  showflagged=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(27),debug=False) 
  
  # 28
  myfunction('bandpass.bcal',caltable2='bandpass.bcal_smooth',xaxis='freq',poln='x',
                  showflagged=True, showtsky=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(28),debug=False)
  
if (tstart==0 or tstart==29):
  # 29
  os.chdir(mydir+'/regression/TWHya')
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',    yaxis='amp',xaxis='freq',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(29),debug=False) 
  
if (tstart==0 or tstart==30):
  # 30
  os.chdir(mydir+'/regression/TWHya')
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',    yaxis='amp',xaxis='freq',
                  showtsky=True, interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(30),debug=False) 
  
  # 31
  myfunction(caltable='X3c1.tsys',    caltable2='X3c1.tsys.fdm',yaxis='amp',xaxis='freq',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(31),debug=False)
  
if (tstart==0 or tstart==32):
  # 32  
  myfunction(caltable='X3c1.tsys',    caltable2='X3c1.tsys.fdm',yaxis='amp',xaxis='freq',
                  showtsky=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(32),debug=False) 
  tstart = 0

if (tstart==0 or tstart==33):
  # 33  
  os.chdir(mydir+'/regression/TWHya')
  myfunction(caltable='X3c1.tsys',caltable2='X3c1.tsys.fdm',    yaxis='both',xaxis='freq',
             chanrange='10~118',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(33),
             debug=False, chanrangeSetXrange=True)

if (tstart==0 or tstart==34):
  # 34
  os.chdir(mydir+'/regression/TWHya')
  tstart = 0
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',    yaxis='amp',xaxis='freq',
             chanrange='1000~3000',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(34),
             debug=False, chanrangeSetXrange=True)
  
  # 35
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',    yaxis='amp',xaxis='freq',
             chanrange='1000~3000',showtsky=True,interactive=False,buildpdf=True,
             figfile=figdir+'regression%02d'%(35),debug=False, chanrangeSetXrange=True) 
  
if (tstart==0 or tstart==36):
  # 36
  myfunction(caltable='X3c1.tsys',    caltable2='X3c1.tsys.fdm',yaxis='both',xaxis='freq',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(36),debug=False) 
  
if (tstart==0 or tstart==37):
  # 37
  myfunction(caltable='X3c1.tsys',    caltable2='X3c1.tsys.fdm',yaxis='both',xaxis='freq',
                  showtsky=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(37),debug=False)
  
  # 38
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',    yaxis='both',xaxis='freq',
                  zoom='intersect',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(38),debug=False)
  
  # 39
  myfunction(caltable='X3c1.tsys',    caltable2='X3c1.tsys.fdm',yaxis='both',xaxis='freq',
                  zoom='intersect',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(39),debug=False)
  
  # 40
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',yaxis='amp',xaxis='freq',poln='XX',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(40),debug=False) 
  
  # 41
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',yaxis='amp',field='1',xaxis='freq',
                  poln='YY',zoom='intersect',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(41),debug=False)
  
  # 42
  myfunction(caltable='X3c1.tsys.fdm',caltable2='X3c1.tsys',yaxis='amp',field='1',xaxis='freq',
                  poln='YY',zoom='intersect',showatm=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(42),debug=False)
  
if (tstart==0 or tstart==43):
  # 43 
  tstart = 0
  myfunction(caltable='X3c1.tsys.fdm',yaxis='amp',field='1',xaxis='chan',poln='YY',zoom='intersect',
                  showatm=True,showimage=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(43),debug=False)
if (tstart==0 or tstart==44):
  #
  #tests for multi-field bandpass solution overlay: in SciVer/TWHya
  # 44
  myfunction('band7multi_a6p7_titan.bcal',caltable2='band7multi_b.bcal',xaxis='freq',
             yaxis='amp',chanrange='',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(44),
             debug=False)
  
  # 45
  myfunction('band7multi_a6p7_titan.bcal',caltable2='band7multi_b.bcal',xaxis='freq',
             yaxis='amp',chanrange='',showflagged=True,interactive=False,buildpdf=True,
             figfile=figdir+'regression%02d'%(45),debug=False)
  
  # 46
  myfunction(caltable='band7multi_b.bcal',caltable3='band7multi_bpoly_a6p7_titan.bcal',
                  caltable2='band7multi_bpoly.bcal',xaxis='freq',yaxis='both',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(46),debug=False)

  #
  #tests for multi-field Tsys solution overlay (new-style cal tables):  
if (tstart==0 or tstart==47 or overlayTime):
  os.chdir(mydir+'/regression/2011.0.00150.S')
  # 47
  myfunction('uid___A002_X54d35d_X761.ms.tsys', overlay='time', xaxis='freq',
                    yaxis='amp', subplot=22,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(47),debug=False)
  
if (tstart==0 or tstart==48):
  os.chdir(mydir+'/regression/2011.0.00150.S')
  # 48
  myfunction('uid___A002_X54d35d_X761.ms.tsys', overlay='antenna', xaxis='freq',
                  yaxis='amp', subplot=22,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(48),debug=False)
  
if (tstart==0 or tstart==49 or overlayTime):
  os.chdir(mydir+'/regression/2011.0.00150.S')
  # 49
  myfunction('uid___A002_X54d35d_X761.ms.tsys', overlay='time', timeranges='0,1',
             interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(49),debug=False)
  
if (tstart==0 or tstart==50 or overlayTime):
  # 50   # plot did not complete in au, but did in task on Apr02, sever error on Apr05
  myfunction('uid___A002_X54d35d_X761.ms.tsys', overlay='time', scans='3,6', interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(50),antenna='0,1',debug=False)
  tstart = 0
  #
  
# This directory got deleted. 
#if (tstart==0 or tstart==51) and False:
  # ALMA data, casa 3.4 tables: (but scan numbers are not correct: -1)
#  os.chdir('/lustre/cv/users/thunter/alma/SV_data_TWHya/3.4guide')
  # 51
#  myfunction(caltable='bandpass_all_bpoly_dv04',xaxis='freq',yaxis='both',interactive=False,
#                  buildpdf=True,figfile=figdir+'regression%02d'%(51),debug=False) 
  #

# This got deleted. 
#if (tstart==0 or tstart==52) and False:
  # ALMA data, FDM+TDM in same caltable
#  os.chdir('/lustre/cv/users/thunter/alma/SV_data_TWHya/3.4guide')
  # 52   # gives invalid value in double_scalars
#  myfunction(caltable='bandpass_all_b_odd_dv04',caltable2='bandpass_all_bpoly_dv04',xaxis='freq',
#             yaxis='both',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(52),debug=False)
  
# This got deleted. 
#  os.chdir('/lustre/cv/users/thunter/alma/SV_data_TWHya/3.4guide')
  # 53
#  myfunction(caltable='bandpass_all_b_odd_dv04',xaxis='freq',yaxis='phase',chanrange='',showatm=True,
#                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(53),debug=False)
  
# This got deleted. 
#if (tstart==0 or tstart==54 or overlayTime or tstart==52 or overlayAntennaTime) and False:
  # 54
#  os.chdir('/lustre/cv/users/thunter/alma/SV_data_TWHya/3.4guide')
#  myfunction(caltable='bandpass_all_b_odd_dv04',xaxis='freq',yaxis='amp',chanrange='',showatm=True,
#                  overlay='antenna,time',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(54),debug=False)
 
# This got deleted. 
#if (tstart==0 or tstart==55 or overlayTime or overlayAntennaTime) and False:
#  os.chdir('/lustre/naasc/users/thunter/alma/SV_data_TWHya/3.4guide')
  # 55
#  myfunction(caltable='bandpass_all_b_odd_dv04',xaxis='freq',yaxis='phase',chanrange='',showatm=True,
#                  overlay='antenna,time',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(55),debug=False)
  #
if (tstart==0 or tstart==56):
  # ALMA 1-pol data:  (scan numbers not correct)
  os.chdir(mydir+'/regression/singlePol')
  # 56
  myfunction('uid___A002_X494155_X23a.ms.split.bandpass',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(56),debug=False)
  #
  # ALMA 4-pol data:  (scan numbers not correct)  This dataset got deleted.
#  os.chdir('/lustre/naasc/users/thunter/cycle1/4pol')
  # 57
#  myfunction('uid___A002_X60e47a_X8a6.ms.spw0_2_4_6.solintinf_7_8125MHz.bcal.s8_2.tbl',xaxis='freq',
#                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(57),debug=False)
  
  # 58
#  myfunction('uid___A002_X60e47a_X8a6.ms.spw0_2_4_6.solintinf_7_8125MHz.bcal.s8_2.tbl',xaxis='freq',
#                  basebands='1,4',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(58),debug=False)
  #
if (tstart==0 or tstart==59 or showfdm):
  # ALMA multi-regions per baseband  OUS = uid://A002/X50bb1c/X6  scan numbers=2,7,14,23
  os.chdir(mydir+'/diah')
  
  # 59
  myfunction('uid___A002_X50bcb3_X3b.ms.tsys',xaxis='freq',showfdm=True,showBasebandNumber=False,
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(59),debug=False)
  
  # 60
  myfunction('uid___A002_X50bcb3_X3b.ms.tsys',xaxis='freq',showfdm=True,showBasebandNumber=True,
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(60),debug=False)
  #
  # EVLA dual-pol data:
if (overlaySpw or test61 or tstart==61 or tstart==0):
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  # new style table from 4.1.0
  # 61
  myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='spw',xaxis='freq',interactive=False,
             buildpdf=True,figfile=figdir+'regression%02d'%(61),debug=False,showBasebandNumber=True)
  
if (overlayBaseband or tstart==0 or tstart==62):
  # 62
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='baseband',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(62),debug=False)
  tstart=0
  # 63
  myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='baseband',xaxis='freq',spw='0,1,14,15',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(63),debug=False)

if (overlaySpw or tstart==0):
  # 64
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='spw',xaxis='freq',basebands=12,
                  showBasebandNumber=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(64),debug=False)
  # old style 3.3 tables
  
if (overlaySpw or tstart==0):
  # 65  different antennas have solutions at different times
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  myfunction('bandpasspcal.bcal',poln='',yaxis='amp',overlay='spw',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(65),debug=False)

if (overlayBaseband or tstart==0 or tstart==66):
  # 66
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  myfunction('bandpasspcal.bcal',poln='',yaxis='amp',overlay='baseband',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(66),debug=False)

if (False or tstart==0 or tstart==67 or overlayTime):
  # 67 
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  myfunction('bandpasspcal.bcal',poln='',yaxis='amp',overlay='time',xaxis='freq',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(67),debug=False)
  
if (False or tstart==0 or tstart==68):
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  # 68
  myfunction('bandpasspcal.bcal',poln='',yaxis='both',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(68),debug=False)
  
  # 69
  myfunction('bandpasspcal.bcal',poln='',yaxis='both',showatm=True,interactive=False,buildpdf=True,
                  figfile=figdir+'regression%02d'%(69),debug=False)
  
  # 70
  myfunction('bandpasspcal.bcal',poln='',yaxis='phase',overlay='antenna',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(70),debug=False)
  
  # 71
  myfunction('bandpasspcal.bcal',poln='',yaxis='amp',overlay='antenna',chanrange='0~30',
             xaxis='freq',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(71),
             debug=False, chanrangeSetXrange=True)
  
if (tstart==72 or tstart==0):
  # 72
  os.chdir('/lustre/cv/users/thunter/evla/10C-186/K_BnA_cont')
  tstart = 0
  myfunction('bandpasspcal.bcal',poln='',yaxis='both',showatm=True,interactive=False,buildpdf=True,
                  figfile=figdir+'regression%02d'%(72),debug=False)
  
  # 73
  myfunction('bandpasspcal.bcal',poln='LL',yaxis='both',interactive=False,buildpdf=True,
                  figfile=figdir+'regression%02d'%(73),debug=False)
  #
  #EVLA single-pol data:
if (overlaySpw or tstart==0 or tstart==74):
  os.chdir('/lustre/cv/users/thunter/evla/AB1346/g19.36')
  # new style table from 4.1.0
  # old style 3.3 tables
  # 74
  myfunction('bandpass.bcal',yaxis='amp',xaxis='freq',overlay='spw',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(74),debug=False)

if (overlayBaseband or tstart==0 or tstart==75):
  os.chdir('/lustre/cv/users/thunter/evla/AB1346/g19.36')
  # 75
  myfunction('bandpass.bcal',yaxis='phase',xaxis='freq',overlay='baseband',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(75),debug=False)

if (False or tstart==0):
  os.chdir('/lustre/cv/users/thunter/evla/AB1346/g19.36')
  # 76
  myfunction('bandpass.bcal',caltable2='bandpass_bpoly.bcal',yaxis='both',xaxis='freq',
                  interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(76),debug=False)

  # 77
  myfunction('bandpass.bcal',caltable2='bandpass_bpoly.bcal',yaxis='both',xaxis='freq',
                  showatm=True,interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(77),debug=False)
  #
  # SMA data
  os.chdir(mydir+'/regression/sma/ngc6334_SM2_filler')
  # 78
  myfunction('sm2_rx0.usb.if2.tsys.ms.bandpass.bcal',interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(78),debug=False)

if (overlayBaseband or tstart==0 or tstart==79):
  os.chdir(mydir+'/regression/sma/ngc6334_SM2_filler')
  # 79
  myfunction('sm2_rx0.usb.if2.tsys.ms.bandpass.bcal',overlay='baseband',
                  xaxis='freq',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(79),debug=False)

  # 80
  myfunction('sm2_rx0.usb.if2.tsys.ms.bandpass.bcal',overlay='baseband',figfileSequential=True,
                  xaxis='freq',interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(80),debug=False)

if (False or tstart==0 or tstart==81 or showfdm):
  # ALMA multi-regions per baseband  OUS = uid://A002/X50bb1c/X6  scan numbers=2,7,14,23
  # single antenna with subplot=11
  os.chdir(mydir+'/diah')
  # 81
  myfunction('uid___A002_X50bcb3_X3b.ms.tsys',xaxis='freq',showfdm=True,showBasebandNumber=False,
             interactive=False,buildpdf=False,figfile=figdir+'regression%02d'%(81),
             subplot=11,antenna=0,debug=False)
  au.buildPdfFromPngs(pnglist=figdir+'regression%02d*.png'%(81), pdfname=figdir+'regression%02d.pdf'%(81))

  # similar to test 50
  os.chdir(mydir+'/regression/2011.0.00150.S')
  # 82
  myfunction('uid___A002_X54d35d_X761.ms.tsys', overlay='antenna', scans='3,6', interactive=False,
                  buildpdf=True,figfile=figdir+'regression%02d'%(82),antenna='0,1',debug=False)
  #

if (False or tstart==0) and (doTask==False and doInternalTask==False):
#if (False or tstart==0 or tstart==83):
    os.chdir(mydir+'/regression')
    # 83
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='5~122',showBasebandNumber=True,
                    groupByBaseband=True,xaxis='freq',interactive=False,buildpdf=True,
                    figfile=figdir+'regression%02d'%(83),debug=False, chanrangeSetXrange=True)
    
if (False or tstart==0 or tstart==84):
    os.chdir(mydir+'/regression')
    # 84
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',showBasebandNumber=True,
#               chanrange='5~122',
               chanrange='92.1875%',
               xaxis='freq',interactive=False,buildpdf=True,
               figfile=figdir+'regression%02d'%(84), antenna=0,debug=False, chanrangeSetXrange=True)

if (False or tstart==0 or tstart==85):
    os.chdir(mydir+'/regression')
    # 85
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='5~122',showBasebandNumber=True,
               xaxis='freq',interactive=False,buildpdf=True, chanrangeSetXrange=True,
               figfile=figdir+'regression%02d'%(85),basebands='1,3',antenna='0,1',debug=False)
if (overlaySpw or test86 or tstart==0 or tstart==86):
    os.chdir(mydir+'/regression')
    print "running test86"
    # 86 
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='',showBasebandNumber=True,
                    xaxis='freq',interactive=False,buildpdf=True,
                    figfile=figdir+'regression%02d'%(86),overlay='spw',antenna='0,1',debug=False)
if (overlayBaseband or tstart==0 or tstart==87):
    os.chdir(mydir+'/regression')
    # 87 
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='',showBasebandNumber=True,
                    xaxis='freq',interactive=False,buildpdf=True,
                    figfile=figdir+'regression%02d'%(87),overlay='baseband',antenna='0,1',debug=False)

if (False or tstart==0 or tstart==88 or overlayTime or showfdm):
    os.chdir(mydir+'/regression')
    # 88 
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='5~122',showBasebandNumber=True,
               xaxis='freq',interactive=False,buildpdf=True, chanrangeSetXrange=True,
               figfile=figdir+'regression%02d'%(88),overlay='time',showatm=True,showfdm=True,debug=False)

if (False or tstart==0 or tstart==89 or overlayTime or showfdm):
    os.chdir(mydir+'/regression')
    # 89
    myfunction('uid___A002_X61d35c_X4f.ms.tsys',chanrange='5~122',showBasebandNumber=True,
               xaxis='freq',interactive=False,buildpdf=True,showfdm=True, chanrangeSetXrange=True,
               figfile=figdir+'regression%02d'%(89),overlay='antenna',antenna='!DV17',showatm=True,debug=False)

    os.chdir(mydir+'/regression/CAS-6147')
    # 90   CAS-6147: cannot handle this table
    myfunction('uid___A002_X71a45c_X1d24.ms.split.bandpass',interactive=False,buildpdf=True,
               figfile=figdir+'regression%02d'%(90),debug=False)

if (tstart==0 or tstart==91 or overlayTime or overlayAntennaTime):
    os.chdir(mydir+'/regression/CAS-6111')
    # 91   CAS-6111: CM01 completely flagged causes no plot produced
    myfunction('uid___A002_X5e971a_X124.ms.hifa_tsyscal.s5_2.tsyscal.tbl',overlay='antenna,time',interactive=False,
               buildpdf=True, figfile=figdir+'regression%02d'%(91),debug=False)

if (overlayBaseband or tstart==0 or tstart==92 or overlayTime):
    os.chdir(mydir+'/regression/tsys_time_labels')
    # 92  CAS-6356: error in the legend colors
    myfunction('uid___A002_X7b13df_X68f.ms.tsys',overlay='time',interactive=False,buildpdf=True,
               figfile=figdir+'regression%02d'%(92),debug=False)
    
if (test93 or overlayBaseband or tstart==0 or tstart==93):
    os.chdir(mydir+'/regression/evla')
    # 93 Emmanuel's user EVLA dataset
    myfunction('finalBPcal.b',  # 13A-400_20140209_1392020120173
               yaxis='amp',xaxis='chan',subplot=11,spw='',debug=False,overlay='baseband',buildpdf=True,
               interactive=False,figfile=figdir+'regression93')

if (overlayBaseband or tstart==0 or tstart==94):
    os.chdir(mydir+'/regression/evla')
    # 94 Emmanuel's user EVLA dataset
    myfunction('finalBPcal.b',  # 13A-400_20140209_1392020120173
               yaxis='amp',xaxis='freq',subplot=11,spw='',debug=False,overlay='baseband',buildpdf=True,
               interactive=False,figfile=figdir+'regression94')   

if (overlayBaseband or tstart==0 or tstart==95):
    os.chdir(mydir+'/regression/band9atmLines')
    # 95  Band 9 atm lines
    myfunction('uid___A002_X41f8d2_X50f.ms.tsys',showatm=True,xaxis='freq',showimage=True,antenna=0,
               yaxis='amp',debug=False,buildpdf=True,interactive=False,figfile=figdir+'regression95')


if (overlayBaseband or tstart==0 or tstart==96):
    os.chdir(mydir+'/regression/infiniteLoop')
    # 96  pipeline-discovered problem, infinite loop
    myfunction(subplot=11, antenna='DA44', overlay='baseband',
               caltable='uid___A002_X652932_X20fb.ms.hif_bandpass.s10_2.spw17_19_21_23_33_35_37_39.channel.solintinf.bcal.tbl',
               figfile=figdir+'regression96', showatm=False, yaxis='amp', vis='uid___A002_X652932_X20fb.ms',
               xaxis='freq', interactive=False, buildpdf=True) 
    # 97  pipeline-discovered problem, infinite loop
    myfunction('uid___A002_X67f61e_X687.ms.hif_bandpass.s10_2.spw17_19_21_23_33_35_37_39.channel.solintinf.bcal.tbl',
               subplot=11, overlay='baseband', figfile=figdir+'regression97', showatm=False, yaxis='amp',
               vis='uid___A002_X652932_X20fb.ms', xaxis='freq', interactive=False, buildpdf=True)

if (overlayTime or tstart==0 or tstart==98):
    # 98
    os.chdir(mydir+'/regression/lindsey_slowness')
    myfunction('uid___A002_X86b751_X4e6.ms.tsys',overlay='time',xaxis='freq',subplot=11,antenna='',
               interactive=False,figfile=figdir+'regression98',buildpdf=True)

if (overlayTime or tstart==0 or tstart==99):
    # 99
    os.chdir(mydir+'/regression/bareTables')
    myfunction('sm2_rx0.usb.if2.tsys.ms.bandpass.bcal',interactive=False,
               buildpdf=True,figfile=figdir+'regression%02d'%(99),debug=False)

if (overlayTime or tstart==0 or tstart==100):
    # 100
    os.chdir(mydir+'/regression/bareTables')
    myfunction('uid___A002_X7ea111_Xc03.ms.tsys',interactive=False,
               buildpdf=True,figfile=figdir+'regression%03d'%(100),debug=False)


if (overlayTime or tstart==0 or tstart==101):
    # 101
    os.chdir(mydir+'/regression/bareTables')
    showBasebandNumber = True
    if (doTask): showBasebandNumber=False
    myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='baseband',xaxis='freq',interactive=False,
               buildpdf=True,figfile=figdir+'regression%03d'%(101),debug=False,showBasebandNumber=showBasebandNumber)

if (overlayTime or tstart==0 or tstart==102):
    # 102
    os.chdir(mydir+'/regression/bareTables')
    # This should fail to create any PDF because BBC_NO is unavailable.
    myfunction('bandpass_4.1.0.bcal',poln='',yaxis='amp',overlay='spw',xaxis='freq',interactive=False,
               buildpdf=True,figfile=figdir+'regression%03d'%(102),debug=False,showBasebandNumber=True)

if (overlayTime or tstart==0 or tstart==103 or overlayAntennaTime or showfdm):
    os.chdir(mydir+'/regression/overlay_antenna_time')
    # This should fail in the task version because BBC_NO is unavailable. (au simply disables showfdm).
    showfdm = True
    if (doTask): showfdm=False
    myfunction('uid___A002_X87b480_Xa11.ms.tsys',overlay='antenna,time',xaxis='freq',yaxis='amp',
               field='4',spw='9,11,13,15',plotrange=[0,0,0,0.000000],figfile=figdir+'regression103',
               showfdm=showfdm,chanrange='12~115',interactive=False, figfileSequential=True, subplot=22, buildpdf=True,
               antenna='DA41,DA42,DA43,DA46,DA50,DA55,DA56,DA57,DA59,DA60,DA62,DA64,DV02,DV05,DV09,DV10,DV11,DV13,DV14,DV15,DV18,DV19,DV25,PM03,PM04', chanrangeSetXrange=True)

if (tstart==0 or tstart==104 or overlayAntennaTime or showfdm):
    os.chdir(mydir+'/regression/ACA')
    myfunction('uid___A002_X856bb8_X79a7.ms.tsys',overlay='antenna,time',xaxis='freq',yaxis='amp',
               figfile=figdir+'regression104',buildpdf=True,
               showfdm=True,interactive=False, figfileSequential=True, subplot=22)

if (tstart==0 or tstart==105 or overlayAntennaTime or showfdm):
    os.chdir(mydir+'/regression/ACA')
    myfunction('uid___A002_X856bb8_X79a7.ms.tsys',overlay='antenna,time',xaxis='freq',yaxis='amp',
               figfile=figdir+'regression105',buildpdf=True,
               showfdm=True,interactive=False, figfileSequential=True, subplot=11)

if (overlayBaseband or tstart==0 or tstart==106):
    os.chdir(mydir+'/regression/infiniteLoop')
    # 106 (same as 96 but with showatm=True)
    myfunction(subplot=11, antenna='DA44', overlay='baseband',
               caltable='uid___A002_X652932_X20fb.ms.hif_bandpass.s10_2.spw17_19_21_23_33_35_37_39.channel.solintinf.bcal.tbl',
               figfile=figdir+'regression106', showatm=True, yaxis='amp', vis='uid___A002_X652932_X20fb.ms',
               xaxis='freq', interactive=False, buildpdf=True) 

if (overlayBaseband or tstart==0 or tstart==107):
    os.chdir(mydir+'/regression')
    # 107   CAS-7368  VLA pipeline dataset (amp)
    myfunction(subplot=11, antenna=0, overlay='baseband',
               caltable='finalBPcal.b', figfile=figdir+'regression107', yaxis='amp', 
               xaxis='freq', interactive=False, buildpdf=True) 

if (overlayBaseband or tstart==0 or tstart==108):
    os.chdir(mydir+'/regression')
    # 108   CAS-7368  VLA pipeline dataset (amp with showatm)
    myfunction(subplot=11, antenna=0, overlay='baseband', showatm=True,
               caltable='finalBPcal.b', figfile=figdir+'regression108', yaxis='amp', 
               xaxis='freq', interactive=False, buildpdf=True, debug=False) 
    
if (overlayBaseband or tstart==0 or tstart==109):
    os.chdir(mydir+'/regression')
    # 109   CAS-7368  VLA pipeline dataset (phase)
    myfunction(subplot=11, antenna=0, overlay='baseband',
               caltable='finalBPcal.b', figfile=figdir+'regression109', yaxis='phase', 
               xaxis='freq', interactive=False, buildpdf=True) 

if (overlayTime or tstart==0 or tstart==110 or showfdm):
    os.chdir(mydir+'/regression/tsysFlagged')
    # 110   4-polarization dataset, where all Tsys timeranges flagged on one spw for DV21
    myfunction(subplot=22, overlay='time',showfdm=True, showatm=True,
               caltable='uid___A002_Xa018c4_X2537.ms.tsys', figfile=figdir+'regression110', 
               xaxis='freq', interactive=False, buildpdf=True, showBasebandNumber=True) 

if (overlayTime or tstart==0 or tstart==111 or showfdm):
    os.chdir(mydir+'/mislabeling')
    # 111   lots of flags
    myfunction(subplot=11, overlay='time,antenna',showfdm=True, showatm=True,
               caltable='uid___A002_Xa6a120_X1d3d.ms.hifa_tsyscal.s6_1.tsyscal.tbl', 
               figfile=figdir+'regression111', 
               xaxis='freq', interactive=False, buildpdf=True, showBasebandNumber=True) 

if (overlayTime or tstart==0 or tstart==112):
    os.chdir(mydir+'/regression/badAntennaFilename')
    myfunction(subplot=11, antenna='3,4', overlay='baseband', caltable='uid___A002_Xabe765_X5bd.ms.bcal', 
               figfile=figdir+'regression112', buildpdf=True,
               showatm=True, yaxis='amp', xaxis='freq', interactive=False)

if (overlayTime or tstart==0 or tstart==113):
    os.chdir(mydir+'/regression/badSpwFilename')
    myfunction(subplot=11, antenna='DA58', overlay='time', caltable='uid___A002_Xa2ce2e_X7bd.ms.tsys', 
               figfile=figdir+'regression113', buildpdf=True,
               showatm=True, yaxis='amp', xaxis='freq', interactive=False)

if (tstart==114 or tstart==0):
  os.chdir('/lustre/cv/users/thunter/evla/AB1346/g19.36')
  # 76
  myfunction('bandpass.bcal',overlay='baseband',yaxis='both',xaxis='freq',
             interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(114),debug=False)

if (tstart==115 or tstart==0):
    # CAS-7965
    os.chdir(mydir+'/regression/CAS-7965')
    myfunction('uid___A002_X99c183_X25b6.ms.split.bandpass',subplot=11,overlay='baseband',xaxis='freq',
               chanrange='90%', interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(115))

if (tstart==116 or tstart==0):
    # CAS-7715  64-channel Band 9 Tsys showing offset in image sideband
    os.chdir(mydir+'/regression/CAS-7715')
    myfunction('uid___A002_Xa1f062_X37e3.ms.tsys',subplot=11,overlay='time',xaxis='freq',
               showatm=True, showimage=True,
               spw='30,32', interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(116))

if (tstart==117 or tstart==0):
    # CAS-7715  128-channel Band 9 Tsys showing offset in image sideband
    os.chdir(mydir+'/regression/CAS-7715')
    myfunction('uid___A002_Xa2ce2e_X54b.ms.tsys',subplot=11,overlay='time',xaxis='freq',
               showatm=True, showimage=True,
               spw='9', interactive=False,buildpdf=True,figfile=figdir+'regression%02d'%(117))

if (tstart==118 or tstart==0):
    # CAS-8261
    os.chdir(mydir+'/regression/pipelineCrashPartialPointingTable')
    myfunction('uid___A002_Xaf2188_X213a.ms.hifa_tsyscal.s6_3.tsyscal.tbl',subplot=11,chanrange='90%',
               antenna='42,43,24,25,26,27,20,21,22,23,28,29,40,41,1,0,3,2,5,4,7,6,9,8,39,38,11,10,13,12,15,14,17,16,19,18,31,30,37,36,35,34,33,32',
               yaxis='tsys', overlay='time', showatm=True, spw='19,21,17,23', xaxis='freq', showfdm=True, 
               interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(118))

if (tstart==119 or tstart==0):
    # CAS-8489, selection by timerange relative to scan
    os.chdir(mydir+'/regression/CAS-8489')
    myfunction('uid___A002_X85c183_X60b.skycal.spw23.tbl',scans='7',spw=23,timeranges='1,2',antenna=0,
               interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(119))

if (tstart==120 or tstart==0):
    # CAS-8489, overlay baseband x-axis range
    tstart = 0
    os.chdir(mydir+'/regression/CAS-8489')
    myfunction('uid___A002_Xb0dfe8_Xcc8.orion_sio.skycal.tbl',overlay='baseband',antenna=0,timeranges='0',
               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(120))

if (tstart==121 or tstart==0):
    # CAS-8489 final, overlay baseband y-axis range
    os.chdir(mydir+'/regression/CAS-8489')
    myfunction('uid___A002_Xb0dfe8_Xcc8.orion_sio.skycal.tbl',overlay='baseband',field='3',antenna=0,
               timeranges='0',scans='9',subplot=11,spw='23,25,27,29',showatm=True,yaxis='amp',
               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(121))

if (tstart==122 or tstart==0):
    # CAS-8655 
    os.chdir(mydir+'/regression/CAS-8655')
    myfunction('uid___A002_X960614_X1379.ms.hifa_tsyscal.s6_3.tsyscal.tbl/',overlay='antenna,time',
               subplot=11,spw='9',showatm=True,yaxis='amp',chanrange='90%',
               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(122))
#vis='/lustre/naasc/sciops/comm/rindebet/pipeline/root/2013.1.01194.S_2016_05_23T02_55_30.859/SOUS_uid___A001_X11f_X4a/GOUS_uid___A001_X11f_X4b/MOUS_uid___A001_X11f_X4c/working/uid___A002_X960614_X1379.ms',

#if (tstart==122 or tstart==0):
#    # CAS-8489 final, overlay spw y-axis range
#    os.chdir(mydir+'/regression/CAS-8489')
#    myfunction('uid___A002_Xb0dfe8_Xcc8.orion_sio.skycal.tbl',overlay='spw',field='3',antenna=0,
#               timeranges='0',scans='9',subplot=11,spw='23,25,27,29',showatm=True,yaxis='amp',
#               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(122))

if (tstart==123 or tstart==0):
    # CAS-9474
    os.chdir(mydir+'/regression/CAS-9474')
    myfunction('16B-402.sb32888048.eb33072402.57733.81140140047.ms.finalBPcal_sm.b',spw='19',
               caltable2='16B-402.sb32888048.eb33072402.57733.81140140047.ms.finalBPcal.b',
               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(123))

if (tstart==124 or tstart==0):
    # SCOPS-4877
    os.chdir(mydir+'/regression/SCOPS-4877') 
    myfunction('uid___A002_Xbf792a_X26ec.ms.tsys',overlay='time',showimage=True,showatm=True,showBasebandNumber=True,
               xaxis='freq', interactive=False, buildpdf=True,figfile=figdir+'regression%02d'%(124))

endTime = time.time()
print "Total execution time = %f seconds = %f minutes" % (endTime-startTime, (endTime-startTime)/60.)
os.chdir(figdir)
#os.system('rm *.png')
ntests = 122
myfiles = os.listdir(figdir)
if (doTask):
    npdfs = 95
else:
    npdfs = ntests
failures = ''
ctr = 0
failureTests = []
for n in range(npdfs):
    if (('regression%02d.pdf' % (n)) not in myfiles):
        failures += '%d ' % (n)
        failureTests.append(n)
        ctr += 1
print "%d failures (failed to produce a PDF): " % (ctr), failures
if (doTask):
    fname = 'cksum.test.task'
elif (doInternalTask):
    fname = 'cksum.test.internalTask'
else:
    fname = 'cksum.test'
print "Computing checksums"
fname2 = fname + '.pages'
#f = open(fname,'w')
f2 = open(fname2,'w')
for t in range(ntests):
#    status, output = commands.getstatusoutput('cksum regression%02d.pdf'%t)
#    cksum = output.split()[0]
    pages = au.countPagesInPDF('regression%02d.pdf'%t)
#    f.write('%2d %3d %10s\n' % (t,pages,cksum))
    f2.write('%2d %3d\n' % (t,pages))
#f.close()
f2.close()
if (tstart == 0):
    print "Checksum differences:"
    os.system('diff cksum.test plotbandpass_regression.cksum')
