#!/usr/bin/python
#
# OTF processing scriopt
#    usage: processOTF.py
#
#    2010-Aug-18 rsimon
"""
Process a collection of PSC data datafiles, to asses the antenna performance
at a variety of elevations.

This script relies on analyzeOTFPSC (including option to not display plots)

usage:
  processOTF.py
"""

import glob, os, sys
from optparse import OptionParser

###--------------------------------
### definitions of functions
###--------------------------------

#   functions unused for now

###--------------------------------
### end definitions
###--------------------------------

analyzeOTFPSC = "/groups/science/analysis_scripts/analyzeOTFPSC.py"
    
AntennaList = ["DV01","DV02","DV03","DV04","DV05","DV06","DV07","DV08","DV09",
               "DV10","DV11","DV12","DV13","DV14","DV15","DV16","DV17","DV18",
               "DV19","DV20","DV21","DV22","DV23","DV24","DV25",
               "DA41","DA42","DA43","DA44","DA45","DA46","DA47","DA48","DA49",
               "DA50","DA51","DA52","DA53","DA54","DA55","DA56","DA57","DA58",
               "DA59","DA60","DA61","DA62","DA63","DA64","DA65",
               "PM01","PM02","PM03","PM04",
               "CM01","CM02","CM03","CM04","CM05","CM06","CM07","CM08","CM09",
               "CM10","CM11","CM12"]

PARSER = OptionParser()
PARSER.set_defaults(datadir       = "./",
                    outputfile    = "",
                    rmsThreshold  = "2.0",
                    maskTime      = "0.8",
                    maskTimeLimit = "2.0",
                    filemask      = "psc",
                    plots         = "n")
PARSER.add_option("-d", "--datadir", dest="datadir", metavar="DataDir",
                  help="The path to the folder with the data files. "+
                  "Default is current directory.")
PARSER.add_option("-o", "--outputfile", dest="outputfile", metavar="OutputFile",
                  help="Name of the file for output text file. "+ 
                  "Default will be in current folder, named \"ResultsOTF...\"")
PARSER.add_option("-r", "--rms", dest="rmsThreshold", metavar="RmsThreshold",
                  help="Normal Pass/Fail criterion for tracking accuracy [\"]. "+
                  "Default is 2.0 arcseconds.")
PARSER.add_option("-m", "--masktime", dest="maskTime", metavar="MaskTime",
                  help="The usual duration of mask at the start of scans [s]. "+
                  "Default value is 0.8 seconds.")
PARSER.add_option("-l", "--masklimit", dest="maskTimeLimit", metavar="MaskTimeLimit",
                  help="Maximum mask time to test at start of scans [s]. "+
                  "Default is 2.0 seconds for the maximum mask time to try.")
PARSER.add_option("-f", "--filemask", dest="filemask", metavar="FileMask",
                  help="File selection filter (e.g., \"psc\"). All files "+
                  "including the mask in their name and ending in .dat will be "+
                  "processed.  Default is to process files named *psc*.dat" )
PARSER.add_option("-p", "--plots", dest="plots", metavar="Plots",
                  help="Show the plots for each processed data file [y or n]. "+
                  "Default is \"n\" to skip doing plots." )

(OPTIONS, ARGS) = PARSER.parse_args()

datadir       = OPTIONS.datadir
rmsThreshold  = float(OPTIONS.rmsThreshold)
maskTime      = float(OPTIONS.maskTime)
maskTimeLimit = float(OPTIONS.maskTimeLimit)
filemask      = OPTIONS.filemask
outputfile    = OPTIONS.outputfile
plots         = OPTIONS.plots

    #   Check for official Antenna name in path, and use in default outputfile
    
if OPTIONS.outputfile == "":
    Antenna = ""
    for item in AntennaList:
        if item in datadir: Antenna = "_" + item
    outputfile = ("ResultsOTF" +
                   Antenna +
                   "_r" + str(rmsThreshold) +
                   "_m" + str(maskTime) + ".txt")
else:
    outputfile = OPTIONS.outputfile

    #   Test parameter validity
if maskTimeLimit < maskTime:
    maskTimeLimit = maskTime
masktimestep = 0.2              # could use (masktimelimit - masktime)/4.0

plots = str.lower(plots)
if plots <> "n": 
    if plots <> "y":
        print "\n\nERROR: Parameter \"plots\" must be y or n; setting to n for now.\n\n"
        plots = "n"

print "\n\nProcessing files named " + datadir + "*" + filemask + "*.dat"
print "using"
print "rmsThreshold  = ", rmsThreshold, " arcseconds."
print "maskTime      = ", maskTime, " seconds."
print "maskTimeLimit = ", maskTimeLimit, " seconds (for testing marginal cases)."
print ("                 If needed, will try mask times from " + str(maskTime) +
                        " to " + str(maskTimeLimit))
print "                 in steps of " + str(masktimestep) + " seconds."
print "outputfile    = ", outputfile

if not os.path.exists(OPTIONS.datadir):
    print "Data folder \"%s\" does not exist." % OPTIONS.datadir
    sys.exit()

#       Retrieve the list of files and print
filelist = glob.glob (datadir + "*" + filemask + "*.dat")
list.sort(filelist)
print "\nThe following data files will be processed:"
i=0
for item in filelist:
    i = i+1
    print repr(i).rjust(2), item
print


#------------------------------------------------------------------------------
#       Loop through the file list and process the files one by one
#------------------------------------------------------------------------------
i=0
for item in filelist:
    i = i+1
    trialcount = 1
    newmasktime = maskTime
#       Open the file for the results for writing
    resultsHandle = open (outputfile, 'a')
    resultsHandle.write (repr(i).rjust(2) + " " + item +
                        " rms<" + str(rmsThreshold) + "arcsec" +
                        " mask<" + str(maskTime) + "seconds\n" )
    resultsHandle.flush()
    
#       Assemble the python call to run the script on the selected file
    commandString = (analyzeOTFPSC + " -f " + item +
                          " -r " + str(rmsThreshold) +
                          " -m " + str(maskTime) +
                          " -y 4" +
                          " -d " + plots +
                          ">> " + outputfile)

    #  Call analyzeOTFPSC to process a single file
    os.system ('python ' + commandString + ' >> ' + outputfile)
 
    resultsHandle = open (outputfile, 'r')
    resultsList = resultsHandle.readlines()
    resultsHandle.close()
    resultstring = resultsList[-1]
    
    #  Substitutiion of FAILED with escape codes for highlighting when printed
    resultstring = (resultstring [0:resultstring.find(' (')] + " :" +
                    resultstring [resultstring.find(' El='):len(resultstring)-1])
    FAILED = resultstring.count ('FAILED!')
    if FAILED:
        resultstring = resultstring.replace ('FAILED!',
                                             '\033[1m\033[31m\033[07mFAILED!\033[0m')
        resultstring = resultstring + ' : m=' + str(maskTime)

    print repr(i).rjust(2)+":", item, ":", resultstring
    
    #  If FAILED, run loop if requested to test other mask times.
    if FAILED:

        while FAILED and maskTimeLimit - newmasktime > .01:
            trialcount = trialcount + 1
            newmasktime = newmasktime + masktimestep
            #   Open the file for the results for writing
            resultsHandle = open (outputfile, 'a')
            resultsHandle.write (repr(i).rjust(2) + " " + item +
                        " rms<" + str(rmsThreshold) + "arcsec" +
                        " mask<" + str(newmasktime) + "seconds\n" )
            resultsHandle.flush()
            commandString = (analyzeOTFPSC + " -f " + item +
                         " -r " + str(rmsThreshold) +
                          " -m " + str(newmasktime) +
                          " -y 4 -d n " +
                         ">> " + outputfile)
            os.system ('python ' + commandString + ' >> ' + outputfile)
 
            resultsHandle = open (outputfile, 'r')
            resultsList = resultsHandle.readlines()
            resultsHandle.close()
            resultstring = resultsList[-1]
            resultstring = (resultstring [0:resultstring.find(' (')] + " :" +
                            resultstring [resultstring.find(' El='):len(resultstring)-1])
            FAILED = resultstring.count ('FAILED!')
            if FAILED:
                resultstring = resultstring.replace ('FAILED!',
                                                     '\033[1m\033[30m\033[07mFAILED!\033[0m')
            else:
                #       Success!  Save new results and reset flag
                resultstring = resultstring.replace ('PASSED ',
                                                     '\033[1m\033[32m\033[07mPASSED \033[0m')
        # print the final result of the mask time tests
        resultstring = resultstring + ' : m=' + str(newmasktime)
        print "  :", item, ":", resultstring
    #   Delete intermediate failed trials from output file, saving only 1st and last trials
        if trialcount > 2:
            lines = len(resultsList)
            del resultsList[lines-(4*(trialcount-1)):lines-4]
            resultsHandle = open (outputfile, 'w')   # open for overwriting
            resultsHandle.writelines(resultsList)
	    resultsHandle.writelines("\n")
            resultsHandle.close
            
    #   blank line after processing each file
    os.system ('echo '' '' >> ' + outputfile )
