from scipy import *
from numpy import *
from pylab import *
from taskinit import *
import pickle
import analysisUtils as aU

import pyplot
import pybnlib
import pyoof
import implot

import moonscans as m

#$Id: reduc_cutscans.py,v 1.6 2010/05/05 21:01:14 dbarkats Exp $


#vis is ms file, aid is antenna ID.
# need to add comments and a proper explanation

def makecutmap(msfile, aid, scan, exclRegion):

    vis=aU.locate(msfile)[0]
    #get info about observations.
    tb.open(vis+'/ANTENNA')
    pos=tb.getcol('POSITION')[:,aid]
    ant=tb.getcol('NAME')[aid]
    pad=tb.getcol('STATION')[aid]
    tb.close()
   
    # get name of source.
    tb.open(vis+ '/SOURCE')
    src=tb.getcol('NAME')[0]
    tb.close()
    #get frequency and define image size and pixel size
    tb.open(vis+'/SPECTRAL_WINDOW')
    freq=tb.getcol('CHAN_FREQ')
    freq=freq[0,0]
    tb.close()
    if freq < 116e9:
        band='3'
    elif freq < 275e9:
        band='6'
    elif freq < 373e9:
        band='7'
    elif freq < 720e9:
        band = '9'
        
    # Determine moon size
    import ephem
    
    src='Moon'
    tb.open(vis+'/POINTING')
    t=tb.getcol('TIME')
    t0str=str(t[0])+'s'
    tt=qa.time(t0str, form='fits')
    dt=tt.replace('-','/').replace('T',' ')
    exec 'j=ephem.%s()' %src
    j.compute(dt)
    diameter=j.size/60
    strdiam="%3.2f"%(diameter)
    tb.close()
    print "Moon size:",diameter,"arcmin"
    
    # get timestamp of first raster scan number scanum
    tb.open(vis)
    Tb = tb.query('ANTENNA1==%d && ANTENNA2==%d && DATA_DESC_ID==%d && SCAN_NUMBER == %d' % (aid, aid ,0, scan))
    sid=Tb.getcol('STATE_ID')
    t0=Tb.getcol('TIME')
    tb.close()
    
    #get data and az/el
    [t, data]=aU.getSpectralAutoData(vis, aid, 0, scan)
    tb.open(vis+'/POINTING')
    off=tb.getcol('POINTING_OFFSET')
    off=off*180/pi
    time=tb.getcol('TIME')
    dir=tb.getcol('DIRECTION')
    dir=dir*180/pi
    tb.close()

    # calc angle of observations
    phi=int(round(degrees(math.atan(off[1,0,0]/off[0,0,0]))))
    if (phi<0): phi=phi+180
    
    print ant+ ' located at '+ pad + ' observing '+src+' at Band'+ band+ ' at '+str(phi)+' deg'
    print 'Wavelength =%f mm' %(3e11/freq)
    #imagefile=vis.split('/')[-1][:-3]+'_'+ant+'_'+pad+'_'+src+'_band'+band+'_'+str(phi)+'deg'
    imagefile=vis.split('/')[-1][:-3]+'_'+ant+'_'+pad+'_'+src+'_'+strdiam+'_band'+band+'_'+str(phi)+'deg'
    print  imagefile   

    # interpolate az/el to time of data
    az=off[0,0,:]
    az=interp(t-t[0],time-time[0],az)
    el=dir[1,0,:]
    el=interp(t-t[0],time-time[0],el)
    secel=1/cos((90.-el)/180.*pi) 
    dist=sqrt(off[0,0,:]**2+off[1,0,:]**2)*off[0,0,:]/abs(off[0,0,:])
    if phi<91 and phi>89:
        dist=off[1,0,:]
    dist=interp(t-t[0],time-time[0],dist)
   
    figure(4, figsize=(10,6))
    clf()
    #subtract background of subscans
    d1_bs=subtract_background(data[0,0,:],secel, sid, True, exclRegion)[0]
    d2_bs=subtract_background(data[1,0,:],secel, sid, False, exclRegion)[0]
    fitrange=subtract_background(data[1,0,:],dist, sid, False, exclRegion)[1]

    #extract subscan data
    d1=extract_data(data[0,0,:],dist, sid)
    d2=extract_data(data[1,0,:],dist, sid)

    #normalize subscans 
    d1_bs=d1_bs/max(d1_bs);
    d2_bs=d2_bs/max(d2_bs);

    #rebin in dist
    nbins=int((max(dist)-min(dist))*40)
    print 'Binning into %i bins'%nbins
    xb=zeros(nbins)
    bd1=zeros([4,nbins])
    bd2=zeros([4,nbins])
    dx=(max(dist)-min(dist))/(nbins)
    cells=min(dist)-dx/2 + arange(nbins+1)*dx
    for i in range(nbins):
        q=find((dist > cells[i]) & (dist <= cells[i+1]))
        bd1[0,i]=mean(dist[q])
        bd2[0,i]=mean(dist[q])
        bd1[1,i]=mean(d1_bs[q])
        bd2[1,i]=mean(d2_bs[q])
        bd1[2,i]=std(d1_bs[q])
        bd2[2,i]=std(d2_bs[q])
        bd1[3,i]=median(d1_bs[q])
        bd2[3,i]=median(d2_bs[q])

    # remove background of min(binned d1)
    #d1=d1-min(bd1[1,:])
    #bd1[1,:]=bd1[1,:]-min(bd1[1,:])
    #bd2[1,:]=bd2[1,:]-min(bd2[1,:])

    #normalize the averaged data
    #sd1=sd1/max(sd1);
    #sd2=sd2/max(sd2);

    xaxis1=bd1[0,:]
    xaxis2=bd2[0,:]
    yaxis1=bd1[3,:]
    yaxis2=bd2[3,:]

    #plot figure
    figure(1, figsize=(10,6))
    clf()
    #plot(dist, d1, '.')
    subplots_adjust(hspace=0)
    subplot(212)
    plot(dist, d1_bs, '.')
    errorbar(bd1[0,:], bd1[1,:], bd1[2,:],fmt='or')
    errorbar(bd2[0,:], bd2[1,:], bd2[2,:],fmt='og')
    title(imagefile+' Moon cuts')
    xx=[-10,10];yy=[0,0];plot(xx,yy)
    ylabel('Relative power (lin)')
    xlim(-4,4)
    ylim([-0.001, 0.001])
    xlabel('Offset dist (deg)')
    #savefig(imagefile+'_lin.png', format='png')

    #plot figure
    #figure(2, figsize=(14,8))
    #clf()
    subplot(211)
    semilogy(dist, (d1_bs), '.')
    #semilogy(bd1[0,:], (bd1[1,:]),'or')
    #semilogy(bd1[0,:], (bd2[1,:]),'og')
    semilogy(xaxis1, yaxis1,'or')
    semilogy(xaxis2, yaxis2,'og')
    semilogy(fitrange, fitrange*0+0.01, 'oc')
    legend(['', 'XX', 'YY'])
    title(imagefile+' Moon cuts')
    #xlabel('Offset [deg]') ; ylabel('Relative power')
    ylabel('Relative power (log)')
    xlim(-4,4)
    ylim([0.001, 1.3])
    #set_yscale('symlog'),basey=10**0.1,linthreshy=[-100,0.001])
    savefig(imagefile+'_log.png', format='png')

    f=open('/home/mzwaan/MOON/%s.sav'%imagefile ,'w')
    #pickle.dump(bd1, f)
    pickle.dump(bd1, f)


def mapcompare(rms, errscl, l,  obsfile):
    # Where phrms is the surface error in meters
    # errscl is the size scale in meters
    # l is the wavelength in meters
    # obsfile is the observed profile

    # wavfront rms
    #if type(rms) is not list:
    #    rms=[rms]
    #phrms=[(2.*x)/l*2*pi for x in rms]
    phrms=(2.*rms)/l*2*pi

    #set diameter of Moon in arcmin
    #moondiam=31

    #read in the observed data
    f=open('/users/mzwaan/MOON/%s.sav'%obsfile,'r')
    data=pickle.load(f)

    #get cutangle from filename
    dg=obsfile.find('band')
    cutangle=int(obsfile[dg::].split('_')[1][0:-3])
    #get moon diameter from filename
    dg=obsfile.find('Moon')
    moondiam=float(obsfile[dg::].split('_')[1])
    print "Moon diameter:",moondiam
    #get antenna name from filename
    dg=obsfile.find('X1')
    antenna=(obsfile[dg::].split('_')[1])
    print "Antenna:",antenna
    if (antenna[0:2]=='DV'):
        anttype='vertex'
    if (antenna[0:2]=='PM'):
        anttype='melco12'
    print "Antenna type:", anttype

    #print cutangle
    
    # Creates a pair of maps
    # which represent the the aperture plane  phase and amplitude distributions
    npix=2048

    strphrms="%4.3f"%(phrms)
    simfileout='moonsim_%s_%s_%s_%s_%s_%s'%(strphrms,str(errscl), str(l), str(cutangle), str(moondiam), anttype)
    location=os.popen('find . -name %s.sav' %simfileout).read().strip()
    # the file does not exist, create it
    if location=='':
        mphase = m.mkALMAAperture(npix=npix,phaserms=phrms, errscale=errscl, ant=anttype, justPhase=True)
        #mamp=m.getAmpMap("vertex", npix)
        mamp=m.getAmpMap(anttype, npix)
        # Plot the phase map
        implot.plotmap(mphase, transf=0, colmap="heat")
        implot.plotmap(mamp, transf=0, colmap="heat")
        # Create the object that transforms the aperture plane
        # distribution to far field. The second parameter is the
        # wavelength of the radiation
        farf= pyoof.FarF(mamp, l)
        # Create a map to store the far field pattern in
        mbeam=pyplot.Map(npix,npix)
        # Compute the far-field power pattern
        farf.Power(mamp,  mphase, mbeam)
        # Plot the beam as on log-scale (second parameter, it has the same
        # meaning as the convention in PGPLOT)
        implot.plotmap(mbeam,   transf=1, colmap="heat")
        # Convolve with a Moon simulation
        msim=m.mkMoonSim(mbeam,moondiam)
        # Plot the beam as on log-scale
        implot.plotmap(msim,  transf=1, colmap="heat")
        # get the cut beam at specfied angle
        msimcut=m.mapCut(msim, cutangle)

        #save output cut beam
        f=open('/users/mzwaan/MOON/%s.sav' %simfileout ,'w')
        pickle.dump(msimcut, f)
    else:
        #read in the cut beam
        f=open('/users/mzwaan/MOON/%s.sav'%simfileout,'r')
        msimcut=pickle.load(f)


    # find the centre
    ycross=0.1
    leng=int(len(data[1,:])/2)
    leftxx=(data[0,range(leng)])
    rightxx=(data[0,range(leng+1,2*leng-1)])
    leftyy=(data[1,range(leng)])
    rightyy=(data[1,range(leng+1,2*leng-1)])
    xx=find(leftyy>ycross)
    leftx=leftxx[xx[0]-1:(xx[0])+1]
    lefty=leftyy[xx[0]-1:(xx[0])+1]
    xx=find(rightyy<ycross)
    rightx=rightxx[xx[0]-1:(xx[0])+1]
    righty=rightyy[xx[0]-1:(xx[0])+1]
    leftc=(ycross-lefty[0])/(lefty[1]-lefty[0])*(leftx[1]-leftx[0])+leftx[0]
    rightc=-1*((ycross-righty[1])/(righty[0]-righty[1])*(rightx[1]-rightx[0]))+rightx[1]
    offset=(leftc+rightc)/2


    # plot latest sim vs vs data
    figure(12, figsize=(10,6))
    clf()
    subplots_adjust(hspace=0)
    subplot(211)
    data[0,:]=data[0,:]-offset
    semilogy(msimcut[0][:]*180/pi,msimcut[1][:]/max(msimcut[1][:]))
    semilogy(data[0,:], data[1,:], '.')
    #q1=find(data[1,:] < 0)
    #semilogy(data[0,q1], data[1,q1], 'or')
    legend(['Simulated ', 'Observed'])
    figtext(0.68, 0.76, 'phrms = %4.3f rad'%(phrms), color='b')
    figtext(0.68, 0.735, 'surface rms = %3.2f $\mu$m'%(rms*1E6), color='b')
    figtext(0.68, 0.72, 'errscl = %3.2f m'%(errscl), color='b')
    figtext(0.68, 0.70, 'lambda = %5.4f mm'%(l*1E3), color='b')
    xlim(-3,3)
    ylim(0.0004,1.3)
    title('%s_%s_compare'%(simfileout, obsfile))
    ylabel('Relative power (log)')

    subplot(212)
    plot(msimcut[0][:]*180/pi,msimcut[1][:]/max(msimcut[1][:]))
    plot(data[0,:], data[1,:], '.')
    #q1=find(data[1,:] < 0)
    #semilogy(data[0,q1], data[1,q1], 'or')
    xlim(-3,3)
    ylim(-0.0004,0.0004)
    xlabel('offset distance (deg)')
    ylabel('Relative power (lin)')
    
    savefig('%s_%s_compare.png'%(simfileout, obsfile), format='png')



def plot_multiple_cuts(cutlist):
    cut = str.split(cutlist)
    figure(10)
    clf()
    for i in cut:
        f=open('/users/mzwaan/MOON/20100206_mooncuts/%s_DV01.sav' %i , 'r')
        data=pickle.load(f)
        semilogy(abs(data[0,:]), '.-')
        legend(cut)
        xlabel('Offset Dist (deg)') ; ylabel('Relative power (dB)')
        title('4 cuts across the sun at 0, 90,45, 135deg, Band7 (radius=4deg)')

    savefig('sun_cut_0_90_45_135_summary2.png', format='png')
        
def subtract_background(data, dist, sid, doplot,ex):
   # subtract background.
   # and fit a polynomial of order deg from from those two set if indices.
   # ex=40 # number of pixels to exclude at edge.
   nbase=35 # number of points to use at each edge to do baseline subtraction.
   deg=1 # degree of baseline subtraction (1= linear)
   for i in unique(sid):
       q=find(sid==i)
       x=dist[q]
       y=data[q]
       xdata=concatenate((x[ex:nbase+ex], x[-ex-nbase:-ex]),0)# concatenating index
       ydata=concatenate((y[ex:nbase+ex], y[-ex-nbase:-ex]),0)
       polycoeffs=polyfit(xdata, ydata, deg)
       yfit=polyval(polycoeffs, x)
       if (doplot):
           plot(x,y)
           plot(x,yfit)
       y = y - yfit
       data[q]=y 
   return data,xdata



def extract_data(data, az, sid):
   for i in unique(sid):
       q=find(sid==i)
       x=az[q]
       y=data[q]
       data[q]=y

   return data
