###
#OOF reduction analysis code.
#Written and maintained by Denis Barkats (dbarkats@alma.cl)

#import reduc_off as r
#run as r.autopipeline()
# or r. pipeline(msfile, aid, zorder )
#where
#msfile= name of msfile  (uid___X02_X1234_X1.ms)
#aid is antenna ID (0, 1, 2)
#zorder is Zernike order to analyse.(5,4,6)

#last update = may 15 2010

###

import pyfits
from pylab import *
import analysisUtils as aU
from taskinit import *
import numpy
import glob

import oofreduce
import oofplot
import ephem

def autopipeline():
   """
   To run the analysis code fuly automatically.
   Needs the existance of the file: oof_data_sofar.txt in same directory.
   Will produce a directory called oofout with all the outputs
   STATUS: Tested and functional
   """
   d=read_oofdatasofar()
   nobs=size(d['uid'])
   for i in range(nobs):
      ants=d['antList'][i].split(' ')
      for j in range(size(ants)):
         aid=j
         msfile=d['uid'][i]+'.ms'
         print aid, msfile
         pipeline(msfile, aid, 5)
   return
         
def pipeline(msfile, aid,  zorder):
   """
   To run the analysis code on one observations
   Is called by autopipeline.
   Will produce a directory called oofout with all the outputs
   msfile: name and location  of observations (ie msfile = '/work_area/dbarkats/uid___X02_X1234_X1.ms')
   aid: antenna id (ie 1)
   zorder: order of the zernike polynomial you want to produce plots.
   STATUS: Tested and functional
   """
   fitsfile=main(msfile, aid)
   print fitsfile
   outdir=oofreduce.Red(fitsfile,nzmax=zorder,extrafit=["x0", "y0"],npix=256,oversample=6) 
   oofplot.PlotDir('%s/z%i/'%(outdir, zorder), bbox=[-1e-3 ,1e-3, -1e-3, 1e-3], umicron=True)
   oofplot.makeHTML('%s/z%i/'%(outdir, zorder))
   return
   
def main(msfile, aid):
   """
   To produce the fits file needed by "pipeline" for 1 observation
   Is called by autopipeline.
   msfile: name and location  of observations (ie msfile = '/work_area/dbarkats/uid___X02_X1234_X1.ms')
   aid: antenna id (ie 1)
   STATUS: Tested and functional
   """
   uid = msfile.split('/')[-1].split('.ms')[0]

   if os.path.isdir(msfile):
      vis = msfile
   else:
      vis=aU.locate(msfile)[0]
   
   #get scannums which has the beam maps
   disp('Searching for scan numbers which contain the beam map...')
   vm = aU.ValueMapping(vis)
   scanList = vm.getScansForIntent('OBSERVE_TARGET#ON_SOURCE')
   if size(scanList) == 0:
      print "No Beam Map scans found, you may need to try again searching changeing OBSERVE_TARGET#ON_SOURCE to OBSERVE_TARGET.ON_SOURCE if this is an older dataset"
      sys.exit()
   
   #get info about observations.
   disp('Gathering information about the observation ...')
   tb.open(vis+'/ANTENNA')
   pos=tb.getcol('POSITION')[:,aid]
   ant=tb.getcol('NAME')[aid]
   pad=tb.getcol('STATION')[aid]
   tb.close()
   
   # get name of source.
   tb.open(vis+ '/SOURCE')
   src=tb.getcol('NAME')[0]
   tb.close()
   
   #get frequency and define image size and pixel size
   tb.open(vis+'/SPECTRAL_WINDOW')
   freq=tb.getcol('CHAN_FREQ')
   freq=freq[0,0]
   tb.close()
   if freq < 116e9:
      band='3'
   elif freq < 275e9:
      band='6'
   elif freq < 373e9:
      band='7'
   elif freq < 720e9:
      band = '9'

   print ant+ ' located at '+ pad + ' observing '+src+' at Band'+ band
   fnameout=vis.split('/')[-1][:-3]+'_'+ant+'_'+pad+'_'+src+'_band'+band+'.fits'
   print  fnameout

   tb.open(vis+'/POINTING')
   t=tb.getcol('TIME')
   tb.close()
   
   #  get size of planet
   tt=qa.time(str(t[0])+'s', form='fits')
   dt=tt.replace('-','/').replace('T',' ')
   exec 'j=ephem.%s()' %src
   j.compute(dt)
   if src == 'Saturn':
      radius= 0.7* (j.radius)
   else:
      radius= (j.radius)
   print radius, degrees(radius)*3600
                 
   # Setup input  OOF file
   fout=pyfits.HDUList()
   phdu=pyfits.PrimaryHDU()
   fout.append(phdu)
   # Write the primary header keys 
   phdu.header.update("TELESC", "ALMA")
   phdu.header.update("FREQ", freq)
   phdu.header.update("OBSTYPE", 'MAP')
   phdu.header.update("OBJECT", src)
   phdu.header.update("OBJRADS", radius)

   #read timeseries beam map data
   for i in range(size(scanumList)):
      scanum=scanumList[i]
      if type(scanumList[1]) ==  type('str'): scanum=int(scanum)
      [t, az, el, data, data_err,Z]=readData(vis, aid, scanum)
      plot(t,data)
      if i==0: Z0=Z

      # TODO remove this hack
      # dB adds in this hack to remove problem caused by CSV-1419,CSV-1900 at the start of data
      t=t[100:]
      data=data[100:]
      data_err=data_err[100:]
      az=az[100:]
      el=el[100:]
         
      dz=Z-Z0
      print scanum, dz
      meanel=mean(el)
      nsamples=size(t)
      f=OOFColumns(nsamples)
           
      # Write the actual data here!
      f.data.field("DX")[:]=az
      f.data.field("DY")[:]=el
      f.data.field("fnu")[:]=data
      f.data.field("ufnu")[:]=data_err
      f.header.update("DZ", dz)
      f.header.update("ELEV", meanel)
      fout.append(f)

   #save the plot to
   savefig('%s_timeseries.png'%(fnameout), format='png')
   
   #write file out
   if(not os.path.isdir(fnameout)):
      os.system('rm -f %s' % fnameout)
   fout.writeto(fnameout)
   fout.close()
   
   del(fout)
   
   return fnameout


def OOFColumns(nrows):
   """
   Create an empty OOF table with the required columns
   """
   coldefs = [
       pyfits.Column("DX", "E", "radians",
                     array=numpy.zeros(nrows)),
       pyfits.Column("DY",   "E",   "radians",
                     array=numpy.zeros(nrows)),
       pyfits.Column("FNu",   "E",   "Jy",
                     array=numpy.zeros(nrows)),
       pyfits.Column("UFNu",   "E",   "Jy",
                     array=numpy.ones(nrows)),
       pyfits.Column("Time",   "E",   "d",
                     array=numpy.zeros(nrows))
       ]
   nh = pyfits.new_table( coldefs )
   return nh

def readData(vis, aid, scanum):     
   """
   read and output the data samples for 1 observation
   Is called by "main"
   
   vis: name and location of observations (ie vis=aU.locate('uid___X02_X1234_X1.ms')
   aid: antenna id (ie 1)
   scanum: scan number of beam map of insterest (ie 7)
   returns  [t, az, el, data, data_err, Z]
   STATUS: Tested and functional
   """
   tb.open(vis)
   Tb = tb.query('ANTENNA1==%d && ANTENNA2==%d && DATA_DESC_ID==%d && SCAN_NUMBER == %d' % (aid, aid ,0, scanum))
   sid=Tb.getcol('STATE_ID')
   t0=Tb.getcol('TIME')
   start_t0=t0[0]
   end_t0=t0[-1]
   tb.close()  
   
   #get the measured focus at start of scan
   focus=aU.getMeasFocus(vis, aid, scanum)
   Z=focus[2]
   [t, data]=aU.getSpectralAutoData(vis, aid, 0, scanum)
   tb.open(vis+'/POINTING')
   off=tb.getcol('POINTING_OFFSET')
   off=off*180/pi
   time=tb.getcol('TIME')
   tb.close()
   
   # interpolate az/el to time of data
   q=find((time < end_t0) & (time > start_t0))
   az=interp(t-t[0],time[q]-time[q[0]],off[0,0,q])*pi/180 # in radians
   el=interp(t-t[0],time[q]-time[q[0]],off[1,0,q])*pi/180 # in radians 
   
   #subtract background and choose pol 0
   data=subtract_background(data[0,0,:],az, sid)
   
   #calculate error by taking rms of last 500 points
   nsamples=size(t)
   data_err=zeros(len(data))
   data_err[:]=std(data[nsamples-500:])
   return [t, az, el, data, data_err, Z]


def subtract_background(data, az, sid):
   """
   subtract background for each az scan.
   and fit a polynomial of order 1
   """
   
   ex=1 # number of pixels to exclude at edge.
   nbase=10 # number of points to use at each edge to do baseline subtraction.
   deg=1 # degree of baseline subtraction (1= linear)
   for i in unique(sid):
       q=find(sid==i)
       x=az[q]
       y=data[q]
       xdata=concatenate((x[ex:nbase+ex], x[-ex-nbase:-ex]),0)# concatenating index
       ydata=concatenate((y[ex:nbase+ex], y[-ex-nbase:-ex]),0)
       polycoeffs=polyfit(xdata, ydata, deg)
       yfit=polyval(polycoeffs, x)
       y = y - yfit
       data[q]=y

   return data

def read_oofdatasofar():
   b=open('oof_data_sofar.txt')
   obs=b.readlines() 
   data={}
   attList = ['uid','antList']
   for i in attList: data[i] = []
   for i in range(size(obs)):
      line=obs[i]
      #print  i, line
      if line[0] != '\n' and line[0] != '#':   
         data['uid'].append(line.split(',')[3].lstrip())
         data['antList'].append(line.split(',')[5].lstrip())
         
   return data

def get_planet_size(msfile):
   """
   Given an msfile (filename only , not location), it will return the  size of the planet at the time
   of observation.
   STATUS: tested and functional
   """
   
   vis=aU.locate(msfile)[0]
   
   # get name of source.
   tb.open(vis+ '/SOURCE')
   src=tb.getcol('NAME')[0]
   tb.close()
   
   tb.open(vis+'/POINTING')
   t=tb.getcol('TIME')
   tb.close()
   
   #  get size of planet
   tt=qa.time(str(t[0])+'s', form='fits')
   dt=tt.replace('-','/').replace('T',' ')
   exec 'j=ephem.%s()' %src
   j.compute(dt)
   if src == 'Saturn':
      radius=  (j.radius)
   else:
      radius= (j.radius)
   print dt, src , j.radius, degrees(radius)*3600.
                


def plot_results():
   """
   function to take the results  in  results_dir and results_dir2 and to plot the
   x0, y0 offsets and  sigma distribution.
   for now planet is hardwired
   STATUS:very much in developpent.
   """
   
   ants=['DV01','DV02','PM03']
   x0={};y0={} ;sigma={} ;band={}; chisq={}
  
   for i in range(size(ants)):
      ant=ants[i]
      x0[ant]=[]; y0[ant]=[]; sigma[ant]=[]; band[ant]=[]; chisq[ant]=[];
      
      results_dir='oofout_results3_extrafit_x0_y0_oversample_6'  # to read x0, y0
      results_dir2='oofout_results3_extrafit_x0_y0_sigma_oversample_6' # to read sigma
      results_dir3='oofout_results3_extrafit_x0_y0_sigma_oversample_6' # to read chisq
      paramfile=glob.glob('%s/*%s*' %(results_dir,ant[i]))
      paramfile2=glob.glob('%s/*%s*' %(results_dir2,ant[i]))
      paramfile3=glob.glob('%s/*%s*' %(results_dir3,ant[i]))
             
      nobs=size(paramfile)
      for k in range(nobs):
         filename="%s/z5/fitpars.fits" %paramfile[k]
         filename2="%s/z5/fitpars.fits" %paramfile2[k]
         filename3="%s/z5/fitinfo.fits" %paramfile3[k]
         band[ant].append(filename.split('band')[1][0])
         params=pyfits.open(filename)[1].data
         params2=pyfits.open(filename2)[1].data
         #print filename, params[2][1], params[3][1], params[1][1]
         x0[ant].append(params[2][1])
         y0[ant].append(params[3][1])
         sigma[ant].append(params2[1][1])
         chisq[ant].append(pyfits.open(filename3)[2].data[1][1])

      band_t=array(band[ant])
      x0_t=array(x0[ant])
      y0_t=array(y0[ant])
      sigma_t=array(sigma[ant])
      b3=band_t=='3'
      b6=band_t=='6'
      b7=band_t=='7'

      figure(1)
      clf()
      plot(x0_t[b3], y0_t[b3],'ro')
      plot(x0_t[b6], y0_t[b6],'go')
      plot(x0_t[b7], y0_t[b7],'bo')
      xlim([-2, 2]); ylim([-2 ,2])
      legend(('Band3', 'Band6','Band7'), loc=1)
      xlabel('Horizontal aperture illumination offset')
      ylabel('Vertical aperture illumination offset');
      title('Aperture illumination offset for %s' %ant[i])
      savefig('%s_aperture_illumination_offset.png'%ant[i])

      figure(2)
      clf()
      hist(sigma_t,20,[0,1])
      xlim([0, 1])
      xlabel('aperture illumination sigma')
      title('Aperture illumination sigma for %s' %ant[i])
      savefig('%s_aperture_illumination_sigma.png'%ant[i])
            
   return x0, y0, sigma, chisq
         
def make_html(dirname):
#now obsolete use oofplot.makeHTML

   #make flickers
   os.system('mkdir %sflicker' %dirname)
   os.system('convert -delay 40 %s/plots/obsbeam-1.png %s/plots/bestfitbeams-0.png %s/flicker/beam1_flick.gif'% (dirname, dirname, dirname))
   os.system('convert -delay 40 %s/plots/obsbeam-2.png %s/plots/bestfitbeams-1.png %s/flicker/beam2_flick.gif'% (dirname, dirname, dirname))
   os.system('convert -delay 40 %s/plots/obsbeam-3.png %s/plots/bestfitbeams-2.png %s/flicker/beam3_flick.gif'% (dirname, dirname, dirname))

   #make flicker html
   outfile=open(dirname+'flicker.html','w')
   outfile.write('<html><head><title>Plots for %s</title></head><body> \n' %(dirname))
   outfile.write('<center> \n')
   outfile.write('<h2>Flicker Plots for %s </h2>\n ' %(dirname))
   outfile.write('</center> \n \n ')
   outfile.write('<a href="flicker/beam1_flick.gif"><img src=flicker/beam1_flick.gif width=25%></a> \n')
   outfile.write('<a href="flicker/beam2_flick.gif"><img src=flicker/beam2_flick.gif width=25%></a> \n')
   outfile.write('<a href="flicker/beam3_flick.gif"><img src=flicker/beam3_flick.gif width=25%></a> \n')
   outfile.write('</body> \n ')
   outfile.write(' </html>  \n ')
   outfile.close()

   # make plots.html
   outfile=open(dirname+'plots.html','w')
   outfile.write('<html><head><title>Plots for %s</title></head><body> \n' %(dirname))
   outfile.write('<center> \n')
   outfile.write('<h2>Plots for %s </h2>\n ' %(dirname))
   outfile.write('</center> \n \n ')
   outfile.write('<p> \n <table border=1, cellpadding=5> \n')
   outfile.write('<tr><td> Aperture amplitude </td><td> Aperture phase </td> </tr>\n' )
   outfile.write('<tr><td><a href="plots/aperture-amplitude.png"><img src="plots/aperture-amplitude.png" width=300></a></td>\n <td><a href="plots/aperture-phase.png"><img src="plots/aperture-phase.png" width=300></a></td></tr> \n ')
   outfile.write(' </table>\n ')
   outfile.write('<p> \n <table border=1, cellpadding=0> \n')
   outfile.write('<tr><td> Observed beam 1 </td><td> Observed beam 2 </td> <td> Observed beam 3 </td> \n <td> best fit beams 1 </td> <td>  best fit beams 2 </td>  <td>  best fit beams 3  </td>  <td> Perfect beams 1</td>  <td> Perfect beams 2 </td>  <td> Perfect beams 3  </td> </tr>\n' )
   outfile.write('<tr><td><a href="plots/obsbeam-1.png"><img src="plots/obsbeam-1.png" width=150></a></td> \n <td><a href="plots/obsbeam-2.png"><img src="plots/obsbeam-2.png" width=150></a></td> \n<td><a href="plots/obsbeam-3.png"><img src="plots/obsbeam-3.png" width=150></a></td><td><a href="plots/bestfitbeams-0.png"><img src="plots/bestfitbeams-0.png" width=150></a></td> \n <td><a href="plots/bestfitbeams-1.png"><img src="plots/bestfitbeams-1.png" width=150></a></td> \n<td><a href="plots/bestfitbeams-2.png"><img src="plots/bestfitbeams-2.png" width=150></a></td> \n <td><a href="plots/perfectbeams-0.png"><img src="plots/perfectbeams-0.png" width=150></a></td> \n <td><a href="plots/perfectbeams-1.png"><img src="plots/perfectbeams-1.png" width=150></a></td> \n<td><a href="plots/perfectbeams-2.png"><img src="plots/perfectbeams-2.png" width=150></a></td></tr> \n ')
   outfile.write(' </table> <p>\n ')
   
   outfile.write('<a href="flicker.html">Flicker plots between observed beams and best fit beams</a> \n')
   outfile.write('</body> \n ')
   outfile.write(' </html>  \n ')
   outfile.close()
   
   return

