#!/usr/bin/env python
#
import analysisUtils as aU
import sys
from itertools import combinations
from pylab import *
from scipy import interpolate
import write_single_day_pager as w

def calibrate(vis):
    """
    This has to be done by hand right now due to  interaciton between
    pylab and split.
    This part calibrates out atmospheric variations
    
    """
    if 1:
        split(vis=vis,outputvis='%s.split' %vis, spw='1,3,5,7',datacolumn='data')
    
        # do gaincal and bandpass cal:
        refant=aU.getAntennaNames(vis)[0]
     
        # first get the time-dependent phase:
        print 'initial phase correction'
        gaincal(vis='%s.split' % vis,caltable='p_time.%s.split.gcal' %vis,spw='0~3:8~120',refant=refant,calmode='p',solint='10s',minsnr=2.0)
     
        # next apply the above and get the bandpass:
        print 'bandpass correction'
        bandpass(vis='%s.split' %vis,caltable='ap_freq.%s.split.bcal' %vis,spw='0~3:8~120',refant=refant,solint='inf',gaintable=['p_time.%s.split.gcal' % vis])

        
        # next apply the bandpass and get the time-dependent phase:
        #print 'phase correction',vis
        #    gaincal(vis='%s.split' % vis,caltable='p_time_freq.%s.split.gcal' %vis,spw='0~3:8~120',refant=refant,combine='',calmode='p',solint='int',minsnr=0,gaintable=['ap_freq.%s.split.bcal' % vis])
            
        # finally apply the bandpass and time-dependent phase:
        print 'applying corrections'
        applycal(vis='%s.split' %vis,calwt=F,gaintable=['p_time.%s.split.gcal' % vis,'ap_freq.%s.split.bcal'   % vis])
               
    

def main(vis):

    if 1:
        antennas = aU.getAntennaNames(vis)
        nants=size(antennas)
        res=zeros([25,2,4,nants,nants])
        res_err=zeros([25,2,4,nants,nants])
        col=zeros([nants,nants])
        for offset in range(25):
            for k in [0] : # spw list
                for ant1,ant2 in combinations(antennas,2):
                    i=aU.getAntennaIndex(vis,ant1)
                    j=aU.getAntennaIndex(vis,ant2)
                    print nants,i,j
                    data = aU.Visibility(vis,antenna1=ant1,antenna2=ant2,spwID=k,scan=1, correctedData=T)
                    dataWal =  aU.Visibility(vis,antenna1=ant1,antenna2=ant2,spwID=k,scan=2+offset,correctedData=T)
                    n1 = double(size(dataWal.specData[0,4:120,:]))
                    n2 = double(size(data.specData[0,4:120,:]))
                    wal0_amp = abs(dataWal.specData[0,4:120,:].mean())
                    wal1_amp = abs(dataWal.specData[1,4:120,:].mean())
                    wal0_rms = abs(dataWal.specData[0,4:120,:].std())
                    wal1_rms = abs(dataWal.specData[1,4:120,:].std())
                    n2 = double(size(data.specData[0,4:120,:]))
                    norm0_amp = abs(data.specData[0,4:120,:].mean())
                    norm1_amp = abs(data.specData[1,4:120,:].mean())
                    norm0_rms = abs(data.specData[0,4:120,:].std())
                    norm1_rms = abs(data.specData[1,4:120,:].std())
            
                    norm0_snr = (norm0_amp/norm0_rms)
                    norm0_snr_err = sqrt((1/n2)+ (norm0_amp/n2)**2)
                    norm1_snr = (norm1_amp/norm1_rms)
                    norm1_snr_err = sqrt((1/n2)+ (norm1_amp/n2)**2)
                    wal0_snr = (wal0_amp/wal0_rms)
                    wal0_snr_err = sqrt((1/n1)+ (wal0_amp/n1)**2)
                    wal1_snr = (wal1_amp/wal1_rms)
                    wal1_snr_err = sqrt((1/n1)+ (wal1_amp/n1)**2)
                    
                    print "NOWalsh snr: %1.3f %3.3f, %3.3f %3.3f" %(norm0_snr, norm1_snr, norm0_snr_err, norm1_snr_err)
                    print "  Walsh snr: %1.3f %3.3f, %3.3f %3.3f" %(wal0_snr, wal1_snr,wal0_snr_err,wal1_snr_err)
                    loss0=(wal0_snr)/(norm0_snr)
                    res[offset,0,k,i,j]=loss0
                    loss0_err = loss0 * sqrt( (wal0_snr_err/wal0_snr)**2 + (norm0_snr_err/norm0_snr)**2)
                    res_err[offset,0,k,i,j]=loss0_err
                    loss1=(wal1_snr)/(norm1_snr)
                    res[offset,1,k,i,j]=loss1
                    loss1_err = loss1 * sqrt( (wal1_snr_err/wal1_snr)**2 + (norm1_snr_err/norm1_snr)**2)
                    res_err[offset,1,k,i,j]=loss1_err
                    
                    print "W/NW loss: Antenna %s-%s, SpwID %i: %1.3f +-%1.4f,  %1.3f +- %1.4f"% (ant1,ant2,k,loss0,loss0_err, loss1, loss1_err)
     
        return res, res_err


def check(a):
    mask =  tri(a.shape[0], k=0)
    b = ma.array(a, mask=mask)
    #b=b.reshape([size(a)])
    return b


def plot_specialresult(vis, res, res_err):

    worder = [0,0,100,105,110,115,120,125,125,125,125] # Walsh_onoff_5

    antennas = aU.getAntennaNames(vis)
    nants=len(antennas)
    vm=aU.ValueMapping(vis)
    sh=shape(res[0,0,:,:])
    djet=cmap_discretize(cm.cool,16)
    bl=0
    
    x=arange(25)*4 - 10
    x[0]=-6

    figure(1,figsize=(12,9));clf()
    for ant1,ant2 in combinations(antennas,2):
        i=aU.getAntennaIndex(vis,ant1)
        j=aU.getAntennaIndex(vis,ant2)
        print nants,i,j
        bl = bl+1
        clf()
        errorbar(x,res[:,0,0,i,j],yerr=res_err[:,0,0,i,j],fmt='o');
        xlabel('Delay offset between FLOOG and DTX  in us (0 = 12ms)');ylabel('Coherence loss between Walsh off and Walsh on')
        ylim([0,1.2]);xlim([0,90]);
        suptitle('Loss of Coherence for BB_1 %s-%s Worder:%i-%i ' %(ant1,ant2,worder[i],worder[j]))
        savefig('%s_coherence_loss_vs_floog_dtx_offset_%s-%s.png'%(vis.split('.ms')[0],ant1,ant2))

def makepager():
    field={}
    field['switch']=[]
    field['names']=[]
    field['names']=[['DV02', 'DV03', 'DV04', 'DV05','DV06','DV07','DV09', 'DV12','DV13','PM02','PM03'],['DV02', 'DV03', 'DV04', 'DV05','DV06','DV09', 'DV12','DV13','PM02','PM03']]
    field['switch']=[[1,1,1,1,1,1,1,1,1,1,1],[1,1,1,1,1,1,1,1,1,1,1]]
    tags=['walsh_onoff_5_coherence_loss_vs_floog_dtx_offset']
    w.write_single_day_pager('../index.html',tags,field,0,1,'png');
      

def plot_result(vis, res, res_err):

    #worder = [1,2,4,5,6,7,11,12,16,13,14,15] # for Walsh_onoff_1  part 1
    #worder = [1,2,64,65,66,67,68,69,16,13,14,15] # for Walsh_onoff_1  part 2
    # worder = [0,0,1,2,3,4,5,6,7,8,9] # for  Walsh_onoff_3
    # worder = [0,10,20,30,40,50,60,70,80,90,100 ] # Walsh_onoff_2
    worder = [0,0,100,105,110,115,120,125,125,125,125] # Walsh_onoff_4
    
    vm=aU.ValueMapping(vis)
    sh=shape(res[0,0,:,:])
    djet=cmap_discretize(cm.cool,16)

    figure(1,figsize=(12,9));clf()
    for k in range(4):
        subplot(2,2,k+1)
        for i in range(sh[0]-1):
                    
            ant1=vm.getAntennaNamesForAntennaId(i)
            worder1=worder[i]
            #col=djet(i/float(sh[0]-1))
            
            x=check(res[0,k,:,:])
            x=x[i,:]
            worder2=[];deltaOrder=[]
            for j in arange(i+1,sh[0]):worder2.append(worder[j])
            for j in range(size(worder2)): deltaOrder.append(abs(worder2[j]-worder1))
            cc=array(deltaOrder)/69.
            col=cm.cool(cc)
            if i == 0:
                ddsave=deltaOrder
            y=check(res[1,k,:,:])
            y=y[i,:]
            scatter(x,y,c=cc,marker='o', s=40,label=ant1,cmap=cm.cool)
            hold(True)
            xlabel('Pol 0');ylabel('Pol 1')
        xlim([0,1]);ylim([0,1]);
        title('SPW %i' %k)

    suptitle('w vs w/o Walsh ratio of SNR of crosscorr amp')
    #legend(loc=4)
    savefig('%s_color.png'%vis.split('.ms')[0])


    figure(2,figsize=(12,9));clf()
    for k in range(4):
        subplot(2,2,k+1)
        
        for i in range(sh[0]-1):
            x=check(res[0,k,:,:]).reshape(size(res[0,k,:,:]))
            y=check(res[1,k,:,:]).reshape(size(res[1,k,:,:]))
            xerr=check(res_err[0,k,:,:]).reshape(size(res_err[0,k,:,:]))
            yerr=check(res_err[1,k,:,:]).reshape(size(res_err[1,k,:,:]))
            errorbar(x,y,xerr=xerr, yerr=yerr,fmt='o')
            xlabel('Pol 0');ylabel('Pol 1')
        title('SPW %i' %k)
        xlim([0,1]);ylim([0,1]);
    suptitle('w vs w/o Walsh ratio of SNR of x-amp for spw %i' %k)
    savefig('%s_error.png'%vis.split('.ms')[0])
            
    figure(3,figsize=(12,9));clf()
    for k in range(4):
        for i in range(sh[0]-1):
            ant1=vm.getAntennaNamesForAntennaId(i)
            worder1=worder[i]
            x=check(res[0,k,:,:])
            x=x[i,:]
            worder2=[];deltaOrder=[]
            for j in arange(i,sh[0]):worder2.append(worder[j])
            for j in range(size(worder2)): deltaOrder.append(abs(worder2[j]-worder1))
            y=check(res[1,k,:,:])
            y=y[i,:]
            xy=(x+y)/2.
            scatter(deltaOrder,xy[i:],marker='o', s=20)
            hold(True)
            xlabel('WalshOrder difference');ylabel('SNR ')
        title('w vs w/o Walsh ratio of SNR of crosscorr amp vs Walsh Order difference')
    savefig('%s_color_dworder.png'%vis.split('.ms')[0])


def cmap_discretize(cmap, N):
    """Return a discrete colormap from the continuous colormap cmap.
    
        cmap: colormap instance, eg. cm.jet. 
        N: Number of colors.
    
    Example
        x = resize(arange(100), (5,100))
        djet = cmap_discretize(cm.jet, 5)
        imshow(x, cmap=djet)
    """

    cdict = cmap._segmentdata.copy()
    # N colors
    colors_i = linspace(0,1.,N)
    # N+1 indices
    indices = linspace(0,1.,N+1)
    for key in ('red','green','blue'):
        # Find the N colors
        D = array(cdict[key])
        I = interpolate.interp1d(D[:,0], D[:,1])
        colors = I(colors_i)
        # Place these colors at the correct indices.
        A = zeros((N+1,3), float)
        A[:,0] = indices
        A[1:,1] = colors
        A[:-1,2] = colors
        # Create a tuple for the dictionary.
        L = []
        for l in A:
            L.append(tuple(l))
        cdict[key] = tuple(L)
    # Return colormap object.
    return matplotlib.colors.LinearSegmentedColormap('colormap',cdict,1024)
