"""
import fomalont_grid as rc
cal = rc.Calsurvey_reduction()
cal.init('uid___A002_X41b513_X36',reducer='E.Fomalont',zband='7',Type='Grid',zbandpass='J1924-292',zImage=True, zrestart = True)
cal.dorun()
"""
import sys
from datetime import datetime
from xmlrpclib import ServerProxy
from pylab import *
import numpy
import time
import os
import glob
from casa import *
import scipy as sp
#import coords as co
import re
import math
from taskinit import *
# import tmUtils as tm
sys.path.insert(0,'/users/rkneissl/AIV/science/analysis_scripts')
# in cville use:
#sys.path.insert(0,'/users/thunter/AIV/science/analysis_scripts')
import analysisUtils as aU
from gencal_cli import gencal_cli as gencal 

#==================================================#
#  Basic Parameters
class Calsurvey_reduction:
        def init(self, asdmName, reducer = None, Type = 'Grid', zsaveplo=True, zrestart=True, zref=None, zbandpass = None, zflux = 1.0,zuseplanet = True, ssf=[1.0,1.0,1.0,1.0], zImage = True, zWVR = True, zflagWVR = [''], zFieldIdWA = 0, zminbl = 4, zplanet = None, zsidebands = False):
		"""Calsurvey Reduction Script
		==========================
		modified: Sep 24, 2011.  Version 1_0   E Fomalont
		Goal:Set up an automatic script to determine flux densities of possible calibrator sources

		modified:Oct 10, 2011.  Version 1_1  E.Fomalont
		Determine spectral windows of average data set.   Did not have
		to be 0,1,2,3.

		modified: Oct 22, 2011 Version 1_2 T. van Kempen
		making it command line only

		modified: Jan 19, 2012 Version 1_3 T. van Kempen
		added option to do WVR

		modified: May 27, 2012 Version 1_4 T. Fomalont
		check zbandpass, add good image parameters, fix refant,
                    add two output text files for
                     low, mid, high resolution results
                     flux results for each antenna for editing
                
                modified: Jun 21, 2012 Version 1_5 N. Marcelino
                only one output directory (containing text files, diagnostic figures and images), 
                choose band automatically, ref_ant fixed, and other small modifications..

		modified: Sep 25, 2015 Version 1 R Kneissl
		compatible with CASA 4.4.0 and new structure of observing sequence
		==========================

		0. Initialization
		*Required parameters*:
                asdmName          -- Name of the ASDM 
		reducer = NAME    -- Name of the reducer
        	[zband = None      -- observed band, this parameter is needed in all reductions] --not needed now auto
                Type = 'Grid'     -- Calsurvey type: Grid/Cone/ACA
                
                zbandpass = None  -- Must be provided if intents not properly filled in.  This is the quasar that was observed multiple time.
                zref = ''         -- antenna name: Will use chosen reference antenna. Empty standard.
                zWVR = True       -- Include WVR in reduction
                zflagWVR = ['']   -- antennas to flag in WVR. example: ['DV03','DA05','PM02'] 
                zdisplay = False  -- Keep this False.  Does not work well.
	        zsaveplo = True   --  T/F save plots during execution (recommented)
                zImage = True     -- To clean and produce images of all sources
		zrestart = True   --  T/F restart completely from beginning
		zuseplanet = True-- if solar system object is not available in cone, F if not
	        zflux = 1.0       -- Flux density of zbandpass.
	                          -- This is only needed if no solar system object is provided.  It will be assumed 1.0 is unspecified and no ss object is observed.
		zsidebands = False -- If True, give results for both sidebands also in bands higher than band 3 
	        """
		import os
		import sys
		from importasdm import importasdm
		from flagmanager import flagmanager
		from flagdata import flagdata
                #from tflagdata import tflagdata
                from flagcmd import flagcmd
		from clearcal import clearcal
		#from taskinit import *
		from listobs import listobs

		print 'Start'
		zvis = asdmName+'.ms'
		#  Can start from asdm if you want
		if os.path.isdir(zvis+'.orig') == True:
			print "Orig File found. Moving .orig file back to MS file"
			if os.path.isdir(zvis) == True: os.system('rm -r -f '+zvis)
			os.system('mv '+zvis+'.orig '+zvis)
		if os.path.isdir(zvis) == False:
			print "No MS found, only ASDM. Please wait while we convert the asdm to MS"
			importasdm(asdm=asdmName,asis='*',useversion='v3')
		self.asdmName = asdmName
		self.reducer = reducer
                #self.zband = zband
		self.zdisplay = False    # Keep F
		self.zsaveplo = zsaveplo    # Keep T for useful plots
		self.zrestart = zrestart    # Set to T after first attemps
		self.zuseplanet = zuseplanet   #  There is a good solar system object for the flux scale
                self.ssf = ssf  # array of factors to correct for line emission/absorption in the ss object, one per spw. Default is [1.0,1.0,1.0,1.0]
		self.zbandpass = zbandpass   # Almost always needed
		self.Type = Type
		self.zflux = zflux   #  Only needed if no ss object is available.
                self.flagdataFile = asdmName+'.flagdata'
		self.zvis = zvis
		self.zplit = zsplit = asdmName + '_split.ms'
		self.tsysfile = tsysfile = asdmName+'.tsys'
		self.zref = zref
		self.zWVR = zWVR
		self.flagWVR = zflagWVR
		self.zImage = zImage
		self.zFieldIdWA = zFieldIdWA  # Temporary option to deal with CSV-600 / ICT-833/4
		self.minbl = zminbl
		if zplanet is None:
			self.zplanet = zplanet
		else:
			self.zplanet = str.split(zplanet,',') #specify the SSO for amplitude calibration
		self.zsidebands = zsidebands
		self.nanteff=[]
		#self.zusescratch=zusescratch
		if self.Type == 'Science':
			print '*** Warning: Science chosen, are you sure? ***'
                self.getSources(zvis,zbandpass,self.zplanet)

		#  Output of initiation:
		print '**********************************************'
		print '      INPUT INFORMATION '
		print ' asdmName          = ', self.asdmName
		if reducer != None: print ' Reduced by person = ', self.reducer
		else:
			print "No reducer determined, you must put in your name"
			sys.exit()
                
                self.getSpecInfo(zvis)
                #if self.zband != None:
                print ' Band = ', self.zband
                #else:
                #        print ' Please, select the ALMA band used'
                #        sys.exit()
                print ' Type    = ',self.Type
                if self.Type != 'Cone':
                     if self.Type != 'Grid':
                         if self.Type != 'ACA':
                               if Type != 'Science':
                                        # if Type != 'Asteroid':
                                        #print ' ******* INCORRECT TYPE SET: ASTEROID REDUCTION IN PREP *****'
                                        print ' ******* INCORRECT TYPE SET *****'
                                        sys.exit()
		if self.zref != None:
			print ' Forcing refant   = ', self.zref
		else:
			print ' No Reference Antenna declared, will be selected in the script.'
			#print ' Using first antenna in array as reference'
		#
                if self.Type == 'ACA': self.zWVR = False
                if self.zWVR == True:
			print ' Including WVR in reduction'
			print ' Antenna flagged for WVR = ', self.flagWVR
		else:
			print ' Not including WVR in reduction'
		#
                if os.path.isfile(self.flagdataFile):
			print ' Flagdata file = ', self.flagdataFile
		else: 
                        print ' No Flagdata file found.' 
                        print ' ***** If you want custom flags, please put them in a file called ', self.flagdataFile
		if zsaveplo:
			print ' Saving calibration and visibility plots '
		else:
			print ' Not saving plots'
                if zImage:
                        print ' Producing and saving clean images '
                else:
                        print ' Not imaging the sources'
		#if self.Type == 'Cone':
		#	if zbandpass !=None:
		#		print ' Forcing bpass/cal= ', zbandpass
		#	else:
		#		listobs(vis=self.zvis)
		#		print ' ****** ERROR *****'
		#		print ' You selected a CONE reduction but no bandpass defined with zbandpass = XXXX !'
		#		print ' ****** Please pick one out  of : '
		#		print self.sourcesList
		#		sys.exit()
		#if self.Type == 'Grid':
                if zbandpass !=None:
                        print ' Forcing bpass/cal= ', zbandpass
                else:
		        print ' Bandpass chosen from intents'
                        print ' ****** PLEASE, make sure there is a bandpass intent in the list'
                    
		if self.zuseplanet==False:
                        print ' No planet present.  Use bandpass as flux calibrator instead with flux = ', self.zflux, ' Jy'
                        print ' ****** PLEASE, make sure you have selected a bandpass and put in its last measured flux.'
                
		if zrestart:
			print ' Restart from scratch. Removing all flagfiles and split files.'
			print ' ------------'
			flagmanager(vis = zvis, mode = 'restore', versionname = 'original')
			clearcal(vis=zvis)
			#os.system('rm -rf *avg.ms *split.ms *ms.flagversions')
                        os.system('rm -rf *avg.ms* *split.ms*')
			os.system('rm -rf *cal_* *kcross *.orig *ble.flagversions')
			os.system('rm -rf flux_scale.tab final_data.ms ant_flux_check')
			os.system('mkdir old_fluxes')
			os.system('mv ALMA* old_fluxes/')
			os.system('rm -rf *.tsys *.wvr')
                        #os.system('rm -rf output_figs images')
                        os.system('rm -rf output_files_prev/')
                        os.system('mv output_files/ output_files_prev')
			print ' Removed all previous flags except flagdata command defined in file ',self.flagdataFile
			print ' ------------'
	       	else:
			print ' Not doing restart, starting from existing flags'
	        
                os.system('mkdir output_files')
                if zsaveplo:
                        os.system('mkdir output_files/diag_figs')
                
                if zImage:
                        os.system('mkdir output_files/images')
                
		listobs(self.zvis)
                print '**********************************************'
                print ' SETUP COMPLETE'
                print ' Observation details are shown in the Log, please take a look.'
                print ' If you are satisfied with the setup and want to proceed, please run cal.dorun()'
		print '**********************************************'


       	#=================================================#
	# subroutine 1 : Initialization
	#
	# get sources and flux calibrators
	def getSources(self,zvis,zbandpass,zplanet):
		vm = aU.ValueMapping(zvis)
		fieldsList_raw = vm.uniqueFields
		fieldsList = []

		seen = set()
		for i in fieldsList_raw:
			if i not in seen:
				fieldsList.append(i)
				seen.add(i)
		fieldIds = vm.fields
                #vm.getIntentsforFieldIds
		print 'fieldsList', fieldsList, 'fieldIds', fieldIds

		self.moons = ['Titan','Callisto','Ganymede']
		for i in self.moons:
			if i in fieldsList:
				self.zFieldIdWA = 0
		msmd.open(zvis)
		# now we get ONLY those that have an ID
		quasarIds = []
		sourcesIds = []
		sourcesList = []
		for i in range (0,len(fieldsList)):
			zIds = vm.getFieldIdsForFieldName(fieldsList[i])
			#modified for FieldNames with multiple FieldIds where the first FieldId gets flagged - RK 20130607
			for j in zIds:
				#print j,fieldsList[i],msmd.intentsforfield(field=j)
				if j in fieldIds:
					if ('CALIBRATE_PHASE#ON_SOURCE') or ('CALIBRATE_BANDPASS#ON_SOURCE') or ('CALIBRATE_FLUX#ON_SOURCE') in msmd.intentsforfield(field=j):
						sourcesIds.append(j)
						sourcesList.append(fieldsList[i])
		msmd.close()
                print 'sourceList',sourcesList,'sourceIds', sourcesIds

		#  Useable planets
		self.possibleFluxCalNames = ['Uranus','Neptune','Titan','Callisto','Ganymede','Mars','Pallas','Vesta','Ceres']
		self.UvlimkL = [80,40,120,150,150,40,370,630,60]
		# the following are either not reliable as fluxCal (Mercury) or model unknown (Juno)
		otherSSObj = ['Juno','Mercury','Venus']
                self.fluxCalNames = [i for i in self.possibleFluxCalNames if i in sourcesList]
		if self.zplanet != None:
			for i in self.fluxCalNames:
				if i not in self.zplanet:
					#print "no flagging"
					flagdata(vis=zvis,flagbackup=False,field=i)
		if len(self.fluxCalNames) > 1:
			if self.zplanet == None:
				self.fluxCalNames = [self.fluxCalNames[0]]
				print self.fluxCalNames
			else:
				self.fluxCalNames = self.zplanet
		self.otherSSNames = [i for i in otherSSObj if i in sourcesList]
		self.planetNames = self.fluxCalNames + self.otherSSNames
                
                self.quasarNames = [i for i in sourcesList if i not in self.planetNames]
		for i in range (0,len(self.quasarNames)):
			qIds = vm.getFieldIdsForFieldName(self.quasarNames[i])[0]

			if qIds in fieldIds:
				quasarIds.append(qIds)

                self.quasarIds = quasarIds
                self.nquasars = len(self.quasarIds)
                print 'quasars', self.quasarNames, self.quasarIds                

                if self.zbandpass !=None:
			if [i for i in self.quasarNames if i in self.zbandpass] == []:
				print 'bandpass name not found in quasar list'
				sys.exit()

                if self.Type == 'Science':
			print 'Starting Science part'
			zIntents = vm.getFieldsForIntent('OBSERVE_TARGET#ON_SOURCE',True)
			print 'Done intents'
			for j in range(0,len(zIntents)):
				print j
				scienceIds = vm.getFieldIdsForFieldName(zIntents[j])
				print scienceIds
				if zIds in scienceIds:
					print 'Removing Source ', fieldsList[i],' due to being science source'
					sourcesList.remove(zIds)
					   
		self.sourcesList = sourcesList
                self.sourcesIds = sourcesIds
		return 

	#==================================================#
	# put the models for the planets
	def putModel(self,zvis):
		from fixplanets import fixplanets
		for i in range (0,len(self.fluxCalNames)):
			fluxCalName = self.fluxCalNames[i]
			#COMMENTED AS FIXPLANETS DO NOT NEED TO BE USED NOW
			#fixplanets(vis = zvis, field = fluxCalName, fixuvw = True)
			setjy(vis = zvis, field = fluxCalName, standard = 'Butler-JPL-Horizons 2012', usescratch = True, useephemdir = True)
			print '******* fixed uvw and inserted model for ', fluxCalName
		#

	#==================================================#
	# get bandpass calibrator
	def getBpCal(self,zsplit):
		vm = aU.ValueMapping(zsplit)
		BandpassName = vm.getFieldsForIntent('CALIBRATE_BANDPASS#ON_SOURCE',True)[0]
                BandpassID = vm.getFieldIdsForFieldName(BandpassName)
		self.BandpassName = BandpassName
                self.BandpassID = str(BandpassID[0])

		return 

        #==================================================#
        # get spectral information
        def getSpecInfo(self,zvis):
                # select science swps by number of channels (64 12m OR 120 aca)
                # pointings seem to have 128 channels, always the case??
                vm = aU.ValueMapping(zvis)
                spwInfo = vm.spwInfo
                dspw = []
                nchan = []
                self.zfreq = []
                for spwId in spwInfo.keys():
                        dfreq = spwInfo[spwId]['meanFreq'] / 1.0E9
                        if spwInfo[spwId]['numChannels'] == 64 or spwInfo[spwId]['numChannels'] == 120:
                                dspw.append(str(spwId))
                                nchan.append(spwInfo[spwId]['numChannels'])
                                self.zfreq.append(dfreq)
                        #
                #
                self.nspw = len(dspw)
                self.nchan = sorted(dict.fromkeys(nchan).keys())[0]
                self.dspw = ','.join(dspw)
                #
                if 84.0 < self.zfreq[0] < 116.0: self.zband = '3'
		if 125.0 < self.zfreq[0] < 163.0: self.zband = '4'
                if 211.0 < self.zfreq[0] < 275.0: self.zband = '6'
                if 275.0 < self.zfreq[0] < 373.0: self.zband = '7'
		if 385.0 < self.zfreq[0] < 500.0: self.zband = '8'
                if 602.0 < self.zfreq[0] < 720.0: self.zband = '9'
		if 787.0 < self.zfreq[0] < 950.0: self.zband = '10'
                
                return

	#==================================================#
	# Choose reference antenna
	#
	# Input is zvis, the ms
	#
	# Reference antenna chosen with properties
        #     Shortest spacing min_sep (30m) to another antenna
        #     No large amount of flagging (<10% more than best antenna)
        #     Nearest geometric center of array
	def get_reference(self,zvis):
                if self.Type == 'ACA':
                    min_sep = 7.0  # Minimum separation is 10 m for 7m antennas
                else: 
                    min_sep = 25.0  # Minimum separation is 25 m for 12m antennas
		minspacing = []
		# open the antenna table for our data set:
		tb.open(zvis+'/ANTENNA')
		ant_name = tb.getcol('NAME')
		nant = ant_name.shape[0]
		#nant = len(ant_name)
		# the antenna position
		ant_pos = tb.getcol('POSITION')
		# close the table
		tb.close()
		#  Get minimum separation for each antenna from another
                for i1 in range (0,nant):
                    minsep = 1.0e15
                    for i2 in range(0,nant):
	                if i1 != i2: 
                            antsep = (ant_pos[0,i1]-ant_pos[0,i2])**2 + (ant_pos[1,i1]-ant_pos[1,i2])**2 + (ant_pos[2,i1]-ant_pos[2,i2])**2
                            minsep = numpy.minimum(antsep,minsep)
                        #
                    #
	            minspacing.append(sqrt(minsep))     
                #
		# This syntax points the flagging tool at our data set
		fg.clearflagselection(0)
		fg.open(zvis)
		fg.setdata()
		fg.setflagsummary()
		#   get a dictionary of flagging statistics
		flag_statistics = fg.run()
		fg.done()
		# this is a dictionary holding flagging statistics sliced many ways
		#print flag_statistics.keys()
		# What we care about is the fraction of data flagged for each
		# antenna. Let's work that out.
		# Initialize an array matched to the offset and name arrays
		frac_flagged = 0.0*ant_pos[0,]
		# ... loop over antennas (there are other iteration approaches here)
		for i in range(0,nant):
			# ... catch antennas not in the list and note them as totally flagged
			if flag_statistics['antenna'].has_key(ant_name[i].upper()) == False:
				frac_flagged[i] = 1.0
			else:
			        # ... note the flagged visibilities for this antenna
			        flagged = float(flag_statistics['antenna'][ant_name[i].upper()]['flagged'])
			        # ... and the total visibilities
			        total = float(flag_statistics['antenna'][ant_name[i].upper()]['total'])
			        # ... work out the fraction flagged and save it
			        frac_flagged[i] = float(flagged/total)
                        #
                        #print ant_name[i], frac_flagged[i]
		#
		frac_flagged_min = numpy.min(frac_flagged)

		# Now use what we know to suggest a reference antenna
		# ... define a minimum separation
                min_space = numpy.min(minspacing)                
                # define some threshold in flagging (this could be refined, here just use an absolute threshold)
		flag_thresh = 1.1 * frac_flagged_min
                # ... these antennas are not too close together and are not too flagged
                #suggest_ind = ((minspacing > min_sep) & (frac_flagged < flag_thresh) & (minspacing<2*minspacing)).nonzero()
                # ... pick the closest one to the array center (remember that we sorted by offset!)
		#self.ref_ant = ant_name[suggest_ind[0].min()]
                # doing a loop instead..
                ref_ant=[]
                for i in range(0,nant):
                        if (frac_flagged[i] < flag_thresh) and (min_sep < minspacing[i] < 2*min_space):
                                ref_ant.append(ant_name[i])
                        #
                # just pick up the first of that list...
                ref_ant = ref_ant[0]
		print 'Choosing reference antenna= ', ref_ant
                self.nant = nant
                self.ant_name = ant_name
                self.ant_pos = ant_pos

		return ref_ant
	#==================================================#
	# get antenna info
	def getAntInfo(self,zvis):
		tb.open(zvis+'/ANTENNA')
		tb.getcol('NAME')
		ant_name = tb.getcol('NAME')
		nant = tb.nrows()
		ant_pos = tb.getcol('POSITION')
		tb.close()
                self.nant = nant
                self.ant_name = ant_name
                self.ant_pos = ant_pos

                return
        
	#==================================================#
	# get source elevations
	def getSouElev(self,visavg,f):
		vm = aU.ValueMapping(visavg)
		sourceScan = vm.getScansForField('sourcesList')
                #sourceScan = vm.getScansForIntent('CALIBRATE_PHASE#ON_SOURCE')
		zref = self.ref_ant
		for i in range (0,len(sourceScan)):
			conditions = aU.listconditions(visavg,sourceScan[i],zref,verbose=False)
			sourceEl = conditions['elevation']
			sourceName = vm.getFieldsForScan(sourceScan[i])
			if sourceEl > 80 or sourceEl < 20:
				InfoLine='%16s %3d %6.2f %8s \n'% (sourceName[0], sourceScan[i], sourceEl, '***')
				f.write('# '+InfoLine)
			else:
				InfoLine='%16s %3d %6.2f \n'% (sourceName[0], sourceScan[i], sourceEl)
				f.write('# '+InfoLine)
			#
		#
		return 

	#==================================================#
        # get weather info
        def getWeather(self,zvis,f):
                vm = aU.ValueMapping(zvis)
                nscans = vm.scans
                sc1 = nscans[0]
                zref = self.ref_ant
                conditions = aU.listconditions(zvis,sc1,zref,asdm=self.asdmName,verbose=False)
                waterVap = conditions['pwv']
                opacity = conditions['tauzenith']
                wind = conditions['windspeed']
                WeatherLine='%8s %6.2f %8s %6.2f %8s %6.2f \n'% ("pwv (mm)=", waterVap, "tau =", opacity, "wind speed (m/s) =", wind)
                #print WeatherLine
                f.write('# '+WeatherLine)
                
        #==================================================#
	# get observations and reduction dates
	def getDates(self):
		vm = aU.ValueMapping(self.zvis)
		nscans = vm.scans
		sc1 = nscans[0]
		tt = vm.getTimesForScan(sc1)
		tt1 = (tt.max() + tt.min()) / 2.0
		tt1 = ((tt1 / 86400.0) + 2400000.5 - 2440587.5) * 86400.0
		zobsdate = time.strftime('%Y-%m-%d', time.gmtime(tt1))
        
		from datetime import date
		ztoday = date.today()
		ztoday = date.isoformat(ztoday)
        
		return zobsdate,ztoday

	#==================================================#
	# subroutine 2 : tsys
	#  Get and apply tsys's
	def doTsys(self,zvis):

		gencal(vis = zvis, caltype = 'tsys', caltable = zvis[:-2]+'tsys')
		#interTsys = aU.InterpolateTsys(zvis[:-2]+'tsys')
		#interTsys.correctBadTimes()
		#interTsys.assignFieldAndScanToSolution() 
        
		#  Apply to each source 
                from recipes.almahelpers import tsysspwmap
                tsysmap = tsysspwmap(vis = zvis, tsystable = zvis[:-2]+'tsys')

                tb.open(zvis[:-2]+'tsys')
		calFieldIds = sorted(dict.fromkeys(tb.getcol('FIELD_ID')).keys())
		tb.close()

		if self.zband == '3':
			nfid=1
		else:
			nfid=2

		if self.zFieldIdWA == 0:
			for i in calFieldIds:
				applycal(vis = zvis, flagbackup = False, gaintable = (zvis[:-2]+'tsys'), spwmap = [tsysmap], field = str(i), gainfield = str(i))
		else:
			for i in calFieldIds:
					if i>nfid:
						applycal(vis = zvis, flagbackup = False, gaintable = (zvis[:-2]+'tsys'), spwmap = [tsysmap], field = str(i), gainfield = str(i))
					else:
						if i==0:
							applycal(vis = zvis, flagbackup = False, gaintable = (zvis[:-2]+'tsys'), spwmap = [tsysmap], field = str(nfid), gainfield = '0')
		#
                # tsys plots
                if self.zsaveplo == True:
                        aU.plotbandpass(caltable=zvis[:-2]+'tsys', overlay='time', xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False, showatm=True, pwv='auto', chanrange='5~60', field='', figfile='output_files/diag_figs/tsysplots')
                        #for i in range(0,len(self.sourcesList)):
                        #   aU.plotbandpass(caltable=zvis[:-2]+'tsys', overlay='antenna', xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False, showatm=True, pwv='auto', chanrange='5~60', field='', timeranges=str(i), figfile='output_files/diag_figs/tsysplots_'+self.sourcesList[i]+'.png')
                
                #--------------------------------------
                # obtain tsys statistics
                print '******* obtaining Tsys statistics '
                caltable = zvis[:-2]+'tsys'
                #nch_low = 15
                #nch_high = 50
                nch_low = self.nchan*25/100
                nch_high = self.nchan-self.nchan*25/100
                zfileTsys = 'output_files/tsys_stat.txt'
                if os.path.exists (zfileTsys): os.system('rm '+zfileTsys)
                f = open (zfileTsys, 'w')
                print '******* opening file '+zfileTsys
                #
                #  Get average tsys per antenna
                f.write('\n\n')
                f.write('*********************************************************** \n')
                f.write('\n')
                f.write('                 MEDIAN TSYS WITH ANTENNA \n')
                f.write('\n')
                f.write('   ANT #      NAME     Median Tsys   \n')
                #
                tsys_ant = []
                for i in range(0,self.nant):
                    gain_ch = []
                    npts = 0
                    tb.open(caltable)
                    tb1 = tb.query('ANTENNA1 == '+str(i)+'')
                    gain = tb1.getcol('FPARAM')
                    nstream = gain.shape[2]
                    for j in range (0,nstream):
                        for k in range(nch_low, nch_high):
                            gain_ch.append(gain[0,k,j])
                            gain_ch.append(gain[1,k,j])
                            npts = npts + 1
                        #
                    #
                    if npts > 30:
                        tsys_ant.append(numpy.median(gain_ch))
                    else:
                        tsys_ant = 0.0
                    #
                #
                tsys_med = numpy.median(tsys_ant)
                tsys_out1 = []
                for i in range(0,self.nant):
                    tsys_diff = numpy.abs(tsys_ant[i]-tsys_med)
                    if tsys_diff > 3*tsys_med: 
                      tsys_out1.append('*')
                    else:
                      tsys_out1.append(' ')
                    #
                    f.write('    %3d     %6s    %7.1f    %3s \n' %(i, self.ant_name[i], tsys_ant[i], tsys_out1[i]))
                #
                #
                #  Get average tsys per each scan/elevation
                vm = aU.ValueMapping(zvis)
                #tsys_scan = vm.getScansForIntent('CALIBRATE_ATMOSPHERE#REFERENCE')
                tsys_scan = vm.getScansForIntent('CALIBRATE_ATMOSPHERE#OFF_SOURCE')
                TX_save = []
                TY_save = []
                elev_save = []
                for i in range(0,len(tsys_scan)):
                    tsys_x = []
                    tsys_y = []
                    tsc = str(tsys_scan[i])
                    tb.open(caltable)
                    tb2 = tb.query('SCAN_NUMBER == '+tsc+'')
                    gain = tb2.getcol('FPARAM')
                    nstream = gain.shape[2]
                    for j in range (0,nstream):
                        for k in range(nch_low, nch_high):
                            tsys_x.append(gain[0,k,j])
                            tsys_y.append(gain[1,k,j])
                        #
                    #
                    T_x = numpy.median(tsys_x)
                    T_y = numpy.median(tsys_y)
                    TX_save.append(T_x)
                    TY_save.append(T_y)
                    conditions = aU.listconditions(vis=zvis,byscan=True,scan=tsc,antenna='0',verbose=False)
                    elev_save.append(conditions['elevation'])
                #
                f.write('\n\n')
                f.write('*********************************************************** \n')
                f.write('\n')
                f.write('              MEDIAN TSYS WITH SOURCE/ELEVATION \n')
                f.write('\n')
                f.write('Scan   Fid                Source      Elev    Median T_x  Median T_y \n')
                #
                TX_med = numpy.median(TX_save)
                TY_med = numpy.median(TY_save)
                tsys_out2 = []
                for i in range(0,len(tsys_scan)):
                    tsc = str(tsys_scan[i])
                    sfield = vm.getFieldsForScan(tsc)
                    fid = vm.getFieldIdsForFieldName(sfield)
                    TX_diff = numpy.abs(TX_save[i]-TX_med)
                    TY_diff = numpy.abs(TY_save[i]-TY_med)
                    if (TX_diff > 3*TX_med) or (TY_diff > 3*TY_med): 
                      tsys_out2.append('*')
                    else:
                      tsys_out2.append(' ')
                    #
                    f.write('%4d  %4d  %24s  %6.1f   %7.1f   %7.1f  %3s \n'% (tsys_scan[i], fid[0], sfield[0], elev_save[i], TX_save[i], TY_save[i], tsys_out2[i]))
                #
                f.close()
                tb.close()
        
	#=================================================#
	# subroutine 2a: WVR
	# Get and apply WVR

	def doWVR(self,zvis):
                from wvrgcal import wvrgcal
                xvis = zvis+'.orig'
		os.system('mv '+zvis+' '+xvis)
		os.system('mv '+zvis+'.flagversions'+' '+xvis+'.flagversions')
                if self.flagWVR == None:
			print '**** No antennas flagged ****'
                        #os.system('wvrgcal --ms '+xvis+' --output '+zvis+'.wvr')
                        # in cville use:
                        #os.system('/users/thunter/usr/wvrgcal-latest/bin/wvrgcal --ms '+xvis+' --output '+zvis+'.wvr')
		        wvrgcal(vis = xvis, caltable = zvis+'.wvr')
                else:
		#	print '**** antennas flagged ****'
                        #os.system('wvrgcal --ms '+xvis+' --output '+zvis+'.wvr --wvrflag '+self.flagWVR)
                        # in cville use:
			#os.system('/users/thunter/usr/wvrgcal-latest/bin/wvrgcal --ms '+xvis+' --output '+zvis+'.wvr --wvrflag '+self.flagWVR)
                        wvrgcal(vis = xvis, caltable = zvis+'.wvr', wvrflag = self.flagWVR)
                
                aU.plotWVRSolutions(caltable=zvis+'.wvr', spw=self.dspw[0], field='', subplot=22, interactive=False, sort='number', antenna=self.ref_ant, xrange=[0,0], yrange=[0,0], figfile='output_files/diag_figs/wvrsolutions', xaxis='ut')
                
                applycal(vis = xvis, flagbackup = False, gaintable = (zvis+'.wvr'))
		split(vis = xvis, datacolumn = 'corrected', outputvis = zvis)

	#==================================================#
	# subroutine 3 : Apriori flagging
	def aprioriFlag(self,zvis):
                from flagmanager import flagmanager
		from flagdata import flagdata
                #from tflagdata import tflagdata
                from flagcmd import flagcmd
		#  Make sure planet models are in to get the planets coordinates before flagging for shadowing
		flagmanager(vis = zvis, mode = 'save', versionname = 'original')
		#tflagdata(vis = zvis, mode = 'shadow', flagbackup = False)
                flagdata(vis = zvis, mode = 'shadow', flagbackup = False)
		print '******* Flagging shadowed data'
		#tflagdata(vis = zvis, mode = 'manual', autocorr = True, flagbackup = False)
                flagdata(vis = zvis, mode = 'manual', autocorr = True, flagbackup = False)
		print '******* Flagging autocorrelation data'
                #  remove special observations
                vm = aU.ValueMapping(self.zvis)
                p=vm.getScansForIntent('CALIBRATE_POINTING#ON_SOURCE')
                f=vm.getScansForIntent('CALIBRATE_FOCUS#ON_SOURCE')
                if len(p) != 0: 
                        #tflagdata(vis = zvis, mode = 'manual', intent = '*POINTING*', flagbackup = False)
                        flagdata(vis = zvis, mode = 'manual', intent = '*POINTING*', flagbackup = False)
                        print '******* Flaging pointing data'
                
		if len(f) != 0:
                        #tflagdata(vis = zvis, mode = 'manual', intent = '*FOCUS*', flagbackup = False)
                        flagdata(vis = zvis, mode = 'manual', intent = '*FOCUS*', flagbackup = False)
                        print '******* Flaging focus data'

                #tflagdata(vis = zvis, mode = 'manual', intent = '*ATMOSPHERE*', flagbackup = False)
                flagdata(vis = zvis, mode = 'manual', intent = '*ATMOSPHERE*', flagbackup = False)
		print '******* Flaging tsys data'
		# flag tables
                tb.open(zvis+"/FLAG_CMD",nomodify=False)

                command=tb.getcol("COMMAND")

                if len(command) > 0:

		     flagcmd(vis = zvis, inpmode = 'table', action = 'plot', useapplied = True)

		     flagcmd(vis = zvis, inpmode = 'table', action = 'apply', useapplied = True)

                else:

                     print "The table FLAG_CMD in ",zvis,"is empty. The flagcmd task was ignored." 

                tb.close()

		flagmanager(vis = zvis, mode = 'save', versionname = 'apriori')

	#==================================================#
	# subroutine 3 : Additional flagging 
	# (obtained after runing once the script and checking the data)
	# we create this file that will include the flag commands and the reason for each flagging
	def addFlag(self,zvis):
                from flagmanager import flagmanager
		from flagdata import flagdata
                #from tflagdata import tflagdata
                from flagcmd import flagcmd
		if os.path.isfile(self.flagdataFile):
                        print '******* Additional Flags added from ', self.flagdataFile
			execfile(self.flagdataFile)
                #

	#==================================================#
	#  flag edge channels
	def flagEdgeChan(self,zsplit):
                from flagmanager import flagmanager
		from flagdata import flagdata
                #from tflagdata import tflagdata
                from flagcmd import flagcmd
		flagmanager(vis = zsplit, mode = 'save', versionname = 'original')
		# XXX needs to be auto from the number of channels
		#if self.Type == 'Cone': flagdata(vis = zsplit, intent = '',spw = ['*:0~15','*:112~127'], flagbackup = False)
                #if self.Type == 'Grid':
                if self.Type == 'ACA':
                        #flagdata(vis = zsplit, mode = 'manual', intent = '',spw = '*:0~9;110~119',flagbackup = False)
                        flagdata(vis = zsplit, mode = 'manual', intent = '',spw = '*:0~9;54~63',flagbackup = False)
                else:
                        flagdata(vis = zsplit, mode = 'manual', intent = '',spw = '*:0~9;54~63',flagbackup = False)
		print '******* Flag edge channels'    
		flagmanager(vis = zsplit, mode = 'save', versionname = 'edge_channels')

	#==================================================#
	# Bandpass solutions
	def doBandpass(self,zsplit):
		#  First remove phase changes over scan
		if self.Type == 'ACA' :
                     gaincal(vis = zsplit, caltable = self.BandpassName+'fcal_g1', field = self.BandpassID, spw = '*:30~89', solint = 'int', refant = self.ref_ant, minblperant=self.minbl)
                else:
                     gaincal(vis = zsplit, caltable = self.BandpassName+'fcal_g1', field = self.BandpassID, spw = '*:17~47', solint = 'int', refant = self.ref_ant, minblperant=self.minbl)
		# plot gain solutions
		if self.zsaveplo == True:
			plotcal(caltable = self.BandpassName+'fcal_g1', xaxis = 'time', yaxis = 'amp', antenna = '', plotrange=[0,0,0.0,00], subplot = 221, iteration = 'spw', showgui = False, figfile = 'output_files/diag_figs/g1_amp.png')
         	# bandpass solutions
		bandpass(vis = zsplit, caltable = self.BandpassName+'fcal_b1', field = self.BandpassID, solnorm = True, gaintable = self.BandpassName+'fcal_g1', refant = self.ref_ant)
		# plot solutions
		if self.zsaveplo == True:
			plotcal(caltable = self.BandpassName+'fcal_b1', xaxis = 'chan', yaxis = 'amp', antenna = '', plotrange=[0,0,0.8,1.2], subplot = 221, iteration = 'spw', showgui = False, figfile = 'output_files/diag_figs/b1_amp.png')
			plotcal(caltable = self.BandpassName+'fcal_b1', xaxis = 'chan', yaxis = 'phase', antenna = '', plotrange=[0,0,-90,90], subplot = 221, iteration = 'spw', showgui = False, figfile = 'output_files/diag_figs/b1_phase.png')
                #
        #

        #-------------------------------------
        # ant_gain_check on bandpass calibrator
        def checkGain(self,zsplit):
                tb.open(zsplit+'/ANTENNA')
                ant_names=tb.getcol('NAME')
                ant_pos=tb.getcol('POSITION')
                nant = len(ant_names)
                tb.close()
                #  Get gaintable
                caltable = self.BandpassName+'fcal_g1'
                tb.open(caltable)
                #time=tb.getcol('TIME')
                #time = time-time[0]+5.0
                ant = tb.getcol('ANTENNA1')
                flag = tb.getcol('FLAG')
                #gain = tb.getcol('GAIN')
                gain = tb.getcol('CPARAM')
                #spw = tb.getcol('CAL_DESC_ID')
                spw = tb.getcol('SPECTRAL_WINDOW_ID')
                g = numpy.abs(gain)
                np=len(ant)
                tb.close()
                spw_un = numpy.unique(spw)
                ant_un = numpy.unique(ant)
                nspw = len(spw_un)
                nant = len(ant_un)
                #  Get average g
                npol = g.shape[0]
                npts = g.shape[2]
                gg = []
                for i in range(0,npol):
                        for j in range(0,npts):
                                if flag[i][0][j] != 1.0: gg.append(g[i][0][j])
                                #if g[i][0][j] != 1.0: gg.append(g[i][0][j])
                        #
                #
                gavg = numpy.median(gg)
                #   Open file for writing
                zfileBp = 'output_files/ant_gain_check.txt'
                #os.system('rm '+zfileBp)
                f = open (zfileBp, 'w')
                #print 'opening file '+zfileBp
                f.write('\n\n')
                f.write('************************************************************************** \n')
                f.write('\n')
                f.write('                    MEDIAN GAIN VALUE = %8.3F \n\n' % (gavg))
                f.write('                RELATIVE ANTENNA GAIN FOR BANDPASS OBSERVATION \n\n')
                f.write('     Antenna          SPW0                SPW1                SPW2                SPW3\n')
                f.write('                   X        Y          X        Y          X        Y          X        Y\n')
                #  get a gain,phase for each ant, spw stream
                g_xall = []
                g_yall = []
                low_gain = 0.60
                high_gain = 1.40
                #  Get gain and phase average and rms for each stream (both pols)
                for iant in range(0,nant):
                        gant_xavg = []
                        gant_yavg = []
                        gant_xrms = []
                        gant_yrms = []
                        g_xstar = []
                        g_ystar = []
                        for ispw in range(0,nspw):
                                g_x = []
                                g_y = []
                                for j in range(0,np):
                                        if (iant == ant[j]) & (ispw == spw[j]):
                                                #if g[0][0][j] != 1.0:
                                                if flag[0][0][j] != 1.0:
                                                        g_x.append(g[0][0][j])
                                                #
                                                #if g[1][0][j] != 1.0:
                                                if flag[1][0][j] != 1.0:
                                                        g_y.append(g[1][0][j])
                                                #
                                        g_xavg = numpy.average(g_x)/gavg
                                        g_xvar = numpy.var(g_x)/gavg
                                        g_yavg = numpy.average(g_y)/gavg
                                        g_yvar = numpy.var(g_y)/gavg
                                #
                                gant_xavg.append(g_xavg)
                                if (g_xavg > high_gain) or (g_xavg < low_gain):
                                        g_xstar.append('**')
                                else:
                                        g_xstar.append('  ')
                                #
                                gant_xrms.append(numpy.sqrt(g_xvar))
                                gant_yavg.append(g_yavg)
                                gant_yrms.append(numpy.sqrt(g_yvar))
                                if (g_yavg > high_gain) or (g_yavg < low_gain):
                                        g_ystar.append('**')
                                else:
                                        g_ystar.append('  ')
                                #
                        #
                        f.write('    %3d-%4s  %6.2f%2s  %6.2f%2s ' % (iant, ant_names[iant], gant_xavg[0], g_xstar[0], gant_yavg[0], g_ystar[0]))
                        for ispw in range (1,nspw-1):
                                f.write (' %6.2f%2s  %6.2f%2s ' % (gant_xavg[ispw], g_xstar[ispw], gant_yavg[ispw], g_ystar[ispw]))
                        #
                        f.write (' %6.2f%2s  %6.2f%2s \n' % (gant_xavg[nspw-1], g_xstar[nspw-1], gant_yavg[nspw-1], g_ystar[nspw-1]))
                #
                f.write ('\n ** means outside of normalized range: %6.2f to %6.2f \n ' % (low_gain, high_gain))
                f.close()
        
        #==================================================#
	# Phase calibration
	#def doGains(self,zsplit,ant):
        def doGains(self,zsplit):
		#   Take out phase fluctuations of bandpass
		gaincal(vis = zsplit, gaintable = self.BandpassName+'fcal_b1', field = self.BandpassID, refant = self.ref_ant, solint = 'int', calmode = 'p', caltable = self.BandpassName+'fcal_g2', minblperant=self.minbl)
		#   Get scan average amp, phase
		gaincal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1',self.BandpassName+'fcal_g2'], field = self.BandpassID, refant = self.ref_ant, solint = 'inf', calmode = 'ap', caltable = self.BandpassName+'fcal_g3', minblperant=self.minbl)
        
                #  Take out phase fluctuations for all sources, NO averaging for plot use only
                gaincal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1',self.BandpassName+'fcal_g2',self.BandpassName+'fcal_g3'], caltable = self.BandpassName+'pcal_NOave_g1', field = '', refant = self.ref_ant, solint = 'int', minsnr=1.5, calmode = 'p', minblperant=self.minbl)
                # plot average phases for all sources
                if self.zsaveplo == True:
                        for i in range(0,self.nant):
                               plotcal(caltable = self.BandpassName+'pcal_NOave_g1', xaxis = 'time', yaxis = 'phase', poln = '', antenna = str(i), plotrange=[0,0,-180,180], showgui = False, figfile = 'output_files/diag_figs/phase_allFields_ant'+self.ant_name[i]+'.png', subplot=111)
                       #
                #
                #  Take out phase fluctuations for all sources over scan of 30 sec, averaging pol
		gaincal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1',self.BandpassName+'fcal_g2',self.BandpassName+'fcal_g3'], caltable = self.BandpassName+'pcal_g1', field = '', refant = self.ref_ant, solint = '30s', minsnr=1.5, gaintype = 'T', calmode = 'p', minblperant=self.minbl)
		#  Get average gain for each source after taking out phase fluctuations
		gaincal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1',self.BandpassName+'fcal_g2',self.BandpassName+'fcal_g3',self.BandpassName+'pcal_g1',], caltable = self.BandpassName+'apcal_g1', field = '', refant = self.ref_ant, minsnr = 1.5, solint = 'inf', gaintype = 'G', calmode = 'ap', minblperant=self.minbl)
		# plot average phases for all sources
		#if self.zsaveplo == True:
                #        for i in range(0,self.nant):
		#		plotcal(caltable = self.BandpassName+'pcal_g1', xaxis = 'time', yaxis = 'phase', poln = '', antenna = str(i), plotrange=[0,0,-180,180], showgui = False, figfile = 'output_files/diag_figs/phase_allFields_ant'+self.ant_name[i]+'.png', subplot=111)
		#	#
                #
        #
	#==================================================#
	# subroutine : Check for crossed-hand delay offsets
	def doKCross(self,zsplit):
		gaincal(vis = zsplit, refant = self.ref_ant, gaintype = 'KCROSS', parang = True, field = self.BandpassID, caltable = self.BandpassName+'kcross', gaintable = [self.BandpassName+'fcal_b1', self.BandpassName+'fcal_g2', self.BandpassName+'fcal_g3',self.BandpassName+'pcal_g1'], minblperant=self.minbl)
		applycal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1', self.BandpassName+'fcal_g2', self.BandpassName+'fcal_g3',self.BandpassName+'pcal_g1',self.BandpassName+'kcross'])
        
		#  Recheck crossed-hand delays
		if self.zsaveplo == True:
                        plotms(vis = zsplit, xaxis = 'freq', yaxis = 'phase', ydatacolumn = 'corrected', correlation = 'XY,YX', avgtime = '50000', avgbaseline = True, field = self.BandpassID, avgscan = True, spw = '', coloraxis = 'corr', avgspw = False, plotfile = 'output_files/diag_figs/Kcross_spw.png')
			#plotxy(vis = zsplit, xaxis = 'channel', yaxis = 'phase', datacolumn = 'corrected', correlation = 'XY,YX', averagemode = 'vector', timebin = '50000', crossbls = True, field = self.BandpassID, crossscans = True, spw = '', multicolor = 'corr', stackspw = False, interactive = False, figfile = 'output_files/diag_figs/Kcross_spw.png')
                #
        #
	#==================================================#
	#  Get UV limit of observed SS objects
	def getUVlim(self):
		fluxCalUVlim = []
		for i in range(0,len(self.possibleFluxCalNames)):
			for j in range(0,len(self.fluxCalNames)):
				if self.possibleFluxCalNames[i] == self.fluxCalNames[j]:
					fluxCalUVlim.append(self.UvlimkL[i])
                        #
                #
		self.fluxCalUVlim = fluxCalUVlim

		return 

        #==================================================#
        #  Get scaling factor from planets
        def getScaleFactor(self,visavg):
                nrat = []
                factorXX = []
                factorYY = []
                ssf = self.ssf
                lines=[]
                print ssf
                print self.fluxCalNames
                # nphillip: if self.fluxCalNames is empty then we're going
                # to bomb later, so give up now
                if len(self.fluxCalNames) == 0:
                    raise Exception("Calsurvey_reduction.getScaleFactor(): self.fluxCalNames is empty, which is an error here. Try to find out why it is empty.")
                #
                vm = aU.ValueMapping(visavg)
                tb.open(visavg)
                # open file for writing scale factors
                zfileSS = 'output_files/ss_factors.txt'
                f = open (zfileSS, 'w')
                f.write('\n')
                f.write('         FLUX FACTORS FOR EACH SS OBJECT \n')
                f.write('\n')
                f.write('       SS obj  npts  spw  ssfac  XXfac  YYfac  XXrat  YYrat\n')
                f.write('\n')
                #
                msmd.open(visavg)
                for i in range (0,len(self.fluxCalNames)):
                     aXX = []
                     aYY = []
                     fluxIds = vm.getFieldIdsForFieldName(self.fluxCalNames[i])
                     print 'fluxIds', fluxIds
                     for j in fluxIds:
                             #if 'CALIBRATE_AMPLI#ON_SOURCE' in msmd.intentsforfield(field=j):
                             fluxId = str(j)
                             print 'fluxId',fluxId
                     for k in range (0,self.nspw):
                        #  get uv data for each object and spw
                        tb1 = tb.query('(FIELD_ID=='+fluxId+') and (DATA_DESC_ID=='+str(k)+')')
                        zuvw = tb1.getcol('UVW')
                        zdata = tb1.getcol('DATA')
                        zmodel = tb1.getcol('MODEL_DATA')
                        zu = zuvw[0]
                        zv = zuvw[1]
                        zdatXX = zdata[0][0]
                        zdatXY = zdata[1][0]
                        zdatYX = zdata[2][0]
                        zdatYY = zdata[3][0]
                        zmodXX = ssf[k]*zmodel[0][0]
                        zmodXY = ssf[k]*zmodel[1][0]
                        zmodYX = ssf[k]*zmodel[2][0]
                        zmodYY = ssf[k]*zmodel[3][0]
                        znum = zuvw.shape[1]
                        #
                        numall = 0
                        modratXX=[]
                        modratYY=[]
                        wtmod=[]
                        uvmax = self.fluxCalUVlim[i] * 300.0/self.zfreq[k]
                        uvmax2 = uvmax*uvmax
                        for j in range (0,znum):
                           if zu[j]**2+zv[j]**2 < uvmax2:
                                wtmod.append(abs(zmodXX[j])**2/1000.0)
                                modratXX.append(zmodXX[j]/zdatXX[j])
                                modratYY.append(zmodYY[j]/zdatYY[j])
                                numall = numall + 1
                           #
                        #
                        if numall > 5:
                                xxavg = (numpy.real(numpy.average(modratXX,None,wtmod)))
                                yyavg = (numpy.real(numpy.average(modratYY,None,wtmod)))
                                nrat.append(numall)
                                #aSPW.append(k)
                                aXX.append(xxavg)
                                aYY.append(yyavg)
                                #line=(' %12s %4d %4d %6.2f %6.2f %6.2f \n'% (self.fluxCalNames[i], nrat[k], k, ssf[k], aXX[k], aYY[k]))
                                factorXX.append(aXX[k])
                                factorYY.append(aYY[k])
                        else:
                                factorXX.append(0.0)
                                factorYY.append(0.0)
                        #
                        #lines.append(line)
                        #print self.fluxCalNames[i], factorXX, factorYY, ratioFactorXX, ratioFactorYY
                        line=(' %12s %4d %4d %6.2f %6.2f %6.2f %6.2f %6.2f \n'% (self.fluxCalNames[i], nrat[k], k, ssf[k], aXX[k], aYY[k], aXX[k]/factorXX[k], aYY[k]/factorYY[k] ))
                        #print line
                        f.write(line)
                     #
                     f.write('\n')
                     #for k in range (0,self.nspw):
                     #      f.write(lines[k])
                     #
                #
		msmd.close()
                tb.close()
		f.close
                # Get the scaling factors for the first fluxCal
                print '*******  First ss object used for normalization'
                useFactorXX = []
                useFactorYY = []
                for k in range (0,self.nspw):
                        useFactorXX.append(factorXX[k])
                        useFactorYY.append(factorYY[k])
                        print '       %12s %6.2f %6.2f  '% (self.fluxCalNames[0], useFactorXX[k], useFactorYY[k])
                #
                self.useFactorXX = useFactorXX
                self.useFactorYY = useFactorYY
                return factorXX,factorYY

	#==================================================#
	# Obtain ratios of all SS objects respect to the first one
	def getRatios(self,visavg,factorXX,factorYY,f):
		# first we obtain the average over spw....
		avgXX_pln = []
		avgYY_pln = []
                for i in range (0,len(self.fluxCalNames)):
                   avgXX = 0.0
                   avgYY = 0.0
                   for k in range (0,self.nspw):
                        ic = self.nspw*i + k
		        avgXX = avgXX + factorXX[ic]
		        avgYY = avgYY + factorYY[ic]
		   #
		   avgXX = avgXX/self.nspw
		   avgYY = avgYY/self.nspw
		   avgXX_pln.append(avgXX)
		   avgYY_pln.append(avgYY)
		#
		#
		# print average results
		print 'Planet  avgXX  avgYY'
		for j in range(0,len(self.fluxCalNames)):
			print '%12s %6.2f %6.2f  '% (self.fluxCalNames[j], avgXX_pln[j], avgYY_pln[j])
		#
		# compute ratios between planets
		ratioXX = 0
		ratioYY = 0
		ratioXX_pln = []
		ratioYY_pln = []
		for j in range (1,len(self.fluxCalNames)):
			ratioXX = avgXX_pln[0]/avgXX_pln[j]
			ratioYY = avgYY_pln[0]/avgYY_pln[j]
			plnRatios='%12s %6.2f %6.2f \n'% (self.fluxCalNames[0]+'/'+self.fluxCalNames[j], ratioXX, ratioYY)
			#print plnRatios
			f.write('# '+plnRatios)
			ratioXX_pln.append(ratioXX)
			ratioYY_pln.append(ratioYY)
		#
		return ratioXX_pln,ratioYY_pln
	#==================================================#
	#  Apply these factors to the data.  This is done by putting in an antenna
	#  amplitude corrections in the appropriate way.  One could also just
	#  Multiply the visibilities.
        def applyFactor(self,visavg):
                if self.zuseplanet:
			for i in range (0,self.nspw):
				gencal(vis = visavg, caltype = 'amp', caltable = 'flux_scale.tab', spw = str(i), pol = 'X', parameter = 1.0/sqrt(self.useFactorXX[i]))
			#
			for i in range (0,self.nspw):
				gencal(vis = visavg, caltype = 'amp', caltable = 'flux_scale.tab', spw = str(i), pol = 'Y', parameter = 1.0/sqrt(self.useFactorYY[i]))
			#
			applycal(vis = visavg, gaintable = 'flux_scale.tab')
		else:
			for i in range (0,self.nspw):
				gencal(vis = visavg, caltype = 'amp', caltable = 'flux_scale.tab', spw = str(i), pol = '', parameter = 1.0/sqrt(self.zflux))
			#
			applycal(vis = visavg, gaintable = 'flux_scale.tab')
                #

                if self.zsaveplo == True:
                        for i in range (0,len(self.sourcesList)):
                                if self.sourcesList[i] in self.fluxCalNames:
					#plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'circle', symbolcolor='00ffff', symbolsize=7, correlation = 'XX,YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=True, plotindex=0,customsymbol=True, plotfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png', overwrite = True)
					plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'square', symbolcolor='00ffff', symbolsize=5, correlation = 'XX', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=True, plotindex=0, customsymbol=True,spw='0,1')
					plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'diamond', symbolcolor='00ffff', symbolsize=7, correlation = 'XX', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=1, customsymbol=True,spw='2,3')
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'square', symbolcolor='green', symbolsize=5, correlation = 'YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=2, customsymbol=True,spw='0,1')
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'diamond', symbolcolor='green', symbolsize=7, correlation = 'YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=3, customsymbol=True,spw='2,3')
					plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'circle', symbolcolor='black', symbolsize=5, correlation = 'XX,YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'model', clearplots=False, plotindex=4, customsymbol=True, title='G %s: %s'%(self.sourcesList[i],visavg), plotfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png', overwrite = True)
                                        #plotxy(vis = visavg, xaxis = 'uvwave', plotsymbol = 'o', correlation = 'XX,YY', averagemode = 'vector', stackspw = True, timebin = '40', field = self.sourcesList[i], overplot = False, plotcolor = 'green', yaxis='amp', datacolumn = 'corrected', interactive = False)
                                        #plotxy(vis = visavg, xaxis = 'uvwave', plotsymbol = 'o', correlation = 'XX,YY', averagemode = 'vector', stackspw = True, timebin = '40', field = self.sourcesList[i], overplot = True, plotcolor = 'blue', yaxis='amp', datacolumn = 'model', title = self.sourcesList[i], interactive = False, figfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png')
                                else:
                                        #plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'circle', symbolcolor='00ffff', symbolsize=7, correlation = 'XX,YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=True, plotindex=0,customsymbol=True, plotfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png', overwrite = True)
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'square', symbolcolor='00ffff', symbolsize=5, correlation = 'XX', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=True, plotindex=0, customsymbol=True,spw='0,1')
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'diamond', symbolcolor='00ffff', symbolsize=7, correlation = 'XX', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=1, customsymbol=True,spw='2,3')
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'square', symbolcolor='green', symbolsize=5, correlation = 'YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=2, customsymbol=True,spw='0,1')
                                        plotms(vis=visavg, xaxis = 'uvwave', symbolshape = 'diamond', symbolcolor='green', symbolsize=7, correlation = 'YY', avgtime = '40', field = self.sourcesList[i], yaxis='amp', ydatacolumn = 'corrected', clearplots=False, plotindex=3, customsymbol=True,spw='2,3', plotfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png', overwrite = True, title='G %s: %s'%(self.sourcesList[i],visavg))
                                        #plotxy(vis = visavg, xaxis = 'uvwave', plotsymbol = 'o', correlation = 'XX,YY', averagemode = 'vector', stackspw = True, timebin = '40', field = self.sourcesList[i], overplot = False, plotcolor = 'green', yaxis='amp', datacolumn = 'corrected', title = self.sourcesList[i], interactive = False, figfile = 'output_files/diag_figs/ampVSuvdist_'+self.sourcesList[i]+'.png')
                        #
                #
        #

        #==================================================#
        #  Get antenna-based fluxes for more editing check
        #  Get antenna-based residuals
        def getAntFlags(self,visavg):
                os.system('rm -rf final_data.ms')
                split(vis = visavg, datacolumn = 'corrected', outputvis = 'final_data.ms', keepflags = False)
                self.getAntInfo('final_data.ms')
                nant = self.nant
                ant_name = self.ant_name
                ant_pos = self.ant_pos
                #  Get baseline lengths to reference antenna
                ant_separation = []
                for i in range(0,nant): 
                        if ant_name[i] == self.ref_ant: RRR = i
                for i in range(0,nant): 
                        separation = numpy.sqrt((ant_pos[0,i]-ant_pos[0,RRR])**2 + (ant_pos[1,i]-ant_pos[1,RRR])**2)
                        ant_separation.append(separation)
                
                os.system('rm -rf ant_flux_check')
                gaincal(vis = 'final_data.ms', refant = str(RRR), caltable = 'ant_flux_check', minsnr = 2.0, minblperant=self.minbl)
                print '*******  Making antenna-based flux density list *******'
                tb.open('ant_flux_check')
                zfileFlux = 'output_files/source_ant_flux.txt'
                f2 = open (zfileFlux, 'w')
                f2.write ('\n  ANTENNA-BASED FLUX DENSITY FOR EACH SOURCE \n')
                for ii in range (0,self.nquasars):
                    cnId = str(self.quasarIds[ii])
                    tb1 = tb.query('FIELD_ID == '+cnId)
                    ant = tb1.getcol('ANTENNA1')
                    flag = tb1.getcol('FLAG')
		    print 'ant', len(ant),ant
                    if len(ant) > 5:
                        #gain = tb1.getcol('GAIN')
                        gain = tb1.getcol('CPARAM')
                        g = numpy.abs(gain)**2
                        gg = 0.5*(g[0][0]+g[1][0])
                        newgg = []
                        for kk in range(0,len(gg)):
                          #if gg[kk] != 1.0: newgg.append(gg[kk])
                          if flag[0][0][kk] != 1 and flag[1][0][kk] != 1: newgg.append(gg[kk])
                        
                        gamed = numpy.median(newgg)
                        inds = numpy.argsort(newgg)
                        ggsort = numpy.take(newgg,inds)
                        numgg = len(newgg)
			print 'numgg',numgg,newgg,len(newgg),ggsort 
                        l1 = 0.75*numgg; l2 = 0.25*numgg
                        gasig = ggsort[l1]-ggsort[l2]
                        f2.write(' \n')
                        f2.write('    %10s   med=%7.3f   rms=%7.3f \n'% (self.quasarNames[ii], gamed, gasig))
                        f2.write('       ANT     MIN     MAX     MED    BSLINE ??\n')
                        lines = []
                        for j in range (0,nant):
                                tb2 = tb.query('(FIELD_ID == '+cnId+') and (ANTENNA1 == '+str(j)+')')
                                #gain = tb2.getcol('GAIN')
                                gain = tb2.getcol('CPARAM')
                                flag = tb2.getcol('FLAG')
                                g = numpy.abs(gain)
                                gg = (0.5*(g[0][0]+g[1][0]))**2
                                newgg = []
                                for kk in range(0,len(gg)):
                                  #if gg[kk] != 1.0: newgg.append(gg[kk])
                                  if flag[0][0][kk] != 1 and flag[1][0][kk] != 1: newgg.append(gg[kk])
                                
                                if len(newgg) == 0:
                                  line='  %3d %4s      ---- NO DATA ----   %8.1f\n' %(j, ant_name[j], ant_separation[j])
                                else:
                                  gmed = numpy.median(newgg)
                                  gmin = numpy.min(newgg)
                                  gmax = numpy.max(newgg)
                                  gerror = ' '
                                  if numpy.abs(gamed-gmed) > 1.5*gasig: gerror = '*'
                                  #if gmax > 2*gmed: gerror = '*'
                                  if numpy.abs(gmax-gmed) > 1.5*gasig: gerror = '*'
                                  #if gmin < 0.5*gmed: gerror = '*'
                                  if numpy.abs(gmed-gmin) > 1.5*gasig: gerror = '*'
                                  line='  %3d %4s  %7.3f %7.3f %7.3f %8.1f %2s\n' %(j, ant_name[j], gmin, gmax, gmed, ant_separation[j], gerror)
                                #
                                lines.append(line)
                        #  
                        inds = numpy.argsort(ant_separation)
                        sortlines = numpy.take(lines,inds)
                        for j in range (0,nant):
                          f2.write(sortlines[j])
                        #
                    #
                tb.close()
                #Flag for SSOs
                vm = aU.ValueMapping(visavg)
		tb.open(visavg)
		msmd.open(visavg)
		if self.zuseplanet:
                    f2.write ('\n  ANTENNA-BASED AMPLITUDE RESIDUALS(corrected-model) FOR EACH SSOs:\n')
                    for i in range (0,len(self.fluxCalNames)):
		        fluxIds = vm.getFieldIdsForFieldName(self.fluxCalNames[i])
			print 'fluxIds', fluxIds
			for j in fluxIds:
				# if 'CALIBRATE_AMPLI#ON_SOURCE' in msmd.intentsforfield(field=j):
				fluxId = str(j)
			print 'fluxId',fluxId
			#  get uv data for each object and spw?
			tb1 = tb.query('FIELD_ID=='+fluxId)
			ant = tb1.getcol('ANTENNA1')
			#flag = tb1.getcol('FLAG')
			print 'ant', len(ant),ant
			if len(ant) > 5:
				zuvw = tb1.getcol('UVW')
				zdata = tb1.getcol('CORRECTED_DATA')
				zmodel = tb1.getcol('MODEL_DATA')
				zu = zuvw[0]
				zv = zuvw[1]

				zres=0.5*(numpy.abs(zdata[0][0])-numpy.abs(zmodel[0][0])+numpy.abs(zdata[3][0])-numpy.abs(zmodel[3][0])) #Still pendig leave only the real part
			
			        #zdatXX = zdata[0][0]
			        #zdatXY = zdata[1][0]
			        #zdatYX = zdata[2][0]
			        #zdatYY = zdata[3][0]
			        #zmodXX = ssf[k]*zmodel[0][0]
			        #zmodXY = ssf[k]*zmodel[1][0]
			        #zmodYX = ssf[k]*zmodel[2][0]
			        #zmodYY = ssf[k]*zmodel[3][0]
                        
				zamed = numpy.median(zres)
				inds = numpy.argsort(zres)
				zsort = numpy.take(zres,inds)
				numz = len(zres)
				print 'numz',numz,zres,len(zres),zsort 
				l1 = 0.75*numz; l2 = 0.25*numz
				zasig = zsort[l1]-zsort[l2]
				f2.write(' \n')
				f2.write('    %10s   med=%7.3f   rms=%7.3f \n'% (self.fluxCalNames[i], zamed, zasig))
				f2.write('       ANT     MIN     MAX     MED    BSLINE ??\n')
				lines = []
				for j in range (0,nant):
				  tb2 = tb.query('(FIELD_ID == '+fluxId+') and (ANTENNA1 == '+str(j)+')')
				  zdata = tb2.getcol('CORRECTED_DATA')
				  zmodel = tb2.getcol('MODEL_DATA')
				
				  if len(zdata) == 0:
				     line='  %3d %4s      ---- NO DATA ----   %8.1f\n' %(j, ant_name[j], ant_separation[j])
				  else:
				     #zres=numpy.real(0.5*(zdata[0][0]-zmodel[0][0]+zdata[3][0]-zmodel[3][0])) #Still pendig leave only the real part
				  
				     zres=0.5*(numpy.abs(zdata[0][0])-numpy.abs(zmodel[0][0])+numpy.abs(zdata[3][0])-numpy.abs(zmodel[3][0])) #Still pendig leave only the real part
				     zmed = numpy.median(zres)
				     zmin = numpy.min(zres)
				     zmax = numpy.max(zres)
				     zerror = ' '
				     if numpy.abs(zamed-zmed) > 1.5*zasig: zerror = '*'
                                     #if zmax > 2*zmed: gerror = '*'
                                     #if numpy.abs(zmax-zmed) > 3.0*zasig: zerror = '*'
                                     #if zmin < 0.5*zmed: gerror = '*'
                                     #if numpy.abs(zmed-zmin) > 3.0*zasig: zerror = '*'
                                     line='  %3d %4s  %7.3f %7.3f %7.3f %8.1f %2s\n' %(j, ant_name[j], zmin, zmax, zmed, ant_separation[j], zerror)
                                  #
				  lines.append(line)
				#  
				inds = numpy.argsort(ant_separation)
				sortlines = numpy.take(lines,inds)
				for j in range (0,nant):
				  f2.write(sortlines[j])
				#
			#
	

		    #
		#
		f2.close()
		tb.close()

        #==================================================#
        # clean and produce images of all observed sources
        def doImage(self,visavg):
		#  Determine imaging cell and imsize
		image_sizes = [216,256,360,432,640,800,1000,1296,1600,2048]
                #  Get antenna list and maximum baseline
		tb.open(visavg+'/ANTENNA')
		antenna = tb.getcol('NAME')
		tb.close()
		nant = len(numpy.unique(antenna))
		baselines = aU.getBaselineLengths(visavg)
		ll = baselines[len(baselines)-1]
		baseline_max = ll[1]
                freq = 100.0
		if self.zband == '6': freq = 250.0
		if self.zband == '7': freq = 350.0
		if self.zband == '9': freq = 650.0
		b = 15000.0 / baseline_max / freq
		if b > 0.20:
			b = numpy.int(b*20)/20.0
		else:
			b = numpy.int(b*100.0)/100.0
                        #
		zcell = str(b)+'arcsec'
		pbsize = 4000.0 / freq
		possible_size = 2.0*pbsize/b
		for zimsize in image_sizes:
			if zimsize > possible_size: break

                print '*** Parameters '
                print '    maximum baselines (m)  = ', baseline_max
                print '    cell spacing           = ', zcell
                print '    image size             = ', zimsize
		zblc = str(int(0.1*zimsize)); ztrc = str(int(0.4*zimsize))
		zbox = zblc+','+zblc+','+ztrc+','+ztrc
                print '    noise box in           = ', zbox
                for i in self.sourcesList:
                        if str(i) in self.planetNames:
                                clean(vis=visavg, imagename = 'output_files/images/'+str(i), field = str(i), niter = 500, mask = 'circle[[128pix,128pix], 14pix]', interactive=False, cell = zcell, imsize = zimsize)
                                a = imstat(imagename = 'output_files/images/'+str(i)+'.image', box = zbox)
                                b = 3*a['rms'][0]
                                imview(contour={'file': 'output_files/images/'+str(i)+'.image', 'levels': [-1, 1, 2, 3, 4, 5, 6, 8, 10, 20, 40, 80, 160, 320, 640, 1280], 'unit': float(b)}, zoom=2, axes = {}, out='output_files/images/Image_'+str(i)+'.png')
                        else:
                                clean(vis=visavg, imagename = 'output_files/images/'+str(i), field = str(i), niter = 100, mask = 'circle[[128pix,128pix], 3pix]', interactive=False, cell = zcell, imsize = zimsize)
                                a = imstat(imagename = 'output_files/images/'+str(i)+'.image', box = zbox)
                                b = 3*a['rms'][0]
                                imview(contour={'file': 'output_files/images/'+str(i)+'.image', 'levels': [-1, 1, 2, 3, 4, 5, 6, 8, 10, 20, 40, 80, 160, 320, 640, 1280], 'unit': float(b)}, zoom=2, axes = {}, out='output_files/images/Image_'+str(i)+'.png')
                        
                
        
        #==================================================#
        # BAND 3: spw 0,1 and 1,2 separately
        def getFluxB3(self,visavg,f):
                tb.open(visavg)
                quasarNames = self.quasarNames
                zfreq = self.zfreq
                nant = self.nant
                #
		zfileAmp = 'output_files/source_structure.txt'
		f1 = open (zfileAmp, 'w')
		f1.write ('\n\n      ANALYSIS OF SOURCE RESOLUTIONS IN THREE PARTS\n\n')
                f1.write ('       SOURCE     FREQ   AVG   RMS  #PTS    n/3*UVMAX  \n')
                f1.write ('                                            (m)     (kL) \n')
                avgfreq = (self.zfreq[0]+self.zfreq[1])*0.5
                for ii in range (0,self.nquasars):
                    cnId = str(self.quasarIds[ii])
                    tb1 = tb.query('FIELD_ID == '+cnId+' and (DATA_DESC_ID==0 or DATA_DESC_ID==1)')
                    zdata = tb1.getcol('CORRECTED_DATA')
		    ztimestamp = tb1.getcol('TIME')
        	    if len(zdata) > 0:
                        zdatXX = zdata[0][0]
			self.nanteff.append(int(0.5*(1.+numpy.sqrt(1.+8.*float(len(zdatXX))/4./float(len(set(ztimestamp)))))+0.5))
                        zdatXY = zdata[1][0]
                        zdatYX = zdata[2][0]
                        zdatYY = zdata[3][0]
                        zuvw = tb1.getcol('UVW')
                        zuv2 = zuvw[0]**2 + zuvw[1]**2
                        zuv = numpy.sqrt(zuv2)
                        uvmax = numpy.max(zuv)
                        uvmaxkL = uvmax * avgfreq / 300.0
                        # create the data arrays for each uv region
                        zdat1_XX = []
                        zdat1_XY = []
                        zdat1_YX = []
                        zdat1_YY = []
                        zdat2_XX = []
                        zdat2_XY = []
                        zdat2_YX = []
                        zdat2_YY = []
                        zdat3_XX = []
                        zdat3_XY = []
                        zdat3_YX = []
                        zdat3_YY = []
                        for j in range (0,len(zuv)-1):
                                if zuv[j] < 1*uvmax/3:
                                        zdat1_XX.append(zdatXX[j])
                                        zdat1_XY.append(zdatXY[j])
                                        zdat1_YX.append(zdatYX[j])
                                        zdat1_YY.append(zdatYY[j])
                                #
                                if 1*uvmax/3 < zuv[j] < 2*uvmax/3:
                                        zdat2_XX.append(zdatXX[j])
                                        zdat2_XY.append(zdatXY[j])
                                        zdat2_YX.append(zdatYX[j])
                                        zdat2_YY.append(zdatYY[j])
                                #
                                if 2*uvmax/3 < zuv[j] < uvmax:
                                        zdat3_XX.append(zdatXX[j])
                                        zdat3_XY.append(zdatXY[j])
                                        zdat3_YX.append(zdatYX[j])
                                        zdat3_YY.append(zdatYY[j])
                                #
                        #
                        # first third: uv < 1/3uvmax
                        amp1_XX = numpy.average(zdat1_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp1_XXerror = numpy.std(zdat1_XX,ddof=0)
                        amp1_XY = numpy.average(zdat1_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp1_XYerror = numpy.std(zdat1_XY,ddof=0)
                        amp1_YX = numpy.average(zdat1_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp1_YXerror = numpy.std(zdat1_YX,ddof=0)
                        amp1_YY = numpy.average(zdat1_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp1_YYerror = numpy.std(zdat1_YY,ddof=0)
                        Npoint1 = len(zdat1_XX)
                        
                        sum1_xxyy = 0.5 * (amp1_XX+amp1_YY)
                        sum1_xxyy_error = 0.5 * (amp1_XXerror+amp1_YYerror)
                        dif1_xxyy = 0.5 * (amp1_XX-amp1_YY)
                        sum1_xyyx = 0.5 * (amp1_XY+amp1_YX)
                        sum1_xyyx_error = 0.5 * (amp1_XYerror+amp1_YXerror)
                        pol1 = numpy.sqrt(dif1_xxyy**2 + sum1_xyyx**2)
                        
                        ampi1 = numpy.abs(sum1_xxyy)
                        ampi1_error = numpy.abs(sum1_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        
                        # second third: 1/3uvmax < uv < 2/3uvmax
                        amp2_XX = numpy.average(zdat2_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp2_XXerror = numpy.std(zdat2_XX,ddof=0)
                        amp2_XY = numpy.average(zdat2_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp2_XYerror = numpy.std(zdat2_XY,ddof=0)
                        amp2_YX = numpy.average(zdat2_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp2_YXerror = numpy.std(zdat2_YX,ddof=0)
                        amp2_YY = numpy.average(zdat2_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp2_YYerror = numpy.std(zdat2_YY,ddof=0)
                        Npoint2 = len(zdat2_XX)
                        
                        sum2_xxyy = 0.5 * (amp2_XX+amp2_YY)
                        sum2_xxyy_error = 0.5 * (amp2_XXerror+amp2_YYerror)
                        dif2_xxyy = 0.5 * (amp2_XX-amp2_YY)
                        sum2_xyyx = 0.5 * (amp2_XY+amp2_YX)
                        sum2_xyyx_error = 0.5 * (amp2_XYerror+amp2_YXerror)
                        pol2 = numpy.sqrt(dif2_xxyy**2 + sum2_xyyx**2)
                        
                        ampi2 = numpy.abs(sum2_xxyy)
                        ampi2_error = numpy.abs(sum2_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        if Npoint2 == 0:
                                ampi2 = 0
                                ampi2_error = 0
                                pol2 = 0
                                sum2_xyyx_error = 0
                        
                        # third third: 2/3uvmax < uv < uvmax
                        amp3_XX = numpy.average(zdat3_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp3_XXerror = numpy.std(zdat3_XX,ddof=0)
                        amp3_XY = numpy.average(zdat3_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp3_XYerror = numpy.std(zdat3_XY,ddof=0)
                        amp3_YX = numpy.average(zdat3_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp3_YXerror = numpy.std(zdat3_YX,ddof=0)
                        amp3_YY = numpy.average(zdat3_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp3_YYerror = numpy.std(zdat3_YY,ddof=0)
                        Npoint3 = len(zdat3_XX)
                        
                        sum3_xxyy = 0.5 * (amp3_XX+amp3_YY)
                        sum3_xxyy_error = 0.5 * (amp3_XXerror+amp3_YYerror)
                        dif3_xxyy = 0.5 * (amp3_XX-amp3_YY)
                        sum3_xyyx = 0.5 * (amp3_XY+amp3_YX)
                        sum3_xyyx_error = 0.5 * (amp3_XYerror+amp3_YXerror)
                        pol3 = numpy.sqrt(dif3_xxyy**2 + sum3_xyyx**2)
                        
                        ampi3 = numpy.abs(sum3_xxyy)
                        ampi3_error = numpy.abs(sum3_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        
                        # print all results
                        if Npoint1 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi1,ampi1_error, Npoint1, 1*uvmax/3, 1*uvmaxkL/3))
			else:
				f1.write('               no data\n')

                        if Npoint2 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi2,ampi2_error, Npoint2, 2*uvmax/3, 2*uvmaxkL/3))
                        else:
				f1.write('               no data\n')

                        if Npoint3 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n\n' % (quasarNames[ii],avgfreq,ampi3,ampi3_error, Npoint3, uvmax, uvmaxkL))
                        else:
				f1.write('               no data\n')
                                
                        ampi123 = [ampi1,ampi2,ampi3]
                        ampi123_error = [ampi1_error,ampi2_error,ampi3_error]
                        pol123 = [pol1,pol2,pol3]
                        sum123_xyyx_error = [sum1_xyyx_error,sum2_xyyx_error,sum3_xyyx_error]
                        Npoint123 = [Npoint1,Npoint2,Npoint3]
                        zwei = numpy.sqrt(Npoint123)
                        zstructure = 0
                        if any(Npoint123)!=0:
                                # Extended emission
                                if ampi1_error > 2*ampi3_error:
                                        zstructure = 1
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be EXTENDED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 1*uvmaxkL/3
                                        uvmaximum = -uvmaxkL
                                # Resolved emission
                                zratio = (ampi1-ampi3)/ampi1
                                if zratio < 0.9 and numpy.abs(ampi1-ampi3) > 2*ampi3_error:
                                        zstructure = 2
                                        ampi = ampi1
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be RESOLVED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = pol1
                                        ampp_error = sum1_xyyx_error
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        # compute uvmax
                                        uv85 = 5*uvmax/6 * numpy.sqrt(0.15/zratio)
                                        uvmaximum = uv85 * avgfreq / 300.0
                                if zstructure == 0:
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = numpy.max(ampi123_error)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        uvmaximum = -uvmaxkL
                                #
                                # PRINT OUTPUT
                                #outline= '%15s, NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f, %4.2f, NE, NE, %7.1f, 100.0, %10s\n' % (sourcesList[i][0:9],avgfreq,ampi,ampi_error, perc, perc_error, uvmaxkL, self.zobsdate)
                                outline= '%12s, NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f, %4.2f, NE, NE, %7.1f, %7.1f, %10s\n' % (quasarNames[ii],avgfreq,ampi,ampi_error, perc, perc_error, uvminimum, uvmaximum, self.zobsdate)
                                f.write(outline)
			#
                    #
                #
                avgfreq = (zfreq[2]+zfreq[3])*0.5
                for ii in range (0,self.nquasars):
                    cnId = str(self.quasarIds[ii])
                    tb2 = tb.query('FIELD_ID == '+cnId+' and (DATA_DESC_ID==2 or DATA_DESC_ID==3)')
                    zdata = tb2.getcol('CORRECTED_DATA')
		    if len(zdata) > 0:
                        zdatXX = zdata[0][0]
                        zdatXY = zdata[1][0]
                        zdatYX = zdata[2][0]
                        zdatYY = zdata[3][0]
                        zuvw = tb2.getcol('UVW')
                        zuv2 = zuvw[0]**2 + zuvw[1]**2
                        zuv = numpy.sqrt(zuv2)
                        uvmax = numpy.max(zuv)
                        uvmaxkL = uvmax * avgfreq / 300.0
                        # create the data arrays for each uv region
                        zdat1_XX = []
                        zdat1_XY = []
                        zdat1_YX = []
                        zdat1_YY = []
                        zdat2_XX = []
                        zdat2_XY = []
                        zdat2_YX = []
                        zdat2_YY = []
                        zdat3_XX = []
                        zdat3_XY = []
                        zdat3_YX = []
                        zdat3_YY = []
                        for j in range (0,len(zuv)-1):
                                if zuv[j] < 1*uvmax/3:
                                        zdat1_XX.append(zdatXX[j])
                                        zdat1_XY.append(zdatXY[j])
                                        zdat1_YX.append(zdatYX[j])
                                        zdat1_YY.append(zdatYY[j])
                                #
                                if 1*uvmax/3 < zuv[j] < 2*uvmax/3:
                                        zdat2_XX.append(zdatXX[j])
                                        zdat2_XY.append(zdatXY[j])
                                        zdat2_YX.append(zdatYX[j])
                                        zdat2_YY.append(zdatYY[j])
                                #
                                if 2*uvmax/3 < zuv[j] < uvmax:
                                        zdat3_XX.append(zdatXX[j])
                                        zdat3_XY.append(zdatXY[j])
                                        zdat3_YX.append(zdatYX[j])
                                        zdat3_YY.append(zdatYY[j])
                                #
                        #
                        # first third: uv < 1/3uvmax
                        amp1_XX = numpy.average(zdat1_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp1_XXerror = numpy.std(zdat1_XX,ddof=0)
                        amp1_XY = numpy.average(zdat1_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp1_XYerror = numpy.std(zdat1_XY,ddof=0)
                        amp1_YX = numpy.average(zdat1_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp1_YXerror = numpy.std(zdat1_YX,ddof=0)
                        amp1_YY = numpy.average(zdat1_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp1_YYerror = numpy.std(zdat1_YY,ddof=0)
                        Npoint1 = len(zdat1_XX)
                        
                        sum1_xxyy = 0.5 * (amp1_XX+amp1_YY)
                        sum1_xxyy_error = 0.5 * (amp1_XXerror+amp1_YYerror)
                        dif1_xxyy = 0.5 * (amp1_XX-amp1_YY)
                        sum1_xyyx = 0.5 * (amp1_XY+amp1_YX)
                        sum1_xyyx_error = 0.5 * (amp1_XYerror+amp1_YXerror)
                        pol1 = numpy.sqrt(dif1_xxyy**2 + sum1_xyyx**2)
                        
                        ampi1 = numpy.abs(sum1_xxyy)
                        ampi1_error = numpy.abs(sum1_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        
                        # second third: 1/3uvmax < uv < 2/3uvmax
                        amp2_XX = numpy.average(zdat2_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp2_XXerror = numpy.std(zdat2_XX,ddof=0)
                        amp2_XY = numpy.average(zdat2_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp2_XYerror = numpy.std(zdat2_XY,ddof=0)
                        amp2_YX = numpy.average(zdat2_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp2_YXerror = numpy.std(zdat2_YX,ddof=0)
                        amp2_YY = numpy.average(zdat2_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp2_YYerror = numpy.std(zdat2_YY,ddof=0)
                        Npoint2 = len(zdat2_XX)
                        
                        sum2_xxyy = 0.5 * (amp2_XX+amp2_YY)
                        sum2_xxyy_error = 0.5 * (amp2_XXerror+amp2_YYerror)
                        dif2_xxyy = 0.5 * (amp2_XX-amp2_YY)
                        sum2_xyyx = 0.5 * (amp2_XY+amp2_YX)
                        sum2_xyyx_error = 0.5 * (amp2_XYerror+amp2_YXerror)
                        pol2 = numpy.sqrt(dif2_xxyy**2 + sum2_xyyx**2)
                        
                        ampi2 = numpy.abs(sum2_xxyy)
                        ampi2_error = numpy.abs(sum2_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        if Npoint2 == 0:
                                ampi2 = 0
                                ampi2_error = 0
                                pol2 = 0
                                sum2_xyyx_error = 0
                        
                        # third third: 2/3uvmax < uv < uvmax
                        amp3_XX = numpy.average(zdat3_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp3_XXerror = numpy.std(zdat3_XX,ddof=0)
                        amp3_XY = numpy.average(zdat3_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp3_XYerror = numpy.std(zdat3_XY,ddof=0)
                        amp3_YX = numpy.average(zdat3_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp3_YXerror = numpy.std(zdat3_YX,ddof=0)
                        amp3_YY = numpy.average(zdat3_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp3_YYerror = numpy.std(zdat3_YY,ddof=0)
                        Npoint3 = len(zdat3_XX)
                        
                        sum3_xxyy = 0.5 * (amp3_XX+amp3_YY)
                        sum3_xxyy_error = 0.5 * (amp3_XXerror+amp3_YYerror)
                        dif3_xxyy = 0.5 * (amp3_XX-amp3_YY)
                        sum3_xyyx = 0.5 * (amp3_XY+amp3_YX)
                        sum3_xyyx_error = 0.5 * (amp3_XYerror+amp3_YXerror)
                        pol3 = numpy.sqrt(dif3_xxyy**2 + sum3_xyyx**2)
                        
                        ampi3 = numpy.abs(sum3_xxyy)
                        ampi3_error = numpy.abs(sum3_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        
                        # print all results
                        if Npoint1 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi1,ampi1_error, Npoint1, 1*uvmax/3, 1*uvmaxkL/3))
			else:
				f1.write('               no data\n')

                        if Npoint2 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi2,ampi2_error, Npoint2, 2*uvmax/3, 2*uvmaxkL/3))
                        else:
				f1.write('               no data\n')

                        if Npoint3 > 0:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n\n' % (quasarNames[ii],avgfreq,ampi3,ampi3_error, Npoint3, uvmax, uvmaxkL))
                        else:
				f1.write('               no data\n')
                                
                        ampi123 = [ampi1,ampi2,ampi3]
                        ampi123_error = [ampi1_error,ampi2_error,ampi3_error]
                        pol123 = [pol1,pol2,pol3]
                        sum123_xyyx_error = [sum1_xyyx_error,sum2_xyyx_error,sum3_xyyx_error]
                        Npoint123 = [Npoint1,Npoint2,Npoint3]
                        zwei = numpy.sqrt(Npoint123)
                        zstructure = 0
                        if any(Npoint123)!=0:
                                # Extended emission
                                if ampi1_error > 2*ampi3_error:
                                        zstructure = 1
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be EXTENDED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 1*uvmaxkL/3
                                        uvmaximum = -uvmaxkL
                                # Resolved emission
                                zratio = (ampi1-ampi3)/ampi1
                                if zratio < 0.9 and numpy.abs(ampi1-ampi3) > 2*ampi3_error:
                                        zstructure = 2
                                        ampi = ampi1
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be RESOLVED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = pol1
                                        ampp_error = sum1_xyyx_error
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        # compute uvmax
                                        uv85 = 5*uvmax/6 * numpy.sqrt(0.15/zratio)
                                        uvmaximum = uv85 * avgfreq / 300.0
                                if zstructure == 0:
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = numpy.max(ampi123_error)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        uvmaximum = -uvmaxkL
                                #
                                # PRINT OUTPUT
                                #outline= '%15s,NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f , %4.2f , NE, NE, %7.1f, 100.0, %10s\n' % (sourcesList[i][0:9],avgfreq,ampi,ampi_error, perc, perc_error, uvmaxkL, self.zobsdate)
                                outline= '%12s, NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f, %4.2f, NE, NE, %7.1f, %7.1f, %10s\n' % (quasarNames[ii],avgfreq,ampi,ampi_error, perc, perc_error, uvminimum, uvmaximum, self.zobsdate)
                                f.write(outline)
			#
                    #
                #
                f1.close()
                tb.close()
        
	#==================================================#
        # BAND 6, 7 and 9: Adding all spw
        def getFlux(self,visavg,f):
                tb.open(visavg)
                quasarNames = self.quasarNames
                zfreq = self.zfreq
                nant = self.nant
                avgfreq = 0.0
                for i in range (0,self.nspw):
                        avgfreq = avgfreq + zfreq[i]
                #       
                avgfreq = avgfreq/4.0
		zfileAmp = 'output_files/source_structure.txt'
		f1 = open (zfileAmp, 'w')
                f1.write ('\n\n      ANALYSIS OF SOURCE RESOLUTIONS IN THREE PARTS\n\n')
                f1.write ('       SOURCE     FREQ   AVG   RMS  #PTS    n/3*UVMAX  \n')
                f1.write ('                                            (m)     (kL) \n')
                for ii in range (0,self.nquasars):
                    cnId = str(self.quasarIds[ii])
                    tb1 = tb.query('FIELD_ID == '+cnId)
                    zdata = tb1.getcol('CORRECTED_DATA')
		    ztimestamp = tb1.getcol('TIME')
        	    if len(zdata) > 0:
                        zdatXX = zdata[0][0]
			self.nanteff.append(int(0.5*(1.+numpy.sqrt(1.+8.*float(len(zdatXX))/4./float(len(set(ztimestamp)))))+0.5))
			zdatXY = zdata[1][0]
                        zdatYX = zdata[2][0]
                        zdatYY = zdata[3][0]
                        zuvw = tb1.getcol('UVW')
                        zuv2 = zuvw[0]**2 + zuvw[1]**2
                        zuv = numpy.sqrt(zuv2)
                        uvmax = numpy.max(zuv)
                        uvmaxkL = uvmax * avgfreq / 300.0
                        # create the data arrays for each uv region
                        zdat1_XX = []
                        zdat1_XY = []
                        zdat1_YX = []
                        zdat1_YY = []
                        zdat2_XX = []
                        zdat2_XY = []
                        zdat2_YX = []
                        zdat2_YY = []
                        zdat3_XX = []
                        zdat3_XY = []
                        zdat3_YX = []
                        zdat3_YY = []
                        for j in range (0,len(zuv)-1):
                                if zuv[j] < 1*uvmax/3:
                                        zdat1_XX.append(zdatXX[j])
                                        zdat1_XY.append(zdatXY[j])
                                        zdat1_YX.append(zdatYX[j])
                                        zdat1_YY.append(zdatYY[j])
                                #
                                if 1*uvmax/3 < zuv[j] < 2*uvmax/3:
                                        zdat2_XX.append(zdatXX[j])
                                        zdat2_XY.append(zdatXY[j])
                                        zdat2_YX.append(zdatYX[j])
                                        zdat2_YY.append(zdatYY[j])
                                #
                                if 2*uvmax/3 < zuv[j] < uvmax:
                                        zdat3_XX.append(zdatXX[j])
                                        zdat3_XY.append(zdatXY[j])
                                        zdat3_YX.append(zdatYX[j])
                                        zdat3_YY.append(zdatYY[j])
                                #
                        #
                        # first third: uv < 1/3uvmax
                        amp1_XX = numpy.average(zdat1_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp1_XXerror = numpy.std(zdat1_XX,ddof=0)
                        amp1_XY = numpy.average(zdat1_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp1_XYerror = numpy.std(zdat1_XY,ddof=0)
                        amp1_YX = numpy.average(zdat1_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp1_YXerror = numpy.std(zdat1_YX,ddof=0)
                        amp1_YY = numpy.average(zdat1_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp1_YYerror = numpy.std(zdat1_YY,ddof=0)
                        Npoint1 = len(zdat1_XX)
                               
                        sum1_xxyy = 0.5 * (amp1_XX+amp1_YY)
                        sum1_xxyy_error = 0.5 * (amp1_XXerror+amp1_YYerror)
                        dif1_xxyy = 0.5 * (amp1_XX-amp1_YY)
                        sum1_xyyx = 0.5 * (amp1_XY+amp1_YX)
                        sum1_xyyx_error = 0.5 * (amp1_XYerror+amp1_YXerror)
                        pol1 = numpy.sqrt(dif1_xxyy**2 + sum1_xyyx**2)
                                
                        ampi1 = numpy.abs(sum1_xxyy)
                        ampi1_error = numpy.abs(sum1_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                                
                        # second third: 1/3uvmax < uv < 2/3uvmax
                        amp2_XX = numpy.average(zdat2_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp2_XXerror = numpy.std(zdat2_XX,ddof=0)
                        amp2_XY = numpy.average(zdat2_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp2_XYerror = numpy.std(zdat2_XY,ddof=0)
                        amp2_YX = numpy.average(zdat2_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp2_YXerror = numpy.std(zdat2_YX,ddof=0)
                        amp2_YY = numpy.average(zdat2_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp2_YYerror = numpy.std(zdat2_YY,ddof=0)
                        Npoint2 = len(zdat2_XX)
                                
                        sum2_xxyy = 0.5 * (amp2_XX+amp2_YY)
                        sum2_xxyy_error = 0.5 * (amp2_XXerror+amp2_YYerror)
                        dif2_xxyy = 0.5 * (amp2_XX-amp2_YY)
                        sum2_xyyx = 0.5 * (amp2_XY+amp2_YX)
                        sum2_xyyx_error = 0.5 * (amp2_XYerror+amp2_YXerror)
                        pol2 = numpy.sqrt(dif2_xxyy**2 + sum2_xyyx**2)
                                
                        ampi2 = numpy.abs(sum2_xxyy)
                        ampi2_error = numpy.abs(sum2_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                        if Npoint2 == 0:
                                ampi2 = 0
                                ampi2_error = 0
                                pol2 = 0
                                sum2_xyyx_error = 0
                                
                        # third third: 2/3uvmax < uv < uvmax
                        amp3_XX = numpy.average(zdat3_XX,weights=None)
                        #dfreeXX = len(zdatXX)-nant
                        amp3_XXerror = numpy.std(zdat3_XX,ddof=0)
                        amp3_XY = numpy.average(zdat3_XY,weights=None)
                        #dfreeXY = len(zdatXY)-nant
                        amp3_XYerror = numpy.std(zdat3_XY,ddof=0)
                        amp3_YX = numpy.average(zdat3_YX,weights=None)
                        #dfreeYX = len(zdatYX)-nant
                        amp3_YXerror = numpy.std(zdat3_YX,ddof=0)
                        amp3_YY = numpy.average(zdat3_YY,weights=None)
                        #dfreeYY = len(zdatYY)-nant
                        amp3_YYerror = numpy.std(zdat3_YY,ddof=0)
                        Npoint3 = len(zdat3_XX)
                                
                        sum3_xxyy = 0.5 * (amp3_XX+amp3_YY)
                        sum3_xxyy_error = 0.5 * (amp3_XXerror+amp3_YYerror)
                        dif3_xxyy = 0.5 * (amp3_XX-amp3_YY)
                        sum3_xyyx = 0.5 * (amp3_XY+amp3_YX)
                        sum3_xyyx_error = 0.5 * (amp3_XYerror+amp3_YXerror)
                        pol3 = numpy.sqrt(dif3_xxyy**2 + sum3_xyyx**2)
                                
                        ampi3 = numpy.abs(sum3_xxyy)
                        ampi3_error = numpy.abs(sum3_xxyy_error)
                        #if ampi_error < 0.05*ampi: ampi_error = 0.05*ampi
                                
                        # print all results
                        if Npoint1 > 5:
			        f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi1,ampi1_error, Npoint1, 1*uvmax/3, 1*uvmaxkL/3))
			else:
				f1.write('               no data\n')

                        if Npoint2 > 5:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n' % (quasarNames[ii],avgfreq,ampi2,ampi2_error, Npoint2, 2*uvmax/3, 2*uvmaxkL/3))
                        else:
				f1.write('               no data\n')

                        if Npoint3 > 5:
				f1.write('%15s %7.2f %5.2f %5.2f %4d %7.1f %7.1f\n\n' % (quasarNames[ii],avgfreq,ampi3,ampi3_error, Npoint3, uvmax, uvmaxkL))
                        else:
				f1.write('               no data\n')
                                
                        ampi123 = [ampi1,ampi2,ampi3]
                        ampi123_error = [ampi1_error,ampi2_error,ampi3_error]
                        pol123 = [pol1,pol2,pol3]
                        sum123_xyyx_error = [sum1_xyyx_error,sum2_xyyx_error,sum3_xyyx_error]
                        Npoint123 = [Npoint1,Npoint2,Npoint3]
                        zwei = numpy.sqrt(Npoint123)
                        zstructure = 0
                        if any(Npoint123)!=0:
                        # Extended emission
                                if ampi1_error > 2*ampi3_error:
                                        zstructure = 1
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be EXTENDED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 1*uvmaxkL/3
                                        uvmaximum = -uvmaxkL
                                # Resolved emission
                                zratio = (ampi1-ampi3)/ampi1
                                if zratio < 0.9 and numpy.abs(ampi1-ampi3) > 2*ampi3_error:
                                        zstructure = 2
                                        ampi = ampi1
                                        ampi_error = ampi1_error
                                        warning='# source '+quasarNames[ii]+' considered to be RESOLVED, check uv plot and image to confirm\n'
                                        #print warning
                                        f.write(warning)
                                        ampp = pol1
                                        ampp_error = sum1_xyyx_error
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        # compute uvmax
                                        uv85 = 5*uvmax/6 * numpy.sqrt(0.15/zratio)
                                        uvmaximum = uv85 * avgfreq / 300.0
                                if zstructure == 0:
                                        ampi = numpy.average(ampi123,weights=zwei)
                                        ampi_error = numpy.max(ampi123_error)
                                        ampp = numpy.average(pol123,weights=zwei)
                                        ampp_error = numpy.max(sum123_xyyx_error)
                                        perc = ampp/ampi
                                        perc_error = ampp_error/ampi
                                        if perc_error < 0.01: perc_error = 0.01
                                        if perc_error > perc: perc = 0.5*(perc+perc_error); perc_error = perc
                                        uvminimum = 0
                                        uvmaximum = -uvmaxkL
                                #
                                # PRINT OUTPUT
                                #outline= '%15s, NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f, %4.2f, NE, NE, %7.1f, 100.0, %10s\n' % (sourcesList[i][0:9],avgfreq,ampi,ampi_error, perc, perc_error, uvmaxkL, self.zobsdate)
                                outline= '%12s, NE, NE, NE, NE, %7.2fE+09, %5.2f, %5.2f,  %4.2f, %4.2f, NE, NE, %7.1f, %7.1f, %10s\n' % (quasarNames[ii],avgfreq,ampi,ampi_error, perc, perc_error, uvminimum, uvmaximum, self.zobsdate)
                                f.write(outline)
                        #
                    #
                #
                f1.close()
                tb.close()
        
        #==================================================#
	#
	#### MAIN PROGRAM ###
	#
	def dorun(self):
		from split import split
		zvis = self.zvis
                
                self.getSpecInfo(zvis)
                if self.zref == None:
                        self.getAntInfo(zvis)
                        self.ref_ant = str(self.ant_name[0])
                else:
                        self.ref_ant = self.zref
                
                #-------------------------------------
                # apriori calibrations: wvr and tsys
		if self.zWVR == True:
			print '******* Perform WVR correction ********'
			self.doWVR(zvis)
                
                print '******* Perform Tsys *******'
                self.getAntInfo(zvis)
                
		self.doTsys(zvis)
		#  Plot tsys
		#if self.zdisplay == True:
			#plotcal(caltable = self.tsysfile, xaxis = 'chan', yaxis = 'amp', plotrange = [0,0,0,300], subplot = 221, iteration = 'spw,antenna')
		#
		#-------------------------------------
		# flagging
		#  Make sure planet models are in to get the planets coordinates before flagging for shadowing
		self.getSources(zvis,self.zbandpass, self.zplanet)
		print "******* Flux calibrator(s) = ", self.fluxCalNames
		if self.zuseplanet: self.putModel(zvis)
		# do initial flagging
		self.aprioriFlag(zvis)
                # additional flagging
		self.addFlag(zvis)

		#-------------------------------------
		# split science spws
                self.getSpecInfo(zvis)
		zsplit = self.zplit
                #print 'stuff ', zvis, zsplit, self.dspw
		print '******* spliting data from ',zvis, ' to ',zsplit
		split(vis = zvis, outputvis = zsplit, datacolumn = 'corrected', spw = self.dspw, keepflags = False)

		self.flagEdgeChan(zsplit)

		self.getSources(zsplit,self.zbandpass, self.zplanet)

		if self.zuseplanet: self.putModel(zsplit)

                # get new spws ids, to be 0,1,2,3
                self.getSpecInfo(zsplit)

                print '*******'
                print '******* SPECTRAL INFORMATION'
		print "******* Number of spectral windows = ", self.nspw
		print "******* Number of channels = ", self.nchan
		print "******* Science spw = ", self.dspw
                print '******* ALMA band =', self.zband
		print '******* frequencies =', self.zfreq
		print '*******'
		print '******* SOURCE INFORMATION'
		print "******* Number of sources   = ", len(self.sourcesList)
		print "******* All sources observed = ", self.sourcesList
		if self.zuseplanet:
			print "******* Flux calibrator(s)  = ", self.fluxCalNames
		else:
			print "******* Assume bandpass as flux calibrator: ", self.zbandpass, ' with flux = ', self.zflux, ' Jy'

                print "******* All observed quasars = ", self.quasarNames
                print '*******'

		#-------------------------------------
		# bandpass calibration

		# First choose the reference antenna
		if self.zref == None:
			ref_ant = self.get_reference(zsplit)
			self.ref_ant = ref_ant
		else:
			self.ref_ant = self.zref
                        self.getAntInfo(zsplit)
                
                print '******* ANTENNA INFORMATION'
                print '******* number of antennas =', self.nant
		print '******* Using reference antenna= ', self.ref_ant
                print '*******'

		if self.zbandpass == None:
			self.getBpCal(zsplit)
                        self.BandpassName = self.BandpassName[0]
		else:
			self.BandpassName = self.zbandpass
			vm = aU.ValueMapping(zsplit)
                        BandpassID = vm.getFieldIdsForFieldName(self.BandpassName)
                        self.BandpassID = str(BandpassID[0])

		print "******* Using bandpass/gain calibrator = ", self.BandpassName, "  ID= ", self.BandpassID
		print "******* Performing Bandpass"
		self.doBandpass(zsplit)

		# plot gain and bandpass solutions
		if self.zdisplay == True:
			plotcal(caltable = self.BandpassName+'fcal_g1', xaxis = 'time', yaxis = 'amp', plotrange=[0,0,0.0,1.0], subplot = 221, iteration = 'spw,antenna')
			plotcal(caltable = self.BandpassName+'fcal_g1', xaxis = 'time', yaxis = 'phase', plotrange=[0,0,-180,180], subplot = 221, iteration = 'spw,antenna')
			plotcal(caltable = self.BandpassName+'fcal_b1', xaxis = 'chan', yaxis = 'amp', plotrange=[0,0,0.9,1.2], subplot = 221, iteration = 'spw,antenna')
			plotcal(caltable = self.BandpassName+'fcal_b1', xaxis = 'chan', yaxis = 'phase', plotrange=[0,0,-180,180], subplot = 221, iteration = 'spw,antenna')

		# ant_gain_check on bandpass calibrator
                self.checkGain(zsplit)
                
                #-------------------------------------
		# gain calibration
		print '******* Calculating Temporal Gain Calibration using bandpass source'
                self.doGains(zsplit)
		#
		if self.zdisplay == True:
			plotcal(caltable = self.BandpassName+'fcal_g2', xaxis = 'time', yaxis = 'phase', plotrange=[0,0,-180,180], subplot = 221, iteration = 'spw,antenna')
			plotcal(caltable = self.BandpassName+'fcal_g3', xaxis = 'time', yaxis = 'phase', plotrange=[0,0,-180,180], subplot = 111)
			plotcal(caltable = self.BandpassName+'fcal_g3', xaxis = 'time', yaxis = 'amp', plotrange=[0,0,0.0,0.0], iteration = 'spw,antenna', subplot = 221)
			plotcal(caltable = self.BandpassName+'pcal_g1', xaxis = 'time', yaxis = 'phase', poln = '', plotrange=[0,0,-180,180], subplot = 221, iteration = 'spw,antenna')
 
		#  Apply all calibrations
		print '******* Apply bandpass and gain calibration of bandpass source'
		applycal(vis = zsplit, gaintable = [self.BandpassName+'fcal_b1', self.BandpassName+'fcal_g2', self.BandpassName+'fcal_g3',self.BandpassName+'pcal_g1'])
		#  Check data

		if self.zdisplay == True:
			plotms(vis = zsplit, xaxis = 'uvdist', yaxis = 'amp', ydatacolumn = 'corrected', correlation = 'XX,YY', avgchannel = 'self.nchan', avgspw = True, coloraxis = 'corr', iteraxis = 'field', xselfscale = True, yselfscale = True)
                
                #-------------------------------------
		#  Check for crossed-hand delay offsets
                self.doKCross(zsplit)
                
		if self.zdisplay == True:
                        plotms(vis = zsplit, xaxis = 'channel', yaxis = 'phase', ydatacolumn = 'corrected', correlation = 'XY,YX', avgtime = '50000', antenna = '', field = self.BandpassID, avgscan = True, avgbaseline = True, spw = '', iteraxis = 'spw', coloraxis = 'corr')
                
                #-------------------------------------
		#  split out and average data
		visavg = self.asdmName+'_avg.ms'
		print '******* Averaging data from ', zsplit, ' to ', visavg
		split(vis = zsplit, outputvis = visavg, datacolumn = 'corrected', timebin = '200s', combine = 'scan', width = self.nchan, keepflags = False)

		flagmanager(vis = visavg, mode='save', versionname='original')

	        #-------------------------------------
		#  NOW DO THE ANALYSIS on averaged data
		self.getSources(visavg,self.zbandpass,self.zplanet)
		print "Flux calibrator(s) = ", self.fluxCalNames
		if self.zuseplanet:
			self.putModel(visavg)
			fluxCalUVlim = self.getUVlim()

                        factorXX,factorYY = self.getScaleFactor(visavg)

                print '******* Apply scale factor to all sources --> corrected fluxes'
		# Was commented, hence giving error
		# un-commented by AH and EF on 08/01/2013
                self.applyFactor(visavg)

		#  Data calibrated

                #--------------------------------------
                #  Get antenna-based fluxes for more editing check
                #  Get antenna-based residuals
                self.getAntFlags(visavg)
                
                #-------------------------------------
                # produce images for all sources
                
                print '******* Imaging sources *******'
                if self.zImage == True:
                        self.doImage(visavg)
                
		#-------------------------------------
		# Get flux densities for all quasars

		zobsdate,ztoday = self.getDates()
		self.zobsdate = zobsdate
                zflux = self.zflux
                
                if self.Type == 'Grid' : zfileRes = 'ALMA_CalSurveyResults_'+self.asdmName+'_g_B'+self.zband+'_'+ztoday+'.txt'
                if self.Type == 'Cone' : zfileRes = 'ALMA_CalSurveyResults_'+self.asdmName+'_c_B'+self.zband+'_'+ztoday+'.txt'
                if self.Type == 'ACA' : zfileRes = 'ALMA_CalSurveyResults_'+self.asdmName+'_a_B'+self.zband+'_'+ztoday+'.txt'
                #if self.Type == 'Asteroid' : zfileRes = 'ALMA_CalSurveyResults_'+self.asdmName+'_s_B'+self.zband+'_'+ztoday+'.txt'
		#if self.Type == 'Grid' : zfileRes = 'ALMA_Calibration_Grid_survey_results_B'+self.zband+'_'+ztoday+'.txt'
		#if self.Type == 'Cone' : zfileRes = 'ALMA_Calibration_Cone_survey_results_B'+self.zband+'_'+self.BandpassName+'_'+ztoday+'.txt'
		#if self.Type == 'Asteroid' : zfileRes = 'ALMA_Calibration_Asteroid_survey_results_B'+self.zband+'_'+ztoday+'.txt'
		os.system('rm '+zfileRes)
		f = open (zfileRes, 'w')
		f.write('#\n')
		f.write('# #################################################################################################\n')
		f.write('# Measurements on '+self.zobsdate+' at Band '+self.zband+' produced by '+self.reducer+'\n')
		f.write('# ASDM used: '+self.asdmName+'\n')
		f.write('#\n')

                if os.path.isdir(self.asdmName) == True and self.Type != 'ACA':
                        f.write('# Weather conditions: \n')
                        self.getWeather(zvis,f)
                        f.write('#\n')
		if self.zWVR == True:
			f.write('# WVR correction applied; antennas flagged for WVR: +self.flagWVR \n')
		else:
			f.write('# No WVR correction applied \n')
                
                if os.path.isfile(self.flagdataFile):
			f.write('# Additional flags applied in: '+self.asdmName+'.flagdata\n')
		else:
			f.write('# No additional flags applied\n')
			 
		f.write('# Using reference antenna: '+self.ref_ant+' \n')
                f.write('# Bandpass calibrator: '+self.BandpassName+' \n')
                
                if self.zuseplanet:
			f.write('# Absolute Flux Density from: '+self.fluxCalNames[0]+' \n')
			if len(self.fluxCalNames)>1:
                                f.write('# Ratios between SS objects: objects, ratioXX, ratioYY \n')
	        		ratioXX_pln,ratioYY_pln = self.getRatios(visavg,factorXX,factorYY,f)
		        #
                else:
			f.write('# No planet observed. Bandpass used as flux calibrator: \n')
                        bandpassFlux='%8s %6.2f %4s \n'% (self.BandpassName, self.zflux, 'Jy')
                        f.write('# '+bandpassFlux)
		#
		f.write('#\n')

                f.write('#           source scan elev \n')
		self.getSouElev(visavg,f)

                f.write('#\n')
		f.write('# Data Format[Units]:\n')
		f.write('# sourceName , ra [Hour:Min:Sec] , ra_err [arcsec], dec [Degree:Min:Sec], dec_err [arcsec],\n')
		f.write('# frequency [Hz], flux [Jy], flux_err [Jy],\n')
		f.write('# degree [%], degree_err [%],  angle [deg], angle_err [deg],\n')
		#f.write('# max_baseline [klambda], min_flux [%], date_observed [YYYY-MM-DDT]\n')
                f.write('# min_baseline [klambda], max_baseline [klambda], date_observed [YYYY-MM-DDT]\n')
		f.write('# NE means No Entry\n')
		f.write('#\n')
		f.write('# #################################################################################################\n')
		#f.write('# source                       freq        S     dS    %P    d%P             uvmx  % res     Date\n')
                f.write('#    source                       freq        S     dS     %P    d%P            uvmin   uvmax     Date\n')

		if (self.zband == '3') or (self.zsidebands):
			self.getFluxB3(visavg,f)
		else:
			self.getFlux(visavg,f)

		f.write('# #################################################################################################\n')
		f.write('# Effective number of antennas: '+str(numpy.mean(self.nanteff))+'\n')
		f.close()
		os.system('cat '+zfileRes)
		print 'Band was ', self.zband
		print 'Effective antenna numbers per source ', self.nanteff

#def angularSeparation(ra0,dec0,ra1,dec1):
#      ra0 *= math.pi/180.
#      dec0 *= math.pi/180.
#      ra1 *= math.pi/180.
#      dec1 *= math.pi/180.
#      deltaLong = ra0-ra1
#      argument1 = (((math.cos(dec1)*math.sin(deltaLong))**2) + ((math.cos(dec0)*math.sin(dec1)-math.sin(dec0)*math.cos(dec1)*math.cos(deltaLong))**2))**0.5
#      argument2 = math.sin(dec0)*math.sin(dec1) + math.cos(dec0)*math.cos(dec1)*math.cos(deltaLong)
#      angle = math.atan2(argument1, argument2) / (math.pi/180.)
#      return(angle)


class checkEd_cat:
	def dec_parse(self,decstring):
		"""
		dec = dec_parse(decstring)

		parse a colon separated string representing declination ito
		degrees.
		"""
		dec = 0.0

		ds = decstring.split(':')
		lds = len(ds)
		if lds >= 1:
			deg = float(ds[0])
			dec += deg
		if lds >= 2:
			minutes = float(ds[1])
			dec += minutes/60.0
		if lds >= 3:
			sec = float(ds[2])
			dec += sec/3600.0
		return dec


	def ra_parse(self,rastring, hours=True):
		"""
		ra = ra_parse(decstring)

		parse a colon separated string representing right ascension ito
		degrees.
		"""
		ra = 0.0

		rs = rastring.split(':')
		lrs = len(rs)
		if lrs >= 1:
			deg = float(rs[0])
			ra += deg
		if hours:
			ra *= 15
		if lrs >= 2:
			minutes = float(rs[1])
			ra += minutes/60.0
		if lrs >= 3:
			sec = float(rs[2])
			ra += sec/3600.0
		return ra


	def checkEdcat(self,RA,Dec,fluxlimit=0.1,separation=10.0):
		"""
		function to check Ed catalogue
		input RA, Dec in format xx:xx:xxx strings
		optional:
		fluxlimit: flux limit at 90 GHz
		separation: separation ind egree
		"""
		ra_s=self.ra_parse(RA)
		dec_s=self.dec_parse(Dec)
		print '*****************************************'
		print ' Backup Cones from the Kneissl/Fomalont list. USE WITH CAUTION'
		print '**** Chosen separation',separation
		print '**** Chosen fluxlimit ',fluxlimit
		print '**** Coorindates:', RA, Dec
		print '*****************************************'
		filename = '/users/tkempen/catalog_gt_0.1.txt'
		f = open(filename)
		i = 0
		j = 0
		for line in iter(f):
			# extract coordinates from line
			i +=1
			if i > 21 :

				line1 = re.sub("\s\s+"," ",line)
				ar=line1.split(' ')
				ra=ar[1]+":"+ar[2]+":"+ar[3] 
				dec=ar[4]+":"+ar[5]+":"+ar[6]
				ra_l=self.ra_parse(ra)
				dec_l=self.dec_parse(dec)
				angle= aU.angularSeparation(ra_s,dec_s,ra_l,dec_l)
				if float(angle) < float(separation) and float(ar[8]) > fluxlimit :
					j +=1
					print "Angular Sep: "+str(round(10*angle)/10.0)+" Name: "+ar[0]+" S90(est.): "+ar[8]+" Jy, Coordinates: "+ra+" "+dec


		f.close()
		print '*****************************************'
		print '**** ',j,' Sources found with separation smaller than ',separation,' degrees above an S90 flux of ',fluxlimit,' Jy'
