from TelCal.AsdmReader import AsdmPlotter

class SamplingOffsets:

   def __init__(self, scans='1', asdm ='', subscans='2'):


       self._scans = scans
       self._asdm = asdm
       self._subscans = subscans


   def getTDMAutoData(self):

       p = AsdmPlotter(self._asdm)
       # autocorrelations:
       p.setX('channel')
       p.setY('autocorr')
       p.setTsysScaling(False)
       p.setSubscans(self._subscans)
       p.setScans(self._scans)
       p.setBasebands('')
       p.setPolarizations('')
       p.configure()
       p.fillData()
       numAntenna = len(p.getAntennas())
       self._autoData = {}
       self._timeData = {}
       self._antennas = p.getAntennas()
       for i in range(p.getNumReaderTables()):
           t = p.getReaderTable(i)
           # print i, t.getAntenna()
           for iant in range(numAntenna):
               if t.getAntenna()== p.getAntennas()[iant]: break
           numChan = t.getNumData()
           name = '%s-%s' % (t.getBasebandNames()[0],
                             t.getPolarizations()[0])
           self._timeData[name] = pl.array(t.getX())
           try: self._autoData[name]
           except KeyError: self._autoData[name]=pl.zeros((numAntenna,numChan))
           self._autoData[name][iant] = pl.array(t.getY())

       self.numChan = numChan

   def getTDMAmpData(self):

       p = AsdmPlotter(self._asdm)
       p.setX('channel')
       p.setY('ampliant')
       p.setTsysScaling(False)
       p.setSubscans(self._subscans)
       p.setScans(self._scans)
       p.setBasebands('')
       p.setPolarizations('')
       p.configure()
       p.fillData()
       numAntenna = len(p.getAntennas())
       self._numAntenna = numAntenna
       self._ampData = {}
       self._timeData = {}
       for i in range(p.getNumReaderTables()):
           t = p.getReaderTable(i)
           for iant in range(numAntenna):
               if t.getAntenna()== p.getAntennas()[iant]: break
           numChan = t.getNumData()
           name = '%s-%s' % (t.getBasebandNames()[0],
                             t.getPolarizations()[0])
           self._timeData[name] = pl.array(t.getX())
           try: self._ampData[name]
           except KeyError: self._ampData[name]=pl.zeros((numAntenna,numChan))            
           self._ampData[name][iant] = pl.array(t.getY())
       self.numChan = numChan
       self._offsets =  self._ampData

   def printOffsets(self, ich=0):

       self.getTDMAutoData()
       self.getTDMAmpData()
       line = '%10s '%' '
       numAntenna = self._numAntenna
       for iant in range(numAntenna):
           line += ' %6s'% self._antennas[iant]
       print line
       names = self._timeData.keys()
       names.sort()
       for name in names:
           line = '%10s ' %name
           for iant in range(numAntenna):
               line += ' %6.3f'% self._ampData[name][iant][ich]
           print line


   def plotCorrectedAuto(self, antenna, maxChannel=10):

       self.getTDMAutoData()
       self.getTDMAmpData()
       names = self._timeData.keys()
       names.sort()
       np = len(names)
       pl.gcf().clf()
       k = 0
       for iant in range(self._numAntenna):
           if antenna== so._antennas[iant]: break
       correction = pl.zeros(maxChannel)

       for name in names:
           k+= 1
           pl.subplot(2,4,k)
           pl.plot(self._autoData[name][iant][0:maxChannel],'o-r')
           pl.title('%s-%s' % (antenna,name))
           for ich in range(maxChannel):
               correction[ich] = 1.-self._ampData[name][iant][ich]
           pl.plot(self._autoData[name][iant][0:maxChannel]*correction**2,'o-g')
           pl.xlim(-3.,maxChannel)
           pl.ylim(ymin=0)
           pl.text(-2., 0.001, '%5.3f'%self._ampData[name][iant][0])
       pl.savefig(asdm+'-'+antenna+'-corrected.png')


   def getTDMRmsData(self, ch1=8, ch2=120):
       # look at the rms in amplitude for all baselines

       p = AsdmPlotter(self._asdm)
       p.setX('channel')
       p.setY('imag')
       p.setSubscans(self._subscans)
       p.setScans(self._scans)
       p.setBasebands('')
       p.setPolarizations('')
       p.configure()
       p.fillData()
       numAntenna = len(p.getAntennas())
       self._numAntenna = numAntenna
       for i in range(p.getNumReaderTables()):
           t = p.getReaderTable(i)
           for iant in range(numAntenna):
               if t.getAntenna()== p.getAntennas()[iant]: break
           numChan = t.getNumData()
           name = '%s-%s' % (t.getBasebandNames()[0],
                             t.getPolarizations()[0])
           #self._timeData[name] = pl.array(t.getX())
           #try: self._ampData[name]
           #except KeyError: self._ampData[name]=pl.zeros((numAntenna,numChan))            
           data = pl.array(t.getY())[ch1:ch2]
           rms = pl.sqrt((data**2).mean())
           print name, t.getBaseline(), rms*sqrt(2*2.e9/128*5.)
       #self.numChan = numChan
       #self._offsets =  self._ampData

pl.clf()

offsets={}
# to run, on STE:
# casapy;  execfile('sampling_offsets.py');
# edit the asdm and scans to point to the atmCals
# cut & paste the following sections

asdm = 'uid___A002_X722b5e_X26'
atm_scans=[1]

#asdm='uid___A002_X7215ac_X91'
#asdm='uid___A002_X722b5e_X26'

for scan in atm_scans:
   so = SamplingOffsets(asdm=asdm, scans=str(scan), subscans='2')
   so.printOffsets()
   names = so._timeData.keys()
   names.sort()
   for name in names:
       try: offsets[name]
       except KeyError: offsets[name] = {}
       for iant in range(so._numAntenna):
           try: offsets[name][so._antennas[iant]]
           except KeyError: offsets[name][so._antennas[iant]] = []
           offsets[name][so._antennas[iant]].append(so._ampData[name][iant][0])
for name in names:
   ants =  offsets[name].keys()
   ants.sort()
   for antenna in ants:
       off = pl.array( offsets[name][antenna])
       # print off
       print '%s %s  min %6.3f max %6.3f average %6.3f rms %6.3f ' %\
             ( name, antenna, off.min(), off.max(), off.mean(), pl.sqrt(off.var()))

alloffsetsmean=[]
for name in names:
   ants =  offsets[name].keys()
   ants.sort()
   for antenna in ants:
       off = pl.array( offsets[name][antenna])
       alloffsetsmean.append(off.mean())

pl.clf()
all = pl.array(alloffsetsmean)
all.sort()
len(all)
pl.hist(all, pl.arange(0.0,0.25,0.01))
pl.xlabel('offset')
pl.ylabel('number of offsets')

pl.savefig(so._asdm+'-offsetHistogram.png')

pl.clf()
color={'XX':'g', 'YY':'b'}
pos = {'XX':0.80, 'YY':0.90}
pl.subplots_adjust(hspace=0.05)
for k in offsets.keys():
    bb = k[0:4]
    pol = k[5:7]
    iplot = int(bb[3])
    nant = len(so._antennas)
    yy = pl.zeros(nant)
    xx = pl.arange(nant)
    i = 0
    for a in so._antennas:
        yy[i] = offsets[k][a][0]
        i += 1

    ax = pl.subplot(4,1,iplot)
    ax.set_xticklabels([])
    pl.plot(xx, yy, 'o', color=color[pol])
    pl.ylabel(bb+' offset')
    pl.text(0.015,pos[pol], pol, ha='left',va='center',
            color=color[pol], transform=ax.transAxes)
    pl.plot((0.01,),(pos[pol],), 'o',color=color[pol], transform=ax.transAxes)
    pl.xlim(-1.0,nant)
    pl.grid()
    if iplot==4:
        pl.xticks(xx,so._antennas,horizontalalignment='left',
                 size='small',rotation='-45')
    if iplot==1:
        pl.title(so._asdm)

pl.savefig(so._asdm+'-offsetSummary.png')
