#!/usr/bin/env python
# simulateDelayServer.py
"""
simulateDelayServer.py
written by R Hiriart
transcribed into python by Denis

$Id: simulateDelayServer.py,v 1.1 2014/07/03 17:46:31 dbarkats Exp $
"""

# Execution starts here
import os, sys
import math
from pylab import *
import analysisUtils as au

# define physical constants
c = 299792458

class simulateDelayServer(object):
    """
    """

    def __init__(self):

        almaLatLong =au.getObservatoryLatLong('ALMA')
        self.almaLat = almaLatLong[0][0]
        self.almaLong = almaLatLong[1][0]

        #print self.almaLat,self.almaLong
        #ref = [2225205.0920000002000, -5440307.7740000002000, -2481019.0660000001000]
        #self.almaLat = math.atan2(ref[2],sqrt(ref[0]**2+ref[1]**2)) *180/pi
        
        print "### Running simulateDelayServer with lat/long = %f, %f" %(self.almaLat,self.almaLong)
        
    def uvw(self,H,Dec,antPos,refPos):
        """
        
        uvw - Returns spatial frequency vector in meters.
        H     Hour angle in hours
        Dec   Declination in degrees
        antPos    Antenna position vector in geocentric coordinates (meters)
        refPos    Reference position vector in geocentric coordinates (meters)
        """
        
        latRad = deg2rad(self.almaLat)
        R = matrix([[cos(latRad), sin(latRad), 0],\
                    [-sin(latRad), cos(latRad), 0],\
                    [0, 0, 1]])
        baseline = dot(R,(refPos - antPos))
        # Uncomment to return uvw in wavelengths.
        #lmda = c / frequency
        #baseline = baseline / lmda
        H = deg2rad(360.0/24.0*H)
        Dec = deg2rad(Dec)
        T = matrix([[sin(H), cos(H), 0],\
             [-sin(Dec)*cos(H), sin(Dec)*sin(H), cos(Dec)],\
             [cos(Dec)*cos(H), -cos(Dec)*sin(H), sin(Dec)]])
        uvw = dot(T , transpose(baseline))
        
        return uvw
    
    def calculateGeometricDelay(self):
        """
        Example 1:
        Calculates the geometrical delay in seconds, for:
        Antenna in antPos
        Array reference in refPos
        Source with RA, Dec  in radians
        UT date YYYY-MM-DDTHH:MM:SS.XXXX
        """
        antPos = array([2225081.0304043, -5440064.01101605, -2481655.10926552])
        refPos = array([2225205.092, -5440307.774, -2481019.066])
        sourcerad = [6.013092907445328, -0.4882134709852743]
        sourcedeg = rad2deg([6.013092907445328, -0.4882134709852743]) 
        RA = sourcedeg[0]/15.0
        Dec = sourcedeg[1]
        print "RA/DEC = %f/%f " %(RA,Dec)
        start='2012-11-01T01:57:04.967'
        mjdsec = au.dateStringToMJDSec(start,verbose=False)
        mjd = au.dateStringToMJD(start, verbose = False)
        LST = au.ComputeLST(mjdsec,self.almaLong)
        H = LST - RA
        
        UVW = self.uvw(H, Dec, antPos, refPos)
        tau = UVW[2]/c
        azel = au.computeAzElFromRADecMJD(sourcerad, mjd)
        print 'MJD = %f'%mjd
        print "Geometrical delay: %.6e s = %.3f ns" %(tau,tau*1e9)
        print 'AZ / EL : %f, %f'%( azel[0]+2*pi,azel[1])
        print 'U: %f' %UVW[0]
        print 'V: %f' %UVW[1]
        print 'W: %f' %UVW[2]

                                   
        return
#
#    // ============================================================================
#    // Example 2:
#    // Calculates geometrical delay and UV curves for a 24 hour period
#    // It leaves these fields as zero if the source is not observable.
#    //
#    // latitude = AlmaLatitude()
#    // longitude = AlmaLongitude()
#    // H = zeros(100, 1)
#    // tau = zeros(100, 1)
#    // U = zeros(100, 1)
#    // V = zeros(100, 1)
#    //for i = 1 : 100
#    //    H(i) = (24.0 / 100.0) * i
#    //    AzEl = toHorizontal(H(i), Dec, latitude)
#    //    UVW = uvw(H(i), Dec, ant, ref)
#    //    if AzEl(2) > 0 then
#    //        tau(i) = UVW(3) / c
#    //        U(i) = UVW(1)
#    //        V(i) = UVW(2)
#    //    else
#    //        tau(i) = 0
#    //        U(i) = 0
#    //        V(i) = 0
#    //    end
#    //end
    
