#!/usr/bin/env python

# v1.11    30 Nov 2015
#
# telcal-phase.py   for offline recording of WVR correction.  for CSV-3189
#
# this should be run on scops
# execfile(telcal-phase.py')
# gng('uid-name',bbn=1,fwrite=True)
#  or gngs('list-filename',bbn=1,fwrite=true)   # to read in multiple gonogo's from list output of
#  shiftlog-query --status "SUCCESS" -it "OTHER" -ib "2015-04-21T00:00:00" -ie "2015-08-01T00:00:00" -sn "gonogo (B3)"
# appends results to /users/wdent/CSV3189/phase_results.txt
# 30 Nov. fix to specific path for asdmExport

import numpy
from numpy import *
from XmlObjectifier import *
import analysisUtils as aU
from analysisUtils import *
import os.path
from scipy import stats

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    
direct=zeros(2)

def gngs(listfile,bbn=1,fwrite=True,verb=False):

  with open(listfile,'r') as fil:
      line_a=[]
      for line in fil:
          line_a.append(line)
  print len(line_a)
  for iline in range(len(line_a)):
#      pl.clf()
      linesplit=line_a[iline].split(',')
      print 'analysing file ',iline, linesplit[5], 'verb=',verb

      EB=linesplit[5].strip('"')
      print EB
      try:
          gng(EB,bbn,fwrite)
      except:
          print 'FAILED to run. going to next file'
      pl.draw()
  print ' DONE'
  

def gng(EB,bbn=1,fwrite=True,verb=False):
#default bbn=1, write to file = True
#for ac in range(1):   # just to run loop once (comment out and use above def otherwise)
  pl.clf()

# baseline monitor lengths
  baselinem=[100.,300.,1000.,3000.,10000.,30000.]
  nmon=len(baselinem)

  asdm=EB.replace('://','___')
  asdm=asdm.replace('/','_')
  if not os.path.exists(asdm):
  	print 'file not found locally - exporting from archive'
  	cmd_export='/opt/DataPacker/bin/asdmExport -m '+EB
  	os.system(cmd_export)

  calph = open(asdm+'/CalPhase.xml')
  calphd = XmlObject(calph.read())
  calph.close()
  rows=calphd.CalPhaseTable.row
  nscans=len(rows)/8  # 8 rows per scan, (U & C, * #spws)
  if verb: print len(rows),nscans, 'rows,scans  in this EB'
  base_mon_c_m=zeros((nscans,nmon)); base_mon_c_s=zeros((nscans,nmon))
  base_mon_u_m=zeros((nscans,nmon)); base_mon_u_s=zeros((nscans,nmon))



# read in weather information
  calw=open(asdm+'/Weather.xml')
  calwx=XmlObject(calw.read())
  calw.close()
  rows=calwx.WeatherTable.row
  nwx=len(rows)
#  print nwx
  winds=zeros(nwx)
  temps=zeros(nwx)
  for ir in range(nwx):
        rown=rows[ir]
        winds[ir]=rown.windSpeed.getValue()
        temps[ir]=rown.temperature.getValue()-273.1
  wind_mean=mean(winds)
  temp_mean=mean(temps)

  for sn in range(nscans):

    if verb: print 'processing scan ',sn
# first find target and flux
    try:
  	calsource=open(asdm+'/Source.xml')
  	calsr= XmlObject(calsource.read())
  	calsource.close()
  	rows=calsr.SourceTable.row
  	nrowsr=len(rows)   # 
  	targets=[];fluxes=[]
        rown=rows[0]

        targets.append(rown.sourceName.getValue())    # target name
        fluxnow=float((rown.flux.getValue()).lstrip().split(' ')[3])   # new flux of this target
        fluxes.append(fluxnow/1000.0)   # convert to Jy

  	print 'using data from',targets[0],' ',fluxes[0],' Jy'
	find_calf=True
    except:
  	print 'failed to find Source.xml table - just using default, 1.0'
  	targets=['default']  #
        fluxes=[1]
        print targets
  	find_calf=False
    ntargets=1

# read in wvrs, and antenna names. find discrepant wvrs to flag those baselines  
    calw=open(asdm+'/CalWVR.xml')
    calwv=XmlObject(calw.read())
    calw.close()
    rows=calwv.CalWVRTable.row
    pwvs=zeros(len(rows)); antenna_names=[]; antenna_flags=[]
    for ir in range(len(rows)):
  	rown=rows[ir]
  	pwvs[ir]=1000.0*rown.water.getValue()
  	antenna_names.append(rown.antennaName.getValue())
    if verb: print 'antennas ',antenna_names
    pwv_mean=stats.nanmean(pwvs)
    pwv_std=stats.nanstd(pwvs)
    for ir in range(len(rows)):
  	if abs(pwvs[ir]-pwv_mean) > pwv_std*3.0:
  		antenna_flags.append(antenna_names[ir])
  		print 'flagged wvr/antenna ',antenna_names[ir], pwvs[ir]
  		pwvs[ir]=nan
    pwv_mean=stats.nanmean(pwvs)   # new values after flaggin
    pwv_std=stats.nanstd(pwvs)
    if verb: print 'mean pwv ',pwv_mean

    calph = open(asdm+'/CalPhase.xml')
    calphd = XmlObject(calph.read())
    calph.close()
    rows=calphd.CalPhaseTable.row
    row_now=rows[(sn-1)]     # (sn-1) is the source number, starting from 0
# antennas & baselines:
    antList=row_now.antennaNames.getValue().split(' ')
    freq=float((row_now.frequencyRange.getValue()).lstrip().split(' ')[2])
    wl=3e8/freq
# wl=360.0/1e6    # uncomment this to give rms in degrees
    baseline_n=[]
    for an in range (3,(len(antList)-1),2): baseline_n.append(antList[an]+' '+antList[an+1])

    baseline_l=(row_now.baselineLengths.getValue()).lstrip().split(' ')
    for ib in range(len(baseline_l)):
  	if ib<len(baseline_l):
  		if baseline_l[ib]=='':
  			baseline_l.pop(ib)
  			print ib
    baseline_l.pop(0); baseline_l.pop(0)
    for ib in range(len(baseline_l)):
  	if ib<len(baseline_l):
  		if baseline_l[ib]=='':
  			baseline_l.pop(ib)
  			print ib
    nbl=len(baseline_l)
    for ib in range(nbl): 
  	baseline_l[ib]=float(baseline_l[ib])
#
    if verb: print 'nbl:',nbl
    lbaselines=log10(baseline_l)
    lbaseline_l=lbaselines.tolist()
    nrows=len(rows)   # assume from here 4 basebands, 2 values (uncorrected, corrected), and 1 phase scan only (ie only wor

    print 'nrows:',nrows    
    rowstart=(bbn)*2 + (sn)*8    # starting row depends on bbn and scan number (sn)
#  for irow in range(rowstart,nrows,(8*ntargets) ):   # step depends on no. of targets  
    irow=rowstart
    damp=0.9
    for irow2 in range(0,2):
		row_now=rows[irow+irow2]
		wvr_corr=row_now.atmPhaseCorrection.getValue()
		bb=row_now.basebandName.getValue()
		direct[0]=row_now.direction.getValue().split(' ')[2]
		direct[1]=row_now.direction.getValue().split(' ')[3]
		if verb: print 'irow,bb,direct',irow, bb, int(bb[3]),direct
		if int(bb[3])==(bbn+1):    # checks that baseband is required value
			if wvr_corr=='AP_CORRECTED':
				if verb: print 'wvr corrected row ',irow
				amps=(row_now.ampli.getValue()).lstrip().split(' ')
				for ia in range(len(amps)):
  					if ia<len(amps):
  						if amps[ia]=='':
  							amps.pop(ia)
				  			print ia
				amps.pop(0);amps.pop(0);amps.pop(0)
				for ia in range(len(amps)):
  					if ia<len(amps):
  						if amps[ia]=='':
  							amps.pop(ia)
				  			print ia
		#		print 'len amps',len(amps)
				amps_2=[]
				for ia in range(nbl):
					amps_2.append((float(amps[ia])+float(amps[ia+nbl]))/2.0)
				amp_med=median(amps_2)   # median value of amplitude
		#		print 'median amp', amp_med
				phases=(row_now.phaseRMS.getValue()).lstrip().split(' ')[3:]   # 2 pols
				phase_2pc=zeros(nbl)
				for iph in range(nbl):
					try:
						phase_2pc[iph]=log10(1e6*wl*(degrees((float(phases[iph])+float(phases[iph+nbl]))/2.0)/360.0))
	#				if baseline_l[iph]<3500.0 and phase_2pc[iph]>15:
	#				print baseline_n[iph],baseline_l[iph], phase_2pc[iph],degrees(float(phases[iph])),degrees(float(phases[iph+nbl]))
	# now go through and remove baselines where amp<median*0.7 (or whatever)
					except:
						phase_2pc[iph]=nan
				phase_2pc=phase_2pc.tolist()
				nrem=0
				for ib in range(0,nbl):
					if amps_2[ib]<damp*amp_med:
		#				print 'removing baseline',ib,baseline_l[ib],baseline_n[ib],amps_2[ib]
						phase_2pc[ib]=nan
						nrem=nrem+1
		#		print 'nrem',nrem
				phase_med=stats.nanmedian(phase_2pc)
				phase_std=stats.nanstd(phase_2pc)
	# now go through and remove baselines where phase_rms>phase_rms_med*3sigma (or whatever)
				for ib in range(0,nbl):
					if (phase_2pc[ib]-phase_med)>3*phase_std:
		#				print 'removing baseline',ib,baseline_l[ib],baseline_n[ib],amps_2[ib]
						phase_2pc[ib]=nan
						nrem=nrem+1
		#		print 'nrem',nrem,phase_med,phase_std
	#  go through baselines and remove those using antennas which have been flagged
				for iflag in range(len(antenna_flags)):
					for ib in range(0,nbl):
						if baseline_n[ib].split(' ')[0][1:5]==antenna_flags[iflag] or\
							baseline_n[ib].split(' ')[1][1:5]==antenna_flags[iflag]:
							phase_2pc[ib]==nan
							nrem=nrem+1
	#						print 'removed',baseline_n[ib]
		#		print 'nrem',nrem
				pl.plot(lbaseline_l,phase_2pc,'o',color='blue',label='corrected')
			else:
				if verb: print 'wvr_uncorrected row ',irow
				amps=row_now.ampli.getValue().split(' ')[3:]
				amps_2=[]
				for ia in range(nbl): amps_2.append((float(amps[ia])+float(amps[ia+nbl]))/2.0)
				amp_med=median(amps_2)   # median value of amplitude
				print 'median amp', amp_med
				phases=row_now.phaseRMS.getValue().split(' ')[3:]   # 2 pols
				phase_2pu=zeros(nbl)
				for iph in range(nbl):
					phase_2pu[iph]=log10(1e6*wl*(degrees((float(phases[iph])+float(phases[iph+nbl]))/2.0)/360.0))
				phase_2pu=phase_2pu.tolist()  # converts to python list
				nrem=0
				for ib in range(0,nbl):
					if amps_2[ib]<damp*amp_med:
		#				print 'removing baseline',ib,baseline_l[ib],baseline_n[ib],amps_2[ib]
						phase_2pu[ib]=nan
						nrem=nrem+1
		#		print 'nrem',nrem
				phase_med=stats.nanmedian(phase_2pu)
				phase_std=stats.nanstd(phase_2pu)
	# now go through and remove baselines where phase_rms>phase_rms_med*3sigma (or whatever)
				for ib in range(0,nbl):
					if (phase_2pu[ib]-phase_med)>3*phase_std:
						if verb: print 'removing baseline',ib,baseline_l[ib],baseline_n[ib],amps_2[ib]
						phase_2pu[ib]=nan
						nrem=nrem+1
		#		print 'nrem',nrem,phase_med,phase_std
	#  go through baselines and remove those using antennas which have been flagged
				for iflag in range(len(antenna_flags)):
					for ib in range(0,nbl):
						if baseline_n[ib].split(' ')[0][1:5]==antenna_flags[iflag] or\
							baseline_n[ib].split(' ')[1][1:5]==antenna_flags[iflag]:
							phase_2pu[ib]==nan
							nrem=nrem+1
		#					print 'removed',baseline_n[ib]
		#		print 'nrem',nrem
				pl.plot(lbaseline_l,phase_2pu,'o',color='white',label='uncorrected')
    pl.xlabel('log baseline  (m)')
    pl.ylabel('log pathlength fluctuations (rms, microns)')
    pl.legend(loc='upper left')

    if verb: print phase_2pu
    if verb: print phase_2pc
    base_mon_c=[[] for i in range(nmon)]
    base_mon_u=[[] for i in range(nmon)]
    base_mon_number=zeros(nmon)   # number of baselines in these ranges
    if verb: print 'nbl: ',nbl
    for iph in range(nbl):
	for imon in range(nmon):
	  if baseline_l[iph]>baselinem[imon] and baseline_l[iph]<baselinem[imon+1]:
	#	print 'bl',baseline_l[iph]
		base_mon_c[imon].append(phase_2pc[iph])
		base_mon_u[imon].append(phase_2pu[iph])
		if not isnan(phase_2pc[iph]):
			base_mon_number[imon]=base_mon_number[imon]+1
#
    for imon in range(nmon):
  	if base_mon_number[imon]>5:
		base_mon_c_m[sn,imon]=10**(stats.nanmedian(base_mon_c[imon]))
		base_mon_c_s[sn,imon]=10**(stats.nanstd(base_mon_c[imon]))
		base_mon_u_m[sn,imon]=10**(stats.nanmedian(base_mon_u[imon]))
		base_mon_u_s[sn,imon]=10**(stats.nanstd(base_mon_u[imon]))
	else:
                base_mon_c_m[sn,imon]=nan; base_mon_u_m[sn,imon]=nan
	#	print 'not enough baselines for range',imon
#    print ' removed ',nrem,' observations because of low fluxes or high sigma rms'

#
#    print ' asdm: ',asdm,'  nrows:',nrows,' scan',(sn+1),' /',nscans
#    for imon in range(nmon-1):
#      print 'baseline',baselinem[imon],'-',baselinem[imon+1],'m   corrected:',"%.1f"%base_mon_c_m[sn,imon],'+/-',"%.1f"%base_mon_c_s[sn,imon],\
#      ' uncorrected:',"%.1f"%base_mon_u_m[sn,imon],'+/-',"%.1f"%base_mon_c_s[sn,imon],' microns  ',int(base_mon_number[imon]),'baselines'

# - - - - - - - - -
  
  base_mon_c_mean=zeros(nmon)
  base_mon_u_mean=zeros(nmon)
  for imon in range(nmon-1):
#      print imon,base_mon_c_m[:,imon],median(base_mon_c_m[:,imon])
      base_mon_c_mean[imon]=stats.nanmedian(base_mon_c_m[:,imon])
      base_mon_u_mean[imon]=stats.nanmedian(base_mon_u_m[:,imon])
      
# final average result:
  print ' asdm: ',asdm,'  nrows:',nrows,' scan',(sn+1),' /',nscans
  for imon in range(nmon-1):
      print 'baseline',baselinem[imon],'-',baselinem[imon+1],'m   corrected:',"%.1f"%base_mon_c_mean[imon],\
      ' uncorrected:',"%.1f"%base_mon_u_mean[imon],' microns  ',int(base_mon_number[imon]),'baselines'

  print 'wvr improvement in rms:'
  for imon in range(nmon-1):
  	if base_mon_c_m[sn,imon]>0.0:
            print 'baseline',baselinem[imon],'-',baselinem[imon+1],'m   : ',base_mon_u_m[sn,imon]/base_mon_c_m[sn,imon]
#  print base_mon_u_mean, base_mon_c_mean
  wvr_change=stats.nanmean(base_mon_u_mean)/stats.nanmean(base_mon_c_mean)   # the mean wvr improvement

# test for go/nogo at different bands:
  wlb=zeros(11)
  pwv_allow=zeros(11)

  wlb[3]=0.003;  pwv_allow[3]=12.0
  wlb[6]=0.0013;  pwv_allow[6]=4.0
  wlb[7]=0.00087;  pwv_allow[7]=2.0
  wlb[8]=0.0006;  pwv_allow[8]=1.0
  wlb[9]=0.00045;  pwv_allow[9]=0.7
  wlb[10]=0.00035;  pwv_allow[10]=0.7

  for imon in range(nmon):
  	if base_mon_number[imon]<3: base_mon_c_mean[imon]=0.0    # ignore baseline ranges with <3 baselines in them
  	
  rms_max=max(base_mon_c_mean)
  print ''
  print 'quick go/nogo check on corrected data: checking pwv and phase noise:'
  print 'band\t pwv\tphase rms'
  hiband=1; hibandp=1; hibandr=1; hibandr2=1
  for iband in range(1,11):
	if wlb[iband]>0.0:
		if pwv_mean<pwv_allow[iband]:
			a1=bcolors.OKGREEN + 'Y ' + bcolors.ENDC
			hibandp=iband
		else:
			a1= bcolors.FAIL + 'N ' + bcolors.ENDC
			
		if 360.0*(rms_max/1e6)/wlb[iband] < 30.0:
			a2= bcolors.OKGREEN + 'Y ' + bcolors.ENDC
			hibandr=iband
		elif 360.0*(rms_max/1e6)/wlb[iband] < 60.0:
			a2= bcolors.WARNING + 'Y ' + bcolors.ENDC
			hibandr2=iband
		else:
			a2= bcolors.FAIL + 'N ' + bcolors.ENDC
		if hibandp==iband and hibandr==iband: hiband=iband   # this is the highest usable band

		print 'B'+str(iband)+'\t'+' '+a1+'\t'+a2


  print bcolors.OKBLUE + 'highest observing band WITHOUT selfcal based on rms:',str(hibandr) + bcolors.ENDC
  print bcolors.OKBLUE + 'highest observing band WITH selfcal based on rms:',str(hibandr2) + bcolors.ENDC
  print ''

# remove nans for fitting
  for ia in range(nbl,0,-1):
    if ia<len(phase_2pc):
  	  if isnan(phase_2pc[ia]):
  		tem=phase_2pc.pop(ia)
  		tem=lbaseline_l.pop(ia)

  datafit=polyfit(lbaseline_l,phase_2pc,2)
#slopefit=datafit[0]; interfit=datafit[1]
  pp=numpy.poly1d(datafit)
  base_fit=arange(min(lbaseline_l),max(lbaseline_l),0.01)
  poly_fit=pp(base_fit)
  pl.plot(base_fit,poly_fit,'g--')
  if len(targets)>1:
  	pl.title((asdm+':  bb'+str(bbn)+' sc'+str(sn)+' '+targets[(targetn-1)]+' '+str("%.2f"%pwv_mean)+'mm cf='+str("%.2f"%wvr_change)))
  else:
  	pl.title((asdm+'   bb'+str(bbn)+' scan'+str(sn)+'  '+targets[0]+' '+str("%.2f"%pwv_mean)+'mm '+str("%.2f"%wvr_change))) 
  print ntargets,' target:',targets,' flux:',fluxes
  print 'mean wvr improvement:',wvr_change
  if wvr_change<1.0: print "WARNING: this is < 1.0 !"
  date=aU.getObservationStartDateFromASDM(asdm)[0]
  date=date[:-3]
  date1=date.split(' ')[0]
  time1=date.split(' ')[1]
  wl=int(wl*1e5)/100.   # wavelenght mm, rounded
  print ''
  print 'output for file: uid, date, UT, wavelength/mm, target,flux/Jy,pwv/mm, pwv_rms/mm, wind, temp, rms0/um, rms1, rms2, rms3, rms4, rms_improvement, hibandr, hibandr2:'
  print ' '
  lineo= EB+','+date1+','+time1+','+str(wl)+','+targets[0]+','+"%.2f"%fluxes[0]+','+str("%.2f"%pwv_mean)+','+str("%.3f"%pwv_std)+','+str("%.1f"%wind_mean)+','+str("%.1f"%temp_mean)+','+str("%.2f"%base_mon_c_mean[0])+','+str("%.2f"%base_mon_c_mean[1])+','+str("%.2f"%base_mon_c_mean[2])+','+str("%.2f"%base_mon_c_mean[3])+','+str("%.2f"%base_mon_c_mean[4])+','+str("%.2f"%wvr_change)+','+str(hibandr)+','+str(hibandr2)+'\n'
  print lineo

  if fwrite:
      print ' appending data to file /users/wdent/CSV3189/phase_results.txt'
      fil=open('/users/wdent/CSV3189/phase_results.txt','a')
      fil.write(lineo)
      fil.close()
