#!/usr/bin/env python
# tmcdbPlayer.py
"""
tmcdbPlayer.py
written by dbarkats

$Id: tmcdbPlayer.py,v 1.39 2015/12/09 17:47:34 thunter Exp $
"""

# Execution starts here
import os, sys
import socket
import getpass
import pickle
import glob
username = getpass.getuser()
sys.path.append('/users/%s/AIV/science/scripts/R10.4_WORKING/'%username)
from FocusModelEditor import FocusModelEditor
from PointingModelEditor import PointingModelEditor
from PositionEditor import PositionEditor
from pylab import *
import tmUtils as tm
from datetime import datetime,timedelta
import warnings
from  getNominalPtBandOffsets import getNominal
import analysisUtils as au
import re
# for getdov
#from operator import itemgetter
#from itertools import groupby

def system_except(cmd):
    ret = os.system(cmd)
    if ret != 0:
        raise Exception("Command '%s' returned non-zero exit status %d" %(cmd, ret))


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    

class tmcdbPlayer(object):
    """
    
    """
  

    def __init__(self, analysisUtilsDirectory=None):

        self.username = getpass.getuser()
        self.hostname = socket.gethostname()
        if (analysisUtilsDirectory == None):
            self.antennaMovesFile = '/users/%s/AIV/science/PadData/antennaMoves.txt'%self.username
        else:
            self.antennaMovesFile = analysisUtilsDirectory+'/PadData/antennaMoves.txt'
        self.antennaMovesDir = os.path.dirname(self.antennaMovesFile)
        if ('osf-red' in self.hostname) or ('sciops' in self.hostname) or ('casa' in self.hostname) or ('red-osf' in self.hostname):
            print '### You are running on an analysis machine \"%s\" ###' %self.hostname
            sys.path.append('/usr/local/lib/python2.7/site-packages') # for suds
            sys.path.append('/usr/lib/python2.4/site-packages') # for TMCDBAccessClient
            from TMCDBAccessClient import TMCDBAccessClient 
            self.tmcdb =   TMCDBAccessClient()
            
        elif ('gns' in self.hostname) or ('crc' in self.hostname):
            # on aos-gns or crc
            #print '### You are running on an STE machine \"%s\" ###'  %self.hostname
            import Acspy.Clients.SimpleClient
            c = Acspy.Clients.SimpleClient.PySimpleClient()
            self.tmcdb = c.getDefaultComponent('IDL:alma/TMCDB/Access:1.0')
            
        self.getAntennaNames()
        self.nominal = getNominal()
        

    def getAntennaNames(self):
        """
        simple list of all antennas
        """
        
        self.antList = ['DV01', 'DV02', 'DV03', 'DV04', 'DV05', 'DV06', 'DV07', 'DV08', 'DV09', 'DV10',\
                        'DV11', 'DV12', 'DV13', 'DV14', 'DV15', 'DV16', 'DV17', 'DV18', 'DV19', 'DV20',\
                        'DV21', 'DV22', 'DV23', 'DV24', 'DV25', 'DA41', 'DA42', 'DA43', 'DA44', 'DA45',\
                        'DA46', 'DA47', 'DA48', 'DA49', 'DA50', 'DA51', 'DA52', 'DA53', 'DA54', 'DA55',\
                        'DA56', 'DA57', 'DA58', 'DA59', 'DA60', 'DA61', 'DA62', 'DA63', 'DA64', 'DA65',\
                        'PM01', 'PM02', 'PM03', 'PM04', 'CM01', 'CM02', 'CM03', 'CM04', 'CM05', 'CM06',\
                        'CM07', 'CM08', 'CM09', 'CM10', 'CM11', 'CM12']
        return self.antList

    def getPadNames(self):
        """
        simple list of all pads
        """
        padList = []
        file = '/users/dbarkats/analysis_scripts/AOS_Pads_XYZ_ENU.txt'
        f = open(file,'r')
        lines = f.readlines()
        for line in lines:
            if not line.startswith('#'):
                tokens = line.split()
                padList.append(tokens[4])
        self.padList = padList[0:-1]
        return self.padList

    def readAntennaMovesTxt(self, cvsup=True, verbose = True):
        """
        Reads the current antennaMoves.txt file
        and outputs a dictionnary of pad and move structure
        
        """
        class stru:
            def __init__(self):
                self.pad = []
                self.start = []
                self.end = []

        #cvs up
        if cvsup == True:
            if verbose == True: print '#### CVS updating antennaMoves.txt... #### '
            wd = os.getcwd()
            os.chdir('%s/'%self.antennaMovesDir)
            up_cmd = 'cvs up antennaMoves.txt'
            system_except(up_cmd)
            os.chdir(wd)
        
        if verbose == True: print '#### Reading antennaMoves.txt... #### '
        a = open(self.antennaMovesFile,'r')
        lines = a.readlines()
        a.close()

        d={}
        for ant in self.antList:
            d[ant]=stru()

        for ant in self.antList:
            i = 0
            for line in lines[1:]:
                if ant in line:
                    date = line.split()[0][0:10]
                    fr = line.split()[2]
                    to = line.split()[3]
                    if i != 0:
                        d[ant].end.append(date)
                    d[ant].pad.append(to)
                    d[ant].start.append(date)
                    i +=1

            d[ant].end.append('')
        self.moves = d
        return d
    
    def printPadHistory(self,ant):
        """
        print pad history for 1 antenna ant
        self.moves is the complete dictionnary of pad History
        """
        print '#### Pad history for %s...#### '%ant
        padH = self.moves[ant]
        for i in range(len(padH.pad)):
            print padH.pad[i],padH.start[i], padH.end[i]

    def checkNewMove(self, verbose = 1):
        """
        check if there is a match between tmcdb pad-antenna association
        and antennaMoves.txt  pad-antenna association.
        If not, it puts out a  list of text lines of the changes needed
        """
        print '#### Checking for a new antenna move... ####'
        now ='%s'%datetime.now()
        newAntennaMove=[]

        if verbose: print 'Ant tmcdb TxtFile'
        for ant in self.antList:
            try:
                tmcdbPadInfo = self.tmcdb.getCurrentAntennaPadInfo(ant).PadName
            except:
                tmcdbPadInfo = 'OSF'
                ##DEBUG##
                #if ant == 'DA58': tmcdbPadInfo  = 'A087'
            try:
                TxtPadInfo = self.moves[ant].pad[-1]
            except:
                TxtPadInfo = 'OSF'

            if tmcdbPadInfo != TxtPadInfo:
                print bcolors.FAIL+'%s %s %s <<<<<'%(ant,tmcdbPadInfo,TxtPadInfo)+bcolors.ENDC
                newAntennaMove.append('%sT00:00 %s %s %s'%(now[0:10],ant,TxtPadInfo, tmcdbPadInfo))
            else:
                if verbose:print '%s %s %s'%(ant,tmcdbPadInfo,TxtPadInfo)
                
        return newAntennaMove

    def editAntennaMovesTxt(self, newAntennaMove):
        """
        Edits the existing antennaMoves.txt file
        and cvs commits it.
        """
        print('### Editing antennaMoves.txt...###')
        # make a copy
        system_except('cp %s %s.backup'%(self.antennaMovesFile,self.antennaMovesFile))

        # Edit the file
        with open(self.antennaMovesFile,'a+') as f:
            for line in newAntennaMove:
                f.write('%s \n'%line)
        f.close()

        #cvs ci
        print '### CVS committing antennaMoves.txt...###'
        os.chdir('%s/'%self.antennaMovesDir)
        ci_cmd = 'cvs ci -m \" Updating file because new antenna move detected: %s \" antennaMoves.txt'\
                 %newAntennaMove
        a = os.popen(ci_cmd).read()
        
        # write log
        with open('Log.txt','a+') as f:
            f.write('Updated antennaMoves.txt because new antenna move detected \n')
            for line in newAntennaMove:
                f.write('%s \n'%line)
            f.write('cvs committed \n')
            f.write(a)
        f.close()

        # rsync to  web page
        host='dbarkats@ssh.alma.cl'
        cmd='rsync -auv --progress /users/dbarkats/PadData/antennaMoves.txt %s:~/public_html/trend_analysis/ ' %(host)
        print "### Rsyncing to dbarkats@ssh.alma.cl... ###"
        os.system(cmd)
    

    def inversePadFind(self, date, ant = None, cvsup=True, verbose = True):
        """
        Given a date, this finds all the antenna pad position at that date
        date has format: '2013-05-23'
        Returns a dict of antenna: Pad name
        """
        pos={}
        move = self.readAntennaMovesTxt(cvsup, verbose)
        if verbose == True: print '### Getting antenna configuration for date %s'%date
        date = au.strDate2MJD(date)
        now =('%s'%datetime.now())[0:10]

        if ant ==None:
            ants = sort(move.keys())
        else:
            ants = [ant]
        for ant in ants:
            # temporarily replace last date with now
            move[ant].end[-1]=now
            for i in range(len(move[ant].start)):
                if (au.strDate2MJD(move[ant].start[i]) > date):
                    pos[ant]='OSF'
                if (au.strDate2MJD(move[ant].start[i]) <= date) & (au.strDate2MJD(move[ant].end[i]) >= date):
                    #print ant, move[ant].pad[i]
                    pos[ant]=move[ant].pad[i]
                    break
        return pos

    def printAntennaConfigurationForDate(self,date):
        # extract the Position Models for that date
        cmd = 'ExportPositionModel.py -t %s -f All_PositionModel_%s.xml' %(date,date)
        print cmd
        system_except(cmd)

        # read the extracted position Model
        p = PositionEditor()
        p.getTmcdbPositionModel('All_PositionModel_%s.xml' %(date))
        
        pos = self.inversePadFind(date)
        ants = sort(pos.keys())
        for ant in ants:
            if pos[ant] == 'OSF':
                continue
            print 'name=%s'%ant
            if ant[0:2] == 'CM':
                print 'diameter=7.0'
            else:
                print 'diameter=12.0'
            if ant[0:2] == 'DV':
                antvector = [2.439, -5.963, -2.738]  # rotated (0, 0, 7.0m)
            else:
                antvector = [2.613, -6.389, -2.933]  # rotated (0, 0, 7.5m)
            padInfo = p.getTmcdbPadPosition(pos[ant])  
            print 'position=[%s,%s,%s]' % \
                (padInfo[0]+antvector[0],padInfo[1]+antvector[1],padInfo[2]+antvector[2])


    def checkLatestPositionModel(self, verbose = False, debug = False):
        """
        Simple script to check the TMCDB for any changes in the pad or antenna position
        """

        # TODO delete history from older than 1 month
        
        # first get the history for all pads/antennas
        print "### Exporting all antenna/pad history from TMCDB..."
        outfile = 'trend_analysis/position_models/All_position_History.xml'
        cmd = 'ExportPositionModel.py -y -f %s' %(outfile)
        if verbose: print cmd
        system_except(cmd)
        p = PositionEditor()
        p.getTmcdbPositionModel(outfile)
        p.getTmcdbPadPositionVersions()
        p.getTmcdbAntennaPositionVersions()
        versions = p._versions

        # save today's raw version history in pickle file for today
        today =('%s'%datetime.now())[0:10]
        pickleFile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%today
        print '### Saving antenna/pad history in pickle for today: %s...'%today
        f = open(pickleFile,'w')
        pickle.dump(versions, f)
        f.close()

        # load the raw version history from yesterday
        yesterday = ('%s'%(datetime.now() - timedelta(days=1)))[0:10]
        pickleFile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%yesterday
        print '### Loading antenna/pad history from yesterday: %s...' %yesterday
        if os.path.exists(pickleFile):
            f = open(pickleFile,'r')
            versions_old = pickle.load(f)
            f.close()
        else:
            print "%s does not exist. exiting"%pickleFile
            exit()
        
        # list to keep track of global updates
        updates = {}

        # check the content of the pickle file against TMCDB
        for ant in self.antList:
            v_old = versions_old[ant].keys()[-1]
            v_new = versions[ant].keys()[-1]
            timestamp_old = versions[ant][v_old]['timestamp'][:16]
            author_old = versions[ant][v_old]['author']

            if v_old != v_new:
                for v in range(v_old+1, v_new+1):
                    timestamp_new = versions[ant][v]['timestamp'][:16]
                    author_new = versions[ant][v]['author']
                    comment = versions[ant][v]['comment']
                    if not timestamp_new in updates.keys():
                        updates[timestamp_new] = {
                            'update_author': author_new,
                            'update_comment': comment,
                            'antPosChange': [],
                            'padPosChange': []}
                    updates[timestamp_new]['antPosChange'].append(ant)
                if verbose:
                    print "%s antenna position updated from version %d to %d" % (ant, v_old, v_new)
                    print "version %d: %s, %s" % (v_old, timestamp_old, author_old)
                    print "version %d: %s, %s, %s" % (v_new, timestamp_new, author_new, comment)
            else:
                if verbose: print "%s no antenna position change" % ant

            # check the pad history 
            try:
                pad = self.tmcdb.getCurrentAntennaPadInfo(ant).PadName
            except:
                pad = 'OSF'
                if verbose: print "%s at OSF" % ant
                continue
            p_old = versions_old[pad].keys()[-1]
            p_new = versions[pad].keys()[-1]
            timestamp_old = (versions_old[pad][p_old]['timestamp'])[:16]
            author_old = versions_old[pad][p_old]['author']

            if p_old != p_new:
                for p in range(p_old+1, p_new+1):
                    timestamp_new = (versions[pad][p]['timestamp'])[:16]
                    author_new = versions[pad][p]['author']
                    comment = versions[pad][p]['comment']
                    if not timestamp_new in updates.keys():
                        updates[timestamp_new] = {
                            'update_author': author_new, 'update_comment': comment,
                            'antPosChange': [], 'padPosChange': []}
                    updates[timestamp_new]['padPosChange'].append(ant)
                if verbose:
                    print "%s pad position updated from version %d to %d" % (pad, p_old, p_new)
                    print "version %d: %s, %s" % (p_old, timestamp_old, author_old)
                    print "version %d: %s, %s, %s" % (p_new, timestamp_new, author_new, comment)
            else:
                if verbose: print "%s on %s, no pad position change" % (ant, pad)

        # Sort out the changes
        timestamps = sorted(updates.keys())
        if timestamps != []:
            print '### Summary of changes ###'
            for timestamp in timestamps:
                print timestamp, updates[timestamp]['update_author'], updates[timestamp]['update_comment']
                print "Antenna position changed: %s" % ','.join(map(str, updates[timestamp]['antPosChange']))
                print "Pad position changed: %s" % ','.join(map(str, updates[timestamp]['padPosChange']))
                print ''
        else:
            print '### No antenna/pad changes from %s to %s ###'%(yesterday,today)

        return

            
        # STILL EXPERIMENTAL, Finished and ready to be deployed. See details in CSV-3164
        print "### Mockup of almaAntPos.txt... ###"
        move = self.readAntennaMovesTxt()

        for timestamp in timestamps:
            if debug: print updates[timestamp]['update_comment']
            result = re.search('uid(.*)result',updates[timestamp]['update_comment'])
            if result != None:
                asdm =  result.group(0)
            else:
                asdm = 'UNKNOWN'
            measTime = '%sT23:00:00'%today  # if this runs once a day, we could put the last day.
            result = re.search('method: (.*)_FITTING',updates[timestamp]['update_comment'])
            if result != None:
                method = result.group(1)
            else:
                method = 'UNKNOWN'
            reducTime = timestamp  # timestamp result was put in TMCDB
            result = re.search('refAnt: (\w{4})',updates[timestamp]['update_comment'])
            if result != None:
                refAnt = result.group(1)
            else:
                refAnt = 'UNKNOWN'

            print '\n<BaselineRun asdm=\"%s\" measTime=\"%s\" posMethod=\"%s\" reducTime=\"%s\" refAnt=\"%s\" >'\
                  %(asdm,measTime,method,reducTime,refAnt)
            
            antChangeList = updates[timestamp]['antPosChange']  + updates[timestamp]['padPosChange']
            for ant in antChangeList:
                if debug: print move[ant].pad[-1], move[ant].start[-1], self.tmcdb.getCurrentAntennaPadInfo(ant).PadName
                antennaInfo = self.tmcdb.getAntennaInfo(ant)
                antennaPadInfo = self.tmcdb.getCurrentAntennaPadInfo(ant)
                print '<Antenna name=\"%s\" pad=\"%s\" putInTime=\"%s\">'%(ant,move[ant].pad[-1], move[ant].start[-1])
                print '<PadPosition X=\"%14.6f\" Y=\"%14.6f\" Z=\"%14.6f\"/>'%(antennaPadInfo.XPosition.value,antennaPadInfo.YPosition.value,antennaPadInfo.ZPosition.value)
                print '<AntennaVector X=\"%8.6f\" Y=\"%8.6f\" Z=\"%8.6f\"/>'%(antennaInfo.XPosition.value,antennaInfo.YPosition.value,antennaInfo.ZPosition.value)
                print '<PositionError X=\"%8.6f\" Y=\"%8.6f\" Z=\"%8.6f\"/>'%(0,0,0)
                print '</Antenna>'
            print '</BaselineRun>'

        return

    def getAllPadTilts(self,antList=None):
        """
        For an list of antennas, get all the AN0 and AW0 coeficients since we started version
        on those parameters.

        """
        AN0 ={}
        AW0 = {}

        outfile = 'All_ArrayConfiguration_History.xml'
        cmd = 'ExportArrayConfigurationModel.py -y -f %s' %(outfile)
        system_except(cmd)
        #p.getTmcdbPositionModel(outfile)
        #p.getTmcdbPadPositionVersions()
        #p.getTmcdbAntennaPositionVersions()
        #versions = p._versions
        
    def readTiltMeterFiles(self,antList=None):
        """
        Read for the input antenna list the whole tiltmeter file and save it in sa structure.
        This will associate a pad position for each time stamp based on antennaMoves.txt
        but will not cvs up to get the latest file.
        only works on osf-red given the path of Tiltmeter files
        """
        class struct:
            def __init__(self):
                self.time = []
                self.AN0 = []
                self.AW0 = []
                self.Tout = []
                self.Tdev = []
                self.pad = []
                
        if antList == None:
            antList = self.getAntennaNames()
        else:
            antList = au.aggregate(antList)
        tilts={}
        for ant in antList:
            print "### Reading tiltmeter files for %s"%ant
            tilts[ant]=struct()     

            f1=glob.glob('/data/RADIO/POINT/Tiltmeter/%s_tiltmeterValues.txt'%ant)
            f2=glob.glob('/data/RADIO/POINT/Tiltmeter/%s_tiltmeterValues_backup*.txt'%ant)
            tiltMeterFiles = f1+f2
            print "Reading tiltmeter files: %s"%tiltMeterFiles
            # read tiltmeter values
            lines = []
            for file in tiltMeterFiles:      
                f = open(file)
                tmp_lines = f.readlines()
                lines = lines+tmp_lines
                f.close()
  
            for line in sort(lines):
                if line[0] == '#': continue
                ls = line.split()
                if ls == [] : continue
                ts = au.dateStringToMJD(ls[0], verbose = False)
                if ts in tilts[ant].time: continue
                tilts[ant].time.append(ts)
                tilts[ant].AN0.append(float(ls[2]))
                tilts[ant].AW0.append(float(ls[3]))
                if ant[0:2] == 'DA':
                    tilts[ant].Tout.append(float(ls[4]))
                    tilts[ant].Tdev.append(mean([float(ls[5]),float(ls[6])]))
                else:
                    tilts[ant].Tout.append(float(ls[6]))
                    tilts[ant].Tdev.append(float(ls[7]))
                pad = self.inversePadFind(ls[0][0:10],ant,cvsup=False,verbose=False)[ant]
                tilts[ant].pad.append(pad)

        return tilts
                                     
    def getdoV(self, antList=None):
        class struct:
            def __init__(self):
                self.N = []
                self.Nerr = []
                self.W = []
                self.Werr = []
                self.time = []
                self.ant = []

        dov = {}
        today = au.dateStringToMJD(('%s'%datetime.now()), verbose = False)
        if antList == None:
            antList = self.getAntennaNames()
        else:
            antList = au.aggregate(antList)          

        tilts = self.readTiltMeterFiles(antList)
        d_AllAnts,doff_AllAnts = self.getAllPointingModel(antList,readFromTMCDB=True)
        movList = self.readAntennaMovesTxt()
        
        for ant in antList:
            print ant
            moves = movList[ant]
            ti = tilts[ant]
            d = d_AllAnts[ant]

            # get unique pads
            upad = unique(ti.pad)

            # Make plots of AN/AW and AN0/AW0 vs time
            figure(1,figsize=(12,8));clf()
            yl = 40
            ax1 = subplot(2,1,1)
            plot(d['timestamp'],d['AN'],'b+')
            plot(ti.time,ti.AN0,'gx')
            ax1.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
            vlines(today,-yl,yl,color='m')
            xlabel('MJD')
            ylabel('AN:+, AN0:x [arcsec]')
            ax1.set_xticklabels(())
            ax2= subplot(2,1,2)
            plot(d['timestamp'],d['AW'],'b+')
            plot(ti.time,ti.AW0,'gx')
            ax2.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
            vlines(today,-yl,yl,color='m')
            xlabel('MJD')
            ylabel('AW:+, AW0:x [arcsec]' )
            
            for pad in upad:
                if 'OSF' in pad: continue
                if pad not in dov.keys():
                    dov[pad]=struct()

                # find indices for tilt AN0/AW0 where antenna is in one pad
                # Need to be careful for indices of an antenna returning to same pad
                #ind = find(array(ti.pad) == pad)
                #qList = []
                #for k, g in groupby(enumerate(ind), lambda (i,x):i-x):
                #    qList.append(map(itemgetter(1), g))

                # find start and end time on that pad
                indList = find(array(moves.pad)== pad)
                
                for ind in indList:
                    tstart = au.dateStringToMJD(moves.start[ind],verbose=False)
                    if moves.end[ind] == '':
                        tend = today
                    else:
                        tend = au.dateStringToMJD(moves.end[ind],verbose = False)
                                        
                    q = where((array(ti.time) > tstart) & (array(ti.time) < tend)) #indices for AN0/AW0
                    qp = where((array(d['timestamp']) > tstart) & (array(d['timestamp']) < tend)) #indices for AN/AW
                    t = (tstart+tend)/2
                    nq = size(q)
                    nqp = size(qp)
                    if nqp == 0:
                        # no valid pointing models for this time range
                        continue
                                 
                    meanAN0 = mean(array(ti.AN0)[q])
                    stdAN0 = std(array(ti.AN0)[q])/sqrt(nq)
                    meanAN = mean(array(d['AN'])[qp])
                    stdAN = std(array(d['AN'])[qp])/sqrt(nqp)
                    meanAW0 = mean(array(ti.AW0)[q])
                    stdAW0 = std(array(ti.AW0)[q])/sqrt(nq)
                    meanAW = mean(array(d['AW'])[qp])
                    stdAW = std(array(d['AW'])[qp])/sqrt(nqp)

                    print 'AN', pad, meanAN, stdAN, meanAN0, stdAN0, meanAN-meanAN0,sqrt(stdAN**2+stdAN0**2)
                    print 'AW', pad, meanAW, stdAW, meanAW0, stdAW0, meanAW-meanAW0,sqrt(stdAW**2+stdAW0**2)
                    dov[pad].N.append(meanAN-meanAN0)
                    dov[pad].Nerr.append(sqrt(stdAN**2+stdAN0**2))
                    dov[pad].W.append(meanAW-meanAW0)
                    dov[pad].Werr.append(sqrt(stdAW**2+stdAW0**2))
                    dov[pad].time.append(t)
                    dov[pad].ant.append(ant)
                                     
                    subplot(2,1,1)
                    errorbar([t],[meanAN0],yerr=[stdAN0],fmt = 'rx',ecolor='r')
                    errorbar([t],[meanAN],yerr=[stdAN],fmt = 'r+',ecolor='r')
                    vlines(tstart,-yl,yl)
                    vlines(tend,-yl,yl)
                    ylim(-yl,yl)
                    text(t,yl+2,pad, rotation = 45,size= 'small')
                    subplot(2,1,2)
                    errorbar([t],[meanAW0],yerr=[stdAW0],fmt = 'rx',ecolor='r')
                    errorbar([t],[meanAW],yerr=[stdAW],fmt = 'r+',ecolor='r')
                    vlines(tstart,-yl,yl)
                    vlines(tend,-yl,yl)
                    ylim(-yl,yl)

            suptitle('%s AN/AN0  AW/AW0 ' %ant, y=0.96, size='large')
            subplots_adjust(hspace = 0.001)
            plotname = '%s_tilts_history.png'%ant
            savefig('trend_analysis/tilt_plots/%s'%plotname)    
           
        return dov

    def plotDov(self, dov):
        """

        """
        
        for pad in sort(dov.keys()):
            nants = size(dov[pad].time)
            if nants > 1:
                #plot dov vs antenna on pad
                figure(1,figsize=(8,5));clf()
                ax = subplot(1,1,1)
                errorbar(arange(nants),dov[pad].N,yerr=dov[pad].Nerr,fmt = 'bo',ecolor='b')
                errorbar(arange(nants),dov[pad].W,yerr=dov[pad].Werr,fmt = 'ro',ecolor='r')
                axhline(-5.92,color='b',linestyle = '--')
                axhline(13.4,color='r',linestyle = '--')
                xlim([-0.5,nants-0.5])
                ax.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
                xticks(range(nants),dov[pad].ant)
               
                ylabel('DoV [arcsec]')
                legend(['North','West'],bbox_to_anchor=(1.1,1.1))
                title('Deviation from vertical for pad %s'%pad)
                plotname = '%s_tilts_history.png'%dov[pad].ant
                savefig('trend_analysis/tilt_plots/%s_dov_vs_ant.png'%pad) 
            
        (X,Y,Z,padList)= au.getPadLOC(returnPads=True)
        vec = []
        vecErr = []
        
        for pad in padList:
            if pad in dov.keys():
                print pad, 'yes'
                vec.append([mean(dov[pad].N),mean(dov[pad].W)])
                vecErr.append([mean(dov[pad].Nerr),mean(dov[pad].Werr)])
                
            else:
                print pad, 'no'
                vec.append([0,0])
                vecErr.append([0,0])

        vec = array(vec)
        vecErr = array(vecErr)
        pList = array(padList)
        q = find(vec[:,0] !=0)
        qSpads= [i for i,x in enumerate(pList[q]) if x[0] == 'S']
        qWpads= [i for i,x in enumerate(pList[q]) if x[0] == 'W']
        qPpads= [i for i,x in enumerate(pList[q]) if x[0] == 'P']
        qNpads= [i for i,x in enumerate(pList[q]) if x[0] == 'N']
        qJpads= [i for i,x in enumerate(pList[q]) if x[0] == 'J']
        qTpads= [i for i,x in enumerate(pList[q]) if x[0] == 'T']
        
        sq = size(q)
        figure(3,figsize=(12,6));clf()
        ax=subplot(2,1,1)
        errorbar(arange(size(q)),vec[q,0],yerr=vecErr[q,0],fmt='bo',ecolor='b')
        xticks(range(sq),array(padList)[q], rotation=90, size='small')
        ax.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        ax.set_xticklabels(())
        xlim([-1,sq])
        axhline(-5.92,color='b',linestyle = '--')
        axvspan(qSpads[0]-.5,qSpads[-1]+.5,color='g',alpha = 0.1)
        axvspan(qWpads[0]-.5,qWpads[-1]+.5,color='r',alpha = 0.1)
        axvspan(qPpads[0]-.5,qPpads[-1]+.5,color='m',alpha = 0.1)
        axvspan(qNpads[0]-.5,qNpads[-1]+.5,color='gray',alpha = 0.1)
        axvspan(qJpads[0]-.5,qJpads[-1]+.5,color='b',alpha = 0.1)
        ylabel('dov North [arcsec]')
        ax=subplot(2,1,2)
        errorbar(arange(sq),vec[q,1],yerr=vecErr[q,1],fmt='ro',ecolor='r')
        xticks(arange(sq),array(padList)[q], rotation=90, size='small')
        xlim([-1,sq])
        ylabel('dov West [arcsec]')
        ax.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        axhline(13.14,color='r',linestyle = '--')
        axvspan(qSpads[0]-.5,qSpads[-1]+.5,color='g',alpha = 0.1)
        axvspan(qWpads[0]-.5,qWpads[-1]+.5,color='r',alpha = 0.1)
        axvspan(qPpads[0]-.5,qPpads[-1]+.5,color='m',alpha = 0.1)
        axvspan(qNpads[0]-.5,qNpads[-1]+.5,color='gray',alpha = 0.1)
        axvspan(qJpads[0]-.5,qJpads[-1]+.5,color='b',alpha = 0.1)
        subplots_adjust(hspace = 0.001)
        savefig('trend_analysis/tilt_plots/dovs_vs_pads.png') 

        figure(4,figsize=(8,8));clf()
        plot(X,Y,'o',mfc='none',markersize=10)
        quiver(array(X)[q],array(Y)[q],-vec[q,1]+13.14,vec[q,0]+5.92)
        #text(X[q],Y[q],pList[q],size='small')

        return vec
            
            
    # TODO
    # Fix PM02 to remove the zeros. Fix the Nans
    # examine the large error bars.
    # repeat analysis with TMCDB values of AN0, AW0
             
    def getAllPositionModel(self,antList=None, versionsOnly=False):
        """
        For a list of antennas antList, this will get all the position models
        including their history from the TMCDB  and output it in a dictionnary of
        models in vaiable pos. The dictionnary will contain all antenna position changes and
        pad position changes.
        versionsOnly = True will return only the version history instead of the positions

        """
        pos = {}
        p = PositionEditor()

        # first get the history for all pads/antennas
        today =('%s'%datetime.now())[0:10]
        pickleFile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%today
        if os.path.isfile(pickleFile):
            print '\n### Loading antenna/pad history from %s...' %pickleFile
            f = open(pickleFile,'r')
            versions = pickle.load(f)
            f.close()
        else:
            outfile = 'All_position_History.xml'
            cmd = 'ExportPositionModel.py -y -f %s' %(outfile)
            system_except(cmd)
            p = PositionEditor()
            p.getTmcdbPositionModel(outfile)
            p.getTmcdbPadPositionVersions()
            p.getTmcdbAntennaPositionVersions()
            versions = p._versions

            # save today's raw version history in pickle file for today
            print '### Saving antenna/pad history in pickle for today: %s...'%today
            f = open(pickleFile,'w')
            pickle.dump(versions, f)
            f.close()

        if versionsOnly == True:
            return versions

        if antList == None: antList = self.getAntennaNames()
        for ant in antList:
            versionsA = versions[ant]
            
            if size(versionsA.keys()) == 1:
                print " WARNING: no versions for %s yet"%ant
                raise warnings.filterwarnings("WARNING: no versions for %s yet"%ant)

            picklefile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%ant
            if os.path.exists(picklefile):
                print '\n### Loading antenna position model from %s...' %picklefile
                f = open(picklefile,'r')
                pos[ant] = pickle.load(f)
                f.close()
                
                nversions = size(pos[ant]['timestamp'])
                if nversions == versionsA.keys()[-1]:
                    print 'Pickle file contains up to versions: %d which is the latest version in TMCDB. No need to ExportPositionModel or update this antenna\' allVersion.pickle file'%(nversions)
                    continue
                else:
                    print 'Pickle file contains up to versions: %d. Exporting up to versions %d'%(nversions,versionsA.keys()[-1])
            else:
                nversions = 1
        
            # now get the antenna position models for all versions
            for i in range(nversions,versionsA.keys()[-1]):
                print  ant,i
                if i == 0: continue
                
                outfile = 'trend_analysis/position_models/%s_position_v%03d.xml'%(ant,i)
                if os.path.exists(outfile):
                    print '%s already exists on disk. No need to Export again'%outfile
                else:
                    cmd = 'ExportPositionModel.py -a %s -v %d -f %s' %(ant,i,outfile)
                    print cmd
                    system_except(cmd)

                if i == 1:
                    pos[ant]={}
                    pos[ant]['version']=[]
                    pos[ant]['timestamp']=[]
                    pos[ant]['author']=[]
                    pos[ant]['comment']=[]
                    for k in range(3):
                        pos[ant][k]= []    

                p.getTmcdbPositionModel(outfile)
                tmppos = p.getTmcdbAntennaPosition(ant)
                
                ts = au.dateStringToMJD(versionsA[i]['timestamp'][0:-7], verbose = False)
                pos[ant]['version'].append(i)
                pos[ant]['timestamp'].append(ts)
                pos[ant]['author'].append(versionsA[i]['author'])
                pos[ant]['comment'].append(versionsA[i]['comment'])
                for k in range(3):
                    pos[ant][k].append(tmppos[k])
                    
            # save data with pickle
            allVersionsPickleFile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%ant
            f = open(allVersionsPickleFile,'w')
            print 'Saving this antennas allVersion pickle file:%s'%allVersionsPickleFile
            pickle.dump(pos[ant], f)
            f.close()

            # delete the individual xml
            cmd = 'rm -Rf trend_analysis/position_models/%s_position_v*.xml' %ant
            system_except(cmd)
                          
        padList = self.getPadNames()
        #padList = ['A001']
        for pad in padList:
            versionsP = versions[pad]
            
            if size(versionsP.keys()) == 1:
                print " WARNING: no versions for %s yet"%pad
                continue
                #raise warnings.filterwarnings("WARNING: no versions for %s yet"%pad)

            picklefile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%pad
            if os.path.exists(picklefile):
                print '\n ### Loading pad position model from %s...' %picklefile
                f = open(picklefile,'r')
                pos[pad] = pickle.load(f)
                f.close()

                nversions = size(pos[pad]['timestamp'])
                if nversions == versionsP.keys()[-1]:
                    print 'Pickle file contains up to versions: %d which is the latest version in TMCDB. No need to ExportPositionModel or update this antenna\' allVersion.pickle file'%(nversions)
                    continue
                else:
                    print 'pickle file contains up to versions: %d. Exporting up to versions %d'%(nversions,versionsP.keys()[-1])
            else:
                nversions = 1

            # now get all the pad position models
            for i in range(nversions,versionsP.keys()[-1]):
                print pad, i
                
                if i == 0: continue
                outfile = 'trend_analysis/position_models/%s_position_v%03d.xml'%(pad,i)
                if os.path.exists(outfile):
                    print '%s already exists on disk. No need to Export again'%outfile
                else:
                    cmd = 'ExportPositionModel.py -p %s -v %d -f %s' %(pad,i,outfile)
                    print cmd
                    system_except(cmd)

                if i == 1:
                    #initialize d
                    pos[pad]={}
                    pos[pad]['version']=[]
                    pos[pad]['timestamp']=[]
                    pos[pad]['author']=[]
                    pos[pad]['comment']=[]
                    for k in range(3):
                        pos[pad][k]= []    

                p.getTmcdbPositionModel(outfile)
                tmppos = p.getTmcdbPadPosition(pad)
                
                ts = au.dateStringToMJD(versionsP[i]['timestamp'][0:-7], verbose = False)
                pos[pad]['version'].append(i)
                pos[pad]['timestamp'].append(ts)
                pos[pad]['author'].append(versionsP[i]['author'])
                pos[pad]['comment'].append(versionsP[i]['comment'])
                for k in range(3):
                    pos[pad][k].append(tmppos[k])

            # save data with pickle
            allVersionsPickleFile = 'trend_analysis/position_models/%s_position_allVersions.pickle'%pad
            f = open(allVersionsPickleFile,'w')
            print 'Saving this pads allVersion pickle file:%s'%allVersionsPickleFile
            pickle.dump(pos[pad], f)
            f.close()

            # delete the individual xml
            cmd = 'rm -Rf trend_analysis/position_models/%s_position_v*.xml' %pad
            system_except(cmd)           

        return pos

    def getAllFocusModel(self,antList=None,versionsOnly=False):
        """
        For a list of antennas ants, this will get all the focus models
        including their history from the TMCDB  and output it in a dictionnary of
        models d (for the model)

        """
        d={}
        doff={}     
        p = FocusModelEditor()

        # first get the focus history for antennas for today
        today =('%s'%datetime.now())[0:10]
        pickleFile = 'trend_analysis/focus_models/%s_focus_allVersions.pickle'%today
        if os.path.isfile(pickleFile):
            print '\n### Loading All ants focus history from %s...' %pickleFile
            f = open(pickleFile,'r')
            versions = pickle.load(f)
            f.close()
        else:
            outfile = 'All_focus_History.xml'
            cmd = 'ExportFocusModel.py -y -f %s' %(outfile)
            print cmd
            system_except(cmd)
            p.getTmcdbFocusModelVersions(outfile)
            versions = p._versions
            # save today's raw version history in pickle file for today
            print '### Saving All ants focus history in pickle for today in : %s...'%pickleFile
            f = open(pickleFile,'w')
            pickle.dump(versions, f)
            f.close()

        if versionsOnly == True:
            return versions

        if antList == None: antList = self.getAntennaNames()
        for ant in antList:
            version = versions[ant]
            
            if size(version.keys()) == 1:
                print " WARNING: no versions for %s yet"%ant
                raise warnings.filterwarnings("WARNING: no versions for %s yet"%ant)

            picklefile = 'trend_analysis/focus_models/%s_focus_allVersions.pickle'%ant
            if os.path.exists(picklefile):
                print '\n### Loading antenna focus model from %s...' %picklefile
                f = open(picklefile,'r')
                d,doff = pickle.load(f)
                f.close()
                
                nversions = size(d[ant]['timestamp'])
                if nversions == version.keys()[-1]:
                    print 'Pickle file contains up to versions: %d --> latest in TMCDB. No need to ExportPositionModel or update %s_focus_allVersion.pickle'%(nversions,ant)
                    continue
                else:
                    print 'Pickle file contains up to versions: %d. Exporting up to versions %d'%(nversions,version.keys()[-1])
            else:
                nversions = 1

            # now get the antenna focus  models for all versions
            for i in range(nversions,version.keys()[-1]):
                print  ant,i

                # now get the models and offset models for that version
                if i == 0: continue
                outfile = 'trend_analysis/focus_models/%s_focus_v%03d.xml'%(ant,i)
                if os.path.exists(outfile):
                    print '%s already exists on disk. No need to Export again'%outfile
                else:
                    cmd = 'ExportFocusModel.py -a %s -v %d -f %s' %(ant,i,outfile)
                    print cmd
                    system_except(cmd)
                        
                p.getTmcdbFocusModel(outfile)
                model = p._models[ant]
                offsetmodel = p._offsetModels[ant]
               
                if i == 1:
                    #initialize d
                    d[ant]={}
                    doff[ant]={}
                    d[ant]['version']=[]
                    d[ant]['timestamp']=[]
                    d[ant]['author']=[]
                    d[ant]['comment']=[]                    
                    for k in ['YC', 'ZTA', 'XR', 'YR', 'ZR', 'ZS']:
                        d[ant][k]= []
                    for k in ['XR','YR','ZR']:
                        doff[ant][k] = {}
                        for l in offsetmodel[k].keys():
                            doff[ant][k][l]= []
                    
                ts = au.dateStringToMJD(version[i]['timestamp'][0:-7], verbose = False)
                d[ant]['version'].append(i)
                d[ant]['timestamp'].append(ts)
                d[ant]['author'].append(version[i]['author'])
                d[ant]['comment'].append(version[i]['comment'])
                for k in model.keys():
                    d[ant][k].append(model[k]*1e3) # convert to mm
                    
                for k in ['XR','YR','ZR']:
                    for l in offsetmodel[k].keys():
                        doff[ant][k][l].append(offsetmodel[k][l]*1e3)  # convert to mm
               
                # save data with pickle
                f = open('trend_analysis/focus_models/%s_focus_allVersions.pickle'%ant,'w')
                pickle.dump([d[ant],doff[ant]], f)
                f.close()

                # delete the individual xml
                cmd = 'rm -Rf trend_analysis/focus_models/%s_focus_v*.xml' %ant
                system_except(cmd)
                          
        return d, doff
        
    def getAllPointingModel(self,antList=None,versionsOnly=False):
        """
        For a list of antennas ants, this will get all the pointing models
        including their history from the TMCDB  and output it in a dictionnary of
        models d (for the model) and doff(for the CA and IE offsets).

        """
        # structure to store the models and the offset models
        d={}
        doff={}
        p = PointingModelEditor()

        # first get the pointing history for antennas for today
        today =('%s'%datetime.now())[0:10]
        pickleFile = 'trend_analysis/pointing_models/%s_pointing_allVersions.pickle'%today
        if os.path.isfile(pickleFile):
            print '\n### Loading All ants pointing history from %s...' %pickleFile
            f = open(pickleFile,'r')
            versions = pickle.load(f)
            f.close()
        else:
            outfile = 'All_pointing_History.xml'
            cmd = 'ExportPointingModel.py -y -f %s' %(outfile)
            print cmd
            system_except(cmd)
            p.getTmcdbPointingModelVersions(outfile)
            versions = p._versions
            # save today's raw version history in pickle file for today
            print '### Saving All ants pointing history in pickle for today in : %s...'%pickleFile
            f = open(pickleFile,'w')
            pickle.dump(versions, f)
            f.close()

        if versionsOnly == True:
            return versions

        if antList == None: antList = self.getAntennaNames()
        for ant in antList:
            version = versions[ant]
            
            if size(version.keys()) == 1:
                print " WARNING: no versions for %s yet"%ant
                raise warnings.filterwarnings("WARNING: no versions for %s yet"%ant)

            picklefile = 'trend_analysis/pointing_models/%s_pointing_allVersions.pickle'%ant
            if os.path.exists(picklefile):
                print '\n### Loading antenna pointing model from %s...' %picklefile
                f = open(picklefile,'r')
                dl,dloff = pickle.load(f)
                f.close()

                d[ant]=dl
                doff[ant]=dloff
                
                nversions = size(dl['timestamp'])
                if nversions == version.keys()[-1]:
                    print 'Pickle file contains up to versions: %d --> latest in TMCDB. No need to ExportPositionModel or update %s_pointing_allVersion.pickle'%(nversions,ant)
                    continue
                else:
                    print 'Pickle file contains up to versions: %d. Exporting up to versions %d'%(nversions,version.keys()[-1])
            else:
                nversions = 1
        
            # now get the antenna pointing  models for all versions
            for i in range(nversions,version.keys()[-1]):
                print  ant,i

                # now get the models and offset models for that version
                if i == 0: continue
                outfile = 'trend_analysis/pointing_models/%s_pointing_v%03d.xml'%(ant,i)
                if os.path.exists(outfile):
                    print '%s already exists on disk. No need to Export again'%outfile
                else:
                    cmd = 'ExportPointingModel.py -a %s -v %d -f %s' %(ant,i,outfile)
                    print cmd
                    system_except(cmd)

                p.getTmcdbPointingModel(outfile)
                model = p._models[ant]
                offsetmodel = p._offsetModels[ant]                
                modelCoeffs = ['CA','IE','IA','AN','AW','NPAE','HESA','HESA2','HESA3',\
                               'HESE','HECE','HECA3','HECA2','HACA3','HACA2','HACA','HASA3','HASA2','HASA']
                if i == 1:
                    #initialize d,doff
                    d[ant]={}
                    doff[ant]={}
                    d[ant]['version']=[]
                    d[ant]['timestamp']=[]
                    d[ant]['author']=[]
                    d[ant]['comment']=[]
                    for k in modelCoeffs:
                        d[ant][k]= []
                        if k in ['CA','IE']:
                            doff[ant][k] = {}
                            for l in offsetmodel[k].keys(): doff[ant][k][l]= []
                    
                ts = au.dateStringToMJD(version[i]['timestamp'][0:-7], verbose = False)
                d[ant]['version'].append(i)
                d[ant]['timestamp'].append(ts)
                d[ant]['author'].append(version[i]['author'])
                d[ant]['comment'].append(version[i]['comment'])
                for k in modelCoeffs:
                    d[ant][k].append(model[k])
                    if k in ['CA','IE']:
                        for l in offsetmodel[k].keys():
                            doff[ant][k][l].append(offsetmodel[k][l])
               
                # save data with pickle
                savefile = 'trend_analysis/pointing_models/%s_pointing_allVersions.pickle'%ant
                print '\n### Saving pointing model to %s...' %savefile
                f = open(savefile,'w')
                pickle.dump([d[ant],doff[ant]], f)
                f.close()

                # delete the individual xml
                cmd = 'rm -Rf trend_analysis/pointing_models/%s_pointing_v*.xml' %ant
                system_except(cmd)
        return d, doff


    def writeVersionsToHTML(self, PtVersions=None, FcVersions=None, PosVersions=None):
        """
        Simple script to write HTML of all versions ( Pt, Focus, Antpos, PadPos)

        """
        print '### Making HTML page of TMCDB versions ... ###'

        if PtVersions == None:
            PtVersions = self.getAllPointingModel(versionsOnly=True)
        if FcVersions == None:
            FcVersions = self.getAllFocusModel(versionsOnly=True)
        if PosVersions == None:
            PosVersions = self.getAllPositionModel(versionsOnly=True)

        antList = self.getAntennaNames()
        padList = self.getPadNames()
        htmldir = 'trend_analysis/history_html'
        
        now =('%s'%datetime.now())
        today = now[0:10]
        antPadDict = self.inversePadFind(today, cvsup=False, verbose=False)
        padAntDict = dict(zip(antPadDict.values(),antPadDict.keys()))
        
        # create individual plot .html pages
        for ant in antList:
            padAnt = antPadDict[ant]
            if padAnt == 'OSF': padAnt = 'A001'
            #print "%s ---> %s"%(ant, padAnt)
            version = PtVersions[ant]
            b= open(htmldir+'/%s_pointing_history.html'%ant,'w')
            b.write('<html>\n ')
            b.write('<body>\n')
            b.write('<table>\n')
            b.write('<tr> \n')
            b.write('  <td colspan="2"> \n')
            b.write('    <center><img src="plots/title.png"></center> \n')
            b.write('    <b><center>Last updated: %s </center></b> \n'%now)
            b.write('    <center><h3> [Pointing] <a href ="%s_focus_history.html"> Focus </a> <a href ="%s_antPos_history.html"> antPos </a> <a href ="A001_padPos_history.html"> <a href="%s_padPos_history.html">padPos </a>   </h3></center> \n'%(ant,ant,padAnt))
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('</table> \n')
            k=0
            for ant2 in antList:
                if ant2 == ant:
                    b.write('<strong>[%s]</strong> '%ant2)
                    k +=1
                else:
                    b.write('<a href="%s_pointing_history.html">%s</a> '%(ant2,ant2))
                    k+=1
                if (k ==12)  or (k ==25) or (k ==37) or (k == 50) or (k == 54) : b.write('<br>\n')
            b.write('<p>\n')
            b.write('<table border="0"  cellpadding = "5" style="font-size:14px;" frame=vsides>\n')
            b.write('<tr><td><strong>VER#</td>  <td style="width:200px;"><strong> DATE</td><td><strong> AUTHOR</td> <td style="width:200px"><strong> COMMENT</td></tr> \n')
            for k in sorted(version.keys(),reverse=True):
                b.write('<tr><td>%03d</td> <td> %s</td> <td> %s </td> <td> %s</td>   </tr>\n'%(k,version[k]['timestamp'][0:19],version[k]['author'],version[k]['comment']))
            b.write('</table> \n')
            b.write('</body> \n')
            b.write('</html> \n')
            b.close()
            os.system( 'chmod 666 '+htmldir+'/%s_pointing_history.html'%ant)

            version = FcVersions[ant]
            b= open(htmldir+'/%s_focus_history.html'%ant,'w')
            b.write('<html>\n ')
            b.write('<body>\n')
            b.write('<table>\n')
            b.write('<tr> \n')
            b.write('  <td colspan="2"> \n')
            b.write('    <center><img src="plots/title.png"></center> \n')
            b.write('    <b><center>Last updated: %s </center></b> \n'%now)
            b.write('    <center><h3> <a href="%s_pointing_history.html"> Pointing </a> [Focus] <a href ="%s_antPos_history.html"> antPos </a> <a href ="%s_padPos_history.html"> padPos </a>   </h3></center> \n'%(ant,ant, padAnt))
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('</table> \n')
            k =0
            for ant2 in antList:
                if ant2 == ant:
                    b.write('<strong>[%s]</strong> '%ant2)
                    k +=1
                else:
                    b.write('<a href="%s_focus_history.html">%s</a> '%(ant2,ant2))
                    k+=1
                if (k ==12)  or (k ==25) or (k ==37) or (k == 50) or (k == 54) : b.write('<br>\n')
            b.write('<p>\n')
            
            b.write('<table border="0"  cellpadding = "5" style="font-size: 14px;" frame = vsides>\n')
            b.write('<tr><td><strong>VER#</td>  <td><strong> DATE</td><td><strong> AUTHOR</td> <td><strong> COMMENT</td></tr> \n')
            for k in sorted(version.keys(),reverse=True):
                b.write('<tr><td>%03d</td> <td> %s</td> <td> %s </td> <td> %s</td>   </tr>\n'%(k,version[k]['timestamp'][0:19],version[k]['author'],version[k]['comment']))
                
            b.write('</table> \n')
            b.write('</body> \n')
            b.write('</html> \n')
            b.close()
            os.system( 'chmod 666 '+htmldir+'/%s_focus_history.html'%ant)

            version = PosVersions[ant]
            b= open(htmldir+'/%s_antPos_history.html'%ant,'w')
            b.write('<html>\n ')
            b.write('<body>\n')
            b.write('<table>\n')
            b.write('<tr> \n')
            b.write('  <td colspan="2"> \n')
            b.write('    <center><img src="plots/title.png"></center> \n')
            b.write('    <b><center>Last updated: %s </center></b> \n'%now)
            b.write('    <center><h3> <a href="%s_pointing_history.html"> Pointing </a><a href="%s_focus_history.html"> Focus </a> [antPos] <a href ="%s_padPos_history.html"> padPos </a>   </h3></center> \n'%(ant,ant, padAnt))
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('</table> \n')
            k =0
            for ant2 in antList:
                if ant2 == ant:
                    b.write('<strong>[%s]</strong> '%ant2)
                    k +=1
                else:
                    b.write('<a href="%s_antPos_history.html">%s</a> '%(ant2,ant2))
                    k+=1
                if (k ==12)  or (k ==25) or (k ==37) or (k == 50) or (k == 54) : b.write('<br>\n')
            b.write('<p>\n')
            
            b.write('<table border="0"  cellpadding = "5" style="font-size: 14px;" frame = vsides>\n')
            b.write('<tr><td><strong>VER#</td>  <td><strong> DATE</td><td><strong> AUTHOR</td> <td><strong> COMMENT</td></tr> \n')
            for k in sorted(version.keys(),reverse=True):
                b.write('<tr><td>%03d</td> <td> %s</td> <td> %s </td> <td> %s</td>   </tr>\n'%(k,version[k]['timestamp'][0:19],version[k]['author'],version[k]['comment']))

            b.write('</table> \n')
            b.write('</body> \n')
            b.write('</html> \n')
            b.close()
            os.system( 'chmod 666 '+htmldir+'/%s_antPos_history.html'%ant)
                
        for pad in padList:
            if pad in padAntDict.keys():
                padAnt = padAntDict[pad]
            else:
                padAnt = 'DV01'
            #print "%s ---> %s"%(pad,padAnt)
            
            version = PosVersions[pad]
            b= open(htmldir+'/%s_padPos_history.html'%pad,'w')
            b.write('<html>\n ')
            b.write('<body>\n')
            b.write('<table>\n')
            b.write('<tr> \n')
            b.write('  <td colspan="2"> \n')
            b.write('    <center><img src="plots/title.png"></center> \n')
            b.write('    <b><center>Last updated: %s </center></b> \n'%now)
            b.write('    <center><h3> <a href="%s_pointing_history.html"> Pointing </a><a href="%s_focus_history.html"> Focus </a> <a href="%s_antPos_history.html"> antPos </a> [padPos]   </h3></center> \n'%(padAnt,padAnt,padAnt))
            b.write('  </td> \n')
            b.write('</tr> \n')
            b.write('</table> \n')
            k =0
            for pad2 in padList:
                if pad2 == pad:
                    b.write('<strong>[%s]</strong> '%pad2)
                    k +=1
                else:
                    b.write('<a href="%s_padPos_history.html">%s</a> '%(pad2,pad2))
                    k+=1
                if mod(k,16) == 0: b.write('<br>\n')
            b.write('<p>\n')
            
            b.write('<table border="0"  cellpadding = "5" style="font-size: 14px;" frame = vsides>\n')
            b.write('<tr><td><strong>VER#</td>  <td><strong> DATE</td><td><strong> AUTHOR</td> <td><strong> COMMENT</td></tr> \n')
            for k in sorted(version.keys(),reverse=True):
                b.write('<tr><td>%03d</td> <td> %s</td> <td> %s </td> <td> %s</td>   </tr>\n'%(k,version[k]['timestamp'][0:19],version[k]['author'],version[k]['comment']))
                
            b.write('</table> \n')
            b.write('</body> \n')
            b.write('</html> \n')
            b.close()
                
        host='dbarkats@ssh.alma.cl'
        cmd='rsync -auv --progress /users/dbarkats/trend_analysis/history_html %s:~/public_html/trend_analysis/ ' %(host)
        print "### Rsyncing to dbarkats@ssh.alma.cl... ###"
        os.system(cmd)

######################################        
###### Trend Plotting functions #######
######################################
        
    def plotAllAntennaFocusHistory(self):
        
        ants = self.getAntennaNames()
        ants = ['DV12']
        d,doff = self.getAllPointingModel(ants)
        self.readAntennaMovesTxt()
        
        #initilize variables to store band pointing offset mean/std
        self.BandOffsets={}
        for ant in ants:
            self.BandOffsets[ant]={}
            self.BandOffsets[ant]['XR']={}
            self.BandOffsets[ant]['YR']={}
            self.BandOffsets[ant]['ZR']={}
            self.plotFocusHistory(ant,d,doff)

        bo = self.BandOffsets
        print '### CA offset parameter summary plot... ####'
        figure(4,figsize=(12,9));clf()
        self.makeOffsetSummarySubPlot(1,bo,doff,'XR',6)
        self.makeOffsetSummarySubPlot(2,bo,doff,'XR',7)
        self.makeOffsetSummarySubPlot(3,bo,doff,'XR',9)
        plotname = 'CA_band_offsets_summary.png'
        savefig('trend_analysis/pointing_plots/%s'%plotname)    

        print '### IE offset parameter summary plot... ####'
        figure(5,figsize=(12,9));clf()
        self.makeOffsetSummarySubPlot(1,bo,doff,'IE',6)
        self.makeOffsetSummarySubPlot(2,bo,doff,'IE',7)
        self.makeOffsetSummarySubPlot(3,bo,doff,'IE',9)
        plotname = 'IE_band_offsets_summary.png'
        savefig('trend_analysis/pointing_plots/%s'%plotname)    

        # get the pointing parameters
        vars = d[ants[0]].keys()
        vars.remove('timestamp')
        vars.remove('comment')
        vars.remove('author')

        # get the stats for all the variables
        stats = self.getStats(d, vars)
        
        # make summary plots for all vars.
        j=0
        for var in vars:
            print '### Main Summary of %sPt parameter plot... ####'%var
            figure(17+j,figsize=(12,9));clf()
            self.makeSummaryPlot(stats,d,var)
            j +=1
            
        return self.BandOffsets

    def plotFocusHistory(self,ant,d=None,doff=None):

        try:
            self.BandOffsets
        except:
            self.BandOffsets={}
            for ant in [ant]:
                self.BandOffsets[ant]={}
                self.BandOffsets[ant]['XR']={}
                self.BandOffsets[ant]['YR']={}
                self.BandOffsets[ant]['ZR']={}
 
        if d == None and doff ==None:
            d,doff = self.getAllFocusModel([ant], readFromTMCDB = False)

        try:
            self.moves
            move = self.moves
        except:
            move = self.readAntennaMovesTxt()
            
        self.movLim = []
        for i in range(size(move[ant].start)):
            self.movLim.append(au.dateStringToMJD(move[ant].start[i], verbose = False))

        self.makeXgridDates()
        print ant
        # plot Master B7 focus model XR, YR, ZR, ZS, YC, ZTA
        print '### Master 6 Focus parameter plot... ####'
        figure(1,figsize=(12,9));clf()
        self.makeSubPlot(1,d,ant,'XR')
        self.makeSubPlot(2,d,ant,'YR')
        self.makeSubPlot(3,d,ant,'ZR')
        self.makeSubPlot(4,d,ant,'ZS')
        self.makeSubPlot(5,d,ant,'YC')
        self.makeSubPlot(6,d,ant,'ZTA')
        suptitle('%s Master B7 focus parameters [mm]' %ant, y=0.96, size='large')
        plotname = '%s_focus_parameters_master.png' %(ant)
        savefig('trend_analysis/focus_plots/%s'%plotname)
        
        # plot XR pointing offsets
        print '### XR Focus offset parameter plot... ####'
        figure(2,figsize=(12,9));clf()
        self.makeOffsetSubPlot(1,d,doff,ant,'XR',3)
        self.makeOffsetSubPlot(2,d,doff,ant,'XR',6)
        self.makeOffsetSubPlot(3,d,doff,ant,'XR',9)
        suptitle('%s XR Focus Band offset (w/r B7) [mm]' %ant, y=0.96, size='large')
        plotname = '%s_XR_focus_band_offsets.png' %(ant)
        savefig('trend_analysis/focus_plots/%s'%plotname)

        # plot YR pointing offsets
        print '### YR Focus offset parameter plot... ####'
        figure(3,figsize=(12,9));clf()
        antList = sort(bo.keys())
        self.makeOffsetSubPlot(1,d,doff,ant,'YR',3)
        self.makeOffsetSubPlot(2,d,doff,ant,'YR',6)
        self.makeOffsetSubPlot(3,d,doff,ant,'YR',9)
        suptitle('%s YR Focus Band offset (w/r B7) [mm]' %ant, y=0.94, size='large')
        plotname = '%s_YR_focus_band_offsets.png' %(ant)
        savefig('trend_analysis/focus_plots/%s'%plotname)

        # plot ZR pointing offsets
        print '### ZR FOcus offset parameter plot... ####'
        figure(2,figsize=(12,9));clf()
        self.makeOffsetSubPlot(1,d,doff,ant,'ZR',3)
        self.makeOffsetSubPlot(2,d,doff,ant,'ZR',6)
        self.makeOffsetSubPlot(3,d,doff,ant,'ZR',9)
        suptitle('%s ZR Focus Band offset (w/r B7) [mm]' %ant, y=0.94, size='large')
        plotname = '%s_CA_band_offsets.png' %(ant)
        savefig('trend_analysis/focus_plots/%s'%plotname)

        return self.BandOffsets  
    
    def plotAllAntennaPointingHistory(self):
        ants = self.getAntennaNames()
        d,doff = self.getAllPointingModel(ants)
        self.readAntennaMovesTxt()
        BandOffsets={}

        for ant in ants:
            BandOffsets[ant]={}
            BandOffsets[ant]['CA']={}
            BandOffsets[ant]['IE']={}
            self.BandOffsets = BandOffsets
            self.plotPointingHistory(ant,d,doff)

        bo = self.BandOffsets
        print '### CA/IE offset parameter summary plot for CMs... ####'
        figure(4,figsize=(15,20));clf()
        cmList  = self.getAntennaNames()[54:66]
        self.makeOffsetSummarySubPlot(1,bo,doff,'CA',4, cmList)
        self.makeOffsetSummarySubPlot(2,bo,doff,'IE',4, cmList)
        self.makeOffsetSummarySubPlot(3,bo,doff,'CA',6, cmList)
        self.makeOffsetSummarySubPlot(4,bo,doff,'IE',6, cmList)
        self.makeOffsetSummarySubPlot(5,bo,doff,'CA',7, cmList)
        self.makeOffsetSummarySubPlot(6,bo,doff,'IE',7, cmList)
        self.makeOffsetSummarySubPlot(7,bo,doff,'CA',8, cmList)
        self.makeOffsetSummarySubPlot(8,bo,doff,'IE',8, cmList)
        self.makeOffsetSummarySubPlot(9,bo,doff,'CA',9, cmList)
        self.makeOffsetSummarySubPlot(10,bo,doff,'IE',9, cmList)
        self.makeOffsetSummarySubPlot(11,bo,doff,'CA',10, cmList)
        self.makeOffsetSummarySubPlot(12,bo,doff,'IE',10, cmList)
        plotname = '7m_CA_IE_band_offsets_summary.png'
        savefig('trend_analysis/pointing_plots/%s'%plotname)    

        print '### CA/IE offset parameter summary plot for 12m... ####'
        figure(5,figsize=(15,20));clf()
        ants12m  = self.getAntennaNames()[0:54]
        self.makeOffsetSummarySubPlot(1,bo,doff,'CA',4, ants12m)
        self.makeOffsetSummarySubPlot(2,bo,doff,'IE',4, ants12m)
        self.makeOffsetSummarySubPlot(3,bo,doff,'CA',6, ants12m)
        self.makeOffsetSummarySubPlot(4,bo,doff,'IE',6, ants12m)
        self.makeOffsetSummarySubPlot(5,bo,doff,'CA',7, ants12m)
        self.makeOffsetSummarySubPlot(6,bo,doff,'IE',7, ants12m)
        self.makeOffsetSummarySubPlot(7,bo,doff,'CA',8, ants12m)
        self.makeOffsetSummarySubPlot(8,bo,doff,'IE',8, ants12m)
        self.makeOffsetSummarySubPlot(9,bo,doff,'CA',9, ants12m)
        self.makeOffsetSummarySubPlot(10,bo,doff,'IE',9, ants12m)
        self.makeOffsetSummarySubPlot(11,bo,doff,'CA',10, ants12m)
        self.makeOffsetSummarySubPlot(12,bo,doff,'IE',10, ants12m)
        plotname = '12m_CA_IE_band_offsets_summary.png'
        savefig('trend_analysis/pointing_plots/%s'%plotname)    

        # get the pointing parameters
        vars = d[ants[0]].keys()
        vars.remove('timestamp')
        vars.remove('comment')
        vars.remove('author')
        vars.remove('version')
        print vars

        # get the stats for all the variables
        stats = self.getStats(d, vars)

        
        # make summary plots for all vars.
        j=0
        for var in vars:
            print '### Main Summary of %s Pt parameter plot... ####'%var
            figure(17+j,figsize=(12,9));clf()
            self.makeSummaryPlot(stats,d,var)
            j +=1
            
        return self.BandOffsets,stats,d,doff

    def plotPointingHistory(self,ant,d=None,doff=None):

        # get means/STDEVS of NPAE and plot against AIV station 3 or Station 4 models
        # TODO: get some stats about changes within a pad.
        # add RMS from  Andreas's outmod

        today =('%s'%datetime.now())[0:10]
        
        if d == None and doff ==None:
            d,doff = self.getAllPointingModel([ant], readFromTMCDB = False)

        try:
            self.moves
            move = self.moves
        except:
            move = self.readAntennaMovesTxt()
            
        self.movLim = []
        for i in range(size(move[ant].start)):
            self.movLim.append(au.dateStringToMJD(move[ant].start[i], verbose = False))

        self.makeXgridDates()
        print ant
        # plot CA, IE, IA, AN, AW, NPAE vs time
        print '### Main 6 Pt parameter plot... ####'
        figure(1,figsize=(24,18));clf()
        self.makeSubPlot(1,d,ant,'CA')
        self.makeSubPlot(2,d,ant,'IE')
        self.makeSubPlot(3,d,ant,'IA')
        self.makeSubPlot(4,d,ant,'AN')
        self.makeSubPlot(5,d,ant,'AW')
        self.makeSubPlot(6,d,ant,'NPAE')
        suptitle('%s Main 6 pointing parameters [arcsec] (Updated: %s)'%(ant,today), y=0.94, size='large')
        plotname = '%s_pointing_parameters_main.png' %(ant)
        savefig('trend_analysis/pointing_plots/%s'%plotname)

         # plot higher order terms
        print '### Main higher order Pt parameter plot... ####'
        figure(6,figsize=(24,18));clf()
        self.makeSubPlot(1,d,ant,'HACA','HACA2','HACA3',[-5,5])
        self.makeSubPlot(2,d,ant,'HASA','HASA2','HASA3',[-5,5])
        self.makeSubPlot(3,d,ant,'HESA','HESA2','HESA3',[-5,5])
        self.makeSubPlot(4,d,ant,'HECA2','HECA3',None,[-5,5])
        self.makeSubPlot(5,d,ant,'HESE', 'HECE',None, [-5,5])
        self.makeSubPlot(6,d,ant,'NPAE')
        suptitle('%s Higher order pointing parameters [arcsec] (Updated: %s)' %(ant,today), y=0.94, size='large')
        plotname = '%s_pointing_parameters_higherOrder.png' %(ant)
        savefig('trend_analysis/pointing_plots/%s'%plotname)    
        
        # plot CA pointing offsets
        print '### CA Pt offset parameter plot... ####'
        figure(2,figsize=(24,18));clf()
        self.makeOffsetSubPlot(1,d,doff,ant,'CA',4)
        self.makeOffsetSubPlot(2,d,doff,ant,'CA',6)
        self.makeOffsetSubPlot(3,d,doff,ant,'CA',7)
        self.makeOffsetSubPlot(4,d,doff,ant,'CA',8)
        self.makeOffsetSubPlot(5,d,doff,ant,'CA',9)
        self.makeOffsetSubPlot(6,d,doff,ant,'CA',10)
        suptitle('%s CA Band offset (w/r B3), [arcsec] (Updated: %s)' %(ant,today), y=0.94, size='large')
        plotname = '%s_CA_band_offsets.png' %(ant)
        savefig('trend_analysis/pointing_plots/%s'%plotname)

        # plot IE pointing offsets
        print '### IE Pt offset parameter plot... ####'
        figure(3,figsize=(24,18));clf()
        self.makeOffsetSubPlot(1,d,doff,ant,'IE',4)
        self.makeOffsetSubPlot(2,d,doff,ant,'IE',6)
        self.makeOffsetSubPlot(3,d,doff,ant,'IE',7)
        self.makeOffsetSubPlot(4,d,doff,ant,'IE',8)
        self.makeOffsetSubPlot(5,d,doff,ant,'IE',9)
        self.makeOffsetSubPlot(6,d,doff,ant,'IE',10)
        suptitle('%s IE Band offset (w/r B3), [arcsec] (Updated: %s)' %(ant,today), y=0.94, size='large')
        plotname = '%s_IE_band_offsets.png' %(ant)
        savefig('trend_analysis/pointing_plots/%s'%plotname)

        return self.BandOffsets
    

    def getStats(self,d,vars):
        ants = self.getAntennaNames()
        move = self.readAntennaMovesTxt()
        now ='%s'%datetime.now()

        latest = 1 # to take mean/std since last pad change
        stats = {}
        for ant in ants:
            stats[ant]={}

            if latest:
                # get period of time  on latest pad
                start = au.strDate2MJD(move[ant].start[-1])
                end = au.strDate2MJD(now)
                
                q = find((array(d[ant]['timestamp'])< end) & (array(d[ant]['timestamp'])> start))
                n = len(q)
            else:
                q = find((array(d[ant]['timestamp'])< end))

            print '%s: %3d  Pt models since last pad change on %s'%(ant, n,move[ant].start[-1])
            for var in vars:
                if n == 0:
                    m = float('nan')
                    s = float('nan')
                else:
                    m = mean(array(d[ant][var])[q])
                    s = std(array(d[ant][var])[q])
                stats[ant][var]=[m,s,n]

        return stats

    def makeSummaryPlot(self,stats,d,var):
        """
        make summary  plots for all main pointing parameters (other than CA and IE)
        make 1 plot per parameter.

        """

        today =('%s'%datetime.now())[0:10]
        b = []
        latest = []
        antList = sort(stats.keys())
        nants = len(antList)
        for ant in antList:
            latest.append(d[ant][var][-1]) 
        for ant in antList:
            b.append(stats[ant][var])

        nsubplots = 1
        ax=subplot(nsubplots,1,1) # 4,5,6,7,8,9,10
        
        errorbar(arange(nants),array(b)[:,0],array(b)[:,1],fmt = '.', markersize=4)
        plot(arange(nants),latest,'ro', markersize = 4)    
        xlim([-1,nants])
        xticks(arange(nants),antList,rotation = 90)
        ax.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        yl = ylim()
        for i in arange(nants):
            text(i,0.86*yl[1],'%d'%(array(b)[i,2]),rotation=90,size='small')
        ylabel(var)

        #add labels to the points in last plot
        title('%s Summary. Blue w/error: historical mean, Red:last value  (Updated: %s)'%(var,today))
        plotname = '%s_pointing_parameter_summary.png' %(var)
        savefig('trend_analysis/pointing_plots/%s'%plotname)
 
        
    def makeOffsetSummarySubPlot(self,n,bo,doff,var,band, antList):
        '''
        plot summary (average and std over time) of CA and IE
        '''

        today =('%s'%datetime.now())[0:10]
        b = []
        latest = []
        nants = len(antList)

        nominal = self.nominal
        for ant in antList:
            latest.append(doff[ant][var][band][-1]) 
        for ant in antList:
            b.append(bo[ant][var][band])

        x = arange(nants)
        y = array(b)[:,0]
        yerr = array(b)[:,1]
        nrows = 6 # 6 bands (4,6,7,8,9,10)
        ncol = 2 # CA and IE
        ax=subplot(nrows,ncol,n)

        errorbar(x,y,yerr,fmt = 'o')
        plot(x,latest,'ro')#, markersize = 3)    
        xlim([-1,nants])
        xticks(x,antList,rotation = 90)
        ax.xaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)

        if ant[0:2] == 'CM':
            nom=nominal['7'][band][var]
        else:
            nom = nominal['12'][band][var]
        if abs(nom) > 50:
            q = find((y != 0) &((abs(y)>abs(0.8*nom)) & (abs(y)<abs(1.2*nom))))
        else:
            q = find(y != 0) 

        ant_mean,ant_std = self.weighted_avg_std(array(b)[q,0],array(b)[q,1])
        
        text(0.3,0.85,'Nominal=%2.1f Mean= %2.2f +- %2.2f'%(nom,ant_mean,ant_std),transform= ax.transAxes)
        axhline(nom,xmin=0, xmax=1, linestyle='--', color = 'k')
                    
        subplots_adjust(hspace = 0.001)
        if (n != nrows*ncol) and (n != nrows*ncol-1):ax.set_xticklabels(())
        ylabel('%s%d [arcsec]'%(var,band))

        #add labels to the points in last plot
        if  n == 1:
            suptitle('%s Band offset Summary. Blue w/error: historical mean, Red:last value (Updated: %s)'%(var,today),y=0.94, size='large')
             
    def makeSubPlot(self,n,d,ant,var,var2=None,var3=None, yl=None):
        nsubplots = 6
        c = 2
        try:
            move = self.moves
        except:
            move = self.readAntennaMovesTxt()
        ax=subplot(nsubplots,1,n)
        for x in [self.y12,self.y14]:
            axvspan(x,x+365,color='g',alpha = 0.1)
        m = mean(d[ant][var])
        s = std(d[ant][var])
        plot(d[ant]['timestamp'],d[ant][var],'o', markersize=3)
        if var3 != None:
            plot(d[ant]['timestamp'],d[ant][var2],'o', markersize=3)
            plot(d[ant]['timestamp'],d[ant][var3],'o', markersize=3)
            legend([var,var2,var2],bbox_to_anchor=(1.1, 1.05), prop={'size':10})
            ylabel(var)
        elif var2 != None:
            plot(d[ant]['timestamp'],d[ant][var2],'o', markersize=3)
            legend([var,var2],bbox_to_anchor=(1.1, 1.05), prop={'size':10})
            ylabel(var)
        else:
            ylabel(var)
        subplots_adjust(hspace = 0.001)
        if n != nsubplots:ax.set_xticklabels(())

        if var == 'ZS' or var == 'YC':
            print m
            ylim((m-0.2,m+0.2))
        elif yl == None:
            if s == 0 : s = 2
            ylim((m-c*s,m+c*s))
        else:
            ylim(yl)
        yl = ylim()
        if n == 1:
            text(self.y12+151,yl[1],'2012', size='x-large', style = 'normal')
            text(self.y13+151,yl[1],'2013', size='x-large', style = 'normal')
            text(self.y14+151,yl[1],'2014', size='x-large', style = 'normal')
            
        for i in range(size(self.movLim)):
            axvline(x=self.movLim[i],color='r')
            if i != 0: text(self.movLim[i]-7,m,move[ant].pad[i-1], rotation= 'vertical', size= 'small')
            text(self.movLim[i]+4,m,move[ant].pad[i], rotation = 'vertical',size= 'small')
        ax.yaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        axvline(x=self.yn, color = 'c')
        xlim([56000,57100])
        #add labels to the points in last plot
        if  n == nsubplots:
            labels = range(len(d[ant][var]))
            labels1 = labels[::10]
            labels1.append(labels[-1])
            for k in labels1:
                annotate(str(k+1),xy = (d[ant]['timestamp'][k],d[ant][var][k]),size = 'x-small',
                         xytext = (0, +2),textcoords = 'offset points', ha = 'right', va = 'bottom',
                         arrowprops = dict(arrowstyle = '->', connectionstyle = 'arc3,rad=0'))
            
    def makeOffsetSubPlot(self,n,d,doff,ant,var,band):

        move = self.moves
        nominal = self.nominal
        c = 2
        nsubplots = 6 # 6 bands
        ax=subplot(nsubplots,1,n) # 4,5,6,7,8,9,10
        for k in [self.y12,self.y14]:
            axvspan(k,k+365,color='g',alpha = 0.1)
        axvline(x=self.yn, color = 'c')
        x = array(d[ant]['timestamp'])
        y = array(doff[ant][var][band])
        plot(x,y,'o', markersize=3)
        
        # exclude incorrect points such as value = 0
        #or abs(value) is lt 1.2 or gt 0.8 nominal
        if ant[0:2] == 'CM':
            nom=nominal['7'][band][var]
        else:
            nom = nominal['12'][band][var]
        if var != 'XR' and var != 'YR' and var != 'ZR':
            if abs(nom) > 50:
                q = find((y != 0) &((abs(y)>abs(0.8*nom)) & (abs(y)<abs(1.2*nom))))
            else:
                q = find(y != 0) 
        else:
            q = find(y != 0)
        
        if len(q) != 0:
            m = mean(y[q])
            s = std(y[q])
        #print ant,var,band,m,s

        self.BandOffsets[ant][var][band]=[m,s]
        
        text(0.8, 0.9, '%2.1f +- %2.2f'%(m,s),transform= ax.transAxes)
        subplots_adjust(hspace = 0.001)
        if n != nsubplots:
            ax.set_xticklabels(())
        if s == 0: s = 4
        ylim((m-c*s,m+c*s))
        
        yl = ylim()
        for i in range(size(self.movLim)):
            axvline(x=self.movLim[i],color='r')
            if i != 0:
                text(self.movLim[i]-7,m,move[ant].pad[i-1], rotation= 'vertical', size= 'small')
            text(self.movLim[i]+4,m,move[ant].pad[i], rotation = 'vertical',size= 'small')
        ax.yaxis.grid(linestyle = '-.',color = 'gray', alpha = 0.8)
        xlim([56000,57100])
        ylabel('%s%d [arcsec]'%(var,band))

        if n == 1:
            text(self.y12+151,yl[1],'2012', size='x-large', style = 'normal')
            text(self.y13+151,yl[1],'2013', size='x-large', style = 'normal')
            text(self.y14+151,yl[1],'2014', size='x-large', style = 'normal')
            

    def makeXgridDates(self):
        now ='%s'%datetime.now()
        #now = now.replace('-','')
        year = now[0:4]
        self.yn = au.strDate2MJD(now)
        self.y11 = au.strDate2MJD('20110101')
        self.y12 = au.strDate2MJD('20120101')
        self.y13 = au.strDate2MJD('20130101')
        self.y14 = au.strDate2MJD('20140101')
        self.ya = au.strDate2MJD('%s0101'%year)
        self.yb = au.strDate2MJD('%s0201'%year)
        self.yc = au.strDate2MJD('%s0301'%year)
        self.yd = au.strDate2MJD('%s0401'%year)
        self.ye = au.strDate2MJD('%s0501'%year)
        self.yf = au.strDate2MJD('%s0601'%year)
        self.yg = au.strDate2MJD('%s0701'%year)
        self.yh = au.strDate2MJD('%s0801'%year)
        self.yi = au.strDate2MJD('%s0901'%year)
        self.yj = au.strDate2MJD('%s1001'%year)
        self.yk = au.strDate2MJD('%s1101'%year)
        self.yl = au.strDate2MJD('%s1201'%year)
                

    def weighted_avg_std(self,values, weights):
        """
        Return the weighted average and standard deviation.
        values, weights -- Numpy ndarrays with the same shape.
        """
        avg = average(values, weights=weights)
        var = average((values-avg)**2, weights=weights)  # Fast and numerically precise
        return (avg, sqrt(var))
