#!/usr/bin/env python
"""
Convert ShiftLog HTML report into plain text.

usage: unmarkup.py -f hoge.html
$Id: unmarkup.py,v 1.2 2013/07/24 14:40:29 tsawada Exp $
"""

import os
import re
import datetime
from HTMLParser import HTMLParser
from optparse import OptionParser

class SLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.initentry()
        self._insidetr = False
        self._prevdata = ''
        self._date = ''
        self._data = ''

    def handle_starttag(self, tag, attrs):
        if tag == 'tr':
            self._insidetr = True
        elif tag == 'tr':
            self._insidetd = True

    def handle_endtag(self, tag):
        if tag == 'tr':
            self._insidetr = False
        if tag == 'td':
            self.flushdata()
            self._insidetd = False
        if tag == 'html':
            self.writeentry()

    def handle_data(self, data):
        sdata = data.strip()
        self._data += sdata

    def flushdata(self):
        if self._insidetr and len(self._data)>0:
            if self._data in ['ENTRY', 'ENGIN', 'MMEX', 'SBEX', 'SHIFT', 'WEATH', 'DOWN', 'ANTEN', 'ARRAY']:
                self.writeentry()
                self.initentry()
                self._entry['mode'] = self._data
            elif self._data[:2] == '--':
                self._entry['status'] = self._data
                self._entry['cmdline'] = self._prevdata
            elif re.match(r'[A-Z][a-z]+day, [0-9]+ [A-Z][a-z]+ [0-9]+', self._data):
                d = self._data.split()
                self._date = datetime.datetime.strptime(self._data, '%A, %d %b %Y').strftime('%Y-%m-%d')
            if self._prevdata == 'Start/End':
                if 'T' in self._data:
                    self._entry['timestamp'] = self._data
                else:
                    self._entry['timestamp'] = '%sT%s' % (self._date, self._data)
            elif self._prevdata == 'ExecBlock':
                self._entry['execblock'] = self._data
            elif self._prevdata == 'SchedBlock':
                self._entry['scdblock'] = self._data
            elif self._prevdata == 'Project Code':
                self._entry['prjcode'] = self._data
            elif self._prevdata == 'PI':
                self._entry['pi'] = self._data
            self._prevdata = self._data
        self._data = ''

    def initentry(self):
        self._entry = {
            'mode': '',
            'cmdline': '',
            'timestamp': '',
            'execblock': '',
            'status': '',
            'prjcode': '',
            'scdblock': '',
            'pi': '',
            'misc': ''}

    def writeentry(self):
        if self._entry['mode'] == 'MMEX':
            print '%s  %s  %s  %s' % (self._entry['timestamp'], \
            self._entry['cmdline'], self._entry['execblock'], self._entry['status'])
        elif self._entry['mode'] == 'SBEX':
            print '%s  %s (Code %s; PI %s; SB %s)  %s  %s' % \
            (self._entry['timestamp'], self._entry['cmdline'], \
            self._entry['prjcode'], self._entry['pi'], self._entry['scdblock'], \
            self._entry['execblock'], self._entry['status'])


PARSER = OptionParser()
PARSER.add_option("-f", "--file", dest="infile", metavar="infile", \
    default="shiftlog.html", help="Input HTML file")
(OPTIONS, ARGS) = PARSER.parse_args()

if not os.path.exists(OPTIONS.infile):
    print "ERROR: %s does not exist." % OPTIONS.infile
    sys.exit()

slp = SLParser()
for line in open(OPTIONS.infile):
    slp.feed(line)
slp.close()
