#!/usr/bin/env python                                                         

import os
import scipy
import pylab
from taskinit import *
from array import array
import math
import time
import cmath
import re
from scipy.optimize import leastsq
import scipy.io.array_import
import numpy
from numpy import *

def writeLine_Pos_onoff_vlsr (inputms):

      """
      
      Script Name: write_line_pos_onoff_vlsr.py 
      
      Written by A. Hales on 2009-07-22
      
      Goal: Find channel number for peak of the line, line width, and peak flux
      There number have to be updated in the source Catalog

      Input: Takes and input ms
      Output: File containing the info to go on Catalog.
              Filename is uid+ _linePositions.dat

      Usage:

      execfile('write_line_pos_onoff_vlsr.py')  , to load functions.
      
      Then run over all sources in MS by:
 
      writeLine_Pos_onoff_vlsr('uid___X56_X75c9_X1.ms')
      
      Or inspect individual scans by:
      
      findLineWindows('uid___X56_X75c9_X1.ms',scan=3)
 
                          
      """
  
      linepos_filename=inputms.split('.ms')[0]+"_linePositions.dat"

      
      if (os.path.exists(linepos_filename)):
          os.remove(linepos_filename)
      ftpoint = open(linepos_filename,'a')
      #write header
       
      tb.open(inputms)
      timefound  = tb.getcol('TIME')
      timefound =timefound[0]
      t0str=str(timefound)+'s'
      #xoff=off[0,0,0];yoff=off[1,0,0]
      a=qa.time(t0str,form=['ymd'])
      b=a.split('/')
      year=b[0]
      month=b[1]
      day=b[2]
      print "These observations were taken in " + year+" "+  month+" " + day
      # write latitude and Date line
      linetpoint = "! Line positions (channels) for "+inputms + " observed on "+  year+" "+  month+" " + day+" \n\n"

      ftpoint.write(linetpoint)
      ftpoint.close()
      tb.close()




      tb.open(inputms)
      d=tb.getcol('SCAN_NUMBER')
      dim=d.shape
      dimo=dim[0]-1
      length=d[dimo]
      # When doing 1-way will be multiples of 2, 2-way of 4
      
      tb.close()
      
     
      for scan0 in range (0,length-1):
        tb.open(inputms)
        scan1=str(2*scan0+1)
        dir2 = "SCAN_NUMBER=="+scan1+ " && DATA_DESC_ID==0 "
        tb2=tb.query(dir2)
        sourcenumber=2*scan0+1
        print "Processing scan " + scan1
        print "Source Name number is " , sourcenumber
       
        tb3= tbtool.create()
        tb3.open(inputms+ '/SOURCE')
        src=tb3.getcol('NAME')[sourcenumber]
        tb3.close()
       
        print "Looking at " + src
        #tit=src + " - ON - " +inputms.split('_')[4] 
        #src=src.replace(".","_")
#        sourcenumber=sourcenumber+2
        #findLineWindows(inputms,scan=scan0)
        onofffit_vlsr(inputms,ant='PM03',scanin=scan1)
        tb2.close()
        tb.close()

      # Get the offset values and print them into file (done by CrossScanOffset routine)

      # End the tpoint file
      ftpoint = open(linepos_filename,'a')
      #write header
      linetpoint = "END \n"
      ftpoint.write(linetpoint)
      ftpoint.close()
      
      return 
#linepos_filename=inputms.split('.ms' )[0]+"_linePositions.dat"






def onofffit_vlsr (inputms,ant='',scanin=1,xmin=3850,xmax=4079):

     """
     
     Script Name: SLP_reduce.py
     
     Written by A. Hales on 2009-07-22
     
     Goal: Reduce Spectral line data taken with spectralLinePointing.py with -c option
     to:
     - Obtain line intesities, width and position in Vlsr frame     

     - plots band in Vlsr
     - fits baseline to sub-band region masking the line region
     - Baseline substrats, and fits a 1-component
       Gaussian
     - Prints in catalog format line flux, Vlsr and line width in m/s,
       where line width is defined as 2*FWHM of the gaussian.

     TODO: Still uses only one antenna (from OnOffScan)
          
     Takes and input ms, should take antenna too, takes first scan of the on/off (the on)
     
     Usage:
     execfile('SLP_reduce.py') 
                   
     """
     
     scanin=int(scanin)
     src=getName(inputms,scanin)
     execfile('/groups/science/scripts/R6.1.0_WORKING/OnOffScan.py')
     print "scan in 1 is", scanin
     print "scan in 2 is", scanin+1
     scan = OnOffScan(inputms,scanin,scanin+1)
     scan.getField()
     scan.getSource()
     scan.getSpectralWindow()
     pl.clf()
     pl.subplot(311)
     pl.plot(scan.chanLsrVel,scan.spec)
     pl.title(src)
     spectrum_source_on_off=scan.spec

     #spectrum_source_on_off=(spectrum_source)


     xmini=xmin
     xmaxi=xmax

     chanwidth=scan.chanLsrVel[100]-scan.chanLsrVel[99] # in m/s

     low_off_1=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini-40
     low_off_2=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini-20
     on_line_1=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini-5
     on_line_2=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini+5
     high_off_1=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini+20
     high_off_2=spectrum_source_on_off[xmini:xmaxi].argmax()+xmini+40
     #for fit
     low_off_11=low_off_1
     high_off_22=high_off_2




     ### Normalize spectrum first

     centre_data=spectrum_source_on_off[low_off_1:high_off_2]

     xx=scipy.array(centre_data)
     centre_data=centre_data.real



     off_data=spectrum_source_on_off[low_off_1:low_off_2]
     off_data=1*off_data.real
     dat1=1*centre_data
     
     lim=round(off_data.mean(dtype=float),3)
     maxlim=2*lim
     vlas=where(dat1>maxlim)
     dat1[vlas]=lim
     masked_centre_data=dat1
     
     (fitted_bb, fitted_aa)=fit_poly1(masked_centre_data)
     
     print "I found this for Centre Scan Baseline fit: the coeffs aa, bb", fitted_aa, fitted_bb
     
     centre_baseline_norm=fitted_bb
     norm_centre_data=centre_data-centre_baseline_norm
     params=[fitted_bb,fitted_aa]
     y_centre_baseline=peval2(xx,params)


     ### Fit gaussian to the normalized spectrum

     #(line_pos,line_sigma,line_peak)=fit_line_3param(spectrum_source_on_off[low_off_1:high_off_2],spectrum_source_on_off[low_off_1:high_off_2].argmax())
     #(line_pos,line_sigma,line_peak)=fit_line_3param(norm_centre_data,spectrum_source_on_off[low_off_1:high_off_2].argmax())
     (line_pos,line_sigma,line_peak)=fit_line_3param(norm_centre_data,spectrum_source_on_off[low_off_1:high_off_2].argmax())
     print "Line position is ", line_pos
     print "True Line position is ", low_off_1+line_pos
     vlsr_line=scan.chanLsrVel[round(low_off_1+line_pos)]
     print "Vlsr of center is for "+src+" ",vlsr_line
     # Using TWICE the FWHM to define channel width for channel average
     linewidth=chanwidth*2*(2.354*line_sigma)
     print "Line FWHM in m/s for "+src+" ", linewidth
     # write file

     linepos_filename=inputms.split('.ms')[0]+"_linePositions.dat"
     f_linepos = open(linepos_filename,'a')
     line_peak2= "%.3f" % line_peak
     linewidth= "%.0f" % linewidth
     vlsr_line= "%.0f" % vlsr_line
     linepos_1 = src + " \"lineLowOff\" : [" + str(low_off_1)+":"+str(low_off_2-low_off_1) + "], "
     linepos_on =    " \"lineOn\" : [" + str(on_line_1)+":"+str(on_line_2-on_line_1) + "], "
     linepos_2 =     " \"lineHighOff\" : [" + str(high_off_1)+":"+str(high_off_2-high_off_1) + "], "
     linepos_flux =  " \"flux\" : ["+  line_peak2 + "], "
     vlsr_line=" \"pkey\" : [-1], \"vlsr\" : "+vlsr_line+ ", \"vwidth\" : "+linewidth+" }, \n"
     #f_linepos.write(linepos_1+linepos_on+linepos_2+linepos_flux)
     f_linepos.write(linepos_1+linepos_on+linepos_2+linepos_flux+vlsr_line)
     f_linepos.close()
     
     

     pars=[line_peak,line_pos,line_sigma]
     x=range(spectrum_source_on_off[int(low_off_11):int(high_off_22)].size)
     y_gau_line=pevalGau(x,pars)
     y_gau_line=y_gau_line#/line_peak

     # Plot centre scan data, line and gaussian fits

     
     pl.subplot(312)


     pl.plot(spectrum_source_on_off,'b')

     pl.subplot(313)
     
     #pl.plot(x,y_gau_line,'b')
     #pl.plot(x,spectrum_source_on_off[low_off_1:high_off_2],'r')
     pl.plot(x,centre_data,'r--')
     pl.plot(x,norm_centre_data,'b')
     pl.plot(x,y_centre_baseline,'g')
     pl.plot(x,y_gau_line,'y')
     filename=linepos_filename=inputms.split('.ms')[0]+"_"+src+".png"
     pl.savefig(filename)

             
     return 
              


############################################################################################################
############################################################################################################

def getName(inputms,scan):

        tb.open(inputms)
        scan1=str(scan)
        dir2 = "SCAN_NUMBER=="+scan1+ " && DATA_DESC_ID==0 "
        tb2=tb.query(dir2)
        print "Processing scan " + scan1
               
        tb3= tbtool.create()
        tb3.open(inputms+ '/SOURCE')
        src=tb3.getcol('NAME')[scan]
        tb3.close()
        print "Source is  " + src
        return src



def residuals2(p, y, x):
    
    a0,a1,a2= p  
    err = y-(a0 + a1*x + a2*(x**2))
    #err= y - poly1d([a0,a1,a2,a3]) 

# For fitting the Voltage
          
def residualsGau(p,y,x):
     a0,a1,a2= p  
     err = y-(p[0]*scipy.exp(-(x-p[1])**2/(2.0*p[2]**2)))
     return err     

def pevalGau(x, p):
     print "Evaluating Gaussian at", p[0]," ",p[1]," ",p[2]
     return (p[0]*scipy.exp(-(x-p[1])**2/(2.0*p[2]**2)))


def residualsGau_2param(p,y,x):
     a0,a1= p  
     err = y-(p[0]*scipy.exp(-(x-p[1])**2/(72/2.35**2)))
     return err     

def pevalGau_2param(x, p):
     print "FWHM at 86 GHz is 72arcsec, then sigma is", 72/2.35
     print "Evaluating Gaussian at", p[0]," ",p[1]     
     return (p[0]*scipy.exp(-(x-p[1])**2/(72/2.35**2)))




# For fitting the Voltage
   
def residuals2(p, y, x):
    
    a0,a1= p  
    err = y-(a0 + a1*x)
    #err= y - poly1d([a0,a1,a2,a3]) 
    return err

def peval2(x, p):
    #return (poly1d([p[0],p[1],p[2],p[3]]))
    return (p[0]+p[1]*x)








def fit_line_3param (spec,center):


      
######### Fit Line
####################################
           Offset=0
           sigma=0
           Peak=0
           
           x=scipy.array(range(spec.size))
           y=scipy.array(spec)
           y=y.reshape(x.shape[0])
           y=y.real

#           pl.figure()
#           pl.subplot(111)
#           pl.plot(x,y,'b')             

           print 'Fitting from 0 to ',spec.size 
     
           param=3
           if (param==3):
                 
              cte = 0
           # Peak position as variable or fix
              off = center
              fwhm=1
              pname = (['a','b','c'])
              p0 = [cte,off,fwhm]
              plsq = leastsq(residualsGau, p0, args=(y,x))
           
        #pl.subplot(313)
        #pl.plot(x,y,'b')
#              pl.plot(x,pevalGau(x,plsq[0]),'g')
        #a=(1,1,1)
        #pl.plot(x,pevalGau(x,a),'g')
        #print pevalGau(x,a)

              weighted_sum=pevalGau(x,plsq[0])*y
              weighted_sum=weighted_sum.sum
              print "Weighted sum is " , weighted_sum
              
              print "Initial parameters"
              for i in range(len(pname)):
                 print "%s = %.4f" % (pname[i], p0[i])
                 
              print "Final parameters"
              for i in range(len(pname)):
                 print "%s = %.4f" % (pname[i], plsq[0][i])
            
              print "Fitted FWHM is ", (2.35)*plsq[0][2]

        
              Offset=plsq[0][1]
              sigma=plsq[0][2]
              Peak=plsq[0][0]

              print "Line pos, sigma and peak are %.2f %.2f %.4f" % (Offset, sigma, Peak)
           return Offset,sigma,Peak



def fit_poly1 (spec):


           from scipy.optimize import leastsq
           import scipy.io.array_import
      
      
           a=0
           b=0
           
           x=scipy.array(range(spec.size))
           y=scipy.array(spec)
           y=y.reshape(x.shape[0])
           y=y.real



           cte1 = 1
           cte2 = 0
           
           pname = (['b','a'])
           p0 = [cte1,cte2]
           plsq = leastsq(residuals2, p0, args=(y,x))
             
           
                        
           print "Fit order 1 Poly Initial parameters"
           for i in range(len(pname)):
                  print "%s = %.4f" % (pname[i], p0[i])
                  
           print "Fit order 1 Poli Final parameters"
           for i in range(len(pname)):
                  print "%s = %.4f" % (pname[i], plsq[0][i])
                  


           
           a=plsq[0][0]
           b=plsq[0][1]

           print "Coefficients are a & b  are %.4f %.4f" % (a,b)
           return a,b


