"""
 function to write pager (one image per pager) for  set of images
 Note that the names of the image files must be the
 same as the html file, for example if the image file is
 maps_filtp1_fd.gif, the html file must be named maps_filtp1_fd.html.

 hpage  - main page for 'up' link, eg 'index.html'. Can also
          switch back if html produced by this code is in a
          subdirectory (like the '20070626/maps' subdirectory above
          e.g. '../mooncontam1.html'

 tags   - output of get_tags, i.e. a cell array, typicaly of dates,
          e.g. {'060501','060502','060503'}. The dates appear at the
          top right of the pager, they are automatically cycled so
          that clicking 'next' on the last day takes you to the first
          day, and vice-versa. This can be just one day of course.

 field  - contains field names of clickable switches
          names field determines what appears in the pager
          switch field determines which names are struck out
          e.g. in the example below you can click between
          field-difference and non-field difference, and p1 p2 and p3
          filtering. Text for m and p0 will appear struck out and so
          do not link to anything.
          Unfortunately it is also hardwired to three field entries,
          but I figured four would be excessive. If you want less
          than three, just assign the field names and switches like
          in field{3} example below.

          field={}
          field['switch']=[]
          field['names']=[]
          field['names']=[['100GHz','150GHz']]
          field['switch']=[[1,1]]
          tags=['uid1' ,'uid2', 'uid3']

sw     - switch to use if no tags cycle - 1 prevents files starting
          with and underscore

 EXAMPLE USAGE: (in the logpage directory you need a subdirectory
 called 'pager' and a logpage html file called 'mooncontam5.html')

 to generate a pager with 100/150 T/Q/U tabs with cycling ability
 through days 060709 10 and 11

write_single_day_pager('mainpage.html',tags,field,0,1);

 command above will produce files like 060709_100GHz_Q.html. Each
 html file should have one corresponding .gif file with exactly the
 same name, i.e. 060709_100GHz_Q.gif.

 the first argument provides a link to the main logpage outside the
 pager. This doesn't have to be in the same directory, i.e.

write_single_day_pager('../mainpage.html',tags,field,0,0);

 will also work.
 if you use 

write_single_day_pager('../pointsources2.html',tags,field,0,0);

 (i.e. last command line argument 0 rather than 1) you get files like
 100GHz_Q_060709.html if you would prefer yourfilenames like that.

 you can adjust the switches so that 100GHz can be turned off by
 using field{1}.switch=[0 1]; the html will look eaxctly the same as
 before, but 100GHz will be 'struck through' so you can't flip to it.

 if you just want to cycle through 100/150 T/Q/U for one day, use 

write_single_day_pager('../pointsources2.html',{''},field,1,1);

 the second to last command line argument removes the first character
 of the html filename. It's a bit screwy.

"""
import os

from scipy import *
from numpy import *
from pylab import *


def write_single_day_pager(hpage,tags,field,sw,stend, format='gif'):

    nelems=len(field['names'])
    for i in range(6):
        if i > nelems:
            field['names'].append([''])
            field['switch'].append([0])
   
    for i in range(size(tags)):
        tag=tags[i]
    
        # loop through files to be written
        for k in range(size(field['names'][0])):
            for l in range(size(field['names'][1])):
                for m in range(size(field['names'][2])):
                    for n in range(size(field['names'][3])):
                        for o in range(size(field['names'][4])):
                            
                            # case when only first field is used
                            if (size(field['names'][1])==1) & (field['switch'][1][0]==0):
                                print "1 field"
                                if(stend):
                                    thispage=tag+'_'+field['names'][0][k]+'.html'
                                else:
                                    thispage=field['names'][0][k]+'_'+tag+'.html'
                                
                            # case where first and second fields are used
                            elif (size(field['names'][2])==1) & (field['switch'][2][0]==0):
                                print "2 fields"
                                if(stend):
                                    thispage=tag+'_'+field['names'][0][k]+'_'+field['names'][1][l]+'.html'	      
                                else:
                                    thispage=field['names'][0][k]+'_'+field['names'][1][l]+'_'+tag+'.html'

                            # case where three fields are used
                            elif (size(field['names'][3])==1) & (field['switch'][3][0]==0):
                                print "3 fields"
                                if(stend):
                                    thispage=tag+'_'+field['names'][0][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'.html'
                                else:
                                    thispage=field['names'][1][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'_'+tag+'.html'
                            # case where four fields are used
                            elif (size(field['names'][4])==1) & (field['switch'][4][0]==0):
                                print "4 fields"
                                if(stend):
                                    thispage=tag+'_'+field['names'][0][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'_'+field['names'][3][n]+'.html'
                                else:
                                    thispage=field['names'][0][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'_'+field['names'][3][n]+'_'+tag+'.html'
                    
                            # case where all five fields are used
                            else:
                                print "5 fields"
                                if(stend):
                                    thispage=tag+'_'+field['names'][0][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'_'+field['names'][3][n]+'_'+field['names'][4][o]+'.html'
                                else:
                                    thispage=field['names'][0][k]+'_'+field['names'][1][l]+'_'+field['names'][2][m]+'_'+field['names'][3][n]+'_'+field['names'][4][o]+'_'+tag+'.html'
                                    
                    
                            if(sw):
                                thispage=thispage[2:ends]
                            thisfile=thispage;
                                                        
                            # open file and write header
                            fh=open(thisfile,'w');
                            fh.write('<HTML>\n\n');
                            fh.write('<BODY TEXT=\"#000000\" BGCOLOR=\"#F0F0F0\">\n\n')
                            fh.write('<a href=\"%s\">up</a>\n\n'%hpage);
                            fh.write('----\n');
	    
                            # loop over first field
                            fh=field_loop(fh,k,field,0,thispage)
	    
                            # loop over second field
                            if ~((size(field['names'][1])==1) & (field['switch'][1][0]==0)):
                                fh.write('--\n');
                                fh=field_loop(fh,l,field,1,thispage)
                    
                            # loop over third field
                            if ~((size(field['names'][2])==1) & (field['switch'][2][0]==0)):
                                fh.write('--\n');
                                fh=field_loop(fh,m,field, 2,thispage);
                    
                            # loop over fourth field
                            if ~((size(field['names'][3])==1) & (field['switch'][3][0]==0)):
                                fh.write('--\n');
                                fh=field_loop(fh,n,field ,3,thispage);

                            # loop over fifth field
                            if ~((size(field['names'][4])==1) & (field['switch'][4][0]==0)):
                                fh.write('--\n');
                                fh=field_loop(fh,o,field, 4,thispage);
                        
                            # write days link
                            print thispage, i
                            fh=days_loop(fh,thispage,tags,i);
                    
                            # plot the image
                            fh.write('<p><img src="%s"  width=60%%>' %(thispage.replace('.html','.%s'%format)))
                            
                            fh.close()
    return

###################################################################
def field_loop(fh,k,field,f,thispage):
    # loop over field

    for n in range(size(field['names'][f])):
        print f,n
        if( (n==k) & (field['switch'][f][n]==1)):
            fh.write('<strong>%s<strong>\n' %field['names'][f][n])
        elif field['switch'][f][n]==0 :
            fh.write('<strike>%s</strike>\n'%field['names'][f][n])
        else:
            fh.write('<a href="%s">%s</a>\n'%(thispage.replace(field['names'][f][k],field['names'][f][n]),field['names'][f][n]));	
        fh.write('--\n');

    return fh

###################################################################
def days_loop(fh,thispage,tags,i):
    # loop over days
    
    # current day
    tag=tags[i];
    # day before
    tagm=shiftarray(tags,1)
    tagm=tagm[i]
    # day after
    tagp=shiftarray(tags,-1); tagp=tagp[i];
    
    if(size(tags)>1):
        fh.write('--\n');
        fh.write('<a href=\"%s\">prev</a>\n'%thispage.replace(tag,tagm));
        fh.write('--\n');
        fh.write('<strong><font color=\"black\">%s</font><strong>\n'%tag);
        fh.write('--\n');
        fh.write('<a href=\"%s\">next</a>\n'%thispage.replace(tag,tagp));  

    return fh
	

def shiftarray(seq, n):
    ss=size(seq)
    n = n % ss
    new=[]
    for i in range(ss):
        new.append(seq[i-n])
    return new
         
