
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

# bpcal.py

# Description:
# ------------
# This module runs the bandpass calibration statistics subsystem of the QA2
# system.

# Functions:
# ----------
# bpcal       - This function runs the bandpass calibration statistics subsystem
#               of the QA2 system.
# bpcal_calc  - This function calculates the bandpass calibration statistics.
# bpcal_write - This function writes the bandpass calibration statistics to a
#               CASA table.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version created with functions bpcal(), bpcal_calc(),
#               and bpcal_write().

# ------------------------------------------------------------------------------

# Imports
# -------

import os

import casa

#import bp_nick

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

# bpcal

# Description:
# ------------
# This function runs the bandpass calibration statistics subsystem of the QA2
# system.

# Inputs:
# -------
# in_table - This python string contains the name of the input bandpass
#            calibration table.
# out_dir  - This python string contains the output directory.

# Outputs:
# --------
# If the function finishes successfully, True is returned via the function
# value.  If the function does not finish successfully, it throws an exception.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version.
# 2011 Jun 13 - Nick Elias, NRAO
#               Improved error handling.
# 2011 Jun 20 - Nick Elias, NRAO
#               Raise an exception if the output table already exists.

# ------------------------------------------------------------------------------

def bpcal( in_table, out_dir ):

	# Calculate the bandpass calibration statistics

	try:
		bpcal_stats_dict = bpcal_calc( in_table )
	except:
		msg = 'bpcal.bpcal: Calculating bandpass calibration '
		msg += 'statistics failed.\n'
		print msg
		raise


	# Write the bandpass calibration statistics to the table

	out_table_root = os.path.basename( os.path.splitext( in_table )[0] )
	out_table = out_dir + '/' + out_table_root + '.bpcal.stats'

	if os.path.exists( out_table ):
		msg = 'bpcal.bpcal: Output file already exists.\n'
		print msg
		raise

	try:
		status = bpcal_write( bpcal_stats_dict, out_table )
	except:
		msg = 'bpcal.bpcal: Writing bandpass calibration statistics '
		msg += 'failed.\n'
		print msg
		raise


	# Return True

	return True

# ------------------------------------------------------------------------------

# bpcal_calc

# Description:
# ------------
# This function calculates the bandpass calibration statistics.

# Inputs:
# -------
# in_table - This python string contains the name of the input bandpass
#            calibration table.

# Outputs:
# --------
# The dictionary containing the bandpass calibration statistics, returned via
# the function value.  If the function does not finish successfully, an
# exception is raised.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version (stub).

# ------------------------------------------------------------------------------

def bpcal_calc( in_table ):

	bpcal_stats_dict = {}

	# Return the dictionary containing the bandpass calibration statistics

	return bpcal_stats_dict

# ------------------------------------------------------------------------------

# bpcal_write

# Description:
# ------------
# This function writes the bandpass calibration statistics to a CASA table.

# Inputs:
# -------
# bpcal_stats_dict - This python dictionary contains the bandpass calibation
#                    statistics.
# out_table        - This python string contains the output table name.

# Outputs:
# --------
# The bandpass calibration statistics table is created in the output directory.
# If the function finishes successfully, True is returned via the function
# value.  If the function does not finish successfully, it throws an exception.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version (stub).

# ------------------------------------------------------------------------------

def bpcal_write( bpcal_stats_dict, out_table ):

	# Return True

	return True

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
