
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

# gcal.py

# Description:
# ------------
# This module runs the gain calibration statistics subsystem of the QA2 system.

# Functions:
# ----------
# gcal       - This function runs the gain calibration statistics subsystem of
#              the QA2 system.
# gcal_calc  - This function calculates the gain calibration statistics.
# gcal_write - This function writes the gain calibration statistics to a CASA
#              table.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version created with functions gcal(), gcal_calc(), and
#               gcal_write().

# ------------------------------------------------------------------------------

# Imports
# -------

import os

import casa

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

# gcal

# Description:
# ------------
# This function runs the gain calibration statistics subsystem of the QA2
# system.

# Inputs:
# -------
# in_table - This python string contains the name of the input gain calibration
#            table.
# out_dir  - This python string contains the output directory.

# Outputs:
# --------
# If the function finishes successfully, True is returned via the function
# value.  If the function does not finish successfully, it raises an exception.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version.
# 2011 Jun 13 - Nick Elias, NRAO
#               Improved error handling.
# 2011 Jun 20 - Nick Elias, NRAO
#               Raise an exception if the output table already exists.

# ------------------------------------------------------------------------------

def gcal( in_table, out_dir ):

	# Calculate the gain calibration statistics

	try:
		gcal_stats_dict = gcal_calc( in_table )
	except:
		msg = 'gcal.gcal: Calculating gain calibration statistics '
		msg += 'failed.\n'
		print msg
		raise


	# Write the gain calibration statistics to the table

	out_table_root = os.path.basename( os.path.splitext( in_table )[0] )
	out_table = out_dir + '/' + out_table_root + '.gcal.stats'

	if os.path.exists( out_table ):
		msg = 'gcal.gcal: Output file already exists.\n'
		print msg
		raise

	try:
		status = gcal_write( gcal_stats_dict, out_table )
	except:
		msg = 'gcal.gcal: Writing gain calibration statistics failed.\n'
		print msg
		raise


	# Return True

	return True

# ------------------------------------------------------------------------------

# gcal_calc

# Description:
# ------------
# This function calculates the gain calibration statistics.

# Inputs:
# -------
# in_table - This python string contains the name of the input gain calibration
#            table.

# Outputs:
# --------
# The dictionary containing the gain calibration statistics, returned via the
# function value.  If the function does not finish successfully, an exception is
# raised.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version (stub).

# ------------------------------------------------------------------------------

def gcal_calc( in_table ):

	gcal_stats_dict = {}

	# Return the dictionary containing the gain calibration statistics

	return gcal_stats_dict

# ------------------------------------------------------------------------------

# gcal_write

# Description:
# ------------
# This function writes the gain calibration statistics to a CASA table.

# Inputs:
# -------
# gcal_stats_dict - This python dictionary contains the gain calibation
#                   statistics.
# out_table       - This python string contains the output table name.

# Outputs:
# --------
# The gain calibration statistics table is created in the output directory.  If
# the function finishes successfully, True is returned via the function value.
# If the function does not finish successfully, it raises an exception.

# Modification history:
# ---------------------
# 2011 May 20 - Nick Elias, NRAO
#               Initial version (stub).

# ------------------------------------------------------------------------------

def gcal_write( gcal_stats_dict, out_table ):

	# Return True

	return True

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
